//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################


/* -------------------------------------------------------------------------------

	File:				CCVSEntries.h
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains code to read/load/write the "Entries" file in
		each working directory.
		 
	Modification History:

		15 	Dec		1998		Ram Rajadhyaksha
		
				Created.
										
-------------------------------------------------------------------------------- */


#pragma once


#include "CRadixTree.h"



class CCVSEntries:

	protected CRadixTree
	
	{
	
		public: 			
			
			enum
				{				
					kMaxBufSize 	= 32768,
					kMaxLineSize 	= 512,
					kMinStackSize	= 4096					
				};
			
			explicit				CCVSEntries();

										 	~CCVSEntries();

			virtual OSErr		LoadWorkingDirectory(	const FSSpec 				*inWorkingDirectory,
																						const SInt8					inPerm = fsRdWrPerm);
																						
			virtual OSErr		LoadEntriesFile(			const FSSpec 				*inEntriesFile,
																						const FSSpec				*inWorkingDirectory,
																						const SInt8					inPerm = fsRdWrPerm);
				
			virtual OSErr		CloseEntriesFile();
			virtual OSErr		FlushEntriesFile(			const Boolean				inTouchModDate = true);
							
			virtual OSErr		AddModifyEntryStr(		const char 					*inEntryStr);		
				
			virtual OSErr		AddModifyEntry(				const Str255				inNamePStr,
																						const Str255 				inRevisionPStr,
																						const Str255 				inTimeStampPStr,
																						const Str255 				inOptionsPStr,
																						const Str255 				inTagPStr,
																						const Boolean 			inIsDirectory);
			
			virtual OSErr		GetEntry(							const Str255				pstrInKey,
																						Str255							pstrOutRevision,
																						Str255							pstrOutTimeStamp,
																						Str255							pstrOutOptions,
																						Str255							pstrOutTag);
																									
			virtual OSErr		RemoveEntry(					const Str255 				inNamePStr);

			static OSErr		GetEntryItems( 				const char 					*strInEntry, 
																						const UInt32 				intInLen, 
																						Str255 							pstrOutName, 
																						Str255 							pstrOutRevision, 
																						Str255 							pstrOutTimeStamp, 
																						Str255 							pstrOutOptions, 
																						Str255 							pstrOutTag);
			
			virtual Boolean	AreYouThisDirectory(	const FSSpec				*inWorkingDirectory);
			
			virtual OSErr		Yield()	{ return noErr; }
																									
		protected:

			virtual void		WalkEntryList( 				CRadixTreeNodeRec 	*inCurNode,
																						const UInt32				inFileLen,
																						char 								*inBuffer, 
																						const UInt32 				inBufLen, 
																						UInt32 							*inOutBufCount,
																						UInt32							*inOutFileOffset,
																						UInt32							*inOutDirCount);			
																						
			FSSpec		mFileSpec;
			FSSpec		mWorkingDir;
			SInt16		mFileRef;
			SInt8			mPerm;
			Boolean		mDirty;
			
	};
	

