//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CCVSRepository.h
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code that does all the network transactions
		with the CVS pserver/kserver and local disk file operations.
		 
	Modification History:

		22	July 	1998		Ram Rajadhyaksha
		
					Modified prototypes for WriteEntry/WriteDirectoryEntry/WriteItemEntry.
				
		20 	July	1998		Simon Fraser
		
					Added many more diffing options.
										
-------------------------------------------------------------------------------- */


#pragma once


#include <Types.h>

#include "CRadixTree.h"
#include "CCVSEntries.h"
#include "CCVSMappings.h"
#include "CCVSServerStream.h"
#include "CVSEngine.h"


//  To turn on the correct AppleSingle format for sending files,
//  uncomment the line below.


#define __NEWAPPLESINGLEFORMAT 1


//  Asychronous IO is currently turned off because there is a bug
//  where if you abort a thread running the async. io, it doesn't
//  wait until the io request has completed before exiting.

//#define ASYNCFILEIO 1

	
class CCVSEntriesCache;

	
class CCVSRepository

	{
	
		public: 
		
			friend class CCVSEntriesCache;
			
			enum
				{
					kDefaultBufferSize							= 128 * 1024,		// used to be 48K
					kDefaultMinimumStackSize				= 8 * 1024,
					
					k8kCommentLimit									= 8 * 1024,
					kMaxCharsPerLine								= 70,
										
					kDefaultYieldCount							= 20,
					
					kPostMessageDataLen							= 0,
					
					kMaxLineSize										= 512,
					kMaxPathSize										= 512					
				};
				
			explicit				CCVSRepository(					const FSSpec											*inLocalRoot,
																							CCVSEngineFileMappingRec					*inFileMappings,
																							const UInt32											inFileMappingsCount,
																							CCVSEngineEventNotifierProcPtr		inNotifier,
																							CCVSEngineStreamNotifierProcPtr		inMessageNotifier,
																							CCVSEngineStreamNotifierProcPtr		inErrorNotifier,
																							void															*inContextPtr,
																							const UInt32											inTimeOut);

										 	~CCVSRepository();

			virtual OSErr		Login(									const CCVSEngineLoginOptions			*inOptions);
						
			virtual OSErr		Add(										const FSSpec 											*inTargetSpec,
																							const CCVSEngineAddOptions				*inOptions);
																																	
			virtual OSErr		Checkout(								const FSSpec 											*inTargetSpec,
																							const CCVSEngineCheckoutOptions		*inOptions);
																						
			virtual OSErr 	Commit(									const FSSpec 											*inTargetSpec,
																							const CCVSEngineCommitOptions			*inOptions);

			virtual OSErr		Diff(										const FSSpec											*inTargetSpec,
																						 	const CCVSEngineDiffOptions				*inOptions,
																						 	CCVSEngineStreamNotifierProcPtr		inDiffNotifier,
																						 	void															*inContextPtr);

			virtual OSErr		Edit( 									const FSSpec 											*inTargetSpec);																																									

			virtual OSErr		Editors( 								const FSSpec 											*inTargetSpec, 
																							CCVSEngineStreamNotifierProcPtr 	inNotifier, 
																							void 															*inContextPtr);																																									
				
			virtual OSErr		Import(									const FSSpec											*inTargetSpec,
																							const CCVSEngineImportOptions			*inOptions);
																																										
			virtual OSErr 	Log(										const FSSpec	 										*inTargetSpec,
																							CCVSEngineStreamNotifierProcPtr		inLogNotifier,
																							void															*inContextPtr);

			virtual OSErr		Remove(									const FSSpec 											*inTargetSpec);

			virtual OSErr		Status(									const FSSpec 											*inTargetSpec,
																							CCVSEngineStreamNotifierProcPtr		inStatusNotifier,
																							void															*inContextPtr);
			
			virtual OSErr		Tag(										const FSSpec 											*inTargetSpec,
																							const CCVSEngineTagOptions				*inTagOptions);

			virtual OSErr		UnEdit( 								const FSSpec 											*inTargetSpec);																																									

			virtual OSErr		Update(									const FSSpec 											*inTargetSpec, 
																							const CCVSEngineUpdateOptions			*inOptions);
				
			virtual OSErr		WatchAddRemove(					const FSSpec											*inTargetSpec,
																							const CCVSEngineWatchOptions			*inOptions);
																							
			virtual OSErr		WatchOnOff(							const FSSpec											*inTargetSpec,
																							const Boolean											inWatch);	

			virtual OSErr		Watchers( 							const FSSpec 											*inTargetSpec, 
																							CCVSEngineStreamNotifierProcPtr 	inNotifier, 
																							void 															*inContextPtr);																																									
			virtual SInt32	Version();
																					
		protected:
	
			virtual void		NegotiateServerOptions();
			
			virtual void		ProcessServerResponse(	const char												*strInOffsetPath,
																							const UInt32											intInOffsetPathLen,
																							const UInt32											intInMergePolicy,
																							const CCVSEngineFolderCreationOptions &foInFolderOptions,
																							CCVSEngineStreamNotifierProcPtr		inMessageNotifier,
																							CCVSEngineStreamNotifierProcPtr   inErrorNotifier,
																							void															*inContextPtr,
																							const Boolean											bInUpdateAnyVersion = false,
																							const Boolean											bInAddRevisionInfo = false,
																							const Boolean											bInResetMRO = false);
	
			virtual Boolean	IsFatalException( 			OSErr 														inErr);
																						
			virtual void		ItemUpdatedMerged(			const	Str255											inUserNamePStr,
																							const UInt32											inMergePolicy,
																							const Boolean											inUpdateParentTag,
																							const char												*inOffsetPathStr,
																							const UInt32											inOffsetPathLen,
																							const CCVSEngineFolderCreationOptions &folderOptions,
																							FSSpec														&outTargetSpec);

			virtual void		ItemCheckedIn(					const	Str255											inUserNamePStr,
																							const UInt32											inMergePolicy,
																							const Boolean											inUpdateParentTag,
																							const char												*inOffsetPathStr,
																							const UInt32											inOffsetPathLen,
																							FSSpec														&outTargetSpec,
																							const Boolean											bInUpdateAnyVersion,
																							const Boolean											bInAddRevisionInfo,
																							const Boolean											bInResetMRO);

			virtual void		ItemRemoved(						const char												*inOffsetPathStr,
																							const UInt32											inOffsetPathLen,
																							FSSpec														&outTargetSpec,
																							const Boolean											inDeleteFile = true);

			virtual void		SetClearStaticDir(			const Boolean											inSetStaticDir,
																							const	Str255											inUserNamePStr,
																							const char												*inOffsetPathStr,
																							const UInt32											inOffsetPathLen,
																							const CCVSEngineFolderCreationOptions &folderOptions,
																							FSSpec														&outTargetSpec);

			virtual void		SetClearStickyDir(			const Boolean											inSetStickyDir,
																							const	Str255											inUserNamePStr,
																							const char												*inOffsetPathStr,
																							const UInt32											inOffsetPathLen,
																							const CCVSEngineFolderCreationOptions &folderOptions,
																							FSSpec														&outTargetSpec);
				
			virtual void		ModuleExpansion(				const UInt32											intInMergePolicy,
																							const Boolean											bInDeep);

			virtual void		MessageOut(							CCVSEngineStreamNotifierProcPtr 	inNotifier,
																							char															*inDataStr,
																							const UInt32											inDataLen,
																							void															*inContextPtr);
																																					
			virtual void	 	ErrorOut( 							CCVSEngineStreamNotifierProcPtr 	inNotifier,
																							char 															*inErrorCStr,
																							const UInt32											inDataLen,
																							void															*inContextPtr);
																																							
			virtual void		AddDirectory(						const FSSpec											&inTargetSpec,
																							const char												*inOffsetPathStr,
																							const UInt32											inOffsetPathLen,
																							const char												*inPathStr,
																							const UInt32											inPathLen);
			
			virtual void    WriteEntry( 						const FSSpec 											&inTargetSpec,
																							const FSSpec											&inBaseSpec, 
																							const Boolean 										inDoMerge,
																							const	Boolean											inDeep,
																							const Boolean											inSendConflictInfo,
																							const Boolean											inSendModifiedBinaryData,
																							const Boolean											inUseUnchanged,
																							const SInt8												inPerm = fsRdWrPerm);

			virtual void    WriteDirectoryEntries( 	const CInfoPBRec 									&inTargetPB, 
																							const FSSpec											&inBaseSpec,
																							const Boolean 										inDoMerge,
																							const Boolean											inDeep,
																							const Boolean											inSendConflictInfo,
																							const Boolean											inSendModifiedBinaryData,
																							const Boolean											inUseUnchanged,
																							const SInt8												inPerm = fsRdWrPerm);
			
			virtual void    WriteItemEntry( 				const CInfoPBRec 									&inTargetPB,
																							const char												*inStrParPath,
																							const UInt32											inParPathLen,
																							const Boolean 										inDoMerge,
																							const Boolean											inSendConflictInfo,
																							const Boolean											inSendModifiedBinaryData,
																							const Boolean											inUseUnchanged,
																							const SInt8												inPerm = fsRdWrPerm);
				
			virtual void		WriteModifiedEntry(			const FSSpec											&inTargetSpec,
																							const UInt32											inEncoding,
																							const SInt8												inPerm = fsRdWrPerm);
	
			virtual void		WriteImportEntries(			const FSSpec											&inTargetSpec,
																							const Boolean											inSendBinaryFiles,
																							UInt32														&outSkippedFiles);
			
			virtual void		WriteImportDirectoryEntries(			
																							const CInfoPBRec									&inTargetPB,
																							const FSSpec											&inBaseSpec,
																							const Boolean											inSendBinaryFiles,
																							UInt32														&outSkippedFiles);
									
			virtual void		FSpSendAppleSingleData(	const FSSpec											&inTargetSpec,
																							const SInt8												inPerm = fsRdWrPerm);

/*
			virtual void		FSpSendPsychoAppleDoubleData(	
																							const FSSpec											&inTargetSpec,
																							const SInt8												inPerm = fsRdWrPerm);
*/
	
			void						FSpSetDirectory(				const FSSpec											&fsInTarget);
																																
			void						SendCheckinComment(			Handle														inCommentDataHndl);
			
			void						ResetFileDates(					const FSSpec											&inTargetSpec,
																							const SInt32											inCreationDate,
																							const SInt32											inModDate);															
			
			void						InitRootDirectory();
			
			void						Yield();
			
			static pascal OSErr SocketYieldProc(		CTCPSocket 												*inSocket, 
																							void 															*inContextPtr);
					
			static pascal OSErr EncodingYieldProc(	void 															*pInContextPtr, 
																							UInt32 														intInMax, 
																							UInt32 														intInComplete);
																					
			void						Initialize();
			
			FSSpec															mLocalRoot;
			Str255															mRemoteRoot;
			Str255															mRemoteUserName;
			Str255															mLocalUserName;
			Str255															mServerName;
			UInt16															mPort;
			AuthenticationMethod								mAuthMethod;

//  Local receive/decoding buffer.
						
			char																*mBuffer;
			
			UInt32															mBufSize,
																					mYieldCount;
			
			CCVSEngineEventNotifierProcPtr			mNotifier;
			CCVSEngineStreamNotifierProcPtr			mMessageNotifier;
			CCVSEngineStreamNotifierProcPtr			mErrorNotifier;

			void																*mContextPtr;
			
			CCVSServerStream										mStream;
			
			CCVSEntriesCache										*mEntries;
			CCVSMappings												mMappings;
						
			static Boolean											sInitialized;
			Boolean															mCanUseFileCompression;

	};
	
	
class CCVSEntriesCache:

	public CCVSEntries
	{
				
		public:
		
			explicit		CCVSEntriesCache(CCVSRepository *inOwner) { mOwner = inOwner; }
									~CCVSEntriesCache() { }
	
			void FlushEntriesToStream( 					CTCPStream 				&inStream );
			
			virtual OSErr	Yield();
			
		protected:
		
			void FlushEntriesToStreamRecursive( CRadixTreeNodeRec *inCurNode, 
																					CTCPStream 				&inStream );
			
			CCVSRepository		*mOwner;
			
	};
	