//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CTCPSocket.h
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code that does all the network transactions.
		 
	Modification History:

		13 Feb 1999		Ram Rajadhyaksha
		
					Elimated CTCPSocketErr exception type and error handling. Modified
					CTCPSocketYieldProc to return OSErr.
										
-------------------------------------------------------------------------------- */


#pragma once


#ifndef __CTCPSOCKET__
#define __CTCPSOCKET__


#include <OpenTransport.h>
#include <OpenTptInternet.h>


class CTCPSocket;


/* Prototype for custom yield routine */
typedef pascal OSErr (*CTCPSocketYieldProc)(CTCPSocket *inSocket, void *inContextPtr);


class CTCPSocket

	{
	
		public:
		
			enum
				{			
					kDefaultTimeOut				= 600,
					kMinCTCPSocketErr			= 600,
					kUnknownErr						= 600,
					kOTNotInstalled				= 601,
					kCouldNotInitialize		= 602,
					kCouldNotCreateSocket	= 603,
					kCouldNotAllocMemory	= 604,
					kCouldNotResolve			= 605,
					kTimeout							= 606,
					kIllegalState					= 607,
					kSocketAbortedErr			= 608,
					kBadParametersErr			= 609,
					kMaxCTCPSocketErr			= 609			
				};
								
			CTCPSocket(											CTCPSocketYieldProc		inYieldProc		= NULL,
																			void									*inContextPtr	= NULL);
																			
			~CTCPSocket();
			
			virtual void 	Bind();
			virtual void	Bind(							const UInt32 		inAddress,
																			const UInt16 		inPort);
			
			virtual void	Unbind();
								
			virtual void	Connect(					const Str255 		inAddressStr,
																			const UInt16		inPort,
																			const UInt32		inTimeOut = kDefaultTimeOut);
								
			virtual void 	Disconnect(				UInt32					inTimeOut = kDefaultTimeOut);
			
			virtual void 	SendData(					void						*inData,
																			UInt32					inDataSize,
																			UInt32					inTimeOut	= kDefaultTimeOut);
											
			virtual void 	ReceiveData(			char						*outData,
																			UInt32					&inOutSize,
																			UInt32					inTimeOut = kDefaultTimeOut);
																			
			virtual InetAddress GetBindAddress () { return mBindAddress; };
			virtual InetAddress GetPeerAddress () { return mPeerAddress; };
			
			virtual	void				GetPeerHostname (char* ioName);

			
/*					
			static pascal void SocketNotifier(		void						*inContextPtr,
																						OTEventCode			inEventCode,
																						OTResult				inResult,
																						void						*inCookie);
*/																									
		protected:
	
			virtual void	SocketYield();
			virtual void	OpenEndpoint();
			virtual void	CloseEndpoint();
			virtual void	Initialize();
			
			EndpointRef						mEndpointRef;
			TEndpointInfo					mEndpointInfo;
			InetSvcRef						mDNSProvider;
			TCall									mTCall;
			CTCPSocketYieldProc		mYieldProc;
			void									*mContextPtr;
			
			InetAddress						mBindAddress;
			InetAddress						mPeerAddress;
			
			static Boolean					sInitialized;
			static OTConfiguration	*sConfig;
			
	};
	
	

#define IsCTCPSocketErr(__theErr)	\
((__theErr >= CTCPSocket::kMinCTCPSocketErr) && (__theErr <= CTCPSocket::kMaxCTCPSocketErr)) 


#endif