//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CTCPStream.h
	
	Authors:		Scott Silver and Ram Rajadhyaksha
	
	Description:
	
		This file contains code to implement a simple stream over TCP/IP.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Fixed casting problem in GetBytes to get it to compile.
							
-------------------------------------------------------------------------------- */


#ifndef __CTCPSTREAM__
#define __CTCPSTREAM__


#include <Types.h>
#include "CTCPSocket.h"
#include "zlib.h"

// Only send data to the socket when we have a newline in the input buffer
#define NEWLINE_MODE			1


class CTCPStream
	{

		public:

			enum 
				{
				
					kTCPStreamCounterFailed 	= 4894,
					kDefaultTimeout						= 60
				
				};

#ifdef NEWLINE_MODE
			enum
				{
					kSendBufferAllocationSize	= 512
				
				};
#endif

															CTCPStream(				const UInt32			inTimeOut = CTCPSocket::kDefaultTimeOut);						
			virtual									~CTCPStream();
			
			virtual void						OpenConnection(		CTCPSocketYieldProc	inYieldProc,
																								void							*inContextPtr,	
																								const Str255			inHostNamePStr,
																								const UInt16			inHostPort,
																								const UInt16			inLocalPort = 0);
												
			virtual void 						CloseConnection();
			
			//	Overridden methods to support streams 

			virtual void						WriteBlock( 			const void 				*inBuffer, 
																								SInt32 						inByteCount);

//#ifdef NEWLINE_MODE
			virtual void						UnbufferedWriteBlock(const void			*inBuffer, 
																								SInt32 						inByteCount);

//#endif
																					
			virtual void						ReadBlock( 				void 							*outBuffer, 
																								SInt32 						inByteCount);

			virtual SInt32 					ReadBlockAndInflate( void *outBuffer, SInt32 inByteCount, UInt32 &outBytesCompressed );

			void										InflateStartup(UInt32 totalCompressedFileSize);
			void										InflateShutdown();

			// Counting to prevent incorrect file lengths
/*			
			void										StartCounting(		UInt32 						inMaxBytes);
			void										StopCounting();
			void										UpdateCounter(		UInt32 						inDelta) { mCounter += inDelta; ThrowIf_(mCounter > mMaxCounter); }
*/

			virtual char 						ReadWord(					char 							*inOutWordCStr, 
																								UInt32 						&inOutLen, 
																								const char 				*inTermCStr = " \n");

			virtual Boolean					ReadDataLine(			char 							*inOutDataStr, 
																								UInt32 						&inOutDataLen);

			virtual Boolean					ReadEntireDataLine(char 						*inOutDataStr, 
																								UInt32 						&inOutDataLen);

			virtual SInt32 					WriteZeroByte();
			
			virtual SInt32					WriteCString(			const char				*inString);
			virtual SInt32					ReadCString(			char							*outString);
			
			virtual SInt32					WritePString(			ConstStr255Param	inString);
			virtual SInt32					ReadPString(			Str255						outString);
																										
			//	Utilities for reading and writing data from the input stream

			CTCPStream&							operator << (			const char				*inString)
				{
					WriteCString(inString);
					return (*this);
				}

			CTCPStream&							operator << (			ConstStr255Param 	inString)	// Pascal string
				{
					WritePString(inString);
					return (*this);
				}

			CTCPStream&							operator << (			char 							inChar)
				{
					WriteBlock(&inChar, sizeof(inChar));
					return (*this);
				}

			CTCPStream&							operator >> (			Str255 						outString)
				{
					ReadPString(outString);
					return (*this);
				}

			CTCPStream&							operator >> (			char* 						outString)
				{
					ReadCString(outString);
					return (*this);
				}

			CTCPStream&							operator >> (			char 							&outChar)
				{
					ReadBlock(&outChar, sizeof(outChar));
					return (*this);
				}
			
																		
		protected:

			virtual void					 	GetBytes(					void 							*outBuffer, 
																								SInt32 						&ioByteCount);
												
			virtual void					 	PutBytes(					const void				*inBuffer,
																								SInt32						&ioByteCount
#ifdef NEWLINE_MODE																								
																								, Boolean					inNewlineBuffer = true	
#endif
																								);
				
			enum 
				{					
					kTimeout 												= 3 * 60 * 60, 	// 	3 minutes
					kConnectionReceiveBufferSize 		= 24 * 1024, 		// 	24k
					kCompressedBufferSize						= 64 * 1024			//  64k ZLib buffer
				};
				
			CTCPSocket			*mSocket;

			UInt32					mCounter;
			UInt32					mMaxCounter;
			UInt32					mTimeOut;
			
		private:

#ifdef NEWLINE_MODE
			char					*mSendBuffer;
			
			UInt32				mSendBufferAllocated;
			UInt32				mSendBufferUsed;
#endif
			unsigned char	mCompressedBuffer[kCompressedBufferSize];
			Boolean				mSkipZHeader;
			UInt32				mTotalCompressedSize;
			UInt32				mTotalCompressedDataRead;
						
			z_stream			mZStreamState;
	};


#endif
