//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CVS Engine API
//
//	  Author: 	Ram Rajadhyaksha
//
//################################################################################


#ifndef __CVSENGINEAPI__
#define __CVSENGINEAPI__


#include <Types.h>


typedef void * CCVSEngineRef;


/*	-------------------------------------------------------------------------
		** Compiler Switches
		------------------------------------------------------------------------- */

/*

		CVSENGINE_KCLIENT_SUPPORT    						(default: commented out)

    Uncomment the #define CVSENGINE_KCLIENT_SUPPORT line to build the Engine
    with Kerberos authentication support. To do this, you will need:
 
  	1. 	The KClient software, and SDK <ftp://ftp.cit.cornell.edu/pub/mac/kerberos/>
  	2. 	The file KClientSupport.cp, which includes the necessary support code.


		CVSENGINE_TERMINATE_WITH_LF  						(default: commented out)

		Uncomment the #define CVSENGINE_TERMINATE_WITH_LF to build MacCVS Pro
		using LF termination when generating CVS administrative information. This
		might be necessary for compatibility with the GNU CVS client in MacOS X.
		

		CVSENGINE_INVISIBLE_CVS_DIRECTORIES    	(default: uncommented)
		
		Uncomment the #define CVSENGINE_INVISIBLE_CVS_DIRECTORIES to ensure that
		all the "CVS" administrative directories are created with the invisible
		bit set. This does not change the visibility of "CVS" directories that
		have already been created.
		
		
		CVSENGINE_ASYNCFILEIO										(default: uncommented)
		
		This #define controls whether or not MacCVS Pro uses chained async-io
		during AppleSingle file decoding.
	
		
		CVSENGINE_USE_MIME_FILE_NAMES						(default: commented out)
		
		Experimental code that enables MacCVS to encode/decode file names
		using MIME conventions so you can checkin files with '/' characters
		and other symbols that are usually illegal in CVS.
			
*/


/*
#define CVSENGINE_KCLIENT_SUPPORT 1
*/

/*
#define CVSENGINE_TERMINATE_WITH_LF 1
*/


#define CVSENGINE_INVISIBLE_CVS_DIRECTORIES 1



#define CVSENGINE_ASYNCFILEIO 1


/*
#define CVSENGINE_USE_MIME_FILE_NAMES 1
*/


/*	-------------------------------------------------------------------------
		** Translation Settings Constants
		------------------------------------------------------------------------- */

enum
	{
	
		kCVSEngineLFTranslation						= 0x00000000,
		kCVSEngineCRTranslation						= 0x00000001,
		kCVSEngineCRLFTranslation					= 0x00000002,
		kCVSEngineTextEncoding						= 0x00000000,
		kCVSEngineBinaryEncoding					= 0x00000001,
		kCVSEngineAppleSingleEncoding		  = 0x00000002,
		kCVSEngineBinhex4Encoding					= 0x00000003,
		kCVSEngineMacBinaryEncoding				= 0x00000004,
		kCVSEngineRBLEncoding							= 0x00000005
	
	};
			

/*	-------------------------------------------------------------------------
		** Merging Policy Settings Constants
		------------------------------------------------------------------------- */

enum
	{
	
		kCVSEngineNoMerging							= 0x00000000,
		kCVSEngineTextMerging						= 0x00000001,
		kCVSEngineTextAndBinaryMerging	= 0x00000002
	
	};


/*	-------------------------------------------------------------------------
		** File action identifiers, from checkout or update
		------------------------------------------------------------------------- */
enum
	{
		kCVSEngineFileUpdatedChar				= 'U',
		kCVSEngineFileAddedChar					= 'N',
		kCVSEngineFileTaggedChar				= 'T',
		kCVSEngineFileNotTaggedChar			= 'W',
		kCVSEngineFileUntaggedChar			= 'D',
		kCVSEngineFileMergedChar				= 'M',			// can also mean no local mods
		kCVSEngineFileConflictsChar			= 'C',
		kCVSEngineFileRemovedChar				= 'R',
		kCVSEngineFileLocallyAddedChar	= 'A',
		kCVSEngineFileUnknownChar				= '?'
	};


typedef UInt32	TEngineUpdateResponseType;

/*	-------------------------------------------------------------------------
		** Response line identifiers, from checkout or update
		------------------------------------------------------------------------- */
enum
	{
		kCVSEngineFileUpdatedResponse,
		kCVSEngineFileAddedResponse,
		kCVSEngineFileTaggedResponse,
		kCVSEngineFileUntaggedResponse,
		kCVSEngineFileNotTaggedResponse,
		kCVSEngineFileMergedResponse,
		kCVSEngineFileConflictsResponse,
		kCVSEngineFileRemovedResponse,
		kCVSEngineFileLocallyAddedResponse,
		kCVSEngineFileUnknownFileResponse,
		kCVSEngineFileOtherResponse
	};


/*	-------------------------------------------------------------------------
		** Default file types
		------------------------------------------------------------------------- */

enum
	{
		kCVSEngineDefaultFileType			= 'TEXT',
		kCVSEngineDefaultFileCreator 	= 'CWIE'
	};
	
	
/*	-------------------------------------------------------------------------
		** Progress Status Struct
		------------------------------------------------------------------------- */

	
struct CCVSEngineProgressStatusRec
	{
	
		UInt32	fMax;
		UInt32	fCompleted;
		
	};
	

typedef struct CCVSEngineProgressStatusRec CCVSEngineProgressStatusRec;
	

/*	-------------------------------------------------------------------------
		** File Type Mapping and Translation Option Struct
		------------------------------------------------------------------------- */


struct CCVSEngineFileMappingRec
	{
	
		Str255		fFileNamePStr;
		OSType		fFileType;
		OSType		fFileCreator;
		UInt32		fTranslation;
		UInt32		fEncoding;

		#ifdef __cplusplus

			void Init()
				{
				
					fFileNamePStr[0] 			= 0x00;
					fFileType							= 0;
					fFileCreator					= 0;
					fTranslation					= 0;
					fEncoding							= 0;
					
				}
				
		#endif
	
	};


typedef struct CCVSEngineFileMappingRec CCVSEngineFileMappingRec;


struct CCVSEngineFolderCreationOptions
	{
	
			Boolean				mFoldersGetListView;
			Boolean				mFoldersInheritLabels;
	
		#ifdef __cplusplus

			void Init()
				{
					
					mFoldersGetListView = true;
					mFoldersInheritLabels = false;
				}

		#endif

	};
	

/*	-------------------------------------------------------------------------
		** Data passed to UCVSRepositoryUtils::CreateItemFromCVSPath()
		------------------------------------------------------------------------- */

struct CCVSEngineFileCreateInfo
	{
			
			OSType				mFileCreator;
			OSType				mFileType;
	
			ScriptCode		mScriptType;
			
			struct CCVSEngineFolderCreationOptions	mFolderOptions;
	
		#ifdef __cplusplus

			void Init()
				{
				
					mFileCreator 	= '????';
					mFileType			= '????';
					
					mScriptType		= smSystemScript;
					
					mFolderOptions.Init();

				}
				
			CCVSEngineFileCreateInfo( OSType inFileCreator, OSType inFileType, ScriptCode inScriptType, const CCVSEngineFolderCreationOptions &folderOptions )
				{
				
					mFileCreator 	= inFileCreator;
					mFileType			= inFileType;
					
					mScriptType		= inScriptType;
					
					mFolderOptions = folderOptions;
					
				}
				
		#endif
	};


/*	-------------------------------------------------------------------------
		** Options Structs for various CVS Engine calls
		------------------------------------------------------------------------- */

	
struct CCVSEngineAddOptions 
	{
		
		Boolean		mAutomagicallyEncode;			
		UInt32		mEncodingOptions;
	
		#ifdef __cplusplus

			void Init()
				{
				
					mAutomagicallyEncode 	= true;
					mEncodingOptions			= 0;
					
				}
				
		#endif
	
	};


/*	-------------------------------------------------------------------------
		** Options Structs for various CVS Engine calls
		------------------------------------------------------------------------- */


struct CCVSEngineCheckoutOptions 
	{
	
		Str255		mModulePStr;
		Str255		mRevisionPStr;
		Str255		mDatePStr;
		
		Boolean		mCheckoutDeep;
		Boolean		mResetStickyTags;
		Boolean		mUpdateEverything;
		Boolean		mUseCompression;
				
		UInt32		mCompressionLevel;
		UInt32		mMergePolicy;
		UInt32		mLFTranslation;

		struct CCVSEngineFolderCreationOptions		mFolderOptions;
		
		#ifdef __cplusplus
		
			void Init()
				{
				
					mModulePStr[0]		= 0;
					mRevisionPStr[0] 	= 0;
					mDatePStr[0]			= 0;

					mCheckoutDeep 		= true;
					mResetStickyTags	= true;
					mUpdateEverything	= true;
					mUseCompression 	= false;
					
					mCompressionLevel = 1;
					mMergePolicy			= kCVSEngineNoMerging;
					mLFTranslation		= kCVSEngineLFTranslation;
					
					mFolderOptions.Init();
				}
				
		#endif
		
	};
	
	
struct CCVSEngineCommitOptions 
	{
					
		Boolean		mUseFileRsrcComment;
		Handle		mCommentText;

		#ifdef __cplusplus

			void Init()
				{
				
					mUseFileRsrcComment = NULL;
					mCommentText 				= NULL;
					
				}
				
		#endif
	
	};
	

struct CCVSEngineDiffOptions
	{
	
		Boolean			mIgnoreCase;					// case insensitive							-i
		Boolean			mIgnoreWhiteSpace;		// ignore white space						-w
		Boolean			mIgnoreSpaceChange;		// ignore space change					-b
		Boolean			mIgnoreBlankLines;		// ignore blank lines 					-B
		Boolean			mShowCFunction;				// show C function							-p

		Boolean			mRCSDiff;							// RCS diff											-n
		Boolean			mContextDiff;					// true for context diff 				-c
		Boolean			mUnifiedDiff;					// UnifiedDiff									-u
		
		Int16				mContextLines;				// num lines of context or 0 for default		

		Boolean			mUseRevisions;				// use the revisions given here
		
		Str255			mOrigRevisionPStr;		// the revision of the original version of the file
		Str255			mDiffRevisionPStr;		// the revision of the update
		
		
		#ifdef __cplusplus
		
			// it is important that you don't change these Init() options,
			// because the compare parsing code relies on them being the defaults
			void Init()
				{
					
					mIgnoreCase 					= false;
					mIgnoreWhiteSpace			= false;
					mIgnoreSpaceChange		= false;
					mIgnoreBlankLines			= false;
					mShowCFunction				= false;
					
					mRCSDiff							= false;
					mContextDiff					= false;
					mUnifiedDiff					= false;
					mContextLines					= 2;
										
					mUseRevisions					= false;
					
					mOrigRevisionPStr[0]  = 0;
					mDiffRevisionPStr[0]	= 0;
					
				}
				
		#endif
	};
	
	
	

enum {
// Authentication types
		kAuthenticationPassword			=	FOUR_CHAR_CODE ('pass'),		// pserver
		kAuthenticationRSH					= FOUR_CHAR_CODE ('RSH '),		// RSH
		kAuthenticationKClient			=	FOUR_CHAR_CODE ('KClt'),		// Kerberos v4 via KClient
		kAuthentication_This_Is_A_Dummy_Value_That_Has_No_Comma_After_It
};

typedef UInt32	AuthenticationMethod;

struct CCVSEngineLoginOptions
	{
				
		UInt16		mPort;
		
		Str255		mServerName;
		Str255		mLocalUserName;
		Str255		mRemoteUserName;
		Str255		mPassword;
		Str255		mCVSCommandText;
		Str255		mCVSRoot;
		
		AuthenticationMethod		mAuthenticationMethod;

		#ifdef __cplusplus
		
			void Init()
				{
				
					mPort 							= 0x00000000;
					
					mServerName[0] 			= 0x00;
					mLocalUserName[0] 	= 0x00;
					mRemoteUserName[0] 	= 0x00;
					mPassword[0] 				= 0x00;
					
					mCVSCommandText[0] 	= 0x03;
					mCVSCommandText[1] 	= 'c';
					mCVSCommandText[2] 	= 'v';
					mCVSCommandText[3]	= 's';
					
					mCVSRoot[0] 				= 0x00;
					
					mAuthenticationMethod = kAuthenticationPassword;
				}
				
		#endif
		
	};
	
	
struct CCVSEngineImportOptions 
	{
	
		Str255		fRepositoryPStr;
		Str255		fVendorTagPStr;
		Str255		fReleaseTagPStr;
		Str255		fBranchTagPStr;
		Boolean		fSendBinaries;
		
		#ifdef __cplusplus
		
			void Init()
				{
				
					fRepositoryPStr[0] 	= 0x00;
					fVendorTagPStr[0] 	= 0x00;
					fReleaseTagPStr[0] 	= 0x00;
					fBranchTagPStr[0]		= 0x00;
					fSendBinaries 			= false;
					
				}
				
		#endif
	
	};

	
struct CCVSEngineTagOptions
	{
	
		Str255		fTagPStr;
		Boolean		fTagDeep;
		Boolean		fBranch;
		Boolean		fRemove;
		Boolean		fDontTagMROFiles;
		
		#ifdef __cplusplus
		
			void Init()
				{
				
					fTagPStr[0] 			= 0x00;
					fTagDeep					= true;
					fBranch						= false;
					fRemove						= false;
					fDontTagMROFiles	= false;
					
				}
				
		#endif
	
	};
	

struct CCVSEngineUpdateOptions 
	{

		Str255		mRevisionPStr;
		Str255		mDatePStr;

		Boolean		mUpdateDeep;
		Boolean		mGetNewDirectories;
		Boolean		mResetStickyTags;
		Boolean		mRevert;
		Boolean		mUseCompression;
				
		UInt32		mCompressionLevel;		
		UInt32		mMergePolicy;
		UInt32		mLFTranslation;

		#ifdef __cplusplus
		
			void Init()
				{

					mRevisionPStr[0] 		= 0,
					mDatePStr[0]				= 0,
				
					mUpdateDeep 				= true;
					mGetNewDirectories 	= false;
					mResetStickyTags		= false;
					mRevert							= false;
					mUseCompression 		= false;
					
					mCompressionLevel 	= 1;
					mMergePolicy				= kCVSEngineNoMerging;
					mLFTranslation			= kCVSEngineLFTranslation;
					
				}
				
		#endif
	
	};
	
				
struct CCVSEngineWatchOptions
	{
		Boolean	mNotification;
	
		#ifdef __cplusplus
			
			void Init()
				{
					mNotification = true;
				}
		
		#endif
	};
	

/*	-------------------------------------------------------------------------
		** Prototypes for CVS Engine Notifiers
		------------------------------------------------------------------------- */


typedef pascal SInt32	(*CCVSEngineEventNotifierProcPtr)(const CCVSEngineRef	 	inEngineRef,
																												const FSSpec					*inTargetItem,
																												const SInt32 					inEvent, 
																												const SInt32 					inError, 
																												const UInt32 					inCookie, 
																												const void 						*inDataPtr, 
																												void 									*inContextPtr);

// Data tyes for notifier proc

enum
	{
		kEngineNotifierTextDataType					= 'text',
		kEngineNotifierFSSpecDataType				= 'FSSp',
		kEngineNotifierNoDataType						= 'none'
	};
	
	
typedef pascal SInt32 (*CCVSEngineStreamNotifierProcPtr)(const CCVSEngineRef 	inEngineRef, 
																												const UInt32					inDataType,
																												const UInt32					inDataLen, 
																												const void 						*inDataPtr, 
																												void 									*inContextPtr);
			

enum
	{
	
		kCurrentCVSEngineVersion						= 1
		
	};
	
	
/*	-------------------------------------------------------------------------
		** CVS Engine Error Codes
		------------------------------------------------------------------------- */
	
enum
	{

		kCVSEngineFileHasNoCkid							= 501,
		kCVSEngineCkidCorrupted							= 502,
		kCVSEngineBadCkidChecksum						= 503,
		kCVSEngineFileHasBeenMoved					= 504,
		kCVSEngineIsFolder									= 505,
		kCVSEngineInvalidCVSPath						= 506,
		kCVSEngineAlreadyExists							= 508,
		kCVSEngineItemNotFound							= 509,
		kCVSEngineIsAlias										= 510,
		kCVSEngineFileHasNoMcvs							= 511,
		kCVSEngineMcvsCorrupted							= 512,
		kCVSEnginePathOverflow							= 513,
		kCVSEngineUnknownMcvsVersion				= 514,
		kCVSEngineUnknownCkidVersion				= 515,
		kCVSEngineCantDuplicateFileErr			= 516,
		kCVSEngineFileInvisibleErr					= 517,
// @@@pmb@@@
		kCVSEngineFileHasNoWindowPos				= 518,
		kCVSEngineNoRevisionDataErr					= 519,
		kCVSEngineFileNameTruncatedErr			= 520,
		kCVSEngineBadRootStrErr							= 521,
		kCVSEngineIsFile										= 522,	
			
		kCVSEngineNetworkTimeoutErr					= 606,
		
		kCVSEngineUnknownErr								= 700,
		kCVSEngineBadParamErr								= 701,
		kCVSEngineServerGaveBadResponseErr	= 703,
		kCVSEngineInvalidFileNameErr				= 704,
		kCVSEngineOutOfMemoryErr						= 705,
		kCVSEngineSkippedFile								= 706,
		kCVSEngineInvalidAppleSingleDataErr	= 707,
		kCVSEngineOperationAbortedErr				= 708,
		kCVSEnginePermissionsErr						= 709,
		kCVSEngineBinaryMergingOffErr				= 710,
		kCVSEngineServerReturnedErr					= 711,
		kCVSEngineUnknownExceptionErr				= 712,
		kCVSEngineFileNotChangedErr					= 713,
		kCVSEngineFileLocallyModifiedErr		= 714,
		kCVSEngineNotValidEncodedDataErr		= 715,
		
		kCVSEngineBadPasswordErr						= 911,
		kCVSEngineNoPortsAvailableErr				= 912
	
	};
	

/*	-------------------------------------------------------------------------
		** CVS Engine Event Codes (for CCVSEngineEventNotifierProc)
		------------------------------------------------------------------------- */
	
enum
	{
	
		kCVSEngineEntryPreActionEvent					= 0x00010001,
		kCVSEngineEntryPostActionEvent				= 0x00010002,
		
		kCVSEngineUpdatePreActionEvent				= 0x00010003,
		kCVSEngineUpdatePostActionEvent				= 0x00010004,
		
		kCVSEngineCheckoutPreActionEvent			= 0x00010005,
		kCVSEngineCheckoutPostActionEvent			= 0x00010006,
		
		kCVSEngineAddPreActionEvent						= 0x00010007,
		kCVSEngineAddPostActionEvent					= 0x00010008,
		
		kCVSEngineCommitPreActionEvent				= 0x00010009,
		kCVSEngineCommitPostActionEvent				= 0x00010010,
		
		kCVSEngineDecodingPreActionEvent			= 0x00010011,
		kCVSEngineDecodingPostActionEvent			= 0x00010012,

		kCVSEngineProgressEvent								= 0x00010013,
		kCVSEngineYieldActionEvent						= 0x00010014,
		
		kCVSEngineModifiedEntryPreActionEvent = 0x00010015,
		kCVSEngineModifiedEntryPostActionEvent= 0x00010016,
		
		kCVSEngineErrorActionEvent						= 0x00010017,
		
		kCVSEngineRemovePreActionEvent				= 0x00010018,
		kCVSEngineRemovePostActionEvent				= 0x00010019,
		
		kCVSEngineUpdateDirectoryPreEvent			= 0x00010020,
		kCVSEngineUpdateDirectoryPostEvent		= 0x00010021,
		
		kCVSEngineCommandActionEvent					= 0x00010022
	
	};
	

/*	-------------------------------------------------------------------------
		** C API for the CVS Engine
		------------------------------------------------------------------------- */


pascal OSErr	CreateCVSEngine( 							const FSSpec											*inLocalRoot,
																						CCVSEngineFileMappingRec					*inFileMappings,
																						const UInt32											inFileMappingsCount,
																						CCVSEngineEventNotifierProcPtr		inNotifier,
																						CCVSEngineStreamNotifierProcPtr		inMessageNotifier,
																						CCVSEngineStreamNotifierProcPtr		inErrorNotifier,
																						void															*inContextPtr,
																						const UInt32											inTimeOut,
																						CCVSEngineRef		 									*outEngineRef);
																						
pascal void		DisposeCVSEngine( 						CCVSEngineRef											inEngineRef);

pascal OSErr	CVSEngineLogin(								CCVSEngineRef											inEngineRef,
																						const CCVSEngineLoginOptions			*inOptions);
																						
pascal OSErr	CVSEngineAdd(									CCVSEngineRef											inEngineRef,
																						const FSSpec											*inTargetItem,
																						const CCVSEngineAddOptions				*inOptions);
																						
pascal OSErr	CVSEngineCheckout(						CCVSEngineRef											inEngineRef,
																						const FSSpec											*inTargetItem,
																						const CCVSEngineCheckoutOptions		*inOptions);
																						
pascal OSErr	CVSEngineCommit(							CCVSEngineRef											inEngineRef,
																						const FSSpec											*inTargetItem,
																						CCVSEngineCommitOptions						*inOptions);
																						
pascal OSErr	CVSEngineDiff(								CCVSEngineRef											inEngineRef,
																						const FSSpec											*inTargetItem,
																						const CCVSEngineDiffOptions				*inOptions,
																						CCVSEngineStreamNotifierProcPtr		inDiffNotifier,
																						void															*inContextPtr);

pascal OSErr	CVSEngineEdit(								CCVSEngineRef											inEngineRef,
																						const FSSpec 											*inTargetItem);

pascal OSErr	CVSEngineEditors(							CCVSEngineRef											inEngineRef,
																						const FSSpec 											*inTargetItem,
																						CCVSEngineStreamNotifierProcPtr		inNotifier,
																						void															*inContextPtr);

pascal OSErr 	CVSEngineImport( 							CCVSEngineRef 										inEngineRef, 
																						const FSSpec 											*inTargetItem, 
																						const CCVSEngineImportOptions 		*inOptions);
																						
pascal OSErr	CVSEngineLog(									CCVSEngineRef											inEngineRef,
																						const FSSpec											*inTargetItem,
																						CCVSEngineStreamNotifierProcPtr		inLogNotifier,
																						void															*inContextPtr);
																						
pascal OSErr	CVSEngineStatus(							CCVSEngineRef											inEngineRef,
																						const FSSpec 											*inTargetItem,
																						CCVSEngineStreamNotifierProcPtr		inStatusNotifier,
																						void															*inContextPtr);
																						
pascal OSErr	CVSEngineTag(									CCVSEngineRef										 	inEngineRef,
																						const FSSpec 											*inTargetItem,
																						const CCVSEngineTagOptions				*inTagOptions);

pascal OSErr	CVSEngineRemove(							CCVSEngineRef											inEngineRef,
																						const FSSpec 											*inTargetItem);

pascal OSErr	CVSEngineUnEdit(							CCVSEngineRef											inEngineRef,
																						const FSSpec 											*inTargetItem);

pascal OSErr	CVSEngineUpdate(							CCVSEngineRef											inEngineRef,
																						const FSSpec 											*inTargetItem, 
																						const CCVSEngineUpdateOptions			*inOptions);

pascal OSErr	CVSEngineWatchOnOff(					CCVSEngineRef											inEngineRef,
																						const FSSpec											*inTargetItem,
																						const Boolean											inWatch);
																						
pascal OSErr  CVSEngineWatchAddRemove(			CCVSEngineRef											inEngineRef,
																						const FSSpec											*inTargetItem,
																						const CCVSEngineWatchOptions			*inOptions);
																										
pascal OSErr	CVSEngineWatchers(						CCVSEngineRef											inEngineRef,
																						const FSSpec 											*inTargetItem,
																						CCVSEngineStreamNotifierProcPtr		inNotifier,
																						void															*inContextPtr);

pascal SInt32	CVSEngineVersion(							CCVSEngineRef											inEngineRef);
				

/*	-------------------------------------------------------------------------
		** C++ API for the CVS Engine
		------------------------------------------------------------------------- */

			
#ifdef __cplusplus


class CCVSEngine

	{
	
		public: 
		
			explicit				CCVSEngine(							const FSSpec											*inLocalRoot,
																							CCVSEngineFileMappingRec					*inFileMappings,
																							const UInt32											inFileMappingsCount,
																							CCVSEngineEventNotifierProcPtr		inNotifier,
																							CCVSEngineStreamNotifierProcPtr		inMessageNotifier,
																							CCVSEngineStreamNotifierProcPtr		inErrorNotifier,
																							void															*inContextPtr);

										 	~CCVSEngine();

			virtual OSErr		Login(									const CCVSEngineLoginOptions			*inOptions);
						
			virtual OSErr		Add(										const CCVSEngineAddOptions				*inOptions);
																																	
			virtual OSErr		Checkout(								const FSSpec 											*inTargetItem,
																							const CCVSEngineCheckoutOptions		*inOptions);

																							
			virtual OSErr 	Commit(									const CCVSEngineCommitOptions			*inOptions);

			virtual OSErr		Diff(										const FSSpec											*inTargetItem,
																						 	const CCVSEngineDiffOptions				*inOptions,
																						 	CCVSEngineStreamNotifierProcPtr		inDiffNotifier,
																						 	void															*inContextPtr);
																							
			virtual OSErr 	Log(										const FSSpec	 										*inTargetItem,
																							CCVSEngineStreamNotifierProcPtr		inLogNotifier,
																							void															*inContextPtr);


			virtual OSErr		Status(									const FSSpec 											*inTargetItem,
																							CCVSEngineStreamNotifierProcPtr		inStatusNotifier,
																							void															*inContextPtr);
		
			virtual OSErr		Tag(										const FSSpec 											*inTargetItem,
																							const CCVSEngineTagOptions				*inTagOptions);

			virtual OSErr		Remove(									const FSSpec 											*inTargetItem);

			virtual OSErr		Update(									const FSSpec 											*inTargetItem, 
																							const CCVSEngineUpdateOptions			*inOptions);
			
			virtual SInt32	CVSEngineVersion();
			
	};
	
	
#endif


#endif
