//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CVSEngineUtils.h
	
	Authors:		Scott M. Silver and Ram Rajadhyaksha
	
	Description:
	
		This file contains all sorts of useful stuff for the CVS engine.
		 
	Modification History:

		15 	Dec		1998		Ram Rajadhyaksha
		
					Integrated Peter Bierman's changes.
										
-------------------------------------------------------------------------------- */


#ifndef __CVSENGINEUTILS__
#define __CVSENGINEUTILS__


#include <Types.h>
#include "CVSEngine.h"


#ifdef __cplusplus
extern "C" {
#endif


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif


struct CkidResourceHeader
	{

		UInt32		fChecksum;
		UInt32		fLocation;
		UInt16		fCurrentVersion;
		UInt16		fCheckedOutBySomeone;
		UInt8			fBranch;
		UInt8			fModifyReadOnly;
		UInt32		fUnused;
		UInt32		fCheckoutTime;
		UInt32		fFilesModificationDate;
		UInt32		fProjectID1;
		UInt32		fProjectID2;
		UInt16		fUserID;
		UInt16		fFileID;
		UInt16		fRevisionID;
		
		//	After this follows:
		//	Path (PString), \0
		//	Username (PString), \0
		//	Revision (PString), \0
		//	FileName (PString), \0
		//	Task (PString), \0
		//	Comment Length
		//	Comment string, \0
		
	};

	
typedef struct CkidResourceHeader CkidResourceHeaderRec, *CkidResourceHeaderPtr, **CkidResourceHeaderHndl;



struct McvsResourceVer1Header
	{
	
		UInt32	fVersion;
	
	};
	

typedef struct McvsResourceVer1Header McvsResourceVer1HeaderRec, *McvsResourceVer1HeaderPtr, **McvsResourceVer1HeaderHndl;

	
struct McvsResourceHeader
	{
	
		UInt32		fVersion;
		UInt32		fFlags;
		UInt32		fRefCon;
		
		//  After this follows:
		//  Remote File Name (PString), \0
		//  Revision (PString), \0
		//  Conflicts (PString), \0
		//  Options (PString), \0
		//  Current Tag (PString), \0
		//  Unused1 (PString), \0
		//  Unused2 (PString), \0
		//  Unused3 (PString), \0
		//  Unused4 (PString), \0
		//  UInt32 flags
		//  UInt32 unused5
		//  UInt16 remotePathLen
		//  Remote Path String
		
	};
	

typedef struct McvsResourceHeader McvsResourceHeaderRec, *McvsResourceHeaderPtr, **McvsResourceHeaderHndl;


struct CkidResourcePB 
	{

		UInt32		fChecksum;
		UInt32		fLocation;
		UInt16		fCurrentVersion;
		UInt16		fCheckedOutBySomeone;
		UInt8			fBranch;
		UInt8			fModifyReadOnly;
		UInt32		fUnused;
		UInt32		fCheckoutTime;
		UInt32		fFilesModificationDate;
		UInt32		fProjectID1;
		UInt32		fProjectID2;
		UInt16		fUserID;
		UInt16		fFileID;
		UInt16		fRevisionID;
		Str255		fRemotePathPStr;
		Str255		fUserNamePStr;
		Str255		fRevisionPStr;
		Str255		fFileNamePStr;
		Str255		fTaskPStr;
		UInt16		fCommentLen;
		Handle		fCommentHndl;
		
	};


typedef struct CkidResourcePB CkidResourcePB;


struct McvsResourcePB
	{
	
		UInt32		fVersion;
		UInt32		fFlags;
		UInt32		fRefCon;
		Str255		fFileNamePStr;
		Str255		fRevisionPStr;
		Str255		fConflictsPStr;
		Str255		fOptionsPStr;
		Str255		fTagPStr;
		Str255		fUnused1PStr;
		Str255		fUnused2PStr;
		Str255		fUnused3PStr;
		Str255		fUnused4PStr;
		Ptr				fRemotePathStr;
		UInt16		fRemotePathLen;
		
	};
	
	
typedef struct McvsResourcePB McvsResourcePB;


struct CkidMcvsResourcePB
	{
	
		CkidResourcePB	ckid;
		McvsResourcePB 	mcvs;
	
	};
	

typedef struct CkidMcvsResourcePB CkidMcvsResourcePB;


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif


enum 
	{
	
		kCurrentCkidVersion 			= 0x00000004,
		kMcvsVersion1Format				= 0xFABB0002,
		kCurrentMcvsVersion				= 0xFABB0003,
		kFileAppleSingleEncoded		= 1 << 0,
		kDefaultCkidLocValue			= 1071985200,
		kCkidResourceID						= 128,
		kMcvsResourceID						= 128,
		kMPSRResourceID						= 1005

	};


enum
	{
		kViewFolderByIcon,
		kViewFolderByName,
		kViewFolderBySize,
		kViewFolderByDate,
		kViewFolderByKind,
		kViewFolderLookDontTouch		// debugging flag
	};
	

// bits for testing pb.hFileInfo.ioFlAttrib to see if file is open
enum
	{
	
		kResForkOpen			= 0x0004,
		kDataForkOpen			= 0x0008,
		kBothForksOpen		= 0x0080
	
	};
	
#define FILE_OPEN(pb)		( (pb.hFileInfo.ioFlAttrib & kResForkOpen) || \
													(pb.hFileInfo.ioFlAttrib & kDataForkOpen) || \
													(pb.hFileInfo.ioFlAttrib & kBothForksOpen) )

#define FSpIsValid(fsSpec) 	( (fsSpec.name[0] > 0) && \
														(fsSpec.parID != 0) && \
														(fsSpec.vRefNum != 0) )
																																								
pascal Boolean	CUIsValidCVSName( 								 	ConstStr255Param				inNamePStr);	

pascal OSErr 		CUIsValidMacName( 									ConstStr255Param 				inNamePStr);

pascal OSErr		CUTranslateFromMacNamePStr( 				ConstStr255Param 				inNamePStr, 
																										Str255 									pstrOutTranslation);
																																																				
pascal OSErr		CUTranslateToMacName(								char										*inOutDataStr,
																										UInt32									&inOutDataLen );

pascal SInt32		CUCompareCVSVersion(							 	char 										*inVersion1CStr, 
																										char 										*inVersion2CStr);

pascal OSErr 		CUISOTimeStrToSeconds( 							char 										*strInTime, 
																										SInt32 									*intOutSeconds);

pascal OSErr 		CUISOTimeStrFromSeconds( 						char 										*strInOutTime, 
																										UInt32 									*intInOutLen, 
																										const SInt32 						intInSeconds );
	
pascal UInt32 	CUUnixTimeStringToMacSeconds( 			char 										*inTimeStr, 
																										SInt32 									*inOutGMTDelta);

pascal Boolean 	CUIsCVSBinaryOption( 								ConstStr255Param 				inOptionPStr);
pascal Boolean 	CUIsAddedOrRemovedRevision( 				ConstStr255Param 				pstrInRevision);

pascal void 		CUMakeDiffCommandLineOptionsString( const CCVSEngineDiffOptions *inDiffOptions,
																										Str255 									outOptionsPStr,
																										Boolean									inDisplayFormat);

pascal OSErr 		CUCreateRootPStr( 									const AuthenticationMethod 	intInMethod, 
																										const Str255 						pstrInUser, 
																										const Str255 						pstrInServer, 
																										const UInt16 						intInPort,
																										const Str255						pstrInPath,
																										Str255									pstrOutRoot);

pascal OSErr		CUParseRootPStr( 										ConstStringPtr					pstrInRoot, 
																										Str255 									pstrOutUser, 
																										Str255 									pstrOutHost, 
																										Str255 									pstrOutPath, 
																										AuthenticationMethod 		*intOutMethod, 
																										UInt16 									*intOutPort);

pascal OSErr 		CUParsePermissionPStr( 							ConstStringPtr					pstrInItem, 
																										Boolean 								*bOutReadOnly);

pascal OSErr 		CUParsePermissionStr( 							const char 							*strInItem, 
																										Boolean 								*bOutReadOnly);

pascal OSErr		CUGetAddressAndPort(								ConstStringPtr					pstrInItem,
																										Str255									pstrOutAddress,
																										UInt16									*intOutPort);
																										
pascal OSErr		FSpSafeCreate(										 	FSSpec									*inOutTargetSpec,
																										const OSType						inFileCreator,
																										const OSType						inFileType,
																										const ScriptCode				inScriptType);

pascal OSErr		FSpSafeDuplicate(										const FSSpec						*inTargetSpec);

pascal OSErr		FSpSafeRename(										 	FSSpec									*inOutTargetItem);

// @@@pmb@@@
pascal OSErr		CUReadWinPosResourceFromRef(				Handle									*winPosRsrcH,
																										const SInt16						inFileRef );

// @@@pmb@@@
pascal OSErr		CUWriteWinPosResourceFromRef(				Handle									winPosRsrcH,
																										const SInt16						inFileRef );
																										
// @@@pmb@@@
pascal OSErr		FSpWriteWinPosResource(							const FSSpec						*inTargetSpec,
																										Handle									winPosRsrcH ); 

			
pascal OSErr		CUReadCkidResourceFromRef(					CkidResourcePB 					*outCkidPB, 
																										const Boolean 					inGetComment, 
																										const SInt16 						inFileRef);
																																														
pascal OSErr		FSpReadCkidResource(								const FSSpec						*inTargetSpec,
																										CkidResourcePB					*outCkidPB,
																										const Boolean						inGetComment);
			
pascal OSErr		CUWriteCkidResourceFromRef(					CkidResourcePB					*inCkidPB,
																										const SInt16						inFileRef);
																																												
pascal OSErr		FSpWriteCkidResource(								const FSSpec						*inTargetSpec,
																										CkidResourcePB					*inCkidPB);

pascal OSErr 		CURemoveCkidMcvsResourceFromRef( 		const Boolean 					inRemoveCkid,
																										const Boolean 					inRemoveMcvs,
																										const SInt16 						inFileRef);


pascal OSErr		FSpRemoveCkidMcvsResource( 					const FSSpec 						*inTargetSpec,
																										const Boolean 					inRemoveCkid,
																										const Boolean 					inRemoveMcvs);
					
pascal OSErr 		FSpCheckForMovement(								const FSSpec						*inTargetSpec,
																										const FSSpec						*inProjectRoot,
																										CkidMcvsResourcePB			*inInfo);
																		
pascal OSErr 		FSpGetPartialPath( 									const FSSpec 						*inTargetSpec, 
																										const FSSpec						*inBaseItem, 
																										UInt32 									*inOutPathLength,
								   																	char 										*outPathStr);
			
pascal OSErr 		FSpGetLocalCVSPath( 								const FSSpec 						*inTargetSpec, 
																										const FSSpec 						*inProjectRoot, 
																										char 										*outPathStr,
																										UInt32									*inOutPathLen );
			
pascal OSErr		FSpGetFromLocalCVSPath( 						const Str255 						inCVSPathPStr,
																										const FSSpec						*inProjectRoot, 
																										FSSpec									*outTargetItem);
	
pascal OSErr		FSpGetDirID(												const FSSpec						*inTargetSpec,
																										SInt32									*outDirID);

pascal OSErr 		FSpSetFolderListView(								const FSSpec 						*inTargetSpec,
																										SInt16 									inFolderView );

pascal OSErr		FSpIsAFile(													const FSSpec 						*inTargetSpec,
																										Boolean									*outFileIsOpen);

pascal OSErr 		FSpIsAlias( 												const FSSpec 						*inTargetSpec);

pascal Boolean 	FSpIsLocked( 												const FSSpec 						*inTargetSpec);

pascal SInt32		FSpGetModDate( 											const FSSpec				 		*inTargetSpec);
				
pascal OSErr		FSpSetModDate( 											const FSSpec 						*inTargetSpec, 
																										const SInt32 						inModDate);
																																														
pascal Boolean 	FSpIsEqual(													const FSSpec 						*inItemOne, 
																										const FSSpec						*inItemTwo);	

pascal Boolean	FSpIsChildOf(												const FSSpec						*inChildItem,
																										const FSSpec						*inParentItem);

pascal OSErr 		FSpGetParentFSSpec( 								const FSSpec						*inTargetSpec, 
																										FSSpec 									*outParentItem);
																											
pascal OSErr		FSpGetFullPathStr(									const FSSpec						*inTargetSpec,
																										UInt32									*inOutPathLen,
																										char										*outPathStr);

pascal OSErr 		FSpCreateTheDamnRsrcFork( 					const FSSpec 						*pInTarget);

pascal OSErr 		FSpCountNumFiles( 									const FSSpec 						*inFolderSpec, 
																										UInt32 									*outNumItems);

pascal OSErr 		FSpGetNthItem( 											const FSSpec 						*inFolderSpec, 
																										UInt32 									itemIndex, 
																										FSSpec 									*outItemSpec);

pascal OSErr		CUIsFileModified(										const CkidResourcePB		*pInPB,
																										ConstStringPtr					pstrInTimeStamp,
																										const SInt32						intInModDate);
																										
pascal Boolean  CUIsRepositoryDir( 									const SInt16 						inVRefNum, 
																										const SInt32 						inParID);

pascal OSErr 		FSpGetRootFileSpec( 								const FSSpec 						*inFolderSpec,
																										FSSpec 									*outTagFileSpec );

pascal Boolean  FSpIsRepositoryDir( 								const FSSpec						*inTargetSpec);
					

pascal OSErr		CUCreateRepositoryDir(							const SInt16						inVRefNum,
																										const SInt32						inParID,
																										const OSType						inCreator,
																										const OSType						inType,
																										const ScriptCode				inScriptType);
																																																	
pascal OSErr		FSpGetRepository(										const FSSpec						*inTargetSpec,
																										Str255									outRepositoryPStr);
																										
pascal OSErr		FSpSetRepository( 									const FSSpec 						*inTargetSpec, 
																										Str255 									inRepositoryPStr);

pascal OSErr		FSpGetRoot(													const FSSpec						*inTargetSpec,
																										Str255									outRootPStr);
																										
pascal OSErr		FSpSetRoot( 												const FSSpec 						*inTargetSpec, 
																										Str255 									inRootPStr);

pascal OSErr		FSpGetFolderTag( 										const FSSpec 						*inTargetSpec, 
																										Str255 									outTagPStr);

pascal OSErr 		FSpSetFolderTag( 										const FSSpec 						*inTargetSpec, 
																										Str255 									inTagPStr, 
																										const OSType 						inFileCreator, 
																										const OSType 						inFileType, 
																										const ScriptCode 				inScriptType);
																											
pascal OSErr		FSpGetParentTag( 										const FSSpec 						*inTargetSpec, 
																										Str255 									outTagPStr);

pascal OSErr		FSpSetParentTag( 										const FSSpec 						*inTargetSpec, 
																										Str255 									inTagPStr);

pascal Boolean 	FSpIsStatic( 												const FSSpec 						*inTargetSpec);

pascal OSErr		FSpSetStatic(												const FSSpec						*inTargetSpec,
																										const Boolean						bInSticky, 
																										const OSType 						inFileCreator, 
																										const OSType 						inFileType, 
																										const ScriptCode 				inScriptType);
			
pascal TEngineUpdateResponseType IdentifyResponseType( 	const char 					*inResponseLine,
																												const UInt32 				inResponseLength );
																												
pascal void GetRelativePathFromResponse( 						const char 							*inResponseLine,
																										const UInt32 						inResponseLength,
																										UInt32 									*inOutPathLength,
								   																	char 										*outPathStr );

pascal OSErr		CUStripCVSPath( 										char									  *inPathStr,
																										UInt32									*inPathLen);

#ifdef __cplusplus
	}	
#endif


#endif
