#include <string.h>

#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "hqx.h"
#include "StFileAccess.h"
#include "CVSEngine.h"
#include "CVSEngineUtils.h"
#include "UCVSRepositoryUtils.h"

#include "CCVSEncoders.h"


OSErr FSpEncodeBinhex4(	FSSpec &inOutTargetSpec, CEncoderYielder inYielder, void *pInContext, const SInt16 inPerm )
	{
	
		OSErr						result 					= noErr;
		Boolean					bFileCreated		= false;
		SInt32					intFileLen			= 0;
		FSSpec					fsEncoded				= { 0, 0, "\p" };
		HQXContextRec		sHQXContext;
		
		memset( (char *)&sHQXContext, 0, sizeof(HQXContextRec) );
		
		try
			{
				sHQXContext.fYieldProc = inYielder;
				sHQXContext.fContext = pInContext;
				sHQXContext.fMaxCount = 0;
				//  create a unique new temp file for the encoding
				
				fsEncoded = inOutTargetSpec;
				result = FSpSafeCreate( &fsEncoded, 'CWIE', 'TEXT', smSystemScript );
				ThrowOSErr( result );
				
				bFileCreated = true;
	
				{				
					SInt16 intDatFileRef = 0;
					SInt16 intResFileRef = 0;
	
					StFileAccess stEncodedItem( fsEncoded );
					StFileAccess stTargetItem( inOutTargetSpec );
					
					stEncodedItem.SetDelete( true );

					//  Since HQXEncode does not open the target file
					//  with fsRdWrPerm, we do it here to protect from
					//  other evil threads.
					
					intDatFileRef = stTargetItem.GetFSDataForkRef( inPerm );
					intResFileRef = stTargetItem.GetFSResForkRef( inPerm );
	
					::GetEOF( intDatFileRef, &intFileLen );
					sHQXContext.fMaxCount = intFileLen;
					::GetEOF( intResFileRef, &intFileLen );
					sHQXContext.fMaxCount += intFileLen;
								
					sHQXContext.fDatFileRef	= stEncodedItem.GetFSDataForkRef();
					
					result = HQXEncode( inOutTargetSpec.name, inOutTargetSpec.vRefNum, inOutTargetSpec.parID, HQXDataSink, (long)&sHQXContext );
					ThrowOSErr( result );
					
					stEncodedItem.SetDelete( false );
				}			
			}			
		catch ( OSErr intException )
			{
				result = intException;
			}		
		catch ( ... )
			{
				result = kCVSEngineUnknownExceptionErr;
			}

		if ( result == noErr )
			{
				inOutTargetSpec = fsEncoded;
			}
			
		return result;
		
	}
		
		
OSErr FSpDecodeBinhex4( const FSSpec *inTargetSpec, CEncoderYielder inYielder, void *pInContext )
	{
	
		OSErr						result 					= noErr;
		Boolean					bFileRenamed		= false;
		SInt32					intFileLen			= 0;
		FSSpec					fsTarget				= { 0, 0, "\p" };
		FSSpec					fsEncoded				= { 0, 0, "\p" };
		HQXContextRec		sHQXContext;
		
		memset( (char *)&sHQXContext, 0, sizeof(HQXContextRec) );
		
		try
			{
				sHQXContext.fYieldProc = inYielder;
				sHQXContext.fContext = pInContext;
				sHQXContext.fMaxCount = 0;
				
				fsTarget = *inTargetSpec;
				fsEncoded = *inTargetSpec;
				result = FSpSafeRename( &fsEncoded );
				ThrowOSErr( result );
				bFileRenamed = true;
				
				sHQXContext.fTarget = fsTarget;
				
				StFileAccess stEncodedAccess( fsEncoded );				
				sHQXContext.fDatFileRef = stEncodedAccess.GetFSDataForkRef( fsRdWrPerm );
				::GetEOF( sHQXContext.fDatFileRef, &intFileLen );
				sHQXContext.fMaxCount = intFileLen;
				
				result = HQXDecode( HQXDataSource, HQXNameFilterProc, false, true, (long)&sHQXContext );				
				ThrowOSErr( result );
				
				stEncodedAccess.SetDelete( true );
			}				
		catch ( OSErr intException )
			{
				result = intException;
			}		
		catch ( ... )
			{
				result = kCVSEngineUnknownExceptionErr;
			}
			
		if ( (result != noErr) && (bFileRenamed) )
			{
				::FSpRename( &fsEncoded, fsTarget.name );
			}
			
		return result;

	}


OSErr	HQXDataSink( void *pInBuffer, long intInOutCount, long intInRefcon )
	{
		
		OSErr					result				= noErr;
		HQXContextRec *pHQXContext 	= (HQXContextRec *)intInRefcon;
		
		if ( pHQXContext != NULL )
			{		
				result = ::FSWrite( pHQXContext->fDatFileRef, &intInOutCount, pInBuffer );
				
				//  If we survived file writing, call our yield function.
				
				if ( result == noErr )
					{
						pHQXContext->fCurCount += intInOutCount;
						
						if ( pHQXContext->fYieldProc != NULL )
							result = (pHQXContext->fYieldProc)( pHQXContext->fContext, pHQXContext->fMaxCount, pHQXContext->fCurCount );			
					}
			}
		else result = kCVSEngineBadParamErr;
		
		return result;
	
	}


OSErr	HQXDataSource( void *pInBuffer, long *intInOutCount, long intInRefcon )
	{
	
		OSErr					result				= noErr;
		HQXContextRec *pHQXContext 	= (HQXContextRec *)intInRefcon;
		
		if ( pHQXContext != NULL )
			{		
				result = ::FSRead( pHQXContext->fDatFileRef, intInOutCount, pInBuffer );
				
				//  If we survived file reading, call our yield function.
				
				if ( result == noErr )
					{
						pHQXContext->fCurCount += *intInOutCount;
						
						if ( pHQXContext->fYieldProc != NULL )
							result = (pHQXContext->fYieldProc)( pHQXContext->fContext, pHQXContext->fMaxCount, pHQXContext->fCurCount );			
					}
			}
		else result = kCVSEngineBadParamErr;
		
		return result;
	
	}


OSErr HQXNameFilterProc( StringPtr strOutName, short *intOutVol, long *intOutDir, long intInRefcon )
	{
		
		OSErr					result				= noErr;
		HQXContextRec *pHQXContext 	= (HQXContextRec *)intInRefcon;
		
		if ( pHQXContext != NULL )
			{		
				*intOutVol = pHQXContext->fTarget.vRefNum;
				*intOutDir = pHQXContext->fTarget.parID;
				PStrCopy( pHQXContext->fTarget.name, strOutName, sizeof(Str31) );
			}
		else result = kCVSEngineBadParamErr;
		
		return result;

	}
	
	
OSErr FSpDecodeAppleSingle( const FSSpec *inTargetSpec, const OSType inCreator, const OSType inType, const ScriptCode inScript, CEncoderYielder inYielder, void *pInContext )
	{
	
		OSErr								result 								= noErr;

		Boolean							bBrainDamagedFormat		= false;
		Boolean							bCreatedFile					= false;
		Boolean							bRenamedFile					= false;
		Boolean							bCalledPreAction			= false;
	
		SInt32							intIOCount						= 0;
																
		UInt32							intMaxOffset					= 0;
		UInt32							intIndex							= 0;
		UInt32							intCurOffset					= 0;
		UInt32							intBytesToCopy				= 0;
		UInt32							intBufSize						= 0;
		
		char								*pBuffer							= NULL;										
		FSSpec							encodedItem						= { 0, 0, "\p" };
		FSSpec							targetSpec						= { 0, 0, "\p" };

		AppleSingleHeader		theHeader;
		AppleSingleEntry		theEntry;
		FInfo								finderInfo;
	
		#ifdef CVSENGINE_ASYNCFILEIO	
		FSAsyncCopyContextPtr	pCopyContext = NULL;		
		#endif
		
		try
			{						
				if ( inTargetSpec == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
					
				pBuffer = ::NewPtr( kDefaultBufferSize );
				
				if ( pBuffer == NULL )
					{
						result = memFullErr;
					}
				else
					{
						result = ::MemError();
					}
					
				ThrowOSErr( result );
				
				intBufSize = kDefaultBufferSize;
	
				#ifdef CVSENGINE_ASYNCFILEIO
				
				pCopyContext = (FSAsyncCopyContextPtr)::NewPtr( sizeof(FSAsyncCopyContextRec) );
				
				if ( pCopyContext == NULL )
					{
						result = memFullErr;
					}
				else
					{
						result = ::MemError();
					}
										
				ThrowOSErr( result );
				
				#endif
						
//  Rename the file we're trying to decode to something squirley...
				
				encodedItem = *inTargetSpec;
						
				result = FSpSafeRename( &encodedItem );
				ThrowOSErr( result );					
				bRenamedFile = true;
												
				targetSpec = *inTargetSpec;
						
				::FSpCreateResFile( &targetSpec, inCreator, inType, inScript );
				result = ::ResError();
				ThrowOSErr( result );						
				bCreatedFile = true;

//  Open both file forks for locking from other threads. Scope
//  is important here because StFileAccess will close files when
//  destroyed.
					
				{				
					SInt16					oldDataForkRef	= 0;
					SInt16					oldResForkRef		= 0;
					SInt16					dataForkRef			= 0;
					SInt16					resForkRef			= 0;
					
					StFileAccess  	targetItemAccess( targetSpec );
					StFileAccess		encodedItemAccess( encodedItem );
					
					targetItemAccess.SetDelete( true );
					
					oldDataForkRef 	= encodedItemAccess.GetFSDataForkRef();
					oldResForkRef 	= encodedItemAccess.GetFSResForkRef();
					
					dataForkRef 		= targetItemAccess.GetFSDataForkRef();
					resForkRef 			= targetItemAccess.GetFSResForkRef();
															
					result = ::GetEOF( oldDataForkRef, (SInt32 *)&intMaxOffset );
					ThrowOSErr( result );
												
					if ( intMaxOffset < sizeof(AppleSingleHeader) )
						{						
							result = kCVSEngineInvalidAppleSingleDataErr;
							
							throw result;					
						}
					
					intIOCount = sizeof( AppleSingleHeader );
						
					result = FSReadOffset( oldDataForkRef, (UInt32 *)&intIOCount, &theHeader, 0 );
					ThrowOSErr( result );
					
					if ( (theHeader.fMagicNumber != kAppleSingleMagicNumber) && (theHeader.fMagicNumber != kAppleDoubleMagicNumber) )
						{						
							result = kCVSEngineInvalidAppleSingleDataErr;

							throw result;							
						}
						
//  Detect for brain damaged MacCVS v1.x format.
					
					if ( theHeader.fVersion == kWeirdoAppleDoubleVersion )		
						{		
							bBrainDamagedFormat = true;				
						}
					else if ( theHeader.fVersion != kAppleSingleVersion )
						{						
							result = kCVSEngineInvalidAppleSingleDataErr;
	
							throw result;						
						}
						
//  MacCVS v1.x encoded files with an AppleDouble signature and 
//  "Macintosh" in the 16 bytes filler field. Who knows why...

					if ( bBrainDamagedFormat == true )
						{								
							if ( !((theHeader.fFiller1 == 'Maci') && (theHeader.fFiller2 == 'ntos') && (theHeader.fFiller3 == 'h   ') && (theHeader.fFiller4 == '    ')) )
								{							
									result = kCVSEngineInvalidAppleSingleDataErr;

									throw result;									
								}
								
//  There are really 3 entries: data, rsrc, and FInfo. Why the field
//  was marked as 2...well, maybe Elvis knows or something...
						
							else if ( theHeader.fEntries == 2 )		
								{					
									theHeader.fEntries++;					
								}			
							else 
								{							
									result = kCVSEngineInvalidAppleSingleDataErr;
									
									throw result;								
								}								
						}
							
					else if ( (theHeader.fFiller1 != 0) || (theHeader.fFiller2 != 0) || (theHeader.fFiller3 != 0) || (theHeader.fFiller4 != 0) )
						{					
							result = kCVSEngineInvalidAppleSingleDataErr;
				
							throw result;							
						}
				
					intCurOffset = sizeof(AppleSingleHeader) + ( theHeader.fEntries * sizeof(AppleSingleEntry) );
																											
					for ( intIndex = 0; ((intIndex < theHeader.fEntries) && (result == noErr)); intIndex++ )
						{					
							intIOCount = sizeof(AppleSingleEntry);
							
							result = FSReadOffset( oldDataForkRef, (UInt32 *)&intIOCount, &theEntry, (sizeof(AppleSingleHeader) + intIndex * sizeof(AppleSingleEntry)) );
							ThrowOSErr( result );
					
//  The MacCVS 1.0x file format offsets are completely incorrect. 
//  Here we correct them.
												
							if ( bBrainDamagedFormat == true )
								{										
									switch ( intIndex )
										{									
											case 0:												
												if ( theEntry.fEntryID != kAppleSingleDataFork )												
													result = kCVSEngineInvalidAppleSingleDataErr;												
												else theEntry.fOffset = intCurOffset;												
											break;
																						
											case 1:											
												if ( theEntry.fEntryID != kAppleSingleResFork )													
													result = kCVSEngineInvalidAppleSingleDataErr;													
												else theEntry.fOffset = intCurOffset;											
											break;
											
											case 2:
												if ( theEntry.fEntryID != kAppleSingleMacFInfo )													
													result = kCVSEngineInvalidAppleSingleDataErr;													
												else theEntry.fOffset = intCurOffset;											
											break;											
										}								
								}
									
//  Do some sanity checking.
							
							if ( theEntry.fOffset > intMaxOffset )	
								{								
									result = kCVSEngineInvalidAppleSingleDataErr;								
								}
							else if ( (theEntry.fOffset + theEntry.fLength) > intMaxOffset )								
								{
									result = kCVSEngineInvalidAppleSingleDataErr;
								}
									
							ThrowOSErr( result );

//  Write the entry to the file. If there are multiple entries with the same type, 
//  the first ones will be overwritten.

							switch ( theEntry.fEntryID )
								{								
									case kAppleSingleDataFork:																			
										result = ::SetEOF( dataForkRef, theEntry.fLength );
										ThrowOSErr( result );
											
										intBytesToCopy = theEntry.fLength;
										
										#ifdef CVSENGINE_ASYNCFILEIO
										
										memset( pCopyContext, 0, sizeof(FSAsyncCopyContextRec) );
										
										pCopyContext->sSrcRef = oldDataForkRef;
										pCopyContext->sDestRef = dataForkRef;
										pCopyContext->sBuffer = pBuffer;
										pCopyContext->sBufMax = intBufSize;
										pCopyContext->sCopyLen = intBytesToCopy;
										pCopyContext->sSrcOffset = theEntry.fOffset;
										
										result = FSAsyncCopy( pCopyContext );
										ThrowOSErr( result );
										
										while ( !(pCopyContext->sFinished) )
											{
												if ( (inYielder != NULL) && (result == noErr) )
													{
														result = (inYielder)( pInContext, intMaxOffset, theEntry.fOffset + pCopyContext->sBytesCopied );							
													}
													
												if ( result != noErr )
													{
														pCopyContext->sKillFlag = true;
													}
											}

										ThrowOSErr( result );
										ThrowOSErr( pCopyContext->sResult );											
			
										#else
										
										while ( intBytesToCopy > 0 ) 
											{												
												intIOCount = intBytesToCopy;
												
												if ( intIOCount > intBufSize )
													intIOCount = intBufSize;
													
												result = FSReadOffset( oldDataForkRef, (UInt32 *)&intIOCount, pBuffer, theEntry.fOffset + (theEntry.fLength - intBytesToCopy) );
												ThrowOSErr( result );
													
												result = FSWriteOffset( dataForkRef, (UInt32 *)&intIOCount, pBuffer, theEntry.fLength - intBytesToCopy );
												ThrowOSErr( result );
													
												intBytesToCopy -= intIOCount;
		
												if ( inYielder != NULL )
													result = (inYielder)( pInContext, intMaxOffset, theEntry.fOffset + (theEntry.fLength - intBytesToCopy) );
											
												ThrowOSErr( result );
											}	
									
											#endif																			
									break;
									
									case kAppleSingleResFork:
										
										result = ::SetEOF( resForkRef, theEntry.fLength );
										ThrowOSErr( result );
												
										intBytesToCopy = theEntry.fLength;

										#ifdef CVSENGINE_ASYNCFILEIO
										
										memset( pCopyContext, 0, sizeof(FSAsyncCopyContextRec) );
										
										pCopyContext->sSrcRef = oldDataForkRef;
										pCopyContext->sDestRef = resForkRef;
										pCopyContext->sBuffer = pBuffer;
										pCopyContext->sBufMax = intBufSize;
										pCopyContext->sCopyLen = intBytesToCopy;
										pCopyContext->sSrcOffset = theEntry.fOffset;
										
										result = FSAsyncCopy( pCopyContext );
										ThrowOSErr( result );
										
										while ( !(pCopyContext->sFinished) )
											{
												if ( (inYielder != NULL) && (result == noErr) )
													{
														result = (inYielder)( pInContext, intMaxOffset, theEntry.fOffset + pCopyContext->sBytesCopied );							
													}
													
												if ( result != noErr )
													{
														pCopyContext->sKillFlag = true;
													}
											}

										ThrowOSErr( result );
										ThrowOSErr( pCopyContext->sResult );											
	
										#else

										while ( intBytesToCopy > 0 )
											{													
												intIOCount = intBytesToCopy;
												
												if ( intIOCount > intBufSize )
													intIOCount = intBufSize;
													
												result = FSReadOffset( oldDataForkRef, (UInt32 *)&intIOCount, pBuffer, theEntry.fOffset + (theEntry.fLength - intBytesToCopy) );
												ThrowOSErr( result );
												
												result = FSWriteOffset( resForkRef, (UInt32 *)&intIOCount, pBuffer, theEntry.fLength - intBytesToCopy );
												ThrowOSErr( result );

												intBytesToCopy -= intIOCount;
		
												if ( inYielder != NULL )
													result = (inYielder)( pInContext, intMaxOffset, theEntry.fOffset + (theEntry.fLength - intBytesToCopy) );															
											
												ThrowOSErr( result );
											}
										
										#endif
																			
									break;
									
									case kAppleSingleMacFInfo:

//  We only care about the first 16 bytes of the entry. Technically,
//  this entry should always be 32 bytes but MacCVS 1.0x was psycho
//  and only sent the 16 byte FInfo data.

										if ( (theEntry.fLength > (sizeof(FInfo) + sizeof(FXInfo))) || (theEntry.fLength < sizeof(FInfo)) )
											{											
												result = kCVSEngineInvalidAppleSingleDataErr;
												throw result;												
											}
																				
										intIOCount = sizeof(FInfo);
																					
										result = FSReadOffset( oldDataForkRef, (UInt32 *)&intIOCount, &finderInfo, theEntry.fOffset );
										ThrowOSErr( result );
	
										result = ::FSpSetFInfo( &targetSpec, &finderInfo );
		
										if ( inYielder != NULL )
											result = (inYielder)( pInContext, intMaxOffset, theEntry.fOffset + theEntry.fLength );															
										
										ThrowOSErr( result );
									break;

//  Skip unknown items.
							
									default:
									
										
									break;
									
								}
							
							intCurOffset += theEntry.fLength;																
						}
				
					targetItemAccess.SetDelete( false );				
				}
																																	
				::FSpDelete( &encodedItem );
			}
		catch ( OSErr intException )
			{		
				result = intException;

				if ( bCreatedFile == true )
					::FSpDelete( &targetSpec );
					
				if ( bRenamedFile == true )
					::FSpRename( &encodedItem, targetSpec.name );
			}			
		catch ( ... )
			{
				result = kCVSEngineUnknownExceptionErr;

				if ( bCreatedFile == true )
					::FSpDelete( &targetSpec );
					
				if ( bRenamedFile == true )
					::FSpRename( &encodedItem, targetSpec.name );
			}
		
		try
			{				
				if ( pBuffer != NULL )
					{
						::DisposePtr( pBuffer );
						pBuffer = NULL;
					}
					
				#ifdef CVSENGINE_ASYNCFILEIO
				
				if ( pCopyContext != NULL )
					{
						::DisposePtr( (Ptr)pCopyContext );
						pCopyContext = NULL;
					}
				
				#endif
			}
		catch ( ... ) 
			{
			}
			
		return result;
				
	}
		
