//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CCVSEngineItemInfoThingy.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		The CCVSEngineItemInfoThingy object tells the caller whether or 
		not they can perform certain operations on the specified FSSpec.
		
		The code is very similiar to stuff found in UProjectActions.cp. 
		Eventually UProjectActions.cp should call this.
										
-------------------------------------------------------------------------------- */


#include "ThrowOSErr.h"
#include "CVSEngineUtils.h"
#include "CCVSEngineItemInfoThingy.h"


CCVSEngineItemInfoThingy::CCVSEngineItemInfoThingy()
	{
	}
	
	
CCVSEngineItemInfoThingy::~CCVSEngineItemInfoThingy()
	{
	}
	
	
#pragma mark -


OSErr		
CCVSEngineItemInfoThingy::GetEntry( const FSSpec &fsInItem, Str255 pstrOutRevision,
																		Str255 pstrOutTimeStamp, Str255 pstrOutOptions, Str255 pstrOutTag )
	{
	
		OSErr		result 		= noErr;
		FSSpec	fsParent 	= { 0 };
		
		result = FSpGetParentFSSpec( &fsInItem, &fsParent );			
		
		if ( result == noErr )
			{
				if ( !mEntries.AreYouThisDirectory(&fsParent) )
					{
						result = mEntries.CloseEntriesFile();
						
						if ( result == noErr )
							{
								result = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
							}
					}
			}
			
		if ( result == noErr )
			{				
				result = mEntries.GetEntry( fsInItem.name, pstrOutRevision, pstrOutTimeStamp, pstrOutOptions, pstrOutTag );
			}
			
		return result;
		
	}

/*
OSErr
CCVSEngineItemInfoThingy::CanAdd( const FSSpec &fsInItem, Boolean &bAnswer )
	{

		OSErr			error					= noErr;
		Boolean		bResult				= false;
		FSSpec		fsParent			= { 0 };
		Str255		pstrRevision 	= "\p";
		
		try
			{
				error = FSpIsAFile( &fsInItem, NULL );
				
				if ( error == noErr )
					{			
						error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
						ThrowOSErr( error );

						if ( FSpIsRepositoryDir(&fsParent) )
							{			
								if ( !mEntries.AreYouThisDirectory(&fsParent) )
									{
										error = mEntries.CloseEntriesFile();
										ThrowOSErr( error );
										
										error = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
										ThrowOSErr( error );
									}
									
								error = mEntries.GetEntry(fsInItem.name, pstrRevision, NULL, NULL, NULL);
								
								if ( error == kCVSEngineNoRevisionDataErr )
									{
										bResult = true;		
									}
								else if ( error == noErr )
									{
										//  Negative revisions can be resurrected.
										
										if ( (pstrRevision[0] > 0) && (pstrRevision[1] == '-') )
											{
												bResult = true;		
											}
									}
							}
					}	
				else if ( error == kCVSEngineIsFolder )
					{		
						if ( !FSpIsRepositoryDir(&fsInItem) )
							bResult = true;		
					}
			}
		catch ( ... )
			{
			
			}
	
		return bResult;	
		
	}
*/	
	
	
OSErr
CCVSEngineItemInfoThingy::CanCommit( const FSSpec &fsInItem, Boolean &bAnswer )
	{

		OSErr								result					= noErr;
	
		FSSpec							fsParent				= { 0 };
		SInt32							intModDate			= 0;
		UInt32							intLen					= 0;
		
		Str255							pstrRevision		= "\p";
		Str255							pstrTimeStamp		= "\p";
		Str255							pstrActualTime 	= "\p";
		
		CkidResourcePB			ckidInfo				= { 0 };
			
		try
			{
				result = FSpIsAFile( &fsInItem, NULL );
				
				if ( result == noErr )
					{	
						intModDate = FSpGetModDate( &fsInItem );
				
						result = FSpGetParentFSSpec( &fsInItem, &fsParent );			
						ThrowOSErr( result );

						if ( FSpIsRepositoryDir(&fsParent) )
							{				
								if ( !mEntries.AreYouThisDirectory(&fsParent) )
									{
										result = mEntries.CloseEntriesFile();
										ThrowOSErr( result );

										result = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
										ThrowOSErr( result );
									}
									
								result = mEntries.GetEntry( fsInItem.name, pstrRevision, pstrTimeStamp, NULL, NULL );
						
								if ( result == noErr )
									{
										if ( CUIsAddedOrRemovedRevision(pstrRevision) )
											bAnswer = true;
									}
									
								if ( (bAnswer == false) && (result == noErr) )
									{
										result = FSpReadCkidResource( &fsInItem, &ckidInfo, false );
															
										if ( result == noErr )
											{															
												result = CUIsFileModified( &ckidInfo, pstrTimeStamp, intModDate );
											}	
										else
											{
												result = CUIsFileModified( NULL, pstrTimeStamp, intModDate );
											}
										
										if ( result == kCVSEngineFileLocallyModifiedErr )
											{
												bAnswer = true;
												result = noErr;
											}
													
										ThrowOSErr( result );
									}
							}
					}
			}
		catch ( ... )
			{
			}
							
		return result;		

	}
	
/*
Boolean
CCVSEngineItemInfoThingy::CanCompareWithOriginal( const FSSpec &fsInItem )
	{
	
		OSErr								error						= noErr;
		Boolean							bResult					= false;
	
		FSSpec							fsParent				= { 0 };
		SInt32							intModDate			= 0;
		UInt32							intLen					= 0;
		
		Str255							pstrRevision		= "\p";
		Str255							pstrTimeStamp		= "\p";
		Str255							pstrActualTime 	= "\p";
		
		CkidResourcePB			ckidInfo				= { 0 };
			
		try
			{	
				error = FSpIsAFile( &fsInItem, NULL );
				
				if ( error == noErr )
					{	
						intModDate = FSpGetModDate( &fsInItem );
				
						error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
						ThrowOSErr( error );

						if ( FSpIsRepositoryDir(&fsParent) )
							{				
								if ( !mEntries.AreYouThisDirectory(&fsParent) )
									{
										error = mEntries.CloseEntriesFile();
										ThrowOSErr( error );

										error = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
										ThrowOSErr( error );
									}
									
								error = mEntries.GetEntry( fsInItem.name, pstrRevision, pstrTimeStamp, NULL, NULL );
						
								if ( error == noErr )
									{
										if ( !CUIsAddedOrRemovedRevision(pstrRevision) )
											{
												error = FSpReadCkidResource( &fsInItem, &ckidInfo, false );
										
												if ( error == noErr )
													{															
														error = CUIsFileModified( &ckidInfo, pstrTimeStamp, intModDate );
													}	
												else
													{
														error = CUIsFileModified( NULL, pstrTimeStamp, intModDate );
													}
												
												if ( error == kCVSEngineFileLocallyModifiedErr )
													{
														bResult = true;
														error = noErr;
													}
													
												ThrowOSErr( error );
											}
									}
							}		
					}
			}
		catch ( ... )
			{
			}
					
		return bResult;	
	
	}
	

Boolean
CCVSEngineItemInfoThingy::CanDiff( const FSSpec &fsInItem )
	{

		OSErr								error						= noErr;
		Boolean							bResult					= false;
		Boolean							bHasCkid				= false;
		
		FSSpec							fsParent				= { 0 };
		SInt32							intModDate			= 0;
		UInt32							intLen					= 0;
		
		Str255							pstrRevision		= "\p";
		Str255							pstrTimeStamp		= "\p";
		Str255							pstrActualTime 	= "\p";
		
		CkidResourcePB			ckidInfo;
				
		error = FSpIsAFile( &fsInItem, NULL );
		
		if ( error == noErr )
			{	
				intModDate = FSpGetModDate( &fsInItem );
		
				error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
				ThrowOSErr( error );

				if ( FSpIsRepositoryDir(&fsParent) )
					{				
						if ( !mEntries.AreYouThisDirectory(&fsParent) )
							{
								error = mEntries.CloseEntriesFile();
								ThrowOSErr( error );

								error = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
								ThrowOSErr( error );
							}
							
						error = mEntries.GetEntry( fsInItem.name, pstrRevision, pstrTimeStamp, NULL, NULL );
				
						if ( error == noErr )
							{				
								if ( !CUIsAddedOrRemovedRevision(pstrRevision) )
									{
										error = FSpReadCkidResource( &fsInItem, &ckidInfo, false );
								
										if ( error == noErr )
											{															
												error = CUIsFileModified( &ckidInfo, pstrTimeStamp, intModDate );
											}	
										else
											{
												error = CUIsFileModified( NULL, pstrTimeStamp, intModDate );
											}
										
										if ( error == kCVSEngineFileLocallyModifiedErr )
											{
												bResult = true;
												error = noErr;
											}
											
										ThrowOSErr( error );
									}
							}
					}	
			}
			
		return bResult;
	
	}
	

Boolean
CCVSEngineItemInfoThingy::CanEdit( const FSSpec &fsInItem )
	{
	
		OSErr								error						= noErr;
		Boolean							bResult					= false;
	
		FSSpec							fsParent				= { 0 };
		SInt32							intModDate			= 0;
		UInt32							intLen					= 0;
		
		Str255							pstrRevision		= "\p";
		Str255							pstrTimeStamp		= "\p";
		Str255							pstrActualTime 	= "\p";
						
		error = FSpIsAFile( &fsInItem, NULL );
		
		if ( error == noErr )
			{	
				intModDate = FSpGetModDate( &fsInItem );
		
				error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
				ThrowOSErr( error );

				if ( FSpIsRepositoryDir(&fsParent) )
					{				
						if ( !mEntries.AreYouThisDirectory(&fsParent) )
							{
								error = mEntries.CloseEntriesFile();
								ThrowOSErr( error );

								error = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
								ThrowOSErr( error );
							}
							
						error = mEntries.GetEntry( fsInItem.name, pstrRevision, pstrTimeStamp, NULL, NULL );
				
						if ( error == noErr )
							{
								if ( !CUIsAddedOrRemovedRevision(pstrRevision) )
									{
										bResult = true;
									}
							}	
					}	
			}

		return bResult;
			
	}
	

Boolean
CCVSEngineItemInfoThingy::CanLog( const FSSpec &fsInItem )
	{

		OSErr			error			= noErr;
		Boolean		bResult		= false;
	
		FSSpec		fsParent	= { 0 };
						
		error = FSpIsAFile( &fsInItem, NULL );
		
		if ( error == noErr )
			{	
				error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
				ThrowOSErr( error );
				
				if ( FSpIsRepositoryDir(&fsParent) )
					{
						if ( !mEntries.AreYouThisDirectory(&fsParent) )
							{
								error = mEntries.CloseEntriesFile();
								ThrowOSErr( error );

								error = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
								ThrowOSErr( error );
							}
							
						error = mEntries.GetEntry( fsInItem.name, NULL, NULL, NULL, NULL );
				
						if ( error == noErr )
							bResult = true;	
					}																		
			}
			
		return bResult;
	
	}
	
*/

				
OSErr
CCVSEngineItemInfoThingy::CanMRO( const FSSpec &fsInItem, Boolean &bAnswer )
	{

		OSErr result = noErr;
		
		result = FSpIsAFile( &fsInItem, NULL );
		
		if ( result == noErr )
			{	
				bAnswer = true;
			}
		else if ( result == kCVSEngineIsFolder )
			{
				result = noErr;
			}
							
		return result;
	
	}

/*
Boolean
CCVSEngineItemInfoThingy::CanRemove( const FSSpec &fsInItem )
	{
	
		OSErr			error						= noErr;
		Boolean		bResult					= false;
	
		FSSpec		fsParent				= { 0 };
		Str255		pstrRevision		= "\p";
				
		error = FSpIsAFile( &fsInItem, NULL );
		
		if ( error == noErr )
			{	
				error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
				ThrowOSErr( error );
				
				if ( FSpIsRepositoryDir(&fsParent) )
					{
						if ( !mEntries.AreYouThisDirectory(&fsParent) )
							{
								error = mEntries.CloseEntriesFile();
								ThrowOSErr( error );

								error = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
								ThrowOSErr( error );
							}
							
						error = mEntries.GetEntry( fsInItem.name, pstrRevision, NULL, NULL, NULL );
				
						if ( error == noErr )
							{
								if ( !CUIsAddedOrRemovedRevision(pstrRevision) )
									{
										bResult = true;	
									}
							}		
					}																
			}
			
		return bResult;
	
	}
	

Boolean
CCVSEngineItemInfoThingy::CanRollback( const FSSpec &fsInItem )
	{
	
		OSErr								error						= noErr;
		Boolean							bResult					= false;
	
		FSSpec							fsParent				= { 0 };
		SInt32							intModDate			= 0;
		UInt32							intLen					= 0;
		
		Str255							pstrRevision		= "\p";
		Str255							pstrTimeStamp		= "\p";
		Str255							pstrActualTime 	= "\p";
		
		CkidResourcePB			ckidInfo;
				
		error = FSpIsAFile( &fsInItem, NULL );
		
		if ( error == noErr )
			{	
				intModDate = FSpGetModDate( &fsInItem );
		
				error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
				ThrowOSErr( error );

				if ( FSpIsRepositoryDir(&fsParent) )
					{				
						if ( !mEntries.AreYouThisDirectory(&fsParent) )
							{
								error = mEntries.CloseEntriesFile();
								ThrowOSErr( error );

								error = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
								ThrowOSErr( error );
							}
							
						error = mEntries.GetEntry( fsInItem.name, pstrRevision, pstrTimeStamp, NULL, NULL );
				
						if ( error == noErr )
							{
								if ( !CUIsAddedOrRemovedRevision(pstrRevision) )
									{
										error = FSpReadCkidResource( &fsInItem, &ckidInfo, false );
								
										if ( error == noErr )
											{															
												error = CUIsFileModified( &ckidInfo, pstrTimeStamp, intModDate );
											}	
										else
											{
												error = CUIsFileModified( NULL, pstrTimeStamp, intModDate );
											}
										
										if ( error == kCVSEngineFileLocallyModifiedErr )
											{
												bResult = true;
												error = noErr;
											}
											
										ThrowOSErr( error );
									}
							}	
					}	
			}

		return bResult;
			
	}
*/	
	
OSErr
CCVSEngineItemInfoThingy::CanTag( const FSSpec &fsInItem, Boolean &bAnswer )
	{
		
		OSErr		result				= noErr;
	
		FSSpec	fsParent			= { 0 };
		Str255	pstrRevision	= "\p";
		
		try
			{				
				result = FSpIsAFile( &fsInItem, NULL );
				
				if ( result == noErr )
					{	
						result = FSpGetParentFSSpec( &fsInItem, &fsParent );			
						ThrowOSErr( result );
						
						if ( FSpIsRepositoryDir(&fsParent) )
							{
								if ( !mEntries.AreYouThisDirectory(&fsParent) )
									{
										result = mEntries.CloseEntriesFile();
										ThrowOSErr( result );

										result = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
										ThrowOSErr( result );
									}
									
								result = mEntries.GetEntry( fsInItem.name, pstrRevision, NULL, NULL, NULL );
						
								if ( result == noErr )
									{
										//  Should we allow tagging of 0 revision files?
										
										if ( !CUIsAddedOrRemovedRevision(pstrRevision) )
											{
												bAnswer = true;												
											}
									}
							}														
					}
				else if ( result == kCVSEngineIsFolder )
					{			
						result = noErr;
						
						if ( FSpIsRepositoryDir(&fsInItem) )
							{
								bAnswer = true;			
							}
					}
			}
		catch ( ... )
			{
			}
			
		return result;
		
	}
	

OSErr
CCVSEngineItemInfoThingy::CanUpdate( const FSSpec &fsInItem, Boolean &bAnswer )
	{
	
		OSErr			result						= noErr;
		Boolean		bResult					= false;
	
		FSSpec		fsParent				= { 0 };
		Str255		pstrRevision		= "\p";
			
		try
			{	
				result = FSpIsAFile( &fsInItem, NULL );
				
				if ( result == noErr )
					{	
						result = FSpGetParentFSSpec( &fsInItem, &fsParent );			
						ThrowOSErr( result );
						
						if ( FSpIsRepositoryDir(&fsParent) )
							{
								if ( !mEntries.AreYouThisDirectory(&fsParent) )
									{
										result = mEntries.CloseEntriesFile();
										ThrowOSErr( result );

										result = mEntries.LoadWorkingDirectory(&fsParent, fsRdPerm);
										ThrowOSErr( result );
									}
									
								result = mEntries.GetEntry( fsInItem.name, pstrRevision, NULL, NULL, NULL );
						
								if ( result == noErr )
									{
										if ( !CUIsAddedOrRemovedRevision(pstrRevision) )
											{
												bAnswer = true;	
											}
									}		
							}																
					}
				else if ( result == kCVSEngineIsFolder )
					{
						result = noErr;
						
						if ( FSpIsRepositoryDir(&fsInItem) )
							{
								bAnswer = true;
							}
					}
			}
		catch ( ... )
			{
			}
			
		return result;
	
	}
	
/*	
Boolean
CCVSEngineItemInfoThingy::CanWatch( const FSSpec &fsInItem )
	{

		OSErr								result						= noErr;
		Boolean							bResult					= false;
	
		FSSpec							fsParent				= { 0 };
		Str255							pstrRevision		= "\p";
						
		error = FSpIsAFile( &fsInItem, NULL );
		
		if ( error == noErr )
			{	
				error = FSpGetParentFSSpec( &fsInItem, &fsParent );			
				ThrowOSErr( error );
				
				if ( FSpIsRepositoryDir(&fsParent) )
					{
						if ( !mEntries.AreYouThisDirectory(&fsParent) )
							{
								error = mEntries.CloseEntriesFile();
								ThrowOSErr( error );

								error = mEntries.LoadWorkingDirectory( &fsParent, fsRdPerm );
								ThrowOSErr( error );
							}
							
						error = mEntries.GetEntry( fsInItem.name, pstrRevision, NULL, NULL, NULL );
				
						if ( error == noErr )
							{
								bResult = true;				
							}								
					}														
			}
		else if ( error == kCVSEngineIsFolder )
			{			
				if ( FSpIsRepositoryDir(&fsInItem) )
					{
						bResult = true;	
					}		
			}
			
		return bResult;
	
	}
*/