//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################


/* -------------------------------------------------------------------------------

	File:				CCVSMappings.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This is a simple container class for the mappings database.
		 
	Modification History:

		24  Apr		1999		Ram Rajadhyaksha
		
				Created.
										
-------------------------------------------------------------------------------- */


#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "CCVSMappings.h"


CCVSMappings::CCVSMappings()
	{
		mMappings = NULL;
	}
	
	
CCVSMappings::CCVSMappings( CCVSEngineFileMappingRec *pInMappings, const UInt32 intInCount )
	{
		try
			{
				mMappings = NULL;
				BuildMappingsTree( pInMappings, intInCount );
			}
		catch ( OSErr intException )
			{
				throw intException;
			}
		catch ( ... )
			{
				Assert_( false );
				ThrowOSErr( kCVSEngineUnknownExceptionErr );
			}
	}
	
	
CCVSMappings::~CCVSMappings()
	{
		try
			{
				if ( mMappings != NULL )
					{
						delete mMappings;
						mMappings;
					}
			}
		catch ( ... )
			{
			}
	}
	
	
#pragma mark -


void
CCVSMappings::BuildMappingsTree( CCVSEngineFileMappingRec *pInMappings, const UInt32 intInCount )
	{
	
		Str255 pstrReversed = "\p";
		
		mMappings = new CRadixTree();	
	
		if ( mMappings == NULL )
			{
				ThrowOSErr( memFullErr );
			}
			
		for ( UInt32 intIndex = 0; intIndex < intInCount; intIndex++ )
			{
				//  If we add a zero length key to the tree, 
				//  it becomes the default item. 
			
				PStrReverse( pInMappings[intIndex].fFileNamePStr, pstrReversed );
				PStrLowercase( pstrReversed );
				
				mMappings->AddKeyPStrToTree( pstrReversed, &pInMappings[intIndex], sizeof(CCVSEngineFileMappingRec) );					
			}
			
	}


Boolean
CCVSMappings::GetMapping( ConstStr255Param pstrInKey, CCVSEngineFileMappingRec 	&sOutMapping, const Boolean bInStrictCompare )
	{

		Boolean	bResult = false;

		sOutMapping.Init();
		
		try
			{				
				if ( mMappings != NULL )
					{	
						Str255	pstrReversed 	= "\p";
						UInt32	intDataSize		= sizeof( CCVSEngineFileMappingRec );
						
						PStrReverse( pstrInKey, pstrReversed );
						PStrLowercase( pstrReversed );
						
						bResult = mMappings->MatchKeyPStr( pstrReversed, &sOutMapping, intDataSize, bInStrictCompare );
						
						if ( bResult == true )
							{
								Assert_( intDataSize == sizeof(CCVSEngineFileMappingRec) );
							}
					}
			}
		catch ( ... )
			{
				Assert_( false );
			}

		if ( !bResult )
			{			
				sOutMapping.fFileType 		= kCVSEngineDefaultFileType;
				sOutMapping.fFileCreator 	= kCVSEngineDefaultFileCreator;
				sOutMapping.fTranslation	= kCVSEngineCRTranslation;
				sOutMapping.fEncoding			= kCVSEngineAppleSingleEncoding;
			}
			
		return bResult;
		
	}
	

OSErr
CCVSMappings::SetMappings( CCVSEngineFileMappingRec *pInMappings, const UInt32 intInCount )
	{
		
		OSErr result = noErr;
		
		try
			{	
				if ( pInMappings != NULL )
					{
						BuildMappingsTree( pInMappings, intInCount );
					}
				else if ( (pInMappings == NULL) && (intInCount == 0) )
					{
						delete mMappings;
						mMappings = NULL;
					}
			}
		catch ( OSErr intException )
			{
				result = intException;
			}
		catch ( ... )
			{
				Assert_( false );
				result = kCVSEngineUnknownExceptionErr;
			}
		
		return result;
		
	}
	

OSErr
CCVSMappings::AddMapping( CCVSEngineFileMappingRec *pInMapping )
	{
	
		OSErr 	result 				= noErr;
		Str255	pstrReversed 	= "\p";
		
		try
			{
				if ( pInMapping != NULL )
					{
						CCVSEngineFileMappingRec sMapping = *pInMapping;
						
						PStrReverse( sMapping.fFileNamePStr, pstrReversed );
						PStrLowercase( pstrReversed );
						
						mMappings->AddKeyPStrToTree( pstrReversed, &sMapping, sizeof(CCVSEngineFileMappingRec) );							
					}
				else
					{
						result = kCVSEngineBadParamErr;
					}
			}
		catch ( OSErr intException )
			{
				result = intException;
			}
		catch ( ... )
			{
				Assert_( false );
				result = kCVSEngineUnknownExceptionErr;
			}
			
		return result;
		
	}