//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CCVSServerStream.cp
//
//	  Author: 	Ram Rajadhyaksha
//
//################################################################################


#include <string.h>
#include <stdio.h>

#include "ThrowOSErr.h"
#include "StringUtils.h"

#include "CVSEngine.h"
#include "CCVSServerStream.h"


UInt16 CCVSServerStream::sRSHPort = kFirstRSHPort;


CCVSServerStream::CCVSServerStream( const UInt32 inTimeOut ):

	CTCPStream( inTimeOut )
	
	{	
	
	}
	
	
CCVSServerStream::~CCVSServerStream()
	{	
	
	}
	
	
void
CCVSServerStream::Login( 	CTCPSocketYieldProc	inYieldProc,
													void							*inContextPtr,
													const Str255 			inServerName, 
													const UInt16 			inPort, 
													const Str255 			inUserName, 
													const Str255 			inPassword, 
													const Str255 			inCVSRoot )
	{
	
		UInt16		thePort 					= CCVSServerStream::kCVSPServerPort;
		UInt32		bufLen						= 0;
		char			tempCStr[256] 		= "",
							*scrambledCStr		= NULL;
	
		if ( inPort != 0 )
			thePort = inPort;
			
		try
			{
			
				OpenConnection( inYieldProc, inContextPtr, inServerName, thePort );

/*
**  Convert the password to a C string and scramble the baby.
*/

				::BlockMoveData( &inPassword[1], tempCStr, inPassword[0] );
				tempCStr[inPassword[0]] = 0x00;
				
				scrambledCStr = ScrambleCVSPassword( tempCStr );

				if ( scrambledCStr == NULL )
					throw( (OSErr)memFullErr );
					
/*
**  Send the auth. request. This is sooo high tech.
*/

				*this << "BEGIN AUTH REQUEST\n";
				
//  Bug fix

				if ( inCVSRoot[0] > 0 )
					{
					
						*this << inCVSRoot;
				
					}
				else *this << "/";
				
				*this << "\n";
				*this << inUserName;
				*this << "\n";
				*this << scrambledCStr;
				*this << "\nEND AUTH REQUEST\n";

				if ( scrambledCStr != NULL )
					{
					
						::DisposePtr( scrambledCStr );
						scrambledCStr = NULL;
						
					}

				bufLen = 255;
				ReadEntireDataLine( tempCStr, bufLen );				
				tempCStr[bufLen] = 0x00;

/*
**  If it doesn't love us, die with an exception.
*/
		
				if ( !strcmp(tempCStr, "I HATE YOU") )	
					{
						ThrowOSErr( kCVSEngineBadPasswordErr );
					}
				else if ( strcmp(tempCStr, "I LOVE YOU") )
					{
						ThrowOSErr( kCVSEngineServerGaveBadResponseErr );
					}
					
				*this << "Root ";

//  Bug fix
				
				if ( inCVSRoot[0] > 0 )
					{
					
						*this << inCVSRoot;
				
					}
				else *this << "/";
				
				*this << "\n";
				
			}
			
		catch ( ... )
			{
			
				if ( scrambledCStr != NULL )
					{
					
						::DisposePtr( scrambledCStr );
						scrambledCStr = NULL;
						
					}
					
				throw;
				
			}
		
	}
	

void
CCVSServerStream::AuthLogin( 	CTCPSocketYieldProc	inYieldProc,
															void								*inContextPtr,
															const Str255 				inServerName, 
															const UInt16 				inPort, 
															const Str255 				inCVSRoot )
	{

		try
			{
			
				KClientLogin( this, inYieldProc, inContextPtr, inServerName, inPort, inCVSRoot );

			}
			
		catch ( ... )
			{
			
				throw;
				
			}
			
	}
	
	
void
CCVSServerStream::RSHLogin( CTCPSocketYieldProc	inYieldProc,
														void								*inContextPtr,
														const Str255 				inServerName, 
														const UInt16 				inPort, 
														const Str255 				inLocalUserName, 
														const Str255 				inRemoteUserName, 
														const Str255				inCVSRoot,
														const Str255				inCVSCommand )
	{
	
		Boolean		bDone				= false;
		Str255		commandStr 	= "\p";
		UInt16		thePort			= CCVSServerStream::kCVSRSHPort;
		UInt32		tryCount		= 0x00000000;
		
		if ( inPort != 0 )
			thePort = inPort;
			
/*
**  This sucks: RSH requires that the **local** port be less than 1024. I don't
**  know why, but it just doesn't work if the port is >= 1024. So, we need to
**  do some lamo port cycling.
*/
		
		while ( !bDone )
			{
			
				sRSHPort++;
				
				if ( sRSHPort > kMaxRSHPort )
					sRSHPort = kFirstRSHPort;
				
				try
					{
						
						OpenConnection( inYieldProc, inContextPtr, inServerName, thePort, sRSHPort );
					
						bDone = true;
						
					}
					
				catch ( OSErr intException ) 
					{

/*
**  Someone else may be using the port or it may be locked from
**  the IPREUSEADDR problem. So, try again.
*/
					
						if ( intException != kOTNoAddressErr )
							throw intException;
							
						tryCount++;
						
						if ( tryCount > 100 )
							ThrowOSErr( kCVSEngineNoPortsAvailableErr );
					
					}
					
			}
			
/*
**  Write NULL to since we're not specifying a stderr port.
*/
		
		WriteZeroByte();
	
		*this << inRemoteUserName;
		WriteZeroByte();
	
		*this << inRemoteUserName;
		WriteZeroByte();

/*
**  Turn on server mode. "cvs -d /my/src/tree server"
*/
				
		*this << inCVSCommand;
		
/*
**  See comments below...
**	*this << " -d ";
**	*this << inCVSRoot;
*/

		*this << " server";
		
		WriteZeroByte();

/*
**  For some reason, the -d argument doesn't work. Send the explicit
**  root command. If the cvs command string is wrong, we'll probably
**  die with an OTLookErr here.
*/
		
		*this << "Root ";

//  Bug fix
		
		if ( inCVSRoot[0] > 0 )
			{
			
				*this << inCVSRoot;
		
			}
		else *this << "/";
		
		*this << "\n";

	}
	
	

/* Map characters to each other randomly and symmetrically, A <--> B.
 *
 * We divide the ASCII character set into 3 domains: control chars (0
 * thru 31), printing chars (32 through 126), and "meta"-chars (127
 * through 255).  The control chars map _to_ themselves, the printing
 * chars map _among_ themselves, and the meta chars map _among_
 * themselves.  Why is this thus?
 *
 * No character in any of these domains maps to a character in another
 * domain, because I'm not sure what characters are legal in
 * passwords, or what tools people are likely to use to cut and paste
 * them.  It seems prudent not to introduce control or meta chars,
 * unless the user introduced them first.  And having the control
 * chars all map to themselves insures that newline and
 * carriage-return are safely handled.
 */

static unsigned char
shifts[] = { 
						0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,
			17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 114, 120,
			53, 79, 96, 109, 72, 108, 70, 64, 76, 67, 116, 74, 68, 87, 111, 52,
			75, 119, 49, 34, 82, 81, 95, 65, 112, 86, 118, 110, 122, 105, 41, 57,
			83, 43, 46, 102, 40, 89, 38, 103, 45, 50, 42, 123, 91, 35, 125, 55,
			54, 66, 124, 126, 59, 47, 92, 71, 115, 78, 88, 107, 106, 56, 36, 121,
			117, 104, 101, 100, 69, 73, 99, 63, 94, 93, 39, 37, 61, 48, 58, 113,
			32, 90, 44, 98, 60, 51, 33, 97, 62, 77, 84, 80, 85, 223, 225, 216,
			187, 166, 229, 189, 222, 188, 141, 249, 148, 200, 184, 136, 248, 190,
			199, 170, 181, 204, 138, 232, 218, 183, 255, 234, 220, 247, 213, 203,
			226, 193, 174, 172, 228, 252, 217, 201, 131, 230, 197, 211, 145, 238,
			161, 179, 160, 212, 207, 221, 254, 173, 202, 146, 224, 151, 140, 196,
			205, 130, 135, 133, 143, 246, 192, 159, 244, 239, 185, 168, 215, 144,
			139, 165, 180, 157, 147, 186, 214, 176, 227, 231, 219, 169, 175, 156,
			206, 198, 129, 164, 150, 210, 154, 177, 134, 127, 182, 128, 158, 208,
			162, 132, 167, 209, 149, 241, 153, 251, 237, 236, 171, 195, 243, 233,
			253, 240, 194, 250, 191, 155, 142, 137, 245, 235, 163, 242, 178, 152 
		};


char *
CCVSServerStream::ScrambleCVSPassword( char *inPassword )
	{

/* SCRAMBLE and DESCRAMBLE work like this:
 *
 * scramble(STR) returns SCRM, a scrambled copy of STR.  SCRM[0] is a
 * single letter indicating the scrambling method.  As of this
 * writing, the only legal method is 'A', but check the code for more
 * up-to-date information.  The copy will have been allocated with
 * malloc(). 
 *
 * descramble(SCRM) returns STR, again in its own malloc'd space.
 * descramble() uses SCRM[0] to determine which method of unscrambling
 * to use.  If it does not recognize the method, it dies with error.
 */

/* Return a malloc'd, scrambled version of STR. */

  
	 	SInt32 	i		= 0x00000000;
	  char 		*s	= NULL;

	  /* +2 to hold the 'A' prefix that indicates which version of
	   * scrambling this is (the first, obviously, since we only do one
	   * kind of scrambling so far), and then the '\0' of course.
	   */
	  	s = (char *) ::NewPtr (strlen (inPassword) + 2);

			if ( s != NULL )
				{
				
			 		s[0] = 'A';   /* Scramble (TM) version prefix. */
			  	strcpy (s + 1, inPassword);

			  	for (i = 1; s[i]; i++)
			   	 	s[i] = shifts[(unsigned char)(s[i])];

				}
				
	  	return s;
	  	
	}


#pragma mark -

	
#ifndef CVSENGINE_KCLIENT_SUPPORT

//  This is just a stub function. For real Kerberos support, you will need
//  to obtain the KClient source and KClientSupport.cp support files.

void KClientLogin( 	CCVSServerStream 		*inStream, 
										CTCPSocketYieldProc	inYieldProc,
										void								*inContextPtr,
										const Str255 				inServerName, 
										const UInt16 				inPort, 
										const Str255 				inCVSRoot )
	{
	
//  We should never get here.

		Assert_( false );
		
		ThrowOSErr( CTCPSocket::kBadParametersErr );
		
	}
	
#endif	
		

