//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################


/* -------------------------------------------------------------------------------

	File:				CRadixTree.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains code for a fast searching algorithm that is very confusing.
		 
	Modification History:

		18  Feb		1999		Ram Rajadhyaksha
		
				Fixed problem where MatchKey() would return a valid item even
				though no key matched exactly (the similiar key stored in the
				tree was longer) and strict compare was on.

		15 	Dec		1998		Ram Rajadhyaksha
		
				Fixed problem with adding keys with zero length context pointers.
										
-------------------------------------------------------------------------------- */


#include <string.h>

#include "CRadixTree.h"


CRadixTree::CRadixTree()
	{
		CreateTree();		
	}


CRadixTree::~CRadixTree()
	{
		DestroyTree();
	}


Boolean
CRadixTree::MatchKeyCStr( const char *inKeyCStr, void *outContextPtr, UInt32 &inOutDataSize, Boolean inStrictCompare )
	{

		Boolean 	result = false;
		UInt32		len			= 0x00000000;

		Assert_( mRootNode != NULL );
		
		if ( (inKeyCStr != NULL) && (outContextPtr != NULL) && (mRootNode != NULL) )
			{
					
				len = strlen( inKeyCStr );
				
				result = MatchRecursive( inKeyCStr, len, outContextPtr, inOutDataSize, 0, mRootNode, inStrictCompare );
				
			}
			
		return result;
			
	}


Boolean
CRadixTree::MatchKeyPStr( const Str255 inKeyPStr, void *outContextPtr, UInt32 &inOutDataSize, const Boolean inStrictCompare )
	{

		Boolean 	result = false;
		UInt32		len			= 0x00000000;
		
		Assert_( mRootNode != NULL );
		
		if ( (inKeyPStr != NULL) && (outContextPtr != NULL) && (mRootNode != NULL) )
			{
			
				result = MatchRecursive( (char *)&inKeyPStr[1], inKeyPStr[0], outContextPtr, inOutDataSize, 0, mRootNode, inStrictCompare );
				
			}
			
		return result;
			
	}


Boolean
CRadixTree::MatchRecursive( const char *inKeyStr, const UInt32 inKeyLen, void *outContextPtr, UInt32 &inOutDataSize, const UInt32 inCurIndex, CRadixTreeNodeRec *inCurNode, const Boolean inStrictCompare )
	{

		CRadixTreeNodeRec			*lastParent		= NULL,
													*lastSibling 	= NULL,
													*lastChild 		= NULL;
		
		UInt32								simCount			= 0x00000000,
													copyLen				= 0x00000000;
													
		Boolean								match					= true,
													result				= false;
	
	
		while ( (match == true) && ((inCurIndex + simCount) < inKeyLen) && (inCurIndex + simCount < inCurNode->fKeyDataLen) )
			{
		
				if (inKeyStr[inCurIndex + simCount] != inCurNode->fKeyData[inCurIndex + simCount]) 
			
					match = false;
			
				else simCount++;
						
			}
			
		if ( match == true )
			{
				
				if ( (inCurIndex + simCount) < inKeyLen )
					{
					
						if ( inCurNode->fChildren != NULL )
							{

								inCurNode = inCurNode->fChildren;
								
								while ( (inCurNode->fKeyData[inCurIndex + simCount] < inKeyStr[inCurIndex + simCount]) && 
												(inCurNode->fSiblings != NULL) &&
												(inCurNode->fSiblings->fKeyData[inCurIndex + simCount] <= inKeyStr[inCurIndex + simCount]) ) 
								
									{
										
										inCurNode = inCurNode->fSiblings;
										
									}
									
								if ( inCurNode->fKeyData[inCurIndex + simCount] == inKeyStr[inCurIndex + simCount] )
								
									result = MatchRecursive( inKeyStr, inKeyLen, outContextPtr, inOutDataSize, inCurIndex + simCount, inCurNode, inStrictCompare );
										
								else if ( inStrictCompare == false )
									{
									
										lastParent = inCurNode->fParent;
										
										while ( (lastParent != NULL) && (lastParent->fSyntheticNode == true) )
											lastParent = lastParent->fParent;
										
										if ( lastParent != NULL )
											{
											
												if ( lastParent->fContextPtr != NULL )
													{
													
														copyLen = lastParent->fContextDataLen;
														
														if ( copyLen > inOutDataSize )
															copyLen = inOutDataSize;
															
														if ( copyLen > 0 )
															::BlockMoveData( lastParent->fContextPtr, outContextPtr, copyLen );
															
														inOutDataSize = copyLen;
													
													}
												else inOutDataSize = 0;
												
												result = true;
											
											}
										
									}
														
							}
						else if ( inStrictCompare == false )
							{
									
								if ( inCurNode->fSyntheticNode == false )
									{
									
										if ( inCurNode->fContextPtr != NULL )
											{
											
												copyLen = inCurNode->fContextDataLen;
												
												if ( copyLen > inOutDataSize )
													copyLen = inOutDataSize;
													
												if ( copyLen > 0 )
													::BlockMoveData( inCurNode->fContextPtr, outContextPtr, copyLen );
													
												inOutDataSize = copyLen;
											
											}
										else inOutDataSize = 0;

										result = true;
										
									}
									
								else
									{

										lastParent = inCurNode->fParent;
										
										while ( (lastParent != NULL) && (lastParent->fSyntheticNode == true) )
											lastParent = lastParent->fParent;
										
										if ( lastParent != NULL )
											{
											
												if ( lastParent->fContextPtr != NULL )
													{
													
														copyLen = lastParent->fContextDataLen;
														
														if ( copyLen > inOutDataSize )
															copyLen = inOutDataSize;
															
														if ( copyLen > 0 )
															::BlockMoveData( lastParent->fContextPtr, outContextPtr, copyLen );
															
														inOutDataSize = copyLen;
													
													}
												else inOutDataSize = 0;
												
												result = true;
											
											}
																			
									}
									
							}
					
					}

//  Bug fix, inCurNode->fKeyDataLen used to be inKeyLen
//  which would accidently match items if the target key 
//  was shorter than the node key and strict compare was true.
				
				else if ( inCurIndex + simCount == inCurNode->fKeyDataLen )
					{
					
						if ( inCurNode->fSyntheticNode == false )
							{
							
								if ( inCurNode->fContextPtr != NULL )
									{
									
										copyLen = inCurNode->fContextDataLen;
										
										if ( copyLen > inOutDataSize )
											copyLen = inOutDataSize;
											
										if ( copyLen > 0 )
											::BlockMoveData( inCurNode->fContextPtr, outContextPtr, copyLen );
											
										inOutDataSize = copyLen;
									
									}
								else inOutDataSize = 0;

								result = true;
								
							}
							
						else if ( inStrictCompare == false )
							{

								lastParent = inCurNode->fParent;
								
								while ( (lastParent != NULL) && (lastParent->fSyntheticNode == true) )
									lastParent = lastParent->fParent;
								
								if ( lastParent != NULL )
									{
									
										if ( lastParent->fContextPtr != NULL )
											{
											
												copyLen = lastParent->fContextDataLen;
												
												if ( copyLen > inOutDataSize )
													copyLen = inOutDataSize;
													
												if ( copyLen > 0 )
													::BlockMoveData( lastParent->fContextPtr, outContextPtr, copyLen );
													
												inOutDataSize = copyLen;
											
											}
										else inOutDataSize = 0;

										result = true;
									
									}
																
							}
								
					}

/*
**  If the target key is shorter than the current node's key,
**  return the current node's first non-synthetic parent.
*/
					
				else if ( inStrictCompare == false )
					{
					
						lastParent = inCurNode->fParent;
						
						while ( (lastParent != NULL) && (lastParent->fSyntheticNode == true) )
							lastParent = lastParent->fParent;
						
						if ( lastParent != NULL )
							{
							
								if ( lastParent->fContextPtr != NULL )
									{
									
										copyLen = lastParent->fContextDataLen;
										
										if ( copyLen > inOutDataSize )
											copyLen = inOutDataSize;
											
										if ( copyLen > 0 )
											::BlockMoveData( lastParent->fContextPtr, outContextPtr, copyLen );
											
										inOutDataSize = copyLen;
									
									}
								else inOutDataSize = 0;

								result = true;
							
							}
					
					}
									
			}

/*
**  If we mismatch in the middle of comparing a node, return
**  the node's first non-synthetic parent.
*/
			
		else if ( (match == false) && (inStrictCompare == false) )
			{
			
				lastParent = inCurNode->fParent;
				
				while ( (lastParent != NULL) && (lastParent->fSyntheticNode == true) )
					lastParent = lastParent->fParent;
				
				if ( lastParent != NULL )
					{
											
						if ( lastParent->fContextPtr != NULL )
							{
							
								copyLen = lastParent->fContextDataLen;
								
								if ( copyLen > inOutDataSize )
									copyLen = inOutDataSize;
									
								if ( copyLen > 0 )
									::BlockMoveData( lastParent->fContextPtr, outContextPtr, copyLen );
									
								inOutDataSize = copyLen;
							
							}
						else inOutDataSize = 0;
	
						result = true;
				
					}
			
			}
			
		return result;
			
	}
	
	
void
CRadixTree::CreateTree()
	{
		
		mRootNode = NULL;
		mKeyCount = 0x00000000;
		
		mRootNode = (CRadixTreeNodeRec *)::NewPtr( sizeof(CRadixTreeNodeRec) );
		
		if ( mRootNode == NULL )
			throw( ::MemError() );
		
		mRootNode->fParent 					= NULL;
		mRootNode->fSiblings 				= NULL;
		mRootNode->fChildren 				= NULL;
		mRootNode->fKeyData 				= NULL;
		mRootNode->fKeyDataLen 			= 0;
		mRootNode->fContextPtr 			= NULL;
		mRootNode->fContextDataLen 	= 0;
		mRootNode->fSyntheticNode 	= true;
	
		mKeyCount++;
	
	}
	
void	
CRadixTree::DestroyTree()
	{
		
		// Assert_( mRootNode != NULL );
		
		if ( mRootNode != NULL )
			{
				DestroyTreeRecursive( mRootNode );
				mRootNode = NULL;
			}
			
		Assert_( mKeyCount == 0 );
		
	}


void
CRadixTree::DestroyTreeRecursive( CRadixTreeNodeRec *inCurNode )
	{
	
		if ( inCurNode != NULL )
			{
			
				DestroyTreeRecursive( inCurNode->fChildren );
				DestroyTreeRecursive( inCurNode->fSiblings );

				Assert_( ((inCurNode->fKeyData != NULL) && (inCurNode->fKeyDataLen > 0)) || ((inCurNode->fKeyData == NULL) && (inCurNode->fKeyDataLen == 0)) );
				Assert_( ((inCurNode->fContextPtr != NULL) && (inCurNode->fContextDataLen > 0)) || ((inCurNode->fContextPtr == NULL) && (inCurNode->fContextDataLen == 0)) );
				
				if ( inCurNode->fKeyData != NULL )
					{
					
						::DisposePtr( (Ptr)inCurNode->fKeyData );
						
						inCurNode->fKeyData = NULL;
						inCurNode->fKeyDataLen = 0;
						
					}

				if ( inCurNode->fContextPtr != NULL )
					{
					
						::DisposePtr( (Ptr)inCurNode->fContextPtr );
						
						inCurNode->fContextPtr = NULL;
						inCurNode->fContextDataLen = 0;
						
					}
					
				::DisposePtr( (Ptr)inCurNode );
				
				Assert_( mKeyCount > 0 );
				
				mKeyCount--;
					
			}
	
	}
	
	
void
CRadixTree::AddKeyCStrToTree( const char *inKeyCStr, const void *inContextPtr, const UInt32 inDataSize )
	{

		Assert_( mRootNode != NULL );

		if ( inKeyCStr != NULL )
			{

				if ( mRootNode != NULL )
					AddKeyRecursive( inKeyCStr, strlen(inKeyCStr), inContextPtr, inDataSize, 0, mRootNode);
		
			}
			
	}


void
CRadixTree::AddKeyPStrToTree( const Str255 inKeyPStr, const void *inContextPtr, const UInt32 inDataSize )
	{

		Assert_( mRootNode != NULL );
		
		if ( mRootNode != NULL )
			AddKeyRecursive( (char *)&inKeyPStr[1], inKeyPStr[0], inContextPtr, inDataSize, 0, mRootNode);
		
	}


void
CRadixTree::AddKeyRecursive( const char *inKeyStr, const UInt32 inKeyLen, const void *inContextPtr, const UInt32 inDataSize, const UInt32 inCurIndex, CRadixTreeNodeRec *inCurNode )
	{

		CRadixTreeNodeRec			*lastParent		= NULL,
													*lastSibling 	= NULL,
													*lastChild 		= NULL;
		
		UInt32								simCount			= 0x00000000;
		Boolean								match					= true;
				
		while ( (match == true) && (inCurIndex + simCount < inKeyLen) && (inCurIndex + simCount < inCurNode->fKeyDataLen) )
			{
		
				if (inKeyStr[inCurIndex + simCount] != inCurNode->fKeyData[inCurIndex + simCount]) 
					
					match = false;
			
				else simCount++;
						
			}

/*
**  If there is a mismatch before the end of the current node key or the target key
**  is shorter than the current node key then we must share a common parent. So
**  create that common parent (which is a synthetic node) and recurse on it.
*/
						
		if ( ((!match) && ((simCount + inCurIndex) < inCurNode->fKeyDataLen)) || ((match) && (inCurNode->fKeyDataLen > inKeyLen)) )
			{
			
				lastParent = inCurNode->fParent;
				
				if ( lastParent != NULL )
					{
					
						lastChild = (CRadixTreeNodeRec *)::NewPtr( sizeof(CRadixTreeNodeRec) );
						
						if ( lastChild == NULL )
							throw ( ::MemError() );
							
						lastChild->fParent = lastParent;
						lastChild->fChildren = inCurNode;
						lastChild->fSiblings = inCurNode->fSiblings;
						lastChild->fKeyData = ::NewPtr( inCurIndex + simCount );
						
						if ( lastChild->fKeyData == NULL )
							{
								
								::DisposePtr( (Ptr)lastChild );
								lastChild = NULL;
								
								throw ( ::MemError() );
							
							}

						::BlockMoveData( inCurNode->fKeyData, lastChild->fKeyData, inCurIndex + simCount );
						
						lastChild->fKeyDataLen = inCurIndex + simCount;
						lastChild->fContextPtr = NULL;
						lastChild->fContextDataLen = 0;
						lastChild->fSyntheticNode = true;
						
						inCurNode->fParent = lastChild;
						inCurNode->fSiblings = NULL;

/*
**  If the parent is non-NIL, see if the the inCurNode
**  is the first child. If it is, we need to update the parent's
**  child pointer to point to us. If not, we need to find the sibling
**  that comes before us in the child list and update it's sibling
**  pointer to point to us.
*/
						
						if ( lastParent != NULL )
							{

/*
**  We're the first child, so this is easy.
*/
							
								if ( lastParent->fChildren == inCurNode )
								
									lastParent->fChildren = lastChild;
								
								else
									{

/*
**  We're not the first child, so find the sibling that points to us.
*/
										
										lastSibling = lastParent->fChildren;
									
										while ( (lastSibling != NULL) && (lastSibling->fSiblings != inCurNode) )
											lastSibling = lastSibling->fSiblings;
											
										Assert_( lastSibling != NULL );
										
										if ( lastSibling != NULL )
											lastSibling->fSiblings = lastChild;
											
									}
							
							}
						
						mKeyCount++;
												
						AddKeyRecursive( inKeyStr, inKeyLen, inContextPtr, inDataSize, inCurIndex + simCount, lastChild );
							
					}					
				
			}

/*
**  Else if the target key has all the same chars up to the current node key, but
**  it has more characters to go, it must be a child.
*/
			
		else if ( (match) && ((simCount + inCurIndex) == inCurNode->fKeyDataLen) && ((simCount + inCurIndex) < inKeyLen) )
			{

/*
**  If there are no children, make ourselves as the first child.
*/
		
				if (inCurNode->fChildren == NULL) 
					{
				
						inCurNode->fChildren = (CRadixTreeNodeRec *)::NewPtr( sizeof(CRadixTreeNodeRec) );
						
						if ( inCurNode->fChildren == NULL )
							throw( ::MemError() );
											
						inCurNode->fChildren->fParent = inCurNode;
						inCurNode->fChildren->fSiblings = NULL;
						inCurNode->fChildren->fChildren = NULL;

						inCurNode->fChildren->fKeyData = ::NewPtr( inKeyLen );
						
						if ( inCurNode->fChildren->fKeyData == NULL )
							{
							
								::DisposePtr( (Ptr)inCurNode->fChildren );
								inCurNode->fChildren = NULL;
								
								throw( ::MemError() );
								
							}
							
						::BlockMoveData( inKeyStr, inCurNode->fChildren->fKeyData, inKeyLen );
						
						inCurNode->fChildren->fKeyDataLen = inKeyLen;
						
						if ( inDataSize > 0 )
							{
							
								inCurNode->fChildren->fContextPtr = ::NewPtr( inDataSize );
								
								if ( inCurNode->fChildren->fContextPtr == NULL )
									{

										::DisposePtr( (Ptr)inCurNode->fChildren->fKeyData );
										::DisposePtr( (Ptr)inCurNode->fChildren );
										inCurNode->fChildren = NULL;
									
										throw( ::MemError() );
									
									}
									
								inCurNode->fChildren->fContextDataLen = inDataSize;
								
								::BlockMoveData( inContextPtr, inCurNode->fChildren->fContextPtr, inCurNode->fChildren->fContextDataLen );
						
							}
						else
							{
								inCurNode->fChildren->fContextPtr = NULL;
								inCurNode->fChildren->fContextDataLen = 0;
							}
							
						inCurNode->fChildren->fSyntheticNode = false;
					 	
					 	mKeyCount++;
					 	
					}

/*
**  Else find the sibling (if there is one) that we should be a child/parent of...
*/
					
				else 
					{

/*
**  Assumption: All children are longer than their parents. Therefore I can
**  assume that (inCurIndex + simCount) is a valid index.
*/
				
						lastParent = inCurNode;
						lastSibling = NULL;
						
						inCurNode = inCurNode->fChildren;
					
						while ( (inCurNode->fKeyData[inCurIndex + simCount] < inKeyStr[inCurIndex + simCount]) && 
										(inCurNode->fSiblings != NULL) &&
										(inCurNode->fSiblings->fKeyData[inCurIndex + simCount] <= inKeyStr[inCurIndex + simCount]) ) 
						
							{
								
								lastSibling = inCurNode;
									
								inCurNode = inCurNode->fSiblings;
								
							}
							
						if ( inCurNode->fKeyData[inCurIndex + simCount] == inKeyStr[inCurIndex + simCount] )
						
							AddKeyRecursive( inKeyStr, inKeyLen, inContextPtr, inDataSize, inCurIndex + simCount, inCurNode );
						
						else 
							{
							
/*
**  Insert ourselves in the sibling list.
*/							

/*
**  We must be the last one.
*/
							
								if ( (lastSibling != NULL) && (inCurNode->fSiblings == NULL) )
									{
									
										lastSibling = inCurNode;

										inCurNode = (CRadixTreeNodeRec *)::NewPtr(sizeof(CRadixTreeNodeRec));
										
										if (inCurNode == NULL) 
											throw( ::MemError() );
										
										inCurNode->fParent = lastParent;
										inCurNode->fSiblings = NULL;
										inCurNode->fChildren = NULL;
										inCurNode->fKeyData = ::NewPtr( inKeyLen );
									
										if ( inCurNode->fKeyData == NULL )
											{
											
												::DisposePtr( (Ptr)inCurNode );
												
												throw ( ::MemError() );
											
											}
													
										::BlockMoveData( inKeyStr, inCurNode->fKeyData, inKeyLen );
										
										inCurNode->fKeyDataLen = inKeyLen;
										
										if ( inDataSize > 0 )
											{
											
												inCurNode->fContextPtr = ::NewPtr( inDataSize );
												
												if ( inCurNode->fContextPtr == NULL )
													{

														::DisposePtr( (Ptr)inCurNode->fKeyData );
														::DisposePtr( (Ptr)inCurNode );
													
														throw( ::MemError() );
													
													}
													
												inCurNode->fContextDataLen = inDataSize;
												
												::BlockMoveData( inContextPtr, inCurNode->fContextPtr, inCurNode->fContextDataLen );

											}
										else
											{
												inCurNode->fContextPtr = NULL;
												inCurNode->fContextDataLen = 0;
											}
											
										inCurNode->fSyntheticNode = false;
										
										lastSibling->fSiblings = inCurNode;
										
										mKeyCount++;
									
									}

/*
**  We're somewhere in the middle.
*/
									
								else if ( (lastSibling != NULL) && (inCurNode->fSiblings != NULL) ) 
									{
								
										lastSibling = inCurNode;
										
										inCurNode = (CRadixTreeNodeRec *)::NewPtr(sizeof(CRadixTreeNodeRec));
										
										if (inCurNode == NULL) 
											throw( ::MemError() );
										
										inCurNode->fParent = lastParent;
										inCurNode->fSiblings = lastSibling->fSiblings;
										inCurNode->fChildren = NULL;
										inCurNode->fKeyData = ::NewPtr( inKeyLen );
									
										if ( inCurNode->fKeyData == NULL )
											{
											
												::DisposePtr( (Ptr)inCurNode );
												
												throw ( ::MemError() );
											
											}
													
										::BlockMoveData( inKeyStr, inCurNode->fKeyData, inKeyLen );
										
										inCurNode->fKeyDataLen = inKeyLen;
						
										if ( inDataSize > 0 ) 
											{
											
												inCurNode->fContextPtr = ::NewPtr( inDataSize );
												
												if ( inCurNode->fContextPtr == NULL )
													{

														::DisposePtr( (Ptr)inCurNode->fKeyData );
														::DisposePtr( (Ptr)inCurNode );
													
														throw( ::MemError() );
													
													}
													
												inCurNode->fContextDataLen = inDataSize;
												
												::BlockMoveData( inContextPtr, inCurNode->fContextPtr, inCurNode->fContextDataLen );

											}
										else
											{
												inCurNode->fContextPtr = NULL;
												inCurNode->fContextDataLen = 0;
											}
											
										inCurNode->fSyntheticNode = false;
										
										lastSibling->fSiblings = inCurNode;
										
										mKeyCount++;
																												
									}

/*
**  Every child is bigger than us or there is only one sibling and it might
**  be bigger than us, so we need to insert ourselves somewhere.
*/				
									
								else 
									{
								
										lastSibling = (CRadixTreeNodeRec *)::NewPtr( sizeof(CRadixTreeNodeRec) );

										if (lastSibling == NULL) 
											throw( ::MemError() );

										lastSibling->fParent = lastParent;
										lastSibling->fChildren = NULL;
										lastSibling->fKeyData = ::NewPtr( inKeyLen );
										
										if ( lastSibling->fKeyData == NULL )
											{
											
												::DisposePtr( (Ptr)lastSibling );
												throw( ::MemError() );
											
											}
											
										::BlockMoveData( inKeyStr, lastSibling->fKeyData, inKeyLen );
												
										lastSibling->fKeyDataLen = inKeyLen;

										if ( inDataSize > 0 )
											{
													
												lastSibling->fContextPtr = ::NewPtr( inDataSize );
												
												if ( lastSibling->fContextPtr == NULL )
													{

														::DisposePtr( (Ptr)lastSibling->fKeyData );
														::DisposePtr( (Ptr)lastSibling );
													
														throw( ::MemError() );
													
													}
													
												lastSibling->fContextDataLen = inDataSize;
												
												::BlockMoveData( inContextPtr, lastSibling->fContextPtr, lastSibling->fContextDataLen );

											}
										else
											{
											
												lastSibling->fContextPtr = NULL;
												lastSibling->fContextDataLen = 0;
												
											}
												
										lastSibling->fSyntheticNode = false;
										
										if ( inCurNode->fKeyData[inCurIndex + simCount] < inKeyStr[inCurIndex + simCount] )
											{
											
												lastSibling->fSiblings = inCurNode->fSiblings;
												inCurNode->fSiblings = lastSibling;
												
											}
											
										else
											{
											
												if ( lastParent->fChildren == inCurNode )
													lastParent->fChildren = lastSibling;
																											
												lastSibling->fSiblings = inCurNode;

											}
											
										mKeyCount++;
								
									}
						
							}
						
					}
				
			}
			
/*
**  Else if the target key length is the same as the current node key length,
**  reset the context ptr.
*/

		else if ( (match) && (inKeyLen == inCurNode->fKeyDataLen) )
			{
			
				if ( inCurNode->fContextPtr != NULL )
					{
					
						::DisposePtr( (Ptr)inCurNode->fContextPtr );
					
						inCurNode->fContextDataLen = 0;
						
					}
					
				if ( inDataSize > 0 )
					{
					
						inCurNode->fContextPtr = ::NewPtr( inDataSize );
						
						if ( inCurNode->fContextPtr == NULL )
							{

								inCurNode->fContextDataLen = 0;
								
								throw( ::MemError() );
							
							}
							
						inCurNode->fContextDataLen = inDataSize;
						
					}
				else
					{
						inCurNode->fContextPtr = NULL;
						inCurNode->fContextDataLen = 0;					
					}

/*
**  If the user set the context data of a synthetic node, it is
**  no longer synthetic.
*/
				
				inCurNode->fSyntheticNode = false;
				
				if ( inDataSize > 0 )
					::BlockMoveData( inContextPtr, inCurNode->fContextPtr, inCurNode->fContextDataLen );
			
			}
		
	}
