//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CTCPSocket.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code that does all the network transactions.
		 
	Modification History:

		13 Feb 1999		Ram Rajadhyaksha
		
					Elimated CTCPSocketErr exception type and error handling. Modified
					CTCPSocketYieldProc to return OSErr.
										
-------------------------------------------------------------------------------- */


#include <string.h>
#include <Threads.h>
#include <Gestalt.h>
#include <OpenTransport.h>
#include <OpenTptInternet.h>

#include "ThrowOSErr.h"
#include "CTCPSocket.h"


Boolean CTCPSocket::sInitialized 			= false;
OTConfiguration	*CTCPSocket::sConfig	= NULL;
	

CTCPSocket::CTCPSocket( CTCPSocketYieldProc inYieldProc, void *inContextPtr )
	{
	
		if ( sInitialized == false )
			Initialize();
			
		mEndpointRef 	= NULL;
		mDNSProvider 	= NULL;
		mYieldProc 		= inYieldProc;
		mContextPtr 	= inContextPtr;
		
		memset( &mEndpointInfo, 0, sizeof(TEndpointInfo) );
		memset( &mTCall, 0, sizeof(TCall) );
		
		OpenEndpoint();
	
	}
	
	
CTCPSocket::~CTCPSocket()
	{
	
		CloseEndpoint();
			
	}
	

#pragma mark -


void
CTCPSocket::Bind()
	{
	
		OSStatus					intOTResult 	= noErr;
		TCall							bind					= {0};
		InetInterfaceInfo	interfaceInfo;
		
		if ( mEndpointRef != NULL )
			{
				/* Set up the bind structure */
				bind.addr.maxlen = sizeof(InetAddress);
				bind.addr.len    = sizeof(InetAddress);
				bind.addr.buf    = (UInt8 *) &mBindAddress;
			
				intOTResult = ::OTBind( mEndpointRef, NULL, (TBind*) &bind );
				ThrowOSErr( intOTResult );
	
				intOTResult = ::OTInetGetInterfaceInfo( &interfaceInfo, kDefaultInetInterface );
				ThrowOSErr( intOTResult );
				
				OTInitInetAddress( &mBindAddress, mBindAddress.fPort, interfaceInfo.fAddress );
				
				intOTResult = ::OTSetAsynchronous( mEndpointRef );
				ThrowOSErr( intOTResult );			
			}
	
	}
	

void
CTCPSocket::Bind( const UInt32 inAddress, const UInt16 inPort )
	{
	
		OSStatus	intOTResult	= noErr;
		TBind			bindInfo 		= { 0 };
		TCall			bind				= { 0 };
		InetInterfaceInfo	interfaceInfo;
		
		if ( mEndpointRef != NULL )
			{			
				bindInfo.addr.buf = (UInt8 *)::NewPtr( sizeof(InetAddress) );
				
				if ( bindInfo.addr.buf == NULL )
					{
						ThrowOSErr( kCouldNotAllocMemory );
					}
				else
					{					
						bindInfo.addr.maxlen = sizeof( InetAddress );
						bindInfo.addr.len = sizeof( InetAddress );
						
						((InetAddress *)bindInfo.addr.buf)->fAddressType 	= AF_INET;
						((InetAddress *)bindInfo.addr.buf)->fPort 				= inPort;
						((InetAddress *)bindInfo.addr.buf)->fHost 				= inAddress;

						/* Set up the bind structure */
						bind.addr.maxlen = sizeof(InetAddress);
						bind.addr.len    = sizeof(InetAddress);
						bind.addr.buf    = (UInt8 *) &mBindAddress;
			
						intOTResult = ::OTBind( mEndpointRef, &bindInfo, (TBind*) &bind );						
						::DisposePtr( (char *)bindInfo.addr.buf );						
						ThrowOSErr( intOTResult );
											
						intOTResult = ::OTInetGetInterfaceInfo( &interfaceInfo, kDefaultInetInterface );						
						ThrowOSErr( intOTResult );

						intOTResult = ::OTSetAsynchronous( mEndpointRef );						
						ThrowOSErr( intOTResult );						
					}		
			}
	
	}
	
	
void
CTCPSocket::Unbind()
	{
	
		OSStatus intOTResult = noErr;
		
		if ( mEndpointRef != NULL )
			{			
				intOTResult = ::OTSetSynchronous( mEndpointRef );					
				ThrowOSErr( intOTResult );
					
				intOTResult = ::OTUnbind( mEndpointRef );				
				ThrowOSErr( intOTResult );		
			}
			
	}
	

#pragma mark -


void
CTCPSocket::Connect( const Str255 inAddressStr, const UInt16 inPort, const UInt32 inTimeOut )
	{
	
		OSStatus			intOTResult 					= noErr;
	
		InetHostInfo	remoteHostInfo 				= {0};
		TCall					remoteHostAddr				= {0};

		UInt32				intStartTicks					= 0;
		Boolean				bDone									= false;
		char					remoteHostCStr[256]		= "";
	
		if ( (mEndpointRef != NULL) && (mDNSProvider == NULL) )
			{				
				::BlockMoveData( &inAddressStr[1], remoteHostCStr, inAddressStr[0] );
				remoteHostCStr[inAddressStr[0]] = 0x00;
				
				if ( mDNSProvider == NULL )
					{						
						mDNSProvider = ::OTOpenInternetServices( kDefaultInternetServicesPath, 0x00000000, &intOTResult );
												
						if ( mDNSProvider == NULL )
							intOTResult = kCouldNotResolve;
							
						if ( intOTResult == noErr )
							intOTResult = ::OTSetAsynchronous( mDNSProvider );						
					}
							
				if ( intOTResult == noErr )
					intOTResult = ::OTInetStringToAddress( mDNSProvider, remoteHostCStr, &remoteHostInfo );
			
				if ( intOTResult == noErr )
					{					
						intStartTicks = ::TickCount();
						
						while ( !bDone )
							{
/*
**  Ok, we're kinda SOL here. There is no way to tell, without a notifer,
**  that our call has completed. So we poll the actual InetHostInfo struct.
**  Yes, this is lame, but can you think of a better way?
*/						
								if ( remoteHostInfo.addrs[0] != 0 )
									{
										bDone = true;
									}
								else if ( ((::TickCount() - intStartTicks) / 60) > inTimeOut )
									{									
										bDone = true;
										intOTResult = kTimeout;										
									}
									
								SocketYield();									
							}														
					}

				if ( intOTResult == noErr )
					{												
						remoteHostAddr.addr.buf = (UInt8 *)::NewPtr( sizeof(InetAddress) );
						
						if ( remoteHostAddr.addr.buf != NULL )
							{							
								remoteHostAddr.addr.maxlen = sizeof(InetAddress);
								remoteHostAddr.addr.len = sizeof(InetAddress);
								((InetAddress *)remoteHostAddr.addr.buf)->fAddressType 		= AF_INET;
								((InetAddress *)remoteHostAddr.addr.buf)->fHost 					= mPeerAddress.fHost = remoteHostInfo.addrs[0];
								((InetAddress *)remoteHostAddr.addr.buf)->fPort 					= mPeerAddress.fPort = inPort;							
							}
						else intOTResult = kCouldNotAllocMemory;				
					}
					
				if ( intOTResult == noErr )
					{					
						intOTResult = ::OTConnect( mEndpointRef, &remoteHostAddr, NULL );
					
						if ( intOTResult == kOTNoDataErr )
							intOTResult = noErr;
							
						if ( intOTResult == kOTBufferOverflowErr )
							intOTResult = noErr;
							
						if ( intOTResult == noErr )
							{								
								bDone = false;
								intStartTicks = ::TickCount();
								
								while ( !bDone )
									{								
										if ( ::OTLook(mEndpointRef) == T_CONNECT )
											{
												bDone = true;
											}
										else if ( ((::TickCount() - intStartTicks) / 60) > inTimeOut )
											{											
												bDone = true;
												intOTResult = kTimeout;											
											}
											
										SocketYield();										
									}								
							}													
					}
					
				if ( intOTResult == noErr )
					{
						intOTResult = ::OTRcvConnect( mEndpointRef, &mTCall );
	
						if ( intOTResult == kOTBufferOverflowErr )
							intOTResult = noErr;					
					}
					
				if ( mDNSProvider != NULL )
					{					
						::OTCloseProvider( mDNSProvider );
						mDNSProvider = NULL;							
					}
					
				if ( remoteHostAddr.addr.buf != NULL )
					::DisposePtr( (char *)remoteHostAddr.addr.buf );						
			}
		else intOTResult = kOTOutStateErr;
						
		ThrowOSErr( intOTResult );
	
	}
	

void											
CTCPSocket::Disconnect( UInt32 inTimeOut )
	{
	
		OSStatus 			intOTResult			= noErr;
		OTResult			endpointState		= 0;
		OTEventCode		endpointEvent		= 0;
		SInt32				intStartTicks		= 0;
		Boolean				bDone						= true;
		
		if ( mEndpointRef != NULL )
			{				
				endpointState = ::OTGetEndpointState( mEndpointRef );
				
				if ( endpointState == T_DATAXFER )
					{				
						intOTResult = ::OTSndOrderlyDisconnect( mEndpointRef );
					
						if ( intOTResult == noErr )
							{						
								intStartTicks = ::TickCount();
								
								while ( !bDone )
									{									
										endpointEvent = ::OTLook( mEndpointRef );
										
										if ( endpointEvent == T_ORDREL )
											{									
												intOTResult = ::OTRcvOrderlyDisconnect( mEndpointRef );											
												bDone = true;											
											}										
										else if ( endpointEvent == T_DISCONNECT )
											{										
												intOTResult = ::OTRcvDisconnect( mEndpointRef, NULL );											
												bDone = true;									
											}									
										else if ( ((::TickCount() - intStartTicks) / 60) > inTimeOut )
											{								
												intOTResult = ::OTSndDisconnect( mEndpointRef, &mTCall );											
												bDone = true;										
											}							
										else SocketYield();											
									}																	
							}							
						else if ( (intOTResult == kOTLookErr) && (::OTLook(mEndpointRef) == T_DISCONNECT) )
							{
								intOTResult = ::OTRcvDisconnect( mEndpointRef, NULL );
							}
					}					
				else if ( endpointState == T_INREL )
					{					
						intOTResult = ::OTRcvOrderlyDisconnect( mEndpointRef );					
					}
				else if ( endpointState > T_IDLE )
					{					
						intOTResult = kOTOutStateErr;					
					}
					
				if ( intOTResult != noErr )
					{					
						::OTCloseProvider( mEndpointRef );
						mEndpointRef = NULL;
						
						OpenEndpoint();
						
						intOTResult = noErr;						
					}					
			}
		else intOTResult = kOTOutStateErr;
		
		ThrowOSErr( intOTResult );
			
	}
	

#pragma mark -


void	
CTCPSocket::SendData(	void *inData, UInt32 inDataSize, UInt32 inTimeOut )
	{
	
		OSStatus		intOTResult	 		= noErr;		
		SInt32			intDataSent			= 0;
		SInt32			intStartTicks		= 0;		
		Boolean			bDone						= false;
		
		Assert_( (mEndpointRef != NULL) && (inData != NULL) && (inDataSize > 0) );
		
		if ( (mEndpointRef != NULL) && (inData != NULL) && (inDataSize > 0) )
			{			
				intStartTicks = ::TickCount();
				
				while ( !bDone )
					{				
						intOTResult = ::OTSnd( mEndpointRef, ((char *)inData) + intDataSent, inDataSize - intDataSent, 0 );
		
						if ( intOTResult == (inDataSize - intDataSent) )
							{							
								intDataSent += intOTResult;
								
								intOTResult = noErr;
								
								bDone = true;								
							}							
						else if ( (intOTResult > 0) || (intOTResult == kOTFlowErr) )
							{				
								if ( intOTResult > 0 )
									{									
										intStartTicks = ::TickCount();
										
										intDataSent += intOTResult;				
									}									
							}
						else if ( intOTResult == kOTLookErr )
							{						
								OTEventCode	theEvent = ::OTLook( mEndpointRef );
								
								bDone = true;							
							}						
						else
							{ 
								bDone = true;
							}
							
						if ( !bDone )
							{
								SocketYield();

								if ( ((::TickCount() - intStartTicks) / 60) > inTimeOut )
									{									
										intOTResult = kTimeout;
										
										bDone = true;								
									}								
							}					
					}				
			}
			
		ThrowOSErr( intOTResult );
	
	}

	
void
CTCPSocket::ReceiveData( char *outData, UInt32 &inOutSize, UInt32 inTimeOut )
	{
	
		OSStatus 	intOTResult 		= noErr;
		OTFlags		rcvFlags				= 0;
		SInt32		intStartTicks		= 0;
		UInt32		intDataFetched	= 0;
		Boolean		bDone 					= false;
		
		if ( mEndpointRef != NULL )
			{		
				intStartTicks = ::TickCount();
				
				while ( !bDone )
					{				
						intOTResult = ::OTRcv( mEndpointRef, ((char *)outData) + intDataFetched, inOutSize - intDataFetched, &rcvFlags );
				
						if ( intOTResult == (inOutSize - intDataFetched) )
							{						
								intDataFetched += intOTResult;
						
								intOTResult = noErr;
						
								bDone = true;									
							}							
						else if ( intOTResult > 0 )
							{							
								intStartTicks = ::TickCount();
							
								intDataFetched += intOTResult;
								
								intOTResult = noErr;									
							}					
						else if ( intOTResult == kOTNoDataErr )
							{
								intOTResult = noErr;
							}
						else 
							{
								bDone = true;
							}
							
						if ( !bDone )
							{						
								SocketYield();
							
								if ( ((::TickCount() - intStartTicks) / 60) > inTimeOut )
									{									
										intOTResult = kTimeout;
										
										bDone = true;										
									}									
							}					
					}
					
				inOutSize = intDataFetched;				
			}
	
		ThrowOSErr( intOTResult );
		
	}
	

#pragma mark -


/*
pascal void 
CTCPSocket::SocketNotifier( void *inContextPtr, OTEventCode inEventCode, OTResult inResult, void *inCookie)
	{

		CTCPSocket	*inSocket	= (CTCPSocket *)inContextPtr;
		
		if ( inSocket != NULL )
			{
			
				if ( inSocket->mWaitEvent == inEventCode )
					{
					
						inSocket->mWaitEvent = 0;
						inSocket->mResult = inResult;
						
					}
				
			}
			
	}
*/


void
CTCPSocket::SocketYield()
	{	
		OSErr intResult = noErr;
		
		if ( mYieldProc != NULL )
			{				
				intResult = (*mYieldProc)(this, mContextPtr);
				ThrowOSErr( intResult );						
			}				
	}
	
	
void
CTCPSocket::OpenEndpoint()
	{
	
		OSStatus intOTResult = noErr;
			
		mEndpointRef = ::OTOpenEndpoint( OTCloneConfiguration(sConfig), 0, &mEndpointInfo, &intOTResult );	
		ThrowOSErr( intOTResult );
			
		if ( mEndpointRef == NULL )
			ThrowOSErr( kCouldNotCreateSocket );
	
	}
	
	
void
CTCPSocket::CloseEndpoint()
	{
	
		if ( mEndpointRef != NULL )
			{			
				::OTCloseProvider( mEndpointRef );				
				mEndpointRef = NULL;				
			}
			
		if ( mDNSProvider != NULL)
			{			
				::OTCloseProvider( mDNSProvider );				
				mDNSProvider = NULL;			
			}
	
	}
	
	
void
CTCPSocket::Initialize()
	{
	
		OSStatus	intOTResult	= noErr;
		SInt32		response		= 0;
		
		if ( sInitialized == true )
			ThrowOSErr( kUnknownErr );
			
		intOTResult = ::Gestalt( gestaltOpenTpt, &response );
	
		if ( intOTResult != noErr )
			intOTResult = kOTNotInstalled;

		if ( intOTResult == noErr )
			if ( response == 0 )
				intOTResult = kOTNotInstalled;
					
		if ( intOTResult == noErr )
			{				
				sConfig = ::OTCreateConfiguration("tcp");
				
				if ( sConfig == NULL )
					intOTResult = kCouldNotInitialize;			
			}
				
		ThrowOSErr( intOTResult );	
		
		sInitialized = true;

	}
	

void
CTCPSocket::GetPeerHostname (char* ioName)
	{
	
		OSStatus	intOTResult 			= noErr;
		OSStatus	intIgnoredResult 	= noErr;

		intOTResult = ::OTSetSynchronous( mEndpointRef );
		ThrowOSErr( intOTResult );
		
		if ( mDNSProvider == NULL )
			{
				mDNSProvider = ::OTOpenInternetServices( kDefaultInternetServicesPath, 0, &intIgnoredResult );
										
				if ( mDNSProvider == NULL )
					ThrowOSErr( kCouldNotResolve );
			}

		intOTResult = ::OTInetAddressToName( mDNSProvider, mPeerAddress.fHost, ioName);
		ThrowOSErr( intOTResult );

		if ( mDNSProvider != NULL )
			{		
				::OTCloseProvider( mDNSProvider );
				mDNSProvider = NULL;					
			}
	
		intOTResult = ::OTSetAsynchronous( mEndpointRef );
		ThrowOSErr( intOTResult );
		
	}
	


