//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CTCPStream.cp
	
	Authors:		Scott Silver and Ram Rajadhyaksha
	
	Description:
	
		This file contains code to implement a simple stream over TCP/IP.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Fixed casting problem in GetBytes to get it to compile.
			
	Notes:
	
		This class used to originally be derived from PowerPlant's LStream class.
		However I ran into a problem that LStream implored you not to throw exceptions
		from PutBytes/GetBytes routines. This presented a big problem with aborting 
		threads because doing so throws a CThreadedTaskException. So, I've decided to 
		ditch the PP class.
					
-------------------------------------------------------------------------------- */


#include <string.h>
#include <stdio.h>

#include "ThrowOSErr.h"
#include "CTCPSocket.h"
#include "CTCPStream.h"


/*
#define CREATE_LOG_FILE 1
#if CREATE_LOG_FILE

#include "CCVSClientApp.h"

#endif
*/


CTCPStream::CTCPStream( const UInt32 inTimeOut )
	{

		mSocket 		= NULL;
		mCounter 		= 0;
		mMaxCounter = 0xFFFFFFFF;
		mTimeOut		= inTimeOut;

		mZStreamState.zalloc 				= NULL;
		mZStreamState.zfree 				= NULL;
		mZStreamState.opaque 				= NULL;
		
		mZStreamState.next_in				= NULL;
		mZStreamState.avail_in			= NULL;
		mZStreamState.total_in			= NULL;
		mZStreamState.next_out			= NULL;
		mZStreamState.avail_out			= kCompressedBufferSize;
		mZStreamState.total_out			= NULL;
		
		int err = inflateInit2(&mZStreamState, -15);
		Assert_(err == 0);
			
#ifdef NEWLINE_MODE
		mSendBuffer	= ::NewPtrClear( kSendBufferAllocationSize );		// don't try to resize this on the fly. You will often
																																// fail because there is a locked block above this one.
		Assert_( mSendBuffer != NULL );
		
		if ( mSendBuffer == NULL )
			{
				throw ::MemError();
			}
		
		mSendBufferAllocated = kSendBufferAllocationSize;
		mSendBufferUsed = 0;
#endif
		
/*
**  Time out should be at least one second.
*/
		
		if ( mTimeOut < 60 )
			mTimeOut = 60;
		
	}


CTCPStream::~CTCPStream()
	{

		try
			{
			
				CloseConnection();
			
			}
			
		catch ( ... )
			{
			
			
			}
		
#ifdef NEWLINE_MODE
		if ( mSendBuffer )
			{
				::DisposePtr( mSendBuffer );
				mSendBuffer = NULL;
			}
#endif

		inflateEnd(&mZStreamState);
	}

void 
CTCPStream::OpenConnection(	CTCPSocketYieldProc inYieldProc, void *inContextPtr, const Str255 inHostNamePStr, const UInt16 inHostPort, const UInt16 inLocalPort )
	{
		
		OSErr			result	= noErr;
		Boolean		done		= false;

// Initialization: Bind to any local port, then connect to the remote host.
// sometimes we can't get a given port so we cycle through them.
		
		if ( mSocket != NULL )
			{
			
				delete mSocket;
				mSocket = NULL;
				
			}
			
		mSocket = new CTCPSocket( inYieldProc, inContextPtr );
		
		if ( inLocalPort != 0 )
		
			mSocket->Bind( 0, inLocalPort);
		
		else mSocket->Bind();
						
		mSocket->Connect( inHostNamePStr, inHostPort, mTimeOut );
			
	}


void CTCPStream::
CloseConnection()
	{

#ifdef NEWLINE_MODE
		Assert_( mSendBufferUsed == 0 );
#endif

		// We're done with the connection bail out.
		
		if ( mSocket != NULL ) 
			{
			
				try 
					{

						mSocket->Disconnect( mTimeOut );	// This yields until we are done

					}
					
				catch ( ... ) 
					{
					
					}
			
				delete mSocket;
				mSocket = NULL;
				
			}
		
	}	

//#ifdef NEWLINE_MODE
// ---------------------------------------------------------------------------
//		 UnbufferedWriteBlock
// ---------------------------------------------------------------------------
//	Write data, specified by a pointer and byte count, to a Stream
//	Don't buffer or use newline mode

void
CTCPStream::UnbufferedWriteBlock( const void *inBuffer, SInt32 inByteCount )
	{

	 	PutBytes( inBuffer, inByteCount
#ifdef NEWLINE_MODE
				 	, false
#endif
				 	);

	}
//#endif

// ---------------------------------------------------------------------------
//		 WriteBlock
// ---------------------------------------------------------------------------
//	Write data, specified by a pointer and byte count, to a Stream

void
CTCPStream::WriteBlock( const void *inBuffer, SInt32 inByteCount )
	{

	 	PutBytes( inBuffer, inByteCount );

	}


// ---------------------------------------------------------------------------
//		 ReadBlock
// ---------------------------------------------------------------------------
//	Read data from a Stream to a buffer

void
CTCPStream::ReadBlock( void *outBuffer, SInt32 inByteCount )
	{

		GetBytes( outBuffer, inByteCount );

	}


// ---------------------------------------------------------------------------
//		 InflateStartup
// ---------------------------------------------------------------------------
void
CTCPStream::InflateStartup(UInt32 totalCompressedFileSize)
{
		mZStreamState.avail_in				= NULL;
		mZStreamState.avail_out				= kCompressedBufferSize;
		
		int err = inflateReset(&mZStreamState);
		Assert_(err == 0);

		mSkipZHeader 				 					= true;
		mTotalCompressedSize 					= totalCompressedFileSize;
		mTotalCompressedDataRead 			= 0;
}

// ---------------------------------------------------------------------------
//		 InflateShutdown
// ---------------------------------------------------------------------------
void
CTCPStream::InflateShutdown()
{
}

// ---------------------------------------------------------------------------
//		 ReadBlockAndInflate
// ---------------------------------------------------------------------------
//	Read data from a Stream to a buffer.  Run the ZLibCompressor's inflate
//  routine on the compressed data, decompress the data into the output
//  buffer.  Stop decompressing when there is enough data in the output buffer
//  to satisfy the original request.
//
// 	Returns: # of bytes we decompressed into the output buffer
//					 (ie, how many bytes are available for writing)

SInt32
CTCPStream::ReadBlockAndInflate( void *outBuffer, SInt32 decompByteCountRequest, UInt32 &outBytesCompressed )
{
	// consume/inflate all bytes left in the compressedBuffer
	Boolean	reachedCompressedStreamEnd = false;

	outBytesCompressed = 0;
	
	// while there are bytes left in the output buffer
	mZStreamState.next_out 	= (UInt8*) outBuffer;
	mZStreamState.avail_out = decompByteCountRequest;
	
	// while there's space in the output buffer, and we're not at the end of the compressed stream
	while ( (mZStreamState.avail_out > 0) && !reachedCompressedStreamEnd)
	{
		unsigned char *compressed_datastart 	= mCompressedBuffer;
		UInt32 				 compressed_datasize 		= kCompressedBufferSize;
		
		// fill up the compressed data buffer if it's empty or needs refilling
		if ( mZStreamState.avail_in == 0 )
		{			
			// if we've read everything within the last 16K of the compressed data, read less than
			// 16K for the last read of the compressed data
			UInt32 compressed_dataremaining = mTotalCompressedSize - mTotalCompressedDataRead;
			if ( compressed_dataremaining < kCompressedBufferSize )
				compressed_datasize = compressed_dataremaining;
			
			ReadBlock( mCompressedBuffer, compressed_datasize );
			mTotalCompressedDataRead += compressed_datasize;
			outBytesCompressed += compressed_datasize;
		}

		// if we haven't yet decompressed any compressed data ever, 
		// skip over the prepended .gz header
		if ( mSkipZHeader )
		{
			unsigned char* header_end 	= ::skip_header(mCompressedBuffer);
			UInt32 				 header_size	= header_end - (unsigned char*) compressed_datastart;
					
			// throw?
			// now compares with null
			Assert_(header_end != NULL);
			
			// the amount we just read must be more than the size of the header
			//  throw?
			Assert_(compressed_datasize >= header_size );
			
			compressed_datastart =  header_end;
			compressed_datasize  -= header_size;
			
			mSkipZHeader = false;
		}

		// If we just read a chunk and skipped over the .gz header
		// otherwise don't modify the next location of compressed data
		if ( mZStreamState.avail_in == 0 )
		{			
			mZStreamState.next_in 	= compressed_datastart;
			mZStreamState.avail_in 	= compressed_datasize;
		}
				
		int zlibError = inflate( &mZStreamState, Z_NO_FLUSH );

		switch (zlibError)
		{
			case Z_OK:
				break;
			case Z_STREAM_END:
				reachedCompressedStreamEnd = true;
				break;
			case Z_NEED_DICT:
				Assert_(false);
				break;
			case Z_MEM_ERROR:
				Assert_(false);
				break;
			case Z_BUF_ERROR:
				// this means the inflater could make no progress (which for us, we just report that)
				// the user should have called needsInput() to determine just that.
				Assert_(false);
				break;
			case Z_DATA_ERROR:
				Assert_(false);
			case Z_STREAM_ERROR:
				break;
			default:
				Assert_("unknown return value from inflate" && false);
		}
	}
		
	// return how much we actually decompressed
	return decompByteCountRequest - mZStreamState.avail_out;
}
		
void 
CTCPStream::GetBytes( void *outBuffer, SInt32 &ioByteCount )
	{

		Boolean					expeditedData 		= false;
		UInt32					ioByteCountCasted = 0;
		
		if ( mSocket == NULL )
			{
				ThrowOSErr( CTCPSocket::kIllegalState );
			}
		else 
			{

//  This is a temporary casting fix. 
	
				ioByteCountCasted = ioByteCount;
				
				mSocket->ReceiveData( (char *)outBuffer, ioByteCountCasted, mTimeOut );
				
				ioByteCount = ioByteCountCasted;
							
			}
			
	}
	

void 
CTCPStream::PutBytes(		const void 		*inBuffer
												, SInt32 				&ioByteCount
#ifdef NEWLINE_MODE
												, Boolean				inNewlineBuffer
#endif
													)
	{

/*
		ExceptionCode	result	= noErr;
		
		#ifdef DEBUG
		
			if (ioByteCount == 0)
				DebugStr("\pSending a zero length packet is special.  Don't do it");

		#endif
		
		#define SEARCH_FOR_NULL 0		

		#if defined(SEARCH_FOR_NULL) && SEARCH_FOR_NULL
			char *cp;
			
			for (cp = (char*) inBuffer; cp < (char*) inBuffer + ioByteCount; cp++)
			{
				if (*cp == '\0')
					DebugStr("\pYou're sending a string with a NULL char in it");
			}
		#endif
			
		#if CREATE_LOG_FILE
			OutputToLog((char*) inBuffer, ioByteCount);
		#endif
*/


		if ( mSocket == NULL )
			{
				ThrowOSErr( CTCPSocket::kIllegalState );
			}
		else if ( ioByteCount == 0 )
			{
				ThrowOSErr( CTCPSocket::kBadParametersErr );
			}
		else if ( ioByteCount > 0 ) 
			{

#ifdef NEWLINE_MODE
	
				if ( inNewlineBuffer )
					{
						// if we are going to have too much to keep in the buffer, send what we have, otherwise append.
					
						if ( mSendBufferUsed + ioByteCount >= mSendBufferAllocated )
							{
								
								mSocket->SendData( mSendBuffer, mSendBufferUsed, mTimeOut );
								mSendBufferUsed = 0;
								
								mSocket->SendData( (char *)inBuffer, ioByteCount, mTimeOut );
							}
						else			// append the new stuff to the buffer
							{
							
								::BlockMoveData( inBuffer, mSendBuffer + mSendBufferUsed, ioByteCount );
								mSendBufferUsed += ioByteCount;
								
								// if there is a newline in the buffer, send up to that
								char		*bufEnd = mSendBuffer + mSendBufferUsed - 1;
								
								while ( bufEnd > mSendBuffer )
									{
										
										if (*bufEnd == '\n')
											{
												bufEnd ++;		//include the newline
												break;
											}
										
										bufEnd --;
									}
									
								if ( bufEnd > mSendBuffer )
									{
										
											mSocket->SendData( mSendBuffer, bufEnd - mSendBuffer, mTimeOut );
									
											// move up any remaining data
											SInt32		dataLeft = mSendBufferUsed - (bufEnd - mSendBuffer);
											
											Assert_( dataLeft >= 0 );
											
											if ( dataLeft > 0 )
												{
													::BlockMoveData( bufEnd, mSendBuffer, dataLeft );
												}
											
											mSendBufferUsed = dataLeft;
									}
							
							}
					}
				else			// no buffering
					{
						if ( mSendBufferUsed > 0 )
							{
								Assert_( false );
								mSocket->SendData( mSendBuffer, mSendBufferUsed, mTimeOut );
								mSendBufferUsed = 0;
							}
							
						mSocket->SendData( (char *)inBuffer, ioByteCount, mTimeOut );
						
					}
#else
				
				mSocket->SendData( (char *)inBuffer, ioByteCount, mTimeOut );
#endif
									
			}
		
	}

/*
void 
CTCPStream::StartCounting(UInt32 inMaxBytes) 
	{

		mCounter = 0; 
		mMaxCounter = inMaxBytes; 

	}


void 
CTCPStream::StopCounting() 
	{ 

		Boolean doThrow =  mCounter != mMaxCounter;
		mCounter = 0;  mMaxCounter = 0xFFFFFFFF;
		ThrowIf_(doThrow);

	}
*/

char 
CTCPStream::ReadWord(	char *inOutWordCStr, UInt32 &inOutLen, const char *inTermCStr )
	{
		
		Boolean		done						= false;
		
		char			*curTerminator	= NULL,
							nextChar				= 0x00,
							result					= 0x00;
		
		UInt32		len							= 0x00000000;
		
// Ask for one character at a time (buffered)
// If we hit any character in the terminators
// return that terminator and return the string (in word)
		
		if ( (inOutWordCStr != NULL) && (inOutLen > 0) && (inTermCStr != NULL) && (strlen(inTermCStr) > 0) ) 
			{
		
				while ( !done ) 
					{
						
						if ( len < inOutLen ) 
							{
								
								ReadBlock( &nextChar, 1 );

// this will wrongly terminate a string
// for whatever reason, CVS likes to send us these
								
								if (nextChar == '\0') 
									continue;
							
// now see if we hit a terminator
						
								curTerminator = (char *)inTermCStr;
								
								while ( (*curTerminator != '\0') && (!done) ) 
									{
									
										if ( nextChar == *curTerminator ) 
											done = true; 
										
										else curTerminator++;	
																			
									}
								
								if ( !done ) 
								
									inOutWordCStr[len++] = nextChar;
												
										
							} else done = true;
						
					}
				
				inOutLen = len;
				result = *curTerminator;
			
			} 
		else inOutLen = 0;
		
		return ( result );
		
	}


Boolean
CTCPStream::ReadDataLine( char *inOutDataStr, UInt32 &inOutDataLen )
	{
		
		char			nextChar			= 0x00;
		
		UInt32 		totalRead			= 0x00000000,
							len						= 0x00000000;

		Boolean		bDone					= false,
							bResult				= false;

		if ( inOutDataStr == NULL )
			ThrowOSErr( CTCPSocket::kBadParametersErr );
			
		while ( (!bDone) && (len < inOutDataLen) ) 
			{
			
			 	ReadBlock( &nextChar, 1 );
				
				if ( nextChar == 0x0A )
					{
					
						bDone = true;
						bResult = true;
						
					}
					
				else inOutDataStr[len++] = nextChar;
				
			}
		
		inOutDataLen = len;
		
		return bResult;
		
	}


Boolean
CTCPStream::ReadEntireDataLine( char *inOutDataStr, UInt32 &inOutDataLen )
	{
		
		char			nextChar			= 0x00;
		
		UInt32 		totalRead			= 0x00000000,
							len						= 0x00000000;

		Boolean		bDone					= false,
							bResult				= true;

		if ( inOutDataStr == NULL )
			ThrowOSErr( CTCPSocket::kBadParametersErr );
			
		while ( !bDone ) 
			{
			
			 	ReadBlock( &nextChar, 1 );
				
				if ( nextChar == 0x0A )
					{
					
						bDone = true;
						
					}
					
				else if ( len < inOutDataLen )
					{
					
						inOutDataStr[len++] = nextChar;
				
					}
					
				else bResult = false;
					
			}
		
		inOutDataLen = len;
		
		return bResult;
		
	}


SInt32 
CTCPStream::WriteZeroByte()
	{

		SInt32	result		= 1;
		char	nullChar 	= 0x00;
		
		WriteBlock( &nullChar, result );

		return result;

	}


SInt32 
CTCPStream::WriteCString( const char *inString )
	{
		
		SInt32	result	= 0x00000000;

		if ( inString == NULL )
			ThrowOSErr( CTCPSocket::kBadParametersErr );
		
		result = strlen( inString );
			
		if ( result > 0 )
			WriteBlock( inString, result );

		return result;

	}


SInt32
CTCPStream::ReadCString( char *outString )
	{
		
		UInt32	len = 255;

		if ( outString == NULL )
			ThrowOSErr( CTCPSocket::kBadParametersErr );

/*
**  Ok, here is why C strings suck: there is no way to tell the
**  length of the passed in stringptr. So, I'm going to assume
**  that the length is at least 255 bytes. Since this is an override
**  of an LStream call, I can't add a size param either.
*/

/*
**  This function does not write the terminator byte. 
*/
		
		ReadDataLine( outString, len );

		return len;
			
	}


SInt32
CTCPStream::WritePString( ConstStr255Param inString )
	{

/*
**  WritePString() writes a Pascal style string to
**  the networking *without* it's length byte.
*/

		SInt32	result	= 0x00000000;

		if ( inString == NULL )
			ThrowOSErr( CTCPSocket::kBadParametersErr );
	
		result = inString[0];
			
		if ( result > 0 )
			WriteBlock( &inString[1], result );

		return result;
			
	}
	

SInt32
CTCPStream::ReadPString( Str255 outString )
	{

		UInt32 	len = 255;

		if ( outString == NULL )
			ThrowOSErr( CTCPSocket::kBadParametersErr );
		
		ReadDataLine( (char *)&outString[1], len );
		
		outString[0] = len;
		
		return ( len + 1 );
		
	}

