//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CVSEngine.cp
//
//	  Author: 	Ram Rajadhyaksha
//
//################################################################################


#include "CCVSRepository.h"
#include "CVSEngine.h"


pascal OSErr CreateCVSEngine( const FSSpec											*inLocalRoot,
															CCVSEngineFileMappingRec					*inFileMappings,
															const UInt32											inFileMappingsCount,
															CCVSEngineEventNotifierProcPtr		inNotifier,
															CCVSEngineStreamNotifierProcPtr		inMessageNotifier,
															CCVSEngineStreamNotifierProcPtr		inErrorNotifier,
															void															*inContextPtr,
															const UInt32											inTimeOut,
															CCVSEngineRef	 										*outEngineRef)
	{
	
		OSErr						result			= noErr;
		CCVSRepository	*repository	= NULL;
		
		if ( outEngineRef != NULL )
			{
			
				try
					{
					
						repository = new CCVSRepository( inLocalRoot, inFileMappings, inFileMappingsCount, inNotifier, inMessageNotifier, inErrorNotifier, inContextPtr, inTimeOut );
					
						*outEngineRef = (CCVSEngineRef)repository;
						
					}
					
				catch ( OSErr inErr )
					{
					
						result = inErr;
						
					}
					
				catch ( ... )
					{
					
						result = kCVSEngineUnknownErr;
						
					}
					
			}
			
		return result;
			
	}
	
																							
pascal void DisposeCVSEngine( CCVSEngineRef inEngineRef )
	{
	
		if ( inEngineRef != NULL )
			{
			
				try
					{
					
						delete ((CCVSRepository *)inEngineRef);
					
					}
					
				catch ( ... )
					{
					
					}
			
			}
			
	}
	
	
#pragma mark -


pascal OSErr CVSEngineLogin( 	CCVSEngineRef 										inEngineRef, 
															const CCVSEngineLoginOptions 			*inOptions )
	{
	
		return ((CCVSRepository *)inEngineRef)->Login( inOptions );
		
	}
	
																							
pascal OSErr CVSEngineAdd( 		CCVSEngineRef 										inEngineRef, 
															const FSSpec 											*inTargetItem, 
															const CCVSEngineAddOptions 				*inOptions )
	{
	
		return ((CCVSRepository *)inEngineRef)->Add( inTargetItem, inOptions );
	
	}
																							
	
pascal OSErr CVSEngineCheckout( CCVSEngineRef 									inEngineRef, 
															const FSSpec 											*inTargetItem, 
															const CCVSEngineCheckoutOptions 	*inOptions )
	{
	
		return ((CCVSRepository *)inEngineRef)->Checkout( inTargetItem, inOptions );
	
	}
	
																							
pascal OSErr CVSEngineCommit( CCVSEngineRef 										inEngineRef, 
															const FSSpec 											*inTargetItem, 
															CCVSEngineCommitOptions 					*inOptions )
	{
	
		return ((CCVSRepository *)inEngineRef)->Commit( inTargetItem, inOptions );
	
	}
	
																							
pascal OSErr CVSEngineDiff( 	CCVSEngineRef 										inEngineRef, 
															const FSSpec 											*inTargetItem, 
															const CCVSEngineDiffOptions				*inOptions,
															CCVSEngineStreamNotifierProcPtr 	inDiffNotifier,
															void 															*inContextPtr )
	{
	
		return ((CCVSRepository *)inEngineRef)->Diff( inTargetItem, inOptions, inDiffNotifier, inContextPtr );
	
	}
	

pascal OSErr CVSEngineImport( CCVSEngineRef 										inEngineRef, 
															const FSSpec 											*inTargetItem, 
															const CCVSEngineImportOptions 		*inOptions)
	{
	
		return ((CCVSRepository *)inEngineRef)->Import( inTargetItem, inOptions );
	
	}


pascal OSErr CVSEngineEdit(		CCVSEngineRef											inEngineRef,
															const FSSpec 											*inTargetItem)
	{
		return ((CCVSRepository *)inEngineRef)->Edit( inTargetItem );
	}


pascal OSErr CVSEngineEditors(
															CCVSEngineRef											inEngineRef,
															const FSSpec 											*inTargetItem,
															CCVSEngineStreamNotifierProcPtr		inNotifier,
															void															*inContextPtr)
	{
		return ((CCVSRepository *)inEngineRef)->Editors( inTargetItem, inNotifier, inContextPtr );
	}

																						
pascal OSErr CVSEngineLog( 		CCVSEngineRef 										inEngineRef,
															const FSSpec 											*inTargetItem, 
															CCVSEngineStreamNotifierProcPtr 	inLogNotifier, 
															void 															*inContextPtr)
	{
	
		return ((CCVSRepository *)inEngineRef)->Log( inTargetItem, inLogNotifier, inContextPtr );
		
	}
	
																							
pascal OSErr CVSEngineStatus(	CCVSEngineRef											inEngineRef,
															const FSSpec 											*inTargetItem,
															CCVSEngineStreamNotifierProcPtr		inStatusNotifier,
															void															*inContextPtr)
	{
	
		return ((CCVSRepository *)inEngineRef)->Status( inTargetItem, inStatusNotifier, inContextPtr );
	
	}
	
																							
pascal OSErr CVSEngineTag(		CCVSEngineRef										 	inEngineRef,
															const FSSpec 											*inTargetItem,
															const CCVSEngineTagOptions				*inTagOptions)
	{
	
		return ((CCVSRepository *)inEngineRef)->Tag( inTargetItem, inTagOptions );
	
	}
	
	
pascal OSErr CVSEngineRemove(	CCVSEngineRef											inEngineRef,
															const FSSpec 											*inTargetItem)
	{
	
		return ((CCVSRepository *)inEngineRef)->Remove( inTargetItem );
	
	}


pascal OSErr CVSEngineUnEdit(	CCVSEngineRef											inEngineRef,
															const FSSpec 											*inTargetItem)
	{
		return ((CCVSRepository *)inEngineRef)->UnEdit( inTargetItem );
	}


pascal OSErr CVSEngineUpdate(	CCVSEngineRef											inEngineRef,
															const FSSpec 											*inTargetItem, 
															const CCVSEngineUpdateOptions			*inOptions)
	{
	
		return ((CCVSRepository *)inEngineRef)->Update( inTargetItem, inOptions );
	
	}
					

pascal OSErr CVSEngineWatchOnOff( CCVSEngineRef inEngineRef, const FSSpec *inTargetItem, const Boolean inWatch )
	{
		return ((CCVSRepository *)inEngineRef)->WatchOnOff( inTargetItem, inWatch );	
	}
					
																						
pascal OSErr CVSEngineWatchAddRemove( CCVSEngineRef inEngineRef, const FSSpec *inTargetItem, const CCVSEngineWatchOptions *inOptions )
	{
		return ((CCVSRepository *)inEngineRef)->WatchAddRemove( inTargetItem, inOptions );	
	}


pascal OSErr CVSEngineWatchers(
															CCVSEngineRef											inEngineRef,
															const FSSpec 											*inTargetItem,
															CCVSEngineStreamNotifierProcPtr		inNotifier,
															void															*inContextPtr)
	{
		return ((CCVSRepository *)inEngineRef)->Watchers( inTargetItem, inNotifier, inContextPtr );
	}


pascal SInt32	CVSEngineVersion(CCVSEngineRef										inEngineRef)
	{
	
		return ((CCVSRepository *)inEngineRef)->Version();
		
	}
	
