//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################


/* -------------------------------------------------------------------------------

	File:				CVSEngineUtils.cp
	
	Authors:		Scott M. Silver and Ram Rajadhyaksha
	
	Description:
	
		This file contains all sorts of useful stuff for the CVS engine.
		 
	Modification History:

		15 	Dec		1998		Ram Rajadhyaksha
		
					Integrated Peter Bierman's changes.
					
		25	July 	1998		Ram Rajadhyaksha
		
					FSpGetLocalCVSPath() no longer returns '.' results if the targetSpec
					and baseSpec are the same directory.
										
-------------------------------------------------------------------------------- */


#include <stdio.h>
#include <string.h>
#include <time.h>

#include "ThrowOSErr.h"
#include "MoreFiles.h"
#include "FileCopy.h"

#include "StringUtils.h"
#include "CVSEngineUtils.h"


#ifndef __VERIFYCHECKSUM
#define __VERIFYCHECKSUM
#endif


unsigned char kCVSDirectoryName[]				= "\pCVS";
unsigned char	kCVSTagName[]							= "\pTag";
unsigned char	kCVSRepositoryName[]			= "\pRepository";
unsigned char kCVSRootName[]						= "\pRoot";
unsigned char kCVSEntriesName[]					= "\pEntries";
unsigned char kCVSEntriesStaticName[] 	= "\pEntries.Static";

#ifdef CVSENGINE_TERMINATE_WITH_LF

unsigned char kTerminatorByte				= 0x0A;

#else

unsigned char kTerminatorByte				= 0x0D;

#endif

unsigned char kCRChar								= 0x0D;
unsigned char kLFChar								= 0x0A;


SInt32	gIntCachedGMT								= -1;


pascal UInt32 GenerateCkidChecksum(	Handle 				inCkidData);
Boolean 			ishex(								char 					c);
static int 		htoi(									unsigned char *s);


pascal Boolean CUIsValidCVSName( ConstStr255Param inNamePStr )
	{

			char *cp;
			
			for (cp = (char*) inNamePStr + 1; cp <= (char*) inNamePStr + inNamePStr[0]; cp++)
				{
				
					if (*cp == '\\'	 || *cp == '/')
						{
						
							return (false);
						
						}
						
				}	
			
			return (true);
			
	}


pascal OSErr CUIsValidMacName( ConstStr255Param inNamePStr )
	{
	
		OSErr		result   = noErr;
		UInt32	intIndex = 0;
		
		if ( (inNamePStr[0] > 31) || (inNamePStr[0] == 0) )
			{
				result = kCVSEngineInvalidFileNameErr;
			}
			
		if ( result == noErr )
			{
				for ( intIndex = 1; ((result == noErr) && (intIndex <= inNamePStr[0])); intIndex++ )
					{
						if ( inNamePStr[intIndex] == ':' )
							{
								result = kCVSEngineInvalidFileNameErr;
							}
					}
			}
			
		return result;
		
	}
	

pascal OSErr CUTranslateFromMacNamePStr( ConstStr255Param inNamePStr, Str255 pstrOutTranslation )
	{

		OSErr									result 	= noErr;
		const unsigned char 	*cpIn;
		unsigned char 				*cpOut	= NULL;
		const char 						*hex 		= "0123456789ABCDEF";
		
		unsigned char isAcceptable[96] =
		//	0 1 2 3 4 5 6 7 8 9 A B C D E F            0123456789ABCDEF
		 	{ 1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,0,    // 2x   !"#$%&'()*+,-./
			 	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,    // 3x  0123456789:;<=>?
			 	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,    // 4x  @ABCDEFGHIJKLMNO
			 	1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,    // 5X  PQRSTUVWXYZ[\]^_
			 	0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,    // 6x  `abcdefghijklmno
			 	1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,0 };  // 7X  pqrstuvwxyz{\}~DEL
				
		if ( inNamePStr == NULL || pstrOutTranslation == NULL )
			{
				result = kCVSEngineBadParamErr;
			}
		else
			{
				pstrOutTranslation[0] = 0;
			}
				
		if ( result == noErr )
			{
				for (cpIn = inNamePStr + 1, cpOut = pstrOutTranslation + 1, pstrOutTranslation[0] = 0;
													cpIn <= inNamePStr + inNamePStr[0]; cpIn++)
					{
						if (*cpIn>=32 && *cpIn<128 && isAcceptable[*cpIn-32])
							{
								*cpOut++ = *cpIn;
								pstrOutTranslation[0]++;
							}
						else
							{
								*cpOut++ = '%';
								*cpOut++ = hex[*cpIn >> 4];
								*cpOut++ = hex[*cpIn & 15];
								pstrOutTranslation[0] += 3;
							}
					}
			}
			
		return result;
		
	}


pascal OSErr CUTranslateToMacName( char *inOutDataStr, UInt32 &inOutDataLen )
	{
		
		UInt32 	inDataLen;
		UInt32 	outDataLen;
		char 		*cpIn;
		char 		*cpOut;
		
		for (inDataLen = inOutDataLen, outDataLen = 0, cpIn = inOutDataStr, cpOut = cpIn;
									inDataLen>0; )
		{
			if (cpIn[0] == '%' && ishex(cpIn[1]) && ishex(cpIn[2]))
			{
				cpOut[0] = htoi((unsigned char *)cpIn + 1);
				cpIn += 2;
				inDataLen -= 2;
			}
			else
				cpOut[0] = cpIn[0];
				
			cpIn++;
			cpOut++;
			
			inDataLen--;
			outDataLen++;
		}
		
		inOutDataLen = outDataLen;
		
		return noErr;
		
	}


#pragma mark -


pascal SInt32 CUCompareCVSVersion( char *inVersion1CStr, char *inVersion2CStr )

/*
**  Ok, there are arguably two ways to do this. The first method is to
**  parse each term out of the version string, convert it to a number,
**  and compare it. This is what I do here. Unfortunately the performance
**  totally blows. The other option I thought of was to run through
**  both strings backwards multiply each term by a power of ten, effectively
**  converting it to a number and then comparing both. The problem with
**  the second method is if you have a 9 digit or greater version string,
**  you will encounter overflow problems because of the 32 bit number limitation.
*/

	{

		SInt32			result 		= 0x00000000,
								term1			= 0x00000000,
								term2			= 0x00000000;
						
		Boolean			done			= false;

		Str255			ver1PStr	= "\p",
								ver2PStr	= "\p";
						
		UInt8				*temp			= NULL;
		
		while ( !done ) 
			{
		
				ver1PStr[0] = 0x00;
				ver2PStr[0] = 0x00;
/*
**  Parse a new term out of the first version string...
*/
				temp = &ver1PStr[1];
		
				while (((*inVersion1CStr) != '.') && ((*inVersion1CStr) != 0x00)) 
					{
					
						*temp = (unsigned char)*inVersion1CStr;	
					
						temp++;
						inVersion1CStr++;
						ver1PStr[0]++;
					
					}
		
				if ( (*inVersion1CStr) == '.' )
					inVersion1CStr++;		
						
/*
**  Convert it to a number or -1 if there was no term...
*/			
				
				if ( ver1PStr[0] > 0 )
					::StringToNum( ver1PStr, &term1 );
		
				else term1 = -1;
				
/*
**  Parse a new term out of the second version string...
*/			
		
				temp = &ver2PStr[1];
		
				while (((*inVersion2CStr) != '.') && ((*inVersion2CStr) != 0x00)) 
					{
					
						*temp = (unsigned char)*inVersion2CStr;	
					
						temp++;
						inVersion2CStr++;
						ver2PStr[0]++;
					
					}
				
				if ((*inVersion2CStr) == '.')
					inVersion2CStr++;
					
				if ( ver2PStr[0] > 0 )
					::StringToNum( ver2PStr, &term2 );
					
				else term2 = -1;
			
				if (term1 > term2) 
					{
					
						result = 1;
						done = true;
							
					}
					
				else if (term1 < term2) 
					{
					
						result = -1;	
						done = true;	
						
					}
					
/*
**  If both terms are -1, then we have reached the end
**  of both version strings and that means they are equal.
*/
				
				else if ((term1 == -1) && (term2 == -1))
					done = true;
					
			}
	
		return result;

	}


pascal OSErr CUISOTimeStrToSeconds( char *strInTime, SInt32 *intOutSeconds )
	{
		
		OSErr					intResult 			= noErr;
		
		UInt32				intCastingCrap 	= 0;
		SInt32				intDay					= 0;
		SInt32				intHour					= 0;
		SInt32				intMin					= 0;
		SInt32				intSec					= 0;
		SInt32				intYear					= 0;
		
		char					strMonth[256]		= "";
		DateTimeRec		dtRec;

		Assert_( strInTime != NULL );
		Assert_( intOutSeconds != NULL );
		
		if ( (strInTime == NULL) || (intOutSeconds == NULL) )
			intResult = kCVSEngineBadParamErr;
		else if ( strlen(strInTime) > 255 )
			intResult = kCVSEngineBadParamErr;
		else
			{			
				memset( &dtRec, 0, sizeof(dtRec) );
/*
**  get our difference from GMT
*/
				if ( gIntCachedGMT == -1 )
					{			
						MachineLocation mlLoc;
				
						::ReadLocation( &mlLoc );
				
				// so it turns out this value they give is a 3 byte SIGNED value..this is to get the
				// right sign extension
				
						gIntCachedGMT = ((SInt32) ((mlLoc.u.gmtDelta & 0x00FFFFFF) << 8)) >> 8; // only top 3 work
					}	
		
				//  sscan has got to be one of the most wonderful functions in
				//  the entire universe.
			
				sscanf( strInTime, "%*s %s %2d %2d:%2d:%2d %4d", strMonth, &intDay, &intHour, &intMin, &intSec, &intYear );
	
				dtRec.year = intYear;			
				dtRec.day = intDay;
				dtRec.hour = intHour;
				dtRec.minute = intMin;
				dtRec.second = intSec;
				
				if ( !strcmp(strMonth, "Jan") )
					dtRec.month = 1;
				else if ( !strcmp(strMonth, "Feb") )
					dtRec.month = 2;
				else if ( !strcmp(strMonth, "Mar") )
					dtRec.month = 3;
				else if ( !strcmp(strMonth, "Apr") )
					dtRec.month = 4;
				else if ( !strcmp(strMonth, "May") )
					dtRec.month = 5;
				else if ( !strcmp(strMonth, "Jun") )
					dtRec.month = 6;
				else if ( !strcmp(strMonth, "Jul") )
					dtRec.month = 7;
				else if ( !strcmp(strMonth, "Aug") )
					dtRec.month = 8;
				else if ( !strcmp(strMonth, "Sep") )
					dtRec.month = 9;
				else if ( !strcmp(strMonth, "Oct") )
					dtRec.month = 10;
				else if ( !strcmp(strMonth, "Nov") )
					dtRec.month = 11;
				else if ( !strcmp(strMonth, "Dec") )
					dtRec.month = 12;
					
				DateToSeconds(&dtRec, &intCastingCrap);
				(*intOutSeconds) = intCastingCrap;
				(*intOutSeconds) += gIntCachedGMT;			
			}
				
		return intResult;
		
	}
	
	
pascal OSErr CUISOTimeStrFromSeconds( char *strInOutTime, UInt32 *intInOutLen, const SInt32 intInSeconds )
	{
	
		OSErr					intResult 		= noErr;
		UInt32				intLen				= 0;
		char					*pStrTime			= NULL;

		DateTimeRec		dtRec;
		tm						tmTime;
		
		Assert_( strInOutTime != NULL );
		Assert_( intInOutLen != NULL );
		
		if ( (strInOutTime == NULL) || (intInOutLen == NULL) )
			intResult = kCVSEngineBadParamErr;
		else if ( (*intInOutLen) == 0 )
			intResult = kCVSEngineBadParamErr;
		else
			{			
				memset( &tmTime, 0, sizeof(tmTime) );
/*
**  get our difference from GMT
*/
				if ( gIntCachedGMT == -1 )
					{			
						MachineLocation mlLoc;
				
						::ReadLocation( &mlLoc );
				
						// so it turns out this value they give is a 3 byte SIGNED value..this is to get the
						// right sign extension
				
						gIntCachedGMT = ((SInt32) ((mlLoc.u.gmtDelta & 0x00FFFFFF) << 8)) >> 8; // only top 3 work
					}	
				
				::SecondsToDate( intInSeconds - gIntCachedGMT, &dtRec );
				
				tmTime.tm_sec = dtRec.second;
				tmTime.tm_min = dtRec.minute;
				tmTime.tm_hour = dtRec.hour;
				tmTime.tm_mday = dtRec.day;
				tmTime.tm_mon = dtRec.month - 1;
				tmTime.tm_year = dtRec.year - 1900;
			
				mktime( &tmTime );
				
				pStrTime = asctime( &tmTime );
				
				if ( pStrTime != NULL )
					{
					
						intLen = strlen( pStrTime );
						
						if ( (intLen > 0) && (intLen < (*intInOutLen)) )
							{
								
								if ( pStrTime[intLen - 1] == '\n' )
									intLen--;
										
								::BlockMoveData( pStrTime, strInOutTime, intLen );
								*intInOutLen = intLen;
								
							}
						else *intInOutLen = 0;
							
					}
				else *intInOutLen = 0;
					
			}
		
		return intResult;
		
	}
	
/*
	UnixTimeToMacSeconds
	
	Takes a unix string from the date output of the log view
	and converts into seconds on this Macintosh.  We take into
	account GMT conversion.
*/


pascal UInt32 CUUnixTimeStringToMacSeconds( char *inTimeStr, SInt32 *inOutGMTDelta )
	{
	
		UInt32				result 			= 0x00000000;
		SInt32				tempNum			= 0x00000000,
									gmtDelta		= -1;
		Str255				tempString	= "\p";
		DateTimeRec		theDate;

		if ( inOutGMTDelta != NULL )
			gmtDelta = *inOutGMTDelta;
			
/*
**  get our difference from GMT
*/

		if ( gmtDelta == -1 )
			{
			
				MachineLocation location;
				
				::ReadLocation(&location);
				
				// so it turns out this value they give is a 3 byte SIGNED value..this is to get the
				// right sign extension
				
				gmtDelta = ((SInt32) ((location.u.gmtDelta & 0x00FFFFFF) << 8)) >> 8; // only top 3 work

				if ( inOutGMTDelta != NULL )
					*inOutGMTDelta = gmtDelta;

			}	
		
		
		if ( inTimeStr != NULL )
			{
			
				if ( strlen(inTimeStr) == 19 ) 
					{
					
						BlockMove(inTimeStr, tempString + 1, 4);
						tempString[0] = 4;
						/*
						**  With six calls to StringToNum(), you might as will
						**  write a quickie CharToNum() function and do the
						**  addition of powers.
						*/		
						StringToNum(tempString, &tempNum);
						theDate.year = tempNum;
						
						tempString[0] = 2;
						tempString[1] = inTimeStr[5];
						tempString[2] = inTimeStr[6];
						
						StringToNum(tempString, &tempNum);
						theDate.month = tempNum;
						
						tempString[1] = inTimeStr[8];
						tempString[2] = inTimeStr[9];

						StringToNum(tempString, &tempNum);
						theDate.day = tempNum;
						
						tempString[1] = inTimeStr[11];
						tempString[2] = inTimeStr[12];
						
						StringToNum(tempString, &tempNum);
						theDate.hour = tempNum;
						
						tempString[1] = inTimeStr[14];
						tempString[2] = inTimeStr[15];
						
						StringToNum(tempString, &tempNum);
						theDate.minute = tempNum;
						
						tempString[1] = inTimeStr[17];
						tempString[2] = inTimeStr[18];
						
						StringToNum(tempString, &tempNum);
						theDate.second = tempNum;
						
						DateToSeconds(&theDate, &result);
						result += gmtDelta;
						
					}
					
			}
				
		return result;
		
	}


pascal Boolean CUIsCVSBinaryOption( ConstStr255Param inOptionPStr )
	{
	
		Boolean bResult = false;
		
		if ( inOptionPStr != NULL )
			if ( inOptionPStr[0] == 3 )
				if ( inOptionPStr[1] == '-' )
					if ( inOptionPStr[2] == 'k' )
						if ( (inOptionPStr[3] == 'o') || (inOptionPStr[3] == 'b') ) 
							bResult = true;
			
		return bResult;
	
	}
	

pascal Boolean CUIsAddedOrRemovedRevision( ConstStr255Param pstrInRevision )
	{
		Boolean bResult = false;
		
		if ( (pstrInRevision[0] > 0) && (pstrInRevision[1] == '-') )
			bResult = true;
		else if ( (pstrInRevision[0] == 1) && (pstrInRevision[1] == '0') )
			bResult = true;
			
		return bResult;
	}


pascal void CUMakeDiffCommandLineOptionsString( const CCVSEngineDiffOptions *inDiffOptions, Str255 ioOptionsPStr, Boolean inDisplayFormat )
	{
		Str255		tempStr;
		
		ioOptionsPStr[0] = 0x00;
		
		if ( inDiffOptions == nil )
			return;
		
		if ( inDisplayFormat )
			{
			
				if ( inDiffOptions->mIgnoreCase )
					PStrConcat( ioOptionsPStr, "\p -i", 255);

				if ( inDiffOptions->mIgnoreWhiteSpace )
					PStrConcat( ioOptionsPStr, "\p -w", 255);
				
				if ( inDiffOptions->mIgnoreSpaceChange )
					PStrConcat( ioOptionsPStr, "\p -b", 255);

				if ( inDiffOptions->mIgnoreBlankLines )
					PStrConcat( ioOptionsPStr, "\p -B", 255);

				if ( inDiffOptions->mRCSDiff )
					PStrConcat( ioOptionsPStr, "\p -n", 255);

				if ( inDiffOptions->mShowCFunction )
					PStrConcat( ioOptionsPStr, "\p -p", 255);

				// context diff
				if ( inDiffOptions->mContextDiff )
					{
						PStrConcat( ioOptionsPStr, "\p -c", 255);
						::NumToString( (Int32)inDiffOptions->mContextLines, tempStr );
						PStrConcat( ioOptionsPStr, tempStr, 255);
					}

				if ( inDiffOptions->mUnifiedDiff )
					{
						PStrConcat( ioOptionsPStr, "\p -u", 255);
						::NumToString( (Int32)inDiffOptions->mContextLines, tempStr );
						PStrConcat( ioOptionsPStr, tempStr, 255);
					}			
			
				if ( inDiffOptions->mUseRevisions )
					{
						PStrConcat( ioOptionsPStr, "\p -r ", 255);
						PStrConcat( ioOptionsPStr, inDiffOptions->mOrigRevisionPStr, 255);
						
						PStrConcat( ioOptionsPStr, "\p -r ", 255);
						PStrConcat( ioOptionsPStr, inDiffOptions->mDiffRevisionPStr, 255);
					}
			
			}
		else
			{			
		
				Str255		optionsStr = "\p";
			
				if ( inDiffOptions->mIgnoreCase )
					PStrConcat( optionsStr, "\pi", 255);

				if ( inDiffOptions->mIgnoreWhiteSpace )
					PStrConcat( optionsStr, "\pw", 255);
				
				if ( inDiffOptions->mIgnoreSpaceChange )
					PStrConcat( optionsStr, "\pb", 255);

				if ( inDiffOptions->mIgnoreBlankLines )
					PStrConcat( optionsStr, "\pB", 255);

				if ( inDiffOptions->mRCSDiff )
					PStrConcat( optionsStr, "\pn", 255);

				if ( inDiffOptions->mShowCFunction )
					PStrConcat( optionsStr, "\pp", 255);

				// context diff
				if ( inDiffOptions->mContextDiff )
					{
						PStrConcat( optionsStr, "\pc", 255);
						::NumToString( (Int32)inDiffOptions->mContextLines, tempStr );
						PStrConcat( optionsStr, tempStr, 255);
					}

				if ( inDiffOptions->mUnifiedDiff )
					{
						PStrConcat( optionsStr, "\pu", 255);
						::NumToString( (Int32)inDiffOptions->mContextLines, tempStr );
						PStrConcat( optionsStr, tempStr, 255);
					}
				
				if ( optionsStr[0] > 0 )
					{
						PStrConcat( ioOptionsPStr, "\p -", 255);
						PStrConcat( ioOptionsPStr, optionsStr, 255);
					}
		
			}
					
	}
	

pascal OSErr CUCreateRootPStr( const AuthenticationMethod intInMethod, const Str255 pstrInUser, const Str255 pstrInServer, const UInt16 intInPort, const Str255 pstrInPath, Str255 pstrOutRoot )
	{
	
		OSErr	intResult = noErr;
		
		if ( (pstrInUser == NULL) || (pstrInServer == NULL) || (pstrOutRoot == NULL) || (pstrInPath == NULL) )
			intResult = kCVSEngineBadParamErr;
		else
			{			
				pstrOutRoot[0] = 0;

/*
**  we must use sizeof(Str255) because sizeof(pstrOutRoot) == 4
*/
				
				switch ( intInMethod )
					{
						case kAuthenticationKClient:
							PStrConcat( pstrOutRoot, "\p:kserver:", sizeof(Str255) );
						break;
						
						case kAuthenticationRSH:
							PStrConcat( pstrOutRoot, "\p:server:", sizeof(Str255) );
						break;
						
						case kAuthenticationPassword:
							PStrConcat( pstrOutRoot, "\p:pserver:", sizeof(Str255) );
						break;
						
						default:
							Assert_( false );
							PStrConcat( pstrOutRoot, "\p:pserver:", sizeof(Str255) );
						break;
					}
			
				if ( pstrInUser[0] > 0 )
					{
						PStrConcat( pstrOutRoot, pstrInUser, sizeof(Str255) );
						PStrConcat( pstrOutRoot, "\p@", sizeof(Str255) );
					}
					
				PStrConcat( pstrOutRoot, pstrInServer, sizeof(Str255) );
				PStrConcat( pstrOutRoot, "\p:", sizeof(Str255) );
				PStrConcat( pstrOutRoot, pstrInPath, sizeof(Str255) );
			}
			
		return intResult;
		
	}
	

pascal OSErr CUParseRootPStr( ConstStringPtr pstrInRoot, Str255 pstrOutUser, Str255 pstrOutHost, Str255 pstrOutPath, AuthenticationMethod *intOutMethod, UInt16 *intOutPort )
	{
		OSErr			result 				= noErr;
		UInt32		intLen				= 0;
		UInt32		intCopyLen		= 0;

		char			strRoot[256]	= "";
		char			*pLineTerm 		= NULL;
		char 			*pCurrent			= NULL;
		Str255		pstrTemp 			= "\p";

		try
			{
				if ( pstrInRoot == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}

				//  Initialize all our possible parameters.
				//  If we exit because the root is 0, these
				//  could have potentially been uninitialized.
				
				if ( pstrOutUser != NULL )
					{
						pstrOutUser[0] = 0;
					}
					
				if ( pstrOutHost != NULL )
					{
						pstrOutHost[0] = 0;
					}
					
				if ( pstrOutPath != NULL )
					{
						pstrOutPath[0] = 0;
					}

				if ( intOutMethod != NULL )
					{
						*intOutMethod = 0;
					}
					
				if ( intOutPort != NULL )
					{
						*intOutPort = 0;
					}

				intLen = pstrInRoot[0];
										
				if ( intLen > 0 )
					{
						::BlockMoveData( &pstrInRoot[1], strRoot, pstrInRoot[0] );
						strRoot[intLen] = 0;
											
						pCurrent = strRoot;

						//  If the string starts with a ':', then it must
						//  include the connection method and user/host name.
						
						if ( *pCurrent == ':' )
							{
								pCurrent++;						
								pLineTerm = strpbrk( pCurrent, ":" );
				
								if ( pLineTerm == NULL )
									{
										ThrowOSErr( kCVSEngineBadRootStrErr );
									}
									
								//  Get the authentication method
								
								if ( intOutMethod != NULL )
									{
										if ( (pLineTerm - pCurrent) <= 255 )
											{
												pstrTemp[0] = pLineTerm - pCurrent;
												::BlockMoveData( pCurrent, &pstrTemp[1], pstrTemp[0] );
												
												//  um, duh.
												PStrLowercase( pstrTemp );
												
												if ( PStrCompare(pstrTemp, "\pserver") )
													{
														*intOutMethod = kAuthenticationRSH;
													}
												else if ( PStrCompare(pstrTemp, "\ppserver") )
													{
														*intOutMethod = kAuthenticationPassword;
													}
												else
													{
														*intOutMethod = 0;
													}
											}
									}
									
								pCurrent = pLineTerm + 1;

								//  Get the user name
								
								pLineTerm = strpbrk( pCurrent, "@:" );
								
								if ( pLineTerm == NULL )
									{
										ThrowOSErr( kCVSEngineBadRootStrErr );
									}

								if ( *pLineTerm	== '@' )
									{
										if ( pstrOutUser != NULL )
											{
												intCopyLen = pLineTerm - pCurrent;
												
												if ( intCopyLen > 255 )
													{
														intCopyLen = 255;
													}
											
												pstrOutUser[0] = intCopyLen;
												::BlockMoveData( pCurrent, &pstrOutUser[1], pstrOutUser[0] );
											}
											
										pCurrent = pLineTerm + 1;
										pLineTerm = strpbrk( pCurrent, ":" );
								
										if ( pLineTerm == NULL )
											{
												ThrowOSErr( kCVSEngineBadRootStrErr );
											}								
									}	
									
								//  Get the server name

								if ( pstrOutHost != NULL )
									{				
										intCopyLen = pLineTerm - pCurrent;
										
										if ( intCopyLen > 255 )
											{
												intCopyLen = 255;
											}

										pstrOutHost[0] = intCopyLen;
										::BlockMoveData( pCurrent, &pstrOutHost[1], pstrOutHost[0] );
									}
										
								pCurrent = pLineTerm + 1;

								//  There is no way to specify a port in the
								//  CVS root string. So we always return 0.
										
								if ( intOutPort != NULL )
									{
										*intOutPort = 0;
									}
							}
						else
							{
								//  Since we're on a Mac, it is going to be
								//  kAuthenticationPassword even though the
								//  root actually specifies a local disk-based
								//  repository.
								
								if ( intOutMethod != NULL )
									{
										*intOutMethod = kAuthenticationPassword;
									}
							}								
								
						//  The rest is the root path

						if ( pstrOutPath != NULL )
							{							
								intCopyLen = ( intLen - (pCurrent - strRoot) );
								
								if ( intCopyLen > 0 )
									{
										if ( intCopyLen > 255 )
											{
												intCopyLen = 255;
											}
											
										pstrOutPath[0] = intCopyLen;
										::BlockMoveData( pCurrent, &pstrOutPath[1], pstrOutPath[0] );		
									}
							}
					}
			}			
		catch ( OSErr intException )
			{
				result = intException;
			}
		catch ( ... )
			{
				Assert_( false );
				result = kCVSEngineUnknownExceptionErr;
			}
		
		return result;
			
	}
	

pascal OSErr CUParsePermissionPStr( ConstStringPtr pstrInItem, Boolean *bOutReadOnly )
	{
	
		OSErr			result 					= noErr;
		char			strPerm[256]	 	= "";
		char 			*pLineTerm			= NULL;
		char			*pCurrent			  = NULL;
		UInt32		intPermLen 			= 0;
		UInt32		intDataLen			= 0;
		Boolean		bFoundW					= false;
			
		if ( (pstrInItem == NULL) || (bOutReadOnly == NULL) )
			{
				result = kCVSEngineBadParamErr;
			}
		else
			{
				intPermLen = pstrInItem[0];
				::BlockMoveData( &pstrInItem[1], strPerm, intPermLen );
				strPerm[intPermLen] = 0;
			
				pCurrent = strPerm;
				pLineTerm = strstr( pCurrent, "u=" );		
				
				if ( pLineTerm != NULL )
					{
						pCurrent = pLineTerm + 2;
						pLineTerm = strpbrk( pCurrent, "," );
					
						if ( pLineTerm != NULL )
							{
								intDataLen = pLineTerm - pCurrent;
							}	
						else if ( pCurrent < (strPerm + intPermLen) )
							{
								intDataLen = (strPerm + intPermLen) - pCurrent;
							}
						else 
							{
								intDataLen = 0;
							}
							
						for ( UInt32 intIndex = 0; (intIndex < intDataLen) && (!bFoundW); intIndex++ )
							{
								if ( (pCurrent[intIndex] == 'w') || (pCurrent[intIndex] == 'W') )
									{
										bFoundW = true;
									}
							}
					
						*bOutReadOnly = !bFoundW;
					}
			}
			
		return result;
		
	}
	
	
pascal OSErr CUParsePermissionStr( const char *strInItem, Boolean *bOutReadOnly )
	{
	
		OSErr 	result 			= noErr;
		Str255	pstrItem 		= "\p";
		UInt32	intCopyLen	= 0;
		
		if ( (strInItem == NULL) || (bOutReadOnly == NULL) )
			{
				result = kCVSEngineBadParamErr;
			}
		else
			{
				intCopyLen = strlen( strInItem );
				
				if ( intCopyLen > 255 )
					{
						intCopyLen = 255;
					}
					
				::BlockMoveData( strInItem, &pstrItem[1], intCopyLen );
				pstrItem[0] = intCopyLen;
				
				result = CUParsePermissionPStr( pstrItem, bOutReadOnly );			
			}
			
		return result;
		
	}
	
	
pascal OSErr CUGetAddressAndPort(	ConstStringPtr pstrInItem, Str255 pstrOutAddress, UInt16 *intOutPort )
	{
	
		OSErr		result 				= noErr;
		char		strItem[256] 	= "";
		char 		*pLineTerm		= NULL;
		Str255	pstrPort			= "\p";
		UInt32	intTemp				= 0;
		UInt32	intCopyLen		= 0;
		
		if ( (pstrInItem != NULL) && (pstrOutAddress != NULL) && (intOutPort != NULL) )
			{
				pstrOutAddress[0] = 0;
				*intOutPort = 0;
				
				::BlockMoveData( &pstrInItem[1], strItem, pstrInItem[0] );
				strItem[pstrInItem[0]] = 0;
				
				pLineTerm = strchr( strItem, ':' );
				
				if ( pLineTerm != NULL )
					{
						intCopyLen = pLineTerm - strItem;
						pstrOutAddress[0] = intCopyLen;
						
						::BlockMoveData( strItem, &pstrOutAddress[1], pstrOutAddress[0] );
												
						if ( (pstrInItem[0] - intCopyLen) > 1 )
							{
								pstrPort[0] = pstrInItem[0] - (intCopyLen + 1);
								::BlockMoveData( pLineTerm + 1, &pstrPort[1], pstrPort[0] );
								
								PStrToUInt32( pstrPort, &intTemp );
								*intOutPort = intTemp;
							}
					}
				else
					{
						/*  Just copy the whole address over if there is no port.
						**  We use sizeof(Str255) because sizeof(pstrOutAddress)
						**  will return 4.
						*/
						
						PStrCopy( pstrInItem, pstrOutAddress, sizeof(Str255) );
					}
			}
		else
			{
				result = kCVSEngineBadParamErr;
			}
			
		return result;
		
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark WINDOW POSITION RESOURCE ROUTINES


// @@@pmb@@@ - get CW window pos resource, for later restore on commit with update
pascal OSErr CUReadWinPosResourceFromRef( Handle *winPosRsrcH, const SInt16 inFileRef )
	{
		OSErr			result				= noErr;
		SInt16		oldResFileRef = 0x0000;
		
		Assert_( winPosRsrcH != NULL );
		
		if ( (inFileRef > 0) && (winPosRsrcH != NULL) )
		{

			oldResFileRef = ::CurResFile();
			::UseResFile( inFileRef );

			*winPosRsrcH = ::Get1Resource( 'MPSR', kMPSRResourceID );
				
			if ( *winPosRsrcH == NULL )
				result = kCVSEngineFileHasNoWindowPos;
			else
				::DetachResource(*winPosRsrcH);

			Assert_( oldResFileRef > 0 );
		
			::UseResFile( oldResFileRef );
		}
		
		
		return result;
	}


// @@@pmb@@@ - save CW window pos resource
pascal OSErr FSpWriteWinPosResource( const FSSpec *inTargetSpec, Handle winPosRsrcH ) 
	{

		OSErr 		result 					= noErr;
		SInt16		fileRef 				= 0;
		SInt16		intOldResRef		= 0;
		FInfo 		finderInfo;
		
		if ( (inTargetSpec != NULL) && (winPosRsrcH != NULL) )
			{			
				Assert_( inTargetSpec->name[0] > 0 );
				Assert_( inTargetSpec->vRefNum != 0 );
				Assert_( inTargetSpec->parID != 0 );
					
				intOldResRef = ::CurResFile();
				
				::SetResLoad( false );		
				fileRef = ::FSpOpenResFile( inTargetSpec, fsRdWrPerm );		
				::SetResLoad( true );

				Assert_( fileRef != 0 );
							
				if ( fileRef <= 0 ) 
					{					
						result = ::ResError();

/*
**  If the file doesn't have a resource fork, create one.
*/
					
						if ( (result == eofErr) && (FSpIsAFile(inTargetSpec, NULL) == noErr) )
							{
								result = ::FSpGetFInfo( inTargetSpec, &finderInfo );
								
								if ( result == noErr )
									{
/*
**  Is it safe to use FSpCreateResFile() since it follows the
**  PMSP?
*/
						
										::FSpCreateResFile( inTargetSpec, finderInfo.fdCreator, finderInfo.fdType, smSystemScript );
										result = ::ResError();				
									}
													
								if ( result == noErr )
									{		
										Assert_( inTargetSpec->name[0] > 0 );
										Assert_( inTargetSpec->vRefNum != 0 );
										Assert_( inTargetSpec->parID != 0 );
									
										::SetResLoad( false );
										fileRef = ::FSpOpenResFile( inTargetSpec, fsRdWrPerm );
										::SetResLoad( true );
	
										Assert_( fileRef != 0 );
								
										if ( fileRef <= 0 )
											result = ::ResError();										
									}								
							}								
					}
					
				if ( (fileRef > 0) && (result == noErr) )
					result = CUWriteWinPosResourceFromRef( winPosRsrcH, fileRef );
				
				if ( fileRef > 0 )	
					::CloseResFile( fileRef );
				
				::UseResFile( intOldResRef );																
			}
			
			return result;

	}


// @@@pmb@@@
pascal OSErr CUWriteWinPosResourceFromRef( Handle winPosRsrcH, const SInt16 inFileRef )
	{
	
		OSErr 		result 				= noErr;
		SInt16		oldResFileRef = 0x0000;

		Assert_( winPosRsrcH != NULL );		
		Assert_( inFileRef > 0 );
		
		if ( (inFileRef > 0) && (winPosRsrcH != NULL) )
			{
			
				oldResFileRef = ::CurResFile();
				::UseResFile( inFileRef );
				
				Handle	oldRsrc	= NULL;
						
				oldRsrc = ::Get1Resource( 'MPSR', kMPSRResourceID );

				if ( (oldRsrc != NULL) && (::ResError() == noErr) )
					{
					
						::RemoveResource( oldRsrc );
						result = ::ResError();

						if ( result == noErr )
							{
							
								::DisposeHandle( oldRsrc );
								oldRsrc = NULL;
								
								::UpdateResFile( inFileRef );
								result = ::ResError();
								
							}

					}

				if ( result == noErr )
					{
					
								::AddResource( winPosRsrcH, 'MPSR', kMPSRResourceID, "\p" );
								result = ::ResError();
								
								if ( result == noErr )
									{
									
										::UpdateResFile( inFileRef );
										result = ::ResError();
									
									}
									
								if ( result == noErr )
									{
										::DetachResource( winPosRsrcH );
										result = ::ResError();
									}
					}

				Assert_( oldResFileRef > 0 );
				::UseResFile( oldResFileRef );
				
			}
			
		return result;
		
	}

	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark 'ckid' ROUTINES


pascal UInt32 GenerateCkidChecksum( Handle inCkidData )
	{
	  
	  SInt32		dataSize				= 0x00000000;
	  UInt32		checksum 				= 0x00000000;
	  UInt32		*dataPtrCasted 	= NULL;;

	  dataSize = ( ::GetHandleSize(inCkidData) / sizeof(SInt32) ) - 1;
	  
	  dataPtrCasted = (UInt32 *) *inCkidData;

/* 
**  skip over first long (checksum field) 
*/  

	  dataPtrCasted++;                  
	  
	  while ( dataSize-- > 0 ) 
	    checksum += *dataPtrCasted++;

	  return checksum;

	}
	

pascal OSErr CUReadCkidResourceFromRef( CkidResourcePB *outCkidPB, const Boolean inGetComment, const SInt16 inFileRef )
	{
	
		OSErr 									result 				= noErr;
		Handle									comment				= NULL;
		CkidResourceHeaderHndl	ckidResource 	= NULL;
		UInt32									ckidSize			= 0x00000000,
														curOffset			= 0x00000000,
														checksum			= 0x00000000;
		UInt16									len						= 0x0000;
		SInt16									oldResFileRef	= 0x0000;	
		
		if ( inFileRef > 0 )
			{
				
				oldResFileRef = ::CurResFile();
				::UseResFile( inFileRef );
			
				ckidResource = (CkidResourceHeaderHndl)::Get1Resource( 'ckid', kCkidResourceID );
				
				if ( ckidResource == NULL )
				
					result = kCVSEngineFileHasNoCkid;
					
				else
					{
					
						::DetachResource( (Handle)ckidResource );

						ckidSize = ::GetHandleSize( (Handle)ckidResource );
						
						if ( !(ckidSize >= sizeof(CkidResourceHeaderRec)) )
							result = kCVSEngineCkidCorrupted;
							
						if ( (**ckidResource).fCurrentVersion != kCurrentCkidVersion )
							result = kCVSEngineUnknownCkidVersion;
							
						if ( result == noErr )
							{

/*
**  Verify checksum
*/
										
								#ifdef __VERIFYCHECKSUM
								
								checksum = GenerateCkidChecksum( (Handle)ckidResource );
								
								if ( checksum != (**ckidResource).fChecksum )
									result = kCVSEngineBadCkidChecksum;
									
								else 
								
								#endif
								
									{
									
										::BlockMoveData( (*ckidResource), outCkidPB, sizeof(CkidResourceHeaderRec) );
																		
										curOffset = sizeof( CkidResourceHeaderRec );

/*
**  Now read the various Pascal strings.
*/

/*
**  Read remote path string...
*/
																			
										if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fRemotePathPStr[1], len );
														
														curOffset += len;
														
													}
														
												outCkidPB->fRemotePathPStr[0] = len;
												
											}
											
										curOffset++;

/*
**  Read user name string...
*/
	
										if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fUserNamePStr[1], len );
														
														curOffset += len;
														
													}
													
												outCkidPB->fUserNamePStr[0] = len;
												
											}

										curOffset++;

/*
**  Read revision string...
*/
							
										if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fRevisionPStr[1], len );
														
														curOffset += len;
														
													}
											
												outCkidPB->fRevisionPStr[0] = len;
												
											}
											
											curOffset++;

/*
**  Read file name string...
*/
									
											if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fFileNamePStr[1], len );
														
														curOffset += len;
														
													}
									
												outCkidPB->fFileNamePStr[0] = len;
												
											}
											
											curOffset++;

/*
**  Read task string...
*/
										
											if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fTaskPStr[1], len );
														
														curOffset += len;
														
													}

												outCkidPB->fTaskPStr[0] = len;
												
											}
											
											curOffset++;
				
/*
**  Now get the comment.
*/
										
											if ( (curOffset < ckidSize) && (inGetComment == true) )
											{
											
												len = *(UInt16 *)( (*(UInt8**)ckidResource) + curOffset );
												
												curOffset += 2;
		
/*
**  Make sure the size is valid...
*/

												outCkidPB->fCommentLen = len;
										
												if ( (curOffset + len) < ckidSize )
													{
														
														comment = ::NewHandle( len );
														
														if ( comment == NULL )
															{
															
																result = ::MemError();
																
																outCkidPB->fCommentLen = 0;
																outCkidPB->fCommentHndl = NULL;
																
															}
														else 
															{
															
																::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), *comment, len );
																
																outCkidPB->fCommentHndl = comment;
															
															}
													
													}

											}
											else 
												{
																	
													outCkidPB->fCommentLen = 0;
													outCkidPB->fCommentHndl = NULL;

												}

/*
**  Something went wrong somewhere...
*/
										
											if ( curOffset > ckidSize )
												result = kCVSEngineCkidCorrupted;
											
									}
									
							}
					
						::DisposeHandle( (Handle)ckidResource );
						ckidResource = NULL;
							
					}
	

				Assert_( oldResFileRef > 0 );
			
				::UseResFile( oldResFileRef );
				
			}
			
		return result;
		
	}
	
	
pascal OSErr FSpReadCkidResource( const FSSpec *inTargetSpec, CkidResourcePB *outCkidPB, const Boolean inGetComment )
	{

		OSErr 		result 				= noErr;
		SInt16		intFileRef		= 0;	
		SInt16 		intOldResRef 	= 0;
		
		if ( (inTargetSpec != NULL)	&& (outCkidPB != NULL) ) 
			{									
				Assert_( inTargetSpec->name[0] > 0 );
				Assert_( inTargetSpec->vRefNum != 0 );
				Assert_( inTargetSpec->parID != 0 );

				intOldResRef = ::CurResFile();
					
				::SetResLoad( false );
				intFileRef = ::FSpOpenResFile( inTargetSpec, fsRdPerm );		
				::SetResLoad( true );

				Assert_( intFileRef != 0 );
							
				if ( intFileRef <= 0 ) 
					{					
						result = ::ResError();								
					}					
				else 
					{					
						result = CUReadCkidResourceFromRef( outCkidPB, inGetComment, intFileRef );						
						
						::CloseResFile( intFileRef );		
						intFileRef = 0;			
					}
				
				::UseResFile( intOldResRef );
			}		
		else 
			{
				result = kCVSEngineBadParamErr;
			}
			
		return result;
			
	}


pascal OSErr CUWriteCkidResourceFromRef( CkidResourcePB *inCkidPB, const SInt16 inFileRef )
	{
	
		OSErr 		result 				= noErr;
		SInt16		oldResFileRef = 0x0000;
		
		if ( inFileRef > 0 )
			{
			
				oldResFileRef = ::CurResFile();
				::UseResFile( inFileRef );
				
				Handle	oldCkidRsrc	= NULL;
						
				oldCkidRsrc = ::Get1Resource( 'ckid', kCkidResourceID );

				if ( (oldCkidRsrc != NULL) && (::ResError() == noErr) )
					{
					
						::RemoveResource( oldCkidRsrc );
						result = ::ResError();

						if ( result == noErr )
							{
							
								::DisposeHandle( oldCkidRsrc );
								oldCkidRsrc = NULL;
								
								::UpdateResFile( inFileRef );
								result = ::ResError();
								
							}

					}

				if ( result == noErr )
					{
					
						Handle	newCkidRsrc 			= NULL;
						SInt32	newCkidRsrcSize 	= 0x00000000;
						
						newCkidRsrcSize = 	sizeof(CkidResourceHeader) + 
																(inCkidPB->fRemotePathPStr[0] + 2) +
																(inCkidPB->fUserNamePStr[0] + 2) +
																(inCkidPB->fRevisionPStr[0] + 2) +
																(inCkidPB->fFileNamePStr[0] + 2) +
																(inCkidPB->fTaskPStr[0] + 2) +
																sizeof(UInt16) +
																inCkidPB->fCommentLen + 1;
																
						newCkidRsrc = ::NewHandle( newCkidRsrcSize );
						
						if ( newCkidRsrc == NULL )
							
							result = ::MemError();
						
						else
							{
								
								UInt32	curIndex 	= 0x00000000,
												checksum	= 0x00000000;

/*
**  Write the current version regardless of the
**  PB passed in.
*/
												
								inCkidPB->fCurrentVersion = kCurrentCkidVersion;
								inCkidPB->fLocation = kDefaultCkidLocValue;
								
								::BlockMoveData( inCkidPB, (char *)*newCkidRsrc, sizeof(CkidResourceHeader) );
								
								curIndex += sizeof(CkidResourceHeader);
								
								::BlockMoveData( &inCkidPB->fRemotePathPStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fRemotePathPStr[0] + 1 );
								
								curIndex += inCkidPB->fRemotePathPStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fUserNamePStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fUserNamePStr[0] + 1 );
								
								curIndex += inCkidPB->fUserNamePStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fRevisionPStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fRevisionPStr[0] + 1 );
								
								curIndex += inCkidPB->fRevisionPStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fFileNamePStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fFileNamePStr[0] + 1 );
								
								curIndex += inCkidPB->fFileNamePStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fTaskPStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fTaskPStr[0] + 1 );
								
								curIndex += inCkidPB->fTaskPStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fCommentLen, ((char *)*newCkidRsrc) + curIndex, sizeof(UInt16) );
							
								curIndex += sizeof(UInt16);
								
								if ( inCkidPB->fCommentHndl != NULL )
									{
									
										::BlockMoveData( *inCkidPB->fCommentHndl, ((char *)*newCkidRsrc) + curIndex, inCkidPB->fCommentLen );
										
										curIndex += inCkidPB->fCommentLen;
								
									}
									
								((char *)*newCkidRsrc)[curIndex] = 0x00;

/*
**  Calculate checksum.
*/						

								checksum = GenerateCkidChecksum( newCkidRsrc );
								
								((CkidResourceHeader *)*newCkidRsrc)->fChecksum = checksum;
								
								::AddResource( newCkidRsrc, 'ckid', kCkidResourceID, "\pProjector Data" );
								result = ::ResError();
								
								if ( result == noErr )
									{
									
										::UpdateResFile( inFileRef );
										result = ::ResError();
									
									}
									
							}	
			
					}

				Assert_( oldResFileRef > 0 );
				::UseResFile( oldResFileRef );
				
			}
			
		return result;
		
	}
	
	
pascal OSErr FSpWriteCkidResource( const FSSpec *inTargetSpec, CkidResourcePB *inCkidPB ) 
	{

		OSErr 		result 				= noErr;
		SInt16		intFileRef 		= 0;
		SInt16		intOldResRef 	= 0;
		Handle		hOldTopMap		= NULL;
		
		if ( (inTargetSpec != NULL) && (inCkidPB != NULL) )
			{			
				Assert_( inTargetSpec->name[0] > 0 );
				Assert_( inTargetSpec->vRefNum != 0 );
				Assert_( inTargetSpec->parID != 0 );
				
/*
**  Using the technique outlined in TN 1120, we
**  check the Resource Manager's map handle to
**  so if it changed. If it doesn't, then the
**  access path passed to us is not a new unique
**  one so we should bail.
*/
	
				intOldResRef = ::CurResFile();
				hOldTopMap = ::LMGetTopMapHndl();
				
				::SetResLoad( false );		
				intFileRef = ::FSpOpenResFile( inTargetSpec, fsRdWrPerm );		
				::SetResLoad( true );

				Assert_( intFileRef != 0 );
							
				if ( intFileRef <= 0 ) 
					{					
						result = ::ResError();

						if ( (result == eofErr) && (FSpIsAFile(inTargetSpec, NULL) == noErr) )
							{
								result = FSpCreateTheDamnRsrcFork( inTargetSpec );
													
								if ( result == noErr )
									{
										::SetResLoad( false );
										intFileRef = ::FSpOpenResFile( inTargetSpec, fsRdWrPerm );
										::SetResLoad( true );
	
										Assert_( intFileRef != 0 );
								
										if ( intFileRef <= 0 )
											{
												result = ::ResError();
											}
											
										/*  No need to check the map handle
										    here since we created the rsrc fork. */
									}									
							}								
					}
				else
					{
						if ( ::LMGetTopMapHndl() == hOldTopMap )
							{
								intFileRef = -1;
								result = opWrErr;
							}
					}
							
				if ( (intFileRef > 0) && (result == noErr) )
					{
						result = CUWriteCkidResourceFromRef( inCkidPB, intFileRef );
					}
					
				if ( intFileRef > 0 )
					{
						::CloseResFile( intFileRef );
						intFileRef = 0;
					}
					
				::UseResFile( intOldResRef );																
			}
			
		return result;

	}


static OSErr CURemoveCkidResourceFromRef( const SInt16 inFileRef )
	{
		
		OSErr		result 				= noErr;
		SInt16	intOldResRef	= 0;
		Handle	oldCkidRsrc		= NULL;

		intOldResRef = ::CurResFile();
		::UseResFile( inFileRef );
		
		oldCkidRsrc = ::Get1Resource( 'ckid', kMcvsResourceID );

		if ( (oldCkidRsrc != NULL) && (::ResError() == noErr) )
			{			
				::RemoveResource( oldCkidRsrc );
				result = ::ResError();
				
				if ( result == noErr )
					{
						::DisposeHandle( oldCkidRsrc );
						oldCkidRsrc = NULL;
						
						::UpdateResFile( inFileRef );
						result = ::ResError();					
					}				
			}

		::UseResFile( intOldResRef );
		
		return result;
		
	}
	

static OSErr CURemoveMcvsResourceFromRef( const SInt16 inFileRef )
	{
	
		OSErr		result 				= noErr;
		SInt16	intOldResRef	= 0;
		Handle	oldMcvsRsrc		= NULL;
		
		intOldResRef = ::CurResFile();
		::UseResFile( inFileRef );
		
		oldMcvsRsrc = ::Get1Resource( 'mcvs', kMcvsResourceID );

		if ( (oldMcvsRsrc != NULL) && (::ResError() == noErr) )
			{			
				::RemoveResource( oldMcvsRsrc );
				result = ::ResError();
				
				if ( result == noErr )
					{
						::DisposeHandle( oldMcvsRsrc );
						oldMcvsRsrc = NULL;
						
						::UpdateResFile( inFileRef );
						result = ::ResError();						
					}				
			}

		::UseResFile( intOldResRef );
		
		return result;
		
	}

	
pascal OSErr CURemoveCkidMcvsResourceFromRef( const Boolean inRemoveCkid, const Boolean inRemoveMcvs, const SInt16 inFileRef )
	{
	
		OSErr	result = noErr;
		
		if ( inRemoveCkid )
			{
				result = CURemoveCkidResourceFromRef( inFileRef );
			}
			
		if ( result == noErr )
			{
				if ( inRemoveMcvs )
					{
						result = CURemoveMcvsResourceFromRef( inFileRef );
					}
			}
						
		return result;
	
	}

pascal OSErr FSpRemoveCkidMcvsResource( const FSSpec *inTargetSpec, const Boolean inRemoveCkid, const Boolean inRemoveMcvs )
	{
	
		OSErr		result 				= noErr;
		SInt16	intFileRef 		= 0;
		SInt16	intOldResRef	= 0;
		Handle	hOldTopMap		= NULL;
		
		if ( inTargetSpec != NULL ) 
			{									
				Assert_( inTargetSpec->name[0] > 0 );
				Assert_( inTargetSpec->vRefNum != 0 );
				Assert_( inTargetSpec->parID != 0 );
					
				intOldResRef = ::CurResFile();
				hOldTopMap = ::LMGetTopMapHndl(); /* see TN 1120 */
				
				::SetResLoad( false );
				intFileRef = ::FSpOpenResFile( inTargetSpec, fsRdWrPerm );		
				::SetResLoad( true );

				Assert_( intFileRef != 0 );
							
				if ( intFileRef <= 0 ) 
					{						
						result = ::ResError();									
					}						
				else 
					{	
						if ( ::LMGetTopMapHndl() == hOldTopMap )
							{
								intFileRef = -1;
								result = opWrErr;
							}					
					}
					
				if ( (intFileRef > 0) && (result == noErr) )
					{		
						result = CURemoveCkidMcvsResourceFromRef( inRemoveCkid, inRemoveMcvs, intFileRef );
							
						::CloseResFile( intFileRef );		
						intFileRef = 0;				
					}
				
				::UseResFile( intOldResRef );
			}			
		else 
			{
				result = kCVSEngineBadParamErr;
			}
			
		return result;
		
	}
																			

//################################################################################
//################################################################################
#pragma mark -
#pragma mark RESPONSE LINE ROUTINES

pascal TEngineUpdateResponseType
IdentifyResponseType( const char *inResponseLine, const UInt32 inResponseLength )
	{
		TEngineUpdateResponseType		respType = kCVSEngineFileOtherResponse;
	
		if ( inResponseLength < 2 || inResponseLine[1] != ' ' )
			return respType;
	
		switch ( inResponseLine[0] )
			{
				case kCVSEngineFileUpdatedChar:
					respType = kCVSEngineFileUpdatedResponse;
					break;
					
				case kCVSEngineFileAddedChar:
					respType = kCVSEngineFileAddedResponse;
					break;
					
				case kCVSEngineFileTaggedChar:
					respType = kCVSEngineFileTaggedResponse;
					break;
					
				case kCVSEngineFileUntaggedChar:
					respType = kCVSEngineFileUntaggedResponse;
					break;
					
				case kCVSEngineFileNotTaggedChar:
					respType = kCVSEngineFileNotTaggedResponse;
					break;
					
				case kCVSEngineFileMergedChar:
					respType = kCVSEngineFileMergedResponse;
					break;
				
				case kCVSEngineFileConflictsChar:
					respType = kCVSEngineFileConflictsResponse;
					break;
				
				case kCVSEngineFileLocallyAddedChar:
					respType = kCVSEngineFileLocallyAddedResponse;
					break;
					
				case kCVSEngineFileRemovedChar:
					respType = kCVSEngineFileRemovedResponse;
					break;
					
				case kCVSEngineFileUnknownChar:
					respType = kCVSEngineFileUnknownFileResponse;
					break;
				
				default:
					Assert_( false );
					break;
			}
	
		return respType;
	}


// copies the path into outPathStr which must already be allocated
pascal void
GetRelativePathFromResponse( const char *inResponseLine, const UInt32 inResponseLength, UInt32 *inOutPathLength, char *outPathStr)
	{
		TEngineUpdateResponseType		respType = IdentifyResponseType( inResponseLine, inResponseLength );
		UInt32											len;
		
		Assert_( *inOutPathLength > 1);
		*outPathStr = '\0';
		
		if (kCVSEngineFileOtherResponse == respType)
			{
				*inOutPathLength = 0;
				return;
			}
			
		const char		*lineEnd = inResponseLine + inResponseLength;
		const char		*pathStart = inResponseLine + 2;			//skip the indicator and space
		const char		*c = pathStart;
		
		while (*c && c < lineEnd)
			{
				if (*c == '\r') break;
				c ++;
			}
			
		len = c - pathStart;
		if (len > *inOutPathLength)
			len = *inOutPathLength;
			
		strncpy( outPathStr, pathStart, len);
		outPathStr[len] = '\0';
		
		*inOutPathLength = len;
	}
	
	

pascal OSErr
CUStripCVSPath( char *inPathStr, UInt32 *inPathLen )
	{

		OSErr		result 	= noErr;
		UInt32 	index 	= 0;
		
		if ( (inPathStr != NULL) && (inPathLen != NULL) && ((*inPathLen) > 0) )
			{
/*
**  Strip beginning '/' chars.
*/			
				while ( (inPathStr[0] == '/') && ((*inPathLen) > 0) )
					{					
						(*inPathLen)--;
						::BlockMoveData( &inPathStr[1], &inPathStr[0], (*inPathLen) );						
					}
/*
**  If we have anything left, clean it up by removing duplicate '/' chars.
*/				
				index = 0;
									
				while ( index < (*inPathLen) )
					{
						if ( (inPathStr[index] == '/') && (index > 0) && (inPathStr[index - 1] == '/') )
							{							
								(*inPathLen)--;
								
								::BlockMoveData( &inPathStr[index], &inPathStr[index - 1], (*inPathLen) - index );								
							}
/*
**  Remove './' items
*/							
						if ( (inPathStr[index] == '.') && ((index + 1) < (*inPathLen)) && (inPathStr[index + 1] == '/') )
							{								
								if ( (index == 0) || ((index > 0) && (inPathStr[index - 1] == '/')) )
									{								
										if ( (index + 2) < (*inPathLen) )
											::BlockMoveData( &inPathStr[index + 2], &inPathStr[index], (*inPathLen) - (index + 2) );
											
										(*inPathLen) -= 2;										
									}
								else index++;								 
							}								
						else index++;									
					}								
			}
			
		return result;		
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark FILE ROUTINES



pascal OSErr
FSpSafeCreate( FSSpec *inOutTargetSpec, const OSType inFileCreator, const OSType inFileType, const ScriptCode inScriptType )
	{

		OSErr						result						= noErr;
										
		Str255					pstrSuffix				= "\p~0";
		Str255					pstrFileName			= "\p";
		FSSpec					fsParent					= { 0, 0, "\p" };
		FSSpec					fsCreatedItem			= { 0, 0, "\p" };
		unsigned char 	cWrapChar					= pstrSuffix[pstrSuffix[0]];

		CInfoPBRec			pb;
		
		if ( inOutTargetSpec == NULL )
			{
				result = kCVSEngineBadParamErr;
			}
		else
			{	
				fsCreatedItem = *inOutTargetSpec;
					
				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= -1;
				pb.hFileInfo.ioNamePtr 			= fsParent.name;
				pb.hFileInfo.ioVRefNum 			= fsCreatedItem.vRefNum;
				pb.hFileInfo.ioDirID 				= fsCreatedItem.parID;
				
				result = ::PBGetCatInfoSync( &pb );

				if ( result == noErr )
					{			
						fsParent.vRefNum = pb.hFileInfo.ioVRefNum;
						fsParent.parID = pb.hFileInfo.ioFlParID;
							
						if ( fsCreatedItem.name[0] >= 29 )
							fsCreatedItem.name[0] = 28;
				
// append tail

						PStrConcat( fsCreatedItem.name, pstrSuffix, sizeof(fsCreatedItem.name) );
						
						do
							{							
								result = ::FSpCreate( &fsCreatedItem, inFileCreator, inFileType, inScriptType );
								
								if ( result != noErr )
									{
										fsCreatedItem.name[fsCreatedItem.name[0]]++;	// next copy
										
										if ( fsCreatedItem.name[fsCreatedItem.name[0]] == cWrapChar )
											result = kCVSEngineCantDuplicateFileErr;
									}									
							} while ( result == dupFNErr );		
							
						if ( result == noErr )
							{
								*inOutTargetSpec = fsCreatedItem;
							}
					}
							
			}

		return result;
							
	}


/*
	SafeDuplicate
	
	Tries to duplicate a file, appending a "~#" on it, trying till
	all 255 chars wrap.
*/


pascal OSErr
FSpSafeDuplicate( const FSSpec *inTargetSpec )
	{

		OSErr						result						= noErr;

		FSSpec					parentFolderSpec	= { 0, 0, "\p" };
										
		Str255					appendStr 				= "\p~0";
		Str255					destName					= "\p";

		unsigned char 	wrapChar 					= appendStr[appendStr[0]];
		
		CInfoPBRec			pb;
		
		if ( inTargetSpec == NULL )
		
			result = kCVSEngineBadParamErr;
			
		else
			{
			
				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= -1;
				pb.hFileInfo.ioNamePtr 			= parentFolderSpec.name;
				pb.hFileInfo.ioVRefNum 			= inTargetSpec->vRefNum;
				pb.hFileInfo.ioDirID 				= inTargetSpec->parID;
				
				result = ::PBGetCatInfoSync( &pb );

				if ( result == noErr )
					{
			
						parentFolderSpec.vRefNum = pb.hFileInfo.ioVRefNum;
						parentFolderSpec.parID = pb.hFileInfo.ioFlParID;
							
						PStrCopy( inTargetSpec->name, destName, sizeof(destName) );
						 
						if ( destName[0] >= 29 )
							destName[0] = 28;
				
// append tail

						PStrConcat( destName, appendStr, sizeof(destName) );

						do
							{	
								
								result = ::FSpFileCopy( inTargetSpec, &parentFolderSpec, destName, NULL, NULL, false );
								
								destName[destName[0]]++;	// next copy
								
								if ( destName[destName[0]] == wrapChar )
									result = kCVSEngineCantDuplicateFileErr;
									
							} while ( result == dupFNErr );
		
					}
							
			}

		return result;
							
	}


pascal OSErr
FSpSafeRename( FSSpec *inOutTargetItem )
	{

		OSErr						result					= noErr;
										
		Str255					destFileName		= "\p",
										indexStr				= "\p0";

		UInt32					index						= 0;
		
		if ( inOutTargetItem == NULL )
		
			result = kCVSEngineBadParamErr;
			
		else
			{
			
				destFileName[0] = 0x00;
				PStrConcat( destFileName, inOutTargetItem->name, sizeof(destFileName) );
				
				if ( destFileName[0] >= 29 )
					destFileName[0] = 28;
					 
				PStrConcat( destFileName, "\p~", sizeof(destFileName) );
				PStrConcat( destFileName, indexStr, sizeof(destFileName) );
						
				result = ::FSpRename( inOutTargetItem, destFileName );
						
				while ( (result == dupFNErr) && (index < 99) )
					{
												
						index++;
						
						destFileName[0] = 0x00;
						PStrConcat( destFileName, inOutTargetItem->name, sizeof(destFileName) );
	
						if ( destFileName[0] >= 29 )
							destFileName[0] = 28;

						PStrConcat( destFileName, "\p~", sizeof(destFileName) );
						
						UInt32ToStr255( index, indexStr );
						PStrConcat( destFileName, indexStr, sizeof(destFileName) );

						result = ::FSpRename( inOutTargetItem, destFileName );
							
					}

/*
**  FSpRename doesn't update the spec.
*/
					
				if ( result == noErr )
					{
					
						PStrCopy( destFileName, inOutTargetItem->name, sizeof(inOutTargetItem->name) );
					
					}

							
			}

		return result;
							
	}


pascal OSErr FSpCheckForMovement(	const FSSpec *inTargetSpec, const FSSpec *inProjectRoot, CkidMcvsResourcePB *inCkidMcvsInfo )
	{
	
		OSErr			result 									= noErr;
		UInt32		actualPathLen 					= 511,
							indicatedPathLen				= 511;	
		
		char			actualPathStr[512]			= "",
							indicatedPathStr[512] 	= "";
				
		if ( (inTargetSpec != NULL) && (inProjectRoot != NULL) && (inCkidMcvsInfo != NULL) )
			{

				result = FSpGetLocalCVSPath( inTargetSpec, inProjectRoot, actualPathStr, &actualPathLen );
				
				if ( result == noErr )
					{
					
						actualPathStr[actualPathLen] = 0x00;

/*
**  If the path is longer than 511 bytes, we're SOL.
*/
						
						if ( (inCkidMcvsInfo->mcvs.fRemotePathLen + inCkidMcvsInfo->mcvs.fFileNamePStr[0] + 1) >= 511 )
							
							result = kCVSEnginePathOverflow;
							
					}
				
				if ( result == noErr )
					{

/*
**  Manually concat string...
*/
					
						indicatedPathLen = inCkidMcvsInfo->mcvs.fRemotePathLen;
						
						if ( indicatedPathLen > 0 )
							{
							
								::BlockMoveData( inCkidMcvsInfo->mcvs.fRemotePathStr, indicatedPathStr, indicatedPathLen );
						
								indicatedPathStr[indicatedPathLen] = '/';
								indicatedPathLen++;
						
							}
							
						::BlockMoveData( &inCkidMcvsInfo->mcvs.fFileNamePStr[1], &indicatedPathStr[indicatedPathLen], inCkidMcvsInfo->mcvs.fFileNamePStr[0] );
						
						indicatedPathLen += inCkidMcvsInfo->mcvs.fFileNamePStr[0];
						
						indicatedPathStr[indicatedPathLen] = 0x00;
						
						if ( indicatedPathLen != actualPathLen )
							result = kCVSEngineFileHasBeenMoved;
							
						else if ( strncmp(actualPathStr, indicatedPathStr, indicatedPathLen) != 0 )
							result = kCVSEngineFileHasBeenMoved;
							
					}
										
			}
		else result = kCVSEngineBadParamErr;
			
		return result;
					
	}
	
	
pascal OSErr FSpGetPartialPath( const FSSpec 	*inTargetSpec, const FSSpec	*inBaseItem, UInt32 *inOutPathLength, char *outPathStr )

/*
**  FSpGetPartialPath returns the partial path between two FSSpecs. It DOES NOT
**  include the name of the base FSSpec in the path string so you will always
**  get partial pathnames. (ie., even if a volume is the base FSSpec)
*/

	{

		OSErr						result					= noErr;
		FSSpec					tempSpec				= { 0, 0, "\p" };
		Handle					tempPathHndl		= NULL;
		UInt16					copyLen					= 0x0000;
		Boolean					foundRoot				= false;
		CInfoPBRec			pb;
		
		#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		
		Str255					pstrTranslated 	= "\p";		
		
		#endif
		
		if ( !FSpIsEqual(inTargetSpec, inBaseItem) ) 
			{
		
				::BlockMoveData( inTargetSpec, &tempSpec, sizeof(FSSpec) );
				
				if ( tempSpec.parID != fsRtParID ) 
					{

						#ifdef CVSENGINE_USE_MIME_FILE_NAMES
						
						CUTranslateFromMacNamePStr( tempSpec.name, pstrTranslated );
						result = ::PtrToHand( &pstrTranslated[1], &tempPathHndl, pstrTranslated[0] );

						#else
						
						result = ::PtrToHand( &tempSpec.name[1], &tempPathHndl, tempSpec.name[0] );

						#endif
					
						if ( result == noErr ) 
							{
					
/* 
**  Get the ancestor directory names 
*/

								pb.dirInfo.ioCompletion = NULL;
								
								pb.dirInfo.ioNamePtr 			= tempSpec.name;
								pb.dirInfo.ioVRefNum 			= tempSpec.vRefNum;
								pb.dirInfo.ioFDirIndex 		= 0;
								pb.dirInfo.ioDrDirID			= 0;
								pb.dirInfo.ioDrParID 			= tempSpec.parID;

/* 
**  loop until we have an error or find the volume root directory
**  or find our base directory
*/
				
								while ( (result == noErr) && (pb.dirInfo.ioDrDirID != fsRtDirID) && (foundRoot == false) ) 
									{
									
										pb.dirInfo.ioFDirIndex 	= -1;
										pb.dirInfo.ioDrDirID 		= pb.dirInfo.ioDrParID;
										
										result = ::PBGetCatInfoSync(&pb);
					
										if ( result == noErr ) 
											{

												tempSpec.vRefNum = pb.dirInfo.ioVRefNum;
												tempSpec.parID = pb.dirInfo.ioDrParID;

												foundRoot = FSpIsEqual( &tempSpec, inBaseItem );

												if ( foundRoot != true ) 
													{

/* 
**  Append colon to directory name 
*/
														
														++tempSpec.name[0];
														tempSpec.name[tempSpec.name[0]] = ':';
														
														/* Add directory name to beginning of fullPath */
														
														#ifdef CVSENGINE_USE_MIME_FILE_NAMES
														
														CUTranslateFromMacNamePStr( tempSpec.name, pstrTranslated );
														(void) ::Munger(tempPathHndl, 0, NULL, 0, &pstrTranslated[1], pstrTranslated[0]);

														#else
														
														(void) ::Munger(tempPathHndl, 0, NULL, 0, &tempSpec.name[1], tempSpec.name[0]);

														#endif
														
														result = ::MemError();
													}
												
/*
**  If we found the base FSSpec, insert a ':' at the
**  beginning to signify the partial path.
*/

												else
													{
													
														tempSpec.name[1] = ':';
														tempSpec.name[0] = 1;

														#ifdef CVSENGINE_USE_MIME_FILE_NAMES															
														
														CUTranslateFromMacNamePStr( tempSpec.name, pstrTranslated );
														(void) ::Munger(tempPathHndl, 0, NULL, 0, &pstrTranslated[1], pstrTranslated[0]);
														
														#else
														
														(void) ::Munger(tempPathHndl, 0, NULL, 0, &tempSpec.name[1], tempSpec.name[0]);			
														
														#endif
														
														result = ::MemError();
													}
						
											}
					
									} 
				
								if ( (result == noErr) && (foundRoot == true) ) 
									{
				
/* 
**  Return the length 
*/

										copyLen = GetHandleSize(tempPathHndl);
					
										if ( copyLen > *inOutPathLength )
											copyLen = *inOutPathLength;

										::BlockMove( *tempPathHndl, outPathStr, copyLen );
										
										*inOutPathLength = copyLen;
					
									}
				
								else 
									{
				
										*inOutPathLength = 0;
										
										if ( !foundRoot )
											result = dirNFErr;
					
									}
				
								::DisposeHandle( tempPathHndl );

							}
		
					}
		
			}
			
		else
			{
			
				result = 1;
				
			}

		return 	result;
	
	}


pascal OSErr FSpGetLocalCVSPath( const FSSpec *inTargetSpec, const FSSpec *inProjectRoot, char *outPathStr, UInt32 *inOutPathLen )
	{
	
		OSErr		result = noErr;
		UInt32	bufLen = 0;
		
		if ( 	(inTargetSpec != NULL) && 
					(inProjectRoot != NULL) && 
					(outPathStr != NULL) && 
					(inOutPathLen != NULL) && 
					((*inOutPathLen) > 0) )
			{
				
				bufLen = (*inOutPathLen);
				(*inOutPathLen) = 0;
				
				result = FSpGetPartialPath( inTargetSpec, inProjectRoot, &bufLen, (char *)outPathStr );

/*
**  If the FSSpecs are the same, return a zero length result.
*/
				
				if ( result == 1 )
					{
					
						result = noErr;
						
						*outPathStr = 0;
						*inOutPathLen = 0;
								
					}
					
				else if ( result == noErr )
					{
					
						(*inOutPathLen) = bufLen;
						
						if ( (*inOutPathLen) > 0 )
							{

/*
**  Convert ':'s to '/'s and remove the leading '/' if there is one.
*/
							
								for ( UInt32 index = 0; index < (*inOutPathLen); index++ )
									if ( outPathStr[index] == ':' )
										outPathStr[index] = '/';
										
								if ( outPathStr[0] == '/' )
									{
									
										(*inOutPathLen)--;
										::BlockMoveData( &outPathStr[1], &outPathStr[0], (*inOutPathLen) );
									
									}
							
							}
					
					}
			
			}
		
		return result;
	
	}
	

pascal OSErr FSpGetFromLocalCVSPath( const Str255 inCVSPathPStr, const FSSpec *inProjectRoot, FSSpec *outTargetItem )
	{
	
		OSErr					result 				= noErr;
		Str255				thePath				= "\p";
		CInfoPBRec		pb;
		SInt32				rootDirID			= 0x00000000;
		UInt32				offset				= 0x00000000;
		Boolean				lookingForDir	= false;
		
		if ( (inCVSPathPStr != NULL) && (inProjectRoot != NULL) && (outTargetItem != NULL) )
			{

				if ( inCVSPathPStr[0] > 0 )
				
					::BlockMoveData( inCVSPathPStr, thePath, inCVSPathPStr[0] + 1 );

				if ( thePath[0] > 0 )
					{

/*
**  Check for multiple '/' sequences and colons...
*/
				
						for ( UInt32 index = 1; ((index <= thePath[0]) && (result == noErr)); index++ )
							{
							
								if ( thePath[index] == '/' )
									{
									
										if ( index > 1 )
										if ( thePath[index - 1] == '/' )
											result = kCVSEngineInvalidCVSPath;
							
									}

/*
**  Die if the path contains colons.
*/
									
								else if ( thePath[index] == ':' )
									result = kCVSEngineInvalidCVSPath;
									
							}

/*
**  Lop off beginning and ending '/'s.
*/
					
						if ( result == noErr )
							{
							
								if ( thePath[thePath[0]] == '/' )
									{
									
										thePath[0]--;
										
										lookingForDir = true;
											
									}
																		
								for ( UInt32 index = 1; index <= thePath[0]; index++ )
									if ( thePath[index] == '/' )
										thePath[index] = ':';

/*
**  Insert ':' for partial pathname.
*/
	
								if ( (thePath[0] > 0) && (thePath[1] != ':') )
									{
										
										if ( thePath[0] < 255 )
											thePath[0]++;
											
										::BlockMoveData( &thePath[1], &thePath[2], thePath[0] - 1 );
										
										thePath[1] = ':';
									
									}
									
							}
						
					}
						
/*
**  If the dillhole user passed in an empty path string,
**  return the root.
*/
					
				if ( result == noErr )
					{
					
						if ( thePath[0] == 0 )
							{
							
								result = kCVSEngineInvalidCVSPath;
								
							}
						else
							{
									
								pb.hFileInfo.ioCompletion = NULL;
								pb.hFileInfo.ioFDirIndex 	= 0;
								pb.hFileInfo.ioNamePtr 		= (UInt8 *)inProjectRoot->name;
								pb.hFileInfo.ioDirID 			= inProjectRoot->parID;
								pb.hFileInfo.ioVRefNum		= inProjectRoot->vRefNum;
								
								result = ::PBGetCatInfoSync( &pb );
								
								if ( result == noErr )
									{
									
#ifdef CVSENGINE_USE_MIME_FILE_NAMES
										UInt32 thePathLen = 0;
										
										thePathLen = thePath[0];
										CUTranslateToMacName( (char*)&thePath[1], thePathLen );
										thePath[0] = thePathLen;
#endif
									
										pb.hFileInfo.ioNamePtr = thePath;
										result = ::PBGetCatInfoSync( &pb );
										
									}

/*
**  Uh oh, if the user specified a path ending with a '/',
**  then they were looking for a directory.
*/
							
								if ( result == noErr )
									if ( (pb.dirInfo.ioFlAttrib & ioDirMask) && (!lookingForDir) )
										result = kCVSEngineIsFolder;
										
								if ( (result == noErr) || (result == kCVSEngineIsFolder) )
									{
									
										outTargetItem->vRefNum = pb.hFileInfo.ioVRefNum;
										outTargetItem->parID = pb.hFileInfo.ioFlParID;
									
										Boolean		done 			= false;
										UInt32		index 		= thePath[0],
															copyLen 	= 0x00000000;
	
/*
**  Strip away the partial pathname, leaving only
**  the filename in the resulting FSSpec.
*/
											
										while ( !done )
											{
												
												if ( index > 0 )
													{
												
														if ( thePath[index] == ':' )
														
															done = true;
														
														else index--;
														
													}
												else done = true;
													
											}
										
										copyLen = thePath[0] - index;
										
										if ( copyLen > 0 )
											::BlockMoveData( &thePath[index + 1], &outTargetItem->name[1], copyLen );
										
										outTargetItem->name[0] = copyLen; 
									
									}

							}
													
					}
			
			}
			
		return result;
	
	}
	

pascal OSErr FSpGetDirID(	const FSSpec *inTargetSpec, SInt32 *outDirID )
	{
	
		OSErr				result = noErr;
		CInfoPBRec	pb;
		
		if ( (inTargetSpec != NULL) && (outDirID != NULL) )
			{

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetSpec->name;
				pb.hFileInfo.ioDirID			= inTargetSpec->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetSpec->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					(*outDirID) = pb.hFileInfo.ioDirID;
						
			}
			
		return result;
	
	}
	
	

/* Private Finder data that we mess with when setting folders to list view */
enum
	{
		kFinderViewAsIconMask			= (1 << 8),
		kFinderViewByNameMask			= (1 << 9),
		kFinderViewByDateMask			= (1 << 8) | (1 << 9),		// I'm not sure what the logic is here
		kFinderViewBySizeMask			= (1 << 10),
		kFinderViewByKindMask			= (1 << 8) | (1 << 10)
	};


pascal OSErr FSpSetFolderListView(	const FSSpec *inTargetSpec, SInt16 inFolderView )
	{
	
		OSErr				result = noErr;
		CInfoPBRec	pb;
		
		if ( inTargetSpec != NULL && kCVSEngineIsFolder == FSpIsAFile(inTargetSpec, NULL) )
			{

				pb.dirInfo.ioCompletion = NULL;
				pb.dirInfo.ioNamePtr 		= (StringPtr)inTargetSpec->name;
				pb.dirInfo.ioDrDirID		= inTargetSpec->parID;
				pb.dirInfo.ioVRefNum 		= inTargetSpec->vRefNum;
				pb.dirInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
				
				if (result == noErr)
					{
						pb.dirInfo.ioDrDirID		= inTargetSpec->parID;
						pb.dirInfo.ioVRefNum 		= inTargetSpec->vRefNum;
						pb.dirInfo.ioFDirIndex 	= 0;
						
						SInt16		flags = 0;
							
						switch ( inFolderView )
							{
								case kViewFolderByIcon:
									flags = kFinderViewAsIconMask;
									break;
									
								case kViewFolderByName:
									flags = kFinderViewByNameMask;
									break;
									
							case kViewFolderBySize:
									flags = kFinderViewBySizeMask;
									break;
								
							case kViewFolderByDate:
									flags = kFinderViewByDateMask;
									break;
									
							case kViewFolderByKind:
									flags = kFinderViewByKindMask;
									break;
								
							case kViewFolderLookDontTouch:
									return noErr;
							}
						
						SetRect( &pb.dirInfo.ioDrUsrWds.frRect, 0, 0, -1, -1 );			// force the finder to make sensible window sizes
						pb.dirInfo.ioDrUsrWds.frView = flags;
						pb.dirInfo.ioDrUsrWds.frFlags = 0x0300;											// This seems to be required to tell the Finder
																																				// not to ignore our flags
						
						result = ::PBSetCatInfoSync( &pb );
					}
			}
			
		return result;
	
	}
	

pascal OSErr FSpIsAFile( const FSSpec *inTargetSpec, Boolean *outFileIsOpen )
	{
	
		OSErr				result = noErr;
		CInfoPBRec	pb;
		
		if ( inTargetSpec != NULL )
			{

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetSpec->name;
				pb.hFileInfo.ioDirID			= inTargetSpec->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetSpec->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					{
						if ( pb.dirInfo.ioFlAttrib & ioDirMask )
						
							result = kCVSEngineIsFolder;
							
						else if ( outFileIsOpen != NULL )
						
							*outFileIsOpen = FILE_OPEN(pb);
							
					}
						
			}
			
		return result;
	
	}
	

pascal OSErr FSpIsAlias( const FSSpec *inTargetSpec )
	{
	
		OSErr				result = noErr;
		CInfoPBRec	pb = {0};
		
		if ( inTargetSpec != NULL )
			{

				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetSpec->name;
				pb.hFileInfo.ioDirID			= inTargetSpec->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetSpec->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					if ( (pb.hFileInfo.ioFlFndrInfo.fdFlags & kIsAlias) )
						result = kCVSEngineIsAlias;
						
			}
			
		return result;
	
	}


pascal Boolean FSpIsLocked( const FSSpec *inTargetSpec )
	{
	
		OSErr				err 			= noErr;
		Boolean			bResult 	= false;
		CInfoPBRec	pb 				= {0};
		
		if ( inTargetSpec != NULL )
			{
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetSpec->name;
				pb.hFileInfo.ioDirID			= inTargetSpec->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetSpec->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				err = ::PBGetCatInfoSync( &pb );
			
				if ( err == noErr )
					{
						if ( pb.hFileInfo.ioFlAttrib & 1 )
							{
								bResult = true;
							}
					}	
			}
			
		return bResult;
	
	}

	
pascal SInt32 FSpGetModDate( const FSSpec *inTargetSpec )
	{
	
		SInt32			result = noErr;
		CInfoPBRec	pb;
		
		if ( inTargetSpec != NULL )
			{

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetSpec->name;
				pb.hFileInfo.ioDirID			= inTargetSpec->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetSpec->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					result = pb.hFileInfo.ioFlMdDat;
						
			}
			
		return result;
	
	}
	

pascal OSErr FSpSetModDate( const FSSpec *inTargetSpec, const SInt32 inModDate )
	{

		OSErr				result 			= noErr;
		CInfoPBRec 	pb;
		
		if ( inTargetSpec == NULL )
			
			result = kCVSEngineBadParamErr;
			
		else
			{
			
				pb.hFileInfo.ioCompletion = nil;
				pb.hFileInfo.ioNamePtr 		= ((FSSpec *)inTargetSpec)->name;
				pb.hFileInfo.ioVRefNum 		= inTargetSpec->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				pb.hFileInfo.ioDirID 			= inTargetSpec->parID;

				result = ::PBGetCatInfoSync( &pb );

				pb.hFileInfo.ioDirID = inTargetSpec->parID;

				if ( pb.hFileInfo.ioFlCrDat > inModDate )
					pb.hFileInfo.ioFlCrDat = inModDate;
					
				pb.hFileInfo.ioFlMdDat = inModDate;

				result = ::PBSetCatInfoSync( &pb );
				
			}

		return result;
		
	} 


pascal Boolean FSpIsEqual( const FSSpec *inItemOne, const FSSpec *inItemTwo )
	{

		Boolean result = false;
		
		if ( (inItemOne != NULL) && (inItemTwo != NULL) ) 
			{
			
				if ( inItemOne->vRefNum == inItemTwo->vRefNum )
				if ( inItemOne->parID == inItemTwo->parID )
					result =  ::EqualString( inItemOne->name, inItemTwo->name, false, true );
				
			}
		
		return result;

	}


pascal Boolean FSpIsChildOf( const FSSpec *inChildItem, const FSSpec *inParentItem )
	{

		OSErr				error			= noErr;
		Boolean			bResult 	= false,
								bFoundIt	= false;
	
		FSSpec			curItem		= { 0, 0, "\p" };
		CInfoPBRec	pb;
		
		if ( (inChildItem != NULL) && (inParentItem != NULL) )
			{
			
				if ( !FSpIsEqual(inChildItem,inParentItem) )
					{
					
						pb.hFileInfo.ioCompletion 	= NULL;
						pb.hFileInfo.ioFDirIndex 		= -1;
						pb.hFileInfo.ioNamePtr 			= curItem.name;
						pb.hFileInfo.ioVRefNum 			= inChildItem->vRefNum;
						pb.hFileInfo.ioDirID 				= inChildItem->parID;
					
						error = ::PBGetCatInfoSync( &pb );
						
						while ( (error == noErr) && (!bFoundIt) && (pb.hFileInfo.ioFlParID != fsRtParID) )
							{
							
								curItem.vRefNum = pb.hFileInfo.ioVRefNum;
								curItem.parID = pb.hFileInfo.ioFlParID;
								
								bFoundIt = FSpIsEqual( &curItem, inParentItem );
							
								if ( !bFoundIt )
									{
									
										pb.hFileInfo.ioDirID = pb.hFileInfo.ioFlParID;
										
										error = ::PBGetCatInfoSync( &pb );
									
									}
										
							}
							
						if ( (error == noErr) && (!bFoundIt) )
							{
							
								curItem.vRefNum = pb.hFileInfo.ioVRefNum;
								curItem.parID = pb.hFileInfo.ioFlParID;
						
								bFoundIt = FSpIsEqual( &curItem, inParentItem );
								
							}

						if ( (error == noErr) && (bFoundIt) )
							{
							
								bResult = true;
							
							}
					
					}
			
			}
			
		return bResult;
		
	}
	
	
pascal OSErr FSpGetParentFSSpec( const FSSpec *inTargetSpec, FSSpec *outParentItem )
	{
	
		OSErr 			result = noErr;
		CInfoPBRec	pb;
		
		if ( (inTargetSpec == NULL) || (outParentItem == NULL) )
		
			result = kCVSEngineBadParamErr;

/*
**  The root doesn't have a parent.
*/
			
		else if ( inTargetSpec->parID == fsRtParID )
		
			result = kCVSEngineBadParamErr;
			
		else
			{
			
				outParentItem->name[0] = 0x00;
				
				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioFDirIndex = -1;
				pb.hFileInfo.ioNamePtr = outParentItem->name;
				pb.hFileInfo.ioVRefNum = inTargetSpec->vRefNum;
				pb.hFileInfo.ioDirID = inTargetSpec->parID;
			
				outParentItem->vRefNum = inTargetSpec->vRefNum;
					
				result = ::PBGetCatInfoSync( &pb );
				
				if ( result == noErr )
					outParentItem->parID = pb.hFileInfo.ioFlParID;
			
			}
			
		return result;
		
	}
	
	
pascal OSErr FSpGetFullPathStr( const FSSpec *inTargetSpec, UInt32 *inOutPathLen, char *outPathStr )
{

	OSErr		result			= noErr;
	FSSpec		tempSpec;
	Handle		tempPathHndl	= NULL;
	CInfoPBRec	pb;
	UInt16		copyLen			= 0x0000;
		
	/* Make a copy of the input FSSpec that can be modified */
	BlockMoveData(inTargetSpec, &tempSpec, sizeof(FSSpec));
	
	if ( tempSpec.parID == fsRtParID ) {
	
		/* The object is a volume */
		
		/* Add a colon to make it a full pathname */
		++tempSpec.name[0];
		tempSpec.name[tempSpec.name[0]] = ':';
		
		copyLen = tempSpec.name[0];
		
		if (copyLen > *inOutPathLen)
			copyLen = *inOutPathLen;
			
		/* We're done */
		BlockMove(&tempSpec.name[1], outPathStr, copyLen);
		
		*inOutPathLen = copyLen;
		
	}
	else {
	
		/* The object isn't a volume */
		
		/* Put the object name in first */
		
		result = PtrToHand(&tempSpec.name[1], &tempPathHndl, tempSpec.name[0]);
		
		if (result == noErr) {
		
			/* Get the ancestor directory names */
			pb.dirInfo.ioNamePtr = tempSpec.name;
			pb.dirInfo.ioVRefNum = tempSpec.vRefNum;
			pb.dirInfo.ioDrParID = tempSpec.parID;
			
			do	/* loop until we have an error or find the root directory */
			{
				pb.dirInfo.ioFDirIndex = -1;
				pb.dirInfo.ioDrDirID = pb.dirInfo.ioDrParID;
				
				result = PBGetCatInfoSync(&pb);
				
				if (result == noErr) {
				
					/* Append colon to directory name */
					++tempSpec.name[0];
					tempSpec.name[tempSpec.name[0]] = ':';
					
					/* Add directory name to beginning of fullPath */
					(void) Munger(tempPathHndl, 0, NULL, 0, &tempSpec.name[1], tempSpec.name[0]);
					result = MemError();
					
				}
				
			} while ( (result == noErr) && (pb.dirInfo.ioDrDirID != fsRtDirID) );
			
			if (result == noErr) {
			
				/* Return the length */
				copyLen = GetHandleSize(tempPathHndl);
				
				if (copyLen > *inOutPathLen)
					copyLen = *inOutPathLen;

				BlockMove(*tempPathHndl, outPathStr, copyLen);
				
				*inOutPathLen = copyLen;
				
			}
			
			else {
			
				*inOutPathLen = 0;
				
			}
			
			DisposeHandle(tempPathHndl);

		}
		
	}
	
	return 	result;
	
}


pascal OSErr FSpCreateTheDamnRsrcFork( const FSSpec *pInTarget )
	{
	
		OSErr		result			= noErr;
		FSSpec	fsTarget 		= { 0, 0, "\p" };
		FInfo		sFinderInfo;
		
		if ( pInTarget == NULL )
			{
				result = kCVSEngineBadParamErr;
			}
		
		if ( result == noErr )
			{
				fsTarget = *pInTarget;
				result = ::FSpGetFInfo( &fsTarget, &sFinderInfo );
			}
							
		if ( result == noErr )
			{
/*
**  Is it safe to use FSpCreateResFile() 
**  since it follows the PMSP?
*/
				::FSpCreateResFile( &fsTarget, sFinderInfo.fdCreator, sFinderInfo.fdType, smSystemScript );
				result = ::ResError();
			}
	
		return result;
			
	}
	

pascal OSErr FSpCountNumFiles( const FSSpec *inFolderSpec, UInt32 *outNumItems )
	{
		
		OSErr				result 		= noErr;
		CInfoPBRec	pb;
		
		if ( inFolderSpec == NULL || outNumItems == NULL )
			{
				result = kCVSEngineBadParamErr;
			}
			
		if ( result != noErr )
			{		
				*outNumItems = 0;
				
				FSSpec testSpec = *inFolderSpec;

				pb.dirInfo.ioCompletion = NULL;
				pb.dirInfo.ioNamePtr 		= (StringPtr)testSpec.name;
				pb.dirInfo.ioDrDirID		= testSpec.parID;
				pb.dirInfo.ioVRefNum 		= testSpec.vRefNum;
				pb.dirInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					{
						if ( (pb.dirInfo.ioFlAttrib & ioDirMask) == 0 )
							{
								result = kCVSEngineIsFile;
							}
						else
							{
								*outNumItems = pb.dirInfo.ioDrNmFls;
							}
					}
			}
						
		return result;
	
	}


pascal OSErr FSpGetNthItem( const FSSpec *inFolderSpec, UInt32 itemIndex, FSSpec *outItemSpec )
	{
		
		OSErr				result 		= noErr;
		FSSpec			fsOutItem	= { 0 };
		CInfoPBRec	pb;

		//  The count for FSpGetNthItem is 1 based.
			
		if ( inFolderSpec == NULL || outItemSpec == NULL || itemIndex == 0 )
			{
				result = kCVSEngineBadParamErr;
			}
				
		if ( result == noErr )
			{
				FSSpec fsTest = *inFolderSpec;

				//  first, get the dirID of this folder
				pb.dirInfo.ioCompletion = NULL;
				pb.dirInfo.ioNamePtr 		= (StringPtr)fsTest.name;
				pb.dirInfo.ioDrDirID		= fsTest.parID;
				pb.dirInfo.ioVRefNum 		= fsTest.vRefNum;
				pb.dirInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
				
				if ( result == noErr )
					{		
						if ( (pb.dirInfo.ioFlAttrib & ioDirMask) == 0 )
							{
								//  Uh oh, it's a file!
								result = kCVSEngineIsFile;
							}
					}
						
				if ( result == noErr )
					{								
						fsOutItem.vRefNum = fsTest.vRefNum;
						fsOutItem.parID = pb.dirInfo.ioDrDirID;
						
						//  now, get the indexed item in this folder
						pb.dirInfo.ioCompletion = NULL;
						pb.dirInfo.ioNamePtr 		= (StringPtr)fsOutItem.name;
						pb.dirInfo.ioFDirIndex 	= itemIndex;
						
						result = ::PBGetCatInfoSync( &pb );
						
						if ( result == noErr )
							{
								*outItemSpec = fsOutItem;
							}
					}
			}
			
		return result;
	
	}


pascal 
OSErr	CUIsFileModified( const CkidResourcePB *pInPB, ConstStringPtr pstrInTimeStamp, const SInt32 intInModDate )
	{
	
		OSErr		result 							= noErr;
		UInt32	intLen 							= 0;
		Str255	pstrActualTimeStamp = "\p";
		
		//  First check the ckid information for the MRO bit.
		
		if ( pInPB != NULL )
			{
				if ( pInPB->fModifyReadOnly > 0 )
					{
						result = kCVSEngineFileLocallyModifiedErr;
					}
			}
		
		//  If the client didn't pass a ckid, or it's not MRO,
		//  then we need to compare the mod. date to the time stamp.
			
		if ( result == noErr )
			{
				if ( pstrInTimeStamp == NULL )
					{
						result = kCVSEngineBadParamErr;
					}
					
				if ( result == noErr )
					{
						//  You must compare time by converting the Mac time to the
						//  ISO time string and comparing the strings. You cannot do
						//  the opposite (converting the strings to seconds) because
						//  CVS may put random conflict information in the string to
						//  ensure that it mismatches.
						
										
						intLen = 255;						
						result = CUISOTimeStrFromSeconds( (char *)&pstrActualTimeStamp[1], &intLen, intInModDate );
					}
					
				if ( result == noErr )
					{
						pstrActualTimeStamp[0] = intLen;
													
						if ( !PStrCompare( pstrInTimeStamp, pstrActualTimeStamp) )
							{
								result = kCVSEngineFileLocallyModifiedErr;
							}
					}
			}
			
		return result;
			
	}
	
	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark REPOSITORY ROUTINES


pascal Boolean CUIsRepositoryDir( const SInt16 inVRefNum, const SInt32 inParID )
	{

		OSErr				err							= noErr;
		Boolean			result 					= false;
		Str255			cvsFolderStr		= "\pCVS";
		CInfoPBRec	pb;
		
		
		pb.hFileInfo.ioCompletion 	= NULL;
		pb.hFileInfo.ioVRefNum 			= inVRefNum;
		pb.hFileInfo.ioNamePtr 			= cvsFolderStr;
		pb.hFileInfo.ioDirID 				= inParID;
		pb.hFileInfo.ioFDirIndex 		= 0;
		
		err = ::PBGetCatInfoSync( &pb );
		
		if ( err == noErr )
			result = true;
		
		return result;
					
	}
	

pascal Boolean FSpIsRepositoryDir( const FSSpec *inTargetSpec )
	{
	
		OSErr				error 	= noErr;
		Boolean			bResult	= false;
		CInfoPBRec	pb;
		
		if ( inTargetSpec != NULL )
			{
			
				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= 0;
				pb.hFileInfo.ioNamePtr 			= (const_cast<FSSpec *>(inTargetSpec))->name;
				pb.hFileInfo.ioVRefNum 			= inTargetSpec->vRefNum;
				pb.hFileInfo.ioDirID 				= inTargetSpec->parID;
				
				error = ::PBGetCatInfoSync( &pb );
				
				if ( error == noErr )
					{
					
						bResult = CUIsRepositoryDir( pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID );
					
					}
		
			}
				
		return bResult;
	
	}


pascal OSErr FSpGetRootFileSpec( const FSSpec *inFolderSpec, FSSpec *outTagFileSpec )
	{
	
		OSErr				error 		= noErr;
		FSSpec			tempSpec 	= { 0, 0, "\p:" };
		
		tempSpec.vRefNum 	= inFolderSpec->vRefNum;
		tempSpec.parID 		= inFolderSpec->parID;
		
		PStrConcat( tempSpec.name, inFolderSpec->name, sizeof(tempSpec.name) );
		PStrConcat( tempSpec.name, "\p:", sizeof(tempSpec.name) );
		PStrConcat( tempSpec.name, kCVSDirectoryName, sizeof(tempSpec.name) );
		PStrConcat( tempSpec.name, "\p:", sizeof(tempSpec.name) );
		PStrConcat( tempSpec.name, kCVSRootName, sizeof(tempSpec.name) );

		error = FSMakeFSSpec( tempSpec.vRefNum, tempSpec.parID, tempSpec.name, outTagFileSpec);
		
		return error;
	
	}
		
pascal OSErr CUCreateRepositoryDir( const SInt16 inVRefNum, const SInt32 inParID, const OSType inCreator, const OSType inType, const ScriptCode inScriptType )
	{
	
		OSErr		result			= noErr;
		FSSpec	tempSpec 		= { 0, 0, "\p" };
		SInt32	dirID				= 0;

		tempSpec.vRefNum 	= inVRefNum;
		tempSpec.parID 		= inParID;
		PStrConcat( tempSpec.name, kCVSDirectoryName, sizeof(tempSpec.name) );

		result = ::FSpDirCreate( &tempSpec, smSystemScript, &dirID );

		#ifdef CVSENGINE_INVISIBLE_CVS_DIRECTORIES

/*
**  Make the directory invisible. I couldn't get 
**  FSpSetFInfo() to work, so I had to use PBSetCatInfoSync.
*/

		if ( result == noErr )
			{				
				CInfoPBRec	cvsDirPB;
				
				cvsDirPB.hFileInfo.ioCompletion = NULL;
				cvsDirPB.hFileInfo.ioFDirIndex = 0;
				cvsDirPB.hFileInfo.ioNamePtr = tempSpec.name;
				cvsDirPB.hFileInfo.ioVRefNum = tempSpec.vRefNum;
				cvsDirPB.hFileInfo.ioDirID = tempSpec.parID;
				
				result = ::PBGetCatInfoSync( &cvsDirPB );
				
				if ( result == noErr )
					{						
						cvsDirPB.hFileInfo.ioFlFndrInfo.fdFlags |= fInvisible;
						
						cvsDirPB.hFileInfo.ioDirID = tempSpec.parID;
						
						result = ::PBSetCatInfoSync( &cvsDirPB );						
					}
			}

		#endif
		
/*			
		if ( result == noErr )
			{
			
				tempSpec.parID = dirID;
				tempSpec.name[0] = 0x00;
				PStrConcat( tempSpec.name, kCVSTagName, sizeof(tempSpec.name) );
				
				result = ::FSpCreate( &tempSpec, inCreator, inType, inScriptType );
				
			}
*/			
		if ( result == noErr )
			{	
				tempSpec.parID = dirID;
				tempSpec.name[0] = 0;
				PStrConcat( tempSpec.name, kCVSRepositoryName, sizeof(tempSpec.name) );
					
				result = ::FSpCreate( &tempSpec, inCreator, inType, inScriptType );				
			}

		if ( result == noErr )
			{	
				tempSpec.parID = dirID;
				tempSpec.name[0] = 0;
				PStrConcat( tempSpec.name, kCVSRootName, sizeof(tempSpec.name) );
					
				result = ::FSpCreate( &tempSpec, inCreator, inType, inScriptType );				
			}

		if ( result == noErr )
			{	
				tempSpec.parID = dirID;
				tempSpec.name[0] = 0;
				PStrConcat( tempSpec.name, kCVSEntriesName, sizeof(tempSpec.name) );
					
				result = ::FSpCreate( &tempSpec, inCreator, inType, inScriptType );			
			}
	
		return result;
		
	}
	
	
pascal OSErr FSpGetRepository( const FSSpec *inTargetSpec, Str255 outRepositoryPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0x0000;
		SInt32			ioCount		= 255;
		Str255			pathPStr	= "\p";

		FSSpec			fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetSpec != NULL) && (outRepositoryPStr != NULL) )
			{
			
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, inTargetSpec->name, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSDirectoryName, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSRepositoryName, sizeof(pathPStr) );
				
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )
							{
								fileRef = 0;
							}
						else
							{					
								result = ::GetEOF( fileRef, &ioCount );
						
								if ( result == noErr )
									if ( ioCount > 255 )
										ioCount = 255;
								
								if ( result == noErr )
									{
									
										result = ::FSRead( fileRef, &ioCount, &outRepositoryPStr[1] );
									
										if ( result == noErr )
											if ( ioCount > 0 )
												{
												
/*
**  Strip off the CR/LF if there is one.
*/
													
													while ( (ioCount > 0) && ((outRepositoryPStr[ioCount] == kCRChar) || (outRepositoryPStr[ioCount] == kLFChar)) )
														ioCount--;
														
													outRepositoryPStr[0] = ioCount;

												}
											else outRepositoryPStr[0] = 0;
											
									}
							
								::FSClose( fileRef );
								fileRef = 0;
						
							}
												
					}
					
				if ( result != noErr )
					outRepositoryPStr[0] = 0;
			
			}
		else result = kCVSEngineBadParamErr;
		
		return result;
		
	}
	

pascal OSErr FSpSetRepository( const FSSpec *inTargetSpec, Str255 inRepositoryPStr )
	{
	
		OSErr				result 					= noErr;
		SInt16			fileRef					= 0;
		SInt32			ioCount					= 0;
		
		Str255			pathPStr				= "\p";
		Str255			pstrRepository 	= "\p";
		FSSpec			fileSpec				= { 0, 0, "\p" };
		
		if ( (inTargetSpec != NULL) && (inRepositoryPStr != NULL) )
			{
				PStrCopy( inRepositoryPStr, pstrRepository, sizeof(pstrRepository) );
			
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, inTargetSpec->name, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSDirectoryName, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSRepositoryName, sizeof(pathPStr) );
				
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )		
							{					
								fileRef = 0;							
							}
						else
							{	
								if ( pstrRepository[0] > 0 )
									{							
										if ( pstrRepository[pstrRepository[0]] != kTerminatorByte )
											{									
												if ( pstrRepository[0] == 255 )
													{
														pstrRepository[pstrRepository[0]] = kTerminatorByte;
													}
												else
													{
														pstrRepository[++pstrRepository[0]] = kTerminatorByte;											
													}
											}
									}
								else
									{
										pstrRepository[++pstrRepository[0]] = kTerminatorByte;	
									}
										
								ioCount = pstrRepository[0];
								
								result = ::SetEOF( fileRef, ioCount );
								
								if ( result == noErr )
									{
										result = ::FSWrite( fileRef, &ioCount, &pstrRepository[1] );
									}
									
								::FSClose( fileRef );
								fileRef = 0;														
							}					
					}			
			}
			
		return result;		
	}
	
	
pascal OSErr FSpGetRoot( const FSSpec *inTargetSpec, Str255 outRootPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0;
		SInt32			ioCount		= 255;
		Str255			pathPStr	= "\p";

		FSSpec			fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetSpec != NULL) && (outRootPStr != NULL) )
			{			
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, inTargetSpec->name, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSDirectoryName, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSRootName, sizeof(pathPStr) );
				
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )							
							fileRef = 0;							
						else
							{						
								result = ::GetEOF( fileRef, &ioCount );
						
								if ( result == noErr )
									if ( ioCount > 255 )
										ioCount = 255;
								
								if ( result == noErr )
									{									
										result = ::FSRead( fileRef, &ioCount, &outRootPStr[1] );
									
										if ( result == noErr )
											if ( ioCount > 0 )
												{												
/*
**  Strip off the CR/LF if there is one.
*/													
													while ( (ioCount > 0) && ((outRootPStr[ioCount] == kCRChar) || (outRootPStr[ioCount] == kLFChar)) )
														ioCount--;
														
													outRootPStr[0] = ioCount;
												}
											else outRootPStr[0] = 0;											
									}
							
								::FSClose( fileRef );
								fileRef = 0;					
							}												
					}
					
				if ( result != noErr )
					outRootPStr[0] = 0;			
			}
		else result = kCVSEngineBadParamErr;
		
		return result;		
	}
	

pascal OSErr FSpSetRoot( const FSSpec *inTargetSpec, Str255 inRootPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0;
		SInt32			ioCount		= 0;
		Str255			pathPStr	= "\p";
		Str255			pstrRoot	= "\p";
		FSSpec			fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetSpec != NULL) && (inRootPStr != NULL) )
			{

				PStrCopy( inRootPStr, pstrRoot, sizeof(pstrRoot) );
			
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, inTargetSpec->name, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSDirectoryName, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSRootName, sizeof(pathPStr) );
				
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )		
							{					
								fileRef = 0;							
							}
						else
							{			
								if ( pstrRoot[0] > 0 )
									{					
										if ( pstrRoot[pstrRoot[0]] != kTerminatorByte )
											{								
												if ( pstrRoot[0] == 255 )
													{
														pstrRoot[pstrRoot[0]] = kTerminatorByte;
													}
												else
													{
														pstrRoot[++pstrRoot[0]] = kTerminatorByte;											
													}
											}
									}
								else
									{
										pstrRoot[++pstrRoot[0]] = kTerminatorByte;
									}
									
								ioCount = pstrRoot[0];
								
								result = ::SetEOF( fileRef, ioCount );
								
								if ( result == noErr )
									{
										result = ::FSWrite( fileRef, &ioCount, &pstrRoot[1] );
									}
									
								::FSClose( fileRef );
								fileRef = 0;														
							}					
					}			
			}
			
		return result;		
	}
	

pascal OSErr FSpGetFolderTag( const FSSpec *inTargetSpec, Str255 outTagPStr )
	{
	
		OSErr				result 				= noErr;
		SInt16			fileRef				= 0x0000;
		SInt32			ioCount				= 255;
		FSSpec			targetTag 		= { 0, 0, "\p" };
		
		if ( (inTargetSpec != NULL) && (outTagPStr != NULL) )
			{
			
				targetTag.vRefNum = inTargetSpec->vRefNum;
				targetTag.parID = inTargetSpec->parID;
				
				PStrConcat( targetTag.name, "\p:", sizeof(targetTag.name) );
				PStrConcat( targetTag.name, inTargetSpec->name, sizeof(targetTag.name) );
				PStrConcat( targetTag.name, "\p:", sizeof(targetTag.name) );
				PStrConcat( targetTag.name, kCVSDirectoryName, sizeof(targetTag.name) );
				PStrConcat( targetTag.name, "\p:", sizeof(targetTag.name) );
				PStrConcat( targetTag.name, kCVSTagName, sizeof(targetTag.name) );
						
				result = ::FSpOpenDF( &targetTag, fsRdWrPerm, &fileRef );
						
				if ( result != noErr )
				
					fileRef = 0;
					
				else
					{
					
						result = ::GetEOF( fileRef, &ioCount );

						if ( result == noErr )
							if ( ioCount > 255 )
								ioCount = 255;
						
						if ( (result == noErr) && (ioCount > 0) )
							result = ::FSRead( fileRef, &ioCount, &outTagPStr[1] );
					
						if ( result == noErr )
							{
		
								while ( (ioCount > 0) && ((outTagPStr[ioCount] == kCRChar) || (outTagPStr[ioCount] == kLFChar)) )
									ioCount--;
										
								outTagPStr[0] = ioCount;
							
							}
							
						::FSClose( fileRef );
								
						fileRef = 0;
			
					}
			
			}
		else result = kCVSEngineBadParamErr;
			
		return result;
	
	}
	
	
pascal OSErr FSpSetFolderTag( const FSSpec *inTargetSpec, Str255 inTagPStr, 
															const OSType inFileCreator, const OSType inFileType, const ScriptCode inScriptType )
	{
	
		OSErr			result 		= noErr;
		SInt16		fileRef		= 0;
		SInt32		ioCount		= 0;
		Str255		pathPStr	= "\p";
		Str255		pstrTag		= "\p";
		FSSpec		fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetSpec != NULL) && (inTagPStr != NULL) )
			{			
				PStrCopy( inTagPStr, pstrTag, sizeof(pstrTag) );

				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, inTargetSpec->name, sizeof(pathPStr) );				
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSDirectoryName, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSTagName, sizeof(pathPStr) );
				
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pathPStr, &fileSpec );
				
				if ( result == fnfErr )
					{
						result = ::FSpCreate( &fileSpec, inFileCreator, inFileType, inScriptType );			
					}
					
				if ( result == noErr )
					{					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )	
							{					
								fileRef = 0;							
							}
						else
							{			
								if ( pstrTag[0] > 0 )
									{					
										if ( pstrTag[pstrTag[0]] != kTerminatorByte )
											{								
												if ( pstrTag[0] == 255 )
													{
														pstrTag[pstrTag[0]] = kTerminatorByte;
													}
												else
													{
														pstrTag[++pstrTag[0]] = kTerminatorByte;											
													}
											}
									}
								else
									{
										pstrTag[++pstrTag[0]] = kTerminatorByte;
									}
									
								result = ::SetEOF( fileRef, pstrTag[0] );
								
								if ( result == noErr )
									{									
										ioCount = pstrTag[0];
								
										if ( ioCount > 0 )
											{
												result = ::FSWrite( fileRef, &ioCount, &pstrTag[1] );								
											}
									}
					
								::FSClose( fileRef );
								fileRef = 0;												
							}										
					}			
			}
		else result = kCVSEngineBadParamErr;
		
		return result;
		
	}


pascal OSErr FSpGetParentTag( const FSSpec *inTargetSpec, Str255 outTagPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0x0000;
		SInt32			ioCount		= 255;
		Str255			pathPStr	= "\p";
		FSSpec			fileSpec	= { 0, 0, "\p" };

		if ( (inTargetSpec != NULL) && (outTagPStr != NULL) )
			{
					
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSDirectoryName, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSTagName, sizeof(pathPStr) );
			
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pathPStr, &fileSpec );
									
				if ( result == noErr )
					{
					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )
						
							fileRef = 0;
							
						else
							{
							
								result = ::GetEOF( fileRef, &ioCount );

								if ( result == noErr )
									if ( ioCount > 255 )
										ioCount = 255;
								
								if ( (result == noErr) && (ioCount > 0) )
									result = ::FSRead( fileRef, &ioCount, &outTagPStr[1] );
							
								if ( result == noErr )
									outTagPStr[0] = ioCount;
									
								::FSClose( fileRef );
								fileRef = 0;
					
							}
										
					}
					
			}
		else result = kCVSEngineBadParamErr;
			
		return result;
		
	}
	

pascal OSErr FSpSetParentTag( const FSSpec *inTargetSpec, Str255 inTagPStr )
	{
	
		OSErr			result 		= noErr;
		SInt16		fileRef		= 0;
		SInt32		ioCount		= 0;
		Str255		pathPStr	= "\p";
		Str255		pstrTag		= "\p";
		FSSpec		fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetSpec != NULL) && (inTagPStr != NULL) )
			{			
				PStrCopy( inTagPStr, pstrTag, sizeof(pstrTag) );
				
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSDirectoryName, sizeof(pathPStr) );
				PStrConcat( pathPStr, "\p:", sizeof(pathPStr) );
				PStrConcat( pathPStr, kCVSTagName, sizeof(pathPStr) );
				
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )		
							{				
								fileRef = 0;							
							}
						else
							{	
								if ( pstrTag[0] > 0 )
									{							
										if ( pstrTag[pstrTag[0]] != kTerminatorByte )
											{								
												if ( pstrTag[0] == 255 )
													{
														pstrTag[pstrTag[0]] = kTerminatorByte;
													}
												else
													{
														pstrTag[++pstrTag[0]] = kTerminatorByte;											
													}
											}
									}
								else
									{
										pstrTag[++pstrTag[0]] = kTerminatorByte;		
									}
									
								result = ::SetEOF( fileRef, pstrTag[0] );
								
								if ( result == noErr )
									{									
										ioCount = pstrTag[0];
								
										if ( ioCount > 0 )
											{
												result = ::FSWrite( fileRef, &ioCount, &pstrTag[1] );								
											}
									}
					
								::FSClose( fileRef );
								fileRef = 0;												
							}										
					}			
			}
		else result = kCVSEngineBadParamErr;
		
		return result;
		
	}

	
pascal Boolean FSpIsStatic( const FSSpec *inTargetSpec )
	{
	
		Boolean		bResult 	= false;
		OSErr			intErr		= noErr;
		FSSpec		fsItem		= { 0 };
		Str255		pstrPath	= "\p";
		
		if ( inTargetSpec != NULL )
			{		
				PStrConcat( pstrPath, "\p:", sizeof(pstrPath) );
				PStrConcat( pstrPath, inTargetSpec->name, sizeof(pstrPath) );
				PStrConcat( pstrPath, "\p:", sizeof(pstrPath) );
				PStrConcat( pstrPath, kCVSDirectoryName, sizeof(pstrPath) );
				PStrConcat( pstrPath, "\p:", sizeof(pstrPath) );
				PStrConcat( pstrPath, kCVSEntriesStaticName, sizeof(pstrPath) );
				
				intErr = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pstrPath, &fsItem );
				
				if ( intErr == noErr )
					{					
						bResult = true;												
					}
			}
		
		return bResult;
		
	}
	

pascal OSErr FSpSetStatic( const FSSpec *inTargetSpec, const Boolean bInSticky, 
														const OSType inFileCreator, const OSType inFileType, const ScriptCode inScriptType )
	{
	
		OSErr			result		= noErr;
		FSSpec		fsItem		= { 0 };
		Str255		pstrPath	= "\p";
		
		if ( inTargetSpec != NULL )
			{		
				PStrConcat( pstrPath, "\p:", sizeof(pstrPath) );
				PStrConcat( pstrPath, inTargetSpec->name, sizeof(pstrPath) );
				PStrConcat( pstrPath, "\p:", sizeof(pstrPath) );
				PStrConcat( pstrPath, kCVSDirectoryName, sizeof(pstrPath) );
				PStrConcat( pstrPath, "\p:", sizeof(pstrPath) );
				PStrConcat( pstrPath, kCVSEntriesStaticName, sizeof(pstrPath) );
				
				result = ::FSMakeFSSpec( inTargetSpec->vRefNum, inTargetSpec->parID, pstrPath, &fsItem );
				
				if ( result == noErr && bInSticky == false )
					{					
						result = ::FSpDelete( &fsItem );					
					}
				else if ( result != noErr && bInSticky )
					{
						result = ::FSpCreate( &fsItem, inFileCreator, inFileType, inScriptType );			
					}
					
				if ( result == fnfErr )
					{
						result = noErr;
					}
			}
		
		return result;
	
	}
	

#pragma mark -


inline Boolean ishex(char c)
	{
		if (c >= '0' && c <= '9')
			return true;
			
		if (c >= 'a' && c <= 'f')
			return true;
			
		if (c >= 'A' && c <= 'F')
			return true;

		return false;
	}


static int htoi(unsigned char *s)
	{
		int value;
		char c;

		c = s[0];
		if (c >= 'A' && c <= 'F')
			c += 'a' - 'A';
		value = (c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10) * 16;

		c = s[1];
		if (c >= 'A' && c <= 'F')
			c += 'a' - 'A';
		value += c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10;

		return value;
	}

