//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				StFileAccess.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		StFileAccess is a stack based class meant to help with dealing with open
		files in exception safe functions. Use StFileAccess to open files, when
		the object is destroyed of, all file descriptors are closed.
		 
	Modification History:

		06 Feb 1999		Ram Rajadhyaksha
		
					Now in separate source file.
										
-------------------------------------------------------------------------------- */


#include "StFileAccess.h"


StFileAccess::StFileAccess( const FSSpec &inTargetSpec )
	{	
		mFileSpec 					= inTargetSpec;
		mFSDataForkRef 			= 0;
		mFSResForkRef 			= 0;
		mResFileRef 				= 0;
		mDeleteOnDestroy 		= false;		
	}
	

StFileAccess::~StFileAccess()
	{		
		CloseAllFileRefs();
		
		if ( mDeleteOnDestroy )
			{			
				::FSpDelete( &mFileSpec );			
			}										
	}
	
	
#pragma mark -


SInt16
StFileAccess::GetFSDataForkRef( const SInt8 inPerm )
	{	
	
		OSErr	error = noErr;
		
		if ( mFSDataForkRef == 0 )
			{			
				error = ::FSpOpenDF( &mFileSpec, inPerm, &mFSDataForkRef );
			
				if ( error != noErr )
					{					
						mFSDataForkRef = 0;						
						throw error;					
					}			
			}
	
		return mFSDataForkRef;		

	}
	

SInt16
StFileAccess::GetFSResForkRef( const SInt8 inPerm )
	{
	
		OSErr	error = noErr;
		
		if ( mFSResForkRef == 0 )
			{		
				error = ::FSpOpenRF( &mFileSpec, inPerm, &mFSResForkRef );
			
				if ( error != noErr )
					{				
						mFSResForkRef = 0;					
						throw error;					
					}			
			}
	
		return mFSResForkRef;
		
	}


SInt16
StFileAccess::GetResFileRef( const SInt8 inPerm )
	{
	
		OSErr		error 			= noErr;
		Handle	hOldTopMap	= NULL;
		
		if ( mResFileRef == 0 )
			{			
				hOldTopMap = ::LMGetTopMapHndl();  //  see TN 1120
					
				::SetResLoad( false );
				mResFileRef = ::FSpOpenResFile( &mFileSpec, inPerm );
				::SetResLoad( true );
				
				if ( mResFileRef <= 0 )
					{					
						mResFileRef = 0;						
						error = ::ResError();					
						throw error;					
					}	
				else
					{
						if ( ::LMGetTopMapHndl() == hOldTopMap )
							{
								mResFileRef = 0;
								error = opWrErr;
								throw error;
							}
					}		
			}
	
		return mResFileRef;
		
	}


void
StFileAccess::SetDelete( const Boolean inDeleteFlag )
	{
	
		mDeleteOnDestroy = inDeleteFlag;
	
	}
	

void
StFileAccess::CloseAllFileRefs()
	{
	
		if ( mFSDataForkRef > 0 )
			{			
				::FSClose( mFSDataForkRef );
				mFSDataForkRef = 0;				
			}
			
		if ( mFSResForkRef > 0 )
			{			
				::FSClose( mFSResForkRef );
				mFSResForkRef = 0;				
			}
			
		if ( mResFileRef > 0 )
			{			
				::CloseResFile( mResFileRef );
				mResFileRef = 0;				
			}
	
	}
	
