/*  FILE:   StringUtils.c

  Copyright  1996 - 1997 by Ram Rajadhyaksha. All rights reserved.
  Some parts copyright  others.

********************************************************************************
*
*
*
*                  G E N E R A L   I N F O R M A T I O N
*
*
*
********************************************************************************
*

  PRIMARY CONTACT:  Ram V. Rajadhyaksha (RVR)
                    http://www.best.com/~ramr
                    ramr@best.com

  REVISIONS:        01.23.97  RVR   Added PStrxx support.
                    01.12.97  RVR   Fixed bug in SkipToEOL()

                    01.01.97  RVR   Added crappy UInt32ToCStr routine.

                    12.23.96  RVR   Combined lots of string utils. in
                                    this file. Some functions still need
                                    to be implemented.

This software is freeware and provided "as-is". You are welcome to use this code
for instructional purposes. If this code or a derivative of it is used in your
software, you must give me explicit credit. Commercial use or distribution is
prohibited without prior written agreement. Questions or comments are greatly
welcome.

********************************************************************************
*
*/


#include "StringUtils.h"



// ---------------------------------------------------------------------------
//		 CopyPStr
// ---------------------------------------------------------------------------
//	Copy a Pascal string
//
//		Returns a pointer to the copied string
//
//		inDestSize specifies the maximum size of the destination (INCLUDING
//			the length byte). The default is sizeof Str255.
//
//		By calling this function as:
//
//			CopyPStr(source, dest, sizeof(dest));
//
//		you can change the declaration of dest without having to
//		remember to change the call.

StringPtr
PStrCopy(
	ConstStr255Param	inSourceStr,
	StringPtr			outDestStr,
	SInt16				inDestSize)
{
	SInt16	dataLen = inSourceStr[0] + 1;
	if (dataLen > inDestSize) {
		dataLen = inDestSize;
	}
	
	BlockMoveData(inSourceStr, outDestStr, dataLen);
	outDestStr[0] = dataLen - 1;
	return outDestStr;
}


// ---------------------------------------------------------------------------
//		 ConcatPStr
// ---------------------------------------------------------------------------
//	Concatenate two Pascal strings. The first string becomes the combination
//	of the first and second strings.
//
//		Returns a pointer to the concatenated string
//
//		inDestSize specifies the maximum size of the concatenated string
//			(INCLUDING the length byte). The default is sizeof Str255.
//
//		By calling this function as:
//
//			ConcatPStr(io, appendMe, sizeof(io));
//
//		you can change the declaration of io without having to
//		remember to change the call.

StringPtr
PStrConcat(
	Str255				ioFirstStr,
	ConstStr255Param	inSecondStr,
	SInt16				inDestSize)
{
								// Limit combined string to inDestSize chars
    SInt16	charsToCopy = inSecondStr[0];
    if (ioFirstStr[0] + charsToCopy > inDestSize - 1) {
    	charsToCopy = inDestSize - 1 - ioFirstStr[0];
    }

								// Copy second to end of first string
    ::BlockMoveData(inSecondStr + 1,  ioFirstStr + ioFirstStr[0] + 1,
    					charsToCopy);
    							// Set length of combined string
    ioFirstStr[0] += charsToCopy;
    
    return ioFirstStr;
}



Boolean 	PStrCompare( const unsigned char *inSrc, const unsigned char *inDst )
{
	
	register Boolean	result 	= false;
	register UInt8		index 	= 0x01;
	
		if (*inSrc == *inDst) {
		
			result = true;
			
			while ((result != false) && (index <= inSrc[0])) {
				
				if (inSrc[index] != inDst[index])
					result = false;
					
				index++;
				
			}
			
		}
			
	return result;
	
}


UInt32		CStrLen(			register char			*inStr)
{
	
	register UInt32		result	= 0x00000000;
	
	while ((*(inStr++)) != 0x00)
		result++;
				
	return result;

}


void		PStrToUInt32(		unsigned char			*inStr,
								UInt32					*outNum)
{

	char	cStr[256];
	
	BlockMove(&inStr[1], cStr, inStr[0]);
	cStr[inStr[0]] = 0x00;

	CStrToUInt32(cStr, outNum);
	
}


void		CStrToUInt32(		register char			*inStr,
								register UInt32			*outNum)
{

	Boolean		done = false;
	
	*outNum = 0x00000000;
	
	while (!done) {

		if (((*inStr) >= 0x30) && ((*inStr) <= 0x39)) {
		
			(*outNum) = (*outNum) * 10;
			
			(*outNum) = (*outNum) + ((*inStr) - 0x30); 
		
			inStr++;
		
		} else done = true;

	}
	
}


void		UInt32ToCStrAtomic( char					*outStr,
								UInt32					inNum,
								UInt32					*inOutIndex,
								const UInt32			inLen);
								
								
void		UInt32ToCStrAtomic( char					*outStr,
								UInt32					inNum,
								UInt32					*inOutIndex,
								const UInt32			inLen)
{
	
	const		asciiOffset	= 0x30;
	UInt16		digit;
	
	if ((inNum / 10) > 0) {
	
		digit = inNum % 10;
		
		inNum = inNum / 10;
		
		UInt32ToCStrAtomic(outStr, inNum, inOutIndex, inLen);
		
	}
	else digit = inNum;
	
	if ((*inOutIndex) < inLen) {
	
		outStr[(*inOutIndex)] = digit + asciiOffset;
	
		(*inOutIndex)++;
		
	}

}


void		UInt32ToStr255(		UInt32					inNum,
								Str255					outStr)
{

	UInt32	len	= 255;
/*
**  This will leave the terminating byte in the string,
**  but since it is impossible to have a number 255 chars.
**  long, it should not matter.
*/
	UInt32ToCStr(inNum, (char *)&outStr[1], &len);
	
	outStr[0] = len;

}


void 		UInt32ToCStr(		UInt32 					inNum, 
								char 					*outStr, 
								UInt32 					*inOutLen)
{
	
	UInt32	inLen	= (*inOutLen);
	
	(*inOutLen) = 0;
	
	if (inLen > 0) {
	
		UInt32ToCStrAtomic(outStr, inNum, inOutLen, inLen - 1);
	
		outStr[(*inOutLen)] = 0x00;
		
	}

}

void PStrReverse( const Str255 inPStr, Str255 outReversePStr )
	{
	
		UInt8	srcLen = 0;
		
		if ( (inPStr != NULL) && (outReversePStr != NULL) )
			{
				
				outReversePStr[0] = 0;
				
				srcLen = inPStr[0];
			
				while ( srcLen > 0 )
					outReversePStr[++outReversePStr[0]] = inPStr[srcLen--];
					
			}
			
	}
	
void PStrLowercase( Str255 inOutPStr )
	{
	
		for ( UInt32 index = 1; index <= inOutPStr[0]; index++ )
			{
		
				if ( (inOutPStr[index] >= 'A') && (inOutPStr[index] <= 'Z') )
					{
					
						inOutPStr[index] = inOutPStr[index] - 'A' + 'a';
		
					}
					
			}
				
	}
	
