//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CCompareWithOriginalView.h
//
//	  Author: 	Dan Clifford
//
//################################################################################


#pragma once


#include <LArray.h>
#include <LWindow.h>
#include <LBroadcaster.h>


enum
	{
	
		kDifferenceLineNormal			= 1,
		kDifferenceLineAdded			= 2,
		kDifferenceLineRemoved		= 3,
		kDifferenceLineChanged		= 4
	
	};


enum
	{
	
		kMpsrResourceType					= 'MPSR',
		kMpsrResourceID						= 1005
	
	};

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

// MPW font resource struct
struct CMpsrRec
	{

		Int16				fFontSize;
		char				fFontName[32];		// font name, as a C string
	
		Int16				fSpaceWidth;			// width of space (in points)
		Int16				fTabWidth;				// spaces per tab
	
		Rect				fZoomedOutWindowBounds;
		Rect				fZoomedInWindowBounds;
		
		UInt32			fLastEditTime;
		
		Int32				fSelStart;
		Int32				fSelEnd;
		
		Int32				fScrollOffset;
		
		Boolean			fAutoIndent;
		Boolean			fShowInvisibles;
		
	};

typedef struct CMpsrRec CMpsrRec;

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif


struct CDiffClauseRec
	{
	
		UInt32	fType;
		UInt32	fOffset;
		UInt32	fLineNumber;
		UInt32	fLineCount;
		Handle	fDataHandle;
	
	}; 


typedef struct CDiffClauseRec CDiffClauseRec;


struct CDiffLineInfoRec
	{
	
		UInt32	fLineType;
		UInt32	fOffset;
	
	}; 


typedef struct CDiffLineInfoRec CDiffLineInfoRec;


class CCompareWithOriginalView: 

	public LView,
	public LBroadcaster

	{

		public:
		
			enum 
				{
					
					class_ID 										= 'cwov',
					msg_GotCompareInformation 	= 10001,
					kTextDisplayLineHeight			= 12,
					kTextDrawBufferSize					= 512,
					kSpacesPerTab 							= 3

				};
		
																				CCompareWithOriginalView(										LStream 				*inStream);
																				CCompareWithOriginalView();
			
			virtual														~CCompareWithOriginalView();
			
			static CCompareWithOriginalView 	*CreateCCompareWithOriginalViewFromStream(	LStream 				*inStream);
			
			virtual void											FinishCreateSelf();
			
			void															SetupColors();
			
			Int16															GetTabWidth( 																const FSSpec 		&inTargetItem );
			
			void 															SetCompareInformation(											const FSSpec		&inTargetItem, 
																																										const LArray	 	&inDiffClauseList);

			virtual void											DrawSelf();	
			
			void															ScrollToLine( 															const UInt32 		inLine);

		private:
		
			void															DrawTabExpanded(														const char			*inText, 
																																										const UInt32 		inTextLen);
			LArray							mLineDescriptions;
			Handle							mCompositeData;
			
			RGBColor						mAddedColor;
			RGBColor						mRemovedColor;
			RGBColor						mChangedColor;
			
			Int16								mSpacesPerTab;

	};
