// ===========================================================================
//	CLocalTreeView.h
//  by Josef W. Wankerl
// ===========================================================================

/*
	Theory of operation:
	
	This class is simply a LTextGroupBox which allows folders to be dropped on it.
	
	To support drag and drop, the LTextGroupBox has been extended with two
	additional member functions which your code must call.
	
	First, you must call SetPreferenceHandler.  This tells CLocalTreeView that
	someone (in this specific case, the Preferences dialog) is ready to
	receive a drop on the LTextGroupBox.  If you do not call SetPreferenceHandler,
	you get the exact same behavior as LTextGroupBox.
	
	Once dropping is enabled by SetPreferenceHandler, CLocalTreeView will
	accept drops of folder items.  When a folder is dropped, the StDialogHandler
	you sent to SetPreferenceHandler will be told about the drop with a
	'GtDr' message.  (It kind of means Get Drop.)
	
	Supposedly your code is now calling DoDialog to receive messages.
	When it sees the 'GtDr' message, call GetDroppedFolder to find out
	which folder was dropped.
	
	It's that simple.
	
	
	Cavets:
	
	This class was written specifically for MacCVS Pro.  It makes some
	of assumptions about the environment in which it operates.  Namely,
	it only accepts drops of folders, and the notification method of the
	drop is only through a StDialogHandler.  Don't reuse this code
	unless you are in the same circumstances (wanting a dropped folder
	on a LTextGroupBox from inside a StDialog).
	
	If you felt ambitious, you could make this class a little more
	generic to overcome these limitations.
*/

#ifndef _H_CLocalTreeView
#define _H_CLocalTreeView
#pragma once

#include <LTextGroupBox.h>
#include <UModalDialogs.h>
#include <LDragAndDrop.h>

#if PP_Uses_Pragma_Import
	#pragma import on
#endif

PP_Begin_Namespace_PowerPlant

// ---------------------------------------------------------------------------

class	CLocalTreeView : public LTextGroupBox, public LDropArea {
private:
	StDialogHandler *fHandler;
	FSSpec fDropped;

public:
	enum { class_ID		= FOUR_CHAR_CODE('cltv'),
		   imp_class_ID	= FOUR_CHAR_CODE('itgb') };
	
						CLocalTreeView(
								LStream*		inStream,
								ClassIDT		inImpID = imp_class_ID);
								
						CLocalTreeView(
								const SPaneInfo&	inPaneInfo,
								const SViewInfo&	inViewInfo,
								Boolean				inPrimary = true,
								ResIDT				inTextTraitsID = 0,
								ConstStringPtr		inTitle = Str_Empty,
								ClassIDT			inImpID = imp_class_ID);
								
	virtual				~CLocalTreeView();
								
	virtual Boolean		PointInDropArea(
								Point				inGlobalPt);
								
	virtual void		HiliteDropArea(
								DragReference		inDragRef);
	virtual void		UnhiliteDropArea(
								DragReference		inDragRef);
								
	virtual Boolean		ItemIsAcceptable(
								DragReference		inDragRef,
								ItemReference		inItemRef);
								
	virtual void		DoDragReceive(
								DragReference		inDragRef);

	void SetPreferenceHandler(StDialogHandler *stHandler);
	FSSpec *GetDroppedFolder(void);
};

PP_End_Namespace_PowerPlant


#if PP_Uses_Pragma_Import
	#pragma import reset
#endif

#endif