//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSAppleEventItem.h
	
	Authors:		Simon Fraser
	
	Description:
	
		This file contains all the code for file objects, which are used for
		Apple Event handling
		 
	Modification History:

							
-------------------------------------------------------------------------------- */


#pragma once


#include "CMacCVSProjectViewItem.h"


class CMacCVSDoc;


class CMacCVSAppleEventItem : public LModelObject, public LBroadcaster
	
	{
			
		private:
			
			typedef LModelObject		Inherited;
			
			typedef struct TStatusEnumMap
				{
					DescType						enumValue;
					ProjectItemStatus		itemStatus;
				};


		public:
		
			enum 
				{
					cMacCVSAppleEventItem				= 'MCfl',
					
					pPathProperty								= 'Path',
					pStatusProperty							= 'Stat',
					pRevisionProperty						= 'Vers'
				};
		
			enum 
				{
					ae_ModifyReadOnly						= 7003,
					ae_Checkin									= 7004,
					ae_Tag											= 7005,
					
					ae_CheckinComment						= 'Cmnt',
					ae_ClearSticky							= 'stik',
					ae_TagRevision							= 'tagr',
					ae_TagIsStatic							= 'tags',
					ae_TagRemoveExisting				= 'tage'
				};
		
			enum 
				{
					eStatusNone									= 0,
					eStatusUnowned							= 'Unow',
					eStatusCheckedOut						= 'cKot',
					eStatusLocallyModified			= 'mRod',
					eStatusCheckedIn						= 'cKin',
					eStatusBroken								= 'Brok',
					eStatusCorrupt							= 'Krpt',
					eStatusDead									= 'Dead'			
				};

											CMacCVSAppleEventItem(					CMacCVSDoc* 						inParentDoc, 
																											const FSSpec& 					fileSpec);
											~CMacCVSAppleEventItem();

			// Apple Events
			
			SInt32					CountSubModels(									DescType								inModelID) const;

			void						GetSubModelByName(							DescType								inModelID,
																											Str255									inName,
																											AEDesc									&outToken) const;
					
			void						GetSubModelByPosition(					DescType								inModelID,
																											SInt32									inPosition,
																											AEDesc									&outToken) const;

			void						GetSubModelByComplexKey(				DescType								inModelID,
																											DescType								inKeyForm,
																											const AEDesc						&inKeyData,
																											AEDesc									&outToken) const;

			void						MakeSelfSpecifier(							AEDesc&									inSuperSpecifier,
																											AEDesc&									outSelfSpecifier) const;
			
			void						GetAEProperty( 									DescType 								inProperty,
																											const AEDesc&						inRequestedType,
																											AEDesc&									outPropertyDesc ) const;
	
			void						SetAEProperty( 									DescType 								inProperty,
																											const AEDesc&						inValue,
																											AEDesc&									outAEReply );
	
			void						HandleAppleEvent( 							const AppleEvent&				inAppleEvent,
																											AppleEvent&							outAEReply,
																											AEDesc&									outResult,
																											long 										inAENumber );



		protected:
						
			void						HandleModifyReadOnlyEvent(			const AppleEvent				&inAppleEvent,
																											AppleEvent							&outAEReply,
																											AEDesc									&outResult,
																											long										inAENumber);

			void						HandleCheckInEvent(							const AppleEvent				&inAppleEvent,
																											AppleEvent							&outAEReply,
																											AEDesc									&outResult,
																											long										inAENumber);

			void						HandleCheckOutEvent(						const AppleEvent				&inAppleEvent,
																											AppleEvent							&outAEReply,
																											AEDesc									&outResult,
																											long										inAENumber);

			void						HandleOpenEvent(								const AppleEvent				&inAppleEvent,
																											AppleEvent							&outAEReply,
																											AEDesc									&outResult,
																											long										inAENumber);

			void						HandleSelectEvent(							const AppleEvent				&inAppleEvent,
																											AppleEvent							&outAEReply,
																											AEDesc									&outResult,
																											long										inAENumber);

			void						HandleTagEvent(									const AppleEvent				&inAppleEvent,
																											AppleEvent							&outAEReply,
																											AEDesc									&outResult,
																											long										inAENumber);

			void						HandleUpdateEvent(							const AppleEvent				&inAppleEvent,
																											AppleEvent							&outAEReply,
																											AEDesc									&outResult,
																											long										inAENumber);
				
		private:
		
			DescType				ItemStatusToEnum(								ProjectItemStatus 			inItemStatus);
			ProjectItemStatus	
											EnumToItemStatus(								DescType								inStatusEnum);					
			
		protected:
		
			FSSpec										mFileSpec;
			CMacCVSDoc								*mParentDoc;

			static TStatusEnumMap			sStatusEnumMap[];

	};
	
	