//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CMacCVSDoc.h
//	
//	  	Authors: 	Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#pragma once


#include <LSingleDoc.h>
#include <LBroadcaster.h>

#include "CMacCVSProjectView.h"
#include "CMacCVSLurkersView.h"

#include "CMacCVSMessageWindow.h"
#include "CThreadedTask.h"
#include "CVSEngine.h"


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif


//  Yes, this is exactly like the CCVSEngineFileMappingRec,
//  but with 68k alignment turned on.

struct CMacCVSPrefsFileMappingRec
	{	
		Str255		fFileNamePStr;
		OSType		fFileType;
		OSType		fFileCreator;
		UInt32		fTranslation;
		UInt32		fEncoding;
	};
	
	
typedef struct CMacCVSPrefsFileMappingRec CMacCVSPrefsFileMappingRec;


// Ditto for the rec that stores defaults for diff and compare

struct CCVSPrefsFileDiffOptions
	{
		Boolean			mIgnoreCase;					// case insensitive							-i
		Boolean			mIgnoreWhiteSpace;		// ignore white space						-w
		Boolean			mIgnoreSpaceChange;		// ignore space change					-b
		Boolean			mIgnoreBlankLines;		// ignore blank lines 					-B
		Boolean			mShowCFunction;				// show C function							-p

		Boolean			mRCSDiff;							// RCS diff											-n
		Boolean			mContextDiff;					// true for context diff 				-c
		Boolean			mUnifiedDiff;					// UnifiedDiff									-u
		
		Int16				mContextLines;				// num lines of context or 0 for default			
	};

typedef struct CCVSPrefsFileDiffOptions CCVSPrefsFileDiffOptions;


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif


enum
	{	
		bUseRSH													= 0,
		kUseRSHFlag											= 1 << bUseRSH,
		
		bOutputFileUpdates							= 1,
		kOutputFileUpdatesFlag					= 1 << bOutputFileUpdates,

		bObsoleteLFTranslation					= 2,
		kObsoleteLFTranslationFlag			= 1 << bObsoleteLFTranslation,
		
		bClearMessageWindow							= 3,
		kClearMessageWindowFlag					= 1 << bClearMessageWindow,

		bOutputDirectoryUpdates					= 4,
		kOutputDirectoryUpdatesFlag			= 1 << bOutputDirectoryUpdates,
	
		bScrollMessageWindow						= 5,
		kScrollMessageWindowFlag				= 1 << bScrollMessageWindow,
		
		bListFolderViews								= 6,
		kListFolderViewsFlag						= 1 << bListFolderViews,
		
		bPropagateFolderLabels					= 7,
		kPropageFolderLabelsFlag				= 1 << bPropagateFolderLabels,		
		
		bSavePassword										= 8,
		kSavePasswordFlag								= 1 << bSavePassword
	};
	
	
//  MacCVS Preferences Format Version 5
//
//    UInt32																	magic number
//		UInt32																	version
//		UInt32																	mergePolicy
//    UInt32																	translation
//    UInt32																	flags
//   	UInt32																	socket timeout
//		PString																	hostName
//		PString																	remoteUser
//		PString																	password
// 		PString																	cvsCommand
// 		PString																	cvsRoot
//  	PString																	cvsModule
//  	PString																	defaultTag
//		UInt32																	tagCount
//		PString[tagCount] 											tags
//		UInt32																	mappingCount
//    CCVSEngineFileMappingRec[mappingCount]	mappings
//		AliasRecord															rootAlias


class CMacCVSDoc: 	
	
	public LSingleDoc,
	public LListener 
	
	{

		private:
		
			typedef LSingleDoc Inherited;

		public:

			enum
				{
					kMacCVSDocFileType						= 'McvD',
					kMacCVSCreator								= 'Mcvs',					
					kDefaultSocketTimeOut					= 3 * 60,

					//  AppleEvent stuff

					//  Command numbers and their arguments	

					cMacCVSProSession							= cDocument,
					ae_Checkout										= 7001,
					ae_Update											= 7002,
					ae_UpdateRevision							= 'tagr',
					ae_UpdateDate									= 'tagd',
					ae_UpdateModuleName						= 'modl',
	
					//  Session properties

					pModuleName										= 'pmnm',
					pDefaultTag										= 'ptag',
					pHostName											= 'honm',
					pRemoteUser										= 'rusr',
					pCVSRoot											= 'cvsr',						
					pLocalRootFile								= 'lrfs',
					pPassword											= 'pass',
					pModified											= 'imod',
					pSessionName									= 'pnam',	

					//  Prefs File Format Versions
	
					kMacCVSPrefsFileMagicNumber 	= 0xBEEFEAFF,
	
					//  This cannot be read anymore
	
					kMacCVSPrefsFileFormat1 			= 0xFEFE0001L,
					kMacCVSPrefsFileFormat2				= 0xFEFE0002L,
					kMacCVSPrefsFileFormat3				= 0xFEFE0003L,
					kMacCVSPrefsFileFormat4				= 0xFEFE0004L,
					kMacCVSPrefsFileFormat5				= 0xFEFE0005L,
					
					//  This are valid versions
					
					kMacCVSPrefsFileFormat6				= 0xFEFE0006L,
					kMacCVSPrefsFileFormat7				= 0xFEFE0007L,
					//  Between 7 and 8 RSH flag was removed and authentication method was added 
					kMacCVSPrefsFileFormat8				= 0xFEFE0008L,
					kMacCVSPrefsFileFormat10			= 0xFEFE0010L,
					kMacCVSReadableFileFormat			= kMacCVSPrefsFileFormat6,
					kMacCVSCurrentFileFormat			= kMacCVSPrefsFileFormat10,
										
					//  Prefs File Item Flags
					
					kMacCVSPrefsFlagsData					= 'flag',
					kMacCVSPrefsMergeData					= 'merg',
					kMacCVSPrefsServerNameData		= 'name',
					kMacCVSPrefsRootServerPathData= 'root',
					kMacCVSPrefsCommandLineData		= 'cmd ',
					kMacCVSPrefsRemoteUserData		= 'ruse',
					kMacCVSPrefsPasswordData			= 'ldat', //  probably don't want to use 'pass' (duh)
					kMacCVSPrefsAliasData					= 'alis',
					kMacCVSPrefsDefaultTagData		= 'dtag',
					kMacCVSPrefsDefaultModuleData = 'dmod',
					kMacCVSPrefsAuthTypeData			= 'auth',
					kMacCVSPrefsTagData						= 'tagl',
					kMacCVSPrefsMappingData				= 'mapp',
					kMacCVSPrefsGzipData					= 'gzip',
					kMacCVSPrefsSocketTimeoutData = 'sock',
					kMacCVSPrefsDiffOptionsData		= 'diff',
					kMacCVSPrefsCompareOptionsData= 'comp',
					kMacCVSPrefsPortData					= 'port',
					kMacCVSPrefsEOFData						= 'eof '										
				};

												CMacCVSDoc(								const FSSpec					&fsInItem,
																									LCommander 						*inSuperCommander);
												~CMacCVSDoc();
												
			void							Init();
			
			static LDocument	*FindFSSpecDocument(			FSSpec								&fileSpec);
			static LDocument*	GetNthDocument(						SInt32 								inPosition);			
										
			void							EditPreferences();
			void							LoadPreferences();
			void							LoadLegacyPreferences(		const UInt32					inVersion);					
			void							SavePreferences();
			void							SetNewPathInPrefsDialog(	LDialogBox						*inPrefDialog, 
																									const FSSpec 					&inFileSpec);															
			void							PostProcessPathChange();
			Boolean						GetPasswordFromUser();
			void							InvalidateCachedPassword();

			virtual void			AttemptClose(							Boolean					 			inRecordIt);		
			virtual Boolean		AttemptQuitSelf(					Int32 								inSaveOption);		
			virtual Boolean 	AllowSubRemoval(					LCommander						*inSub);				
			virtual Boolean		ObeyCommand(							CommandT 							inCommand, 
																									void									*ioParam);				
			virtual void			FindCommandStatus(				CommandT 							inCommand,
																									Boolean 							&outEnabled, 
																									Boolean 							&outUsesMark,
																									Char16 								&outMark, 
																									Str255 								outName);
			virtual void			ListenToMessage(					MessageT							inMessage,
																									void									*ioParam);

				

			virtual void			GetSubModelByName(				DescType							inModelID,
																									Str255								inName,
																									AEDesc								&outToken) const;
			virtual void			GetSubModelByPosition(		DescType							inModelID,
																									SInt32								inPosition,
																									AEDesc								&outToken) const;					
			virtual SInt32		CountSubModels(						DescType 							inModelID) const;
			virtual void			GetSubModelByComplexKey(	DescType							inModelID,
																									DescType							inKeyForm,
																									const AEDesc&					inKeyData,
																									AEDesc&								outToken) const;
			virtual void			GetAEProperty(						DescType 							inProperty,
										  														const AEDesc 					&inRequestedType,
										 	 														AEDesc								&outPropertyDesc) const;
			virtual void			SetAEProperty(						DescType							inProperty,
																									const AEDesc					&inValue,
																									AEDesc								&outAEReply);
			void							HandleAppleEvent( 				const AppleEvent			&inAppleEvent,
																									AppleEvent						&outAEReply,
																									AEDesc								&outResult,
																									long									inAENumber);
			void							HandleCheckoutEvent( 			const AppleEvent			&inAppleEvent,
																									AppleEvent						&outAEReply,
																									AEDesc								&outResult,
																									long									inAENumber);																								
			void							HandleUpdateEvent( 				const AppleEvent			&inAppleEvent,
																									AppleEvent						&outAEReply,
																									AEDesc								&outResult,
																									long									inAENumber);
					
	
			void							GetLoginOptions(				 	CCVSEngineLoginOptions 
																																				&sOutLoginOptions);																								
			void							GetHostNamePStr(					Str255								outHostNamePStr);
			void							GetRemoteUserPStr(				Str255 								outRemoteUserPStr);
			void							GetPasswordPStr(					Str255 								outPasswordPStr);
			void							GetCVSCommandPStr(				Str255								outCVSCommandPStr);
			void							GetCVSRootPStr(						Str255								outCVSRootPStr);
			void							GetModulePStr(						Str255 								outModulePStr);
			void							GetDefaultTagPStr(				Str255								outTagPStr);

			Boolean						GetDefaultCheckoutInProgress()
																											{ return mCheckingOutDefaultModule; }
			void							SetDefaultCheckoutInProgress(		
																									Boolean 							inInProgress);
			
			void							GetDefaultsForDiffOrCompare( 
																									CCVSEngineDiffOptions &outOptions,
																									Boolean 							isCompare);
			
			void							SetDefaultsForDiffOrCompare( 
																									const CCVSEngineDiffOptions &inOptions,
																									Boolean 							isCompare);

			void							GetProjectRootFileSpec( 	FSSpec 								&outProjectRoot) const
																											{ outProjectRoot = mRootFileSpec; }																										 
			Boolean						IsProjectRootSpecified() const 
																											{ return mProjectRootSpecified; }		
			UInt32						GetTranslation()							{ return 0; }
			UInt32						GetSocketTimeOut()						{ return mSocketTimeOut; }
			UInt32						GetCompressionLevel()					{ return mCompressionLevel; }
			UInt32						GetMergePolicy() 							{ return mCVSMergePolicy; }
			UInt16						GetHostPort()									{ return mPort; }
			
			Boolean						GetOutputFileUpdates() 				{ return mOutputFileUpdates; } 
			Boolean						GetOutputDirectoryUpdates() 	{ return mOutputDirectoryUpdates; } 

			Boolean						GetUseRSH() 									{ return (mAuthenticationMethod == kAuthenticationRSH); }
			AuthenticationMethod	
												GetAuthenticationMethod()			{ return mAuthenticationMethod; }
			Boolean						GetClearMessageWindow()				{ return mClearMessageWindow; }
			Boolean						GetScrollMessageWindow()			{ return mScrollMessages; }
			Boolean						GetNewFoldersUseListView()		{ return mNewFoldersUseListView; }
			Boolean						GetNewFoldersInheritLabels()	{ return mNewFoldersInheritLabels; }			
			LArray						*GetTagListPtr() 							{ return mTagList; }
			void							GetDocumentFileSpec( 			FSSpec 								&outDocumentSpec) const
																											{ outDocumentSpec = mDocumentFileSpec; }
			virtual void 			PostMessage(							const UInt32 					inMessageType, 
																									const char 						*inMessageText, 
																									const UInt32 					inMessageLen,
																									const void						*inMessageData,
																									const UInt32					inDataSize);					
	
			virtual void			FindLurkers(							LArray								*inItemList);		
			virtual void			OrphanFiles(							LArray								*inItemList);
			

			static Boolean		AreTagListsEqual(					LArray								&inTagList1,
																									LArray								&inTagList2);				
			virtual void			UseNewTagList(						LArray								&inNewTagList);

						
			virtual Boolean		AllowKillAllTasks();
			virtual void			KillAllTasks();
			void							AddToTaskList(						CThreadedTask					*inTask);
			void							RemoveFromTaskList(				CThreadedTask 				*inTask);


			void							BringToFront();			
			LWindow* 					NewProjectWindow(					FSSpec								*inAnchorFileSpec,
																									FSSpec								*inProjectRootFileSpec,
																									StringPtr							inWinTitle = NULL);
			LWindow*					NewViewWindow(						FSSpec 								*inAnchorRootFileSpec, 
																									FSSpec 								*inProjectRootFileSpec,
																									Boolean								&outMadeNew,
																									Boolean 							inSelectIfExists = true);			
			void							PositionDocWindow(				LWindow								*inDocWindow);
			void							SaveDocWindowPosition( 		LWindow 							*inDocWindow );	
			void							UpdateItemInProjectViews(	const FSSpec					*inItemToUpdate,
																									const Boolean					inUpdateAllViews = true);
			Boolean						IsOurWindow(							LWindow								*inWindow);		
			void							AddToWindowList(					LWindow								*inWindow);
			Boolean						RemoveFromWindowList(			LWindow								*inWindow);
			
			void							InitDefaultMappingList();
			
			static OSErr			BuildFileMappingList(			const LArray					*pInMappings,
																									CCVSEngineFileMappingRec
																																				**pOutMappings,
																									UInt32								*intOutCount);
				
			LArray						*GetFileMappingList( void );
			void							SetFileMappingList(				const CCVSEngineFileMappingRec 
																																				*pInMappingsList,
																									const UInt32					intInCount);
			void							SetFileMappingList(				const LArray					&cInMappings);
																																													
			static CMacCVSProjectView	
												*GetProjectWindowView(		LWindow								*inWindow);
			static CMacCVSLurkersView	
												*GetLurkersWindowView(		LWindow								*inWindow);
			static SInt16			GetWindowKind(						LWindow								*inWindow);
			
		protected:
		
			void							KillAllViews();
			void							OpenFile(									FSSpec 								&inFileSpec);
			
			Boolean						CheckCVSServerSettings();
			
			
			FSSpec											mRootFileSpec;
			FSSpec											mDocumentFileSpec;
									
			Str255											mModuleName;
			Str255											mDefaultTag;
			Str255											mHostName;
			Str255											mLocalUser; 
			Str255											mRemoteUser;
			Str255											mPassword;
			Str255											mCVSCommand;
			Str255											mCVSRoot;
			
			UInt32											mCVSMergePolicy;
			UInt32											mSocketTimeOut;
			UInt32											mCompressionLevel;
			UInt16											mPort;
			
			Boolean											mOutputFileUpdates;
			Boolean											mOutputDirectoryUpdates;
			Boolean											mClearMessageWindow;
			Boolean											mScrollMessages;
			Boolean											mNewFoldersUseListView;
			Boolean											mNewFoldersInheritLabels;
			Boolean											mSavePassword;
			Boolean											mPasswordValid;
			
			// Separate storage for authentication method and the menu item that the user selected
			// The reason for this is that the item number is more convinient for dialog handling,
			// but to avoid API lossage if we reorder items in the menu, we have independent constants
			// for authentication methods.
			AuthenticationMethod				mAuthenticationMethod;
			SInt16											mAuthenticationMethodItem;
			
			Boolean											mCheckingOutDefaultModule;
			
			UInt16											mPrefPanel;
			Boolean											mProjectRootSpecified;
		
			CCVSPrefsFileDiffOptions		mDiffOptions;
			CCVSPrefsFileDiffOptions		mCompareOptions;
			
			LArray											*mTagList;
			LArray											*mMappingList;					
			LArray											*mWindowList;
			LArray											*mConnectionList;
			
			CMacCVSMessageWindow				*mMessageWindow;

	};

