//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSLogView.h
//
//	 	Author: 	Ram Rajadhyaksha
//
//################################################################################


#pragma once


#include <Threads.h>
#include <LHierarchyTable.h>
#include <LListener.h>

#include "CThreadedTask.h"


struct CMacCVSLogViewCellRec
	{

		Str255			fVersionPStr;
		Str255			fAuthorPStr;
		UInt32			fDate;
		char				*fCommentStr;
		UInt32			fCommentLen;
		UInt16			fCachedHeight;
		UInt16			fFlags;

	};
	

typedef CMacCVSLogViewCellRec CMacCVSLogViewCellRec;
typedef CMacCVSLogViewCellRec *CMacCVSLogViewCellPtr;


struct CMacCVSLogViewEventRec
	{

		UInt32		fEventType;
		void			*fEventData;

	};


typedef CMacCVSLogViewEventRec CMacCVSLogViewEventRec;
typedef CMacCVSLogViewEventRec *CMacCVSLogViewEventPtr;


//################################################################################


class CMacCVSLogViewThread;


class CMacCVSLogView: 

	public LHierarchyTable,
	public LCommander,
	public LListener,
	public LBroadcaster
	
	{
		
		friend class CMacCVSLogViewThread;
		
		public:
		
			enum 
				{
				
					class_ID												= 'LWVW',
					kMaxCommentLen									= 0x3FFF,
					kDefaultLogViewThreadStackSize	= 16 * 1024,
					kCMacCVSLogViewNullEvent				= 0,
					kCMacCVSLogViewSortByAuthor			= 1,
					kCMacCVSLogViewSortByVersion		= 2,
					kCMacCVSLogViewSortByDate				= 3,
					kCMacCVSLogViewAddItemEvent			= 4,
					kCMacCVSLogViewExpandEvent			= 5,
					kCMacCVSLogViewExpandDeepEvent	= 6,
					kCMacCVSLogViewCollapseEvent		= 7,
					kCMacCVSLogViewCollapseDeepEvent= 8,
				
					kCellTextOffset									= 30,
					kCellVersionTextOffset					= 190,
					kCellAuthorTextOffset						= 30,
					kCellDateTextOffset							= 270,
					kCellChildOffset								= 15,
					kCellTextTraitsID								= 130,
					kCellTextOffsetFromBottom				= 4,
					kCMacCVSLogItemNoLongerExists		= -1,
					
					kDefaultYieldCount							= 200,
							
					kDefaultColumnWidth							= 385
			
				};

															CMacCVSLogView(								LStream 					*inStream);
			virtual 								~CMacCVSLogView();
			
			static CMacCVSLogView 	*CreateLogViewFromStream(			LStream 					*inStream);

			virtual void						FinishCreateSelf();
			
			virtual void 						ListenToMessage(							MessageT					inMessage,
																														void							*ioParam);

			virtual Boolean					ObeyCommand(									CommandT					inCommand,
																														void							*ioParam);	

			virtual void						FindCommandStatus(						CommandT 					inCommand,
																														Boolean						&outEnabled,
																														Boolean 					&outUsesMark,
																														Char16						&outMark,
																														Str255 						outName);
																														
			virtual void						SetLogItem(										const FSSpec			&inLogItem)		{ mLogItem = inLogItem; }
			virtual void						GetLogItem(										FSSpec						&outLogItem)	{ outLogItem = mLogItem; }
						
			virtual void 						ExpandRow(										TableIndexT				inWideOpenRow);
			virtual void 						DeepExpandRow(								TableIndexT				inWideOpenRow);

			virtual void 						CollapseRow(									TableIndexT				inWideOpenRow);
			virtual void 						DeepCollapseRow(							TableIndexT				inWideOpenRow);
				
			virtual void 						PostLogViewEvent(							const CMacCVSLogViewEventRec &inEvent);
			
			virtual Boolean					GetNextLogViewEvent(					CMacCVSLogViewEventRec &inEvent);

/*
			virtual TableIndexT 		HandleExpandEvent(						CMacCVSLogViewCellRec	*inExpandItem,
																														const TableIndexT inParentIndex = 1);
			
			virtual void 						HandleExpandDeepEvent(				CMacCVSLogViewCellRec	*inExpandItem);
	
			virtual TableIndexT			HandleCollapseEvent(					CMacCVSLogViewCellRec	*inCollapseItem,
																														const TableIndexT inParentIndex = 1);
																					
			virtual void						HandleCollapseDeepEvent(			CMacCVSLogViewCellRec	*inCollapseItem);

*/										
			virtual void 						SortByAuthor(void);
			virtual void 						SortByVersion(void);
			virtual void 						SortByDate(void);

			virtual void 						AddLogEntry(									const Str255 			inAuthorPStr, 	
																														const Str255 			inVersionPStr, 
																														const UInt32 			inDate,			
																														const char 				*inCommentStr,
																														const UInt32			inCommentLen);
		
			virtual void						ThreadedTaskDied(							const CThreadedTask					*inTask);
			
			static Int32 						CompareAuthor(								const CMacCVSLogViewCellRec *inItemOne,
																														const CMacCVSLogViewCellRec	*inItemTwo);
																														
			static Boolean 					IsEqualAuthor(								const CMacCVSLogViewCellRec *inItemOne,
																														const CMacCVSLogViewCellRec *inItemTwo);
																														
			static Int32 						CompareVersion(								const CMacCVSLogViewCellRec	*inItemOne,
																														const CMacCVSLogViewCellRec	*inItemTwo);
																														
			static Boolean 					IsEqualVersion(								const CMacCVSLogViewCellRec	*inItemOne,
																														const CMacCVSLogViewCellRec	*inItemTwo);
		
			static Int32 						CompareDate(									const CMacCVSLogViewCellRec	*inItemOne, 
																														const CMacCVSLogViewCellRec *inItemTwo);
																														
			static Boolean 					IsEqualDate(									const CMacCVSLogViewCellRec	*inItemOne,
																														const CMacCVSLogViewCellRec	*inItemTwo);
			
			virtual void						AdaptToSuperFrameSize(				Int32												inSurrWidthDelta,
																														Int32												inSurrHeightDelta,
																														Boolean											inRefresh);
									
		protected:

			virtual void 						HiliteCellActively(						const STableCell						&inCell,
																														Boolean											inHilite);
																														
			virtual void 						HiliteCellInactively(					const STableCell						&inCell,
																														Boolean											inHilite);
			
			virtual void						ClickSelf( 										const SMouseDownEvent				&inMouseDown);
			
			virtual void 						ClickCell(										const STableCell 						&inCell,
																														const SMouseDownEvent 			&inMouseDown);

			virtual void 						DrawSelf();
			
			virtual void 						DrawCell( 										const STableCell						&inCell,
																														const Rect									&inLocalRect);

			virtual void						SortLogView(									const UInt32								inSortType);
			virtual void 						RebuildLogView(								const Boolean								inRefresh = true);
			virtual LComparator			*GetCurrentComparator();
			
			virtual void 						InsertLogEntry(								const CMacCVSLogViewCellRec	*inNewEntry,
																														const Boolean								inRefresh = true);
			
			virtual TableIndexT 		ConvertCellRecToWideIndex(		const CMacCVSLogViewCellRec *inCell,
																														const TableIndexT						inParentIndex = 1);

			virtual CMacCVSLogViewCellRec *ConvertWideIndexToCellRec(const TableIndexT 				inRow);	

			UInt16									GetTextHeightGivenWidth(			char												*inTextStr, 
																														const UInt32 								inTextLength, 
																														Rect 												&inTextRect);
																														
			virtual void						GetDiffs( 										const CMacCVSLogViewCellRec *inFromCellRec,
																														const CMacCVSLogViewCellRec *inToCellRec,
																														Boolean 										inAdvancedDiff = false);

			virtual void						DoUpdate( 										ConstStringPtr 							pstrInRevision);
																														
			CMacCVSLogViewThread		*mViewThread;
		
			LArray									*mLogEntries;
			LArray									*mViewEventQueue;

			FSSpec									mLogItem;
	
			UInt16 									mCurrentSortMethod;
			
			static UInt32						sYieldCount;
			
	};


class CMacCVSLogViewThread:

	public CThreadedTask
	
	{
	
		friend class CMacCVSLogView;

		public:
		
										CMacCVSLogViewThread( CMacCVSLogView &inOwnerView);
			virtual				~CMacCVSLogViewThread();
			
			virtual	void	Run();
			
		protected:
		
			CMacCVSLogView		*mOwnerView;
			
	};
	

//################################################################################
//################################################################################


class	CMacCVSLogViewAuthorComparator: 

	public LComparator 
	
	{
	
		public:
												CMacCVSLogViewAuthorComparator() { }
			virtual						~CMacCVSLogViewAuthorComparator() { }
					
			virtual Int32			Compare(		const void*			inItemOne,
																		const void* 		inItemTwo,
																		Uint32					inSizeOne,
																		Uint32					inSizeTwo) const;
										
			virtual Boolean		IsEqualTo(	const void*			inItemOne,
																		const void* 		inItemTwo,
																		Uint32					inSizeOne,
																		Uint32					inSizeTwo) const;
																		
	};


class	CMacCVSLogViewVersionComparator: 

	public LComparator 
	
	{
	
		public:
												CMacCVSLogViewVersionComparator() { }
			virtual						~CMacCVSLogViewVersionComparator() { }
					
			virtual Int32			Compare(		const void*			inItemOne,
																		const void* 		inItemTwo,
																		Uint32					inSizeOne,
																		Uint32					inSizeTwo) const;
										
			virtual Boolean		IsEqualTo(	const void*			inItemOne,
																		const void* 		inItemTwo,
																		Uint32					inSizeOne,
																		Uint32					inSizeTwo) const;
																		
	};


class	CMacCVSLogViewDateComparator: 

	public LComparator 
	
	{
	
		public:
												CMacCVSLogViewDateComparator() { }
			virtual						~CMacCVSLogViewDateComparator() { }
					
			virtual Int32			Compare(		const void*			inItemOne,
																		const void* 		inItemTwo,
																		Uint32					inSizeOne,
																		Uint32					inSizeTwo) const;
										
			virtual Boolean		IsEqualTo(	const void*			inItemOne,
																		const void* 		inItemTwo,
																		Uint32					inSizeOne,
																		Uint32					inSizeTwo) const;
																		
	};

