//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSLurkersView.h
//
//		Authors: 	Ram Rajadhyaksha
//
//################################################################################


#pragma once


#include <Threads.h>
#include <LComparator.h>
#include <LTableView.h>
#include <LArray.h>

#include "CCVSEntries.h"
#include "CThreadedTask.h"
#include "CMacCVSProjectViewItem.h"
#include "CMacCVSProjectViewFile.h"
#include "CMacCVSProjectViewFolder.h"
#include "CMacCVSProjectViewRootFolder.h"


typedef UInt32 CMacCVSLurkersViewEventType;
typedef UInt32 CMacCVSLurkersViewSortType;


struct CMacCVSLurkersViewEventRec 
	{

		CMacCVSLurkersViewEventType		fEventType;
		CMacCVSProjectViewItem				*fEventItem;
		void													*fEventData;
		
	};


typedef struct CMacCVSLurkersViewEventRec CMacCVSLurkersViewEventRec;


class CMacCVSLurkersViewThread;
class CMacCVSLurkersViewSelector;
class ULurkersKeyScrollAttachment;


class CMacCVSLurkersView: 	

	public LTableView,
	public LBroadcaster,
	public LListener,
	public LCommander 
	
	{
				
		friend class CMacCVSLurkersViewThread;
		friend class CMacCVSLurkersViewSelector;
		friend class ULurkersKeyScrollAttachment;
			
		public :
			
			enum 
				{
				
					class_ID												= 'LurV',

					kViewDeathEvent									= 0,
					kViewUpdateItemEvent						= 1,
					kViewUpdateItemFSSpecEvent			= 2,
					kViewAddItemEvent								= 3,
					kViewSortByNameEvent						= 4,
					kViewSortByStatusEvent					= 5,
					kViewSortByVersionEvent					= 6,
					kViewSortByDateEvent						= 7,
					kViewSortByTagEvent							= 8,

					kNameSortMessage								= 'stnm',
					kStatusSortMessage							= 'stst',
					kVersionSortMessage							= 'stvr',
					kDateSortMessage								= 'stdt',
					kTagSortMessage									= 'sttg',
					kNewViewMessage									= 'VIEW',
					kDefaultYieldCount							= 16,
					kDefaultViewThreadStackSize 		= 16 * 1024,  // 16K stack space
					kDefaultUpdateTime							= 4,

					kProjectNoLongerExistsRow				= -1,
					kFirstFileStatusIconSuiteID			= 150,
					
					kCellIconOffset									= 20, 
					kCellTextOffset									= 42,
					kFileStatusIconOffset						= 266,
					kFileLockIconOffset							= 283,
					kCellVersionTextOffset					= 300,
					kCellTagTextOffset							= 360,
					kCellModificationTextOffset			= 498,

					kFileStatusIconWidth						= 16,
					kFileStatusIconHeight						= 16,
					kFileLockIconWidth							= 16,
					kFileLockIconHeight							= 16,
					
					kCellChildOffset								= 15,
					kCellTextTraitsID								= 140,
					kCellTextTraitsItalicID					= 141,
					kCellTextOffsetFromBottom				= 6,
					kCellFolderIconID								= kGenericFolderIconResource,
					kCellFileIconID									= kGenericDocumentIconResource,
					kCellLockIconID									= 200,

					kNotSorted											= 0,
					kNameSorted											= 1,
					kStatusSorted										= 2,
					kVersionSorted									= 3,
					kTagSorted											= 4,
					kDateSorted											= 5,

					kDefaultForegroundUpdateTime		= 1 * 60, 
					kDefaultBackgroundUpdateTime		= 4 * 60,

					kDefaultRowWidth								= 610,
					kDefaultRowHeight								= 20

				};
					
																	CMacCVSLurkersView(						LStream 											*inStream);
			virtual											~CMacCVSLurkersView();
			
			virtual	void								SetProjectRoot(								const FSSpec									*inProjectRootSpec);
			virtual void								UpdateProjectItem( 						const FSSpec 									*inTargetItem);
			
			// Selection

			virtual UInt32							CountSelectedItems(void) const;
			virtual LArray							*GetFlatSelection();
			
			virtual void								AddProjectItem( 							const FSSpec									*inItemFileSpec);
			virtual void								ReleaseReadLocks();
			
			virtual void 								SelectionChanged();
			
			virtual void								ShowAllCells();

			//	Command handling

			virtual Boolean							CheckIfAnyItemsQualify(				CommandT											inCommand);
			
			virtual void								BeTarget();
			virtual void								DontBeTarget();
			
			virtual Boolean							ObeyCommand(									CommandT											inCommand,
																																void													*ioParam);	

			virtual void								FindCommandStatus(						CommandT 											inCommand,
																																Boolean												&outEnabled,
																																Boolean 											&outUsesMark,
																																Char16												&outMark,
																																Str255 												outName);

			virtual void 								ListenToMessage(							MessageT											inMessage,
																																void													*ioParam);

			virtual void 								SortProjectView(							CMacCVSLurkersViewSortType 		inSortMethod);

		protected:
		
			virtual	void								FinishCreateSelf(void);

			virtual TableIndexT					InsertProjectItem(						CMacCVSProjectViewItem				*inItem);
				
			virtual TableIndexT					RemoveProjectItem(						const CMacCVSProjectViewItem	*inItem,
																																const Boolean									inRefresh);

			virtual void								RefreshEntryData( 						CMacCVSProjectViewItem 				*inItem);																										

			CMacCVSProjectViewItem			*ConvertTableIndexToItem(			const TableIndexT							inIndex) const;
			
			TableIndexT 								ConvertItemToTableIndex(			const CMacCVSProjectViewItem 	*inItem,
																																const TableIndexT							inStartIndex = 1) const;

			//	Drawing
	
			virtual	void								DrawSelf(void);
			
			virtual	void 								DrawCell(											const STableCell							&inCell,
																																const Rect										&inLocalRect);

			virtual	void								DrawCellIcon(									const STableCell							&inCell,
																																const Rect										&inLocalRect,
																																Boolean												inHilite);

			virtual void								HiliteSelection(							Boolean												inActively,
																																Boolean 											inHilite);
			
			virtual void								HiliteCellActively(						const STableCell							&inCell,
																																Boolean												inHilite);

			virtual void								HiliteCellInactively(					const STableCell							&inCell,
																																Boolean												inHilite);

			virtual	void 								CalcCellTextFrame(						const STableCell							&inCell,
																																Rect													&outLocalFrame);

			virtual void								CalcCellHotSpots( 						const STableCell 							&inCell,
																																Rect													&outIconFrame,
																																Rect													&outTextFrame);
			
			virtual void								HideSelf();

			virtual void								AdaptToNewSurroundings();

			//  Selection
			
			virtual void								ClickSelf(										const SMouseDownEvent					&inMouseDown);
																																
			virtual void 								ClickCell(										const STableCell							&inCell,
																																const SMouseDownEvent					&inMouseDown);
			
			virtual Boolean							ClickSelect(									const STableCell							&inCell,
																																const SMouseDownEvent					&inMouseDown);

			virtual	void								GetCellIconRgn(								const STableCell							&inCell,
																																Boolean												inHilite,
																																RgnHandle											ioRgnH);

			virtual void								GetHiliteRgn( 								RgnHandle 										ioHiliteRgn);
			
			virtual void								GetLocalCellRectAnywhere( 		const STableCell							&inCell,
																																Rect 													&outCellRect ) const;
			//  Threading

			void												ProcessProjectViewEvent(		 	CMacCVSLurkersViewEventRec 				&inEventRecord);	
			virtual Boolean							GetNextProjectViewEvent(			CMacCVSLurkersViewEventRec 				&inEventRecord);
			virtual void								PostProjectViewEvent(					const CMacCVSLurkersViewEventRec 	&inEventRecord);
									
			//	Event Handlers
										
			virtual Boolean							HandleKeyPress(								const EventRecord							&inKeyEvent);
										
			virtual void 								HandleUpdateEvent(						CMacCVSProjectViewItem 				*inUpdateItem);
						
			virtual void 								HandleUpdateFSSpecEvent( 			const FSSpec 									&inUpdateItem);
									
			virtual void								RebuildSortedProjectView(			Boolean												inRefresh);
																		
			virtual LComparator					*GetCurrentComparator();
						
			virtual void								ThreadedTaskDied(							const CThreadedTask						*inTask);

			void												UserDraggedSelection() { LPane::sClickCount = 0; mUserDraggedSelection = true; }

			CMacCVSLurkersViewThread				*mViewThread;
		
			LArray													*mViewEventQueue;
			LArray													*mItemList;
		
			FSSpec													mProjectRootFileSpec;
		
			CMacCVSLurkersViewSortType			mCurrentSortMethod;
				
			UInt32													mWatchTime;
			
			Boolean													mUserDraggedSelection;
			RGBColor												mBGColor;
			
			CCVSEntries											mEntries;
			
			static UInt32										sYieldCount;
			static Handle										sFileStatusIconSuites[kMacCVSProjectorTotalCount];
			
	};


class CMacCVSLurkersViewThread:

	public CThreadedTask
	
	{
	
		friend class CMacCVSLurkersView;

		public:
		
										CMacCVSLurkersViewThread( CMacCVSLurkersView &inOwnerView);
			virtual				~CMacCVSLurkersViewThread();
			
			virtual	void	Run();
			
		protected:
		
			CMacCVSLurkersView		*mOwnerView;
	
	};
	
