//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSMessageView.h
//
//	  Author: 	Ram Rajadhyaksha and others (unknown)
//
//################################################################################


#pragma once


#include <LTableView.h>
#include <LCommander.h>

#include "CMacCVSMessageWindow.h"


struct CMacCVSMessageViewCellDataRec
	{
		
		UInt16			fCellType;				//we may have thousands of these floating around,
		UInt16			fCellFlags;				//so be frugal with memory
		char				*fCellTextPtr;
		UInt32			fCellTextLen;
		void				*fCellData;				//data type depends on fCellType

	};


typedef struct CMacCVSMessageViewCellDataRec CMacCVSMessageViewCellDataRec;

	
class CMacCVSMessageView: 

	public LTableView, 
	public LCommander,
	public LBroadcaster
	
	{

		public:

			enum 
				{
				
					class_ID										= 'msvw',
					kErrorIconID								= 1280,
					kWarningIconID							= 1281,
					kMessageIconID							= 1282,
					kFileUpdatedIconID					= 1283,
					kFileMergedIconID						= 1284,
					kFileConflictIconID					= 1285,
					
				// message types
					kMessageTypeError						= 1,
					kMessageTypeWarning					= 2,
					kMessageTypeMessage					= 3,
					kMessageTypeFile						= 4,
				
				// message flags
				// bits 0-7 apply to all cell types
				
				
				// bits 8-15 are specific to each cell type
				
				// error message flags
				
				// warning message flags
				
				// message message flags
				
				// file message flags
					kFileMessageUpdatedFlag			= (1 << 8),
					kFileMessageMergedFlag			= (1 << 9),
					kFileMessageConflictFlag		= (1 << 10),
				
					kErrorMask									= 0x01,
					kWarningMask								= 0x02,
					kMessageMask								= 0x04,
					kFileMask										= 0x08
				};
							
			enum
				{
					kDefaultTextOffset					= 26,
					
					kDefaultRowHeight						= 26,
					kDefaultColumnWidth					= 500,

					kMessagesSetAllID						= -1
				
				};

																	CMacCVSMessageView(			LStream 				*inStream);
																	~CMacCVSMessageView();
		
			static CMacCVSMessageView 	*CreateCMacCVSMessageViewFromStream(
																													LStream 				*inStream);

			virtual void								AdaptToSuperFrameSize(	Int32						inSurrWidthDelta,
																													Int32						inSurrHeightDelta,
																													Boolean					inRefresh);

			virtual void								FindCommandStatus(			CommandT 				inCommand,
																													Boolean 				&outEnabled, 
																													Boolean 				&outUsesMark,
																													Char16 					&outMark, 
																													Str255 					outName);
			
			virtual Boolean							ObeyCommand(						CommandT				inCommand,
																													void						*ioParam);

			virtual Boolean 						IsMessageTypeEnabled(		const UInt32 		inMessageType);
			
			virtual void								ShowMessageType(				const UInt32		inMessageType,
																													const Boolean 	inRefresh = true);
																													
			virtual void								DontShowMessageType(		const UInt32		inMessageType,
																													const Boolean 	inRefresh = true);
			
			virtual void								PostMessage(						const UInt32 		inMessageType, 
																													const char 			*inMessageText, 
																													const UInt32 		inMessageLen,
																													const void			*inMessageData,
																													const UInt32		inDataSize);	

			virtual void								SetScrollToNewMessages(	Boolean					inScroll)
																														{
																																	mScrollToNewMessages = inScroll;
																														};
			virtual void								ClearMessages();

			virtual void								ClickCell(							const STableCell			&inCell,
																													const SMouseDownEvent	&inMouseDown);
			
			virtual void								OpenCell( 							const STableCell			&inCell );
			
		protected:
	
			virtual void								FinishCreateSelf();
	
			virtual void								SetMessageMask(					const UInt32 		inNewMask);

			virtual void 								DrawCell(								const STableCell &inCell,
																													const Rect			&inLocalRect);
			
			virtual void								DeleteAllMessages();		
						
			LArray		*mMessageItems;
			
			UInt32		mMessageMask;
			
			Boolean		mScrollToNewMessages;
	};
	