//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSMessageWindow.h
	
	Authors:		Ram Rajadhyaksha and others (unknown)
	
	Description:
	
		This file contains code to implement the message window in MacCVS.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Added Andy Nicholas' grayscale changes.
							
-------------------------------------------------------------------------------- */


#pragma once


#include <LWindow.h>
#include <LStdControl.h>
#include <LListener.h>
#include <LBevelButton.h>
#include <LPushButton.h>

#include "CMacCVSThreadedWindow.h"

class CMacCVSDoc;
class CMacCVSMessageView;

class CMacCVSMessageWindow: 
	
	public CMacCVSWindow,
	public LBroadcaster,
	public LListener 
	
	{
	
		public:
		
			enum 
				{
				
					class_ID										= 'mswn',
					
					kErrorButtonPaneID					= 'berr',
					kErrorButtonValueMessage		= 'berr',
					
					kWarningButtonPaneID				= 'bwar',
					kWarningButtonValueMessage	= 'bwar',
				
					kMessageButtonPaneID				= 'bmsg',
					kMessageButtonValueMessage	= 'bmsg',
					
					kFileButtonPaneID						= 'bfil',
					kFileButtonValueMessage			= 'bfil',
					
					kErrorCountCaptionID				= 'cerr',
					kWarningCountCaptionID			= 'cwar',
					kMessageCountCaptionID			= 'cmsg',
					kFileCountCaptionID					= 'cfil',
					
					kClearMessagesButtonID			= 'Cler',
					kClearMessagesButtonMessage = 'Cler',
					
					kMessageWindowResID 				= 2048
				
				};
			
																		CMacCVSMessageWindow(				LStream 					*inStream);
																		~CMacCVSMessageWindow();
			
			static CMacCVSMessageWindow 	*CreateCMacCVSMessageWindowFromStream(	
																																LStream 					*inStream);

			static CMacCVSMessageWindow		*CreateCMacCVSMessageWindow(CMacCVSDoc				*inOwnerDoc,
																																const Str255 			inDescriptor,
																																const Boolean			inShow = false,
																																const Boolean 		inScrollMessages = true);
						
			virtual void 									ListenToMessage(						MessageT 					inMessage, 
																																void 							*ioParam);

			virtual Boolean 							ObeyCommand(								CommandT 					inCommand, 
																																void 							*ioParam);

			virtual void									ClearMessages();
			
			virtual void 									PostMessage(								const UInt32 			inMessageType, 
																																const char 				*inMessageText, 
																																const UInt32 			inMessageLen,
																																const void				*inMessageData,
																																const UInt32			inDataSize);
																																
		 	virtual UInt32								GetMessagesCount();
		 	
		 	virtual void									SetScrollToNewMessages(	Boolean					inScroll);
		 	
		 	virtual void									ShowMessageWindow();

		protected:
		
			virtual void 									FinishCreateSelf();

			LPushButton						*mClearMessagesButton;
				
			LBevelButton					*mErrorButton;
			LBevelButton					*mWarningButton;
			LBevelButton					*mMessageButton;
			LBevelButton					*mFileButton;
			
			LCaption							*mErrorCountCaption;
			LCaption							*mWarningCountCaption;
			LCaption							*mMessageCountCaption;
			LCaption							*mFileCountCaption;
			
			CMacCVSMessageView		*mMessageView;

			UInt32								mErrorItemsCount;
			UInt32								mWarningItemsCount;
			UInt32								mMessageItemsCount;
			UInt32								mFileItemsCount;

	};
