//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSProjectView.h
	
	Authors:		Dan Clifford and Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code for the neato-keen Finder-like display.
		 
	Modification History:

		10 Jan 1998
		
					New gray scale look.

-------------------------------------------------------------------------------- */


#pragma once


#include <Threads.h>
#include <LComparator.h>
#include <LHierarchyTable.h>
#include <LArray.h>

#include "CMacCVSProjectViewItem.h"
#include "CMacCVSProjectViewFile.h"
#include "CMacCVSProjectViewFolder.h"
#include "CMacCVSProjectViewRootFolder.h"

#include "UKeyScrollAttachments.h"

#include "CThreadedTask.h"


typedef UInt32 ProjectViewEventType;
typedef UInt32 ProjectViewSortMethod;


struct CMacCVSProjectViewEventRec 
	{

		ProjectViewEventType				fEventType;
		CMacCVSProjectViewItem			*fEventItem;
		void												*fEventData;
		
	};


typedef struct CMacCVSProjectViewEventRec CMacCVSProjectViewEventRec;


struct CMacCVSProjectViewNewViewEventRec
	{

		FSSpec	fFileSpec;
		Rect		fFromCellRect;
		
	};

struct CMacCVSProjectViewNewViewSelectFileEventRec
	{

		FSSpec	fFolderSpec;
		FSSpec	fFileToSelectSpec;
		Rect		fFromCellRect;
		
	};


typedef struct CMacCVSProjectViewNewViewEventRec CMacCVSProjectViewNewViewEventRec;


class CMacCVSProjectViewThread;
class CMacCVSProjectViewSelector;


class CMacCVSProjectView: 	

	public LHierarchyTable,
	public LBroadcaster,
	public LListener,
	public LCommander
	
	{
	
		friend class CMacCVSProjectViewThread;
		friend class CMacCVSProjectViewSelector;
		friend class UProjectKeyScrollAttachment;
		
		public :
			
			enum 
				{
				
					class_ID														= 'PrjV',

					kNameSortMessage										= 'stnm',
					kStatusSortMessage									= 'stst',
					kVersionSortMessage									= 'stvr',
					kDateSortMessage										= 'stdt',
					kTagSortMessage											= 'sttg',
					kNewViewMessage											= 'VIEW',
					kNewViewSelectFileMessage						= 'VISF',
					kDefaultYieldCount									= 16,
					kDefaultViewThreadStackSize 				= 24 * 1024,  // 24K stack space
					kViewThreadMinimumStackSize					= 4 * 1024,
					kDefaultForegroundUpdateTime				= 1 * 60, 
					kDefaultBackgroundUpdateTime				= 4 * 60,

					kViewDeathEvent											= 0,
					kViewExpandProjectFolderEvent				= 1,
					kViewExpandProjectFolderDeepEvent		= 2,
					kViewCollapseProjectFolderEvent			= 3,
					kViewCollapseProjectFolderDeepEvent	= 4,
					kViewUpdateProjectItemEvent					= 5,
					kViewUpdateProjectItemFSSpecEvent		= 6,
					kViewSelectProjectItemFSSpecEvent		= 7,
					kViewSortByNameEvent								= 8,
					kViewSortByStatusEvent							= 9,
					kViewSortByVersionEvent							= 10,
					kViewSortByDateEvent								= 11,
					kViewSortByTagEvent									= 12,

					kDefaultRowWidth										= 610,
					kDefaultRowHeight										= 20,

					kCellIconOffset											= 20, 
					kCellTextOffset											= 42,
					kFileStatusIconOffset								= 266,
					kFileLockIconOffset									= 283,
					kCellVersionTextOffset							= 300,
					kCellTagTextOffset									= 360,
					kCellModificationTextOffset					= 508,
					
					kFileStatusIconWidth								= 16,
					kFileStatusIconHeight								= 16,
					kFileLockIconWidth									= 16,
					kFileLockIconHeight									= 16,
					
					kCellChildOffset										= 15,
					kCellTextTraitsID										= 140,
					kCellTextTraitsItalicID							= 141,
					kCellTextOffsetFromBottom						= 6,
					
					kCellFolderIconID										= kGenericFolderIconResource,
					kCellFileIconID											= kGenericDocumentIconResource,
					kCellLockIconID											= 200,
					kNotSorted													= 0,
					kNameSorted													= 1,
					kStatusSorted												= 2,
					kVersionSorted											= 3,
					kTagSorted													= 4,
					kDateSorted													= 5,
	
					kProjectNoLongerExistsRow						= -1,
					kFirstFileStatusIconSuiteID					= 150
						
				};
		
																	CMacCVSProjectView(						LStream 											*inStream);
			virtual											~CMacCVSProjectView();
			
			//	Accessors
		
			void												Shutdown();
			
			virtual void								UpdateProjectItem(						const FSSpec									*inTargetItem);

			CMacCVSProjectViewFolder		*GetRootItem();
			virtual void 								SetRootItem(									FSSpec												*inAnchorFileSpec,
																																FSSpec												*inProjectRootFileSpec);

			// Selection

			virtual UInt32							CountSelectedItems(void) const;
			
			virtual void								SelectItem(										const FSSpec									*inFileSpec);
			
			virtual void								SelectionChanged();
			
			virtual void								ResizeViewColumns( 						const SInt16 									inWindowWidth );
			
			virtual LArray							*GetFlatSelection();
			virtual LArray							*GetCollapsedSelection();
			
			void												GetNthItemFSSpec(							const UInt32									intInIndex,
																																FSSpec												&fsOutItem);
			
			void												GetNamedItemFSSpec(						ConstStringPtr								pstrInName,
																																FSSpec												&fsOutItem);
			
			UInt32											GetVisibleItemCount();
																																
			virtual	void 								ExpandRow(										TableIndexT 									inWideOpenRow);
			virtual	void 								DeepExpandRow(								TableIndexT 									inWideOpenRow);
			virtual void								CollapseRow(									TableIndexT										inWideOpenRow);
			virtual void								DeepCollapseRow(							TableIndexT 									inWideOpenRow);

			//	Command handling

			virtual Boolean							CheckIfAnyItemsQualify(				CommandT											inCommand);
			
			virtual void								BeTarget();
			virtual void								DontBeTarget();
			
			virtual Boolean							ObeyCommand(									CommandT											inCommand,
																																void													*ioParam);	

			virtual void								FindCommandStatus(						CommandT 											inCommand,
																																Boolean												&outEnabled,
																																Boolean 											&outUsesMark,
																																Char16												&outMark,
																																Str255 												outName);

			virtual void 								ListenToMessage(							MessageT											inMessage,
																																void													*ioParam);

			virtual void 								SortProjectView(							ProjectViewSortMethod 				inSortMethod);
									
		protected :

			virtual	void								FinishCreateSelf(void);

			CMacCVSProjectViewItem			*ConvertTableIndexToItem(			const TableIndexT							inIndex) const;
			
			TableIndexT 								ConvertItemToTableIndex(			const CMacCVSProjectViewItem 	*inItem,
																																const TableIndexT							inStartIndex = 1) const;

			TableIndexT									GetFirstChildIndex(						const TableIndexT 						inParentWideIndex);
			
			TableIndexT									GetNextSiblingIndex(					const TableIndexT 						inWideIndex);

			virtual void								RemoveTableItems(							TableIndexT 									inParentIndex, 
																																LArray 												*inDeletedItems,
																																Boolean												inRefresh);
																															
			virtual void								UpdateFolderContents(					TableIndexT 									inWideIndex, 
																																Boolean												inRefresh);

			virtual void								RefreshEntryData( 						CMacCVSProjectViewItem 				*inItem);

			//	Drawing
	
			virtual	void								DrawSelf(void);
			
			virtual	void 								DrawCell(											const STableCell							&inCell,
																																const Rect										&inLocalRect);

			virtual	void								DrawCellIcon(									const STableCell							&inCell,
																																const Rect										&inLocalRect,
																																Boolean												inHilite);

			virtual void								HiliteCellActively(						const STableCell							&inCell,
																																Boolean												inHilite);

			virtual void								HiliteCellInactively(					const STableCell							&inCell,
																																Boolean												inHilite);

			virtual	void 								CalcCellTextFrame(						const STableCell							&inCell,
																																Rect													&outLocalFrame);

			virtual void								CalcCellHotSpots( 						const STableCell 							&inCell,
																																Rect													&outIconFrame,
																																Rect													&outTextFrame);

			virtual void								HideSelf();

			virtual void								AdaptToNewSurroundings();
			
			//  Selection
			
			virtual void								ClickSelf(										const SMouseDownEvent					&inMouseDown);
																																
			virtual void 								ClickCell(										const STableCell							&inCell,
																																const SMouseDownEvent					&inMouseDown);
			
			virtual Boolean							ClickSelect(									const STableCell							&inCell,
																																const SMouseDownEvent					&inMouseDown);

			virtual	void								GetCellIconRgn(								const STableCell							&inCell,
																																Boolean												inHilite,
																																RgnHandle											ioRgnH);
			
			virtual void								GetHiliteRgn( 								RgnHandle 										ioHiliteRgn);


			virtual void								GetLocalCellRectAnywhere( 		const STableCell							&inCell,
																																Rect													&outCellRect ) const;
			
			virtual void								MakeNewViewForCell( 					const STableCell 							&theCell );
			
			
			//	Event Handlers

			void												ProcessProjectViewEvent(			CMacCVSProjectViewEventRec 		&inEventRecord);	

			virtual Boolean							GetNextProjectViewEvent(			CMacCVSProjectViewEventRec 					&inEventRecord);
			virtual void								PostProjectViewEvent(					const CMacCVSProjectViewEventRec 		&inEventRecord);
			
			virtual void								WatchFolder(									CMacCVSProjectViewFolder			*inFolder);
			virtual void								StopWatchingFolder(						CMacCVSProjectViewFolder			*inFolder);
			virtual void								StopWatchingFolderDeep(				CMacCVSProjectViewFolder			*inFolder);
			
			virtual void								RefreshViewContents();

			virtual void								ThreadedTaskDied(							const CThreadedTask						*inTask);
										
			virtual void 								HandleUpdateEvent(						CMacCVSProjectViewItem	 			*inUpdateItem,
																																const Boolean									inShowIndicator = false);
					
			virtual void 								HandleUpdateFSSpecEvent( 			const FSSpec 									&inUpdateItem);
			
			virtual void 								HandleSelectFSSpecEvent( 			const FSSpec 									&inSelectItem);

			virtual TableIndexT 				HandleExpandEvent(						CMacCVSProjectViewItem				*inExpandItem,
																																const Boolean									inShowIndicator = false,
																																const TableIndexT 						inParentIndex = 1);
			
			virtual void								HandleExpandDeepFolder(				CMacCVSProjectViewFolder 			*inExpandItem, 
																																TableIndexT 									inParentIndex);
																		
			virtual void 								HandleExpandDeepEvent(				CMacCVSProjectViewItem				*inExpandItem);
	
			virtual TableIndexT					HandleCollapseEvent(					CMacCVSProjectViewItem				*inCollapseItem,
																																const TableIndexT							inParentIndex = 1);
			
			virtual void								HandleCollapseDeepFolder(			CMacCVSProjectViewFolder 			*inCollapseItem, 
																																TableIndexT 									inParentIndex);
																		
			virtual void								HandleCollapseDeepEvent(			CMacCVSProjectViewItem				*inCollapseItem);
			
			virtual void								RebuildSortedProjectView(			CMacCVSProjectViewFolder 			*inParentFolder, 
																																TableIndexT 									inParentIndex,
																																Boolean												inRefresh);
			
			virtual LComparator					*GetCurrentComparator();

			void												UserDraggedSelection() { LPane::sClickCount = 0; mUserDraggedSelection = true; }
			
			
			
			SInt16													mIconOffset;
			SInt16													mTextOffset;
			SInt16													mLockIconOffset;
			SInt16													mStatusIconOffset;
			SInt16													mVersionOffset;
			SInt16													mTagOffset;
			SInt16													mDateOffset;
			
			RGBColor												mBGColor;
			
			CMacCVSProjectViewThread				*mViewThread;
			CMacCVSProjectViewRootFolder		*mAnchorItem;
		
			LArray													*mViewEventQueue;
			LArray													*mWatchList;
		
			FSSpec													mProjectRootFileSpec;
		
			ProjectViewSortMethod						mCurrentSortMethod;
			Boolean													mCreateNewView;
			Boolean													mUserDraggedSelection;
			
			UInt32													mWatchTime;
									
			static UInt32										sYieldCount;
			static Handle										sFileStatusIconSuites[kMacCVSProjectorTotalCount];
			
	};


class CMacCVSProjectViewThread:

	public CThreadedTask
	
	{
	
		friend class CMacCVSProjectView;

		public:
		
										CMacCVSProjectViewThread( CMacCVSProjectView &inOwnerView);
			virtual				~CMacCVSProjectViewThread();
			
			virtual	void	Run();
			
			virtual void	KickThread() {
																				mNextUpdateTime = 0;
																	};
			
		protected:
		
			CMacCVSProjectView		*mOwnerView;
	
		private:
			
			UInt32								mNextUpdateTime;
	};
	
	
//################################################################################
#pragma mark LCOMPARATORS


class	LSortByNameComparator: 

	public LComparator 
	
	{
	
		public:
			
											LSortByNameComparator() { }
			virtual					~LSortByNameComparator() { }
					
			virtual Int32		Compare(		const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
										
			virtual Boolean	IsEqualTo(	const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
																	
			virtual	Int32		CompareToKey(		const void*	inItem,
																			Uint32			inSize,
																			const void*	inKey) const;
	};


class	LSortByStatusComparator: 

	public LComparator 
	
	{

		public:
		
											LSortByStatusComparator() { }
			virtual					~LSortByStatusComparator() { }
					
			virtual Int32		Compare(		const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
										
			virtual Boolean	IsEqualTo(	const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
																	
	};


class	LSortByDateComparator: 

	public LComparator 
	
	{
	
		public:
		
											LSortByDateComparator() { }
			virtual					~LSortByDateComparator() { }
					
			virtual Int32		Compare(		const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
										
			virtual Boolean	IsEqualTo(	const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
	};


class	LSortByVersionComparator: 

	public LComparator 
	
	{

		public:
		
											LSortByVersionComparator() { }
			virtual					~LSortByVersionComparator() { }
					
			virtual Int32		Compare(		const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
										
			virtual Boolean	IsEqualTo(	const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
																	
	};


class	LSortByTagComparator: 

	public LComparator 

	{
	
		public:

											LSortByTagComparator() { }
			virtual					~LSortByTagComparator() { }
				
			virtual Int32		Compare(		const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
										
			virtual Boolean	IsEqualTo(	const void*			inItemOne,
																	const void* 		inItemTwo,
																	Uint32					inSizeOne,
																	Uint32					inSizeTwo) const;
									
	};

//################################################################################
