//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSProjectViewItem.h
//
//	 	Authors: 	Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#pragma once


#include <Files.h>


#define FASTER_FIND_ICON			1				// cache CInfoPBRecs to make Find_icon faster

typedef SInt16 ProjectItemStatus;


enum 
	{

		kMacCVSProjectorStatusNone				= -2,
		kMacCVSProjectorStatusUnknown			= -1,
		kMacCVSProjectorStatusUnowned			= 0,
		kMacCVSProjectorStatusCheckedOut	= 1,
		kMacCVSProjectorStatusMRO					= 2,
		kMacCVSProjectorStatusCheckedIn		= 3,
		kMacCVSProjectorStatusBroken			= 4,
		kMacCVSProjectorStatusCorrupt			= 5,
		kMacCVSProjectorStatusDead				= 6,
		kMacCVSProjectorTotalCount				= kMacCVSProjectorStatusDead + 1

	};


class CMacCVSProjectViewItem 
	{ 
		
	
		public :
		
		
															CMacCVSProjectViewItem(		CMacCVSProjectViewItem			*inParentItem, 
																												const FSSpec 								*inFileSpec,
																												Boolean										inIsAlias = false);
								
			virtual									~CMacCVSProjectViewItem();
			
			//	Accessors

			virtual Boolean					CanBeAdded();
			virtual Boolean					CanEditComment();
			
			virtual Boolean					CanCheckin();
			virtual Boolean					CanCheckout();
			
			virtual Boolean					CanCompareWithOriginal();
			
			virtual Boolean					CanClearDateStickyTag();
			
			virtual Boolean					CanDiff();

			virtual Boolean					CanImport();
			virtual Boolean					CanEdit();
			virtual Boolean					CanWatch();
			virtual Boolean					CanGetLog();

			virtual Boolean					CanMRO();
			virtual Boolean					CanRevert();
			virtual Boolean					CanOrphanFiles();
			
			virtual Boolean					CanGetStatus();
			virtual Boolean					CanTag();
			virtual Boolean					CanRemove();

			virtual Boolean					CanUpdate();
			virtual Boolean					CanUpdateTo();
			
			virtual CMacCVSProjectViewItem	
															*GetParent() const;
			
			void										GetNamePStr(							Str255 											outNamePStr) const;
			void										GetNameCStr(							char 												*outNameCStr) const;
			
			virtual void						GetFSSpec(								FSSpec 											&outFileSpec) const;

#ifdef FASTER_FIND_ICON
			virtual CInfoPBRec			*GetFileInfoRecPtr() {	return mFileInfoRec;	}
#endif

			virtual void						GetCachedLocalPathPStr(		Str255 											outLocalPathPStr) const;
			virtual void						GetCachedLocalPathCStr(		char 												*outLocalPathCStr) const;
			
			virtual void						GetCachedLocalPathStr(		Handle											inOutLocalPathStrHndl) const;
																		
			virtual UInt32					GetModificationDate();
			UInt32									GetCachedModificationDate() { return mCachedModificationDate; }
			
			virtual void						GetStickyTagPStr(					Str255 											outTagPStr);
			virtual void						GetStickyTagCStr(					char 												*outTagCStr);
				
			virtual ProjectItemStatus		
															GetProjectItemStatus();

			virtual Boolean					GetVisibility();
					
			virtual LArray					*GetContainedItemsPtr();
			virtual UInt32					GetTotalContainedItemCount();
			virtual	Boolean					IsContainer() const;
			virtual	Boolean					IsCVSContainer() const;
		
			virtual void						TouchFinderItem();
			
			virtual Boolean					IsAlias();
			virtual Boolean					IsLocked();
			virtual Boolean					IsOutOfDate();
			virtual Boolean					IsVisibleToUser() { return mVisible; }
			virtual void						SetIsVisible(				Boolean inIsVisible) { mVisible = inIsVisible; }
		
			virtual	Boolean					IsChildOf(								const CMacCVSProjectViewItem 	*inItem) const;
			virtual	Boolean					IsDescendentOf(						const CMacCVSProjectViewItem 	*inItem) const;
			virtual	Boolean					IsSiblingOf(							const CMacCVSProjectViewItem 	*inItem) const;
		
			virtual Boolean					ExistsLocal();
			
			virtual void						ValidateContents( 				const FSSpec 									*inProjectRootFileSpec);
			virtual void						Invalidate();
			
			void										OpenWithFinder();

			//  Comparators

			static Int32						CompareName(				CMacCVSProjectViewItem 			*inItemOne, 
																									CMacCVSProjectViewItem 			*inItemTwo);

			static Boolean					IsEqualName(				CMacCVSProjectViewItem 			*inItemOne, 
																									CMacCVSProjectViewItem 			*inItemTwo);
															
			static Int32						CompareDate(				CMacCVSProjectViewItem 			*inItemOne, 
																									CMacCVSProjectViewItem 			*inItemTwo);

			static Boolean					IsEqualDate(				CMacCVSProjectViewItem 			*inItemOne, 
																									CMacCVSProjectViewItem 			*inItemTwo);
					
			static Int32						CompareStatus(			CMacCVSProjectViewItem 			*inItemOne, 
																									CMacCVSProjectViewItem 			*inItemTwo);

			static Boolean					IsEqualStatus(			CMacCVSProjectViewItem 			*inItemOne, 
																									CMacCVSProjectViewItem 			*inItemTwo);
			
		protected :
				
			CMacCVSProjectViewItem		*mParentItem;

#ifdef FASTER_FIND_ICON
			CInfoPBRec								*mFileInfoRec;
#endif

			FSSpec										mFileSpec;	
			UInt32										mCachedModificationDate;
			Boolean										mIsAlias;
			Boolean										mIsInvalid;
			Boolean										mVisible;
			Boolean										mInRepository;
			Boolean										mLocked;
			
			Str255										mStickyTagString;

	};


