//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSThreadedWindow.h
//	
//	  Authors: 	Simon Fraser
//
//################################################################################

#pragma once


#include "LWindow.h"
#include "CThreadedTask.h"


class CSpinIndicator;
class CMacCVSDoc;

static const UInt32		kCurrentWindowVersion = 0x0001;

enum
	{
		kWindowPosResType = 'Wpst'
	};
	
	
enum
	{
		
		kProjectViewWindowPositionResID				= 128,
		kLurkersViewWindowPositionResID,
		kMessageWindowPositionResID,
		kProgressWindowPositionResID,
		kCompareWindowPositionResID,
		kLogWindowPositionResID,
		kStatusWindowPositionResID

	};


// a class of windows with support for saving and restoring window positions
class CMacCVSWindow:

	public LWindow
	
	{
	
		friend class CMacCVSDoc;
		
		private:
		
			typedef 	LWindow		Inherited;
		
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
			typedef struct {
				UInt32						mVersion;
				Rect							mWindowFrame;
				UInt32						filler[13];			// space for later expansion
			} TWindowPos;

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

		public:
	
												CMacCVSWindow(LStream *inStream);
												CMacCVSWindow(LStream *inStream, CMacCVSDoc *inOwnerDoc);
												
			virtual						~CMacCVSWindow();
	
			virtual void			FinishCreateSelf();
			
			virtual void			SetOwnerDoc(const CMacCVSDoc *inOwnerDoc);
			CMacCVSDoc*				GetOwnerDoc() { return mOwnerDoc; }

			virtual void			SetWindowPosition();
			virtual void			SaveWindowPosition();
												
		protected:
												
							void			SafeSetWindowBounds( const Rect &inWindowBounds );
							void			ReadWindowPosResource( Rect &outWindowBounds );
							void			AddOrChangeWindowPosResource( const Rect &inWindowBounds );
							
		protected:
			
			CMacCVSDoc				*mOwnerDoc;
			
			Int16							mWindowPosResID;
			Boolean						mWindowPositionSaved;
			
	};


	
class CMacCVSThreadedWindow:
	public CMacCVSWindow
	
	{
	
	private:
		
		typedef 	CMacCVSWindow		Inherited;
		
	public:
	
												CMacCVSThreadedWindow(LStream *inStream);
												
			virtual						~CMacCVSThreadedWindow();
			
			virtual void 			FinishCreateSelf();
	
			virtual Boolean		HandleKeyPress( 			const EventRecord		&inKeyEvent );

			virtual void			ThreadedTaskDied( 		const CThreadedTask *inTask );

	protected:
			
			void							PrimeWindowTask( 			UInt32							inDelayTicks = 0);
			
			void							TerminateWindowTask();
			
			CBroadcastingThreadedTask			*mWindowTask;
			CSpinIndicator								*mSpinnyThingy;

	};