//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMultiProgressWindow.h
//
//	  Author: 	Ram Rajadhyaksha
//
//################################################################################


#pragma once

#include <LWindow.h>
#include <QDOffscreen.h>
#include <LListener.h>
#include <LBroadcaster.h>
#include <LStdControl.h>
#include <LButton.h>
#include <LOffscreenView.h>

#include "CProgressIndicator.h"
#include "CMacCVSThreadedWindow.h"


typedef struct	SProgressItemRec		SProgressItemRec;
typedef					SProgressItemRec		*SProgressItemPtr;


struct SProgressItemRec 
	{
		UInt32			fID;
		Str255			fStrMessage;
		Str255			fStrItemCount;
		Str255			fStrItemText;
		UInt32			fMaxValue;
		UInt32			fCurValue;
		UInt32			fUpdateTime;
	};
	
	
class CMultiProgressWindow: 
	
	public LWindow,
	public LBroadcaster,
	public LListener
	
	{		
		private:			
	
			typedef LWindow		Inherited;
			
		public:
	
			enum 
				{			
					class_ID 										= 'mpgw',
					kCMultiProgressWindowPPobID	= 129,
					kMaxProgressItems						= 64,
					kProgressBarSubID						= 1,
					kStopButtonSubID						= 2,
					
					kProgressBarAbsolute				= 0,
					kProgressBarIndeterminate		= 1,
					kProgressBarTimeBased				= 2,
					
					kStatusItemHeight						= 80,
					kStatusItemWidth						= 350,
					
					kMessageItemWidth						= 206,
					kMessageItemHeight					= 16,
					kMessageItemTop							= 8,
					kMessageItemLeft						= 12,
					
					kItemCountWidth							= 52,
					kItemCountHeight						= 16,
					kItemCountTop								= 8,
					kItemCountLeft							= 290,
					
					kItemTextWidth							= 330,
					kItemTextHeight							= 16,
					kItemTextTop								= 26,
					kItemTextLeft								= 12,
					
					kItemProgressWidth					= 250,
					kItemProgressHeight					= 16,
					kItemProgressTop						= 50,
					kItemProgressLeft						= 12,
					
					kStopButtonWidth						= 60,
					kStopButtonHeight						= 20,
					kStopButtonTop							= 47,
					kStopButtonLeft							= 278
				};
		
															CMultiProgressWindow(									LStream 				*inStream);
															~CMultiProgressWindow();
		
			static CMultiProgressWindow 	
														*CreateCMultiProgressWindowFromStream(	LStream 				*inStream);
		
			virtual UInt32					NewProgressIndicator();
			virtual void						DeleteProgressIndicator(							const UInt32		intInID);

			static UInt32						NewGlobalProgressIndicator();
			static void							DeleteGlobalProgressIndicator(				const UInt32		intInID);
			
			static CMultiProgressWindow	
															*GetGlobalCMultiProgressWindow();
			
			static void							DisposeGlobalCMultiProgressWindow();
			
			virtual void						SetProgressType( 											const UInt32		intInID, 
																																		const UInt32		intInType);
																																																													
			virtual void 						SetMaxValue(													const UInt32		intInID, 
																																		const UInt32 		intInMax);
																																
			
			virtual void						SetCurValue(													const UInt32		intInID, 
																																		const UInt32 		intInMax);
			
																															
			virtual void						MakeTimeBasedProgress(								const UInt32 		intInID, 
																																		const UInt32 		inExpectedTime);
																																				
			virtual void						SetProgressMessageText(								const UInt32		intInID,
																																		ConstStringPtr 	pstrInText,
																																		const	Boolean		bInRefresh = true);

			virtual void						SetCurrentItemText(										const UInt32		intInID,
																																		ConstStringPtr	pstrInText,
																																		const	Boolean		bInRefresh = true);

			virtual void						SetProgressCountText(									const UInt32		intInID,
																																		UInt32					intInValue,
																																		const	Boolean		bInRefresh = true);
			UInt32									GetProgressItemCount() { return mItemCount; }																																			
			void										DrawSelf();
			void										FinishCreateSelf();
	
			virtual void						ListenToMessage(											MessageT				inMessage,
																																		void						*ioParam);
																																								
		protected:
	
			void										DrawItem(															const UInt32		intInIndex,
																																		const UInt32		intInMode);

		 	void										DrawProgressMessageText(							const UInt32		intInIndex);
			void										DrawCurrentItemText(									const UInt32		intInIndex);
			void										DrawProgressCountText(								const UInt32		intInIndex);
																																			
			void										MigrateIndicators(										const UInt32		intInIndex);
			Boolean									GetPIDPane(														const UInt32		intInID,
																																		UInt32					&intOutIndex,
																																		SProgressItemRec &outItem);
			
		  UInt32									mNewID;
			UInt32									mItemCount;
			SProgressItemRec				**mItemsH;
			
			GWorldPtr								mMessagePixels;
			GWorldPtr								mItemTextPixels;
			GWorldPtr								mItemCountPixels;	
	
			static CMultiProgressWindow *sGlobalItem;																																	
	};
	
