//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CTagPopup.h
	
	Authors:		Scott Silver, Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code for the new tag pop-up menu.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes. Added
				  #include <LPopupButton.h> for new grayscale pop-up class. Now inherits
				  from LPopupButton instead of LStdPopupMenu.
							
-------------------------------------------------------------------------------- */


#include <LPopupButton.h>


class CTagPopup: 

	public LPopupButton 
	
	{

		public:

			virtual enum 
				{
								
					class_ID 										= 'TPOP',
					kNewTagDialogPPobID					= 142,
					kNewTagDialogOKButtonID			= 'okok',
					kNewTagDialogPopupID				= 'TAGP',
					kNewTagTEPaneID							= 'TAGE',
					kTipOfTrunkMenuItemID				= 1,
					kRevisionTagMenuItemID			= 1,
					kDateTagMenuItemID					= 2,
					kInvalidTagEnteredErr				= 400,
					kTagAlreadyExistsErr				= 401
							
				};
			

													CTagPopup(						LStream					*inStream):
													
														LPopupButton(inStream),
														mTagList( sizeof(Str255) )
														 
														{
														
															mLastItemSelected = kTipOfTrunkMenuItemID;
															
														}
								
			virtual							~CTagPopup()
														{
														
														}
		
			static CTagPopup*		CreateTagPopupStream(	LStream					*inStream) 
														{ 
														
															return new CTagPopup( inStream ); 
															
														}
			
			virtual	void				FinishCreateSelf();
			
			// loading and saving tags
			
			void 								InsertCustomTagPStr(	const Str255 		inCustomTagItemPStr);
			void								InsertCustomTagCStr(	const char 			*inCustomTagItemCStr);
			
			void								RemoveCustomTagPStr(	Str255					outCustomTagItemPStr);	
			void								RemoveCustomTagCStr(	char 						*outCustomTagItemCStr, 
																								UInt32 					&inOutBufSize);
			
			void								SetTagList(						LArray					&inTagList);
			LArray							*GetTagList();
			
			// initialization before display
			
			void								SetDefaultTag(				const Str255		inDefaultTag);
			
			// current tag manipulation

			void								GetCurrentTag(				Str255					outCurrentTag);		
			Boolean							CanRemoveCurrentTag();
			void								RemoveCurrentTag();
			
			// action procedure
			
			void								DoMouseDownInTagPopup(void);
			
		protected:
		
			LArray	mTagList;
			UInt32	mLastItemSelected;
			
	};

