//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CThreadedTask.h
//	
//	  	Authors: 	Ram Rajadhyaksha
//
//################################################################################


#pragma once


#include <Threads.h>


typedef UInt32	CThreadedTaskException;


class CThreadedTask
	
	{
	
		public:
		
			enum
				{
				
					kUnknownErr									= 800,
					kNoThreadMgrErr							= 801,
					kThreadNotFoundErr					= 802,
					kThreadClassNotInitedErr		= 803,
					kThreadProtocolErr					= 804,
					kDefaultTaskThreadStackSize	= 32 * 1024,
					kThreadDiedException				= 'DIED'
				
				};
				
														CThreadedTask( 					Boolean 						inHeavyWeightThread = false );
				
														CThreadedTask( 					const UInt32 				inStackSize,
																										Boolean 						inHeavyWeightThread = false );
		
			static void						Initialize();

			void									ShutdownThread();
			void									DeleteThread();

			void									Sleep();
			void									Resume();
			void									DelayedResume(					UInt32							inDelayTicks);
			
			static void						Yield();
			static UInt32					CountHeavyWeightThreads() {
																												return sHeavyThreadCount;
																											}
																
			virtual void					Run() = 0;
	
			UInt32						mMessageSet;
			
		protected:
		
			virtual 							~CThreadedTask();

			static pascal void 		*ThreadStart( 					CThreadedTask 			*inThreadedTask);	
			virtual void					CreateThread( 					const UInt32 				inStackSize = kDefaultTaskThreadStackSize );

			void									InsertThreadInList();
			void									RemoveThreadFromList();
			
			ThreadID					mThreadID;
			UInt32						mStartDelayTicks;
			
			Boolean						mThreadKilled;
			Boolean						mExceptionSafe;	
			Boolean						mHeavyWeightThread;			// 'heavy' means that this thread affects WNE sleep time
			
			static Boolean		sInitialized;
			static Boolean		sHasThreadMgr;
			static ThreadID		sMainThreadID;
			static Handle			sThreadList;
			static UInt32			sThreadCount;
			static UInt32			sHeavyThreadCount;
			
	};
	

// This class is used for threads that are associated with a spinner in the window,
// and broadcast to start and stop the spinner
 
class CBroadcastingThreadedTask:
	
	public CThreadedTask,
	public LBroadcaster
	
	{
	
		public:
		
												CBroadcastingThreadedTask( 		Boolean 						inHeavyWeightThread = false );
				
												CBroadcastingThreadedTask( 		const UInt32 				inStackSize,
																											Boolean 						inHeavyWeightThread = false );
			virtual void			Run();
			
	
		protected:
			
			virtual void			RunSelf() {}
				
	};
	
	
	
class CThreadedTaskYieldAttachment: 

	public LAttachment
	 	
 	{
						  
		public:
		
						CThreadedTaskYieldAttachment(	const SInt32		inQuantum = -1);
		
			void	ExecuteSelf(									MessageT				inMessage, 
																					void						*ioParam);

		protected:
		
			SInt32		mQuantum;
			UInt32		mNextTicks;

	};

