//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		UCVSUtils.h
//
//	  Authors: 	Scott M. Silver and Ram Rajadhyaksha
//
//################################################################################

#pragma once

#include <Types.h>


/*
**  Turn on 'ckid' checksuming
*/

#define __VERIFYCHECKSUM


typedef SInt32	UCVSUtilsErr;


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif


struct CkidResourceHeader
	{

		UInt32		fChecksum;
		UInt32		fLocation;
		UInt16		fCurrentVersion;
		UInt16		fCheckedOutBySomeone;
		UInt8			fBranch;
		UInt8			fModifyReadOnly;
		UInt32		fUnused;
		UInt32		fCheckoutTime;
		UInt32		fFilesModificationDate;
		UInt32		fProjectID1;
		UInt32		fProjectID2;
		UInt16		fUserID;
		UInt16		fFileID;
		UInt16		fRevisionID;
		
		//	After this follows:
		//	Path (PString), \0
		//	Username (PString), \0
		//	Revision (PString), \0
		//	FileName (PString), \0
		//	Task (PString), \0
		//	Comment Length
		//	Comment string, \0
		
	};

	
typedef struct CkidResourceHeader CkidResourceHeaderRec, *CkidResourceHeaderPtr, **CkidResourceHeaderHndl;



struct McvsResourceVer1Header
	{
	
		UInt32	fVersion;
	
	};
	

typedef struct McvsResourceVer1Header McvsResourceVer1HeaderRec, *McvsResourceVer1HeaderPtr, **McvsResourceVer1HeaderHndl;

	
struct McvsResourceHeader
	{
	
		UInt32		fVersion;
		UInt32		fFlags;
		UInt32		fRefCon;
		
		//  After this follows:
		//  Remote File Name (PString), \0
		//  Revision (PString), \0
		//  Conflicts (PString), \0
		//  Options (PString), \0
		//  Current Tag (PString), \0
		//  Unused1 (PString), \0
		//  Unused2 (PString), \0
		//  Unused3 (PString), \0
		//  Unused4 (PString), \0
		//  UInt32 flags
		//  UInt32 unused5
		//  UInt16 remotePathLen
		//  Remote Path String
		
	};
	

typedef struct McvsResourceHeader McvsResourceHeaderRec, *McvsResourceHeaderPtr, **McvsResourceHeaderHndl;


struct CkidResourcePB 
	{

		UInt32		fChecksum;
		UInt32		fLocation;
		UInt16		fCurrentVersion;
		UInt16		fCheckedOutBySomeone;
		UInt8			fBranch;
		UInt8			fModifyReadOnly;
		UInt32		fUnused;
		UInt32		fCheckoutTime;
		UInt32		fFilesModificationDate;
		UInt32		fProjectID1;
		UInt32		fProjectID2;
		UInt16		fUserID;
		UInt16		fFileID;
		UInt16		fRevisionID;
		Str255		fRemotePathPStr;
		Str255		fUserNamePStr;
		Str255		fRevisionPStr;
		Str255		fFileNamePStr;
		Str255		fTaskPStr;
		UInt16		fCommentLen;
		Handle		fCommentHndl;
		
	};


typedef struct CkidResourcePB CkidResourcePB;


struct McvsResourcePB
	{
	
		UInt32		fVersion;
		UInt32		fFlags;
		UInt32		fRefCon;
		Str255		fFileNamePStr;
		Str255		fRevisionPStr;
		Str255		fConflictsPStr;
		Str255		fOptionsPStr;
		Str255		fTagPStr;
		Str255		fUnused1PStr;
		Str255		fUnused2PStr;
		Str255		fUnused3PStr;
		Str255		fUnused4PStr;
		Ptr				fRemotePathStr;
		UInt16		fRemotePathLen;
		
	};
	
	
typedef struct McvsResourcePB McvsResourcePB;


struct CkidMcvsResourcePB
	{
	
		CkidResourcePB	ckid;
		McvsResourcePB 	mcvs;
	
	};
	

typedef struct CkidMcvsResourcePB CkidMcvsResourcePB;


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif


enum 
	{

		kMaxMacFileNameLen 					= 31,
		kSecondsInDay								= 24L * 60L * 60L
		
	};


class UCVSUtils
	{
	
		public:
			
			enum 
				{
				
					kCurrentCkidVersion 			= 4,
					kMcvsVersion1Format				= 0xFABB0002,
					kCurrentMcvsVersion				= 0xFABB0003,
					kInvalidParam 						= 500,
					kFileHasNoCkid						= 501,
					kCkidCorrupted						= 502,
					kBadCkidChecksum					= 503,
					kFileHasBeenMoved					= 504,
					kIsFolder									= 505,
					kInvalidCVSPath						= 506,
					kInvalidFileName					= 507,
					kAlreadyExists						= 508,
					kItemNotFound							= 509,
					kFileHasNoMcvs						= 511,
					kMcvsCorrupted						= 512,
					kPathOverflow							= 513,
					kUnknownMcvsVersion				= 514,
					kUnknownCkidVersion				= 515,
					kFileAppleSingleEncoded		= 1 << 0

				};
			
			static UInt32						TruncateFileNameCStr(		const char*							inSrcName,
																											char*										inDestName,
																											const UInt32						inDesiredNameLen);
			
			static UInt32 					TruncateFileNamePStr(		const Str255						inSrcName,
																											Str255									inDestName,
																											const UInt32						inDesiredNameLen);
		
			static UInt32						TruncateFileName( 			const char							*inSrcName,
																											const UInt32						inSrcNameLen,
																											char										*inDestName,
																											const UInt32						inDesiredNameLen);

			static Boolean					ExtractPhrase( 					char 										*inSrcData, 
																											UInt32 									inSrcLen, 
																											char 										inSeperator, 
																											UInt32 									inWhichPhrase, 
																											char 										*outPhrase, 
																											UInt32 									*outPhraseLen);

			static SInt32						CompareCVSVersion(			char 										*inVersion1Str, 
																											char 										*inVersion2Str);
																											
			static void							RelativeDateString(			UInt32 									printDateSeconds,
																											Str255									outDateString);
																											
			static Boolean					IsAllowableCVSName( 		StringPtr								inName);	
			
			static char*						FilterErrorHeader(			const char							*inErrorStr);
			
			static UInt32						UnixTimeToMacSeconds(		char 										*timeString);

			static OSErr						SafeDuplicate(					FSSpec									&inSrc);

			static OSErr						ReadMcvsResourceFromRef(McvsResourcePB 					*inMcvsPB,
																											const SInt16						inFileRef);

			static OSErr						ReadMcvsResource(				const FSSpec						*inTargetFile,
																											McvsResourcePB					*outMcvsPB);

			static OSErr						WriteMcvsResourceFromRef(	McvsResourcePB 				*inMcvsPB,
																											const SInt16						inFileRef);
																																															
			static OSErr						WriteMcvsResource(			const FSSpec						*inTargetFile,
																											McvsResourcePB					*inMcvsPB);
			
			static OSErr						ReadCkidResourceFromRef(CkidResourcePB 					*outCkidPB, 
																											const Boolean 					inGetComment, 
																											const SInt16 						inFileRef);
																																														
			static OSErr						ReadCkidResource(				const FSSpec						*inTargetFile,
																											CkidResourcePB					*outCkidPB,
																											const Boolean						inGetComment = true);
			
			static OSErr						WriteCkidResourceFromRef(CkidResourcePB					*inCkidPB,
																											const SInt16						inFileRef);
																																												
			static OSErr						WriteCkidResource(			const FSSpec						*inTargetFile,
																											CkidResourcePB					*inCkidPB);

			static OSErr						ReadCkidMcvsResourceFromRef( CkidMcvsResourcePB *outCkidMcvsPB, 
																											const Boolean 					inGetComment, 
																											const SInt16 						inFileRef);
																											
			static OSErr						ReadCkidMcvsResource(		const FSSpec						*inTargetFile,
																											CkidMcvsResourcePB			*outCkidMcvsPB,
																											Boolean									inGetComment = true);
			
			static OSErr						WriteCkidMcvsResourceFromRef(	CkidMcvsResourcePB *inCkidMcvsPB, 
																											const SInt16 						inFileRef);
																																												
			static OSErr						WriteCkidMcvsResource(	const FSSpec						*inTargetFile,
																											CkidMcvsResourcePB			*inCkidMcvsPB);
			
			static void							ExtractVersionAndTagFromCkidInfo( CkidResourcePB *inCkidInfo, 
																											Str255 									outVerPStr, 
																											Str255 									outTagPStr);
																													
			static pascal UInt32 		GenerateCkidChecksum( 	Handle 									inCkidData );
			
			static pascal OSErr			CreateItemFromCVSPath(	const char							*inCVSPathStr,
																											const UInt32						inPathLen,
																											const	FSSpec						*inProjectRoot,
																											FSSpec									*outTargetItem,
																											OSType									inCreator,
																											OSType									inType,
																											ScriptCode							inScriptType);
																											
			static UInt32						CountCVSPathItems(			const char						 	*inPathStr,
																											const UInt32						inPathLen);
			
			static pascal OSErr			GetCVSPathItem(					const char						  *inPathStr,
																											const UInt32						inPathLen,
																											const UInt32						inItemIndex,
																											Str255									outItemPStr);
				
			static OSErr						StripCVSPath( 					char									  *inPathStr,
																											UInt32									*inPathLen);
																																																																															
			static pascal OSErr 		CheckForMovement(				const FSSpec						*inTargetItem,
																											const FSSpec						*inProjectRoot,
																											CkidMcvsResourcePB			*inInfo);
																		
			static pascal OSErr 		FSpGetPartialPath( 			const FSSpec 						*inTargetItem, 
																											const FSSpec						*inBaseItem, 
																											UInt32 									*inOutPathLength,
								   																		char 										*outPathStr);
			
			static pascal OSErr 		FSpGetLocalCVSPath( 		const FSSpec 						*inTargetItem, 
																											const FSSpec 						*inProjectRoot, 
																											char 										*outPathStr,
																											UInt32									*inOutPathLen );
			
			static pascal OSErr			GetFSpFromLocalCVSPath( const Str255 						inCVSPathPStr,
																											const FSSpec						*inProjectRoot, 
																											FSSpec									*outTargetItem);
	
			static pascal OSErr			FSpGetDirID(						const FSSpec						*inTargetItem,
																											SInt32									*outDirID);
																											
			static pascal OSErr			FSpIsAFile(							const FSSpec 						*inTargetItem);
					
			static pascal SInt32		FSpGetModDate( 					const FSSpec				 		*inTargetItem);
				
			static pascal OSErr			FSpSetModDate( 					FSSpec 									*inTargetItem, 
																											SInt32 									inModDate);
																																														
			static pascal Boolean 	FSpIsEqual(							const FSSpec 						*inItemOne, 
																											const FSSpec						*inItemTwo);	

			static pascal Boolean		FSpIsChildOf(						const FSSpec						*inChildItem,
																											const FSSpec						*inParentItem);
																											
			static pascal Boolean   IsRepositoryDir( 				const SInt16 						inVRefNum, 
																											const SInt32 						inParID);
			
			static pascal Boolean   FSpIsRepositoryDir( 		const FSSpec						*inTargetItem);
					
			static pascal OSErr			CreateRepositoryDir(		const SInt16						inVRefNum,
																											const SInt32						inParID,
																											const OSType						inCreator,
																											const OSType						inType,
																											const ScriptCode				inScriptType);
																																																	
			static pascal OSErr			FSWriteOffset(					const SInt32						inFileRef,
																											UInt32									*inOutCount,
																											void 										*inData, 
																											const UInt32						inOffset = 0);
																											
			static pascal OSErr			FSReadOffset(						const SInt32						inFileRef,
																											UInt32									*inOutCount,
																											void 										*inData, 
																											const UInt32						inOffset = 0);

/*																											
			static pascal OSErr			FSpWriteRF(							const FSSpec 						*inTargetItem, 
																											char 										*inData, 
																											const UInt32 						inDataLen,
																											const UInt32						inOffset);	
			
			static OSErr						FSpEncodeAppleSingle( 	FSSpec 									*inTargetItem, 
																											Handle 									*outDataHndl);
*/			

			static OSErr						FSpGetRepository(				const FSSpec						*inTargetItem,
																											Str255									outRepositoryPStr);
			
			static OSErr						FSpSetRepository( 			const FSSpec 						*inTargetItem, 
																											Str255 									inRepositoryPStr);

			static OSErr						FSpGetFolderTag( 				const FSSpec 						*inTargetItem, 
																											Str255 									outTagPStr);
																											
			static OSErr						FSpGetParentTag( 				const FSSpec 						*inTargetItem, 
																											Str255 									outTagPStr);

			static OSErr						FSpSetParentTag( 				const FSSpec 						*inTargetItem, 
																											Str255 									inTagPStr);
			
			static OSErr						FSpGetFullPathStr(			const FSSpec						*inTargetItem,
																											UInt32									*inOutPathLen,
																											char										*outPathStr);
																																																																																																																															
			static pascal void 			Rot15PStr( 							Str255 									inOutCryptStr );
			static pascal void 			ReverseRot15PStr( 			Str255 									inOutCryptStr );

			static void							UInt32ToStr255(					UInt32									inNum,
																											Str255									outStr);
																											
			static void							UInt32ToCStr(						UInt32 									inNum, 
																											char 										*outStr, 
																											UInt32 									*inOutLen);		
																											
			static void 						CStrToUInt32(						register char						*inStr,
																											register UInt32					*outNum);

			static char 						ReadWord( 							const char 							*inBuf, 
																											const UInt32 						inBufLen, 
																											UInt32 									*inOutIndex, 
																											char 										*outWordStr, 
																											UInt32 									*inOutLen, 
																											const char 							*inTerm = " \n");
																																																																	
		private:

			static void							UInt32ToCStrAtomic( 		char										*outStr,
																											UInt32									inNum,
																											UInt32									*inOutIndex,
																											const UInt32						inLen);
								

			static SInt32 sSecondsFromGMT;
			
	};

