//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				UProjectActions.h
	
	Authors:		Ram Rajadhyaksha and others (unknown)
	
	Description:
	
		This file contains code to implement operations from the
		menu commands.
		 
	Modification History:

		21 Apr 1999		Josef W. Wankerl
		
					Added Reveal In Finder support.
		
		10 Jan 1998			Ram Rajadhyaksha
		
					Made several changes for compatiblity with new Entries file scheme.
							
-------------------------------------------------------------------------------- */


#pragma once


#include <LArray.h>
#include <LAction.h>

#include "CVSEngine.h"
#include "CCVSEntries.h"
#include "CMacCVSOrphanFilesTask.h"


enum
	{
		kDirectorySpinnerStreamID		= 'sTRm'
	};


struct SDirectorySpinnerStreamData
	{
		OSType				dataID;
		CMacCVSDoc		*ownerDoc;
		LArray				*itemsArray;
	};


enum 
	{
		STRx_AddHelpCaptions				= 130,
		str_NonBinaryText						= 1,
		str_BinaryText							= 2,
		str_AppleSingleText					= 3				
	};


enum
	{	
		kOpenFilesListDialogID					= 156,
		
		kOpenFilesListOKButtonID				= 'okok',
		kOpenFilesListCancelButtonID 		= 'cncl',
		kOpenFilesListTableID						= 'LIST'		
	};

	
enum
	{
		kDiffOptionsDialogID								= 157,
		
		kDiffOptionsOKButtonID							= 'okok',
		kDiffOptionsCancelButttonID					= 'cncl',
		
		kDiffOptionsWhiteSpaceRadioGroupID	= 'Wspg',
		kDiffOptionsDiffWhiteSpaceRadio			= 'Dwsp',
		kDiffOptionsIgnoreWhiteSpaceRadio		= 'Iwsp',
		kDiffOptionsIgnoreSpaceChangeRadio	= 'Ispc',
		kDiffOptionsIgnoreBlankLinesRadio		= 'Ibll',
		
		kDiffOptionsIgnoreCaseCheckboxID		= 'igcs',
		kDiffOptionsShowFunctionCheckboxID	= 'sbsd',
		kDiffOptionsUseForCompareCheckboxID = 'Ucmp',
		kDiffOptionsUseForDiffCheckboxID		=	'Udif',
		
		kDiffOptionsRegularDiffRadioID			= 'Rdif',
		kDiffOptionsRCSDiffRadioID					= 'rcsd',
		kDiffOptionsContextDiffRadioID			= 'cntx',
		kDiffOptionsUnifiedDiffRadioID			= 'unif',
		kDiffOptionsContextLinesEditTextID	= 'lctx',
		
		kDiffOptionsUseRevsCheckboxGroupID	= 'Revs',
		kDiffOptionsFromRevEditTextID				= 'Ftxt',
		kDiffOptionsToRevEditTextID					= 'Ttxt',
				
		kDiffOptionsCommandLineLabel				= 'Optn'	
	};
	
	
enum
	{
		msg_CancelAll = 10090	
	};
	
	
class CMacCVSDoc;


class CProjectAction: 

	public LAction
	
	{
		
		public:
		
			enum
				{				
					kProjectActionCancelled	= 900					
				};
												
												CProjectAction(								CMacCVSDoc			*inOwnerDoc,
																											LArray					*inItemList = NULL,
																											ResIDT					inStringResID = STRx_RedoEdit,
																											Int16						inStringIndex = str_RedoUndo);							
			virtual						~CProjectAction();
			virtual Boolean		IsPostable(void) const;

			//  Suspends the event. Resumes it when command is complete
			
			virtual void			SetCompletionEvent(						AppleEvent 			currentEvent, 
																											AppleEvent 			replyEvent);

		protected:

			virtual	Boolean		IsItemQualified(							const FSSpec		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL) = 0;
			virtual	Boolean		CheckAllItemsQualify( 				Boolean 				inWarnOfOpenFiles = false);
			
			virtual void			UndoSelf(void);

			CMacCVSDoc			*mOwnerDoc;
			LArray					mItemList;

			CCVSEntries			mEntries;
			
			AppleEvent			mCurrentEvent;
			AppleEvent			mReplyEvent;	//  If action needs to deliver an AE reply, it goes here
			
	};


// JWW - Added Reveal In Finder action
class CProjectRevealAction : public CProjectAction
	{
	
		public:								
												CProjectRevealAction(					CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
	};


class CProjectCheckoutDefaultAction : public CProjectAction
	{
	
		public:							
												CProjectCheckoutDefaultAction(CMacCVSDoc 			*inOwnerDoc);										
												CProjectCheckoutDefaultAction(		
																											CMacCVSDoc			*inOwnerDoc,
																											CCVSEngineCheckoutOptions		
																																			*inCheckoutOptions);												
												~CProjectCheckoutDefaultAction();						

		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
			CCVSEngineCheckoutOptions		mCustomOptions;
			Boolean											mUseCustomOptions;
				
	};


class CProjectCheckoutModuleToAction : public CProjectAction
	{
	
		public:							
												CProjectCheckoutModuleToAction(		
																											CMacCVSDoc 			*inOwnerDoc);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			Boolean						GetDesiredModule(							Str255					pstrOutModule,
																											Str255					pstrOutTag);
	};


class CProjectAddAction : public CProjectAction
	{
	
		public:							
												CProjectAddAction(						CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											Boolean					inAskOptions = false);	
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);		
			Boolean 					GetEncodingPreference( 				const FSSpec		&fsInItem,
																											CCVSEngineFileMappingRec		
																																			&sOutMapping,
																											Boolean					&bOutSkipThisFile);	
			
			Boolean 	mAskForEncoding;
				
	};


class CProjectCommitAction : public CProjectAction
	{
	
		public:								
												CProjectCommitAction(					CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const Boolean		bInAsk = true);
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
			FSSpec		mLocalRoot;
			Boolean		mAskUser;
	};


class CProjectCompareAction : public CProjectAction
	{
	
		public:							
												CProjectCompareAction(				CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);	
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
			FSSpec		mLocalRoot;
			
	};


class CProjectDiffAction : public CProjectAction
	{
	
		public:
								
												CProjectDiffAction(						CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											Boolean					inShowAdvancedDialog = false);
		
			static Boolean		GetDiffOptions(								CMacCVSDoc 			*inOwnerDoc,
																											CCVSEngineDiffOptions 					
																																			&ioDiffOptions,
																											Boolean					inUseDefaults = true,
																											Boolean 				inLockRevisions = false);

		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
						
			FSSpec		mLocalRoot;		
			Boolean		mShowAdvancedDialog;
			
	};



class CProjectEditAction : public CProjectAction
	{
	
		public:								
												CProjectEditAction(						CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const Boolean		inEdit = true);		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			Boolean						WarnUser();
			
			Boolean	mEdit;
						
	};


class CProjectGetFileInfoAction : public CProjectAction
	{
	
		public:								
												CProjectGetFileInfoAction(		CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
			FSSpec		mLocalRoot;
			
	};


class CProjectGetFileStatusAction : public CProjectAction
	{
	
		public:								
												CProjectGetFileStatusAction(	CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
			FSSpec		mLocalRoot;
			
	};


class CProjectShowEditorsWatchersAction : public CProjectAction
	{	
	
		public:										
												CProjectShowEditorsWatchersAction(		
																											CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const Boolean		inShowEditors = true);	
		protected:
			
			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL);
			virtual void			RedoSelf(void);
		
			Boolean			bShowEditors;	
			FSSpec			mLocalRoot;		
						
	};


class CProjectImportAction : public CProjectAction
	{
	
		public:
								
												CProjectImportAction(					CMacCVSDoc 			*inOwnerDoc);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			virtual Boolean		GetImportOptions(							FSSpec					&fsOutSpec,
																											Str255					pstrOutRepository,
																											Str255					pstrOutVendorTag,
																											Str255					pstrOutReleaseTag,
																											Str255					pstrOutBranch);
	};


class CProjectRemoveAction : public CProjectAction
	{
	
		public:
								
												CProjectRemoveAction(					CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			Boolean						WarnUser();
			
			FSSpec			mLocalRoot;
		
	};


class CProjectRevertToAction : public CProjectAction
	{
	
		public:								
												CProjectRevertToAction(				CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const Boolean		inAskUser);		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			Boolean						WarnUser();
			
			FSSpec			mLocalRoot;
			Boolean			mAskUser;
						
	};


class CProjectTagAction : public CProjectAction
	{
	
		public:
								
												CProjectTagAction(						CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);

												CProjectTagAction(						CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											Str255					inTagString,
																											Boolean					inTagIsStatic,
																											Boolean					inRemoveExistingTags);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
		 	Boolean						GetDesiredTag(								Str255					pstrOutTag,
																											Boolean					&bOutRemove,
																											Boolean					&bOutStaticTag);
			Boolean						WarnUserOfImpendingDoom(			ConstStringPtr 	pstrInTag);
																										
			FSSpec			mLocalRoot;
			Boolean			mHaveData;
			Str255			mTagPStr;
			Boolean			mTagIsStatic;
			Boolean			mRemoveExistingTag;
	};


class CProjectUpdateAction : public CProjectAction
	{
	
		public:								
												CProjectUpdateAction(					CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const Boolean		inIsCheckout);
																											
												CProjectUpdateAction(					CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const Boolean		inIsCheckout,
																											const CCVSEngineUpdateOptions	
																																			*inOptions);											
												~CProjectUpdateAction();
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL);
			virtual void			RedoSelf(void);
			
			FSSpec										mLocalRoot;
			
			CCVSEngineUpdateOptions		mCustomOptions;
			Boolean										mIsCheckout;
			Boolean										mUseCustomOptions;
			
	};


class CProjectUpdateToAction : public CProjectAction
	{
	
		public:
								
												CProjectUpdateToAction(				CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const Boolean		inIsCheckout = false);												
												~CProjectUpdateToAction();
				
			Boolean						GetDesiredRevision(						const Boolean		inIsCheckout,
																											const Boolean		inMultipleItems,
																											Str255					pstrOutTag);
											
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
			FSSpec			mLocalRoot;
			Boolean			mIsCheckout;
			
	};


class CProjectWatchAction : public CProjectAction
	{
	
		public:
								
												CProjectWatchAction(					CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											const UInt32 		intInAction,
																											const CCVSEngineWatchOptions *inOptions = NULL);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
				
			UInt32 									mAction;
			CCVSEngineWatchOptions	mOptions;					
	};


class CProjectEditCommentAction : public CProjectAction
	{
	
		public:								
												CProjectEditCommentAction(		CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL );
			virtual void			RedoSelf(void);
			
			FSSpec		mLocalRoot;
		
	};


class CProjectOrphanFilesAction : public CProjectAction
	{
	
		public:								
												CProjectOrphanFilesAction(		CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList,
																											CMacCVSOrphanFilesQueryRec	
																																			*inQuery);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL);
			virtual void			RedoSelf(void);
			
			FSSpec												mLocalRoot;			
			CMacCVSOrphanFilesQueryRec		mQueryRec;
		
	};


class CProjectMROAction : public CProjectAction
	{
	
		public:
								
												CProjectMROAction(						CMacCVSDoc 			*inOwnerDoc,
																											LArray					*inItemList);
		
		protected:

			virtual	Boolean		IsItemQualified( 							const FSSpec 		&inTargetItem,
																											Boolean					*outFileWasOpen = NULL);
			virtual void			RedoSelf(void);
			
			FSSpec		mLocalRoot;
		
	};


Boolean EditCheckoutComment( 	const FSSpec 	*inTargetItem, 
															CMacCVSDoc 		*inOwnerDoc,
															const Boolean inAskAlways, 
															const Boolean inIsCommit, 
															Boolean 			&cancelAll);
															
