//################################################################################
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				IDEMacCVS.h
	
	Authors:		Ram Rajadhyaksha and others
	
	Description:
	
		This file contains the plug-in code for the CodeWarrior IDE CVS plug-in.

-------------------------------------------------------------------------------- */


#ifndef __IDEMACCVS__
#define __IDEMACCVS__

#include "DropInVCS.h" 


enum
	{
		kDefaultSocketTimeout	= 30
	};
	

typedef struct
	{
		CWPluginContext	fCWContext;
	} IDEMacCVSContextRec;
	
	
Boolean 	CheckCommandSupportedMatrix(
																	CWVCSCommand 							request, 
																	Boolean 									advanced, 
																	Boolean										recursive);

Boolean 	ValidateDatabasePath(		CWVCSDatabaseConnection 	*theConnection);
Boolean 	ValidateLocalRootPath(	CWVCSDatabaseConnection 	*theConnection);
Boolean 	ValidateUserAndPassword(CWVCSDatabaseConnection 	*theConnection);

OSErr			ParseCVSRootStr(			 	char											*strInRoot,
																	Str255										pstrOutUserName,
																	Str255										pstrOutServerName,
																	Str255										pstrOutRoot,
																	AuthenticationMethod			*intOutMethod,
																	UInt16										*intOutPort);

void			file_add( 							CWPluginContext		 				parameterBlock);	// add
void			file_checkin( 					CWPluginContext					 	parameterBlock);	// checkin
void			file_checkout( 					CWPluginContext					 	parameterBlock);	// checkout
void			file_comment( 					CWPluginContext 					parameterBlock);	// comment
void			file_delete( 						CWPluginContext 					parameterBlock);	// delete
void			file_destroy( 					CWPluginContext 					parameterBlock);	// destroy
void			file_difference( 				CWPluginContext 					parameterBlock);	// difference
void			file_get( 							CWPluginContext 					parameterBlock);	// get
void			file_history( 					CWPluginContext 					parameterBlock);	// histroy
void			file_label( 						CWPluginContext 					parameterBlock);	// label
void			file_properties( 				CWPluginContext 					parameterBlock);	// properties
void			file_purge( 						CWPluginContext 					parameterBlock);	// purge
void			file_rename( 						CWPluginContext 					parameterBlock);	// rename
void			file_rollback(					CWPluginContext 					parameterBlock);	// rollback
void			file_status( 						CWPluginContext 					parameterBlock);	// status
void			file_undo_checkout(		  CWPluginContext 					parameterBlock);	// undo checkout
void			file_version( 					CWPluginContext 					parameterBlock);	// version

void			database_connect( 			CWPluginContext 					parameterBlock);	// database connect
void			database_disconnect(		CWPluginContext			 			parameterBlock);	// database disconnect
void			database_variables(			CWPluginContext 					parameterBlock);	// database variables

CCVSEngineRef			
					database_startup( void );

CWResult	VCS_initialization( 		CWPluginContext 					parameterBlock);	// VCS initialization
CWResult	VCS_termination( 				CWPluginContext 					parameterBlock);	// VCS termination
void			VCS_about( 							CWPluginContext 					parameterBlock);	// VCS about


pascal SInt32	
					EventNotifier(					const CCVSEngineRef	 			inEngineRef,
																	const FSSpec							*inTargetItem,
																	const SInt32 							inEvent, 
																	const SInt32 							inError, 
																	const UInt32 							inCookie, 
																	const void 								*inDataPtr, 
																	void 											*inContextPtr);

pascal SInt32 
					MessageNotifier(				const CCVSEngineRef 			inEngineRef, 
																	const UInt32							inDataType,
																	const UInt32							inDataLen, 
																	const void 								*inDataPtr, 
																	void 											*inContextPtr);

pascal SInt32 
					ErrorNotifier(					const CCVSEngineRef 			inEngineRef, 
																	const UInt32							inDataType,
																	const UInt32							inDataLen, 
																	const void 								*inDataPtr, 
																	void 											*inContextPtr);

void			InitializePowerPlant( void );
void 			FinalizePowerPlant( void );

DialogPtr	GetPowerPlantDialog(		const Int16								inPPobID);
void			ModalPowerPlantDialog( 	const DialogPtr 					pInDialog );

void			ShowPluginDialog(				const DialogPtr						pDialog,
																	CWPluginContext 					parameterBlock);
																	
void			HidePluginDialog(				const DialogPtr						pDialog,
																	CWPluginContext 					parameterBlock);

#endif

