//################################################################################
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				IDEMacCVS.cp
	
	Authors:		Ram Rajadhyaksha and others
	
	Description:
	
		This file contains the plug-in code for the CodeWarrior IDE CVS plug-in.

-------------------------------------------------------------------------------- */


#ifndef TRUE
#define TRUE 	1
#endif

#ifndef FALSE
#define FALSE	0
#endif


void Assert_( Boolean inCase )
	{
		if ( inCase == false )
			{
				::DebugStr("\pAssert!" );			
			}
	}
	
	
#include <Files.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <OpenTransport.h>
#include <Windows.h>
#include <Dialogs.h>

#include <PP_Types.h>
#include <PP_Messages.h>
#include <UEnvironment.h>
#include <UException.h>
#include <UDrawingState.h>
#include <URegistrar.h>
#include <LPeriodical.h>
#include <LComparator.h>
#include <LGrafPortView.h>
#include <LCaption.h>
#include <LEditField.h>
#include <LGroupBox.h>
#include <LStdControl.h>
#include <LPicture.h>
#include <LRadioGroupView.h>

#include "DropInVCS.h" 

#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "CVSEngine.h"
#include "CVSEngineUtils.h"
#include "NewWithFree.h"
#include "FreeAllMallocMemory.h"
#include "IDEMacCVSUtils.h"

#include "IDEMacCVS.h"

// error checking

#define CHECKERR(pb, x) if((x)!=cwNoErr) {\
	CWSetCommandStatus((pb), x);\
	return((x));\
}


#define ThrowCWResult(err)		  \
	do {													\
		CWResult	__theErr = err;		\
		if (__theErr != cwNoErr) {	\
			throw(__theErr);					\
		}														\
	} while (false)


Str255 								gUserName			= "\p";
Str255 								gPassword			= "\p";
Str255 								gServerName		= "\p";
Str255 								gServerRoot		= "\p";
UInt16 								gServerPort		= 0;
AuthenticationMethod	gAuthMethod 	= 0;
FSSpec								gFSRoot				= { 0 };
IDEMacCVSContextRec		*gContext			= NULL;


#if CW_USE_PRAGMA_EXPORT
#pragma export on
#endif


CWPLUGIN_ENTRY(CWPlugin_GetDropInFlags)(const DropInFlags **flags, long *flagsSize)
	{
		static const DropInFlags sFlags = {
			3,
			CWDROPINVCSTYPE,
			VCS_API_CURRENT_VERSION,
			vcsDoesntPrePostSingleFile | vcsDoesntPrePostRecursive,
			0,
			VCS_API_CURRENT_VERSION
		};


		*flags = &sFlags;
		*flagsSize=sizeof(sFlags);

		return cwNoErr;
	}


CWPLUGIN_ENTRY(CWPlugin_GetDropInName)(const char ** dropinName)
	{
		static const char *sDropInName = "IDE MacCVS Pro";

		*dropinName=sDropInName;

		return cwNoErr;
	}


CWPLUGIN_ENTRY(CWPlugin_GetPanelList)(const CWPanelList** panelList)

{
	// +++Turn this on when the sample panel has been converted!

	static const char* sPanelName = "Sample Pref";
	static CWPanelList sPanelList = {kCurrentCWPanelListVersion, 1, &sPanelName};

	*panelList = &sPanelList;

	return cwNoErr;
}


#if CW_USE_PRAGMA_EXPORT
#pragma export off
#endif


/*--------------------------------------------------------------------------------------
 * matrix to hold values to indicate if commands are supported by the plugin.
 *
 * Set any entry to FALSE if your plugin does not allow it.  Entries already set to FALSE
 * will never be sent to the plugin.
 *---------------------------------------------------------------------------------------*/


struct sRoutineSupportMatrix
{
	CWVCSCommand	command;							//  VCS command

	Boolean				bDefault;							//  default mode
	Boolean				bAdvanced;						//  advanced mode
	Boolean				bRecursive;						//  recursive mode
	Boolean				bAdvancedRecursive;		//  advanced recursive mode
}

	sRoutineSupportMatrix [] =
{
	{ reqPrefsChange,				TRUE,		FALSE,	FALSE,	FALSE	},
	{ reqIdle,							TRUE,		FALSE,	FALSE,	FALSE	},
	{ reqInitialize,				TRUE,		FALSE,	FALSE,	FALSE	},
	{ reqTerminate,					TRUE,		FALSE,	FALSE,	FALSE	},

	{ reqAbout,							TRUE,		FALSE,	FALSE,	FALSE	},

	{ reqDatabaseConnect,		FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqDatabaseDisconnect,FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqDatabaseVariables,	FALSE,	FALSE,	FALSE,	FALSE	},

	{ reqFileAdd,						TRUE,		TRUE,		FALSE,	FALSE	},
	{ reqFileBranch,				FALSE,	FALSE,	FALSE,	FALSE	},  //  maybe we could make this tag?
	{ reqFileCheckin,				TRUE,		TRUE,		FALSE,	FALSE	},
	{ reqFileCheckout,			TRUE,		TRUE,		FALSE,	FALSE	},
	{ reqFileComment,				FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileDelete,				FALSE,	FALSE,	FALSE,	FALSE	},  //  this is probably too dangerous to support
	{ reqFileDestroy,				FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileDifference,		FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileGet,						TRUE,		TRUE,		FALSE,	FALSE	},
	{ reqFileHistory,				TRUE,	  FALSE,	FALSE,	FALSE	},
	{ reqFileLabel,					FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileProperties,		FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFilePurge,					FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileRename,				FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileRollback,			TRUE,		FALSE,	FALSE,	FALSE	},
	{ reqFileShare,					FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileStatus,				TRUE,		FALSE,	FALSE,	FALSE	},
	{ reqFileUndoCheckout,	FALSE,	FALSE,	FALSE,	FALSE	},
	{ reqFileVersion,				TRUE,		FALSE,	FALSE,	FALSE	},
	{ reqFileView,					FALSE,	FALSE,	FALSE,	FALSE	}
};


#define kNumberRoutineSupportEntries	28

// The following funciton is used in the negotiation phase to check in the matrix to see if the
// commadn is supported.


Boolean 
CheckCommandSupportedMatrix( CWVCSCommand request, Boolean advanced, Boolean recursive )
	{
		int i;

		for(i=0; i<kNumberRoutineSupportEntries; i++)
			{
				if(sRoutineSupportMatrix[i].command==request)
					{
						if(advanced && recursive)
							return(sRoutineSupportMatrix[i].bAdvancedRecursive);
						else if(advanced)
							return(sRoutineSupportMatrix[i].bAdvanced);
						else if(recursive)
							return(sRoutineSupportMatrix[i].bRecursive);
						else
							return(sRoutineSupportMatrix[i].bDefault);
					}
			}

		return(false);
	}


Boolean RequestRequiresDatabase( long request )
	{
		//  RequestRequiresDatabase() is a routine that takes in a 
		//  VCS request block and returns a Boolean indicating if it 
		//  requires a valid database or not

		//  Since the concept of using referring to a "database" with
		//  a Mac FSSpec is completely lame, we always ignore this.
		
		Boolean needsDatabase = false;

		switch(request)
			{
				case reqDatabaseConnect:
				case reqDatabaseDisconnect:
				case reqDatabaseVariables:
				case reqFileAdd:
				case reqFileCheckin:
				case reqFileCheckout:
				case reqFileComment:
				case reqFileDelete:
				case reqFileDestroy:
				case reqFileDifference:
				case reqFileGet:
				case reqFileHistory:
				case reqFileLabel:
				case reqFileProperties:
				case reqFilePurge:
				case reqFileRename:
				case reqFileRollback:
				case reqFileStatus:
				case reqFileUndoCheckout:
				case reqFileVersion:
				case reqFileBranch:
				case reqFileShare:
				case reqFileView:		
					needsDatabase = true;			
				break;

				default:
					needsDatabase = false;
				break;
			}

		return needsDatabase;
	}


Boolean ValidateDatabasePath( CWVCSDatabaseConnection *theConnection )
	{
		//  ValidateDatabasePath() is a funciton that takes a pointer 
		//  to a database path and checks to make sure it is valid.

		//  Since the "database path" in the VCS API is a file specifier,
		//  we just ignore it.
		
		return true;
	}


Boolean
ValidateLocalRootPath( CWVCSDatabaseConnection *theConnection )
	{
		//  ValidateLocalRootPath() is a function that takes a 
		//  pointer to a database connection structure and
		//  makes sure the local root is valid:

		Boolean bResult = false;

		if ( theConnection == NULL )
			{
				ThrowOSErr( kCVSEngineBadParamErr );
			}
		
		if ( FSpIsAFile(&theConnection->sProjectRoot, NULL) == kCVSEngineIsFolder )
			{	
				gFSRoot = theConnection->sProjectRoot;
				bResult = true;
			}
			
		return bResult;
	}


Boolean 
ValidateUserAndPassword( CWVCSDatabaseConnection *theConnection )
	{
		//  ValidateUserAndPassword() is a function that takes a 
		//  pointer to a database connection structure and checks 
		//  to make sure that a username and password are specified

		OSErr		result 			= noErr;
		Boolean toReturn 		= false;

		UInt32	intCopyLen 	= 0;
		UInt32	intLen			= 0;

		if ( (theConnection == NULL) || (theConnection->pUsername == NULL) )
			{
				ThrowOSErr( kCVSEngineBadParamErr );
			}
			
		if ( strlen(theConnection->pUsername) > 0 )
			{
				result = ParseCVSRootStr( theConnection->pUsername, gUserName, gServerName, gServerRoot, &gAuthMethod, &gServerPort );

				if ( result == noErr )
					{
						if ( gAuthMethod == kAuthenticationPassword )
							{
								intLen = strlen( theConnection->pPassword );

								if ( intLen > 0 )
									{
										intCopyLen = intLen;

										if ( intCopyLen > 255 )
											{
												intCopyLen = 255;
											}

										gPassword[0] = intCopyLen;
										::BlockMoveData( theConnection->pPassword, &gPassword[1], gPassword[0] );

										toReturn = true;
									}
							}
						else
							{
								toReturn = true;
							}				
					}
			}

		return toReturn;
	}


OSErr
ParseCVSRootStr( char *strInRoot, Str255 pstrOutUserName, Str255 pstrOutServerName, Str255 pstrOutRoot, AuthenticationMethod *intOutMethod, UInt16 *intOutPort )
	{
		OSErr		result 			= noErr;
		UInt32	intLen			= 0;
		UInt32	intCopyLen	= 0;

		char		*pLineTerm 	= NULL;
		char 		*pCurrent		= NULL;
		Str255	pstrTemp 		= "\p";

		try
			{
				if ( 	(strInRoot == NULL) || 
							(pstrOutUserName == NULL) || 
							(pstrOutServerName == NULL) || 
							(pstrOutRoot == NULL) ||
				     	(intOutMethod == NULL) || 
				     	(intOutPort == NULL) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
			
				intLen = strlen( strInRoot );
				
				if ( intLen == 0 )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
					
				pCurrent = strInRoot;
				
				if ( *pCurrent == ':' )
					{
						//  If the string starts with a ':', then it must
						//  include the connection method.
						
						pCurrent++;						
						pLineTerm = strpbrk( pCurrent, ":" );
		
						if ( pLineTerm == NULL )
							{
								ThrowOSErr( kCVSEngineBadParamErr );
							}
							
						//  Get the authentication method
						
						if ( (pLineTerm - pCurrent) <= 255 )
							{
								pstrTemp[0] = pLineTerm - pCurrent;
								::BlockMoveData( pCurrent, &pstrTemp[1], pstrTemp[0] );
								
								//  um, duh.
								PStrLowercase( pstrTemp );
								
								if ( PStrCompare(pstrTemp, "\pserver") )
									{
										*intOutMethod = kAuthenticationRSH;
									}
								else if ( PStrCompare(pstrTemp, "\ppserver") )
									{
										*intOutMethod = kAuthenticationPassword;
									}
								else
									{
										*intOutMethod = 0;
									}
							}
	
						pCurrent = pLineTerm + 1;
					}
				else
					{
						*intOutMethod = kAuthenticationPassword;
					}
						
				//  Get the user name
				
				pLineTerm = strpbrk( pCurrent, "@:" );
				
				if ( pLineTerm == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}

				if ( *pLineTerm	== '@' )
					{
						intCopyLen = pLineTerm - pCurrent;
						
						if ( intCopyLen > 255 )
							{
								intCopyLen = 255;
							}
							
						pstrOutUserName[0] = intCopyLen;
						::BlockMoveData( pCurrent, &pstrOutUserName[1], pstrOutUserName[0] );
						
						pCurrent = pLineTerm + 1;

						pLineTerm = strpbrk( pCurrent, ":" );
				
						if ( pLineTerm == NULL )
							{
								ThrowOSErr( kCVSEngineBadParamErr );
							}								
					}	
					
				//  Get the server name
				
				intCopyLen = pLineTerm - pCurrent;
				
				if ( intCopyLen > 255 )
					{
						intCopyLen = 255;
					}

				pstrOutServerName[0] = intCopyLen;
				::BlockMoveData( pCurrent, &pstrOutServerName[1], pstrOutServerName[0] );
				
				pCurrent = pLineTerm + 1;

				//  I guess we should check for a port
				//  number specifier too...
						
				*intOutPort = 0;
		
				//  The rest is the root path
				
				intCopyLen = ( intLen - (pCurrent - strInRoot) );
				
				if ( intCopyLen > 0 )
					{
						if ( intCopyLen > 255 )
							{
								intCopyLen = 255;
							}
												
						pstrOutRoot[0] = intCopyLen;
						::BlockMoveData( pCurrent, &pstrOutRoot[1], pstrOutRoot[0] );		
					}
			}			
		catch ( OSErr intException )
			{
				result = intException;
			}
		catch ( ... )
			{
				result = kCVSEngineUnknownExceptionErr;
			}
		
		return result;
			
	}
	
	
#pragma mark -


pascal short 
main( CWPluginContext parameterBlock )
	{	
		
		CWResult 	theErr				= cwNoErr;

		Boolean 	isInV1Mode 		= false;
		Boolean 	bNegotiation	= false;
		Boolean		bAdvanced			= false;
		Boolean	  bRecursive		= false;

		short			intResult			= 0;
		SInt32		intRequest		= 0;
	
		try
			{
				//  perform v1/v2 negotiation
				theErr = CWAllowV1Compatibility( parameterBlock, true, &isInV1Mode );
				ThrowCWResult( theErr );
			
				//  get the request
				theErr = CWGetPluginRequest( parameterBlock, &intRequest );
				ThrowCWResult( theErr );

				if ( (intRequest == reqInitialize) || (intRequest == reqTerminate) )
					{
						goto MAIN_SWITCH;
					}
					
				//  set the command status
				theErr = CWSetCommandStatus( parameterBlock, cwCommandStatusUnknown );
				ThrowCWResult( theErr );

				//  get the variant flags
				theErr = CWIsCommandSupportedRequest( parameterBlock, &bNegotiation );
				ThrowCWResult( theErr );

				theErr = CWIsAdvancedRequest( parameterBlock, &bAdvanced );
				ThrowCWResult( theErr );

				theErr = CWIsRecursiveRequest( parameterBlock, &bRecursive );
				ThrowCWResult( theErr );

				//  get a a connection structure
				
				//  for some reason, this sometimes returns a bunk data structure
				//  and we crash
				
				CWVCSDatabaseConnection sConnection;
				memset( &sConnection, 0, sizeof(CWVCSDatabaseConnection) );
				
				theErr = CWGetDatabaseConnectionInfo( parameterBlock, &sConnection );
				ThrowCWResult( theErr );

				// validate the database path
				if ( RequestRequiresDatabase(intRequest) && (bNegotiation == false) && !ValidateDatabasePath(&sConnection) ) 
					{
						//Invalid Database path
						theErr = cwCommandStatusConnectionLost;
						ThrowCWResult( theErr );
					}

				if ( RequestRequiresDatabase(intRequest) && (bNegotiation == false) && !ValidateLocalRootPath(&sConnection) ) 
					{
						theErr = cwCommandStatusConnectionLost;
						ThrowCWResult( theErr );
					}

				if ( ValidateUserAndPassword(&sConnection) == false )
					{
						//  Invalid Password
						theErr = cwCommandStatusInvalidLogin;
						ThrowCWResult( theErr );
					}

				// handle negotiation requests
				if ( bNegotiation && intRequest != reqInitialize && intRequest != reqTerminate ) 
					{					
						// check if command is supported
						
						CWVCSCommandStatus status=(CheckCommandSupportedMatrix(intRequest, bAdvanced, bRecursive)) ?
							cwCommandStatusSupported : cwCommandStatusUnsupported;
						
						theErr = CWSetCommandStatus(parameterBlock, status);
						ThrowCWResult( theErr );
						
						//  Since we're negotiation commands, we better tell
						//  the IDE what menu text we want.
						
						if ( !isInV1Mode )
							{
								CWVCSCommandDescription sDescription;
								memset( &sDescription, 0, sizeof(sDescription) );
								sDescription.version = cwCommandDescriptionVersion;

								if ( intRequest == reqFileGet )
									{
										sprintf( sDescription.menuItem, "%s", "Update" );
										
										theErr = CWSetCommandDescription( parameterBlock, &sDescription );
										ThrowCWResult( theErr );
									}
								else if ( intRequest == reqFileCheckin )
									{
										sprintf( sDescription.menuItem, "%s", "Commit" );
										
										theErr = CWSetCommandDescription( parameterBlock, &sDescription );
										ThrowCWResult( theErr );
									}
								else if ( intRequest == reqFileHistory )
									{
										sprintf( sDescription.menuItem, "%s", "Log" );
										
										theErr = CWSetCommandDescription( parameterBlock, &sDescription );
										ThrowCWResult( theErr );
									}
							}
							
						long retValue=CWDonePluginRequest(parameterBlock, status);
						return(retValue);
					}
			
				// switch statement
			
			MAIN_SWITCH:
			
				CWResult initTermResult;
				
				switch ( intRequest )
					{
						case reqInitialize:		
							initTermResult=VCS_initialization								( parameterBlock ); break;
						case reqTerminate:			
							initTermResult=VCS_termination									( parameterBlock ); break;
							
						case reqAbout:							VCS_about							( parameterBlock ); break;
				
						case reqDatabaseConnect:		database_connect			( parameterBlock ); break;
						case reqDatabaseDisconnect:	database_disconnect		( parameterBlock ); break;
						case reqDatabaseVariables:	database_variables		( parameterBlock ); break;
				
						case reqFileAdd:						file_add							( parameterBlock ); break;
						case reqFileCheckin:				file_checkin					( parameterBlock ); break;
						case reqFileCheckout:				file_checkout					( parameterBlock ); break;
						case reqFileComment:				file_comment					( parameterBlock ); break;
						case reqFileDelete:					file_delete						( parameterBlock ); break;
						case reqFileDestroy:				file_destroy					( parameterBlock ); break;
						case reqFileDifference:			file_difference				( parameterBlock ); break;
						case reqFileGet:						file_get							( parameterBlock ); break;
						case reqFileHistory:				file_history					( parameterBlock ); break;
						case reqFileLabel:					file_label						( parameterBlock ); break;
						case reqFileProperties:			file_properties				( parameterBlock ); break;
						case reqFilePurge:					file_purge						( parameterBlock ); break;
						case reqFileRename:					file_rename						( parameterBlock ); break;
						case reqFileRollback:				file_rollback					( parameterBlock ); break;
						case reqFileStatus:					file_status						( parameterBlock ); break;
						case reqFileUndoCheckout:		file_undo_checkout		( parameterBlock ); break;
						case reqFileVersion:				file_version					( parameterBlock ); break;
					}
								
				if( !((intRequest==reqInitialize) || (intRequest==reqTerminate)) )
					{
						// get the return status
					
						CWVCSCommandStatus theStatus;
					
						theErr = CWGetCommandStatus( parameterBlock, &theStatus );
						ThrowCWResult( theErr );
					
						intResult = CWDonePluginRequest( parameterBlock, theStatus );
					}
				else
					{
						intResult = CWDonePluginRequest( parameterBlock, initTermResult );
					}
					
			}
		catch( OSErr intException )
			{
				//  What should we do with the OSErr?
				
				CWSetCommandStatus( parameterBlock, cwCommandStatusFailed );
				intResult = CWDonePluginRequest( parameterBlock, cwCommandStatusFailed );
			}			
		catch( CWResult cwError )
			{
				CWSetCommandStatus( parameterBlock, cwError );
				intResult = CWDonePluginRequest( parameterBlock, cwError );
			}			
		catch ( ... )
			{
				CWSetCommandStatus( parameterBlock, cwCommandStatusFailed );
				intResult = CWDonePluginRequest( parameterBlock, cwCommandStatusFailed );

				//  Unknown exception
				Assert_( false );
			}
				
		return intResult;
		
	}


#pragma mark -


void 
file_add( CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void file_checkin( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_checkout( CWPluginContext parameterBlock )
	{
	
		OSErr 							result 				= noErr;
		CWResult						cwError				= 0;
		Boolean							bAdvanced			= false;
		UInt32							intItemCount 	= 0;
		CWVCSCheckoutState	intState			= 0;
		CCVSEngineRef				pEngine				= NULL;

		Str255							pstrRevision	= "\p";
		DialogPtr						pDialog				= NULL;
		LGrafPortView				*pView				= NULL;
		
		try
			{
				if ( !FSpIsValid(gFSRoot) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}

				cwError = CWIsAdvancedRequest( parameterBlock, &bAdvanced );
				ThrowCWResult( cwError );

				if ( bAdvanced )
					{
						pDialog = GetPowerPlantDialog( 128 );
						ThrowIfNil_( pDialog );
						
						pView = (LGrafPortView *) ::GetWRefCon( pDialog );
						ThrowIfNil_( pView );
						
						ShowPluginDialog( pDialog, parameterBlock );
						pView->Draw( NULL );
						
						ModalPowerPlantDialog( pDialog );
					}
								
				cwError = CWGetVCSItemCount( parameterBlock, &intItemCount );
				ThrowCWResult( cwError );
				
				for ( UInt32 intIndex = 0; intIndex < intItemCount; intIndex++ )
					{
						CWVCSItem sItem;
					
						cwError = CWGetVCSItem( parameterBlock, intIndex, &sItem );
						ThrowCWResult( cwError );
								
						cwError = CWPreFileAction( parameterBlock, &sItem.fsItem );
						ThrowCWResult( cwError );
									
						//  Connect to the repository
						pEngine = database_startup();
						
						CCVSEngineUpdateOptions sUpdateOptions;
						sUpdateOptions.Init();
						
						sUpdateOptions.mResetStickyTags = true;
						sUpdateOptions.mGetNewDirectories = true;
						sUpdateOptions.mUpdateDeep = true;
						
						result = CVSEngineUpdate( pEngine, &sItem.fsItem, &sUpdateOptions );
						
						if ( result == kCVSEngineNoRevisionDataErr )
							{
								result = noErr;
							}
							
						ThrowOSErr( result );

						cwError = CWPostFileAction( parameterBlock, &sItem.fsItem );
						ThrowCWResult( cwError );
						
						DisposeCVSEngine( pEngine );
						pEngine = NULL;
						
						if ( FSpIsAFile(&sItem.fsItem, NULL) == noErr )
							{
								result = FSpGetRevisionInfo( &sItem.fsItem, pstrRevision, NULL, NULL, NULL );
								ThrowOSErr( result );
							
								sItem.version.eVersionForm = cwVersionFormAlpha;
								
								//  Since we pass this pascal string as a
								//  char *, we better make sure it's 0 terminated.
								
								if ( pstrRevision[0] == 255 )
									{
										pstrRevision[0]--;
									}						
								pstrRevision[pstrRevision[0] + 1] = 0;
								
								sItem.version.sVersionData.pAlpha = (char *)&pstrRevision[1];
								intState = cwCheckoutStateCheckedOut; //  um, sure
							}
							
											
						cwError = CWFileStateChanged( parameterBlock, &sItem.fsItem, intState, sItem.version );
						ThrowCWResult( cwError );
							
						sItem.eItemStatus = cwItemStatusSucceeded;
				
						cwError = CWSetVCSItem( parameterBlock, intIndex, &sItem );
						ThrowCWResult( cwError );
					}	
				
				CWVCSCommandStatus status = cwCommandStatusSucceeded;
				CWSetCommandStatus( parameterBlock, status );
			}
		catch ( ... )
			{
				try
					{
						if ( pEngine != NULL )
							{
								DisposeCVSEngine( pEngine );
								pEngine = NULL;
							}
					}
				catch ( ... )
					{
					}
					
				throw;				
			}
	}


void 
file_comment(	CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_delete( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;			
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_destroy( CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_difference( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_get( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void
file_history( CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_label( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_properties( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_purge(	CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_rename( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_rollback( CWPluginContext parameterBlock )
	{
		OSErr 							result 				= noErr;
		CWResult						cwError				= 0;
		UInt32							intItemCount 	= 0;
		CWVCSCheckoutState	intState			= 0;
		CCVSEngineRef				pEngine				= NULL;

		Str255							pstrRevision	= "\p";
		
		if ( !FSpIsValid(gFSRoot) )
			{
				ThrowOSErr( kCVSEngineBadParamErr );
			}
		
		cwError = CWGetVCSItemCount( parameterBlock, &intItemCount );
		ThrowCWResult( cwError );
		
		for ( UInt32 intIndex = 0; intIndex < intItemCount; intIndex++ )
			{
				CWVCSItem sItem;
			
				cwError = CWGetVCSItem( parameterBlock, intIndex, &sItem );
				ThrowCWResult( cwError );
	
				if ( FSpIsAFile(&sItem.fsItem, NULL) == noErr )
					{					
						cwError = CWPreFileAction( parameterBlock, &sItem.fsItem );
						ThrowCWResult( cwError );
									
						result = CreateCVSEngine( &gFSRoot, NULL, NULL, EventNotifier, MessageNotifier, ErrorNotifier, NULL, kDefaultSocketTimeout, &pEngine );
						Assert_( result == noErr );
						
						CCVSEngineLoginOptions sLoginOptions;
						sLoginOptions.Init();
									
						PStrCopy( gServerName, sLoginOptions.mServerName, sizeof(sLoginOptions.mServerName) );
						PStrCopy( gUserName, sLoginOptions.mRemoteUserName, sizeof(sLoginOptions.mRemoteUserName) );
						PStrCopy( gPassword, sLoginOptions.mPassword, sizeof(sLoginOptions.mPassword) );
						PStrCopy( gServerRoot, sLoginOptions.mCVSRoot, sizeof(sLoginOptions.mCVSRoot) );
						sLoginOptions.mAuthenticationMethod = gAuthMethod;
						sLoginOptions.mPort = gServerPort;
						
						result = CVSEngineLogin( pEngine, &sLoginOptions );
						ThrowOSErr( result );
						
						CCVSEngineUpdateOptions sUpdateOptions;
						sUpdateOptions.Init();
						
						sUpdateOptions.mResetStickyTags = true;
						sUpdateOptions.mGetNewDirectories = true;
						sUpdateOptions.mUpdateDeep = true;
						sUpdateOptions.mRevert = true;
						
						result = CVSEngineUpdate( pEngine, &sItem.fsItem, &sUpdateOptions );
						ThrowOSErr( result );
						
						DisposeCVSEngine( pEngine );
						pEngine = NULL;
						
						result = FSpGetRevisionInfo( &sItem.fsItem, pstrRevision, NULL, NULL, NULL );
						ThrowOSErr( result );
					
						sItem.version.eVersionForm = cwVersionFormAlpha;
						
						//  Since we pass this pascal string as a
						//  char *, we better make sure it's 0 terminated.
						
						if ( pstrRevision[0] == 255 )
							{
								pstrRevision[0]--;
							}						
						pstrRevision[pstrRevision[0] + 1] = 0;
						
						sItem.version.sVersionData.pAlpha = (char *)&pstrRevision[1];
						
						cwError = CWFileStateChanged( parameterBlock, &sItem.fsItem, intState, sItem.version );
						ThrowCWResult( cwError );
							
						sItem.eItemStatus = cwItemStatusSucceeded;
				
						cwError = CWSetVCSItem( parameterBlock, intIndex, &sItem );
						ThrowCWResult( cwError );
					}
			}	
		
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus( parameterBlock, status );
	}


void 
file_status( CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void 
file_undo_checkout( CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}
	
	
void 
file_version( CWPluginContext parameterBlock )
	{			
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus( parameterBlock, status );
	}


#pragma mark -


void database_connect( CWPluginContext parameterBlock )
	{	
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void database_disconnect( CWPluginContext	parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


void database_variables( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}


CCVSEngineRef			
database_startup( void )
	{
				
		OSErr											result 					= noErr;
		CCVSEngineRef 						pEngine 				= NULL;

		CCVSEngineLoginOptions 		sLoginOptions;
		sLoginOptions.Init();

		try
			{
				//  Instantiate CVS Engine
				result = CreateCVSEngine( &gFSRoot, NULL, NULL, EventNotifier, MessageNotifier, ErrorNotifier, NULL, kDefaultSocketTimeout, &pEngine );
				ThrowOSErr( result );				
				
				//  Login to repository
				PStrCopy( gServerName, sLoginOptions.mServerName, sizeof(sLoginOptions.mServerName) );
				PStrCopy( gUserName, sLoginOptions.mRemoteUserName, sizeof(sLoginOptions.mRemoteUserName) );
				PStrCopy( gPassword, sLoginOptions.mPassword, sizeof(sLoginOptions.mPassword) );
				PStrCopy( gServerRoot, sLoginOptions.mCVSRoot, sizeof(sLoginOptions.mCVSRoot) );
				sLoginOptions.mAuthenticationMethod = gAuthMethod;
				sLoginOptions.mPort = gServerPort;
				
				result = CVSEngineLogin( pEngine, &sLoginOptions );
				ThrowOSErr( result );
			}
		catch ( ... )
			{
				try
					{
						if ( pEngine != NULL )
							{
								DisposeCVSEngine( pEngine );
								pEngine = NULL;
							}
					}
				catch ( ... )
					{
					}
					
				throw;
			}
			
		return pEngine;
		
	}
	
	
#pragma mark -


CWResult 
VCS_initialization( CWPluginContext	pb )
	{
		OSErr 							result 		= noErr;
		CWVCSCommandStatus 	sStatus;

		//  Initialize Open Transport system.		
		
		result = InitOpenTransport();
			
		InitializePowerPlant();
				
		if ( result == noErr )
			{
				sStatus = cwCommandStatusSucceeded;
			}
		else
			{
				sStatus = cwCommandStatusFailed;
			} 
				
		if ( gContext == NULL )
			{
				gContext = (IDEMacCVSContextRec *)::NewPtrClear( sizeof(IDEMacCVSContextRec) );
			}

		if ( gContext != NULL )
			{
				gContext->fCWContext = pb;
			}
		
		if ( gContext == NULL )
			{
				sStatus = cwCommandStatusFailed;
			}
				
		CWSetCommandStatus( pb, sStatus );
		
		return cwNoErr;
	}


CWResult VCS_termination( CWPluginContext	parameterBlock )
	{
		//  Should we unload Open Transport?
		
		try
			{
				CloseOpenTransport();
				FinalizePowerPlant();
				FreeAllCplusMemory();
				//  FreeAllMallocMemory();
				
				if ( gContext != NULL )
					{
						::DisposePtr( (Ptr)gContext );
						gContext = NULL;
					}
			}
		catch ( ... )
			{
			}
			
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
		
		return cwNoErr;
	}


void 
VCS_about( CWPluginContext parameterBlock )
	{
		CWVCSCommandStatus status = cwCommandStatusSucceeded;
		CWSetCommandStatus(parameterBlock, status);
	}
	

#pragma mark -


pascal SInt32	
EventNotifier( const CCVSEngineRef inEngineRef, const FSSpec *inTargetItem,	const SInt32 inEvent, 
							 const SInt32 inError, const UInt32 inCookie, const void *inDataPtr, void	*inContextPtr)
	{
	
		SInt32 intResult = 1;
		
		try
			{
				switch ( inEvent )
					{
						case kCVSEngineAddPreActionEvent:
							intResult = 1;
						break;
						
						case kCVSEngineEntryPreActionEvent:
							intResult = 1;
						break;
						
						case kCVSEngineUpdatePreActionEvent:
							intResult = 1;
						break;
						
						case kCVSEngineUpdatePostActionEvent:
						
						break;
					}
			}
		catch ( ... )
			{
			
			}
			
		return intResult;
		
	}
	

pascal SInt32 
MessageNotifier( const CCVSEngineRef inEngineRef, const UInt32 inDataType, const UInt32 inDataLen, 
									const void *inDataPtr, void *inContextPtr )
	{
	
	}


pascal SInt32 
ErrorNotifier( const CCVSEngineRef inEngineRef, const UInt32 inDataType, const UInt32 inDataLen, 
									const void *inDataPtr, void *inContextPtr )
	{
	
	}


#pragma mark -


void
InitializePowerPlant( void )
	{

		// set up the QuickDraw globals.  Since this is a CR, we have
		// to get/use the qd of our host
		
		try
			{
				UQDGlobals::SetCodeResourceQDGlobals();

				// let's register our pane classes so we can use the stream
				// constructors when reading
			
				RegisterClass_(LGrafPortView);
				RegisterClass_(LCaption);
				RegisterClass_(LEditField);
				RegisterClass_(LGroupBox);
				RegisterClass_(LStdButton);
				RegisterClass_(LStdRadioButton);
				RegisterClass_(LStdCheckBox);
				RegisterClass_(LPicture);
				RegisterClass_(LRadioGroupView);
			}
		catch ( ... )
			{
			}
			
	}
	
	
void
FinalizePowerPlant( void )
	{

		// first, we shall eliminate any PP-allocated objects
		
		try
			{
				URegistrar::DisposeClassTable();
				LPeriodical::DeleteIdlerAndRepeaterQueues();
										
				LComparator *theComparator = LComparator::GetComparator();
				delete theComparator;
				theComparator = nil;
				
				LLongComparator *theLongComparator = LLongComparator::GetComparator();
				delete theLongComparator;
				theLongComparator = nil;
			}
		catch ( ... )
			{
			}
			
	}
	
	
DialogPtr
GetPowerPlantDialog( const Int16 inPPobID )
	{
	
		DialogPtr		pDialog 	= NULL;
		GrafPtr			pOldPort 	= NULL;
		Rect				sBounds		= { 0 };
		Rect				sPPBounds	= { 0 };
		
		::GetPort( &pOldPort );
		pDialog =  ::GetNewDialog( 128, NULL, (WindowPtr)-1 );
		ThrowIfNil_( pDialog );
		::SetPort( pDialog );
				
		LGrafPortView *pPanel = LGrafPortView::CreateGrafPortView( inPPobID, nil );
		ThrowIfNil_( pPanel );
					
		WindowPeek	wp = (WindowPeek) pPanel->GetMacPort();
		wp->controlList = nil;
	
		WindowPtr	pWindow = pPanel->GetMacPort();
		
		::SetWRefCon( pWindow, (long)pPanel );
		
		pPanel->CalcPortFrameRect( sPPBounds );
		
		Int16 intWidth = sPPBounds.right - sPPBounds.left;
		Int16 intHeight = sPPBounds.bottom - sPPBounds.top;
		
		::SizeWindow( pDialog, intWidth, intHeight, false );		
		::SetPort( pOldPort );
		
		return pDialog;
		
	}


void
ModalPowerPlantDialog( const DialogPtr pInDialog )
	{
	
		Boolean				bGotEvent	= false;
		Boolean				bDone			= false;
		EventRecord		sEvent 		= { 0 };
		
		LGrafPortView	*pView		= NULL;
		
		pView = (LGrafPortView *)::GetWRefCon( pInDialog );
		
		while ( !bDone )
			{
				bGotEvent = ::WaitNextEvent( everyEvent, &sEvent, 60, NULL );
				
				if ( bGotEvent )
					{
						switch ( sEvent.what )
							{
								case mouseDown:
									pView->ClickInContent( sEvent );
								break;
								
								case keyDown:
								case keyUp:
								case autoKey:
									pView->DoKeyPress( sEvent );
								break;
								
								case updateEvt:
									pView->UpdatePort();
								break;
								
								case activateEvt:
								
								break;
							}
					}
			}
			
	}
	
	
void
ShowPluginDialog( const DialogPtr pDialog, CWPluginContext parameterBlock )
	{
	
		if ( pDialog != NULL )
			{
				CWPreDialog( parameterBlock );
				::ShowWindow( pDialog );
			}
		
	}
	
	
void			
HidePluginDialog( const DialogPtr pDialog, CWPluginContext parameterBlock )
	{
	
		if ( pDialog != NULL )
			{
				CWPostDialog( parameterBlock );
				::HideWindow( pDialog );	
			}
			
	}
	

void
StoreContext( const IDEMacCVSContextRec *pInContext )
	{
	
	}
	
	
void
RemoveContext( const IDEMacCVSContextRec *pInContext )
	{
	
	}
	
	