//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CBroadcastTableTextView.cp
//
//	  Authors:	Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#include <LTableMonoGeometry.h>
#include <LTableSingleSelector.h>
#include <LTableArrayStorage.h>

#include "UKeyScrollAttachments.h"
#include "CBroadcastTableTextView.h"


CBroadcastTableTextView*	
CBroadcastTableTextView::CreateCBroadcastTableTextViewFromStream(LStream *inStream)
	{
	
		return new CBroadcastTableTextView( inStream );
	
	}


CBroadcastTableTextView::CBroadcastTableTextView( LStream * inStream ):

	LTableView( inStream )

	{
	
		mTextTraits = kTextTraitsID;
		
		UTextTraits::SetPortTextTraits( mTextTraits );
		
		::GetFontInfo( &mFontInfo);
		
		SetTableGeometry( new LTableMonoGeometry(this, 
								/* width */kDefaultWidth,
								/* height */ mFontInfo.ascent + mFontInfo.descent + mFontInfo.leading));
								
		SetTableSelector( new LTableSingleSelector(this) );
		SetTableStorage( new LTableArrayStorage( this, sizeof( CColorTableTextRec )));
		SetUseDragSelect( true );
		
		SPoint32	theScrollAmount = { 0, mFontInfo.ascent + mFontInfo.descent + mFontInfo.leading };
		SetScrollUnit( theScrollAmount );
		
	}
	
	
		
CBroadcastTableTextView::~CBroadcastTableTextView()
	{

	}


void
CBroadcastTableTextView::FinishCreateSelf( void )
	{
		//	Attachment for keyboard navigation
		AddAttachment( new USimpleKeyScrollAttachment(this) );
		
		LTableView::FinishCreateSelf();
	}
	
void
CBroadcastTableTextView::DrawCell( const STableCell	&inCell, const Rect	&inLocalRect )
	{

		UInt32							ioCount = sizeof( CColorTableTextRec );
		CColorTableTextRec 	cellData;

		GetCellData( inCell, &cellData, ioCount );

		UTextTraits::SetPortTextTraits( mTextTraits );
		
		::BackColor( whiteColor);
		::RGBForeColor( &cellData.fColor );
		::MoveTo(inLocalRect.left + 4, inLocalRect.bottom - mFontInfo.descent - mFontInfo.leading );
		
		::DrawString( cellData.fStr );
	//	::ForeColor( redColor );
	//	::MoveTo(inLocalRect.left + 4, inLocalRect.bottom - mFontInfo.descent - mFontInfo.leading );
	//	::DrawString( ct.fStr );

	}


void
CBroadcastTableTextView::SelectionChanged()
	{
	
		BroadcastMessage( msg_SelectionChanged, NULL );
	
	}