//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CCompareSwitcherView.cp
//
//	  Authors:	Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#include <LArray.h>
#include <LString.h>
#include <LTableView.h>
#include <stdio.h>


#include "CCompareSwitcherView.h"
#include "CBroadcastTableTextView.h"
#include "CCompareWithOriginalView.h"


enum
	{
		kDifferenceLineAddedColorResID = 128,
		kDifferenceLineRemovedColorResID,
		kDifferenceLineChangedColorResID
	};


CCompareSwitcherView*
CCompareSwitcherView::CreateCCompareSwitcherViewFromStream( LStream *inStream )
	{
	
		return new CCompareSwitcherView( inStream );

	}
	
	
CCompareSwitcherView::CCompareSwitcherView( LStream *inStream ): 

	LView(inStream)

	{
		
		mLineInfo = new LArray( sizeof(UInt32) );
		ThrowIfNil_( mLineInfo );
		
		// default colors
		mAddedColor.red = 0;
		mAddedColor.green = 33825;		//added
		mAddedColor.blue = 5000;

		mRemovedColor.red = 40535;		//removed
		mRemovedColor.green = 0;
		mRemovedColor.blue = 0;
															
		mChangedColor.red = 48000;		//changed
		mChangedColor.green = 48000;
		mChangedColor.blue = 0;
	}


CCompareSwitcherView::~CCompareSwitcherView()
	{
	
		if ( mLineInfo != NULL )
			{
			
				delete mLineInfo;
				mLineInfo = NULL;
	
			}
			
	}
					
					
void
CCompareSwitcherView::FinishCreateSelf()
	{
		
		mTableView = dynamic_cast<CBroadcastTableTextView *>( FindPaneByID(CBroadcastTableTextView::class_ID) );
		ThrowIfNil_( mTableView );
		
		mTextView = dynamic_cast<CCompareWithOriginalView *>( FindPaneByID(CCompareWithOriginalView::class_ID) );
		ThrowIfNil_( mTextView );
		
		mTableView->AddListener( this );
		mTableView->InsertCols( 1, 0, NULL, 0, false );
		mTextView->AddListener( this );
		
		SetupColors();
		
	}

void
CCompareSwitcherView::SetupColors()
	{
		Handle		rgbHandle = NULL;
	
		rgbHandle = ::GetResource('rgb ', kDifferenceLineAddedColorResID);
		
		if (rgbHandle != NULL)
			{
				mAddedColor = **(RGBColor **)rgbHandle;
				::ReleaseResource(rgbHandle);
			}
			
		rgbHandle = ::GetResource('rgb ', kDifferenceLineRemovedColorResID);
		
		if (rgbHandle != NULL)
			{
				mRemovedColor = **(RGBColor **)rgbHandle;
				::ReleaseResource(rgbHandle);
			}
			
		rgbHandle = ::GetResource('rgb ', kDifferenceLineChangedColorResID);
		
		if (rgbHandle != NULL)
			{
				mChangedColor = **(RGBColor **)rgbHandle;
				::ReleaseResource(rgbHandle);
			}
			
	}
	
	
// Does all the work
void	
CCompareSwitcherView::ListenToMessage( MessageT inMessage, void *ioParam )
	{
		
		switch ( inMessage )	
			{
		
				case CCompareWithOriginalView::msg_GotCompareInformation:
					{
					
						LArray 				*lineDescriptions 				= (LArray *)ioParam;
						UInt32 				currentLineType 					= kDifferenceLineNormal;
						UInt32 				currentLineTypeStartedAt 	= 0;
						ArrayIndexT 	index											= 0;
						
						// Go through the list of lines, generating an entry for each diff
						
						for ( index = 1; index <= (lineDescriptions->GetCount() + 1 /* +1 to catch the last line */) ; index++ )
							{

								char 								text[255]		= "";

								CDiffLineInfoRec 		dli					= { 0, 0 };
								CColorTableTextRec 	tableEntry;

								if ( !lineDescriptions->FetchItemAt( index, &dli ))	// Smallish hack, to catch the last line
									dli.fLineType = -1;

								if ( dli.fLineType != currentLineType )
									{
										if ( currentLineType != kDifferenceLineNormal )

	// Record the last line run
											{
											
												switch (currentLineType)
													{
												
														case kDifferenceLineNormal:
															// doNothing
													
														break;
													
														case kDifferenceLineAdded:	// Green
												
															tableEntry.fColor = mAddedColor;
															sprintf( text, "Line %d: %d lines added", currentLineTypeStartedAt, index - currentLineTypeStartedAt);
															c2pstr(text);
															LString::CopyPStr( (unsigned char *)text, tableEntry.fStr, sizeof (tableEntry.fStr));
												
														break;
											
														case kDifferenceLineRemoved:	// Red
													
															tableEntry.fColor = mRemovedColor;
															sprintf( text, "Line %d: %d lines removed", currentLineTypeStartedAt, index - currentLineTypeStartedAt);
															c2pstr(text);
															LString::CopyPStr( (unsigned char *)text, tableEntry.fStr, sizeof (tableEntry.fStr));
														
														break;
														
														case kDifferenceLineChanged:	// Yellow
														
															tableEntry.fColor = mChangedColor;
															sprintf( text, "Line %d: %d lines changed", currentLineTypeStartedAt, index - currentLineTypeStartedAt);
															c2pstr(text);
															LString::CopyPStr( (unsigned char *)text, tableEntry.fStr, sizeof (tableEntry.fStr));
															
														break;
															
													}

												// Add new entry to the table
												TableIndexT row, col;
												mTableView->GetTableSize(row, col);
												mTableView->InsertRows(1, row, &tableEntry, sizeof(tableEntry), false);
												// Make our record
												mLineInfo->InsertItemsAt( 1, LArray::index_Last, &currentLineTypeStartedAt, sizeof (currentLineTypeStartedAt));

											}
											
										// Start a new run
										currentLineTypeStartedAt = index;
										currentLineType = dli.fLineType;
					
									}
					
							}	// for
							
						mTableView->SelectCell( STableCell(1,1) );
						
						Refresh();
			
					}
				break;
				
				case CBroadcastTableTextView::msg_SelectionChanged:
					{
					
						STableCell cell;
					
						if ( mTableView->GetNextSelectedCell(cell))
							{
								
								UInt32 line	= 0x00000000;
								
								mLineInfo->FetchItemAt( cell.row, &line );
								mTextView->ScrollToLine( line );
							
							}
					
					}
				break;
				
			}

	}