//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 	CCompareWithOriginalView.cp
//
//	  Author: Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#include <LArray.h>

#include "CThreadedTask.h"
#include "UMacCVSUtils.h"

#include "CCompareWithOriginalView.h"


enum
	{
		kDifferenceLineAddedColorResID = 128,
		kDifferenceLineRemovedColorResID,
		kDifferenceLineChangedColorResID
	};


CCompareWithOriginalView::CCompareWithOriginalView( LStream *inStream ):

	LView( inStream ),
	mLineDescriptions( sizeof(CDiffLineInfoRec) )

	{

		mCompositeData		= NULL;
		
		// default colors
		mAddedColor.red = 0;
		mAddedColor.green = 33825;
		mAddedColor.blue = 5000;

		mRemovedColor.red = 40535;
		mRemovedColor.green = 0;
		mRemovedColor.blue = 0;
															
		mChangedColor.red = 48000;
		mChangedColor.green = 48000;
		mChangedColor.blue = 0;
		
		mSpacesPerTab = kSpacesPerTab;

	}


CCompareWithOriginalView::CCompareWithOriginalView():

	mLineDescriptions( sizeof(CDiffLineInfoRec) )
	
	{

		mCompositeData		= NULL;
		
		// default colors
		mAddedColor.red = 0;
		mAddedColor.green = 33825;
		mAddedColor.blue = 5000;

		mRemovedColor.red = 40535;
		mRemovedColor.green = 0;
		mRemovedColor.blue = 0;
															
		mChangedColor.red = 48000;
		mChangedColor.green = 48000;
		mChangedColor.blue = 0;
		
		mSpacesPerTab = kSpacesPerTab;

	}


CCompareWithOriginalView *
CCompareWithOriginalView::CreateCCompareWithOriginalViewFromStream( LStream *inStream )
	{
	
		return ( new CCompareWithOriginalView(inStream) );
	
	}


CCompareWithOriginalView::~CCompareWithOriginalView()
	{
	
		if ( mCompositeData != NULL )
			{
			
				::DisposeHandle( mCompositeData );
				mCompositeData = NULL;
			
			}
	
	}


void 
CCompareWithOriginalView::FinishCreateSelf()
	{
		SetupColors();
	}

void
CCompareWithOriginalView::SetupColors()
	{
		Handle		rgbHandle = NULL;
	
		rgbHandle = ::GetResource('rgb ', kDifferenceLineAddedColorResID);
		
		if (rgbHandle != NULL)
			{
				mAddedColor = **(RGBColor **)rgbHandle;
				::ReleaseResource(rgbHandle);
			}
			
		rgbHandle = ::GetResource('rgb ', kDifferenceLineRemovedColorResID);
		
		if (rgbHandle != NULL)
			{
				mRemovedColor = **(RGBColor **)rgbHandle;
				::ReleaseResource(rgbHandle);
			}
			
		rgbHandle = ::GetResource('rgb ', kDifferenceLineChangedColorResID);
		
		if (rgbHandle != NULL)
			{
				mChangedColor = **(RGBColor **)rgbHandle;
				::ReleaseResource(rgbHandle);
			}
			
	}
	

Int16
CCompareWithOriginalView::GetTabWidth( const FSSpec &inTargetItem )
	{
		
		Int16			tabWidth 				= mSpacesPerTab;
		Int16			resForkRefNum 	= -1;
		SInt16		intOldResRef		= 0;
		
		intOldResRef = ::CurResFile();
		
		resForkRefNum = ::FSpOpenResFile( &inTargetItem, fsRdPerm );
		
		if ( resForkRefNum != -1 )
			{
			
				CMpsrRec		**mprsResource = nil;
				
				mprsResource = (CMpsrRec **)::Get1Resource( kMpsrResourceType, kMpsrResourceID );
			
				if (mprsResource != nil)
					{				
						tabWidth = (**mprsResource).fTabWidth;					
					}
			
				::CloseResFile(resForkRefNum);
			}
			
		::UseResFile( intOldResRef );
		
		return tabWidth;
	}
	
	
	
void 
CCompareWithOriginalView::SetCompareInformation( const FSSpec &inTargetItem, const LArray &inDiffClauseList )
	{
	
		const							kWriteBufferSize			= 1024;
		
		UInt32						sourceFileLength 			= 0,
											currentSourceLine 		= 0,
											currentSourceChar			= 0,
											writtenCharacters 		= 0,
											lastLineBeginning 		= 0,
											linesToAddOrChange 		= 0,
											currentClauseNum 			= 1,
											lastClause 						= inDiffClauseList.GetCount();
											
		char							readChar 							= '\n';

		Boolean						lineChangedOrAdded 		= false,
											firstTime 						= true;
											
		CDiffClauseRec		currentClause;

		CDiffLineInfoRec	lineInformation;
	
		char							writeBuffer[kWriteBufferSize]	= "";
		UInt32						writeBufferUsed 							= 0;

		Handle						originalFileSource		= NULL;
		
		try
			{
			
// Get the tab width from the MPSR resource in the file, if it has one
				mSpacesPerTab = GetTabWidth( inTargetItem );

				mLineDescriptions.RemoveItemsAt( mLineDescriptions.GetCount(), LArray::index_First );

				if ( mCompositeData != NULL )
					{
					
						::DisposeHandle( mCompositeData );
						mCompositeData = NULL;
					
					}

//  The compositeStream is a handle containing the combination of both the
//  local source file and the diffs in the remote source file.

				LHandleStream	compositeStream;		
				
				LFileStream	sourceFile( inTargetItem );
				sourceFile.OpenDataFork( fsRdPerm );
				sourceFileLength = sourceFile.GetLength();	
				originalFileSource = sourceFile.ReadDataFork();	
				
				while ( true )
					{
				
						if ( (readChar == '\n') || (readChar == '\r') || (sourceFileLength == 0) )
							{
							
								if ( (currentSourceLine % 500) == 0 )
									CThreadedTask::Yield();
				
								if ( !firstTime )
									{
									
										lineInformation.fOffset = lastLineBeginning;
										lineInformation.fLineType = kDifferenceLineNormal;
										
										if ( linesToAddOrChange )
											{
											
												linesToAddOrChange--;
												lineInformation.fLineType = lineChangedOrAdded;
											
											}
										
										mLineDescriptions.InsertItemsAt(1, LArray::index_Last, &lineInformation); 				
									
									}
								else firstTime = false;
						
								if ( currentClauseNum <= lastClause )
									{				
								
										inDiffClauseList.FetchItemAt( currentClauseNum, &currentClause );
										
										if ( currentClause.fLineNumber == ++currentSourceLine )
											{
											
												compositeStream.WriteData(writeBuffer, writeBufferUsed);
												writeBufferUsed = 0;

// 	If there are deleted lines from the original file, then
//  add them to the composite file... they will show up in 
//  red.

												if ( currentClause.fType == kDifferenceLineRemoved )
													{
													
														Assert_( currentClause.fDataHandle != NULL );
														
														if ( currentClause.fDataHandle != NULL )
															{
															
																StHandleLocker	dataHndlLock( currentClause.fDataHandle );
																UInt32	dataSize = ::GetHandleSize( currentClause.fDataHandle );
																UInt32	curOffset = 0;
																
																for ( UInt32 currentLine = 0; currentLine < currentClause.fLineCount; currentLine++ )
																	{
																			
																		const		kLineBufLen = 256;
																		char		lineStr[kLineBufLen] = "";
																		UInt32	lineStrLen	= 0;
																		
																		lineStrLen = kLineBufLen - 1;
																		UMacCVSUtils::ReadWord( *(currentClause.fDataHandle), dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																		lineStr[lineStrLen] = 0;
																
																		lineInformation.fOffset = writtenCharacters;
																		lineInformation.fLineType = kDifferenceLineRemoved;
																		
																		compositeStream.WriteData( lineStr, lineStrLen );
																		compositeStream.WriteData( "\r", 1 );
																		
																		mLineDescriptions.InsertItemsAt( 1, LArray::index_Last, &lineInformation ); 				
																		writtenCharacters += lineStrLen + 1;

																	}
																	
															}
														
													}
												
												else
													{
													
														lineChangedOrAdded = currentClause.fType;
														linesToAddOrChange = currentClause.fLineCount;
													
													}
											
												currentClauseNum++;
												
											}
													
									}
								
								lastLineBeginning = writtenCharacters;
								
							}

						if ( sourceFileLength-- == 0 )
							{
							
								if ( (readChar != '\n') || (readChar == '\r') )
									{
									
										lineInformation.fOffset = lastLineBeginning;
										lineInformation.fLineType = kDifferenceLineNormal;
										
										if ( linesToAddOrChange )
											{
											
												linesToAddOrChange--;
												lineInformation.fLineType = lineChangedOrAdded;
											
											}
										
										mLineDescriptions.InsertItemsAt( 1, LArray::index_Last, &lineInformation ); 			
									
									}
									
								break;
								
							}
							
//  Get the next character in the source file and echo it to the
//  composite file.				

						readChar = ((char *)(*originalFileSource))[currentSourceChar++];
						writeBuffer[writeBufferUsed++] = readChar;
						
						if ( writeBufferUsed == kWriteBufferSize )
							{
							
								compositeStream.WriteData( writeBuffer, writeBufferUsed );
								writeBufferUsed = 0;
							
							}
						
						writtenCharacters++;
				
					}
				
				if ( writeBufferUsed > 0 )
					{
					
						compositeStream.WriteData( writeBuffer, writeBufferUsed );
						writeBufferUsed = 0;
				
					}
						
				mCompositeData = compositeStream.DetachDataHandle();
			
				UInt32	imageHeight = 0x00000000;
					
				imageHeight = kTextDisplayLineHeight * mLineDescriptions.GetCount();
				ResizeImageTo( 100, imageHeight, false );
						
				BroadcastMessage( msg_GotCompareInformation, &mLineDescriptions );
				Refresh();

				if ( originalFileSource != NULL )
					{	
					
						::DisposeHandle( originalFileSource );
						originalFileSource = NULL;
						
					}
									
			}
			
		catch ( ... )
			{
			
				if ( originalFileSource != NULL )
					{	
					
						::DisposeHandle( originalFileSource );
						originalFileSource = NULL;
						
					}
					
				throw;
			
			}
		
	}
	

void
CCompareWithOriginalView::DrawSelf()
	{
	
		SInt32		firstLineToDraw	= 0x00000000,
							lastLineToDraw	= 0x00000000,
							totalLines			= 0x00000000;
		
		if ( mCompositeData != NULL )
			{
			
				totalLines = mLineDescriptions.GetCount();
				
//	Compute the first and last lines of the text file
//	that we need to draw.
				
				firstLineToDraw = ( mFrameLocation.v - mImageLocation.v ) / kTextDisplayLineHeight - 1;
				
				if ( firstLineToDraw < 1 )
					firstLineToDraw = 1;
						
				lastLineToDraw = ( mFrameLocation.v - mImageLocation.v + mFrameSize.height ) / kTextDisplayLineHeight + 1;
				
				if ( lastLineToDraw > totalLines )
					lastLineToDraw = totalLines;
				
				::TextFont( kFontIDMonaco );
				::TextSize( 9 );
				
				StHandleLocker lock( mCompositeData );
				
				for ( UInt32 currentLineToDraw = firstLineToDraw; currentLineToDraw <= lastLineToDraw; currentLineToDraw++ )
					{
					
						Rect								textRect 		= { 0, 0, 0, 0 };
						CDiffLineInfoRec		lineInfo		= { 0, 0 };
						
						UInt32							lineLength	= 0x00000000,
																tabOffset 	= 0x00000000,
																ioCount			= sizeof( CDiffLineInfoRec );

						Point								localPoint	= { 0, 0 };
						SPoint32						inImagePt;
						
						inImagePt.h = 0;
						inImagePt.v = (currentLineToDraw - 1) * kTextDisplayLineHeight;

						ImageToLocalPoint(inImagePt, localPoint);
					
						::SetRect( &textRect, 0, localPoint.v, 1000, localPoint.v + kTextDisplayLineHeight );
								
						mLineDescriptions.FetchItemAt( currentLineToDraw, &lineInfo, ioCount );
						
						Assert_( ioCount == sizeof(CDiffLineInfoRec) );
						
						switch( lineInfo.fLineType)
							{
							
								case kDifferenceLineNormal:	
							
									::ForeColor(whiteColor);
						
								break;
									
								case kDifferenceLineAdded:
									
									::RGBForeColor(&mAddedColor);
									
								break;
									
								case kDifferenceLineRemoved:
									
									::RGBForeColor(&mRemovedColor);
									
								break;
									
								case kDifferenceLineChanged:
									
									::RGBForeColor(&mChangedColor);
									
								break;
									
							}
						
						::PaintRect( &textRect );

						if (currentLineToDraw == totalLines)
							{
							
								lineLength = ::GetHandleSize( mCompositeData ) - lineInfo.fOffset;
							
							}
						
						else
							{
							
								CDiffLineInfoRec	nextLineInfo	= { 0, 0 };
								
								ioCount = sizeof( CDiffLineInfoRec );
								
								mLineDescriptions.FetchItemAt( currentLineToDraw + 1, &nextLineInfo, ioCount );
								
								Assert_( ioCount == sizeof(CDiffLineInfoRec) );
								
								lineLength = nextLineInfo.fOffset - lineInfo.fOffset - 1;
							
							}
						
						if ( lineLength > kTextDrawBufferSize )
							lineLength = kTextDrawBufferSize;
						
						::MoveTo( 5, localPoint.v + kTextDisplayLineHeight - 3 );
						
						char	*currentChar = (char *)*mCompositeData + lineInfo.fOffset;

						DrawTabExpanded( currentChar, lineLength );	
						
					}
					
			} 
						
	}



void
CCompareWithOriginalView::ScrollToLine( const UInt32 inLine )
	{
	
		UInt32	line = inLine;
		
		if ( line > 0 )
			line--;
			
		if ( line > 5 )
			line --;	// Give it some space on top
			
		ScrollImageTo( 0, line * kTextDisplayLineHeight, true );
	
	}


void 
CCompareWithOriginalView::DrawTabExpanded( const char *inText, const UInt32 inTextLen )
	{

		char					*drawBuffer	= NULL;

		UInt32				srcIndex		= 0,
									dstIndex		= 0;
					
		::TextMode( srcOr );
		::ForeColor( blackColor );
		
		// at max all characters are tabs
		
		drawBuffer = (char *)::NewPtr( inTextLen * mSpacesPerTab );
		
		if ( drawBuffer == NULL )
			throw( ::MemError() );
				
		for ( srcIndex = 0; srcIndex < inTextLen; srcIndex++ )
			{
			
				if ( inText[srcIndex] == '\t' )
					{
						
						UInt32		spacesToAdd = mSpacesPerTab - (dstIndex % mSpacesPerTab);
						
						for ( UInt32 index = 0; index < spacesToAdd; index++ )
							{
							
								drawBuffer[dstIndex] = ' ';
								dstIndex++;
					
							}
							
					}
				else
					{
						
						drawBuffer[dstIndex] = inText[srcIndex];
						dstIndex++;
						
					}

			}
		
		::DrawText( drawBuffer, 0, dstIndex );
		::DisposePtr( drawBuffer );
		
	}


