//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CDividedView.cp
//
//	  Authors:	Ram Rajadhyaksha
//
//################################################################################

#include "CDividedView.h"


#ifndef VERT_CURSOR 
#define VERT_CURSOR 1120
#endif

#ifndef HORI_CURSOR
#define HORI_CURSOR 1136
#endif


CDividedView::CDividedView()
	{
	
		fPane1ID 			= 0;
		fPane2ID 			= 0;
		fPane1MinSize = 1;
		fPane2MinSize = 1;
		fInitialDelta = 1;
		fDragBarSize	= kDefaultDragBarSize;
		fIsVertical 	= false;	
	
	}
	
	
CDividedView::CDividedView( LStream *inStream ) : LView( inStream )
	{
	
		fPane1ID 			= 0;
		fPane2ID 			= 0;
		fPane1MinSize = 1;
		fPane2MinSize = 1;
		fInitialDelta = 1;
		fDragBarSize	= kDefaultDragBarSize;
		fIsVertical 	= false;
		
		if ( inStream != NULL )
			{
					
				*inStream >> fPane1ID;
				*inStream >> fPane2ID;
				*inStream >> fPane1MinSize;
				*inStream >> fPane2MinSize;
				*inStream >> fInitialDelta;
				*inStream >> fDragBarSize;
				*inStream >> fIsVertical;
				
				if ( fPane1MinSize == 0 )
					fPane1MinSize = 1;
					
				if ( fPane2MinSize == 0 )
					fPane2MinSize = 1;
					
				if ( fInitialDelta < fPane1MinSize )
					fInitialDelta = fPane1MinSize;
					
				if ( fDragBarSize == 0 )
					fDragBarSize = kDefaultDragBarSize;
				
			}
		
	}
	
	
CDividedView::~CDividedView()
	{
	
	}
	
	
CDividedView *
CDividedView::CreateCDividedViewFromStream( LStream *inStream )
	{
	
		return new CDividedView( inStream );
	
	}
	

#pragma mark -


void
CDividedView::SetSubViews( 	const PaneIDT inPane1ID, const PaneIDT inPane2ID, const UInt16 inPane1MinSize,
														const UInt16 inPane2MinSize, const UInt16 inInitialSize, const UInt16 inDragBarSize, const Boolean inVertical )
	{

//  SetSubViews() is the only accessor. You can use this if you don't
//  want to define the pane with Constructor or whatever...
	
		fPane1ID = inPane1ID;
		fPane2ID = inPane2ID;

		fPane1MinSize = inPane1MinSize;

		if ( fPane1MinSize == 0 )
			fPane1MinSize = 1;
		
		fPane2MinSize = inPane2MinSize;	
					
		if ( fPane2MinSize == 0 )
			fPane2MinSize = 1;
			
		fInitialDelta = inInitialSize;
		
		if ( fInitialDelta < fPane1MinSize )
			fInitialDelta = fPane1MinSize;
			
		fDragBarSize = inDragBarSize;
		
		if ( fDragBarSize == 0 )
			fDragBarSize = kDefaultDragBarSize;
			
		fIsVertical = inVertical;
		
		if ( (fPane1ID != NULL) && (fPane2ID != NULL) )
			{
			
				fPane1 = FindPaneByID( fPane1ID );
				ThrowIfNil_( fPane1 );
				
				fPane2 = FindPaneByID( fPane2ID );
				ThrowIfNil_( fPane2 );
			
				SetupSubPanes();
				
			}
				
	}
	
	
#pragma mark -

	
void
CDividedView::FinishCreateSelf()
	{
	
		if ( (fPane1ID != NULL) && (fPane2ID != NULL) )
			{
			
				fPane1 = FindPaneByID( fPane1ID );
				ThrowIfNil_( fPane1 );
				
				fPane2 = FindPaneByID( fPane2ID );
				ThrowIfNil_( fPane2 );
			
				SetupSubPanes();
				
			}
	
	}
	

void
CDividedView::ResizeFrameBy( Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh )
	{
	
		LView::ResizeFrameBy( inWidthDelta, inHeightDelta, inRefresh );
	
		ReadjustSubPanes();
	}
		
void
CDividedView::ClickSelf( const SMouseDownEvent &inMouseDown )
	{
	
		Int32					totalDragDist		= 0;
		Int16					hDragDist		 		= 0;
		Int16					vDragDist				= 0;
		Rect					dragLimitRect 	= { 0, 0, 0, 0 };
		Rect					frameRect 			= { 0, 0, 0, 0 };
		Rect					dragBarRect 		= { 0, 0, 0, 0 };
		Rect					slopRect				= { 0, 0, 0, 0 };
		SDimension16	pane1Size				= { 0, 0 };
		
		if ( (fPane1 != NULL) && (fPane2 != NULL) )
			{
			
				FocusDraw();

				CalcLocalFrameRect( frameRect );
				dragLimitRect = frameRect;
				slopRect = frameRect;
				
				fPane1->GetFrameSize( pane1Size );
				
				if ( fIsVertical )
					{

						dragBarRect.top = 0;
						dragBarRect.left = pane1Size.width;
						dragBarRect.bottom = frameRect.bottom;
						dragBarRect.right = dragBarRect.left + fDragBarSize;
						
					}
					
				else
					{
					
						dragBarRect.top = pane1Size.height;
						dragBarRect.left = 0;
						dragBarRect.bottom = dragBarRect.top + fDragBarSize;
						dragBarRect.right = frameRect.right;
					
					}

//  if the user actually hit in the drag bar area, do our
//  wonderful 'thang...
					
				if ( ::PtInRect(inMouseDown.whereLocal, &dragBarRect) )
					{
					
						StRegion dragBarRgn( dragBarRect );
							
						if ( fIsVertical )
							{
							
								if ( (dragLimitRect.right - dragLimitRect.left) > (fPane1MinSize + fPane2MinSize + fDragBarSize) )
									{
									
										dragLimitRect.left += fPane1MinSize;
										dragLimitRect.right -= fPane2MinSize;
										
									}

//  uhoh, the pane is too small for both min. sizes, so
//  bail on letting the user drag this thing by setting the
//  limit rect to that of the drag bar
									
								else
									{
										dragLimitRect = dragBarRect;
									}
									
							}
							
						else
							{
								if ( (dragLimitRect.bottom - dragLimitRect.top) > (fPane1MinSize + fPane2MinSize + fDragBarSize) )
									{
										dragLimitRect.top += fPane1MinSize;
										dragLimitRect.bottom -= fPane2MinSize;
									}
									
								else
									{
										dragLimitRect = dragBarRect;
									}
																			
							}			
							
// Drag it

						totalDragDist = ::DragGrayRgn(	(RgnHandle)dragBarRgn, 
																						inMouseDown.whereLocal, 
																						&dragLimitRect, 
																						&slopRect, 
																						fIsVertical ? hAxisOnly : vAxisOnly, 
																						NULL );
	
						if ( (totalDragDist != 0x80008000) && (totalDragDist != 0) )
							{

//  now we need to erase over the new drag bar rect
//  so it gets rid of random garbage

									
								hDragDist = LoWord( totalDragDist );
								vDragDist = HiWord( totalDragDist );
									
//  this is so hi tech
						
								if ( fIsVertical )
									{
										
										fPane1->ResizeFrameBy( hDragDist, 0, false );
										fPane2->MoveBy( hDragDist, 0, false );
										fPane2->ResizeFrameBy( -hDragDist, 0, false );
								
									}
									
								else
									{
									
										fPane1->ResizeFrameBy( 0, vDragDist, false );
										fPane2->MoveBy( 0, vDragDist, false );
										fPane2->ResizeFrameBy( 0, -vDragDist, false );
								
									}

//  refocus because messing with the above panes might
//  cause a focus change

								FocusDraw();
								
								::OffsetRect( &dragBarRect, hDragDist, vDragDist );
								::EraseRect( &dragBarRect );			
								
								InvalPortRect(&dragBarRect);
								
								fPane1->Refresh();
								fPane2->Refresh();	
								Refresh();
									
							}
							
					}
					
			}
			
	}
	

void 
CDividedView::AdjustCursorSelf( Point inPortPt, const EventRecord &inMacEvent )

	{
		
		ResIDT cursID = HORI_CURSOR;
		
		if ( fIsVertical )
			cursID = VERT_CURSOR;
			
		CursHandle h = ::GetCursor( cursID );
		SignalIf_( h == NULL );
		
		if ( h != NULL )
			::SetCursor( *h );

	}


void
CDividedView::DrawSelf()
	{
	
		Rect	divPaneRect = { 0, 0, 0, 0 };
		Rect	barRect		= { 0, 0, 0, 0 };
		SDimension16 topLeftPaneSize = { 0, 0 };
		
		if ( fPane1 != NULL )
			{
			
				CalcLocalFrameRect( divPaneRect );
				fPane1->GetFrameSize( topLeftPaneSize );
		
				barRect = divPaneRect;
				
				if ( fIsVertical )
					{
					
						barRect.left += topLeftPaneSize.width;
						barRect.right = barRect.left + fDragBarSize;
						
					}
				else
					{
					
						barRect.top += topLeftPaneSize.height;
						barRect.bottom = barRect.top + fDragBarSize;
						
					}
				
				Rect				frameRect 	= barRect;
				RGBColor		lightGrey 	= { 0xFFFF, 0xFFFF, 0xFFFF };
				RGBColor		mediumGrey 	= { 0xEEEE, 0xEEEE, 0xEEEE };
				RGBColor		darkGrey 		= { 0xBBBB, 0xBBBB, 0xBBBB };
				StColorPenState	savedState;
				
				savedState.Normalize();
				
				BackColor(whiteColor);
				ForeColor(blackColor);

				::RGBForeColor(&lightGrey);

				MoveTo(frameRect.left, frameRect.bottom - 2);
				LineTo(frameRect.left, frameRect.top);
				LineTo(frameRect.right - 2, frameRect.top);
				
				::RGBForeColor(&darkGrey);
				
				MoveTo(frameRect.right - 1, frameRect.top + 1);
				LineTo(frameRect.right - 1, frameRect.bottom - 1);
				LineTo(frameRect.left + 1, frameRect.bottom - 1);
				
				::RGBForeColor(&mediumGrey);
				
				MoveTo(frameRect.left, frameRect.bottom - 1);
				Line(0, 0);

				MoveTo(frameRect.right - 1, frameRect.top);
				Line(0, 0);

				::InsetRect(&frameRect, 1, 1);
				::PaintRect(&frameRect);
		
			}
	
	}
	
	
	
#pragma mark -


void
CDividedView::SetupSubPanes()
	{

//  SetupSubPanes() sets up the initial subviews with respect to minimum 
//  and initial size values.

		SDimension16	divPaneSize	= { 0, 0 };
		SPoint32			divPaneLoc	= { 0, 0 };
		
		if ( (fPane1 != NULL) && (fPane2 != NULL) )
			{
			
				GetFrameSize( divPaneSize );
				GetFrameLocation( divPaneLoc );
				
				if ( fIsVertical )
					{
			
//  Situation 1: pane 1 with initial size and pane 2 with min. size fit in view

						if ( (fInitialDelta + fPane2MinSize + fDragBarSize) <= divPaneSize.width )
							{
					
								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( fInitialDelta, divPaneSize.height, false );
								
								fPane2->PlaceInSuperFrameAt( fInitialDelta + fDragBarSize, 0, false );
								fPane2->ResizeFrameTo( divPaneSize.width - (fInitialDelta + fDragBarSize), divPaneSize.height, false );
								
								fPane1->Refresh();
								fPane2->Refresh();			
								Refresh();
								
							}

//  Situation 2: pane 1 with min size and pane 2 with min size fit in view

						else if ( (fPane1MinSize + fPane2MinSize + fDragBarSize) <= divPaneSize.width )
							{

								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( divPaneSize.width - fDragBarSize - fPane2MinSize, divPaneSize.height, false );
								
								fPane2->PlaceInSuperFrameAt( divPaneSize.width - fPane2MinSize, 0, false );
								fPane2->ResizeFrameTo( fPane2MinSize, divPaneSize.height, false );
								
								fPane1->Refresh();
								fPane2->Refresh();								
								Refresh();
								
							}
	
//  Situation 3: pane 1 and 2 with min size don't fit in view, resize to fit
				
						else if ( (fDragBarSize + 2) <= divPaneSize.width )
							{
							
								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( (divPaneSize.width - fDragBarSize) / 2, divPaneSize.height, false );
								
								fPane2->PlaceInSuperFrameAt( divPaneSize.width - ((divPaneSize.width - fDragBarSize) / 2), 0, false );
								fPane2->ResizeFrameTo( divPaneSize.width - ((divPaneSize.width - fDragBarSize) / 2), divPaneSize.height, false );
								
								fPane1->Refresh();
								fPane2->Refresh();								
								Refresh();
								
							}

//  Situation 4: the view is just too small. Hide it.
					
						else
							{
						
								Hide();
							
							}
					
					}
					
				else				// horizontal
					{
										
						if ( (fInitialDelta + fPane2MinSize + fDragBarSize) <= divPaneSize.height )
							{
							
								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( divPaneSize.width, fInitialDelta, false );
								
								fPane2->PlaceInSuperFrameAt( 0, fInitialDelta + fDragBarSize, false );
								fPane2->ResizeFrameTo( divPaneSize.width, divPaneSize.height - (fInitialDelta + fDragBarSize), false );
								
								fPane1->Refresh();
								fPane2->Refresh();
								Refresh();
								
							}

						else if ( (fPane1MinSize + fPane2MinSize + fDragBarSize) <= divPaneSize.height )
							{
											
								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( divPaneSize.width, divPaneSize.height - fPane2MinSize - fDragBarSize, false );
								
								fPane2->PlaceInSuperFrameAt( 0, divPaneSize.height - fPane2MinSize, false );
								fPane2->ResizeFrameTo( divPaneSize.width, fPane2MinSize, false );
								
								fPane1->Refresh();
								fPane2->Refresh();
								Refresh();
								
							}
							
						else if ( (fDragBarSize + 2) <= divPaneSize.height )
							{
							
								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( divPaneSize.width, (divPaneSize.height - fDragBarSize) / 2, false );
								
								fPane2->PlaceInSuperFrameAt( 0, divPaneSize.height - ((divPaneSize.height - fDragBarSize) / 2), false );
								fPane2->ResizeFrameTo( divPaneSize.width, divPaneSize.height - ((divPaneSize.height - fDragBarSize) / 2), false );
								
								fPane1->Refresh();
								fPane2->Refresh();
								Refresh();
									
							}
							
						else
							{
							
								Hide();
								
							}
					
					}
					
			}
			
	}
	
	
void
CDividedView::ReadjustSubPanes()
	{

//  ReadjustSubPanes() re-arranges our two subviews when the window is
//  resized.
	
		SDimension16	divPaneSize		= { 0, 0 };
		SDimension16	pane1Size			= { 0, 0 };
		SDimension16	pane2Size			= { 0, 0 };
		Int16					newPane1Size	= 0;
		Int16					newPane2Size	= 0;
		float					sizeRatio 		= 0;
				
		if ( (fPane1 != NULL) && (fPane2 != NULL) )
			{

				FocusDraw();
				
//  if we were shrunk so small that we needed to hide ourselves,
//  show the world that we exist...
			
				if ( !IsVisible() )
					Show();
					
				GetFrameSize( divPaneSize );
			
				fPane1->GetFrameSize( pane1Size );
				fPane2->GetFrameSize( pane2Size );

//  Resizing strategy:
//
//  Ok, we first try to preserve the ratio between the 2 panes. If one of
//  the panes ends up smaller than it's min. value, we make it adjust things
//  appropriately. If the window is just too small for both min. sized panes
//  to fit, we divide the view in 2. If the window is really, really small,
//  we bail and hide ourselves.
					
				if ( fIsVertical )
					{

						if ( (fPane1MinSize + fPane2MinSize + fDragBarSize) <= divPaneSize.width )
							{
					
								sizeRatio = pane1Size.width / (float)(pane1Size.width + pane2Size.width);
								newPane1Size = sizeRatio * divPaneSize.width;
								
								if ( newPane1Size < fPane1MinSize )
									newPane1Size = fPane1MinSize;

//  due to rounding errors, pane1 will tend to "creep" and get bigger
//  and bigger. we attempt to prevent that here.
	
								if ( newPane1Size > pane1Size.width )
									{
									
										if ( (newPane1Size - pane1Size.width) < 2 )
											newPane1Size = pane1Size.width;
										
									}
									
								else if ( pane1Size.width > newPane1Size )
									{
									
										if ( (pane1Size.width - newPane1Size) < 2 )
											newPane1Size = pane1Size.width;
											
									}
								
								newPane2Size = divPaneSize.width - fDragBarSize - newPane1Size;
								
								if ( newPane2Size < fPane2MinSize )
									{
									
										newPane2Size = fPane2MinSize;
										newPane1Size = divPaneSize.width - fDragBarSize - newPane2Size;
										
									}
									
								fPane1->ResizeFrameTo( newPane1Size, divPaneSize.height, false );
								
								fPane2->PlaceInSuperFrameAt( newPane1Size + fDragBarSize, 0, false );
								fPane2->ResizeFrameTo( newPane2Size, divPaneSize.height, false );
																	
								fPane1->Refresh();
								fPane2->Refresh();			
								Refresh();
								
							}

						else if ( (fDragBarSize + 2) <= divPaneSize.width )
							{

								newPane1Size = (divPaneSize.width - fDragBarSize) / 2;
								newPane2Size = divPaneSize.width - fDragBarSize - newPane1Size;
								
								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( newPane1Size, divPaneSize.height, false );
								
								fPane2->PlaceInSuperFrameAt( divPaneSize.width - newPane2Size, 0, false );
								fPane2->ResizeFrameTo( divPaneSize.width - newPane2Size, divPaneSize.height, false );
								
								fPane1->Refresh();
								fPane2->Refresh();								
								Refresh();
								
							}
				
						else
							{
						
								Hide();
							
							}
					
					}
					
				else
					{
					
						if ( (fPane1MinSize + fPane2MinSize + fDragBarSize) <= divPaneSize.height )
							{
					
								sizeRatio = pane1Size.height / (float)(pane1Size.height + pane2Size.height);
								newPane1Size = sizeRatio * divPaneSize.height;
								
								if ( newPane1Size < fPane1MinSize )
									newPane1Size = fPane1MinSize;

//  due to rounding errors, pane1 will tend to "creep" and get bigger
//  and bigger. we attempt to prevent that here.
												
								if ( newPane1Size > pane1Size.height )
									{
									
										if ( (newPane1Size - pane1Size.height) < 2 )
											newPane1Size = pane1Size.height;
										
									}
									
								else if ( pane1Size.height > newPane1Size )
									{
									
										if ( (pane1Size.height - newPane1Size) < 2 )
											newPane1Size = pane1Size.height;
											
									}
																
								newPane2Size = divPaneSize.height - fDragBarSize - newPane1Size;
								
								if ( newPane2Size < fPane2MinSize )
									{
									
										newPane2Size = fPane2MinSize;
										newPane1Size = divPaneSize.height - fDragBarSize - newPane2Size;
										
									}
									
								fPane1->ResizeFrameTo( divPaneSize.width, newPane1Size, false );
								
								fPane2->PlaceInSuperFrameAt( 0, newPane1Size + fDragBarSize, false );
								fPane2->ResizeFrameTo( divPaneSize.width, newPane2Size, false );
								
								fPane1->Refresh();
								fPane2->Refresh();			
								Refresh();
								
							}

						else if ( (fDragBarSize + 2) <= divPaneSize.width )
							{

								newPane1Size = (divPaneSize.height - fDragBarSize) / 2;
								newPane2Size = divPaneSize.height - fDragBarSize - newPane1Size;
								
								fPane1->PlaceInSuperFrameAt( 0, 0, false );
								fPane1->ResizeFrameTo( divPaneSize.width, newPane1Size, false );
								
								fPane2->PlaceInSuperFrameAt( 0, divPaneSize.height - newPane2Size, false );
								fPane2->ResizeFrameTo( divPaneSize.width, newPane2Size, false );
								
								fPane1->Refresh();
								fPane2->Refresh();								
								Refresh();
								
							}
				
						else
							{
						
								Hide();
							
							}
							
					}
					
			}
			
	}
	
	
	