//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSAddTask.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains code to run an add operation.
		 
	Modification History:

		26	July	1998		Ram Rajadhyaksha
		
					Added mappings table support for add keyword bug fix.
					
		7 	July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes. 
							
-------------------------------------------------------------------------------- */


#include "ThrowOSErr.h"
#include "CMacCVSDoc.h"
#include "CMultiProgressWindow.h"
#include "CVSEngine.h"

#include "CMacCVSAddTask.h"


CMacCVSAddTask::CMacCVSAddTask( LArray *inItems, CMacCVSDoc *inOwnerDoc ):

	CMacCVSOperationTask( inOwnerDoc )
	
	{		
		mEngineRef							= NULL;
		mProgressID							= 0;
		mOldMaxValue 						= 0;
		mIterations							= 0;	
		mReset									= false;

		if ( inItems != NULL )
			{		
				mItemList = *inItems;		//  this makes a copy of the array
			}
		else
			{
				mItemList = NULL;
			}			
	}

																					
CMacCVSAddTask::~CMacCVSAddTask()
	{	
		try
			{
				if ( mEngineRef != NULL )
					{			
						DisposeCVSEngine( mEngineRef );
						mEngineRef = NULL;			
					}
					
				if ( mProgressID != 0 )
					{
						CMultiProgressWindow *pProgressWindow = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWindow != NULL );
						
						if ( pProgressWindow != NULL )
							{
								pProgressWindow->RemoveListener( this );
							}
							
						CMultiProgressWindow::DeleteGlobalProgressIndicator( mProgressID );
					}
			}
		catch ( ... )
			{
			}
	}

	
#pragma mark -


void
CMacCVSAddTask::Run()
	{
	
		OSErr									result					= noErr;
		FSSpec								projectRoot			= { 0, 0, "\p" };
		CMultiProgressWindow	*pProgressWin	 	= NULL;
		
		try
			{
				mProgressID = CMultiProgressWindow::NewGlobalProgressIndicator();
				pProgressWin =  CMultiProgressWindow::GetGlobalCMultiProgressWindow();
				ThrowIfNil_( pProgressWin );
				pProgressWin->AddListener( this );
			
				LStr255	pstrTitle;
					
				if ( mItemList.GetCount() == 1 )
					{				
						pstrTitle = "Adding: ";
						
						CMacCVSAddTaskItem sFirstItem;
						
						if ( mItemList.FetchItemAt(LArray::index_First, &sFirstItem) )
							{
								pstrTitle += sFirstItem.mFileSpec.name;		
							}
					}
				else
					{
						pstrTitle = "Adding files";
					}

				pProgressWin->SetProgressMessageText( mProgressID, pstrTitle );
				pProgressWin->SetCurrentItemText( mProgressID, "\pConnecting to server" );
				pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );
				
				CCVSEngineLoginOptions sLoginOptions;
				sLoginOptions.Init();
				
				mOwnerDoc->GetHostNamePStr( sLoginOptions.mServerName );
				sLoginOptions.mPort = mOwnerDoc->GetHostPort();
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mRemoteUserName );
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mLocalUserName );
				mOwnerDoc->GetPasswordPStr( sLoginOptions.mPassword );
				mOwnerDoc->GetCVSCommandPStr( sLoginOptions.mCVSCommandText );
				mOwnerDoc->GetCVSRootPStr( sLoginOptions.mCVSRoot );
				sLoginOptions.mAuthenticationMethod = mOwnerDoc->GetAuthenticationMethod();
				
				mOwnerDoc->GetProjectRootFileSpec( projectRoot );
								
				pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarAbsolute );
								
				// this is where we go through the array of items
				
				LArrayIterator				itemIterator( mItemList );
				CMacCVSAddTaskItem		sCurItem;
				
				while ( itemIterator.Next(&sCurItem) )
					{
						result = CreateCVSEngine( &projectRoot, mMappings, mMappingsCount, Notifier, MessageOutNotifier, ErrorOutNotifier, this, mOwnerDoc->GetSocketTimeOut(), &mEngineRef );
						ThrowOSErr( result );
						
						result = CVSEngineLogin( mEngineRef, &sLoginOptions );
						ThrowOSErr( result );

						result = CVSEngineAdd( mEngineRef, &sCurItem.mFileSpec, &sCurItem.mOptions );
						ThrowOSErr( result );
						
						DisposeCVSEngine( mEngineRef );
						mEngineRef = NULL;
					}
			}
		catch ( ... )
			{
				if ( result == noErr )
					{
						result = kCVSEngineUnknownErr;
					}
					
				PostOverallError( result );
			}
				
	}
	

pascal SInt32		
CMacCVSAddTask::Notifier(	const CCVSEngineRef					inEngineRef,
													const FSSpec								*inTargetItem,
													const SInt32 								inEvent, 
													const SInt32 								inError, 
													const UInt32 								inCookie, 
													const void 									*inDataPtr, 
													void 												*inContextPtr)
	{

		const	unsigned char	pstrUpdating[] 			= "\pUpdating: ";
	
		OSErr								error								= noErr;
		Boolean							bResult							= true;
		UInt32							intCopyLen					= 0;		
		
		Str255							pstrPath						= "\p";
		Str255							pstrItem						= "\p";
		CMacCVSAddTask			*theTask						= (CMacCVSAddTask *)inContextPtr;
		
		CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
		Assert_( pProgressWin != NULL );
		
		if ( theTask != NULL )
			{			
				try
					{				
						switch ( inEvent )
							{						
								case kCVSEngineAddPreActionEvent:

									if ( inError != noErr )
										{
											theTask->PostFileError( inTargetItem, (char *)inDataPtr, inCookie, inError );
										}
										
									pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarAbsolute );	
									pProgressWin->SetCurrentItemText( theTask->mProgressID, "\pAdding Item:");
									pProgressWin->SetCurValue( theTask->mProgressID, 0 );		
									pProgressWin->SetMaxValue( theTask->mProgressID, 1 );					
										
									if ( (inDataPtr != NULL) && (inCookie > 0) )
										{									
											intCopyLen = inCookie;
											
											if ( intCopyLen > 255 )
												intCopyLen = 255;
												
											::BlockMoveData( inDataPtr, &pstrPath[1], intCopyLen );
											pstrPath[0] = intCopyLen;
											
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrPath );
										}										
										
								break;
												
								case kCVSEngineProgressEvent:

									if ( inError != noErr )
										{
											theTask->PostFileError( inTargetItem, NULL, 0, inError );
										}
										
									if ( inDataPtr != NULL )
										{									
											if ( theTask->mOldMaxValue != ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax )
												{												
													theTask->mOldMaxValue = ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax;
													pProgressWin->SetMaxValue( theTask->mProgressID, theTask->mOldMaxValue );								
												}
												
											pProgressWin->SetCurValue( theTask->mProgressID, ((CCVSEngineProgressStatusRec *)inDataPtr)->fCompleted );						
										}
													
								break;

								case kCVSEngineUpdatePreActionEvent:
										
									if ( theTask->mReset )
										{
											pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarAbsolute );
											theTask->mReset = false;
										}
																
									pProgressWin->SetCurValue( theTask->mProgressID, 0 );

									::BlockMoveData( &pstrUpdating[1], &pstrItem[1], pstrUpdating[0] );
										pstrItem[0] = pstrUpdating[0];
										
									Assert_( (inDataPtr != NULL) && (inCookie > 0) );
									
									if ( (inDataPtr != NULL) && (inCookie > 0) )
										{																							
											intCopyLen = inCookie;
											
											if ( intCopyLen > (255 - pstrItem[0]) )
												{
													intCopyLen = (255 - pstrItem[0]);
												}
												
											::BlockMoveData( inDataPtr, &pstrItem[pstrItem[0] + 1], intCopyLen );
											pstrItem[0] += intCopyLen;
											
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );											
										}
									else 	
										{
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );
										}
										
								break;
								
								case kCVSEngineUpdatePostActionEvent:

									if ( (inError != noErr) && (inError != kCVSEngineSkippedFile) )
										theTask->PostFileError( inTargetItem, (char *)inDataPtr, inCookie, inError );
																					
								break;
								
								case kCVSEngineAddPostActionEvent:

//  Don't report kCVSEngineServerReturnedErrs because the actual 
//  CVS Engine call will return it and we will report it then.

									if ( (inError != noErr) && (inError != kCVSEngineServerReturnedErr) )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );

									if ( inError == noErr )
										{								
											theTask->mIterations++;
											pProgressWin->SetProgressCountText( theTask->mProgressID, theTask->mIterations );							
										}

									if ( inTargetItem != NULL )
										{
											theTask->mOwnerDoc->UpdateItemInProjectViews( inTargetItem );										
										}
										
								break;

								case kCVSEngineCommandActionEvent:
								
									pProgressWin->SetCurrentItemText( theTask->mProgressID, "\pWaiting for response" );
									pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );
									theTask->mReset = true;
									
								break;
							
								case kCVSEngineYieldActionEvent:
								
									CThreadedTask::Yield();
																			
								break;
								
								default:
								
									if ( inError != noErr )
										{
											theTask->PostFileError( inTargetItem, NULL, 0, inError );
										}
										
								break;						
							}				
					}			
				catch ( ... )
					{				
						if ( inEvent == kCVSEngineYieldActionEvent )
							{
								bResult = false;
							}
					}
					
			}
					
		return bResult;
		
	}
	
	
void		
CMacCVSAddTask::ListenToMessage( MessageT inMessage, void *ioParam )
	{	
		if ( ioParam != NULL )
			{
				UInt32 intID = *(UInt32 *)ioParam;
				
				if ( (intID == mProgressID) && (inMessage == msg_Cancel) )
					{
						ShutdownThread();
					}	
			}	
	}
	
	
void
CMacCVSAddTask::GetTaskName (Str255 outNamePStr)
	{
		LStr255		theNameStr;

		theNameStr = "\padd";
			
		if ( theNameStr[0] > 0 )
			::BlockMoveData( theNameStr, outNamePStr, theNameStr[0] + 1 );
	}
	