//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSApp.cp
	
	Authors:		Dan Clifford, Ram Rajadhyaksha
	
	Description:
	
		This file contains the main application routine and CMacCVSApp object.
		 
	Modification History:

		11 Apr	1999		Josef W. Wankerl
		
					Register the CLocalTreeView class to support drops of folders
					in the preferences local tree view area.
		
		10 Apr	1999		Josef W. Wankerl
		
					Made Navigation Services aware.
		
		7 July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes. 
							
-------------------------------------------------------------------------------- */


#include <OpenTransport.h>
#include <Appearance.h>

#if __profile__
#include <UProfiler.h>
#endif

#include <Files.h>
#include <Threads.h>
#include <StandardFile.h>

#include <PP_Messages.h>
#include <PP_Resources.h>
#include <PPobClasses.h>
#include <URegistrar.h>
#include <UControlRegistry.h>
#include <UEnvironment.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <UModalDialogs.h>
#include <UConditionalDialogs.h>		// JWW - For Navigation Services

#include <LGrowZone.h>
#include <LWindow.h>
#include <LEditField.h>
#include <LIconPane.h>
#include <LCicnButton.h>
#include <LGroupBox.h>
#include <LCheckBoxGroupBox.h>
#include <LRadioGroupView.h>
#include <LTextButton.h>
#include <LTable.h>
#include <LClipboard.h>
#include <LFileStream.h>
#include <LOffscreenView.h>
#include <LTextEditView.h>
#include <LScrollerView.h>

#include "TrapTester.h"
#include "StringUtils.h"

#include "MacCVSRandomFrontEndCrap.h"

#include "CWindowMenu.h"
#include "UErrorDialog.h"

#include "CThreadedTask.h"

#include "CTagPopup.h"
#include "CMacCVSLurkersView.h"
#include "CMacCVSLurkersWindow.h"

#include "CMacCVSMessageWindow.h"
#include "CMacCVSMessageView.h"

#include "CMultiProgressWindow.h"
#include "CSpinIndicator.h"
#include "CMacCVSProjectViewHeader.h"
#include "CMacCVSProjectViewWindow.h"
#include "CMacCVSProjectView.h"

#include "CMacCVSMappingsView.h"

#include "CMacCVSCompareWindow.h"
#include "CCompareWithOriginalView.h"
#include "CBroadcastTableTextView.h"
#include "CDividedView.h"
#include "CCompareSwitcherView.h"

#include "CMacCVSLogWindow.h"
#include "CMacCVSLogView.h"
#include "CMacCVSStatusWindow.h"

#include "CMacCVSAddTask.h"
#include "CMacCVSCommitTask.h"
#include "CMacCVSUpdateTask.h"
#include "CMacCVSImportTask.h"
#include "CMacCVSRemoveTask.h"
#include "CMacCVSTagTask.h"

#include "CLocalTreeView.h"		// JWW - For folder droping on local tree view

#include "CMacCVSDoc.h"
#include "CMacCVSApp.h"


//
// ===========================================================================
//		 Main Program
// ===========================================================================


static LFileStream 	*gLogStream = NULL;
LApplication 				*CMacCVSApp::sApplication = NULL;


void main( void )
	{
		
		OSErr 	error 					= noErr;
		SInt32	gestaltResponse = 0;

#if __profile__
		StProfileSection		profiler("\pMacCVS dump", 2500, 100);
#endif

		try
			{
								
//  Set Debugging options
				
				SetDebugThrow_( debugAction_Nothing );
				SetDebugSignal_( debugAction_LowLevelDebugger );

				InitializeHeap( 3 );		
				UQDGlobals::InitializeToolbox( &qd );

//  Check for Gestalt	
	
				if ( TrapAvailable(_Gestalt) == false )
					AlertUser( eOldSystemErr, true );

//  Check for Drag and Drop
	
				error = Gestalt( gestaltDragMgrAttr, &gestaltResponse );

				if ( (error != noErr) || ((gestaltResponse & (1 << gestaltDragMgrPresent)) == 0) )
					AlertUser( eNoDragAndDropErr, true );

//  Check for Threads

				error = Gestalt( gestaltThreadMgrAttr, &gestaltResponse );
	
				if ( (error != noErr) || ((gestaltResponse & (1 << gestaltThreadMgrPresent)) == 0) )		
					AlertUser( eNoThreadsMgrErr, true );

//  Check for Open Transport

				error = Gestalt( kOTSelector, &gestaltResponse );

				if ( (error != noErr) || (gestaltResponse == 0) )		
					AlertUser( eNoOTAvailableErr, true );
			
				error = InitOpenTransport();
	
				if ( error != noErr )
					AlertUser( eOTWontInitErr, true );
				
				new LGrowZone( CMacCVSApp::kDefaultGrowZone );		

//  Initialize our threads class

				CThreadedTask::Initialize();
				
				CMacCVSApp	theApp;		

				theApp.Run();
				
			}			
		catch ( ... )
			{
			
			}
			
		try
			{
				CMultiProgressWindow::DisposeGlobalCMultiProgressWindow();
			}
		catch ( ... )
			{
			
			}
	}


//################################################################################
//################################################################################
#pragma mark -


CMacCVSApp::CMacCVSApp()
	{
		mAppResRef = ::CurResFile();
		mSleepTime = 1;				//this gets modified depending on whether we are on duty or not, and what threads are active

// Init the environment. Why doesn't LApplication do this?
		
		UEnvironment::InitEnvironment();
		
//  Register functions to create core PowerPlant classes
		
		RegisterClass_(LButton);
		RegisterClass_(LCaption);
		RegisterClass_(LDialogBox);
		RegisterClass_(LEditField);
		RegisterClass_(LListBox);
		RegisterClass_(LPane);
		RegisterClass_(LPicture);
		RegisterClass_(LPlaceHolder);
		RegisterClass_(LPrintout);
		RegisterClass_(LScroller);
		RegisterClass_(LStdControl);					// still needed?
		RegisterClass_(LStdButton);						// still needed?
		RegisterClass_(LStdCheckBox);					// still needed?
		RegisterClass_(LStdRadioButton);			// still needed?
		RegisterClass_(LStdPopupMenu);				// still needed?
		RegisterClass_(LView);
		RegisterClass_(LWindow);

		RegisterClass_(LRadioGroup);
		RegisterClass_(LTabGroup);
		RegisterClass_(LRadioGroupView);
		RegisterClass_(LCheckBoxGroupBox);
		
		RegisterClass_(LCicnButton);
		RegisterClass_(LOffscreenView);
		//RegisterClass_(LActiveScroller);
		RegisterClass_(LTableView);
		RegisterClass_(LIconPane);
		RegisterClass_(LGroupBox);
		RegisterClass_(LTextButton);
		RegisterClass_(LTextEditView);
		RegisterClass_(LColorEraseAttachment);
		
		UControlRegistry::RegisterClasses();

		RegisterClass_(CSpinIndicator);
		RegisterClass_(CTagPopup);

		RegisterClass_(CMacCVSProjectViewHeader);

		RegisterClass_(CMacCVSProjectViewWindow);
		RegisterClass_(CMacCVSProjectView);
		
		RegisterClass_(CMacCVSLurkersWindow);
		RegisterClass_(CMacCVSLurkersView);

		RegisterClass_(CMacCVSMappingsView);
		
		RegisterClass_(CMacCVSMessageWindow);
		RegisterClass_(CMacCVSMessageView);
		
		RegisterClass_(CMultiProgressWindow);
				
		RegisterClass_(CMacCVSLogWindow);
		RegisterClass_(CMacCVSLogView);
		
		RegisterClass_(CMacCVSStatusWindow);
		
		RegisterClass_(CMacCVSCompareWindow);
		RegisterClass_(CCompareSwitcherView);
		RegisterClass_(CBroadcastTableTextView);
		RegisterClass_(CCompareWithOriginalView);
		RegisterClass_(CDividedView);
			
		// JWW - For folder dropping on local tree view
		RegisterClass_(CLocalTreeView);
		
		sApplication = this;
		
		new LClipboard;
		
	}


CMacCVSApp::~CMacCVSApp()
	{

	}


void
CMacCVSApp::Initialize()
	{

//  Add the window menu attachment

		CWindowMenu		*windowMenu = new CWindowMenu(135);

		AddAttachment( windowMenu );

	}


//################################################################################
//################################################################################
#pragma mark -


void
CMacCVSApp::ShowAboutBox()
	{
	
		Rect				rOffscreen 				= { 0, 0, 0, 0 };
		Rect				rOnscreen					= { 0, 0, 0, 0 };
		Rect				rZoomFrom					= { 0, 0, 0, 0 };
		Rect				rDialog						= { 0, 0, 0, 0 };
		
		GDHandle		hMainDevice 			= ::GetMainDevice();
		
		GWorldPtr		pTextWorld				= NULL;
		GWorldPtr		pOldWorld					= NULL;
		
		GDHandle		hOldDevice				= NULL;
		
		SInt16			intCurOffset 			= 2 * kAboutBoxRowHeight;
		SInt32			intLastTick				= 0;
		SInt32			intYieldCount			= 0;
		
		Int32				intSleepTime			= 1;
		MessageT		theMessage				= 0;
				
		GrafPtr			aboutBoxPort			= NULL;

		Boolean			bLegalPaneShowing = false;
		
		try 
			{
				{	
					CSplashDialogHandler aboutHandler( 136, this );

					LDialogBox *aboutBox = dynamic_cast<LDialogBox *>( aboutHandler.GetDialog() );
					ThrowIfNil_( aboutBox );
						
//  set the version caption to this version
			
					LCaption *versionCaption = dynamic_cast<LCaption *>( aboutBox->FindPaneByID('verC') );
					ThrowIfNil_( versionCaption );
					
					Str255		versionString = "\p";
					GetVersionPStr(versionString);
					versionCaption->SetDescriptor(versionString);
					
					LPushButton *creditsButton = dynamic_cast<LPushButton *>( aboutBox->FindPaneByID('CRBT') );
					ThrowIfNil_( creditsButton );
					creditsButton->Disable();
					
					LPushButton *legalButton = dynamic_cast<LPushButton *>( aboutBox->FindPaneByID('LEBT') );
					ThrowIfNil_( legalButton );
					legalButton->Enable();
					
					LScrollerView	*scroller = dynamic_cast<LScrollerView *>( aboutBox->FindPaneByID('SCRL') );
					ThrowIfNil_( scroller );
					scroller->Hide();
					
					LTextEditView *pTextView = dynamic_cast<LTextEditView *>( aboutBox->FindPaneByID('LEGL') );
					ThrowIfNil_( pTextView );
					
					LPane	*creditsPane = aboutBox->FindPaneByID('CRED');
					ThrowIfNil_( creditsPane );
					creditsPane->Hide();
					
					creditsPane->CalcPortFrameRect( rOnscreen );
					
					rOffscreen = rOnscreen;
					::OffsetRect( &rOffscreen, -rOffscreen.left, -rOffscreen.top );
								
					//::NewGWorld( &pTextWorld, 8, &rOffscreen, NULL, hMainDevice, noNewDevice );
							
					::NewGWorld( &pTextWorld, 0, &rOffscreen, NULL, NULL, useTempMem );
					
					Assert_( pTextWorld != NULL );
					
					if ( pTextWorld != NULL ) 
						{
						
//  make sure pix map is locked down. If we cannot allocate
//  memory for it, then don't deal with the GWorld at all
				
							if ( !::LockPixels(::GetGWorldPixMap(pTextWorld)) )
								{						
									::DisposeGWorld( pTextWorld );
									pTextWorld = NULL;					
								}						
						}

//  do the gratuitous zoom rects
							
					aboutBox->GetGlobalBounds( rDialog );		
					ZoomRects( &rZoomFrom, &rDialog, kNumberOfZoomSteps + 2, kZoomAccelerate );
					aboutBox->Show();
					
					intSleepTime = mSleepTime;		
					SetSleepTime(0);
					
					while ( theMessage != msg_OK )
						{
							theMessage = aboutHandler.DoDialog();

							if ( theMessage == 'LEBT' )			// legal button
								{					
									aboutBox->FocusDraw();
									
									Rect		scrollerRect;
									scroller->CalcPortFrameRect( scrollerRect );
									::EraseRect( &scrollerRect );
									
									scroller->Show();
									
									bLegalPaneShowing = true;
									intCurOffset = 2 * kAboutBoxRowHeight;
									intLastTick = 0;
									
									::EraseRect( &rOnscreen );
									
									creditsButton->Enable();
									legalButton->Disable();						
									
									aboutBox->SwitchTarget( pTextView );	
									UpdateMenus();			
								}						
							else if ( theMessage == 'CRBT' )		// credits button
								{
									scroller->Hide();
									
									Rect		scrollerRect;
									scroller->CalcPortFrameRect( scrollerRect );
									::EraseRect( &scrollerRect );
									
									bLegalPaneShowing = false;

									creditsButton->Disable();
									legalButton->Enable();	

									aboutBox->SwitchTarget( aboutBox );	
									UpdateMenus();					
								}
								
							if ( (::TickCount() >= kAboutBoxScrollSpeed + intLastTick) && (!bLegalPaneShowing) ) 
								{

//  DoDialog does not yield, so we have to

									if ( (intYieldCount % 20) == 0 )
										CThreadedTask::Yield();
										
									intYieldCount++;

									if ( pTextWorld != NULL ) 
										{						
											::GetGWorld( &pOldWorld, &hOldDevice );
											::SetGWorld( pTextWorld, NULL );
																												
											RenderAboutBoxText( rOffscreen, intCurOffset );
																		
											::SetGWorld( pOldWorld, hOldDevice );

											aboutBox->FocusDraw();
											aboutBoxPort = aboutBox->GetMacPort();
				
											::CopyBits(	(BitMap*)*pTextWorld->portPixMap, 
																	&aboutBoxPort->portBits,
																	&rOffscreen,
																	&rOnscreen,
																	srcCopy,
																	NULL);
										}
									else 
										{

//  If we didn't have enough memory for the GWorld, we draw 
//  direct to screen. This will look ugly but it will work.
									
											aboutBox->FocusDraw();																								
											RenderAboutBoxText( rOnscreen, intCurOffset );								
										}

									if ( intLastTick == 0 )
										{
											intLastTick = ::TickCount() + kAboutBoxScrollDelay;
										}
									else 
										{
											intLastTick = ::TickCount();
										}						
								}		
						}

					SetSleepTime( intSleepTime );
					
					if ( pTextWorld != NULL )
						{
							::DisposeGWorld( pTextWorld );
							pTextWorld = NULL;
						}
				}
				
				ZoomRects( &rDialog, &rZoomFrom, kNumberOfZoomSteps + 2, kZoomDecelerate );
			}	
		catch ( ... )
			{
				if ( pTextWorld != NULL )
					{
						::DisposeGWorld( pTextWorld );
						pTextWorld = NULL;
					}
					
				throw;
			}
				
	}


void
CMacCVSApp::RenderAboutBoxText( Rect &inFrameRect, Int16 &inOutOffset )
	{
		
		Str255			pstrTemp				= "\p";
		UInt16			intCurIndex			= 1;
		UInt16			intCurLine			= 0;
		SInt16			intCurBlend			= 0;
		UInt32			intCurGrayIndex = 0xFFFFFFFF;

		RGBColor		rgbCurGray			= { 0, 0, 0 };

		Boolean			bCenter 				= false;
		Boolean 		bNotFinished  	= false;
		
		RgnHandle		pOldClipRgn			= NULL;
		RgnHandle		pNewClipRgn			= NULL;
		
		try
			{
				::ForeColor( blackColor );
				::PenNormal();

				::TextFont( kAboutBoxTextFont );
				::TextSize( kAboutBoxTextSize );
				
				::EraseRect( &inFrameRect );
				
				pOldClipRgn = ::NewRgn();
				pNewClipRgn	= ::NewRgn();
				
				if ( (pOldClipRgn != NULL) && (pNewClipRgn != NULL) )
					{
						::RectRgn( pNewClipRgn, &inFrameRect );
						::GetClip( pOldClipRgn );
						::SetClip( pNewClipRgn );
					}
				
				while ( true ) 
					{			
						::GetIndString( pstrTemp, kAboutBoxStringID, intCurIndex );
							
						if ( pstrTemp[0] == 0 )
							break;
							
//  check for "underline row" command (a string ending in "-")
//  or a "bold" command (a string ending in "+")

						switch (pstrTemp[pstrTemp[0]])
							{
								case '-':
									pstrTemp[0]--;
									::TextFace( underline );
								break;
							
								case '+':
									pstrTemp[0]--;
									::TextFace( bold );
								break;
							}
			
						if ( pstrTemp[pstrTemp[0]] == '/' )
							{
								pstrTemp[0]--;
								
								::MoveTo(	inFrameRect.left + 5, 
													intCurLine * kAboutBoxRowHeight + inFrameRect.top + inOutOffset );	
								bNotFinished = true;	
							}
						else if ( pstrTemp[pstrTemp[0]] == '*' )
							{					
								pstrTemp[0]--;
								
								::MoveTo(	inFrameRect.left + ((inFrameRect.right - inFrameRect.left - ::StringWidth(pstrTemp)) / 2), 
													intCurLine * kAboutBoxRowHeight + inFrameRect.top + inOutOffset );
							}
						else if ( bNotFinished )
							{
								::MoveTo(	inFrameRect.left + ((inFrameRect.right - inFrameRect.left) / 2), 
													intCurLine	* kAboutBoxRowHeight + inFrameRect.top + inOutOffset );	
													
								bNotFinished = false;	
							}
						else
							{
								::MoveTo(	inFrameRect.left + 5, 
													intCurLine * kAboutBoxRowHeight + inFrameRect.top + inOutOffset );		
							}
							
						if ( (((intCurLine * kAboutBoxRowHeight) + inOutOffset) > 0) &&
								 ((intCurLine * kAboutBoxRowHeight) + inOutOffset <= (inFrameRect.bottom - inFrameRect.top)) )
							{
								::DrawString( pstrTemp );
							}
							
						::TextFace( 0 );	
						
						if ( !bNotFinished )	
							intCurLine++;			

						intCurIndex++;	
					}
				
//  check to see if we should wrap around
		
				inOutOffset--;
				
				if ( (-inOutOffset) > ((intCurIndex - 1) * kAboutBoxRowHeight) )
					inOutOffset = inFrameRect.bottom - inFrameRect.top;
					

//  ok, I like had to try 20 different modes to get this to work
	
				::PenMode( addMax );

//  loop through each blend mode and draw lines on top and bottom of
//  offscreen to create a fading effect
		
				for ( intCurBlend = 0; intCurBlend < kBlendStages; intCurBlend++ ) 
					{			
						rgbCurGray.red = rgbCurGray.blue = rgbCurGray.green = intCurGrayIndex;
						
						::RGBForeColor(&rgbCurGray);
						
						::MoveTo( inFrameRect.left, inFrameRect.top + intCurBlend );
						::Line( inFrameRect.right - inFrameRect.left, 0 );
						
						::MoveTo( inFrameRect.left, inFrameRect.bottom - intCurBlend );
						::Line( inFrameRect.right - inFrameRect.left, 0);
						
						intCurGrayIndex -= 65536L / kBlendStages;		
					}
					
				::PenNormal();
				::ForeColor( blackColor );

				if ( pOldClipRgn != NULL )
					{
						::SetClip( pOldClipRgn );
						::DisposeRgn( pOldClipRgn );
						pOldClipRgn = NULL;
					}

				if ( pNewClipRgn != NULL )
					{
						::DisposeRgn( pNewClipRgn );
						pNewClipRgn = NULL;
					}
			}
			
		catch ( ... )
			{
				if ( pOldClipRgn != NULL )
					{
						::DisposeRgn( pOldClipRgn );
						pOldClipRgn = NULL;
					}
					
				if ( pNewClipRgn != NULL )
					{
						::DisposeRgn( pNewClipRgn );
						pNewClipRgn = NULL;
					}
			
				throw;
			}
		
	}


//################################################################################
//################################################################################
#pragma mark -


void		
CMacCVSApp::ProcessNextEvent()
	{
		try
			{			
				CThreadedTask::Yield();
				
				
//	WNE optimization	
				static UInt32			sNextWNETicks = 0;
				const	UInt32			kWNEIntervalTicks = 4;

				
//	Tweak the sleep time depending on the running tasks
				Int32			theSleepTime;
					
				if ( CThreadedTask::CountHeavyWeightThreads() > 0 )
					{					
						if ( IsOnDuty() )
							{
								theSleepTime = 0;
							}
						else
							{
								theSleepTime = 2;
								sNextWNETicks = 0;
							}
					}
				else
					{
						theSleepTime = 4;
						sNextWNETicks = 0;
					}
				
				if ( sNextWNETicks < ::LMGetTicks() )
					{
					
//  There is a problem with menus being properly updated. This
//  hack solves the problem for now.

						if ( IsOnDuty() && GetUpdateCommandStatus() ) 
							{				
								UpdateMenus();				
							}
						
						SetSleepTime( theSleepTime );
						
						LApplication::ProcessNextEvent();
					
						sNextWNETicks = ::LMGetTicks() + kWNEIntervalTicks;
					}		
			}			
		catch ( OSErr inErr )
			{			
				UErrorDialog	theErrorDialog( inErr, this );			
			}		
		catch ( ExceptionCode inErr )
			{			
				UErrorDialog	theErrorDialog( (OSErr)inErr, this );			
			}		
		catch ( ... )
			{		
				Assert_( false );
				
				//  Uh oh, we should never get here.
				
				UErrorDialog	theErrorDialog( (OSErr)0, this );
			}
	
	}



void
CMacCVSApp::ClickMenuBar( const EventRecord	&inMacEvent )
	{

//  We shouldn't call this funky inherited method but it's too
//  late in the development cycle for me to change this.
		
		UpdateMenus();
		LDocApplication::ClickMenuBar(inMacEvent);
		
	}
	

Boolean
CMacCVSApp::ObeyCommand( CommandT	inCommand, void *ioParam )
	{
		
		StandardFileReply		sReply;
		Boolean							cmdHandled = true;

		switch (inCommand) 
			{
			
				case cmd_New:
								
//					StandardPutFile("\pChoose a location for the session:", "\pUntitled Session", &sReply);
					
					// JWW - Use Navigation Services if possible
					bool isReplacing;
					sReply.sfGood = UConditionalDialogs::AskSaveFile("\pUntitled Session", CMacCVSDoc::kMacCVSDocFileType, sReply.sfFile, isReplacing, kNavDefaultNavDlogOptions);
					
					if (sReply.sfGood)
						{							
							CreateNewDocumentAt( &(sReply.sfFile) );									
						}
				
				break;
					
				case cmd_Open:
				
					OSType typeList = CMacCVSDoc::kMacCVSDocFileType;
					
					UDesktop::Deactivate();
					
//					StandardGetFile( NULL, 1, &typeList, &sReply );

					// JWW - Use Navigation Services if possible
					sReply.sfGood = UConditionalDialogs::AskChooseOneFile(CMacCVSDoc::kMacCVSDocFileType, sReply.sfFile, kNavDefaultNavDlogOptions);

					UDesktop::Activate();
				
					if (sReply.sfGood)
						{
							OpenDocument( &sReply.sfFile );
						}
						
				break;

				default:
				
					cmdHandled = LApplication::ObeyCommand(inCommand, ioParam);
				
				break;					
			}
		
		return cmdHandled;
		
	}


void
CMacCVSApp::FindCommandStatus( CommandT	inCommand, Boolean &outEnabled, Boolean &outUsesMark, Char16 &outMark, Str255 outName )
	{

		switch (inCommand) 
			{
			
				case cmd_New:
				case cmd_Open:
				
					outEnabled = true;
				
				break;

				default:
					
//  enable the windows in the window menu

					ResIDT 	menuID;
					Int16 	menuItem;

					if (IsSyntheticCommand(inCommand, menuID, menuItem) && menuID == 135)
						{
							outEnabled = true;
						}
					else
						{
							LApplication::FindCommandStatus( inCommand, outEnabled, outUsesMark, outMark, outName );
						}
				break;			
			}
		
	}


//################################################################################
//################################################################################
#pragma mark -


void
CMacCVSApp::GetSubModelByPosition( DescType inModelID, SInt32 inPosition, AEDesc& outToken ) const
	{
		switch (inModelID)
			{		
				case cDocument:
					LDocument* theDoc = CMacCVSDoc::GetNthDocument(inPosition);
					
					if (theDoc != nil)
						{
							PutInToken(theDoc, outToken);
						}
					else
						{
							ThrowOSErr_( errAENoSuchObject );
						}
					break;
					
				default:
					Inherited::GetSubModelByPosition(inModelID, inPosition, outToken);
				break;
			}
	}


LModelObject* 
CMacCVSApp::HandleCreateElementEvent( DescType inElemClass, DescType inInsertPosition, LModelObject *inTargetObject, 
																			const AppleEvent	&inAppleEvent, AppleEvent &outAEReply )
	{
		
		switch ( inElemClass ) 
			{		
				case cDocument:
					{				
						FSSpec 					fsNewItem					= { 0, 0, "\p" };
						Str255 					pstrName					= "\p";
						OSErr 					error							= noErr;
						LModelObject		*pNewDocument			= NULL;
						
						StAEDescriptor	theDesc;
						
						theDesc.GetParamDesc( inAppleEvent, keyAEData, typeChar );						
						UExtractFromAEDesc::ThePString( theDesc.mDesc, pstrName );
						
						error = ::FSMakeFSSpec( 0, 0, pstrName, &fsNewItem );
						
						if ( error == fnfErr )
							{
								error = noErr;
							}
							
						ThrowIfOSErr_( error );
							
						//  Since CreateNewDocumentAt() can throw
						//  non-PP exceptions, we better catch them.
						
						try
							{
								pNewDocument = CreateNewDocumentAt( &fsNewItem, false );
							}
						catch ( ExceptionCode intException )
							{
								Throw_( intException );
							}
						catch ( OSErr intException )
							{
								Throw_( intException );
							}		
						catch ( ... )
							{
								Assert_( false );
								
								//  Uh oh, we should never get here.
								
								throw;
							}			
							
						return pNewDocument;					
					}
				
				break;
				
				default:
				
					return LDocApplication::HandleCreateElementEvent( inElemClass, inInsertPosition, inTargetObject, inAppleEvent, outAEReply );
					
				break;				
			}
			
	}				


//################################################################################
//################################################################################
#pragma mark -


LModelObject*
CMacCVSApp::MakeNewDocument()
	{
		
		//  This does nothing right now because our
		//  document object requires a file on the disk
		//  to exist. 
		
		return Inherited::MakeNewDocument();
		
	}


void
CMacCVSApp::OpenDocument( FSSpec *inMacFSSpec )
	{

		CMacCVSDoc *pNewDoc	= NULL;
		
		try 
			{				
				pNewDoc = (CMacCVSDoc *)CMacCVSDoc::FindFSSpecDocument( *inMacFSSpec );
								
				if ( pNewDoc == NULL )
					{				
						pNewDoc = new CMacCVSDoc( *inMacFSSpec, this );
						pNewDoc->Init();
						pNewDoc->SetSuperModel( this );						
					}					
				else
					{					
						pNewDoc->BringToFront();					
					}
			}		
		catch ( ... )
			{		
				if ( pNewDoc != NULL )
					{
						delete pNewDoc;
						pNewDoc = NULL;
					}
					
				throw;
			}
		
	}


CMacCVSDoc *
CMacCVSApp::CreateNewDocumentAt( FSSpec *inFileSpec, const Boolean inAskUser )
	{

		CMacCVSDoc 	*pNewDoc = NULL;
		
		::FSpDelete( inFileSpec );
		
		try 
			{				
				pNewDoc = new CMacCVSDoc( *inFileSpec, this );
				pNewDoc->Init();
				pNewDoc->SetSuperModel( this );	
			}		
		catch ( ... )
			{
				if ( pNewDoc != NULL )
					{
						delete pNewDoc;
						pNewDoc = NULL;
					}
					
				throw;
			}
			
		return pNewDoc;
		
	}


void
CMacCVSApp::GetVersionPStr( Str255 outVersionPStr )
	{
		
		Handle	versResource	= NULL;
		SInt16	oldResRef 		= ::CurResFile();
		
		Assert_( mAppResRef > 0 );
		
		::UseResFile( mAppResRef );
		
		versResource = ::GetResource('vers', 1);
		
		if ( versResource == NULL )
		
			outVersionPStr[0] = 0;
		
		else
			{

//  For some reason, doing a sizeof(outVersionPStr) returns 4 bytes! So,
//  pass sizeof(Str255) to the PStrCopy() function.
				
// 6 bytes to the version number string		

				PStrCopy( (const unsigned char*) *versResource + 6, outVersionPStr, sizeof(Str255) );
				
				::ReleaseResource(versResource);
				
			}
				
		Assert_( oldResRef > 0 );
		
		::UseResFile( oldResRef );
				
	}

	
#pragma mark -

	
void
CSplashDialogHandler::FindCommandStatus( CommandT inCommand, Boolean &outEnabled, Boolean &outUsesMark, Char16 &outMark, Str255 outName )
	{
		outEnabled = false;	
	}


CSplashAttachment::CSplashAttachment( LDialogBox *inDialog ) : LAttachment(msg_Click)
	{
		SetOwnerHost( NULL );
		mDialog = inDialog;
	}


void 
CSplashAttachment::ExecuteSelf(MessageT inMessage, void *ioParam)
	{		
		if (inMessage == msg_Event) 
			{				
				if (((EventRecord *)ioParam)->what == mouseDown)
					BroadcastMessage(msg_Cancel);				
			}		
		else if (inMessage == msg_Click) 
			{			
				BroadcastMessage(msg_Cancel);			
			}	
	}


#pragma mark -


//  This used to be used for logging the TCP streams but is
//  now obsolete.


void
OutputToLog(char *logString, Size logStringSize)
	{
		if ( gLogStream == NULL ) 
			{
			
				ProcessInfoRec		informationRecord;
				ProcessSerialNumber	psn = { 0, kCurrentProcess };
				FSSpec				processSpec;
				
				informationRecord.processInfoLength = sizeof(ProcessInfoRec);
				informationRecord.processAppSpec = &processSpec;
				informationRecord.processName = NULL;
				
				GetProcessInformation(&psn, &informationRecord);
				
				BlockMoveData("\pMacCVS Log", processSpec.name, 11);
				
				FSpDelete(&processSpec);
				
				gLogStream = new LFileStream(processSpec);
				gLogStream->CreateNewFile('CWIE', 'TEXT');
				gLogStream->OpenDataFork(fsRdWrPerm);
		
			}
		
		gLogStream->WriteData(logString, logStringSize);

		if (logString[logStringSize] != '\n') 
			{		
				logStringSize = 1;
				gLogStream->WriteData("\n", logStringSize);		
			}	
		
	}
