//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSAppleEventItem.cp
	
	Authors:		Simon Fraser
	
	Description:
	
		This file contains all the code for file objects, which are used for
		Apple Event handling
		 
	Modification History:

							
-------------------------------------------------------------------------------- */


#include <string.h>
#include <LCommander.h>

#include "StringUtils.h"
#include "CMacCVSDoc.h"
#include "CVSEngineUtils.h"
#include "UProjectActions.h"
#include "CCVSEngineItemInfoThingy.h"

#include "CMacCVSAppleEventItem.h"


CMacCVSAppleEventItem::TStatusEnumMap CMacCVSAppleEventItem::sStatusEnumMap[] = {
		
		{		eStatusUnowned,						kMacCVSProjectorStatusUnowned				},
		{		eStatusCheckedOut,				kMacCVSProjectorStatusCheckedOut		},
		{		eStatusLocallyModified,		kMacCVSProjectorStatusMRO						},
		{		eStatusCheckedIn,					kMacCVSProjectorStatusCheckedIn			},
		{		eStatusBroken,						kMacCVSProjectorStatusBroken				},
		{		eStatusCorrupt,						kMacCVSProjectorStatusCorrupt				},
		{		eStatusDead,							kMacCVSProjectorStatusDead					},
		{		eStatusNone,							-1	}				// marks list end
	
	};
	

CMacCVSAppleEventItem::CMacCVSAppleEventItem(CMacCVSDoc* inParentDoc, const FSSpec& fileSpec):	
	LModelObject(inParentDoc, cMacCVSAppleEventItem)
	{
		
		mParentDoc = inParentDoc;
		mFileSpec = fileSpec;
			
		// check that the file exists
		
		OSErr	err = ::FSpIsAFile( &mFileSpec, NULL );
		
		if ( err == kCVSEngineIsFolder )
			err = noErr;
			
		ThrowIfOSErr_( err );
		
		AddListener( mParentDoc );
	}
	
	
	
CMacCVSAppleEventItem::~CMacCVSAppleEventItem()
	{
	}
	

#pragma mark -
#pragma mark APPLE EVENTS


void
CMacCVSAppleEventItem::MakeSelfSpecifier( AEDesc&	inSuperSpecifier, AEDesc&	outSelfSpecifier) const
	{

		StAEDescriptor	keyData;
		OSErr	err = ::AECreateDesc(typeChar, mFileSpec.name + 1, mFileSpec.name[0], &keyData.mDesc);
		ThrowIfOSErr_(err);
		
		err = ::CreateObjSpecifier(cMacCVSAppleEventItem, &inSuperSpecifier, formName, &keyData.mDesc, false, &outSelfSpecifier);
		ThrowIfOSErr_(err);
		
	}
	
	
	
void
CMacCVSAppleEventItem::GetSubModelByName( DescType inModelID, Str255 inName, AEDesc &outToken ) const
	{
	
		Boolean bFoundIt = false;
		
		switch (inModelID)
			{		
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					{
						FSSpec fsItem = { 0 };
						
						OSErr	err = ::FSpGetFromLocalCVSPath( inName, &mFileSpec, &fsItem );
						
						if ( err == kCVSEngineIsFolder )
							{
								err = noErr;
							}
							
						if ( err != noErr )
							{
								ThrowOSErr_(err);
							}
							
						// create an object for this file					
						CMacCVSAppleEventItem	*pProjectItem = new CMacCVSAppleEventItem( mParentDoc, fsItem );
						ThrowIfNil_( pProjectItem );
										
						pProjectItem->SetLaziness( true );
						
						PutInToken( pProjectItem, outToken );
					}
				break;
					
				default:
					Inherited::GetSubModelByName(inModelID, inName, outToken);
				break;			
			}
	}


void
CMacCVSAppleEventItem::GetSubModelByPosition( DescType inModelID, SInt32 inPosition, AEDesc &outToken ) const
	{

		switch (inModelID)
			{			
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					{
						FSSpec fsItem = { 0 };
						
						OSErr	err = FSpGetNthItem( &mFileSpec, inPosition, &fsItem );
						ThrowIfOSErr_(err);
						
						// create an object for this file					
						CMacCVSAppleEventItem	 *pProjectItem = new CMacCVSAppleEventItem( mParentDoc, fsItem );
						ThrowIfNil_( pProjectItem );
						
						pProjectItem->SetLaziness( true );
						
						PutInToken( pProjectItem, outToken );
					}
				break;
										
				default:		
					Inherited::GetSubModelByPosition(inModelID, inPosition, outToken);
				break;
			}
			
	}
	

SInt32
CMacCVSAppleEventItem::CountSubModels(DescType inModelID) const
	{
		
		SInt32 intSubModelCount = 0;
		
		switch (inModelID)
			{			
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					{
						UInt32 numItems = 0;
					
						OSErr	err = FSpCountNumFiles( &mFileSpec, &numItems );
						ThrowIfOSErr_( err );
					
						intSubModelCount = numItems;
					}	
				break;
					
					
				default:					
					intSubModelCount = Inherited::CountSubModels(inModelID);
				break;
			}
	
		return intSubModelCount;
		
	}


// support 'whose' clauses
void
CMacCVSAppleEventItem::GetSubModelByComplexKey(	DescType inModelID, DescType inKeyForm, 
																								const AEDesc& inKeyData, AEDesc& outToken) const
	{

		switch (inModelID)
			{
			
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					
					switch ( inKeyForm )
						{						
							case formName:
							
							break;
								
							case CMacCVSAppleEventItem::pPathProperty:
								
							break;
								
							case CMacCVSAppleEventItem::pStatusProperty:
								
							break;						
						}				
						
				break;
					
					
				default:					
					ThrowOSErr_(errAENoSuchObject);

					// this is bad; it calls Debugger() in optimized builds!
					//Inherited::GetSubModelByComplexKey(inModelID, inKeyForm, inKeyData, outToken);
				break;
			}
			
	}


void
CMacCVSAppleEventItem::GetAEProperty( DescType inProperty, const AEDesc &inRequestedType, AEDesc &outPropertyDesc ) const
	{

		OSErr	err = noErr;

		switch ( inProperty ) 
			{
			
				case pName:			
					err = ::AECreateDesc(typeChar, (Ptr) mFileSpec.name + 1, StrLength(mFileSpec.name), &outPropertyDesc);				
				break;
					
				case pPathProperty:
					{
						FSSpec		rootSpec;
						mParentDoc->GetProjectRootFileSpec(rootSpec);
						
						const UInt32		kMaxPathLen = 1024;
						
						UInt32		pathLen = kMaxPathLen;
						char			path[kMaxPathLen + 1];
						
						err = ::FSpGetLocalCVSPath(&mFileSpec, &rootSpec, path, &pathLen);
						ThrowIfOSErr_(err);
						
						err = ::AECreateDesc(typeChar, (Ptr)path, pathLen, &outPropertyDesc);
					}
				break;
					
				case pStatusProperty:
					{
						//CMacCVSProjectViewItem		*projectFile = GetProjectViewItem(true);
						
						//DescType			statusEnum = ItemStatusToEnum(projectFile->GetProjectItemStatus());
						
						//err = ::AECreateDesc(typeEnumerated, (Ptr)&statusEnum, sizeof(DescType), &outPropertyDesc);
						//ThrowIfOSErr_(err);
					}
				break;
				
				case pRevisionProperty:
					{
					/*
						CMacCVSProjectViewItem		*projectItem = GetProjectViewItem(true);
						CMacCVSProjectViewFile		*projectFile = dynamic_cast<CMacCVSProjectViewFile *>(projectItem);						
						ThrowIfNil_(projectFile);
						
						Str255		versionStr;
						
						projectFile->GetVersionPStr(versionStr);
						
						err = ::AECreateDesc(typeChar, (Ptr)versionStr + 1, StrLength(versionStr), &outPropertyDesc);

						ThrowIfOSErr_(err);*/
					}
				break;
						
				default:			
					Inherited::GetAEProperty(inProperty, inRequestedType, outPropertyDesc);					
				break;					
			}

	}
	
	
void
CMacCVSAppleEventItem::SetAEProperty( DescType inProperty, const AEDesc &inValue, AEDesc &outAEReply )
	{
		
		OSErr	err = noErr;

		switch ( inProperty ) 
			{			
				default:				
					Inherited::SetAEProperty( inProperty, inValue, outAEReply );					
				break;					
			}

	}
	

void
CMacCVSAppleEventItem::HandleModifyReadOnlyEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{
		
		OSErr											result		= noErr;	
		Boolean										bCanDoIt 	= false;
		CCVSEngineItemInfoThingy	cCVSItem;
		
		result = cCVSItem.CanMRO( mFileSpec, bCanDoIt );
		ThrowIfOSErr_( result );
		
		if ( !bCanDoIt )
			{
				ThrowIfOSErr_( paramErr );
			}
	
		LArray				theItemList(sizeof(FSSpec));
		
		theItemList.InsertItemsAt( 1, LArray::index_Last, &mFileSpec );

		// this is synchronous
		LCommander::PostAnAction( new CProjectMROAction(mParentDoc, &theItemList) );
		
	}
	
		
void
CMacCVSAppleEventItem::HandleCheckInEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{
		
		OSErr 					err 					= noErr;
		Boolean					bCanDoIt 			= false;
		Boolean					bHasComment		= false;
		Handle 					hComment 			= NULL;
		CkidResourcePB	sCkidInfo			= { 0 };
		
		try
			{
				CCVSEngineItemInfoThingy cCVSItem;

				err = cCVSItem.CanCommit( mFileSpec, bCanDoIt );
				ThrowIfOSErr_( err );
				
				if ( !bCanDoIt )
					{
						ThrowIfOSErr_( paramErr );
					}
			
				//  Get comment argument.
				
				try
					{
						StAEDescriptor		theRevisionDesc;
						
						theRevisionDesc.GetParamDesc( inAppleEvent, ae_CheckinComment, typeChar );

						//  make a copy of the comment data
						hComment = theRevisionDesc.mDesc.dataHandle;
						err = ::HandToHand( &hComment );
						
						if ( err != noErr )
							{
								hComment = NULL;
							}
							
						ThrowIfOSErr_( err );
					}
				catch ( ... )
					{
						hComment = NULL;
					}
				
				err = FSpReadCkidResource( &mFileSpec, &sCkidInfo, false );
			
				if ( err != noErr )
					{
						//  Uh oh, no ckid. Set up a fake one.
						
						memset( &sCkidInfo, 0, sizeof(sCkidInfo) );
						
						sCkidInfo.fModifyReadOnly = 1;
						PStrCopy( mFileSpec.name, sCkidInfo.fFileNamePStr, sizeof(sCkidInfo.fFileNamePStr) );
					}
					
				sCkidInfo.fCommentHndl = hComment;
				
				if ( hComment != NULL )
					{
						sCkidInfo.fCommentLen = ::GetHandleSize( hComment );
					}
					
				err = FSpWriteCkidResource( &mFileSpec, &sCkidInfo );
				ThrowIfOSErr_( err );
								
				bHasComment = true;
					
				LArray cItemList( sizeof(FSSpec) );
				
				cItemList.InsertItemsAt( 1, LArray::index_Last, &mFileSpec );
				
				CProjectCommitAction *pCommitAction = new CProjectCommitAction( mParentDoc, &cItemList, false );
				ThrowIfNil_( pCommitAction );
				
				pCommitAction->SetCompletionEvent( inAppleEvent, outAEReply );		
				LCommander::PostAnAction( pCommitAction );

				if ( hComment != NULL )
					{
						::DisposeHandle( hComment );
						hComment = NULL;
					}
			}
		catch ( ... )
			{
				if ( hComment != NULL )
					{
						::DisposeHandle( hComment );
						hComment = NULL;
					}
					
				throw;
			}
			
	}
	

void
CMacCVSAppleEventItem::HandleCheckOutEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{

		OSErr 		err 					= noErr;
		Boolean		bCanDoIt 			= false;
		Boolean		bClearSticky	= false;
	
		try
			{
				CCVSEngineUpdateOptions	sUpdateOptions;
				sUpdateOptions.Init();
				sUpdateOptions.mGetNewDirectories = true;
				sUpdateOptions.mMergePolicy = mParentDoc->GetMergePolicy();
				
				CCVSEngineItemInfoThingy cCVSItem;

				err = cCVSItem.CanUpdate( mFileSpec, bCanDoIt );
				ThrowIfOSErr_( err );
				
				if ( !bCanDoIt )
					{
						ThrowIfOSErr_( paramErr );
					}			

				//  Get revision argument.
			
				try
					{		
						StAEDescriptor stRevDesc;
						
						stRevDesc.GetParamDesc( inAppleEvent, CMacCVSDoc::ae_UpdateRevision, typeChar );
						UExtractFromAEDesc::ThePString( stRevDesc.mDesc, sUpdateOptions.mRevisionPStr );
					}			
				catch ( ... )
					{			
					}

				//  Get date argument.
					
				try
					{			
						StAEDescriptor stDateDesc;

						stDateDesc.GetParamDesc( inAppleEvent, CMacCVSDoc::ae_UpdateDate, typeChar );
						UExtractFromAEDesc::ThePString( stDateDesc.mDesc, sUpdateOptions.mDatePStr );
					}			
				catch ( ... )
					{			
					}

				try
					{
						StAEDescriptor	stStickyDesc;
						
						stStickyDesc.GetParamDesc( inAppleEvent, ae_ClearSticky, typeBoolean );
						UExtractFromAEDesc::TheBoolean( stStickyDesc.mDesc, bClearSticky );
					}					
				catch ( ... )
					{					
					}

				sUpdateOptions.mResetStickyTags = bClearSticky;
						
				LArray cItemList( sizeof(FSSpec) );			
				cItemList.InsertItemsAt( 1, LArray::index_Last, &mFileSpec );
				
				CProjectUpdateAction *pUpdateAction = new CProjectUpdateAction( mParentDoc, &cItemList, true, &sUpdateOptions );
				ThrowIfNil_( pUpdateAction );
				
				pUpdateAction->SetCompletionEvent( inAppleEvent, outAEReply );		
				LCommander::PostAnAction( pUpdateAction );
			}
		catch ( ... )
			{
				throw;
			}
	
	}
	

void
CMacCVSAppleEventItem::HandleOpenEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{
		
		OSErr	err = noErr;
		CMacCVSProjectViewNewViewEventRec	sBroadcast = { 0 };
		
		err = ::FSpIsAFile(&mFileSpec, nil);
		
		if ( err != kCVSEngineIsFolder )
			{
				ThrowOSErr_( 1 );
			}
		
		sBroadcast.fFileSpec = mFileSpec;
		BroadcastMessage( CMacCVSProjectView::kNewViewMessage, &sBroadcast );
		
	}
	

void
CMacCVSAppleEventItem::HandleSelectEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{

		OSErr	err = noErr;
		CMacCVSProjectViewNewViewSelectFileEventRec	sBroadcast = { 0 };
		
		FSSpec	fsParent = { 0 };
		
		FSpGetParentFSSpec( &mFileSpec, &fsParent );
		
		sBroadcast.fFolderSpec = fsParent;
		sBroadcast.fFileToSelectSpec = mFileSpec;
		BroadcastMessage( CMacCVSProjectView::kNewViewSelectFileMessage, &sBroadcast );
		
	}
	

void
CMacCVSAppleEventItem::HandleTagEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{

		OSErr 		err 							= noErr;
		Boolean		bCanDoIt 					= false;
		Boolean		bStatic						= true;
		Boolean		bRemoveExisting		= false;
		Str255		pstrTagRevision		= "\p";
		
		try
			{
				CCVSEngineTagOptions	sTagOptions;
				sTagOptions.Init();				
				
				CCVSEngineItemInfoThingy cCVSItem;

				err = cCVSItem.CanTag( mFileSpec, bCanDoIt );
				ThrowIfOSErr_( err );
				
				if ( !bCanDoIt )
					{
						ThrowIfOSErr_( paramErr );
					}			

				//  Get revision argument.		
				try
					{
						StAEDescriptor	stRevDesc;
						
						stRevDesc.GetParamDesc( inAppleEvent, ae_TagRevision, typeChar );
						UExtractFromAEDesc::ThePString( stRevDesc.mDesc, pstrTagRevision );
					}					
				catch ( ... )
					{					
					}
					
				//  Get static/branch argument.			
				try
					{
						StAEDescriptor	stStaticDesc;
						
						stStaticDesc.GetParamDesc( inAppleEvent, ae_TagIsStatic, typeBoolean );
						UExtractFromAEDesc::TheBoolean( stStaticDesc.mDesc, bStatic );
					}					
				catch ( ... )
					{					
					}

				//  Get the "remove existing tags" argument.			
				try
					{
						StAEDescriptor	stRemoveDesc;
						
						stRemoveDesc.GetParamDesc( inAppleEvent, ae_TagRemoveExisting, typeBoolean );
						UExtractFromAEDesc::TheBoolean( stRemoveDesc.mDesc, bRemoveExisting );
					}					
				catch ( ... )
					{				
					}
								
				LArray cItemList( sizeof(FSSpec) );			
				cItemList.InsertItemsAt( 1, LArray::index_Last, &mFileSpec );
	
				CProjectTagAction	*pTagAction = new CProjectTagAction( mParentDoc, &cItemList, pstrTagRevision, bStatic, bRemoveExisting );
				ThrowIfNil_( pTagAction );
		
				pTagAction->SetCompletionEvent( inAppleEvent, outAEReply );
				LCommander::PostAnAction( pTagAction );
			}
		catch ( ... )
			{
				throw;
			}
	
	}
	
	
void
CMacCVSAppleEventItem::HandleUpdateEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{

		OSErr 		err 					= noErr;
		Boolean		bCanDoIt 			= false;
		Boolean		bClearSticky	= false;
		
		try
			{
				CCVSEngineUpdateOptions	sUpdateOptions;
				sUpdateOptions.Init();				
				sUpdateOptions.mMergePolicy = mParentDoc->GetMergePolicy();
				
				CCVSEngineItemInfoThingy cCVSItem;

				err = cCVSItem.CanUpdate( mFileSpec, bCanDoIt );
				ThrowIfOSErr_( err );
				
				if ( !bCanDoIt )
					{
						ThrowIfOSErr_( paramErr );
					}			

				//  Get revision argument.			
				try
					{		
						StAEDescriptor stRevDesc;
						
						stRevDesc.GetParamDesc( inAppleEvent, CMacCVSDoc::ae_UpdateRevision, typeChar );
						UExtractFromAEDesc::ThePString( stRevDesc.mDesc, sUpdateOptions.mRevisionPStr );
					}			
				catch ( ... )
					{			
					}

				//  Get date argument.					
				try
					{			
						StAEDescriptor stDateDesc;

						stDateDesc.GetParamDesc( inAppleEvent, CMacCVSDoc::ae_UpdateDate, typeChar );
						UExtractFromAEDesc::ThePString( stDateDesc.mDesc, sUpdateOptions.mDatePStr );
					}			
				catch ( ... )
					{			
					}

				try
					{
						StAEDescriptor	stStickyDesc;
						
						stStickyDesc.GetParamDesc( inAppleEvent, ae_ClearSticky, typeBoolean );
						UExtractFromAEDesc::TheBoolean( stStickyDesc.mDesc, bClearSticky );
					}					
				catch ( ... )
					{					
					}

				sUpdateOptions.mResetStickyTags = bClearSticky;
						
				LArray cItemList( sizeof(FSSpec) );			
				cItemList.InsertItemsAt( 1, LArray::index_Last, &mFileSpec );
				
				CProjectUpdateAction *pUpdateAction = new CProjectUpdateAction( mParentDoc, &cItemList, false, &sUpdateOptions );
				ThrowIfNil_( pUpdateAction );
				
				pUpdateAction->SetCompletionEvent( inAppleEvent, outAEReply );		
				LCommander::PostAnAction( pUpdateAction );
			}
		catch ( ... )
			{
				throw;
			}

	}
	
	
void
CMacCVSAppleEventItem::HandleAppleEvent( const AppleEvent	&inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{
	
		switch ( inAENumber )
			{			
				case ae_ModifyReadOnly:
					HandleModifyReadOnlyEvent( inAppleEvent, outAEReply, outResult, inAENumber );
				break;
									
				case ae_Checkin:
					HandleCheckInEvent( inAppleEvent, outAEReply, outResult, inAENumber );
				break;

				case CMacCVSDoc::ae_Checkout:
					HandleCheckOutEvent( inAppleEvent, outAEReply, outResult, inAENumber );
				break;

				case ae_Open:
					HandleOpenEvent( inAppleEvent, outAEReply, outResult, inAENumber );
				break;
	/*			
				case ae_Select:
					HandleSelectEvent( inAppleEvent, outAEReply, outResult, inAENumber );			
				break;
	*/			
				case ae_Tag:
					HandleTagEvent( inAppleEvent, outAEReply, outResult, inAENumber );
				break;

				case CMacCVSDoc::ae_Update:
					HandleUpdateEvent( inAppleEvent, outAEReply, outResult, inAENumber );
				break;

				default:
					Inherited::HandleAppleEvent( inAppleEvent, outAEReply, outResult, inAENumber );				
				break;				
			}
	}
	
	

#pragma mark -


DescType
CMacCVSAppleEventItem::ItemStatusToEnum(ProjectItemStatus inItemStatus)
	{
		if (inItemStatus >= 0 && inItemStatus < kMacCVSProjectorTotalCount)
			return sStatusEnumMap[inItemStatus].enumValue;					// assumes status values are consecutive

		else
			return eStatusNone;

	}
	
	
ProjectItemStatus
CMacCVSAppleEventItem::EnumToItemStatus(DescType inStatusEnum)
	{
		
		TStatusEnumMap		*mapEntry = sStatusEnumMap;
		
		for (mapEntry = sStatusEnumMap; mapEntry->enumValue != eStatusNone; mapEntry ++)
			{
				if (mapEntry->enumValue == inStatusEnum)
					return mapEntry->itemStatus;
			}
			
		return kMacCVSProjectorStatusUnknown;

	}
				


