//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSCheckoutTask.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains code to run a check out operation.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes.
							
-------------------------------------------------------------------------------- */


#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "CMacCVSDoc.h"
#include "CMultiProgressWindow.h"
#include "CVSEngine.h"

#include "CMacCVSCheckoutTask.h"


CMacCVSCheckoutTask::CMacCVSCheckoutTask( const CCVSEngineCheckoutOptions *inOptions,
																					CMacCVSDoc 												*inOwnerDoc):
		
	CMacCVSOperationTask( inOwnerDoc )
	
	{
		mEngineRef							= NULL;
		mProgressID							= 0;
		mOldMaxValue 						= 0;
		mIterations 						= 0;
		mReset									= false;
			
		ThrowIfNil_( inOptions );
		
		mOptions = *inOptions;
					
		ThrowIfNil_( mOwnerDoc );
		
		//  Um, what is this for? -ram
	/*	
		if ( mOptions.mUpdateEverything )
			{
				mOwnerDoc->SetDefaultCheckoutInProgress(true);
			}
			*/
	}	


CMacCVSCheckoutTask::~CMacCVSCheckoutTask()
	{
		try
			{	
				if ( mEngineRef != NULL )
					{			
						DisposeCVSEngine( mEngineRef );			
						mEngineRef = NULL;			
					}

				if ( mProgressID != 0 )
					{
						CMultiProgressWindow *pProgressWindow = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWindow != NULL );
						
						if ( pProgressWindow != NULL )
							{
								pProgressWindow->RemoveListener( this );
							}
							
						CMultiProgressWindow::DeleteGlobalProgressIndicator( mProgressID );
					}
							
				ThrowIfNil_ ( mOwnerDoc );
				/*
				if ( mOptions.mUpdateEverything )
					mOwnerDoc->SetDefaultCheckoutInProgress(false);*/
			}
		catch ( ... )
			{
			}
	}
	
	
void
CMacCVSCheckoutTask::Run()
	{
	
		OSErr		result				= noErr;
		FSSpec	fsProjectRoot	= { 0, 0, "\p" };
				
		try
			{
				mProgressID = CMultiProgressWindow::NewGlobalProgressIndicator();
				Assert_( mProgressID != 0 );
				
				CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
				ThrowIfNil_( pProgressWin );
				
				pProgressWin->AddListener( this );
				
				LStr255	pstrItemTitle("\pChecking out: ");
				LStr255	pstrDocName;
				mOwnerDoc->GetDescriptor( pstrDocName );
				
				pstrItemTitle += pstrDocName;

				pProgressWin->SetProgressMessageText( mProgressID, pstrItemTitle );
				pProgressWin->SetCurrentItemText( mProgressID, "\pConnecting to server");
				pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );
				
				CCVSEngineLoginOptions sLoginOptions;
				sLoginOptions.Init();
				
				mOwnerDoc->GetHostNamePStr( sLoginOptions.mServerName );
				sLoginOptions.mPort = mOwnerDoc->GetHostPort();
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mRemoteUserName );
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mLocalUserName );
				mOwnerDoc->GetPasswordPStr( sLoginOptions.mPassword );
				mOwnerDoc->GetCVSCommandPStr( sLoginOptions.mCVSCommandText );
				mOwnerDoc->GetCVSRootPStr( sLoginOptions.mCVSRoot );
				sLoginOptions.mAuthenticationMethod = mOwnerDoc->GetAuthenticationMethod();
				
				mOwnerDoc->GetProjectRootFileSpec( fsProjectRoot );
				
				result = CreateCVSEngine( &fsProjectRoot, mMappings, mMappingsCount, Notifier, MessageOutNotifier, ErrorOutNotifier, this, mOwnerDoc->GetSocketTimeOut(), &mEngineRef );
				ThrowOSErr( result );
					
				result = CVSEngineLogin( mEngineRef, &sLoginOptions );
				ThrowOSErr( result );
				
				pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarAbsolute );
				
				result = CVSEngineCheckout( mEngineRef, &fsProjectRoot, &mOptions );
				ThrowOSErr( result );
			}					
		catch ( ... )
			{
				if ( result == noErr )
					{
						result = kCVSEngineUnknownErr;
					}
					
				PostOverallError( result );
			}
			
		if ( result != noErr )
			{
				AddToCompletionEvent( result );
			}
			
		SendCompletionEvent();
				
	}
	
	
pascal SInt32		
CMacCVSCheckoutTask::Notifier(	const CCVSEngineRef					inEngineRef,
																const FSSpec								*inTargetItem,
																const SInt32 								inEvent, 
																const SInt32 								inError, 
																const UInt32 								inCookie, 
																const void 									*inDataPtr, 
																void 												*inContextPtr)
	{

		const	unsigned char	pstrUpdating[] 			= "\pChecking Out: ";
		const	unsigned char	pstrSendingEntry[] 	= "\pSending: ";
		const	unsigned char	pstrDecoding[] 			= "\pDecoding: ";
	
		Boolean								bResult				= true;
		UInt32								intCopyLen		= 0;
		Str255								pstrItem			= "\p";
		FSSpec								fsTarget			= { 0, 0, "\p" };
		CMacCVSCheckoutTask		*theTask			= (CMacCVSCheckoutTask *)inContextPtr;
		
		Assert_( theTask != NULL );
		
		if ( theTask != NULL )
			{			
				try
					{
						CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWin != NULL );
	
						if ( inTargetItem != NULL )
							{
								fsTarget = *inTargetItem;
							}
					
						switch ( inEvent )
							{
								case kCVSEngineEntryPreActionEvent:
									{
										if ( inError != noErr )
											theTask->PostFileError( inTargetItem, (char *)inDataPtr, inCookie, inError );

										pProgressWin->SetCurValue( theTask->mProgressID, 0 );

										::BlockMoveData( &pstrSendingEntry[1], &pstrItem[1], pstrSendingEntry[0] );
										pstrItem[0] = pstrSendingEntry[0];
										
										Assert_( (inDataPtr != NULL) && (inCookie > 0) );
										
										if ( (inDataPtr != NULL) && (inCookie > 0) )
											{																							
												intCopyLen = inCookie;
												
												if ( intCopyLen > (255 - pstrItem[0]) )
													{
														intCopyLen = (255 - pstrItem[0]);
													}
													
												::BlockMoveData( inDataPtr, &pstrItem[pstrItem[0] + 1], intCopyLen );
												pstrItem[0] += intCopyLen;
												
												pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );											
											}
										else 
											{
												pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );
											}
											
										theTask->mIterations++;
										pProgressWin->SetProgressCountText( theTask->mProgressID, theTask->mIterations );											
									}																
								break;

								case kCVSEngineCommandActionEvent:
								
									theTask->mIterations = 0;
									pProgressWin->SetCurValue( theTask->mProgressID, 0 );
									pProgressWin->SetCurrentItemText( theTask->mProgressID, "\pWaiting for new files" );
									pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );			
									theTask->mReset = true;
									
								break;
								
								case kCVSEngineUpdatePreActionEvent:
																																
									if ( theTask->mReset == true )
										{
											pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarAbsolute );
											theTask->mReset = false;
										}
																
									pProgressWin->SetCurValue( theTask->mProgressID, 0 );

									::BlockMoveData( &pstrUpdating[1], &pstrItem[1], pstrUpdating[0] );
										pstrItem[0] = pstrUpdating[0];
										
									Assert_( (inDataPtr != NULL) && (inCookie > 0) );
									
									if ( (inDataPtr != NULL) && (inCookie > 0) )
										{																							
											intCopyLen = inCookie;
											
											if ( intCopyLen > (255 - pstrItem[0]) )
												{
													intCopyLen = (255 - pstrItem[0]);
												}
												
											::BlockMoveData( inDataPtr, &pstrItem[pstrItem[0] + 1], intCopyLen );
											pstrItem[0] += intCopyLen;
											
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );											
										}
									else 
										{
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );
										}
										
								break;

								case kCVSEngineUpdatePostActionEvent:

									if ( (inError != noErr) && (inError != kCVSEngineSkippedFile) )
										theTask->PostFileError( inTargetItem, (char *)inDataPtr, inCookie, inError );
											
									if ( inError != kCVSEngineOperationAbortedErr )
										{																					
											theTask->mIterations++;
											pProgressWin->SetProgressCountText( theTask->mProgressID, theTask->mIterations );								
										}
									
								break;

								case kCVSEngineCheckoutPreActionEvent:
								case kCVSEngineCheckoutPostActionEvent:

								//  These events are obsolete.
								
								Assert_( false );
								
								break;
															
								case kCVSEngineDecodingPreActionEvent:

									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );

									pProgressWin->SetCurValue( theTask->mProgressID, 0 );
																	
									if ( fsTarget.name[0] > 0 )
										{	
											PStrConcat( pstrItem, pstrDecoding, sizeof(pstrItem) );	
											PStrConcat( pstrItem, fsTarget.name, sizeof(pstrItem) );
							
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );										
										}
										
								break;
								
								case kCVSEngineProgressEvent:

									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );
									
									if ( inDataPtr != NULL )
										{								
											if ( theTask->mOldMaxValue != ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax )
												{										
													theTask->mOldMaxValue = ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax;
													pProgressWin->SetMaxValue( theTask->mProgressID, theTask->mOldMaxValue );								
												}
												
											pProgressWin->SetCurValue( theTask->mProgressID, ((CCVSEngineProgressStatusRec *)inDataPtr)->fCompleted);						
										}
													
								break;
								
								case kCVSEngineYieldActionEvent:
									
									CThreadedTask::Yield();
										
								break;
								
								default:
								
									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );
								
								break;					
							}				
					}				
				catch ( ... )
					{

//  If an exception was thrown during the yield,
//  we need to abort.
					
						if ( inEvent == kCVSEngineYieldActionEvent )
							bResult = 0;
							
					}					
			}
			
		return bResult;
				
	}

		
void		
CMacCVSCheckoutTask::ListenToMessage( MessageT inMessage, void *ioParam )
	{
		if ( ioParam != NULL )
			{
				UInt32 intID = *(UInt32 *)ioParam;
				
				if ( (intID == mProgressID) && (inMessage == msg_Cancel) )
					{
						ShutdownThread();
					}
			}
	}
	

void
CMacCVSCheckoutTask::GetTaskName (Str255 outNamePStr)
	{
		LStr255		theNameStr;

		theNameStr = "\pcheckout";
			
		if ( theNameStr[0] > 0 )
			::BlockMoveData( theNameStr, outNamePStr, theNameStr[0] + 1 );
	}
