//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSCommitTask.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains code to run a commit operation.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes.
							
-------------------------------------------------------------------------------- */

#include <LTextEditView.h>

#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "CMacCVSDoc.h"
#include "CMultiProgressWindow.h"
#include "CMacCVSCommitTask.h"



CMacCVSCommitTask::CMacCVSCommitTask( LArray *inItems, CMacCVSDoc	*inOwnerDoc ):
	
	CMacCVSOperationTask( inOwnerDoc )
	
	{	
		mEngineRef							= NULL;
		mProgressID							= 0;		
		mOldMaxValue 						= 0;
		mIterations							= 0;
		mReset									= false;
		
		ThrowIfNil_( mOwnerDoc );
		
		if ( inItems != NULL )
			{		
				mItemList = *inItems;		//  this makes a copy of the array
			}
		else
			{
				mItemList = NULL;
			}					
	}


CMacCVSCommitTask::~CMacCVSCommitTask()
	{		
		try
			{
				if ( mEngineRef != NULL )
					{			
						DisposeCVSEngine( mEngineRef );			
						mEngineRef = NULL;				
					}
							
				if ( mProgressID != 0 )
					{
						CMultiProgressWindow *pProgressWindow = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWindow != NULL );
						
						if ( pProgressWindow != NULL )
							{
								pProgressWindow->RemoveListener( this );
							}
							
						CMultiProgressWindow::DeleteGlobalProgressIndicator( mProgressID );
					}
			}
		catch ( ... )
			{
			}
			
		CMacCVSCommitTaskItem	 sCurItem;
		LArrayIterator				 itemIterator( mItemList );
		
		while ( itemIterator.Next(&sCurItem) )
			{
				if ( sCurItem.mOptions.mCommentText != NULL )
					{				
						::DisposeHandle( sCurItem.mOptions.mCommentText );
						sCurItem.mOptions.mCommentText = NULL;
					}
			}
	}
	

#pragma mark -


void
CMacCVSCommitTask::Run()
	{
	
		OSErr		result				= noErr;
		FSSpec	fsProjectRoot	= { 0, 0, "\p" };
		
		try
			{
				mProgressID = CMultiProgressWindow::NewGlobalProgressIndicator();
				Assert_( mProgressID != 0 );
				
				CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
				ThrowIfNil_( pProgressWin );
				
				pProgressWin->AddListener( this );
				
				LStr255	pstrItemTitle;
				pstrItemTitle = "\Committing: ";
							
				LStr255	pstrDocName;
				mOwnerDoc->GetDescriptor( pstrDocName );
				pstrItemTitle += pstrDocName;
		
				pProgressWin->SetProgressMessageText( mProgressID, pstrItemTitle );
	
				CCVSEngineLoginOptions sLoginOptions;
				sLoginOptions.Init();
				
				mOwnerDoc->GetHostNamePStr( sLoginOptions.mServerName );
				sLoginOptions.mPort = mOwnerDoc->GetHostPort();
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mRemoteUserName );
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mLocalUserName );
				mOwnerDoc->GetPasswordPStr( sLoginOptions.mPassword );
				mOwnerDoc->GetCVSCommandPStr( sLoginOptions.mCVSCommandText );
				mOwnerDoc->GetCVSRootPStr( sLoginOptions.mCVSRoot );
				sLoginOptions.mAuthenticationMethod = mOwnerDoc->GetAuthenticationMethod();
				
				mOwnerDoc->GetProjectRootFileSpec( fsProjectRoot );
				
				LArrayIterator itemIterator( mItemList );
				CMacCVSCommitTaskItem	sCurItem;

				//  Because the CVS server is lame, we drop the connection after
				//  each operation to insure that everything works ok.

				mIterations = 0;
								
				while ( itemIterator.Next(&sCurItem) )
					{		
						pProgressWin->SetCurrentItemText( mProgressID, "\pConnecting to server");
						pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );

						result = CreateCVSEngine( &fsProjectRoot, mMappings, mMappingsCount, Notifier, MessageOutNotifier, ErrorOutNotifier, this, mOwnerDoc->GetSocketTimeOut(), &mEngineRef );
						ThrowOSErr( result );
					
						result = CVSEngineLogin( mEngineRef, &sLoginOptions );
						ThrowOSErr( result );

						pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarAbsolute );

						pProgressWin->SetCurrentItemText( mProgressID, "\pSending Entry" );					
						pProgressWin->SetProgressCountText( mProgressID, mIterations );					
						pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarAbsolute );	
						
						result = CVSEngineCommit( mEngineRef, &sCurItem.mFileSpec, &sCurItem.mOptions );
						
						if ( result != noErr )
							{
								PostFileError( &sCurItem.mFileSpec, NULL, 0, result );
								PostOverallError( result );
							}

						mIterations++;
						pProgressWin->SetProgressCountText( mProgressID, mIterations );
						
						DisposeCVSEngine( mEngineRef );
						mEngineRef = NULL;
					}						
			}			
		catch ( ... )
			{
				if ( result == noErr )
					{
						result = kCVSEngineUnknownErr;
					}
					
				PostOverallError( result );
			}
	
		if ( result != noErr )
			{
				AddToCompletionEvent( result );
			}
			
		SendCompletionEvent();
							
	}
	
	
pascal SInt32		
CMacCVSCommitTask::Notifier(	const CCVSEngineRef					inEngineRef,
															const FSSpec								*inTargetItem,
															const SInt32 								inEvent, 
															const SInt32 								inError, 
															const UInt32 								inCookie, 
															const void 									*inDataPtr, 
															void 												*inContextPtr)
	{
	
		const	unsigned char	pstrUpdating[] 			= "\pCommitting: ";
		const	unsigned char	pstrSendingEntry[] 	= "\pSending: ";
		const	unsigned char	pstrDecoding[] 			= "\pDecoding: ";
	
		OSErr								error								= noErr;
		Boolean							bResult							= true;
		UInt32							intCopyLen 					= 0;
		Str255							pstrItem						= "\p";
		FSSpec							fsTarget						= { 0, 0, "\p" };
		
		CMacCVSCommitTask		*theTask 						= (CMacCVSCommitTask *)inContextPtr;
				
		if ( theTask != NULL )
			{
				try
					{
						CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWin != NULL );

						if ( inTargetItem != NULL )
							{
								fsTarget = *inTargetItem;
							}
								
						switch ( inEvent )
							{				
								case kCVSEngineEntryPreActionEvent:
									{
										if ( inError != noErr )
											theTask->PostFileError( inTargetItem, (char *)inDataPtr, inCookie, inError );

										pProgressWin->SetCurValue( theTask->mProgressID, 0 );

										::BlockMoveData( &pstrSendingEntry[1], &pstrItem[1], pstrSendingEntry[0] );
										pstrItem[0] = pstrSendingEntry[0];
										
										Assert_( (inDataPtr != NULL) && (inCookie > 0) );
										
										if ( (inDataPtr != NULL) && (inCookie > 0) )
											{																							
												intCopyLen = inCookie;
												
												if ( intCopyLen > (255 - pstrItem[0]) )
													{
														intCopyLen = (255 - pstrItem[0]);
													}
													
												::BlockMoveData( inDataPtr, &pstrItem[pstrItem[0] + 1], intCopyLen );
												pstrItem[0] += intCopyLen;
												
												pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );											
											}
										else 
											{
												pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );
											}
									}															
								break;
								
								case kCVSEngineCommitPreActionEvent:

									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );

									if ( theTask->mReset )
										{
											pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarAbsolute );
											theTask->mReset = false;
										}
											
								break;

								case kCVSEngineCommandActionEvent:
								
									pProgressWin->SetCurValue( theTask->mProgressID, 0 );
									pProgressWin->SetCurrentItemText( theTask->mProgressID, "\pWaiting for new files" );
									pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );			
									theTask->mReset = true;
									
								break;
	
								case kCVSEngineCommitPostActionEvent:

//  Don't report kCVSEngineServerReturnedErrs because the actual CVS Engine call
//  will return it and we will report it then.
								
									if ( (inError != noErr) && (inError != kCVSEngineServerReturnedErr) && (inError != kCVSEngineFileNotChangedErr) )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );

//  Ok, since the CVS Engine does *not* change the mod. date on a committed
//  file (because it will inadventently "touch" it in CodeWarrior, we need
//  to tell the CMacCVSDoc to update it's views.
	
									if ( inTargetItem != NULL )
										{
											theTask->mOwnerDoc->UpdateItemInProjectViews( inTargetItem );										
										}
																		
								break;

								case kCVSEngineUpdatePreActionEvent:
										
									if ( theTask->mReset )
										{
											pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarAbsolute );
											theTask->mReset = false;
										}
																
									pProgressWin->SetCurValue( theTask->mProgressID, 0 );

									::BlockMoveData( &pstrUpdating[1], &pstrItem[1], pstrUpdating[0] );
										pstrItem[0] = pstrUpdating[0];
										
									Assert_( (inDataPtr != NULL) && (inCookie > 0) );
									
									if ( (inDataPtr != NULL) && (inCookie > 0) )
										{																							
											intCopyLen = inCookie;
											
											if ( intCopyLen > (255 - pstrItem[0]) )
												{
													intCopyLen = (255 - pstrItem[0]);
												}
												
											::BlockMoveData( inDataPtr, &pstrItem[pstrItem[0] + 1], intCopyLen );
											pstrItem[0] += intCopyLen;
											
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );											
										}
									else 	
										{
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );
										}
										
								break;
								
								case kCVSEngineUpdatePostActionEvent:

									if ( (inError != noErr) && (inError != kCVSEngineSkippedFile) )
										theTask->PostFileError( inTargetItem, (char *)inDataPtr, inCookie, inError );
																					
								break;

								case kCVSEngineDecodingPreActionEvent:

									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );

									pProgressWin->SetCurValue( theTask->mProgressID, 0 );
																	
									if ( fsTarget.name[0] > 0 )
										{	
											PStrConcat( pstrItem, pstrDecoding, sizeof(pstrItem) );	
											PStrConcat( pstrItem, fsTarget.name, sizeof(pstrItem) );
							
											pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );										
										}
																		
								break;
								
								case kCVSEngineProgressEvent:

									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );
									
									if ( inDataPtr != NULL )
										{								
											if ( theTask->mOldMaxValue != ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax )
												{										
													theTask->mOldMaxValue = ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax;
													pProgressWin->SetMaxValue( theTask->mProgressID, theTask->mOldMaxValue );								
												}
												
											pProgressWin->SetCurValue( theTask->mProgressID, ((CCVSEngineProgressStatusRec *)inDataPtr)->fCompleted);						
										}
													
								break;

								case kCVSEngineYieldActionEvent:
									
									CThreadedTask::Yield();
								
								break;
								
								default:
								
									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );
								
								break;
							
							}
							
					}
					
				catch ( ... )
					{
					
						if ( inEvent == kCVSEngineYieldActionEvent )
							bResult = false;
					
					}
					
			}
			
		return bResult;
		
	}
	
	
void		
CMacCVSCommitTask::ListenToMessage( MessageT inMessage, void *ioParam )
	{
		if ( ioParam != NULL )
			{
				UInt32 intID = *(UInt32 *)ioParam;
				
				if ( (intID == mProgressID) && (inMessage == msg_Cancel) )
					{
						ShutdownThread();
					}
			}
	}


void
CMacCVSCommitTask::GetTaskName (Str255 outNamePStr)
	{
		LStr255		theNameStr;

		theNameStr = "\pcommit";
			
		if ( theNameStr[0] > 0 )
			::BlockMoveData( theNameStr, outNamePStr, theNameStr[0] + 1 );
	}


