//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CMacCVSCompareTask.cp
//	
//	  	Authors: 	Ram Rajadhyaksha
//
//################################################################################


#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"

#include "CVSEngine.h"
#include "CMacCVSCompareWindow.h"
#include "CMacCVSCompareTask.h"


CMacCVSCompareTask::CMacCVSCompareTask( CMacCVSCompareWindow *inOwnerWindow, CMacCVSDoc *inOwnerDoc, const FSSpec *inTargetItem, const CCVSEngineDiffOptions *inCompareOptions ):

	CMacCVSWindowTask( inOwnerWindow, inOwnerDoc ), mCompareOptions( *inCompareOptions )
	{
		mTargetItem.name[0] = 0;
		mTargetItem.parID 	= 0;
		mTargetItem.vRefNum = 0;
		
		if ( inTargetItem != NULL )
			{
				mTargetItem = *inTargetItem;
			}
			
		SetEventNotifier( Notifier );
	}

	
CMacCVSCompareTask::~CMacCVSCompareTask()
	{	
		// the superclass kills the engineRef
	}


#pragma mark -


OSErr
CMacCVSCompareTask::DoEngineCommand()
	{
		OSErr	result;
		
		try
			{
				result = CVSEngineDiff( mEngineRef, &mTargetItem, &mCompareOptions, CMacCVSCompareWindow::DiffNotifier, mOwnerWindow );		
				ThrowIfError_( result );
				
				ThrowIfNil_( mOwnerWindow );
				CMacCVSCompareWindow *pCompareWindow = dynamic_cast<CMacCVSCompareWindow *>( mOwnerWindow );
				ThrowIfNil_( pCompareWindow );
				
				pCompareWindow->ParseDiffOutput();
			}
		catch( OSErr intException )
			{
				result = intException;
			}
		catch( ExceptionCode intException )
			{
				result = intException;
			}
		catch ( ... )
			{
				Assert_( false );
				result = kCVSEngineUnknownExceptionErr;
			}
			
		return result;
	}	


pascal SInt32		
CMacCVSCompareTask::Notifier(	const CCVSEngineRef					inEngineRef,
															const FSSpec								*inTargetItem,
															const SInt32 								inEvent, 
															const SInt32 								inError, 
															const UInt32 								inCookie, 
															const void 									*inDataPtr, 
															void 												*inContextPtr)
	{
		OSErr									error				= noErr;
		Boolean								bResult			= true;
		CMacCVSCompareTask		*theTask		= (CMacCVSCompareTask *)inContextPtr;
		
		if ( theTask != NULL )
			{			
				try
					{
						if ( inError != noErr )
							{
								theTask->PostFileError( inTargetItem, NULL, 0, inError );
							}
									
						if ( inEvent == kCVSEngineYieldActionEvent )
							{
								CThreadedTask::Yield();
							}
					}					
				catch ( ... )
					{					
						bResult = false;						
					}
			}
			
		return bResult;		
	}
	

