//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSCompareWindow.cp
//
//	  Authors:	Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#include <UMemoryMgr.h>
#include <LWindow.h>
#include <LBroadcaster.h>
#include <LTextEditView.h>

#include "UMacCVSUtils.h"
#include "CThreadedTask.h"
#include "CMacCVSCompareTask.h"

#include "CCompareWithOriginalView.h"
#include "CMacCVSCompareWindow.h"


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS/DESTRUCTORS


CMacCVSCompareWindow::CMacCVSCompareWindow( LStream *inStream ): 

	CMacCVSThreadedWindow( inStream )
	
	{
		
		mCompareView		= NULL;
		mCompareTable		= NULL;
		mDataHndl				= NULL;
		
		mTargetItem.name[0] = 0;
		mTargetItem.vRefNum = 0;
		mTargetItem.parID = 0;
		
		mWindowPosResID		= kCompareWindowPositionResID;
		
	}


CMacCVSCompareWindow::~CMacCVSCompareWindow()
	{
			
		if ( mDataHndl != NULL )
			::DisposeHandle( mDataHndl );
			
	}


CMacCVSCompareWindow *
CMacCVSCompareWindow::CreateCMacCVSCompareWindowFromStream( LStream *inStream )
	{

		return new CMacCVSCompareWindow( inStream );

	}


void 
CMacCVSCompareWindow::FinishCreateSelf()
	{

		Inherited::FinishCreateSelf();
		
		mCompareView = dynamic_cast<CCompareWithOriginalView *>( FindPaneByID(CCompareWithOriginalView::class_ID) );
		ThrowIfNil_( mCompareView );

		//AddListener( mSpinnyThingy );
		
		mCompareTable = dynamic_cast<CBroadcastTableTextView *>( FindPaneByID(CBroadcastTableTextView::class_ID) );
		ThrowIfNil_( mCompareTable );
		
		SetLatentSub( mCompareTable );
	}


void
CMacCVSCompareWindow::Compare( const FSSpec *inTargetItem, CMacCVSDoc *inOwnerDoc, const void *inCommandData, UInt32 inDelayTicks )
	{
	
		Assert_( inTargetItem != NULL );
		Assert_( inOwnerDoc != NULL );
		Assert_( mWindowTask == NULL );
		
		LStr255	titleString;
		
		if ( (mWindowTask == NULL) && (inTargetItem != NULL) && (inOwnerDoc != NULL) )
			{
			
				GetDescriptor( titleString );
				titleString = titleString + inTargetItem->name;
				SetDescriptor( titleString );

				mTargetItem = *inTargetItem;

				Show();
	
				mWindowTask = new CMacCVSCompareTask( this, inOwnerDoc, inTargetItem, (const CCVSEngineDiffOptions *)inCommandData );
				ThrowIfNil_( mWindowTask );
			
				PrimeWindowTask(inDelayTicks);
								
			}
	
	}
	

pascal SInt32
CMacCVSCompareWindow::DiffNotifier( const CCVSEngineRef inEngineRef, const UInt32 inDataType, const UInt32 inDataLen, const void *inData, void *inContextPtr )
	{
	
		OSErr								result			= noErr;
		SInt32							offset 			= 0x00000000;
		CMacCVSCompareWindow		*theWindow 	= (CMacCVSCompareWindow *)inContextPtr;
		
		if ( theWindow != NULL )
			{
			
				try
					{

//  Just stuff all the diff stream output into a handle.
						
						if ( inDataLen > 0 )
							{
								
								if ( theWindow->mDataHndl != NULL )
									{
									
										offset = ::GetHandleSize( theWindow->mDataHndl );
										
										::SetHandleSize( theWindow->mDataHndl, offset + inDataLen );
										
										result = ::MemError();
										
										if ( result != noErr )
											throw result;
											
									}
								
								else
									{
									
										theWindow->mDataHndl = ::NewHandle( inDataLen );
										
										if ( theWindow->mDataHndl == NULL )
											{
											
												result = ::MemError();
												throw result;
												
											}
									
									}
									
								::BlockMoveData( inData, ((UInt8 *)(*(theWindow->mDataHndl))) + offset, inDataLen );
								
							}
							
					}
					
				catch ( ... )
					{
					
					
					}
					
			}
							
		return inDataLen;
		
	}
	

void
CMacCVSCompareWindow::ParseDiffOutput()
	{
	
		const						kLineBufLen											= 256;
		
		Boolean					bDone														= false;
		UInt32					dataSize 												= 0,
										curOffset												= 0,
										lineStrLen											= 0,
										firstLine												= 0,
										lastLine												= 0;
		
		char						lineStr[kLineBufLen]						= "",
										separator												= 0;
	
		SInt32					linesPreviouslyAddedAndRemoved 	= 0;
		
		CDiffClauseRec	theDiffClause;

		if ( mDataHndl != NULL )
			{
			
				LArray differenceClauseData( sizeof(CDiffClauseRec) );
				
				dataSize = ::GetHandleSize( mDataHndl );
				Assert_( dataSize > 0 );
				
				if ( dataSize > 0 )
					{

//  I just copied this code from version 1.0. 
//  Don't blame me for writing it! - rvr
					
						try
							{
							
								StHandleLocker	lock( mDataHndl );
							
								while ( !bDone )
									{
							
										lineStrLen = kLineBufLen - 1;
										UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, " \r" );
										lineStr[lineStrLen] = 0x00;
										
										if ( curOffset >= dataSize )
										
											bDone = true;
												
										else if ( !strcmp(lineStr, "diff") )
											{
												
												separator = 0x00;
												
												while ( (separator != '\r') && (curOffset < dataSize) )
													{
													
														lineStrLen = kLineBufLen - 1;
														separator = UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
														lineStr[lineStrLen] = 0x00;
												
													}
													
												if ( separator != '\r' )
													bDone = true;
												
												while ( !bDone )
													{

														theDiffClause.fType 			= 0;
														theDiffClause.fOffset 		= 0;
														theDiffClause.fLineNumber = 0;
														theDiffClause.fLineCount 	= 0;
														theDiffClause.fDataHandle	= NULL;
					
														lineStrLen = kLineBufLen - 1;
														separator = UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "acd,\r" );
														lineStr[lineStrLen] = 0x00;

														if ( curOffset >= dataSize )
														
															bDone = true;
																					
														else 
															{
					
																parserReset:
														
																if ( (separator == 'a') || (separator == 'c') )
																	{
														
																		UInt32	firstLine = 0;
																		UInt32 	lastLine	= 0;

//  We just got an add clause.  Append it to our list.

																		if (separator == 'c')
																			{
																			
																				theDiffClause.fType = kDifferenceLineChanged;
																				theDiffClause.fLineNumber = atoi(lineStr) + linesPreviouslyAddedAndRemoved;
																			
																			}
																		else
																			{
																			
																				theDiffClause.fType = kDifferenceLineAdded;
																				theDiffClause.fLineNumber = atoi(lineStr) + 1 + linesPreviouslyAddedAndRemoved;
																			
																			}
					
																		lineStrLen = kLineBufLen - 1;
																		separator = UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, ",\r" );
																		lineStr[lineStrLen] = 0;
										
																		firstLine = atoi(lineStr);
											
																		if (separator == ',')
																			{
																			
																				lineStrLen = kLineBufLen - 1;
																				separator = UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																				lineStr[lineStrLen] = 0;
																				
																				lastLine = atoi(lineStr);
																			
																			}
											
																		else lastLine = firstLine;
											
																		theDiffClause.fLineCount = lastLine - firstLine + 1;
																		linesPreviouslyAddedAndRemoved += theDiffClause.fLineCount;
																								
																		theDiffClause.fDataHandle = NULL;
											
																		if ( theDiffClause.fType == kDifferenceLineAdded )
																			{
																			
																				for ( UInt32 currentLine = 0; currentLine < theDiffClause.fLineCount; currentLine++ )
																					{			
																				
																						lineStrLen = kLineBufLen - 1;
																						UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																						lineStr[lineStrLen] = 0;
																						
																					}
																			
																			}

																		else
																			{

//  For changes, we need to swallow an extra set of lines.

																				while ( true )
																					{
																					
																						lineStrLen = kLineBufLen - 1;
																						UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																						lineStr[lineStrLen] = 0;
																						
																						if ( !strncmp(lineStr, "---", 3) )
																							break;
																							
																						else if ( curOffset >= dataSize )
																							{
																							
																								bDone = true;
																								break;
																								
																							}
																							
																						linesPreviouslyAddedAndRemoved--;
																					
																					}
																					
																				if ( !bDone )
																					{
																					
																						for ( UInt32 currentLine = 0; currentLine < theDiffClause.fLineCount; currentLine++ )
																							{

																								lineStrLen = kLineBufLen - 1;
																								UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																								lineStr[lineStrLen] = 0;

																							}
																					
																					}	
																											
																			}
											
																		differenceClauseData.InsertItemsAt(1, LArray::index_Last, &theDiffClause, sizeof(CDiffClauseRec));

																	}
					
																else if ( (separator == ',') || (separator == 'd') )
																	{
					
																		Boolean	resetParser = false;
																	
																		UInt32	firstLine		= 0;
																		UInt32	lastLine		= 0;
																		UInt32 	linesToEat 	= 0;
																					
																		firstLine = atoi(lineStr);
																		
																		if ( separator == ',' )
																			{
							
																				lineStrLen = kLineBufLen - 1;
																				separator = UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "dc" );
																				lineStr[lineStrLen] = 0;
												
																				lastLine = atoi(lineStr);
																			
																			}
													
																		else lastLine = firstLine;
							
																		lineStrLen = kLineBufLen - 1;
																		UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																		lineStr[lineStrLen] = 0;
							
																		if (separator == 'c')
																			{
																			
																				theDiffClause.fType 			= kDifferenceLineChanged;
																				theDiffClause.fLineNumber = atoi(lineStr);
																				
																				linesToEat = lastLine - firstLine + 1;
																				theDiffClause.fLineCount = 0;
																			
																			}
																			
																		else
																			{
																				
																				theDiffClause.fType 			= kDifferenceLineRemoved;
																				theDiffClause.fLineNumber = atoi(lineStr) + 1;
																				
																				linesToEat = lastLine - firstLine + 1;
																				theDiffClause.fLineCount = linesToEat;
																			
																			}
													
																		linesPreviouslyAddedAndRemoved -= linesToEat;
																		LHandleStream	theLineDataHndl;

																		if ( theDiffClause.fType == kDifferenceLineChanged )								
																			{
																			
																				for ( UInt32 currentLine = 0; currentLine < linesToEat; currentLine++ )
																					{
																					
																						lineStrLen = kLineBufLen - 1;
																						UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																						lineStr[lineStrLen] = 0;
							
																					}

//  Read the ---- line	
							
																				lineStrLen = kLineBufLen - 1;
																				UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																				lineStr[lineStrLen] = 0;

//	This is SOOOO disgusting.  The problem is, there is no terminator
//	for the second half of the change clause in this case.  This means
//	that we have to "look ahead" for the appropriate characters that
//	might begin the new clause, and when we see them, "goto" to the
//	right place in the parser.
														
																				while ( true )
																					{

																						lineStrLen = kLineBufLen - 1;
																						separator = UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "acd>,\r" );
																						lineStr[lineStrLen] = 0;
																						
																						if ( curOffset >= dataSize )
																							{
																							
																								bDone = true;
																								break;
																								
																							}
																																							
																						else
																							{
																																										
//	If the line begins with a ">", it is still part of the 
//	change clause.  Otherwise, it is a new clause, and
//	we have to reset the parser.

																								if ( separator == '>' )
																									{
																									
																										linesPreviouslyAddedAndRemoved++;
																										theDiffClause.fLineCount++;
							
																										lineStrLen = kLineBufLen - 1;
																										UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																										lineStr[lineStrLen] = 0;
																										
																									}
																							
																								else
																									{
																									
																										resetParser = true;
																										break;
																								
																									}
																									
																							}
															
																					}
														
																			}
													
																		else
																			{
																			
																				for ( UInt32 currentLine = 0; currentLine < linesToEat; currentLine++ )
																					{
																				
																						lineStrLen = kLineBufLen - 1;
																						UMacCVSUtils::ReadWord( *mDataHndl, dataSize, &curOffset, lineStr, &lineStrLen, "\r" );
																						lineStr[lineStrLen] = 0;

//  Don't include the '< ' characters.
																			
																						if ( lineStrLen > 2 )
																							{
																							
																								theLineDataHndl.WriteBlock( &lineStr[2], lineStrLen - 2 );
																						
																							}
																							
																						theLineDataHndl.WriteBlock( "\r", 1 );
																																							
																					}
																			
																			}
																		
																		if ( theLineDataHndl.GetLength() > 0 )
																			{
																			
																				theDiffClause.fDataHandle = theLineDataHndl.DetachDataHandle();
																				
																			}
																			
																		differenceClauseData.InsertItemsAt( 1, LArray::index_Last, &theDiffClause, sizeof(CDiffClauseRec) );
																	
																		if ( resetParser )
																			goto parserReset;
												
																	} //  if sep = ',', etc.
																	
															}
															
													}
													
											}
										
									}
									
								mCompareView->SetCompareInformation( mTargetItem, differenceClauseData );
			
								LArrayIterator	itemIterator( differenceClauseData );
									
								while ( itemIterator.Next(&theDiffClause) )
									{
									
										if ( theDiffClause.fDataHandle != NULL )
											::DisposeHandle( theDiffClause.fDataHandle );
											
									}
								
							}
							
						catch ( ... )
							{
							
								LArrayIterator	itemIterator( differenceClauseData );
									
								while ( itemIterator.Next(&theDiffClause) )
									{
									
										if ( theDiffClause.fDataHandle != NULL )
											::DisposeHandle( theDiffClause.fDataHandle );
											
									}
									
								throw;
							
							}
							
					}
					
			}
	
	}
	
