//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CMacCVSDiffTask.cp
//	
//	  	Authors: 	Ram Rajadhyaksha
//
//################################################################################


#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"

#include "CVSEngine.h"
#include "CMacCVSDiffWindow.h"
#include "CMacCVSDiffTask.h"


CMacCVSDiffTask::CMacCVSDiffTask( const CMacCVSDiffWindow *inOwnerWindow, CMacCVSDoc *inOwnerDoc, const FSSpec *inTargetItem ):

	CMacCVSOperationTask( inOwnerDoc )
		
	{

		mOwnerWindow 				= NULL;
		mEngineRef 					= NULL;
		mTargetItem.name[0] = 0x00;
		mTargetItem.vRefNum = 0;
		mTargetItem.parID 	= 0;
		
		ThrowIfNil_( inOwnerWindow );
		ThrowIfNil_( inTargetItem );

		mOwnerWindow = const_cast<CMacCVSDiffWindow *>( inOwnerWindow );
		mTargetItem = *inTargetItem;
			
	}
	
	
CMacCVSDiffTask::~CMacCVSDiffTask()
	{
	
		if ( mEngineRef != NULL )
			{
			
				DisposeCVSEngine( mEngineRef );
				
				mEngineRef = NULL;
				
			}
		
		mOwnerWindow->ThreadedTaskDied( this );
		
	}
	
	
void
CMacCVSDiffTask::Run()
	{
	
		OSErr		result 				= noErr;
		FSSpec	projectRoot		= { 0, 0, "\p" };
		
		try
			{

				mOwnerWindow->TurnOnSpinnyThingy();
				
				CCVSEngineLoginOptions loginOptions;
				loginOptions.Init();
				
				mOwnerDoc->GetHostNamePStr( loginOptions.mServerName );
				mOwnerDoc->GetRemoteUserPStr( loginOptions.mRemoteUserName );
				mOwnerDoc->GetRemoteUserPStr( loginOptions.mLocalUserName );
				mOwnerDoc->GetPasswordPStr( loginOptions.mPassword );
				mOwnerDoc->GetCVSCommandPStr( loginOptions.mCVSCommandText );
				mOwnerDoc->GetCVSRootPStr( loginOptions.mCVSRoot );
				loginOptions.mUseRSH = mOwnerDoc->GetUseRSH();
				
				mOwnerDoc->GetProjectRootFileSpec( projectRoot );
				
				result = CreateCVSEngine( &projectRoot, NULL, 0, Notifier, NULL, ErrorOutNotifier, this, mOwnerDoc->GetSocketTimeOut(), &mEngineRef );
				
				if ( result != noErr )
					throw result;
									
				result = CVSEngineLogin( mEngineRef, &loginOptions );
				
				if ( result != noErr )
					throw result;
					
				result = CVSEngineDiff( mEngineRef, &mTargetItem, CMacCVSDiffWindow::DiffNotifier, mOwnerWindow );
				
			}
			
		catch ( ... )
			{

				if ( result == noErr )
					result = kCVSEngineUnknownErr;
					
				PostOverallError( result );

			}
	
		mOwnerWindow->TurnOffSpinnyThingy();
							
	}
	

pascal SInt32		
CMacCVSDiffTask::Notifier(	const CCVSEngineRef					inEngineRef,
														const FSSpec								*inTargetItem,
														const SInt32 								inEvent, 
														const SInt32 								inError, 
														const UInt32 								inCookie, 
														const void 									*inDataPtr, 
														void 												*inContextPtr)
	{
	
		OSErr								error				= noErr;
		Boolean							bResult			= true;
		CMacCVSStatusTask		*theTask		= (CMacCVSStatusTask *)inContextPtr;
		
		if ( theTask != NULL )
			{
			
				try
					{

						if ( inError != noErr )
							theTask->PostFileError( inTargetItem, NULL, 0, inError );
										
						if ( inEvent == kCVSEngineYieldActionEvent )
							CThreadedTask::Yield();
			
					}
					
				catch ( ... )
					{
					
						bResult = false;
						
					}
					
			}
			
		return bResult;
		
	}
	

