//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSDoc.cp
	
	Authors:		Dan Clifford, Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code for the CMacCVSDoc object.
		 
	Modification History:

		21 Apr 1999		Josef W. Wankerl
		
					Added Reveal In Finder support.
		
		13 Apr	1999		Josef W. Wankerl
		
					Now uses Navigation Services during "Save As..." with the PowerPlant
					officially sanctioned method.
		
		11 Apr	1999		Josef W. Wankerl
		
					Added drag and drop support for the local tree view area.  Any
					folder dropped onto the local tree view will be recognized and
					set in the prefernces dialog.
		
		10 Apr	1999		Josef W. Wankerl
		
					Made Navigation Services aware.
		
		7 July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes. Added
				  #include <LPopupButton.h> for new grayscale pop-up class.
							
-------------------------------------------------------------------------------- */


#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <LFileStream.h>
#include <LTextButton.h>
#include <UModalDialogs.h>
#include <UStandardFile.h>
#include <LEditText.h>
#include <LRadioButton.h>
#include <LRadioGroupView.h>
#include <LCheckbox.h>
#include <LCheckBoxGroupBox.h>
#include <LPopupButton.h>
#include <LPopupGroupBox.h>
#include <UConditionalDialogs.h>		// JWW - For Navigation Services

#include "CLocalTreeView.h"				// JWW - For folder droping on local tree view

#include "MacCVSPPConstants.h"

#include "StringUtils.h"
#include "UErrorDialog.h"

#include "MacCVSRandomFrontEndCrap.h"

#include "CTagPopup.h"
#include "CSpinIndicator.h"

#include "CVSEngine.h"
#include "CVSEngineUtils.h"

#include "CMacCVSApp.h"
#include "UMacCVSUtils.h"

#include "CMacCVSAppleEventItem.h"

#include "CMacCVSProjectViewWindow.h"
#include "CMacCVSProjectView.h"

#include "CMacCVSLurkersWindow.h"
#include "CMacCVSLurkersView.h"

#include "CMacCVSOrphanFilesTask.h"

#include "CMacCVSMappingsView.h"

#include "CMacCVSMessageView.h"
#include "CMacCVSMessageWindow.h"

#include "CThreadedTask.h"

#include "UProjectActions.h"
#include "CMacCVSDoc.h"


CMacCVSDoc::CMacCVSDoc( const FSSpec &fsInItem, LCommander *inSuperCommander ): 

	LSingleDoc( inSuperCommander )
	
	{
		mDocumentFileSpec 					= fsInItem;
		
		if ( !FSpIsValid(mDocumentFileSpec) )
			{
				ThrowOSErr_( fnfErr );
			}
			
		mRootFileSpec.name[0]				= 0;
		mRootFileSpec.vRefNum				= 0;
		mRootFileSpec.parID					= 0;
			
		mModuleName[0]							= 0;
		mDefaultTag[0]							= 0;
		mHostName[0]								= 0;
		mLocalUser[0]								= 0;
		mRemoteUser[0]							= 0;
		mPassword[0]								= 0;
		
		mCVSCommand[0]							= 3;
		mCVSCommand[1]							= 'c';
		mCVSCommand[2]							= 'v';
		mCVSCommand[3]							= 's';
		
		mCVSRoot[0]									= 0;
		
		mCVSMergePolicy 						= kCVSEngineNoMerging;
		mSocketTimeOut							= kDefaultSocketTimeOut;
		mCompressionLevel						= 0;
		mPort												= 0;
		
		mOutputFileUpdates					= false;
		mOutputDirectoryUpdates			= true;
		mAuthenticationMethod				= kAuthenticationPassword;
		mAuthenticationMethodItem 	= 0;
		
		mClearMessageWindow					= false;
		mScrollMessages							= false;
		mNewFoldersUseListView			= true;
		mNewFoldersInheritLabels 		= false;
		mSavePassword								= false;
		mPasswordValid							= false;
				
		mCheckingOutDefaultModule 	= false;
		
		mPrefPanel									= kCheckoutUpdatePanelMenuItemID;
		mProjectRootSpecified				= false;
		
		mTagList 										= NULL;
		mMappingList								= NULL;
		mWindowList									= NULL;
		mConnectionList							= NULL;

		mMessageWindow							= NULL;
		
		mFile 											= NULL;
				
		mCompareOptions.mIgnoreCase					= false;
		mCompareOptions.mIgnoreWhiteSpace		= false;
		mCompareOptions.mIgnoreSpaceChange	= false;
		mCompareOptions.mIgnoreBlankLines		= false;
		mCompareOptions.mShowCFunction			= false;
		mCompareOptions.mRCSDiff						= false;
		mCompareOptions.mContextDiff				= false;
		mCompareOptions.mUnifiedDiff				= false;
		mCompareOptions.mContextLines			 	= 0;

		mDiffOptions.mIgnoreCase						= false;
		mDiffOptions.mIgnoreWhiteSpace			= false;
		mDiffOptions.mIgnoreSpaceChange			= false;
		mDiffOptions.mIgnoreBlankLines			= false;
		mDiffOptions.mShowCFunction					= false;
		mDiffOptions.mRCSDiff								= false;
		mDiffOptions.mContextDiff						= false;
		mDiffOptions.mUnifiedDiff						= false;
		mDiffOptions.mContextLines			 		= 0;

		SetModelKind( cMacCVSProSession );
	}


CMacCVSDoc::
~CMacCVSDoc()
	{
	
		if ( mTagList != NULL ) 
			{			
				delete mTagList;			
				mTagList = NULL;				
			}
		
		if ( mMappingList != NULL )
			{			
				delete mMappingList;
				mMappingList = NULL;				
			}
			
		if ( mWindowList != NULL )
			{			
				delete mWindowList;				
				mWindowList = NULL;				
			}
		
		if ( mConnectionList != NULL )
			{
				delete mConnectionList;				
				mConnectionList = NULL;				
			}
			
		if ( mMessageWindow != NULL )
			{			
				delete mMessageWindow;				
				mMessageWindow = NULL;			
			}
		
		if ( mFile != NULL ) 
			{		
				mFile->CloseDataFork();			
			}
		
	}


void
CMacCVSDoc::Init()
	{

		SetUseSubModelList( true );

		mTagList = new LArray( sizeof(Str255) );
		ThrowIfNil_( mTagList );
		
		mMappingList = new LArray( sizeof(CCVSEngineFileMappingRec) );
		ThrowIfNil_( mMappingList );
				
		mWindowList = new LArray( sizeof(LWindow *) );
		ThrowIfNil_( mWindowList );
		
		mConnectionList = new LArray( sizeof(CThreadedTask *) );
		ThrowIfNil_( mConnectionList );

		OpenFile( mDocumentFileSpec );
		
		mWindow = NewProjectWindow( &mRootFileSpec, NULL, mDocumentFileSpec.name );
		ThrowIfNil_( mWindow );
				
		mWindow->Show();
				
	}

	
LDocument*
CMacCVSDoc::FindFSSpecDocument( FSSpec &fileSpec )
	{

		LArrayIterator	stIterator(sDocumentList, LArrayIterator::from_Start);
		CMacCVSDoc	 		*pDoc = NULL;
					
		while ( stIterator.Next(&pDoc) ) 
			{
				FSSpec	fsCurSpec;
			
				pDoc->mFile->GetSpecifier(fsCurSpec);
			
				if ( LFile::EqualFileSpec(fsCurSpec, fileSpec ) )
					{
						return pDoc;
					}
			}
		
		return NULL;
		
	}


Boolean
CMacCVSDoc::CheckCVSServerSettings()
	{

		//  Returns true if the server settings are all filled in, 
		//  otherwise show an alert and return false.

		Boolean	bServerSettingsOK = false;
		
		switch ( mAuthenticationMethod ) 
			{
				case kAuthenticationPassword:
				case kAuthenticationRSH:
				
					bServerSettingsOK = (mHostName[0] != 0) && (mRemoteUser[0] != 0) && (mCVSCommand[0] != 0);
				
				break;
				
				case kAuthenticationKClient:
				
					bServerSettingsOK = mHostName != 0;
					
				break;

				default:
				
					//  If we get there, then prefs contain an authentication method 
					//  we don't know about, likely because it's unsupported (export restrictions)
				
					UErrorDialog theErrorDialog( kAuthenticationMethodUnknownErr, this );

					return false;
				break;
			}
		
		if ( !bServerSettingsOK )
			{				
				UErrorDialog theErrorDialog( kNeedServerSettingsErr, this );
			}
		
		return bServerSettingsOK;
		
	}
	
	
LDocument*
CMacCVSDoc::GetNthDocument(SInt32 inPosition)
	{
	
		CMacCVSDoc *pDoc = NULL;
		
		if ( LDocument::GetDocumentList().FetchItemAt(inPosition, pDoc) )
			{
				return pDoc;
			}
			
		return NULL;
		
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark PREFERENCES


void
CMacCVSDoc::EditPreferences()
	{
	
		Boolean			bDone 						= false;
		Boolean			bPathWasChanged 	= false;
		
		Str255			pstrTemp					= "\p";
		Str255			pstrHostAddress		= "\p";
		
		UInt32			intCopyLen				= 0;
		UInt32			intTemp						= 0;
		UInt16			intTempPort				= 0;
		
		FSSpec			fsRoot						= { 0, 0, "\p" };

		LStr255			pstrTitle;
		
		StDialogHandler stHandler( kMacCVSPreferenceWindowID, this );
			
		LDialogBox *pPrefsDialog	= dynamic_cast<LDialogBox *>( stHandler.GetDialog() );
		ThrowIfNil_( pPrefsDialog );
		
		GetDescriptor( pstrTitle );
		pstrTitle.Append( "\p Preferences" );
		pPrefsDialog->SetDescriptor( pstrTitle );
		
		pPrefsDialog->SetDefaultButton( kPrefsOKButtonID );
		pPrefsDialog->SetCancelButton( kPrefsCancelButtonID );
		
//  JWW - Setup the folder drop area
		
		CLocalTreeView *pLocalTreeView = dynamic_cast<CLocalTreeView *>( pPrefsDialog->FindPaneByID(kLocalTreeViewID) );
		ThrowIfNil_( pLocalTreeView );
		
		// JWW - Tell the drop area about the preference message handler
		pLocalTreeView->SetPreferenceHandler(&stHandler);
		
//  Setup the panel views.
		
		LView *pRemoteView = dynamic_cast<LView *>( pPrefsDialog->FindPaneByID(kRemotePanelViewID) );
		ThrowIfNil_( pRemoteView );
		
		LView *pCheckoutView = dynamic_cast<LView *>( pPrefsDialog->FindPaneByID(kCheckoutUpdatePanelViewID) );
		ThrowIfNil_( pCheckoutView );

		LView *pMappingView = dynamic_cast<LView *>( pPrefsDialog->FindPaneByID(kFileMappingPanelViewID) );
		ThrowIfNil_( pMappingView );

		LView *pMiscView = dynamic_cast<LView *>( pPrefsDialog->FindPaneByID(kMiscPanelViewID) );
		ThrowIfNil_( pMiscView );

//  Setup popup menus.
		
		LPopupButton *pPanelPopup = dynamic_cast<LPopupButton *>( pPrefsDialog->FindPaneByID(kPrefsPanelPopupID) );
		ThrowIfNil_( pPanelPopup );
		
		LPopupButton *pTagTypePopup = dynamic_cast<LPopupButton *>( pPrefsDialog->FindPaneByID(kDefaultTagPopupID) );
		ThrowIfNil_( pTagTypePopup );

		LPopupGroupBox *pAuthMethodPopup = dynamic_cast<LPopupGroupBox *>( pPrefsDialog->FindPaneByID(kAuthenticationMethodPopupID) );
		ThrowIfNil_( pAuthMethodPopup );
		
// First disable all items, then enable ones we know about

		for (SInt16 i = pAuthMethodPopup -> GetMinValue (); i <= pAuthMethodPopup -> GetMaxValue (); i++) 
			{
				::DisableItem ( pAuthMethodPopup -> GetMacMenuH (), i );
			}
		
		::EnableItem ( pAuthMethodPopup -> GetMacMenuH (), kAuthenticationPasswordItem );
		::EnableItem ( pAuthMethodPopup -> GetMacMenuH (), kAuthenticationRSHItem );
		
		#ifdef CVSENGINE_KCLIENT_SUPPORT
				
		::EnableItem ( pAuthMethodPopup -> GetMacMenuH (), kAuthenticationKClientItem );
		
		#endif
		
		switch (mAuthenticationMethod) 
			{
				case kAuthenticationPassword:
					mAuthenticationMethodItem = kAuthenticationPasswordItem;
				break;
					
				case kAuthenticationRSH:
					mAuthenticationMethodItem = kAuthenticationRSHItem;
				break;

				case kAuthenticationKClient:					
					mAuthenticationMethodItem = kAuthenticationKClientItem;	
				break;
					
				default:
					
					// Red alert! This means that the prefs have an auth type that this version doesnt know
					// about. Set the default.
					
					mAuthenticationMethodItem = kAuthenticationPasswordItem;
					Assert_( false );
				break;				
			}
		
		pAuthMethodPopup->SetValue ( mAuthenticationMethodItem );
		pAuthMethodPopup->AddListener( &stHandler );

		LPopupButton *pCompressionLevel = dynamic_cast<LPopupButton *>( pPrefsDialog->FindPaneByID(kCompressionPopupID) );
		ThrowIfNil_( pCompressionLevel );
		
		if ( (mCompressionLevel > 0) && (mCompressionLevel <= 9) )
			{
				pCompressionLevel->SetValue( mCompressionLevel );
			}
		else
			{
				pCompressionLevel->SetValue( 1 );
			}
			
//  Setup checkboxes.

		LCheckBox *pFileUpdatesCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kOutputFileUpdatesID) );
		ThrowIfNil_( pFileUpdatesCheckBox );
		pFileUpdatesCheckBox->SetValue( mOutputFileUpdates );	

		LCheckBox *pDirUpdatesCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kOutputDirectoryUpdatesID) );
		ThrowIfNil_( pDirUpdatesCheckBox );
		pDirUpdatesCheckBox->SetValue( mOutputDirectoryUpdates );	

		LCheckBox *pSavePasswordCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kSavePasswordCheckBoxID) );
		ThrowIfNil_( pSavePasswordCheckBox );
		pSavePasswordCheckBox->SetValue( mSavePassword );	
			
		LCheckBox *pMessageWinCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kClearMessageWindowCheckBoxID) );
		ThrowIfNil_( pMessageWinCheckBox );
		pMessageWinCheckBox->SetValue( mClearMessageWindow );	

		LCheckBox *pScrollMessagesCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kScrollMessageWindowCheckBoxID) );
		ThrowIfNil_( pScrollMessagesCheckBox );
		pScrollMessagesCheckBox->SetValue( mScrollMessages );	

		LCheckBox *pListViewCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kFolderListViewCheckBoxID) );
		ThrowIfNil_( pListViewCheckBox );
		pListViewCheckBox->SetValue( mNewFoldersUseListView );

		LCheckBox *pLabelsCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kPropagateFolderLabelsCheckBoxID) );
		ThrowIfNil_( pLabelsCheckBox );
		pLabelsCheckBox->SetValue( mNewFoldersInheritLabels );

		LCheckBox *pCompressionCheckBox = dynamic_cast<LCheckBox *>( pPrefsDialog->FindPaneByID(kUseCompressionCheckBoxID) );
		ThrowIfNil_( pCompressionCheckBox );
		pCompressionCheckBox->SetValue( (mCompressionLevel > 0) );
		
		if ( mCompressionLevel > 0 )
			{
				pCompressionLevel->Enable();
			}
		else
			{
				pCompressionLevel->Disable();
			}
			
//  Setup edit fields. (ugh)
		
		LEditText *remoteHostField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kHostNamePreferenceItemID) );
		ThrowIfNil_( remoteHostField );
		
		if ( mPort == 0 )
			{
				remoteHostField->SetDescriptor( mHostName );
			}
		else
			{
				PStrCopy( mHostName, pstrHostAddress, sizeof(pstrHostAddress) );
				UInt32ToStr255( mPort, pstrTemp );
				PStrConcat( pstrHostAddress, "\p:", sizeof(pstrHostAddress) );
				PStrConcat( pstrHostAddress, pstrTemp, sizeof(pstrHostAddress) );

				remoteHostField->SetDescriptor( pstrHostAddress );			
			}
			
			
		LEditText *userNameField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kCVSUserNamePreferenceItemID) );
		ThrowIfNil_( userNameField );
		userNameField->SetDescriptor(mRemoteUser);

		LEditText *passwordField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kCVSRemotePasswordItemID) );
		ThrowIfNil_( passwordField );
		passwordField->SetDescriptor( mPassword );
				
		LEditText *cvsRootField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kCVSRootPreferenceItemID) );
		ThrowIfNil_( cvsRootField );
		cvsRootField->SetDescriptor(mCVSRoot);

		LEditText *cvsCommandField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kCVSCommandPreferenceItemID) );
		ThrowIfNil_( cvsCommandField );
		cvsCommandField->SetDescriptor( mCVSCommand );
		
		if ( mAuthenticationMethodItem == kAuthenticationRSHItem ) 
			{
				cvsCommandField->Enable();
				userNameField->Enable();
				passwordField->Disable();
				pSavePasswordCheckBox->Disable();
			} 
		else if ( mAuthenticationMethodItem == kAuthenticationPasswordItem ) 
			{
				cvsCommandField->Disable();
				userNameField->Enable();
				pSavePasswordCheckBox->Enable();
				
				if ( mSavePassword )
					{
						passwordField->Enable();
					}
				else
					{
						passwordField->Disable();
					}
			} 
		else 
			{
				cvsCommandField->Disable();
				userNameField->Disable();
				passwordField->Disable();
				pSavePasswordCheckBox->Disable();
			}
			
		LEditText *defaultModuleField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kModuleNameItemID) );
		ThrowIfNil_( defaultModuleField );
		defaultModuleField->SetDescriptor(mModuleName);

		LEditText *defaultTagField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kDefaultTagItemID) );
		ThrowIfNil_( defaultTagField );

		Assert_( mDefaultTag[0] != 1 );
		
		if ( mDefaultTag[0] > 1 )
			{				
				pstrTemp[0] = mDefaultTag[0] - 1;				
				::BlockMoveData( &mDefaultTag[2], &pstrTemp[1], pstrTemp[0] );
					
				if ( mDefaultTag[1] == 'T' )
					{
						pTagTypePopup->SetValue( kRevisionMenuItemID );
					}
				else if ( mDefaultTag[1] == 'D' )
					{
						pTagTypePopup->SetValue( kDateMenuItemID );
					}
				else
					{
						pTagTypePopup->SetValue( kRevisionMenuItemID );
						Assert_( false );
					}
					
				defaultTagField->SetDescriptor( pstrTemp );
			}
			
		LEditText *networkTimeOutField = dynamic_cast<LEditText *>( pPrefsDialog->FindPaneByID(kNetworkTimeOutItemID) );
		ThrowIfNil_( networkTimeOutField );
		networkTimeOutField->SetValue( mSocketTimeOut / 60 );

//  Setup radio buttons.
		
		LRadioButton* textBinaryMergingButton = dynamic_cast<LRadioButton *>( pPrefsDialog->FindPaneByID(kAutoMergeTextBinaryID) );
		ThrowIfNil_( textBinaryMergingButton );

		LRadioButton* textOnlyMergingButton = dynamic_cast<LRadioButton *>( pPrefsDialog->FindPaneByID(kAutoMergeTextOnlyID) );
		ThrowIfNil_( textOnlyMergingButton );
		
		LRadioButton* neverMergeButton = dynamic_cast<LRadioButton *>( pPrefsDialog->FindPaneByID(kNeverAutoMergeID) );
		ThrowIfNil_( neverMergeButton );

		textBinaryMergingButton->SetValue( false );
		textOnlyMergingButton->SetValue( false );
		neverMergeButton->SetValue( false );
		
		switch ( mCVSMergePolicy )
			{			
				case kCVSEngineTextAndBinaryMerging:	
					textBinaryMergingButton->SetValue( true );			
				break;
				
				case kCVSEngineTextMerging:				
					textOnlyMergingButton->SetValue( true );					
				break;
				
				case kCVSEngineNoMerging:
					neverMergeButton->SetValue( true );				
				break;
				
				default:				
					Assert_( false );
					neverMergeButton->SetValue( true );					
				break;			
			}
			
		SetNewPathInPrefsDialog( pPrefsDialog, mRootFileSpec );		

//  Setup mappings view.

		CMacCVSMappingsView *pMappingsTableView = dynamic_cast<CMacCVSMappingsView *>( pPrefsDialog->FindPaneByID(CMacCVSMappingsView::class_ID) );
		ThrowIfNil_( pMappingsTableView );
		
		pMappingsTableView->AddListener( &stHandler );

		{	
			CCVSEngineFileMappingRec	sMapping;
			LArrayIterator						stIterator( *mMappingList );
		
			while ( stIterator.Next(&sMapping) )
				{				
					pMappingsTableView->InsertNewItem( sMapping );				
				}		
		}
		
//  Setup buttons
		
		LPushButton	*addMappingButton = dynamic_cast<LPushButton *>( pPrefsDialog->FindPaneByID(kFileMappingAddButtonID) );
		ThrowIfNil_( addMappingButton );
		addMappingButton->Enable();
		
		LPushButton	*editMappingButton = dynamic_cast<LPushButton *>( pPrefsDialog->FindPaneByID(kFileMappingEditButtonID) );
		ThrowIfNil_( editMappingButton );

		LPushButton	*removeMappingButton = dynamic_cast<LPushButton *>( pPrefsDialog->FindPaneByID(kFileMappingRemoveButtonID) );
		ThrowIfNil_( removeMappingButton );
		
		{
			CCVSEngineFileMappingRec	theCurItem;
			UInt32										theCurIndex = 0;
		
			STableCell theFirstCell( 1,1 );
			
			pMappingsTableView->SelectCell( theFirstCell );
								
			theCurIndex = pMappingsTableView->GetSelectedItem( theCurItem );
								
			if ( theCurIndex > 0 )
				{
//  Don't allow removal of the default item.
									
					if ( theCurItem.fFileNamePStr[0] == 0 )
						{						
							removeMappingButton->Disable();
						}						
					else
						{						
							removeMappingButton->Enable();						
						}
						
					editMappingButton->Enable();				
				}													
			else
				{
					editMappingButton->Disable();
					removeMappingButton->Disable();			
				}		
		}
	
//  Setup the first pane.

		switch ( mPrefPanel )
			{		
				case kCheckoutUpdatePanelMenuItemID:
					pRemoteView->Hide();
					pMappingView->Hide();
					pMiscView->Hide();
					
					pCheckoutView->Show();

					pPanelPopup->SetValue( mPrefPanel );
					pPrefsDialog->SwitchTarget( defaultModuleField );					
				break;

				case kRemotePanelMenuItemID:
					pCheckoutView->Hide();
					pMappingView->Hide();
					pMiscView->Hide();

					pRemoteView->Show();
									
					pPanelPopup->SetValue( mPrefPanel );
					pPrefsDialog->SwitchTarget( remoteHostField );
				break;
								
				case kFileMappingPanelMenuItemID:
					pRemoteView->Hide();										
					pCheckoutView->Hide();
					pMiscView->Hide();

					pMappingView->Show();
					pPanelPopup->SetValue( mPrefPanel );				
				break;
				
				case kMiscPanelMenuItemID:	
					pRemoteView->Hide();										
					pCheckoutView->Hide();
					pMappingView->Hide();

					pMiscView->Show();
					pPanelPopup->SetValue( mPrefPanel );		
				break;
				
				default:

//  This case should never happen.
										
					Assert_( false );
					
					pRemoteView->Hide();
					pMappingView->Hide();
					pMiscView->Hide();

					pCheckoutView->Show();

					pPanelPopup->SetValue( kCheckoutUpdatePanelMenuItemID );
					pPrefsDialog->SwitchTarget( defaultModuleField );				
				break;			
			}

		stHandler.AddAttachment( new CThreadedTaskYieldAttachment );
		pPrefsDialog->Show();
		
		while ( !bDone ) 
			{		
				switch ( stHandler.DoDialog() ) 
					{

//  Ok, this sucks: We need to check if the user changed anything.
				
						case msg_OK:
							remoteHostField->GetDescriptor( pstrTemp );

							CUGetAddressAndPort( pstrTemp, pstrHostAddress, &intTempPort );
							
							if ( !::EqualString(pstrHostAddress, mHostName, true, true) )
								{								
									SetModified( true );
									PStrCopy( pstrHostAddress, mHostName, sizeof(mHostName) );							
								}
								
							if ( mPort != intTempPort )
								{
									SetModified( true );
									mPort = intTempPort;								
								}
								
							userNameField->GetDescriptor( pstrTemp );

							if ( !::EqualString(pstrTemp, mRemoteUser, true, true) )
								{								
									SetModified( true );
									PStrCopy( pstrTemp, mRemoteUser, sizeof(mRemoteUser) );								
								}
								
							passwordField->GetDescriptor( pstrTemp );

							if ( !::EqualString(pstrTemp, mPassword, true, true) )
								{							
									SetModified( true );
									PStrCopy( pstrTemp, mPassword, sizeof(mPassword) );								
								}
							
							cvsRootField->GetDescriptor( pstrTemp );

							if ( !::EqualString(pstrTemp, mCVSRoot, true, true) )
								{							
									SetModified( true );
									PStrCopy( pstrTemp, mCVSRoot, sizeof(mCVSRoot) );

//  Strip off invalid '/' characters...

									if ( (mCVSRoot[0] > 1) && (mCVSRoot[mCVSRoot[0]] == '/') )
										while ( (mCVSRoot[0] > 1) && (mCVSRoot[mCVSRoot[0]] == '/') )
											mCVSRoot[0]--;								
								}

							cvsCommandField->GetDescriptor( pstrTemp );

							if ( !::EqualString(pstrTemp, mCVSCommand, true, true) )
								{							
									SetModified( true );
									PStrCopy( pstrTemp, mCVSCommand, sizeof(mCVSCommand) );								
								}
										
							defaultModuleField->GetDescriptor( pstrTemp );

							if ( !::EqualString(pstrTemp, mModuleName, true, true) )
								{							
									SetModified( true );
									PStrCopy( pstrTemp, mModuleName, sizeof(mModuleName) );								
								}

							defaultTagField->GetDescriptor( pstrTemp );

							if ( pstrTemp[0] > 0 )
								{							
									intCopyLen = pstrTemp[0];
									
									if ( intCopyLen == 255 )
										{
											intCopyLen--;
										}
																	
									::BlockMoveData( &pstrTemp[1], &pstrTemp[2], intCopyLen );
									
									if ( pTagTypePopup->GetValue() == kRevisionMenuItemID )
										{
											pstrTemp[1] = 'T';
										}
									else
										{
											pstrTemp[1] = 'D';
										}
										
									pstrTemp[0] = intCopyLen + 1;
	
									if ( !::EqualString(pstrTemp, mDefaultTag, true, true) )
										{								
											SetModified( true );
											PStrCopy( pstrTemp, mDefaultTag, sizeof(mDefaultTag) );										
										}
										
									// if the default tag is not in the list, add it
									if ( mTagList != NULL )
										{
											Str255 cleanTagStr = "\p";			//ensure that whole string is zeroed out
											
											PStrCopy( mDefaultTag, cleanTagStr, sizeof(cleanTagStr) );
											
											// add if not there already
											if ( LArray::index_Bad == mTagList->FetchIndexOf( cleanTagStr, sizeof(Str255)) )
												mTagList->AddItem(cleanTagStr, sizeof(Str255) );										
										}								
								}
							else if ( mDefaultTag[0] > 0  )
								{								
									mDefaultTag[0] = 0;
									SetModified( true );						
								}
							
							intTemp = networkTimeOutField->GetValue();
							
							if ( intTemp == 0 )
								{
									intTemp = 1;
								}
								
							if ( (intTemp * 60) != mSocketTimeOut )
								{							
									mSocketTimeOut = intTemp * 60;
									SetModified( true );								
								}
							
							if ( mSavePassword != pSavePasswordCheckBox->GetValue() )
								{
									SetModified( true );
									mSavePassword = pSavePasswordCheckBox->GetValue();
								}
							
							if ( mSavePassword )
								{
									mPasswordValid = true;
								}
										
							if ( mOutputFileUpdates != pFileUpdatesCheckBox->GetValue() )
								{							
									SetModified( true );
									mOutputFileUpdates = pFileUpdatesCheckBox->GetValue();							
								}
								
							if ( mOutputDirectoryUpdates != pDirUpdatesCheckBox->GetValue() )
								{								
									SetModified( true );
									mOutputDirectoryUpdates = pDirUpdatesCheckBox->GetValue();							
								}
								
							if ( mAuthenticationMethodItem != pAuthMethodPopup->GetValue() )
								{							
									SetModified( true );
									
									mAuthenticationMethodItem = pAuthMethodPopup->GetValue();
									
									switch ( mAuthenticationMethodItem ) 
										{
											case kAuthenticationPasswordItem:
												mAuthenticationMethod = kAuthenticationPassword;
											break;
							
											case kAuthenticationRSHItem:
												mAuthenticationMethod = kAuthenticationRSH;
											break;

											#ifdef CVSENGINE_KCLIENT_SUPPORT
									
											case kAuthenticationKClientItem:
												mAuthenticationMethod = kAuthenticationKClient;	
											break;
									
											#endif
										
											default:
									
												// This means that the user somehow managed to select a menu item we thought we disabled
												SignalPStr_ ("\pAuthentication method unknown or disabled");
									
												mAuthenticationMethod = kAuthenticationPassword;
												Assert_( false );
											break;
										}
								}
								
							if ( mClearMessageWindow != pMessageWinCheckBox->GetValue() )
								{								
									SetModified( true );
									mClearMessageWindow = pMessageWinCheckBox->GetValue();								
								}

							if ( mScrollMessages != pScrollMessagesCheckBox->GetValue() )
								{								
									SetModified( true );
									mScrollMessages = pScrollMessagesCheckBox->GetValue();								
								}

							if ( mNewFoldersUseListView != pListViewCheckBox->GetValue() )
								{								
									SetModified( true );
									mNewFoldersUseListView = pListViewCheckBox->GetValue();								
								}

							if ( mNewFoldersInheritLabels != pLabelsCheckBox->GetValue() )
								{								
									SetModified( true );
									mNewFoldersInheritLabels = pLabelsCheckBox->GetValue();							
								}

							if ( pCompressionCheckBox->GetValue() > 0 )
								{
									if ( mCompressionLevel != pCompressionLevel->GetValue() )
										{
											SetModified( true );
											mCompressionLevel = pCompressionLevel->GetValue();
										}
								}
							else
								{
									if ( mCompressionLevel != 0 )
										{
											SetModified( true );
											mCompressionLevel = 0;
										}
								}

							if ( textBinaryMergingButton->GetValue() > 0 )
								{
									if ( mCVSMergePolicy != kCVSEngineTextAndBinaryMerging )
										{										
											SetModified( true );
											mCVSMergePolicy = kCVSEngineTextAndBinaryMerging;										
										}
								}
							else if ( textOnlyMergingButton->GetValue() > 0 )
								{							
									if ( mCVSMergePolicy != kCVSEngineTextMerging )
										{									
											SetModified( true );
											mCVSMergePolicy = kCVSEngineTextMerging;
										}										
								}							
							else if ( mCVSMergePolicy != kCVSEngineNoMerging )
								{								
									SetModified( true );
									mCVSMergePolicy = kCVSEngineNoMerging;								
								}
								
//  If the user changed the file mappings list, save the new 
//  settings.
								
							if ( pMappingsTableView->IsDirty() )
								{									
									CCVSEngineFileMappingRec	sMapping;
										
									SetModified( true );
	
									pMappingsTableView->Hide();							
									mMappingList->RemoveItemsAt( mMappingList->GetCount(), LArray::index_First );
									
									while ( pMappingsTableView->RemoveItem(1, sMapping) )
										{									
											mMappingList->InsertItemsAt( 1, LArray::index_Last, &sMapping, sizeof(CCVSEngineFileMappingRec) );									
										}										
								}
								
							if ( bPathWasChanged )
								{							
									if ( !FSpIsEqual( &fsRoot, &mRootFileSpec ) )
										{									
											SetModified( true );
											
											mRootFileSpec = fsRoot;
											PostProcessPathChange();				
										}									
								}
								
							bDone = true;

						break;

						// JWW - "Get Drop" message handler
						case 'GtDr':
							// JWW - Get the folder which was dropped on to the local tree view area
							fsRoot = *pLocalTreeView->GetDroppedFolder();
							// JWW - Update the preferences dialog to reflect the dropped folder
							SetNewPathInPrefsDialog( pPrefsDialog, fsRoot );
							bPathWasChanged = true;									
						break;
						
						case kCVSLocalDirectoryButtonMessageValue:
							{						
								StandardFileReply	ioReply;
								SInt32 theDirID;
								
								// JWW - Use Navigation Services if available
//								if ( UStandardFile::GetDirectory(ioReply) ) 
								if ( UConditionalDialogs::AskChooseFolder(ioReply.sfFile, theDirID) ) 
									{								
										fsRoot = ioReply.sfFile;
										SetNewPathInPrefsDialog( pPrefsDialog, fsRoot );
										bPathWasChanged = true;									
									}									
							}							
						break;

						case msg_Cancel:						
							bDone = true;						
						break;
					
						case kPrefsPanelPopupValueMessage:

//  Save the setting for the next time the user
//  opens the prefs dialog.
							
							mPrefPanel = pPanelPopup->GetValue();
							
							switch ( mPrefPanel )
								{							
									case kCheckoutUpdatePanelMenuItemID:
										pRemoteView->Hide();
										pMappingView->Hide();
										pMiscView->Hide();
										
										pCheckoutView->Show();

										pPanelPopup->SetValue( mPrefPanel );
										pPrefsDialog->SwitchTarget( defaultModuleField );									
									break;

									case kRemotePanelMenuItemID:
										pCheckoutView->Hide();
										pMappingView->Hide();
										pMiscView->Hide();

										pRemoteView->Show();
														
										pPanelPopup->SetValue( mPrefPanel );
										pPrefsDialog->SwitchTarget( remoteHostField );
									break;
													
									case kFileMappingPanelMenuItemID:
										pRemoteView->Hide();										
										pCheckoutView->Hide();
										pMiscView->Hide();

										pMappingView->Show();
										pPanelPopup->SetValue( mPrefPanel );								
									break;
									
									case kMiscPanelMenuItemID:						
										pRemoteView->Hide();										
										pCheckoutView->Hide();
										pMappingView->Hide();

										pMiscView->Show();
										pPanelPopup->SetValue( mPrefPanel );							
									break;
									
									default:
//  This case should never happen.
										
										Assert_( false );
										
										pRemoteView->Hide();
										pMappingView->Hide();
										pMiscView->Hide();

										pCheckoutView->Show();

										pPanelPopup->SetValue( kCheckoutUpdatePanelMenuItemID );
										pPrefsDialog->SwitchTarget( defaultModuleField );							
									break;								
								}
								
						break;
						
						case kAuthenticationMethodPopupID:						
							if ( pAuthMethodPopup->GetValue() == kAuthenticationRSHItem ) 
								{
									cvsCommandField->Enable();
									userNameField->Enable();
									passwordField->Disable();
									pSavePasswordCheckBox->Disable();
								} 
							else if ( pAuthMethodPopup->GetValue() == kAuthenticationPasswordItem ) 
								{
									cvsCommandField->Disable();
									userNameField->Enable();
									pSavePasswordCheckBox->Enable();

									if ( pSavePasswordCheckBox->GetValue() > 0 )
										{
											passwordField->Enable();
										}
									else
										{
											passwordField->Disable();
										}
								} 
							else 
								{
									cvsCommandField->Disable();
									userNameField->Disable();
									passwordField->Disable();
									pSavePasswordCheckBox->Disable();
								}
						break;
						
						case kUseCompressionCheckBoxID:
							if ( pCompressionCheckBox->GetValue() > 0 )
								{
									pCompressionLevel->Enable();
								}
							else
								{
									pCompressionLevel->Disable();
								}
						break;
						
						case kSavePasswordCheckBoxID:
							if ( pSavePasswordCheckBox->GetValue() > 0 )
								{
									if ( 	(pAuthMethodPopup->GetValue() == kAuthenticationRSHItem) || 
												(pAuthMethodPopup->GetValue() == kAuthenticationPasswordItem) )
										{
											passwordField->Enable();
										}
										
									mPasswordValid = true;
								}
							else
								{
									passwordField->Disable();
									passwordField->SetDescriptor( "\p" );
									mPasswordValid = false;
								}
						break;
						
						case CMacCVSMappingsView::kCMacCVSMappingsViewMessage:						
							{							
								CCVSEngineFileMappingRec	theCurItem;
								UInt32										theCurIndex = 0;
								
								theCurIndex = pMappingsTableView->GetSelectedItem( theCurItem );
								
								if ( theCurIndex > 0 )
									{

//  Don't allow removal of the default item.
									
										if ( theCurItem.fFileNamePStr[0] == 0 )
											{										
												removeMappingButton->Disable();
											}											
										else
											{										
												removeMappingButton->Enable();										
											}
											
										editMappingButton->Enable();								
									}																	
								else
									{
										editMappingButton->Disable();
										removeMappingButton->Disable();							
									}						
							}					
						break;
						
						case kFileMappingAddButtonMessage:						
							{							
								CCVSEngineFileMappingRec	theCurItem;
								TableIndexT								theNewItemIndex = 0,
																					theSelectedIndex = 0;
								
								theNewItemIndex = pMappingsTableView->AddNewItem();
								
								if ( theNewItemIndex > 0 )
									{								
										STableCell	theLastCell( theNewItemIndex, 1 );
										
										pMappingsTableView->SelectCell( theLastCell );
										
										theSelectedIndex = pMappingsTableView->GetSelectedItem( theCurItem );
							
										if ( theSelectedIndex > 0 )
											{
												if ( theCurItem.fFileNamePStr[0] == 0 )
													removeMappingButton->Disable();
													
												else removeMappingButton->Enable();
														
												editMappingButton->Enable();										
											}																			
										else
											{
												editMappingButton->Disable();
												removeMappingButton->Disable();									
											}
									}								
							}							
						break;
						
						case kFileMappingRemoveButtonMessage:							
							{						
								CCVSEngineFileMappingRec	theCurItem;
								UInt32										theCurIndex = 0;
																					
								theCurIndex = pMappingsTableView->GetSelectedItem( theCurItem );
								
								if ( theCurIndex > 0 )
								
//  Don't allow removal of zero length mapping- that is the
//  default item!

									if ( theCurItem.fFileNamePStr[0] > 0 )
										{									
											pMappingsTableView->RemoveItem( theCurIndex, theCurItem );
											pMappingsTableView->SetDirtyFlag( true );

//  Now update the selection and the buttons.
												
											theCurIndex = pMappingsTableView->GetSelectedItem( theCurItem );
								
											if ( theCurIndex > 0 )
												{
													if ( theCurItem.fFileNamePStr[0] == 0 )
														{
															removeMappingButton->Disable();
														}
													else 
														{
															removeMappingButton->Enable();
														}
													editMappingButton->Enable();											
												}																					
											else
												{
													editMappingButton->Disable();
													removeMappingButton->Disable();											
												}										
										}
								}						
						break;
						
						case kFileMappingEditButtonMessage:					
							pMappingsTableView->EditCurrentItem();							
						break;										
					}			
			}		
	}		

	
void
CMacCVSDoc::LoadPreferences()
	{

		OSErr					error							= noErr;
		LFileStream		*pFileStream 			= NULL;

		Boolean				bDone							= false;
		UInt32				intTemp						= 0;
		UInt32				intPrefVersion		= 0;
		UInt32				intKey						= 0;
		UInt32				intTagCount				= 0;
		UInt32				intMappingCount		= 0;
		UInt32				intDataSize				= 0;
		UInt16				tempUInt16				= 0;
		
		FSSpec				fsTarget					= { 0, 0, "\p" };
		Boolean				bWasChanged				= false;
		Boolean				bReadMappings			= false;
		
		AliasHandle		hAlias						= NULL;
		AliasRecord		sAliasHead;
		
		pFileStream = dynamic_cast<LFileStream *>( mFile );
		ThrowIfNil_( pFileStream );
		
		try
			{			
				pFileStream->SetMarker(0, streamFrom_Start);
				pFileStream->ReadBlock( &intTemp, sizeof(intTemp) );

				if ( intTemp != kMacCVSPrefsFileMagicNumber )
					{
						throw( (OSErr)kUnknownFileFormatErr );
					}
	
				pFileStream->ReadBlock( &intPrefVersion, sizeof(intPrefVersion) );
					
				if ( intPrefVersion >= kMacCVSReadableFileFormat && intPrefVersion < kMacCVSCurrentFileFormat )
					{
						LoadLegacyPreferences( intPrefVersion );					
					}
				else if ( intPrefVersion == kMacCVSPrefsFileFormat10 )
					{
						while ( !bDone )
							{
								pFileStream->ReadBlock( &intKey, sizeof(intKey) );
								
								switch ( intKey )
									{
										//  Host Name
										
										case kMacCVSPrefsServerNameData:										
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize > 255 )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else if ( intDataSize > 0 )
												{										
													pFileStream->ReadPString( mHostName );
													UMacCVSUtils::ReverseRot15PStr( mHostName );
												}

											Assert_( mHostName[0] + 1 == intDataSize );											
										break;

										case kMacCVSPrefsPortData:
											
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
								
											if ( intDataSize != sizeof(UInt32) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &intTemp, sizeof(intTemp) );
													mPort = intTemp;
												}												
											
										break;
										
										//  User Name
										
										case kMacCVSPrefsRemoteUserData:									
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize > 255 )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else if ( intDataSize > 0 )
												{										
													pFileStream->ReadPString( mRemoteUser );
													UMacCVSUtils::ReverseRot15PStr( mRemoteUser );
												}
								
											Assert_( mRemoteUser[0] + 1 == intDataSize );									
										break;

										//  Password
										
										case kMacCVSPrefsPasswordData:								
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize > 255 )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else if ( intDataSize > 0 )
												{										
													pFileStream->ReadPString( mPassword );
													UMacCVSUtils::ReverseRot15PStr( mPassword );									
												}
								
											Assert_( mPassword[0] + 1 == intDataSize );	
											mPasswordValid = true;
										break;

										//  Root
										
										case kMacCVSPrefsRootServerPathData:								
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize > 255 )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else if ( intDataSize > 0 )
												{										
													pFileStream->ReadPString( mCVSRoot );
													UMacCVSUtils::ReverseRot15PStr( mCVSRoot );				
												}
								
											Assert_( mCVSRoot[0] + 1 == intDataSize );			
										break;

										//  Command Line
										
										case kMacCVSPrefsCommandLineData:								
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize > 255 )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else if ( intDataSize > 0 )
												{										
													pFileStream->ReadPString( mCVSCommand );
													UMacCVSUtils::ReverseRot15PStr( mCVSCommand );
												}
								
											Assert_( mCVSCommand[0] + 1 == intDataSize );							
										break;

										//  Module
										
										case kMacCVSPrefsDefaultModuleData:								
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize > 255 )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else if ( intDataSize > 0 )
												{										
													pFileStream->ReadPString( mModuleName );
													UMacCVSUtils::ReverseRot15PStr( mModuleName );
												}
								
											Assert_( mModuleName[0] + 1 == intDataSize );							
										break;

										//  Default Tag
										
										case kMacCVSPrefsDefaultTagData:								
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize > 255 )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else if ( intDataSize > 0 )
												{										
													pFileStream->ReadPString( mDefaultTag );
													UMacCVSUtils::ReverseRot15PStr( mDefaultTag );
												}
								
											Assert_( mDefaultTag[0] + 1 == intDataSize );							
										break;
	
										//  Tag List
										
										case kMacCVSPrefsTagData:								
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );

											if ( intDataSize > 0 )
												{
													//  Better remove items just in case there are more
													//  than 1 tag data sets.
													
													mTagList->RemoveItemsAt( mTagList->GetCount(), LArray::index_First );
													
													for ( UInt32 intIndex = 1; intIndex <= intDataSize; intIndex++ )
														{												
															Str255 pstrCurTag	= "\p";
												
															pFileStream->ReadPString( pstrCurTag );
															UMacCVSUtils::ReverseRot15PStr( pstrCurTag );
												
															mTagList->InsertItemsAt( 1, LArray::index_Last, pstrCurTag, sizeof(Str255) );
														}
												}											
										break;

										//  Mappings List
										
										case kMacCVSPrefsMappingData:			
											Assert_( bReadMappings == false );					
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );

											if ( intDataSize > 0 )
												{
													mMappingList->RemoveItemsAt( mMappingList->GetCount(), LArray::index_First );
													
													for ( UInt32 intIndex = 1; intIndex <= intDataSize; intIndex++ )
														{												
															CMacCVSPrefsFileMappingRec		sAligned;
															CCVSEngineFileMappingRec			sNative;
												
															//  Read the aligned struct and stuff it into the non-aligned struct.
										
															pFileStream->ReadBlock( &sAligned, sizeof(sAligned) );
															
															::BlockMoveData( sAligned.fFileNamePStr, sNative.fFileNamePStr, sAligned.fFileNamePStr[0] + 1 );
															sNative.fFileType 		= sAligned.fFileType;
															sNative.fFileCreator 	= sAligned.fFileCreator;
															sNative.fTranslation 	= sAligned.fTranslation;
															sNative.fEncoding 		= sAligned.fEncoding;
															
															mMappingList->InsertItemsAt( 1, LArray::index_Last, &sNative, sizeof(sNative) );
														}
												}														
											bReadMappings = true;							
										break;

										case kMacCVSPrefsDiffOptionsData:
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize != sizeof(CCVSPrefsFileDiffOptions) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &mDiffOptions, sizeof(CCVSPrefsFileDiffOptions) );
												}																				
										break;

										case kMacCVSPrefsCompareOptionsData:
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize != sizeof(CCVSPrefsFileDiffOptions) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &mCompareOptions, sizeof(CCVSPrefsFileDiffOptions) );
												}																				
										break;

										//  Merge preference
										
										case kMacCVSPrefsMergeData:
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize != sizeof(UInt32) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &intTemp, sizeof(intTemp) );
													mCVSMergePolicy = intTemp;
												}												
										break;
										
										//  Authentication
										
										case kMacCVSPrefsAuthTypeData:
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize != sizeof(UInt32) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &intTemp, sizeof(intTemp) );
													mAuthenticationMethod = intTemp;
												}												
										break;

										//  Socket timeout
										
										case kMacCVSPrefsSocketTimeoutData:
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize != sizeof(UInt32) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &intTemp, sizeof(intTemp) );
													mSocketTimeOut = intTemp;
												}												
										break;

										//  Gzip settings
										
										case kMacCVSPrefsGzipData:
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize != sizeof(UInt32) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &intTemp, sizeof(intTemp) );
													mCompressionLevel = intTemp;
												}												
										break;

										//  Flags
										
										case kMacCVSPrefsFlagsData:
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											if ( intDataSize != sizeof(UInt32) )
												{
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
											else
												{
													pFileStream->ReadBlock( &intTemp, sizeof(intTemp) );													
													
													mOutputFileUpdates = ( intTemp & kOutputFileUpdatesFlag ) != 0;						
													mClearMessageWindow = ( intTemp & kClearMessageWindowFlag ) != 0;							
													mOutputDirectoryUpdates = ( intTemp & kOutputDirectoryUpdatesFlag ) != 0;													
													mScrollMessages = ( intTemp & kScrollMessageWindowFlag ) != 0;													
													mNewFoldersUseListView = ( intTemp & kListFolderViewsFlag ) != 0;
													mNewFoldersInheritLabels = ( intTemp & kPropageFolderLabelsFlag ) != 0;
													mSavePassword = ( intTemp & kSavePasswordFlag ) != 0;
												}												
										break;
										
										//  Alias
										
										case kMacCVSPrefsAliasData:								
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											
											//  intDataSize is the size of the WHOLE alias data
											
											if ( intDataSize >= sizeof(AliasRecord) )
												{								
													pFileStream->ReadBlock( &sAliasHead, sizeof(AliasRecord) );
										
													hAlias = (AliasHandle)::NewHandle( sAliasHead.aliasSize );
										
													if ( hAlias == NULL )
														throw( ::MemError() );
											
													::BlockMoveData( &sAliasHead, *hAlias, sizeof(AliasRecord) );

													if ( (sAliasHead.aliasSize - sizeof(AliasRecord)) > 0 )
														{														
															StHandleLocker( (Handle)hAlias );

															// Bug fix sfraser -- were overwriting the end of the handle
															pFileStream->ReadBlock( (((char *)*hAlias) + sizeof(AliasRecord)), sAliasHead.aliasSize - sizeof(AliasRecord) );
														}
														
													error = ::ResolveAlias( NULL, hAlias, &fsTarget, &bWasChanged );
														
													if ( error == noErr )
														{														
															mRootFileSpec = fsTarget;														
															mProjectRootSpecified = true;															
														}

													if ( hAlias != NULL )
														{
															::DisposeHandle( (Handle)hAlias );
															hAlias = NULL;
														}
												}
											else
												{
													Assert_( false );
													pFileStream->SetMarker( intDataSize, streamFrom_Marker );
												}
										break;
																												
										case kMacCVSPrefsEOFData:
											bDone = true;
										break;
										
										default:
											//  Uh oh, unknown item.
											
											Assert_( false );
											pFileStream->ReadBlock( &intDataSize, sizeof(intDataSize) );
											pFileStream->SetMarker( intDataSize, streamFrom_Marker );									
										break;
									}
							}
							
						if ( !bReadMappings )
							{
								InitDefaultMappingList();
							}
							
						if ( !mProjectRootSpecified )
							{
								//  If we didn't load the project root, 
								//  better warn the user.
								throw( (OSErr)kNoRootSpecifiedErr );
							}
					}	
				else
					{
						if ( intPrefVersion < kMacCVSCurrentFileFormat )
							{			
								throw( (OSErr)kOutdatedPrefsErr );
							}	
						else 
							{
								throw( (OSErr)kUnknownPrefsVersionErr );
							}
					}								
			}			
		catch ( ... )
			{		
				if ( hAlias != NULL )
					{
						::DisposeHandle( (Handle)hAlias );
						hAlias = NULL;
					}
				throw;		
			}
			
	}

	
void
CMacCVSDoc::LoadLegacyPreferences( const UInt32 inVersion )
	{

		OSErr					error							= noErr;
		LFileStream		*pFileStream 			= NULL;

		UInt32				intTemp						= 0;
		UInt32				intTagCount				= 0;
		UInt32				intMappingsCount	= 0;
		
		FSSpec				fsTarget					= { 0, 0, "\p" };
		Boolean				bWasChanged				= false;

		AliasHandle		hAlias						= NULL;
		AliasRecord		sAliasHead;
		
		pFileStream = dynamic_cast<LFileStream *>( mFile );
		ThrowIfNil_( pFileStream );
		
		try
			{			
				if ( (inVersion >= kMacCVSReadableFileFormat) && (inVersion < kMacCVSCurrentFileFormat) ) 
					{				
						pFileStream->ReadBlock( &intTemp, sizeof(UInt32) );
						mCVSMergePolicy = intTemp;
						
						pFileStream->ReadBlock( &intTemp, sizeof(UInt32) );
						//mLFTranslation = intTemp;

						//  Read and translate flags.

						pFileStream->ReadBlock( &intTemp, sizeof(UInt32) );

						if (( intTemp & kUseRSHFlag ) != 0) 
							{
								mAuthenticationMethod = kAuthenticationRSH;
							} 
						else 
							{
								mAuthenticationMethod = kAuthenticationPassword;
							}

						mOutputFileUpdates = ( intTemp & kOutputFileUpdatesFlag ) != 0;						
						mClearMessageWindow = ( intTemp & kClearMessageWindowFlag ) != 0;					
						mOutputDirectoryUpdates = ( intTemp & kOutputDirectoryUpdatesFlag ) != 0;						
						mScrollMessages = ( intTemp & kScrollMessageWindowFlag ) != 0;					
						mNewFoldersUseListView = ( intTemp & kListFolderViewsFlag ) != 0;
						mNewFoldersInheritLabels = ( intTemp & kPropageFolderLabelsFlag ) != 0;

						//	Authentication method, only in version 8 and later
						//	In versions before 8, authentication method is either password or RSH and
						//	it's read from the flags above
						
						if ( inVersion >= kMacCVSPrefsFileFormat8)
							{
								pFileStream->ReadBlock ( &intTemp, sizeof(UInt32) );
								mAuthenticationMethod = intTemp;
							}

						pFileStream->ReadBlock( &intTemp, sizeof(UInt32) );
						mSocketTimeOut = intTemp;
							
						pFileStream->ReadPString( mHostName );
						UMacCVSUtils::ReverseRot15PStr( mHostName );
						
						pFileStream->ReadPString( mRemoteUser );
						UMacCVSUtils::ReverseRot15PStr( mRemoteUser );
						
						pFileStream->ReadPString( mPassword );
						UMacCVSUtils::ReverseRot15PStr( mPassword );
						mPasswordValid = true;
						
						pFileStream->ReadPString( mCVSCommand );
						UMacCVSUtils::ReverseRot15PStr( mCVSCommand );
						
						pFileStream->ReadPString( mCVSRoot );
						UMacCVSUtils::ReverseRot15PStr( mCVSRoot );
													
						pFileStream->ReadPString( mModuleName );
						UMacCVSUtils::ReverseRot15PStr( mModuleName );

						pFileStream->ReadPString( mDefaultTag );
						UMacCVSUtils::ReverseRot15PStr( mDefaultTag );
						
						//	Read diff and compare defaults
						
						if ( inVersion >= kMacCVSPrefsFileFormat7)
							{						
								pFileStream->ReadBlock( &mDiffOptions, sizeof(CCVSPrefsFileDiffOptions) );
								pFileStream->ReadBlock( &mCompareOptions, sizeof(CCVSPrefsFileDiffOptions) );							
							}

						//  Read stored tags.
						
						pFileStream->ReadBlock( &intTagCount, sizeof(UInt32) );

						mTagList->RemoveItemsAt( mTagList->GetCount(), LArray::index_First );

						for ( UInt32 intIndex = 1; intIndex <= intTagCount; intIndex++ )
							{							
								Str255 pstrCurTag	= "\p";
								
								pFileStream->ReadPString( pstrCurTag );
								UMacCVSUtils::ReverseRot15PStr( pstrCurTag );
								
								mTagList->InsertItemsAt( 1, LArray::index_Last, pstrCurTag, sizeof(Str255) );									
							}

						//  Read file mapping preferences.

						pFileStream->ReadBlock( &intMappingsCount, sizeof(UInt32) );
							
						mMappingList->RemoveItemsAt( mMappingList->GetCount(), LArray::index_First );
															
						for ( UInt32 intIndex = 1; intIndex <= intMappingsCount; intIndex++ )
							{							
								CMacCVSPrefsFileMappingRec	sAligned;
								CCVSEngineFileMappingRec		sNative;

								//  Read the aligned struct and stuff it into the non-aligned struct.
								
								pFileStream->ReadBlock( &sAligned, sizeof(sAligned) );
								
								::BlockMoveData( sAligned.fFileNamePStr, sNative.fFileNamePStr, sAligned.fFileNamePStr[0] + 1 );
								sNative.fFileType 		= sAligned.fFileType;
								sNative.fFileCreator 	= sAligned.fFileCreator;
								sNative.fTranslation 	= sAligned.fTranslation;
								sNative.fEncoding 		= sAligned.fEncoding;
								
								mMappingList->InsertItemsAt( 1, LArray::index_Last, &sNative, sizeof(CCVSEngineFileMappingRec) );									
							}

						//  If we have an alias, load that baby in.
							
						if ( pFileStream->GetLength() > pFileStream->GetMarker() )
							{							
								pFileStream->ReadBlock( &sAliasHead, sizeof(AliasRecord) );
								
								hAlias = (AliasHandle)::NewHandle( sAliasHead.aliasSize );
								
								if ( hAlias == NULL )
									{
										throw( ::MemError() );
									}
									
								::BlockMoveData( &sAliasHead, *hAlias, sizeof(AliasRecord) );
								
								if ( (sAliasHead.aliasSize - sizeof(AliasRecord)) > 0 )
									{								
										StHandleLocker( (Handle)hAlias );
										
										// pFileStream->ReadBlock( (((char *)*hAlias) + sizeof(AliasRecord)), sAliasHead.aliasSize );
										
										// Bug fix sfraser -- were overwriting the end of the handle
										pFileStream->ReadBlock( (((char *)*hAlias) + sizeof(AliasRecord)), sAliasHead.aliasSize - sizeof(AliasRecord) );								
									}
									
								error = ::ResolveAlias( NULL, hAlias, &fsTarget, &bWasChanged );
									
								if ( error == noErr )
									{							
										mRootFileSpec = fsTarget;								
										mProjectRootSpecified = true;									
									}

								if ( hAlias != NULL )
									{
										::DisposeHandle( (Handle)hAlias );
										hAlias = NULL;
									}
							}
						else 
							{
								throw( (OSErr)kNoRootSpecifiedErr );
							}
					}
				else 
					{						
						if ( inVersion < kMacCVSCurrentFileFormat )
							{
								throw( (OSErr)kOutdatedPrefsErr );
							}
						else 
							{
								throw( (OSErr)kUnknownPrefsVersionErr );
							}
					}					
			}		
		catch ( ... )
			{			
				if ( hAlias != NULL )
					{
						::DisposeHandle( (Handle)hAlias );
						hAlias = NULL;
					}
					
				throw;							
			}
			
	}
	

void
CMacCVSDoc::SavePreferences()
	{

		OSErr						error						= noErr;
		LFileStream			*pFileStream	 	= NULL;

		UInt32					intTemp 				= 0;
		UInt32					intFlags				= 0;
		UInt16					tempUInt16			= 0;

		Str255					pstrTemp				= "\p";
		AliasHandle			hAlias 					= NULL;
		
		pFileStream	= dynamic_cast<LFileStream *>( mFile );
		ThrowIfNil_( pFileStream );
		
		try
			{			
				pFileStream->SetMarker( 0, streamFrom_Start );

				//  Magic Number and Version

				intTemp = kMacCVSPrefsFileMagicNumber;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );

				intTemp = kMacCVSCurrentFileFormat;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );

				//  Write Flags

				intTemp = kMacCVSPrefsFlagsData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = sizeof( UInt32 );
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
	
				if ( GetUseRSH () )
					{
						intFlags |= kUseRSHFlag;
					}
				if ( mOutputFileUpdates )
					{
						intFlags |= kOutputFileUpdatesFlag;
					}				
				if ( mClearMessageWindow )
					{
						intFlags |= kClearMessageWindowFlag;
					}
				if ( mOutputDirectoryUpdates )
					{
						intFlags |= kOutputDirectoryUpdatesFlag;
					}
				if ( mScrollMessages )
					{
						intFlags |= kScrollMessageWindowFlag;
					}
				if ( mNewFoldersUseListView )
					{
						intFlags |= kListFolderViewsFlag;
					}
				if ( mNewFoldersInheritLabels )
					{
						intFlags |= kPropageFolderLabelsFlag;
					}
				if ( mSavePassword )
					{
						intFlags |= kSavePasswordFlag;
					}
					
				pFileStream->WriteBlock( &intFlags, sizeof(UInt32) );

				//	Write merge method
				
				intTemp = kMacCVSPrefsMergeData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = sizeof( UInt32 );
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				pFileStream->WriteBlock( &mCVSMergePolicy, sizeof (UInt32) );

				//	Write authentication method
				
				intTemp = kMacCVSPrefsAuthTypeData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = sizeof( UInt32 );
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				pFileStream->WriteBlock( &mAuthenticationMethod, sizeof (UInt32) );

				//  Socket time out

				intTemp = kMacCVSPrefsSocketTimeoutData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = sizeof( UInt32 );
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				pFileStream->WriteBlock( &mSocketTimeOut, sizeof(UInt32) );

				//  GZip

				intTemp = kMacCVSPrefsGzipData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = sizeof( UInt32 );
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				pFileStream->WriteBlock( &mCompressionLevel, sizeof(UInt32) );

				//  Port data

				if ( mPort != 0 )
					{
						intTemp = kMacCVSPrefsPortData;
						pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
						intTemp = sizeof( UInt32 );
						pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
						intTemp = mPort;
						pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
					}
						
				//  Write all the preference field pstrings with our simple 
				//  Rot 15 "encryption". This will confuse the casual browsers 
				//  a little bit. (a *very little* bit)

				//  Host Name
				intTemp = kMacCVSPrefsServerNameData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = mHostName[0] + 1;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				PStrCopy( mHostName, pstrTemp, sizeof(pstrTemp) );
				UMacCVSUtils::Rot15PStr( pstrTemp );
				pFileStream->WritePString( pstrTemp );

				//  User Name
				intTemp = kMacCVSPrefsRemoteUserData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = mRemoteUser[0] + 1;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				PStrCopy( mRemoteUser, pstrTemp, sizeof(pstrTemp) );
				UMacCVSUtils::Rot15PStr( pstrTemp );
				pFileStream->WritePString( pstrTemp );

				//  Password
				if ( mSavePassword )
					{
						Assert_( mPasswordValid == true );
						
						intTemp = kMacCVSPrefsPasswordData;
						pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
						intTemp = mPassword[0] + 1;
						pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
						PStrCopy( mPassword, pstrTemp, sizeof(pstrTemp) );
						UMacCVSUtils::Rot15PStr( pstrTemp );
						pFileStream->WritePString( pstrTemp );
					}
					
				//  Command line
				intTemp = kMacCVSPrefsCommandLineData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = mCVSCommand[0] + 1;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				PStrCopy( mCVSCommand, pstrTemp, sizeof(pstrTemp) );
				UMacCVSUtils::Rot15PStr( pstrTemp );
				pFileStream->WritePString( pstrTemp );

				//  Server root
				intTemp = kMacCVSPrefsRootServerPathData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = mCVSRoot[0] + 1;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				PStrCopy( mCVSRoot, pstrTemp, sizeof(pstrTemp) );
				UMacCVSUtils::Rot15PStr( pstrTemp );
				pFileStream->WritePString( pstrTemp );

				//  Default module
				intTemp = kMacCVSPrefsDefaultModuleData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = mModuleName[0] + 1;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				PStrCopy( mModuleName, pstrTemp, sizeof(pstrTemp) );
				UMacCVSUtils::Rot15PStr( pstrTemp );
				pFileStream->WritePString( pstrTemp );

				//  Default tag
				intTemp = kMacCVSPrefsDefaultTagData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = mDefaultTag[0] + 1;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				PStrCopy( mDefaultTag, pstrTemp, sizeof(pstrTemp) );
				UMacCVSUtils::Rot15PStr( pstrTemp );
				pFileStream->WritePString( pstrTemp );

				//	Write diff and compare defaults
				intTemp = kMacCVSPrefsDiffOptionsData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = sizeof(CCVSPrefsFileDiffOptions);
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				pFileStream->WriteBlock( &mDiffOptions, sizeof(CCVSPrefsFileDiffOptions) );
				
				intTemp = kMacCVSPrefsCompareOptionsData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				intTemp = sizeof(CCVSPrefsFileDiffOptions);
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );	
				pFileStream->WriteBlock( &mCompareOptions, sizeof(CCVSPrefsFileDiffOptions) );
				
				//  Write tag list

				intTemp = kMacCVSPrefsTagData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
		
				intTemp = mTagList->GetCount();				
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				
				for ( UInt32 intIndex = 1; intIndex <= intTemp; intIndex++ )
					{						
						Str255 pstrCurTag = "\p";
						
						mTagList->FetchItemAt( intIndex, pstrCurTag );						
						UMacCVSUtils::Rot15PStr( pstrCurTag );
								
						pFileStream->WritePString( pstrCurTag );								
					}

				//  Write mappings list

				intTemp = kMacCVSPrefsMappingData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
		
				intTemp = mMappingList->GetCount();	
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
				
				for ( UInt32 intIndex = 1; intIndex <= intTemp; intIndex++ )
					{						
						CMacCVSPrefsFileMappingRec	sAligned;
						CCVSEngineFileMappingRec		sNative;
						
						mMappingList->FetchItemAt( intIndex, &sNative );
									
						::BlockMoveData( sNative.fFileNamePStr, sAligned.fFileNamePStr, sNative.fFileNamePStr[0] + 1 );
						sAligned.fFileType 			= sNative.fFileType;
						sAligned.fFileCreator 	= sNative.fFileCreator;
						sAligned.fTranslation 	= sNative.fTranslation;
						sAligned.fEncoding 			= sNative.fEncoding;

						//  I do sizeof(sAligned) because I was a moron and named
						//  both the structs something very similiar.
	
						pFileStream->WriteBlock( &sAligned, sizeof(sAligned) );									
					}

				//  If the user has specified a root directory, save the alias.
					
				if ( (mRootFileSpec.name[0] != 0) && (mRootFileSpec.vRefNum != 0) && (mRootFileSpec.parID != 0) )
					{					
						error = ::NewAlias( NULL, &mRootFileSpec, &hAlias );
						
						if ( error == noErr )
							{						
								StHandleLocker( (Handle)hAlias );
	
								intTemp = kMacCVSPrefsAliasData;
								pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );			
								intTemp = (*hAlias)->aliasSize;
								pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );
						
								pFileStream->WriteBlock( *hAlias, (*hAlias)->aliasSize );
							}
												
						if ( hAlias != NULL )
							{
								::DisposeHandle( (Handle)hAlias );
								hAlias = NULL;
							}
					}

				//  Write terminator
				intTemp = kMacCVSPrefsEOFData;
				pFileStream->WriteBlock( &intTemp, sizeof(UInt32) );

				pFileStream->SetLength( pFileStream->GetMarker() );		
			}		
		catch ( ... )
			{			
				if ( hAlias != NULL )
					{
						::DisposeHandle( (Handle)hAlias );
						hAlias = NULL;
					}
				throw;			
			}
			
	}


void
CMacCVSDoc::SetNewPathInPrefsDialog( LDialogBox* inPrefDialog, const FSSpec &inFileSpec )
	{

		OSErr				error					= noErr;
		LCaption*		pathCaption		= NULL;
		UInt32			fullPathLen		= 255;
		Str255			pathStr				= "\p";
		Rect				pathCaptionRect = { 0, 0, 0, 0 };
		
		if ( inPrefDialog != NULL ) 
			{
			
				if ( (inFileSpec.name[0] != 0) && (inFileSpec.vRefNum != 0) && (inFileSpec.parID != 0) )
					{
					
						error = FSpGetFullPathStr( &inFileSpec, &fullPathLen, (char *)&pathStr[1] );

						if ( error == noErr ) 
							{

								pathStr[0] = fullPathLen;

							}
						else PStrCopy( "\pAn error occurred!", pathStr, sizeof(pathStr) );
				
					}
				else PStrCopy( "\pNo path specified.", pathStr, sizeof(pathStr) );
				
				pathCaption = dynamic_cast<LCaption *>( inPrefDialog->FindPaneByID(kCVSLocalDirectoryLabelID) );
				
				ThrowIfNil_( pathCaption );

//  Truncate the string if we need to.
				
				pathCaption->CalcLocalFrameRect( pathCaptionRect );
				
				StTextState	theTextState;
				UTextTraits::SetPortTextTraits( 136 );
				
				::TruncString( pathCaptionRect.right - pathCaptionRect.left - 2, pathStr, truncMiddle );
				
				pathCaption->SetDescriptor( pathStr );
			
			}
		
	}


void CMacCVSDoc::PostProcessPathChange()
	{

		//  PostProcessPathChange() does all the clean-up needed when
		//  the user changes the local project root path.	

		LWindow *pWindowItem	= NULL;

		//  Nuke all running network connections.
		KillAllTasks();
		
		//  Delete all extra project views.
		if ( mWindowList != NULL ) 
			{		
				LArrayIterator	stIterator( *mWindowList );
				
				while ( stIterator.Next(&pWindowItem) ) 
					{
						mWindowList->RemoveItemsAt( 1, stIterator.GetCurrentIndex() );
					
						delete pWindowItem;				
					}				
			}
			
		Assert_( mWindow != NULL );
		
		if ( mWindow != NULL )
			{				
				CMacCVSProjectView 	*pProjectView	= NULL;

				LPane	*pPane = mWindow->FindPaneByID( CMacCVSProjectView::class_ID );
				
				pProjectView = dynamic_cast<CMacCVSProjectView *>(pPane);
				ThrowIfNil_( pProjectView );
				
				pProjectView->SetRootItem( &mRootFileSpec, NULL );				
			}
		else
			{
				mWindow = NewProjectWindow( &mRootFileSpec, NULL, mDocumentFileSpec.name );
			}
			
		ThrowIfNil_( mWindow );
			
		mWindow->Show();
		
		if ( FSpIsValid(mRootFileSpec) )
			{
				mProjectRootSpecified = true;
			}
	}


Boolean
CMacCVSDoc::GetPasswordFromUser()
	{
	
		Boolean bResult				 	= false;
		Boolean	bDone						= false;

		if ( (!mPasswordValid) && (mAuthenticationMethod == kAuthenticationPassword) )
			{			
				StDialogHandler stHandler( kPasswordDialogPPobID, mSuperCommander );
				stHandler.AddAttachment( new CThreadedTaskYieldAttachment );
				
				LDialogBox *pDialogBox = dynamic_cast<LDialogBox *>( stHandler.GetDialog() );
				ThrowIfNil_( pDialogBox );
				
				LEditText *pPasswordField = dynamic_cast<LEditText *>( pDialogBox->FindPaneByID(kPasswordDialogFieldID) );
				ThrowIfNil_( pPasswordField );
				
				pDialogBox->SetDefaultButton( kPasswordDialogOKButtonID );
				pDialogBox->SetCancelButton( kPasswordDialogCancelButtonID );
				
				pDialogBox->Show();
				
				while ( !bDone )
					{			
						switch ( stHandler.DoDialog() )
							{ 					
								case msg_OK:
									bResult = true;							
									bDone = true;	
									
									pPasswordField->GetDescriptor( mPassword );	
									mPasswordValid = true;				
								break;

								case msg_Cancel:						
									bDone = true;
								break;												
							}			
					}
			}
		else
			{
				bResult = true;
			}
			
		return bResult;
	
	}


void
CMacCVSDoc::InvalidateCachedPassword()
	{
		if ( !mSavePassword )
			{
				mPasswordValid = false;
			}
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark COMMAND HANDLING


void
CMacCVSDoc::AttemptClose( Boolean inRecordIt )
	{

		Boolean	bResult = false;
		
		bResult = AttemptQuitSelf(kAEAsk);
		
		if ( bResult == true )
			{
				LSingleDoc::AttemptClose( inRecordIt );
			}
			
	}


Boolean
CMacCVSDoc::AttemptQuitSelf( Int32 inSaveOption )
	{
	
		LWindow		*pWindowItem		= NULL;
		
		Boolean		bDone 					= false;
		Boolean		bWillClose			= true;

		if ( (mConnectionList->GetCount() > 0) && (inSaveOption == kAEAsk) )
			{
				bWillClose = AllowKillAllTasks();		
			}

		if ( bWillClose && IsModified() )
			{			
				bWillClose = false;
		
				LDialogBox	*pDialogBox				= NULL;
				LCaption		*pMessageText			= NULL;
			
				Str255			pstrDocName 			= "\p";
				LStr255			pstrMessage;

				StDialogHandler	stHandler( kSaveDialogPPobID, mSuperCommander );
				
				pDialogBox = dynamic_cast<LDialogBox *>( stHandler.GetDialog() );	
				ThrowIfNil_( pDialogBox );
				
				pMessageText = dynamic_cast<LCaption *>( pDialogBox->FindPaneByID(kSaveDialogCaptionID) );
				ThrowIfNil_( pMessageText );
				
				GetDescriptor( pstrDocName );
				
				pstrMessage.Append( "\pDo you want to save changes to " );
				pstrMessage.Append( pstrDocName );
				pstrMessage.Append( "\p before closing?" );
				
				pMessageText->SetDescriptor( pstrMessage );

				pDialogBox->SetDefaultButton( kSaveDialogSaveButtonID );
				pDialogBox->SetCancelButton( kSaveDialogCancelButtonID );
				
				pDialogBox->Show();
				
				while ( !bDone )
					{			
						switch ( stHandler.DoDialog() )
							{ 						
								case msg_OK:

									SavePreferences();
									
									bWillClose = true;							
									bDone = true;

									//  hack, we don't want PP to ask
									//  the user again in LDocument::AttemptClose().
									SetModified( false );
								break;

								case msg_Cancel:
								
									bDone = true;
									
								break;
								
								case kSaveDialogDontSaveButtonValueMessage:
								
									bWillClose = true;
									bDone = true;
									
									//  hack, we don't want PP to ask
									//  the user again in LDocument::AttemptClose().
									SetModified( false );
								break;														
							}				
					}			
			}
			
		if ( bWillClose == true )
			{
				KillAllTasks();
				KillAllViews();
			}
					
		return bWillClose;
		
	}


Boolean
CMacCVSDoc::AllowSubRemoval( LCommander *inSub )
	{
	
		Boolean		bResult 			= false;
		LWindow 	*pWindowItem 	= dynamic_cast<LWindow *>( inSub );
		
		Assert_( pWindowItem != NULL );
		
		if ( pWindowItem != NULL )
			{			
				if ( pWindowItem != mWindow )
					{						
						if ( GetWindowKind(pWindowItem) != 0 )
							{									
								RemoveFromWindowList( pWindowItem );
								
								bResult = true;							
							}
						else 
							{
								Assert_( false );
							}
					}
				else
					{		
						bResult = LSingleDoc::AllowSubRemoval( inSub );										
					}						
			}
		else 
			{
				bResult = LSingleDoc::AllowSubRemoval( inSub );
			}
			
		return bResult;
			
	}
	
	
Boolean
CMacCVSDoc::ObeyCommand( CommandT inCommand, void *ioParam )
	{

		Boolean 						cmdHandled 			= true;
		LArray							*pItemList 			= NULL;
		LWindow							*pTopWindow			= NULL;
		CMacCVSProjectView	*pProjectView		= NULL;
		CMacCVSLurkersView	*pLurkersView		= NULL;
			
		try
			{					
				pTopWindow = UDesktop::FetchTopRegular();								
				Assert_( pTopWindow != NULL );
							
				if ( IsOurWindow(pTopWindow) )
					{
						if ( GetWindowKind(pTopWindow) == kProjectViewWindowKind )		
							{
								pProjectView = GetProjectWindowView( pTopWindow );
							}
						else if ( GetWindowKind(pTopWindow) == kLurkersViewWindowKind )
							{
								pLurkersView = GetLurkersWindowView( pTopWindow );
							}
					}
					
				switch ( inCommand ) 
					{

						case cmd_Save:
						
							SavePreferences();
							SetModified( false );
							
						break;
						
						case cmd_SaveAs:
							{
								LStr255		oldName(mDocumentFileSpec.name);
								FSSpec	newFileSpec;
								
								if ( AskSaveAs(newFileSpec, true) )			// can throw
									{
										
										delete mFile;
										mFile = nil;
										
										mDocumentFileSpec = newFileSpec;
										
										mFile = new LFileStream(mDocumentFileSpec);		
										mIsSpecified = true;

										mFile->CreateNewDataFile( kMacCVSCreator, kMacCVSDocFileType );
										mFile->OpenDataFork(fsRdWrPerm);
										
										SavePreferences();
										SetModified( false );
										
										Assert_(mWindow);										
										mWindow->SetDescriptor(mDocumentFileSpec.name);
												
									}
							}
						break;
						
						// JWW - Added Reveal In Finder case 04/21/99
						case cmd_RevealInFinder:				
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{
									LCommander::PostAnAction( new CProjectRevealAction(this, pItemList) );
									
									delete pItemList;
									pItemList = NULL;								
								}							
						break;
								
						
						case cmd_EditPreferences:
						
							EditPreferences();
						
							if ( mMessageWindow )
								mMessageWindow->SetScrollToNewMessages( mScrollMessages );
								
						break;

						case cmd_AddFile:					
							if ( ! CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{							
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectAddAction(this, pItemList) );
										}
										
									delete pItemList;
									pItemList = NULL;								
								}					
						break;
						
						case cmd_CheckoutDefaultModule:							
							if ( CheckCVSServerSettings() && GetPasswordFromUser() )
								{
									LCommander::PostAnAction( new CProjectCheckoutDefaultAction(this) );
								}							
						break;
						
						case cmd_CheckoutModuleTo:					
							if ( CheckCVSServerSettings() && GetPasswordFromUser() )
								{
									LCommander::PostAnAction( new CProjectCheckoutModuleToAction(this) );
								}						
						break;
						
						case cmd_Checkout:					
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectUpdateAction(this, pItemList, true) );
										}
										
									delete pItemList;
									pItemList = NULL;									
								}								
						break;

						case cmd_CheckoutTo:
							
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectUpdateToAction(this, pItemList, true) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}
						break;

						case cmd_Import:						
							if ( CheckCVSServerSettings() && GetPasswordFromUser() )
								{
									LCommander::PostAnAction( new CProjectImportAction(this) );
								}
						break;

						case cmd_RevertFile:						
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
											
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectRevertToAction(this, pItemList, false) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}						
						break;
						
						case cmd_Commit:				
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectCommitAction(this, pItemList) );
										}
										
									delete pItemList;
									pItemList = NULL;								
								}							
						break;

						case cmd_Edit:
						case cmd_UnEdit:
							
							if ( !CheckCVSServerSettings() )
								{
									break;
								}
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectEditAction(this, pItemList, (inCommand == cmd_Edit)) );				
										}
										
									delete pItemList;
									pItemList = NULL;
								}							
										
						break;
						
						case cmd_ShowEditors:		
						case cmd_ShowWatchers:				
							if ( !CheckCVSServerSettings() )
								{
									break;
								}
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectShowEditorsWatchersAction(this, pItemList, (inCommand == cmd_ShowEditors)) );				
										}
										
									delete pItemList;
									pItemList = NULL;
								}							
						break;
							
						case cmd_WatchOn:
						case cmd_WatchOff:
						case cmd_WatchAdd:
						case cmd_WatchRemove:
						
							if ( !CheckCVSServerSettings() )
								{
									break;
								}
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{
									UInt32 intOperation = 0;
									CCVSEngineWatchOptions sOptions;
									
									sOptions.Init();
									
									switch( inCommand )
										{
											case cmd_WatchOn:
												intOperation = 1;
											break;
											
											case cmd_WatchOff:
												intOperation = 2;
											break;
											
											case cmd_WatchAdd:
												intOperation = 3;
												sOptions.mNotification = true;
											break;
											
											case cmd_WatchRemove:
												intOperation = 3;
												sOptions.mNotification = false;
											break;
											
											default:
												Assert_( false );
											break;
										}
																				
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectWatchAction(this, pItemList, intOperation, &sOptions) );				
										}
										
									delete pItemList;
									pItemList = NULL;
								}							
						
						break;
							
						case cmd_CompareWithOriginal:					
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectCompareAction(this, pItemList) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}					
						break;
	
						case cmd_Diff:
						case cmd_AdvancedDiff:
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectDiffAction(this, pItemList, inCommand == cmd_AdvancedDiff) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}
						break;
																				
						case cmd_GetFileLog:						
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
								
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectGetFileInfoAction(this, pItemList) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}								
						break;
						
						case cmd_GetFileStatus:						
							if ( !CheckCVSServerSettings() )
								{
									break;
								}
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectGetFileStatusAction(this, pItemList) );				
										}
										
									delete pItemList;
									pItemList = NULL;
								}							
						break;

						case cmd_Tag:				
							if ( !CheckCVSServerSettings() )
								{
									break;
								}
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{							
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectTagAction(this, pItemList) );
										}
										
									delete pItemList;
									pItemList = NULL;									
								}							
						break;

						case cmd_RemoveFiles:						
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectRemoveAction(this, pItemList) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}						
						break;		
						
						case cmd_Update:				
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectUpdateAction(this, pItemList, false) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}
						break;
						
						case cmd_UpdateTo:
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetCollapsedSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
										
							if ( pItemList != NULL )
								{
									if ( GetPasswordFromUser() )
										{
											LCommander::PostAnAction( new CProjectUpdateToAction(this, pItemList, false) );
										}
										
									delete pItemList;
									pItemList = NULL;
								}
						break;

						case cmd_ModifyReadOnly:						
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{								
									LCommander::PostAnAction( new CProjectMROAction(this, pItemList) );
						
									delete pItemList;
									pItemList = NULL;								
								}					
						break;
						
						case cmd_EditComment:						
							if ( pProjectView != NULL )
								{
									pItemList = pProjectView->GetFlatSelection();
								}
							else if ( pLurkersView != NULL )
								{
									pItemList = pLurkersView->GetFlatSelection();
								}
									
							if ( pItemList != NULL )
								{							
									LCommander::PostAnAction( new CProjectEditCommentAction(this, pItemList) );
						
									delete pItemList;
									pItemList = NULL;								
								}						
						break;
						
						case cmd_FindLurkers:
					
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( IsProjectRootSpecified() )
								{	
									if ( pProjectView != NULL )
										{
											pItemList = pProjectView->GetCollapsedSelection();
										}
									else if ( pLurkersView != NULL )
										{
											pItemList = pLurkersView->GetFlatSelection();
										}
										
									Assert_( pItemList != NULL );
									
									if ( pItemList != NULL )
										{								
											FindLurkers( pItemList );
											
											delete pItemList;
											pItemList = NULL;										
										}										
								}
											
						break;

						case cmd_OrphanFiles:
					
							if ( !CheckCVSServerSettings() )
								break;
								
							if ( IsProjectRootSpecified() )
								{
									if ( pProjectView != NULL )
										{
											pItemList = pProjectView->GetCollapsedSelection();
										}
									else if ( pLurkersView != NULL )
										{
											pItemList = pLurkersView->GetFlatSelection();
										}
										
									Assert_( pItemList != NULL );
									
									if ( pItemList != NULL )
										{									
											OrphanFiles(pItemList);
								
											delete pItemList;
											pItemList = NULL;										
										}									
								}
											
						break;

						case cmd_MessageWindow:
							
							if ( mMessageWindow != NULL )
								{								
									if ( !mMessageWindow->IsVisible() )
										{
											mMessageWindow->ShowMessageWindow();
										}
									else 
										{
											UDesktop::SelectDeskWindow( mMessageWindow );
										}
								}
									
						break;
												
						default:					
							cmdHandled = LSingleDoc::ObeyCommand(inCommand, ioParam);					
						break;				
					}					
			}			
		catch ( ... )
			{			
				if ( pItemList != NULL )
					{				
						delete pItemList;
						pItemList = NULL;						
					}
					
				throw;			
			}
					
		return cmdHandled;
		
	}


void
CMacCVSDoc::FindCommandStatus( CommandT inCommand, Boolean &outEnabled, Boolean &outUsesMark, Char16 &outMark, Str255 outName )
	{

		LWindow							*topWindow 		= UDesktop::FetchTopRegular();
		CMacCVSProjectView	*projectView	= NULL;
		
		outEnabled = false;

		topWindow = UDesktop::FetchTopRegular();
		Assert_( topWindow != NULL );
								
		switch (inCommand) 
			{
			
			case cmd_SaveAs:

				// this is enabled by the root project view window
				outEnabled = false;
			
			break;
			
			case cmd_FindLurkers:
			case cmd_OrphanFiles:
			
				if ( IsProjectRootSpecified() )
					if ( IsOurWindow(topWindow) )
						if ( (GetWindowKind(topWindow) == kProjectViewWindowKind) || (GetWindowKind(topWindow) == kLurkersViewWindowKind) )		

							outEnabled = true;
					
			break;
				
			case cmd_EditPreferences:
			
				outEnabled = true;
				
			break;
			
			/*
			case cmd_Checkout:
			case cmd_Update:
			
				if ( IsProjectRootSpecified() )
					if ( ! GetDefaultCheckoutInProgress() )
						{
							
							// this sucks, but the project view is too dumb to know its owning doc, and therefore get checkoutInProgress
							projectView = GetProjectWindowView( mWindow );
							ThrowIfNil_( projectView );
							
							outEnabled = projectView->CheckIfAnyItemsQualify( inCommand );
						}
			break;
			
			case cmd_CheckoutTo:
			case cmd_UpdateTo:
			
				if ( IsProjectRootSpecified() )
					if ( ! GetDefaultCheckoutInProgress() )
						{
							
							// this sucks, but the project view is too dumb to know its owning doc, and therefore get checkoutInProgress
							projectView = GetProjectWindowView( mWindow );
							ThrowIfNil_( projectView );
							
							outEnabled = projectView->CheckIfAnyItemsQualify( inCommand );
						}
					
			break;
			*/
			
			case cmd_CheckoutDefaultModule:
			
				if ( IsProjectRootSpecified() )
					if ( mModuleName[0] > 0 )
						if ( ! GetDefaultCheckoutInProgress() )
							outEnabled = true;
						
			break;

			case cmd_CheckoutModuleTo:

				if ( IsProjectRootSpecified() )
						outEnabled = true;
								
			break;
			
			case cmd_Import:

				if ( IsProjectRootSpecified() )
					if ( ! GetDefaultCheckoutInProgress() )
						outEnabled = true;
			
			break;
			
			case cmd_PageSetup:
			
				outEnabled = false;
				
			break;
			
			case cmd_Print:
			
				outEnabled = false;
				
			break;
			
			case cmd_MessageWindow:
			
				if ( mMessageWindow != NULL )
					{
						
						if ( !mMessageWindow->IsVisible() )
							{
							
								if ( mMessageWindow->GetMessagesCount() > 0 )
									outEnabled = true;
									
							}
							
						else outEnabled = true;
						
					}
					
			break;
			
			default:
			
				LSingleDoc::FindCommandStatus(inCommand, outEnabled, outUsesMark, outMark, outName);
			
			break;
			
			}
		
	}


void
CMacCVSDoc::ListenToMessage(MessageT inMessage, void *ioParam)
	{

		switch ( inMessage )
			{
			
				case CMacCVSProjectView::kNewViewMessage:
					
					if ( ioParam != NULL )
						{
	
							CMacCVSProjectViewNewViewEventRec		inViewData;
							LWindow 														*theWindow 			= NULL;
							Rect																globalPortRect	= { 0, 0, 0, 0 };
							Boolean															isNewWindow			= false;
							
							inViewData = *(CMacCVSProjectViewNewViewEventRec *)ioParam;
							
							if ( FSpIsChildOf(&inViewData.fFileSpec, &mRootFileSpec) )
								{
									theWindow = NewViewWindow( &inViewData.fFileSpec, &mRootFileSpec, isNewWindow );
								
									if ( isNewWindow )
										{
											//  AE open events don't have cell coordinates
											//  because we are lame. So, don't zoom from 0,0,0,0.
											
											if ( 	inViewData.fFromCellRect.top != 0 && 
														inViewData.fFromCellRect.bottom != 0 &&
														inViewData.fFromCellRect.left != 0 && 
														inViewData.fFromCellRect.right != 0 )
												{
													theWindow->GetGlobalBounds( globalPortRect );											
													ZoomRects( &inViewData.fFromCellRect, &globalPortRect, CMacCVSApp::kNumberOfZoomSteps, kZoomAccelerate );												
												}
	
											theWindow->Show();
										}
								}
						}
						
				break;
				
				
				case CMacCVSProjectView::kNewViewSelectFileMessage:
					if ( ioParam != NULL )
						{
	
							CMacCVSProjectViewNewViewSelectFileEventRec		inViewData;			//ack!
							CMacCVSProjectViewWindow 											*theWindow 			= NULL;
							Rect																					globalPortRect	= { 0, 0, 0, 0 };
							Boolean																				isNewWindow			= false;
							
							inViewData = *(CMacCVSProjectViewNewViewSelectFileEventRec *)ioParam;
	
							if ( FSpIsEqual(&inViewData.fFolderSpec, &mRootFileSpec) )
								{
									this->BringToFront();
									
									if ( FSpIsValid(inViewData.fFileToSelectSpec) )
										{
											theWindow = dynamic_cast<CMacCVSProjectViewWindow *>( mWindow );
											ThrowIfNil_( theWindow );
											
											theWindow->SelectFile( &inViewData.fFileToSelectSpec );
										}
								}
							else if ( FSpIsChildOf(&inViewData.fFolderSpec, &mRootFileSpec) )
								{						
									theWindow = dynamic_cast<CMacCVSProjectViewWindow *>( NewViewWindow( &inViewData.fFolderSpec, &mRootFileSpec, isNewWindow ) );
								
									if ( isNewWindow )
										{
											theWindow->GetGlobalBounds( globalPortRect );											
											ZoomRects( &inViewData.fFromCellRect, &globalPortRect, CMacCVSApp::kNumberOfZoomSteps, kZoomAccelerate );											
											theWindow->Show();
										}
									
									theWindow->SelectFile( &inViewData.fFileToSelectSpec );
								}
						}
						
				break;
				
				
			}
			
	}

	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark APPLE EVENTS


void
CMacCVSDoc::GetSubModelByName( DescType inModelID, Str255 inName, AEDesc &outToken ) const
	{
	
		switch ( inModelID )
			{			
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					{
						FSSpec fsItem = { 0 };
						
						OSErr	err = ::FSpGetFromLocalCVSPath( inName, &mRootFileSpec, &fsItem );
						
						if ( err == kCVSEngineIsFolder )
							{
								err = noErr;
							}
							
						ThrowIfOSErr_( err );
						
						//  create an object for this file					
						CMacCVSAppleEventItem	*pProjectItem = new CMacCVSAppleEventItem( const_cast<CMacCVSDoc *>(this), fsItem );
						ThrowIfNil_( pProjectItem );
						
						pProjectItem->SetLaziness( true );
						PutInToken( pProjectItem, outToken );
					}
				break;
					
				default:
					Inherited::GetSubModelByName( inModelID, inName, outToken );
				break;			
			}
		
	}


void
CMacCVSDoc::GetSubModelByPosition( DescType inModelID, SInt32 inPosition, AEDesc &outToken ) const
	{
		
		switch ( inModelID )
			{		
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:	
					{				
						// get the nth file in the root directory
						FSSpec fsItem = { 0 };
						
						OSErr	err = FSpGetNthItem( &mRootFileSpec, inPosition, &fsItem );
						ThrowIfOSErr_( err );
						
						// create an object for this file					
						CMacCVSAppleEventItem *pProjectItem = new CMacCVSAppleEventItem( const_cast<CMacCVSDoc *>(this), fsItem );
											
						pProjectItem->SetLaziness( true );						
						PutInToken( pProjectItem, outToken );
					}
				break;					
					
				default:					
					Inherited::GetSubModelByPosition( inModelID, inPosition, outToken );
				break;
			}
			
	}
	

SInt32
CMacCVSDoc::CountSubModels( DescType inModelID ) const
	{
		
		SInt32		intSubModelCount 	= 0;
		UInt32 		intNumItems 			= 0;
	
		switch (inModelID)
			{			
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					
					// get the number of files in the root directory
						
					OSErr	err = FSpCountNumFiles( &mRootFileSpec, &intNumItems );
					ThrowIfOSErr_( err );
					
					intSubModelCount = intNumItems;
					
				break;
										
				default:				
					intSubModelCount = Inherited::CountSubModels( inModelID );
				break;
			}
	
		return intSubModelCount;
		
	}


void
CMacCVSDoc::GetSubModelByComplexKey( DescType inModelID, DescType inKeyForm, const AEDesc& inKeyData,
																		 AEDesc& outToken) const
	{

		// support 'whose' clauses

		switch ( inModelID )
			{		
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					
					switch ( inKeyForm )
						{					
							case formName:
							break;
								
							case CMacCVSAppleEventItem::pPathProperty:
							break;
								
							case CMacCVSAppleEventItem::pStatusProperty:
							break;				
						}												
				break;
										
				default:					
					ThrowOSErr_( errAENoSuchObject );

					//  this is bad; it calls Debugger() in optimized builds!
					//  Inherited::GetSubModelByComplexKey(inModelID, inKeyForm, inKeyData, outToken);
				break;
			}
			
	}


void
CMacCVSDoc::GetAEProperty( DescType inProperty, const AEDesc &inRequestedType, AEDesc &outPropertyDesc ) const
	{
		
		OSErr	err = noErr;

		switch ( inProperty ) 
			{		
				case pName:					// Document title				
					err = ::AECreateDesc(typeChar, (Ptr) mDocumentFileSpec.name + 1, 
										StrLength(mDocumentFileSpec.name), &outPropertyDesc);					
				break;
			
				case pModuleName:				
					err = ::AECreateDesc(typeChar, (Ptr) mModuleName + 1,
										StrLength(mModuleName), &outPropertyDesc);				
				break;

				case pDefaultTag:				
					err = ::AECreateDesc(typeChar, (Ptr) mDefaultTag + 1,
										StrLength(mDefaultTag), &outPropertyDesc);				
				break;

				case pHostName:				
					err = ::AECreateDesc(typeChar, (Ptr) mHostName + 1,
										StrLength(mHostName), &outPropertyDesc);			
				break;

				case pRemoteUser:				
					err = ::AECreateDesc(typeChar, (Ptr) mRemoteUser + 1,
										StrLength(mRemoteUser), &outPropertyDesc);				
				break;

				case pCVSRoot:			
					err = ::AECreateDesc(typeChar, (Ptr) mCVSRoot + 1,
										StrLength(mCVSRoot), &outPropertyDesc);			
				break;

				case pPassword:			
					//  err = ::AECreateDesc(typeChar, (Ptr) mPassword + 1,
					//					StrLength(mPassword), &outPropertyDesc);		
				
					//  No way hose are we gonna give
					//  out our password.
					
					ThrowOSErr_( errAEUnknownObjectType );
				break;

				case pLocalRootFile:

					//  If the project root isn't specified, the 
					//  resulting AEDesc will contain a NULL FSSpec.
				
					err = ::AECreateDesc(typeFSS, &mRootFileSpec,
									sizeof(mRootFileSpec) , &outPropertyDesc);					
				break;

				default:		
					LModelObject::GetAEProperty(inProperty, inRequestedType, outPropertyDesc);			
				break;		
			}

		ThrowIfOSErr_( err );
		
	}


void
CMacCVSDoc::SetAEProperty( DescType inProperty, const AEDesc &inValue, AEDesc &outAEReply )
	{
		
		Str255 pstrTemp = "\p";
		
		switch ( inProperty ) 
			{			
				case pModuleName:				
					UExtractFromAEDesc::ThePString( inValue, mModuleName );
					SavePreferences();					
				break;

				case pDefaultTag:		
					
					UExtractFromAEDesc::ThePString( inValue, pstrTemp );
					
					if ( pstrTemp[0] > 0 )
						{
							if ( pstrTemp[1] != 'T' && pstrTemp[1] != 'D' && pstrTemp[1] != 't' && pstrTemp[1] != 'd' )
								{
									ThrowOSErr_( paramErr );
								}
								
							if ( pstrTemp[1] == 't' )
								{
									pstrTemp[1] = 'T';
								}
							else if ( pstrTemp[1] == 'd' )
								{
									pstrTemp[1] = 'D';
								}
								
							PStrCopy( pstrTemp, mDefaultTag, sizeof(mDefaultTag) );
						}
					else
						{
							mDefaultTag[0] = 0;
						}
							
					SavePreferences();					
				
				break;

				case pHostName:				
					UExtractFromAEDesc::ThePString( inValue, mHostName );
					SavePreferences();					
				break;

				case pRemoteUser:				
					UExtractFromAEDesc::ThePString( inValue, mRemoteUser );
					SavePreferences();				
				break;

				case pCVSRoot:				
					UExtractFromAEDesc::ThePString( inValue, mCVSRoot );
					SavePreferences();					
				break;

				case pPassword:				
					UExtractFromAEDesc::ThePString( inValue, mPassword );
					SavePreferences();			
				break;
					
				case pLocalRootFile:
					if ( inValue.descriptorType == typeFSS )
						{						
							ThrowIfNil_( inValue.dataHandle );
							
							mRootFileSpec = *(FSSpec *)(*inValue.dataHandle);
						
							PostProcessPathChange();
							SavePreferences();					
						}					
					else if ( inValue.descriptorType == typeAlias )
						{					
							OSErr			error 				= noErr;
							Boolean		bChangedFlag	= noErr;
							
							ThrowIfNil_( inValue.dataHandle );
						
							error = ::ResolveAlias( NULL, (AliasHandle)inValue.dataHandle, &mRootFileSpec, &bChangedFlag );
							ThrowIfOSErr_( error );

							PostProcessPathChange();
							SavePreferences();						
						}					
					else 
						{
							LSingleDoc::SetAEProperty( inProperty, inValue, outAEReply );
						}
				break;
				
				default:			
					LSingleDoc::SetAEProperty( inProperty, inValue, outAEReply );					
				break;					
			}
			
	}


void CMacCVSDoc::HandleAppleEvent( const AppleEvent	&inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{	
		try
			{
				switch ( inAENumber )
					{					
						case ae_Checkout:				
							HandleCheckoutEvent( inAppleEvent, outAEReply, outResult, inAENumber );					
						break;

						case ae_Update:				
							HandleUpdateEvent( inAppleEvent, outAEReply, outResult, inAENumber );				
						break;

						default:			
							LSingleDoc::HandleAppleEvent( inAppleEvent, outAEReply, outResult, inAENumber );							
						break;						
					}
			}
		catch ( OSErr intException )
			{
				ThrowOSErr_( intException );
			}
		catch ( ExceptionCode intException )
			{
				throw intException;
			}
		catch ( ... )
			{
				//  I'm guessing that we better translate all
				//  exceptions into PowerPlant types so the
				//  base class's HandleAppleEvent() does the
				//  right thing with the exception.
				
				ThrowOSErr_( errAEEventNotHandled );
			}
	}


void CMacCVSDoc::HandleCheckoutEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{

		CCVSEngineCheckoutOptions sCheckoutOptions;
		Str255 pstrRemoteRoot = "\p";

//  Don't do the event if the local or remote roots
//  aren't specified.
		
		sCheckoutOptions.Init();
		
		if ( IsProjectRootSpecified() == false )
			{
				ThrowOSErr_( errAEEventNotHandled );
			}
			
		GetCVSRootPStr( pstrRemoteRoot );
		
		if ( pstrRemoteRoot[0] == 0 )
			{
				ThrowOSErr_( errAEEventNotHandled );
			}
			
//  Get revision argument.
	
		try
			{		
				StAEDescriptor theRevisionDesc;
				
				theRevisionDesc.GetParamDesc( inAppleEvent, ae_UpdateRevision, typeChar );
				UExtractFromAEDesc::ThePString( theRevisionDesc.mDesc, sCheckoutOptions.mRevisionPStr );
			}			
		catch ( ... )
			{			
			}

//  Get date argument.
			
		try
			{			
				StAEDescriptor theDateDesc;

				theDateDesc.GetParamDesc( inAppleEvent, ae_UpdateDate, typeChar );
				UExtractFromAEDesc::ThePString( theDateDesc.mDesc, sCheckoutOptions.mDatePStr );
			}			
		catch ( ... )
			{			
			}

//  Get module argument.
			
		try
			{			
				StAEDescriptor theModuleDesc;

				theModuleDesc.GetParamDesc( inAppleEvent, ae_UpdateModuleName, typeChar );
				UExtractFromAEDesc::ThePString( theModuleDesc.mDesc, sCheckoutOptions.mModulePStr );
			}		
		catch ( ... )
			{		
			}

		sCheckoutOptions.mMergePolicy = GetMergePolicy();
		sCheckoutOptions.mLFTranslation = GetTranslation();

		//  If the user didn't specify a module, use the default one.

		if ( sCheckoutOptions.mModulePStr[0] == 0 )
			{
				GetModulePStr( sCheckoutOptions.mModulePStr );
			}
			
		CProjectCheckoutDefaultAction	*pAction = new CProjectCheckoutDefaultAction( this, &sCheckoutOptions );
		ThrowIfNil_( pAction );
		
		pAction->SetCompletionEvent( inAppleEvent, outAEReply );
		LCommander::PostAnAction( pAction );
		
	}


void CMacCVSDoc::HandleUpdateEvent( const AppleEvent &inAppleEvent, AppleEvent &outAEReply, AEDesc &outResult, long inAENumber )
	{
		
		Str255 	pstrRemoteRoot	= "\p";
		FSSpec 	fsLocalRoot			= { 0, 0, "\p" };

		CCVSEngineUpdateOptions	sUpdateOptions;
		sUpdateOptions.Init();
		
		LArray itemList( sizeof(FSSpec) );

		//  Don't do the event if the local or remote roots
		//  aren't specified.
		
		if ( IsProjectRootSpecified() == false )
			{
				ThrowOSErr_( errAEEventNotHandled );
			}
			
		GetCVSRootPStr( pstrRemoteRoot );
		
		if ( pstrRemoteRoot[0] == 0 )
			{
				ThrowOSErr_( errAEEventNotHandled );
			}
			
		//  Get revision argument.
	
		try
			{			
				StAEDescriptor theRevisionDesc;
				
				theRevisionDesc.GetParamDesc( inAppleEvent, ae_UpdateRevision, typeChar );
				UExtractFromAEDesc::ThePString( theRevisionDesc.mDesc, sUpdateOptions.mRevisionPStr );
			}			
		catch ( ... )
			{			
			}

		//  Get date argument.
			
		try
			{			
				StAEDescriptor theDateDesc;

				theDateDesc.GetParamDesc( inAppleEvent, ae_UpdateDate, typeChar );
				UExtractFromAEDesc::ThePString( theDateDesc.mDesc, sUpdateOptions.mDatePStr );
			}			
		catch ( ... )
			{
			}
			
		sUpdateOptions.mMergePolicy = GetMergePolicy();
		sUpdateOptions.mLFTranslation = GetTranslation();

		GetProjectRootFileSpec( fsLocalRoot );
		
		itemList.InsertItemsAt( 1, LArray::index_First, &fsLocalRoot );
		
		CProjectUpdateAction	*pAction = new CProjectUpdateAction( this, &itemList, false, &sUpdateOptions );
		ThrowIfNil_( pAction );
		
		pAction->SetCompletionEvent( inAppleEvent, outAEReply );
		LCommander::PostAnAction( pAction );

	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


//  Ahhhh! Ahhhh! So many accessors!

void
CMacCVSDoc::GetLoginOptions( CCVSEngineLoginOptions &sOutLoginOptions )
	{
		GetHostNamePStr( sOutLoginOptions.mServerName );
		GetRemoteUserPStr( sOutLoginOptions.mRemoteUserName );
		GetRemoteUserPStr( sOutLoginOptions.mLocalUserName );
		GetPasswordPStr( sOutLoginOptions.mPassword );
		GetCVSCommandPStr( sOutLoginOptions.mCVSCommandText );
		GetCVSRootPStr( sOutLoginOptions.mCVSRoot );
		sOutLoginOptions.mAuthenticationMethod = GetAuthenticationMethod();	
		sOutLoginOptions.mPort = mPort;
	}
	

void
CMacCVSDoc::GetHostNamePStr(Str255 outHostNamePStr)
	{
		
		if ( outHostNamePStr != NULL )
			::BlockMoveData( mHostName, outHostNamePStr, mHostName[0] + 1 );
			
	}


void
CMacCVSDoc::GetRemoteUserPStr(Str255 outRemoteUserPStr)
	{

		if ( outRemoteUserPStr != NULL )
			::BlockMoveData( mRemoteUser, outRemoteUserPStr, mRemoteUser[0] + 1 );

	}


void
CMacCVSDoc::GetPasswordPStr(Str255 outPasswordPStr)
	{

		if ( outPasswordPStr != NULL )
			::BlockMoveData( mPassword, outPasswordPStr, mPassword[0] + 1 );

	}


void
CMacCVSDoc::GetCVSCommandPStr(Str255 outCVSCommandPStr)
	{

		if ( outCVSCommandPStr != NULL )
			::BlockMoveData( mCVSCommand, outCVSCommandPStr, mCVSCommand[0] + 1 );

	}


void
CMacCVSDoc::GetCVSRootPStr(Str255 outRootPStr)
	{

		if ( outRootPStr != NULL )
			::BlockMoveData( mCVSRoot, outRootPStr, mCVSRoot[0] + 1 );

	}


void
CMacCVSDoc::GetModulePStr(Str255 outModulePStr)
	{

		if ( outModulePStr != NULL )
			::BlockMoveData( mModuleName, outModulePStr, mModuleName[0] + 1 );

	}


void
CMacCVSDoc::GetDefaultTagPStr( Str255 outTagPStr )
	{

		if ( outTagPStr != NULL )
			{
			
				if ( mDefaultTag[0] > 0 )

					::BlockMoveData( mDefaultTag, outTagPStr, mDefaultTag[0] + 1 );
	
				else outTagPStr[0] = 0x00;
				
			}
			
	}

	
void
CMacCVSDoc::SetDefaultCheckoutInProgress(Boolean inInProgress)
	{
		mCheckingOutDefaultModule = inInProgress;
	}
	

void
CMacCVSDoc::GetDefaultsForDiffOrCompare( CCVSEngineDiffOptions &outOptions, Boolean isCompare)
	{
	
		outOptions.Init();
		
		if ( isCompare )
			{
			
				outOptions.mIgnoreCase 				= mCompareOptions.mIgnoreCase;
				outOptions.mIgnoreWhiteSpace 	= mCompareOptions.mIgnoreWhiteSpace;
				outOptions.mIgnoreSpaceChange	= mCompareOptions.mIgnoreSpaceChange;
				outOptions.mIgnoreBlankLines 	= mCompareOptions.mIgnoreBlankLines;
				
				// disallow option that would mess up our compare code
				outOptions.mShowCFunction 		= false;

				outOptions.mRCSDiff		 				= false;
				outOptions.mContextDiff		 		= false;
				outOptions.mUnifiedDiff		 		= false;
				
				outOptions.mContextLines		 	= 0;
			
			}
		else
			{

				outOptions.mIgnoreCase 				= mDiffOptions.mIgnoreCase;
				outOptions.mIgnoreWhiteSpace 	= mDiffOptions.mIgnoreWhiteSpace;
				outOptions.mIgnoreSpaceChange	= mDiffOptions.mIgnoreSpaceChange;
				outOptions.mIgnoreBlankLines 	= mDiffOptions.mIgnoreBlankLines;
				
				outOptions.mShowCFunction 		= mDiffOptions.mShowCFunction;

				outOptions.mRCSDiff		 				= mDiffOptions.mRCSDiff;
				outOptions.mContextDiff		 		= mDiffOptions.mContextDiff;
				outOptions.mUnifiedDiff		 		= mDiffOptions.mUnifiedDiff;
				
				outOptions.mContextLines		 	= mDiffOptions.mContextLines;
				
			}
	}

void
CMacCVSDoc::SetDefaultsForDiffOrCompare( const CCVSEngineDiffOptions &inOptions, Boolean isCompare)
	{
	
		if ( isCompare )
			{
			
				mCompareOptions.mIgnoreCase					= inOptions.mIgnoreCase;
				mCompareOptions.mIgnoreWhiteSpace		= inOptions.mIgnoreWhiteSpace;
				mCompareOptions.mIgnoreSpaceChange	= inOptions.mIgnoreSpaceChange;
				mCompareOptions.mIgnoreBlankLines		= inOptions.mIgnoreBlankLines;
		
				// Prevent illegal options for compare being saved
				mCompareOptions.mShowCFunction			= false;

				mCompareOptions.mRCSDiff						= false;
				mCompareOptions.mContextDiff				= false;
				mCompareOptions.mUnifiedDiff				= false;
			
				mCompareOptions.mContextLines			 	= 0;
			
			}
		else
			{
			
				mDiffOptions.mIgnoreCase					= inOptions.mIgnoreCase;
				mDiffOptions.mIgnoreWhiteSpace		= inOptions.mIgnoreWhiteSpace;
				mDiffOptions.mIgnoreSpaceChange		= inOptions.mIgnoreSpaceChange;
				mDiffOptions.mIgnoreBlankLines		= inOptions.mIgnoreBlankLines;
	
				mDiffOptions.mShowCFunction				= inOptions.mShowCFunction;

				mDiffOptions.mRCSDiff							= inOptions.mRCSDiff;
				mDiffOptions.mContextDiff					= inOptions.mContextDiff;
				mDiffOptions.mUnifiedDiff					= inOptions.mUnifiedDiff;
			
				mDiffOptions.mContextLines			 	= inOptions.mContextLines;
			
			}
	
		SetModified( true );
	}

	
void
CMacCVSDoc::PostMessage( const UInt32 inMessageType, const char *inMessageText, const UInt32 inMessageLen, const void *inMessageData, const UInt32 inDataSize )
	{
	
		const UInt32 	kMaxMessageLen		= 450;
		
		char					strMessage[512] 	= "";
		UInt32				intMessageLen			= 0;
		UInt32				intMessageType 		= inMessageType;		
		Boolean				bOkToPost 				= true;
		
		Assert_( (inMessageText != NULL) && (inMessageLen > 0) );

		intMessageLen = inMessageLen;
			
		if ( intMessageLen >= kMaxMessageLen )
			{
				//  Limit to 450 characters because we 
				//  may append text to the message later.
				intMessageLen = kMaxMessageLen;
			}
			
		::BlockMoveData( inMessageText, strMessage, intMessageLen );
		
		//  If the user has turned of "Display Every File Update", 
		//  then don't post any "U Update Message, etc." messages.
		
		if ( intMessageType == CMacCVSMessageView::kMessageTypeFile )
			{
				TEngineUpdateResponseType	respType = IdentifyResponseType( strMessage, intMessageLen );
			
				if ( !mOutputFileUpdates )
					{							
						switch ( respType )
							{
								case kCVSEngineFileUpdatedResponse:			//  File has been updated
								case kCVSEngineFileAddedResponse:				//  File has been added (import)
								case kCVSEngineFileTaggedResponse:			//  File has been tagged
								case kCVSEngineFileUntaggedResponse:		//  File has been untagged
									bOkToPost = false;
								break;
									
								default:																//  We DO want to post conflict errors	
								break;																	//  and other important messages.
							}
					}

				if ( bOkToPost )
					{
						if ( 	respType == kCVSEngineFileMergedResponse || 
									respType == kCVSEngineFileConflictsResponse ||
									respType == kCVSEngineFileLocallyAddedResponse )
							{							
								if ( strMessage[intMessageLen - 1] == '\r')
									{
										intMessageLen--;
									}
									
								strMessage[intMessageLen] = '\0';
								
								switch ( respType )
									{
										case kCVSEngineFileMergedResponse:
											strcat(strMessage, "\r     Locally modified");
											intMessageLen = strlen( strMessage );
										break;
											
										case kCVSEngineFileConflictsResponse:
											strcat(strMessage, "\r      Unresolved conflicts!");
											intMessageLen = strlen( strMessage );
										break;
										
										case kCVSEngineFileLocallyAddedResponse:
											strcat(strMessage, "\r    Locally added");
											intMessageLen = strlen( strMessage );
										break;
									}
							}								
					}
					
				if ( respType == kCVSEngineFileConflictsResponse )
					{
						//  Feature request kbrade@netscape.com.
						
						intMessageType = CMacCVSMessageView::kMessageTypeError;
					}
			}
			
		//  Change "Updating", "Importing", and "Tagging" messages to 
		//  kMessageTypeMessage instead of kMessageTypeWarning. For some 
		//  reason, these messages are sent by CVS in the error out stream 
		//  instead of the message out stream.
	
		else if ( intMessageType == CMacCVSMessageView::kMessageTypeWarning )
			{		
				if ( inMessageLen >= 8 )
					{					
						if ( !strncmp(strMessage, "Updating ", 9) )
							{
								intMessageType = CMacCVSMessageView::kMessageTypeMessage;
								bOkToPost = mOutputDirectoryUpdates;
							}							
						else if ( !strncmp(strMessage, "Importing ", 10) )
							{
								intMessageType = CMacCVSMessageView::kMessageTypeMessage;
								bOkToPost = mOutputDirectoryUpdates;
							}							
						else if ( !strncmp(strMessage, "Tagging ", 8) )
							{
								intMessageType = CMacCVSMessageView::kMessageTypeMessage;
								bOkToPost = mOutputDirectoryUpdates;
							}						
						else if ( !strncmp(strMessage, "Untagging ", 10) )
							{
								intMessageType = CMacCVSMessageView::kMessageTypeMessage;
								bOkToPost = mOutputDirectoryUpdates;
							}							
					}			
			}
					
		if ( bOkToPost )
			{		
				if ( mMessageWindow == NULL )
					{					
						LStr255 mwTitle;
						
						GetDescriptor( mwTitle );
						mwTitle.Append("\p Messages");
						
						mMessageWindow = CMacCVSMessageWindow::CreateCMacCVSMessageWindow( this, mwTitle, true, mScrollMessages );
						mMessageWindow->SetOwnerDoc(this);
						mMessageWindow->AddListener(this);
					}
					
				mMessageWindow->PostMessage( intMessageType, strMessage, intMessageLen, inMessageData, inDataSize );	
			}
					
	}
		
	
#pragma mark -
#pragma mark FIND LURKERS/ORPHAN FILES


void
CMacCVSDoc::FindLurkers( LArray *inItemList )
	{

		Boolean						done 							= false;
		LDialogBox				*lurkersDialog		= NULL;
		
		LCheckBoxGroupBox	*useTagGroup			= NULL;
		
		LRadioGroupView		*searchRadioGroup	= NULL,
											*findRadioGroup		= NULL,
											*tagRadioGroup		= NULL;
				
		LCheckBox					*findNewCheckbox	= NULL;
		CTagPopup					*tagPopup					= NULL;
		LPushButton				*removeButton			= NULL;
		
		UInt32						lastMenuItem			= 0,
											ioCount						= sizeof( FSSpec );
											
		Str255						theNewTag					= "\p";
		
		LArray						*newTagList				= NULL;
		
		Boolean						tagsModified			= false;
		MessageT					message						= 0;
		
		FSSpec						theCurItem				= { 0 };
		
		static CMacCVSLurkersQueryRec			lastQuery = { "\p", kLurkersMatchIgnoreTag, true, true };
		
		ThrowIfNil_( inItemList );
		
		try
			{

//  Remove all non-CVS directories from the selection.
			
				LArrayIterator	itemIterator( *inItemList );
				
				while ( itemIterator.Next(&theCurItem, ioCount) )
					{
					
						Assert_( ioCount == sizeof(FSSpec) );
						ioCount = sizeof( FSSpec );

						Boolean	keepItem = false;
												
						if ( FSpIsAFile(&theCurItem, NULL) == kCVSEngineIsFolder )
							if ( FSpIsRepositoryDir(&theCurItem) )
								keepItem = true;
								
						if ( !keepItem )
							inItemList->RemoveItemsAt( 1, itemIterator.GetCurrentIndex() );
					
					}
					
				StDialogHandler		lurkersDialogHandler( kLurkersDialogPPobID, this );

				lurkersDialogHandler.AddAttachment( new CThreadedTaskYieldAttachment );
				
				lurkersDialog = dynamic_cast<LDialogBox *>( lurkersDialogHandler.GetDialog() );															
				ThrowIfNil_( lurkersDialog );

// Get the radio groups
				searchRadioGroup = dynamic_cast<LRadioGroupView *>( lurkersDialog->FindPaneByID(kSearchRadioGroupID) );
				ThrowIfNil_( searchRadioGroup );

				findRadioGroup = dynamic_cast<LRadioGroupView *>( lurkersDialog->FindPaneByID(kFindRadioGroupID) );
				ThrowIfNil_( findRadioGroup );

				tagRadioGroup = dynamic_cast<LRadioGroupView *>( lurkersDialog->FindPaneByID(kTagRadioGroupID) );
				ThrowIfNil_( tagRadioGroup );

				useTagGroup = dynamic_cast<LCheckBoxGroupBox *>( lurkersDialog->FindPaneByID(kUseTagGroupID) );
				ThrowIfNil_( useTagGroup );
				
// Set the search radio group
				if ( inItemList->GetCount() == 0 )
					{
						searchRadioGroup->SetCurrentRadioID( kSearchInTreeRadioButtonID );
						
						LRadioButton		*searchSelectionRadio = dynamic_cast<LRadioButton *>( lurkersDialog->FindPaneByID(kSearchInSelectionRadioButtonID) );
						ThrowIfNil_( searchSelectionRadio );
						searchSelectionRadio->Disable();
						
					}
				else
					{
						searchRadioGroup->SetCurrentRadioID( kSearchInSelectionRadioButtonID );
					}

// Init the modified group
				if ( lastQuery.fFindOnlyMRO )
					findRadioGroup->SetCurrentRadioID(kFindModifiedRadioButtonID);
				else
					findRadioGroup->SetCurrentRadioID(kFindUnmodifiedRadioButtonID);

				findNewCheckbox = dynamic_cast<LCheckBox *>( lurkersDialog->FindPaneByID(kFindNewCheckboxID) );
				ThrowIfNil_( findNewCheckbox );
				findNewCheckbox->SetValue( lastQuery.fFindNewFiles );

// Get the tag popup
				tagPopup = dynamic_cast<CTagPopup *>( lurkersDialog->FindPaneByID(kTagPopupID) );
				ThrowIfNil_( tagPopup );
				
				removeButton = dynamic_cast<LPushButton *>( lurkersDialog->FindPaneByID(kLurkersDialogRemoveButtonID) );
				ThrowIfNil_( removeButton );
				
				tagPopup->SetTagList( *mTagList );
				
				if ( lastQuery.fTagPStr[0] == 0 )
				{
					Str255			defaultTagStr = "\p";
					
					GetDefaultTagPStr(defaultTagStr);
					tagPopup->SetDefaultTag(defaultTagStr);
					
				}
				else
				{
					tagPopup->SetDefaultTag(lastQuery.fTagPStr);
				}

// Init the tag radios
				useTagGroup->SetValue( (Int16)(lastQuery.fMatchTag != kLurkersMatchIgnoreTag) );

				switch ( lastQuery.fMatchTag )
				{
					case kLurkersMatchMatchTag:
						tagRadioGroup->SetCurrentRadioID(kOnTagRadioButtonID);
						break;
						
					case kLurkersMatchDontMatchTag:
					default:
						tagRadioGroup->SetCurrentRadioID(kOffTagRadioButtonID);
						break;
				}
		
				lurkersDialog->SetDefaultButton( kLurkersDialogOKButtonID );
				lurkersDialog->SetCancelButton( kLurkersDialogCancelButtonID );
				lurkersDialog->Show();
				
				while ( !done )
					{
				
						message = lurkersDialogHandler.DoDialog();
						
						switch( message )
							{
							
								case msg_Nothing:
										// this sucks, but the LCheckboxGroupBox does not broadcast when it gets clicked
										if ( useTagGroup->GetValue() == Button_On)
										{
										
											if ( tagPopup->CanRemoveCurrentTag() )
												removeButton->Enable();
											else
												removeButton->Disable();

										}
									break;
									
								case msg_OK:
								case msg_Cancel:
									{
									
										CMacCVSLurkersQueryRec	theQuery 				= { "\p", kLurkersMatchIgnoreTag, false };
											
										done = true;
									
										newTagList = tagPopup->GetTagList();
										
										Assert_( newTagList != NULL );
										
										if ( newTagList != NULL )
											{
											
												tagsModified = !AreTagListsEqual( *mTagList, *newTagList );
												
												if ( tagsModified )	
													{
														
														SetModified( true );
														
														UseNewTagList( *newTagList );
													
													}
													
												delete newTagList;
												newTagList = NULL;
													
											}
									
// Save the query even if the user cancelled the dialog. They'll probably come back

							//  Setup the lurkers query rec.
									
									tagPopup->GetCurrentTag( theQuery.fTagPStr );

//  If the user chose to search the whole tree, delete
//  all the selections and add the project root to the
//  item list.

									if ( searchRadioGroup->GetCurrentRadioID() == kSearchInTreeRadioButtonID )
										{
										
											inItemList->RemoveItemsAt( inItemList->GetCount(), LArray::index_First );

											GetProjectRootFileSpec( theCurItem );

											ioCount = sizeof( FSSpec );
											inItemList->InsertItemsAt( 1, LArray::index_First, &theCurItem, ioCount );
											
										}
										
									theQuery.fFindOnlyMRO = ( findRadioGroup->GetCurrentRadioID() == kFindModifiedRadioButtonID );
									theQuery.fFindNewFiles = findNewCheckbox->GetValue() != 0;
									
									if (useTagGroup->GetValue())
										{
											switch ( tagRadioGroup->GetCurrentRadioID() )
											{
												case kOnTagRadioButtonID:
													theQuery.fMatchTag = kLurkersMatchMatchTag;
													break;
											
												case kOffTagRadioButtonID:
													theQuery.fMatchTag = kLurkersMatchDontMatchTag;
													break;
													
											}
										}
									else
										{
												theQuery.fMatchTag = kLurkersMatchIgnoreTag;
										}
																		
									lastQuery = theQuery;
									
									if ( (message == msg_OK) && (IsProjectRootSpecified()) )
										{

											CMacCVSLurkersWindow		*lurkersWindow 	= NULL;
										
											FSSpec									targetSpec			= {  0, 0, "\p" };

//  Load and show the window.
												
											lurkersWindow = dynamic_cast<CMacCVSLurkersWindow *>( LWindow::CreateWindow(kMacCVSLurkersWindowID, this) );
											ThrowIfNil_( lurkersWindow );
											
											lurkersWindow->SetOwnerDoc(this);
											lurkersWindow->SetWindowPosition();
											lurkersWindow->Show();
											AddToWindowList( lurkersWindow );

//  Find the lurkers.
											
											lurkersWindow->FindLurkers( this, inItemList, &theQuery );
																			
										}
										
									}			
									break;
								
								case kUseTagCheckboxGroupMessageValue:
									
									if ( useTagGroup->GetValue() == Button_On)
										{
										
											if ( tagPopup->CanRemoveCurrentTag() )
												removeButton->Enable();
											else
												removeButton->Disable();

										}
									break;
								
								case kLurkersDialogRemoveButtonMessageValue:

									tagPopup->RemoveCurrentTag();

									if ( tagPopup->CanRemoveCurrentTag() )
									
										removeButton->Enable();
									
									else removeButton->Disable();
																	
									break;
								
								case kTagPopupMessageValue:
								
									tagPopup->DoMouseDownInTagPopup();
									
									if ( tagPopup->CanRemoveCurrentTag() )
									
										removeButton->Enable();
									
									else removeButton->Disable();
										
									break;
																	
							}
				
											
					}
					
			}
			
		catch ( ... )
			{
			
				if ( newTagList != NULL )
					{
					
						delete newTagList;
						newTagList = NULL;
						
					}
			
			
			}
	
	}
	

void
CMacCVSDoc::OrphanFiles( LArray *inItemList )
	{

		Boolean						done 							= false;
		LDialogBox				*wipeInfoDialog		= NULL;
		
		LCheckBox					*removeCkidCheckbox					= NULL,
											*removeMcvsCheckbox					= NULL,
											*removeDirectoriesCheckbox	= NULL;
		
		LRadioButton			*orphanTreeRadio						= NULL,
											*orphanSelectedFilesRadio		= NULL;
						
		UInt32						ioCount						= sizeof( FSSpec );
											
		MessageT					message						= 0;
		
		FSSpec						theCurItem				= { 0 };
				
		ThrowIfNil_( inItemList );
		
		try
			{					
				StDialogHandler		wipeInfoDialogHandler( kOrphanFilesDialogPPobID, this );

				wipeInfoDialogHandler.AddAttachment( new CThreadedTaskYieldAttachment );
				
				wipeInfoDialog = dynamic_cast<LDialogBox *>( wipeInfoDialogHandler.GetDialog() );															
				ThrowIfNil_( wipeInfoDialog );

// Get the radio buttons
				orphanTreeRadio = dynamic_cast<LRadioButton *>( wipeInfoDialog->FindPaneByID(kOrphanEntireTreeRadioID) );
				ThrowIfNil_( orphanTreeRadio );
				
				orphanSelectedFilesRadio = dynamic_cast<LRadioButton *>( wipeInfoDialog->FindPaneByID(kOrphanSelectedFilesRadioID) );
				ThrowIfNil_( orphanSelectedFilesRadio );

				if ( inItemList->GetCount() == 0 )
					{
						orphanTreeRadio->Enable();
						orphanTreeRadio->SetValue(Button_On);
						
						orphanSelectedFilesRadio->Disable();
						orphanSelectedFilesRadio->SetValue(Button_Off);
					}
				else
					{
						orphanTreeRadio->Enable();
						orphanSelectedFilesRadio->Enable();
						
						orphanTreeRadio->SetValue(Button_Off);
						orphanSelectedFilesRadio->SetValue(Button_On);
					}
					
// Get the check boxes
				removeCkidCheckbox = dynamic_cast<LCheckBox *>( wipeInfoDialog->FindPaneByID(kRemoveCkidResourcesCheckboxID) );
				ThrowIfNil_( removeCkidCheckbox );

				removeMcvsCheckbox = dynamic_cast<LCheckBox *>( wipeInfoDialog->FindPaneByID(kRemoveMcvsResourcesCheckboxID) );
				ThrowIfNil_( removeMcvsCheckbox );

				removeDirectoriesCheckbox = dynamic_cast<LCheckBox *>( wipeInfoDialog->FindPaneByID(kRemoveCVSDirectoriesCheckboxID) );
				ThrowIfNil_( removeDirectoriesCheckbox );

				wipeInfoDialog->SetDefaultButton( kOrphanFilesDialogOKButtonID );
				wipeInfoDialog->SetCancelButton( kOrphanFilesDialogCancelButtonID );
				wipeInfoDialog->Show();
				
				while ( !done )
					{
				
						message = wipeInfoDialogHandler.DoDialog();
						
						// disable OK if no checkboxes checked
						
						
						switch( message )
							{
							
								case msg_OK:
								case msg_Cancel:
									{
									
										CMacCVSOrphanFilesQueryRec	theQuery 				= { false, false, false };
											
										done = true;
									
										theQuery.fRemoveCkid = removeCkidCheckbox->GetValue() != 0;
										theQuery.fRemoveMcvs = removeMcvsCheckbox->GetValue() != 0;
										theQuery.fRemoveCvsDirecories = removeDirectoriesCheckbox->GetValue() != 0;
										theQuery.fOrphanEntireTree = orphanTreeRadio->GetValue() != 0;
																
										if ( (message == msg_OK) && (IsProjectRootSpecified()) )
											{

												FSSpec									targetSpec			= {  0, 0, "\p" };

//  Orphan the files and folders
												
												LCommander::PostAnAction( new CProjectOrphanFilesAction(this, inItemList, &theQuery) );
																				
											}
										
									}			
									break;
								
							}
				
											
					}
					
			}
			
		catch ( ... )
			{
						
			
			}
	
	}
	
	
#pragma mark -
#pragma mark TAG LIST RELATED


Boolean
CMacCVSDoc::AreTagListsEqual( LArray &inTagList1, LArray &inTagList2 )
	{
	
		Boolean		bResult 			= true;
		UInt32		intMaxCount		= 0;
		UInt32		intSizeOne		= 0;
		UInt32		intSizeTwo		= 0;
						
		Str255		pstrItemOne					= "\p";
		Str255		pstrItemTwo					= "\p";
						
		if ( inTagList1.GetCount() != inTagList2.GetCount() )
			{
				bResult = false;
			}
		else
			{			
				intMaxCount = inTagList1.GetCount();
				
				for ( UInt32 index = 1; (index <= intMaxCount) && (bResult); index++ )
					{					
						intSizeOne = intSizeTwo = sizeof(Str255);
						
						inTagList1.FetchItemAt( index, pstrItemOne, intSizeOne );
						inTagList2.FetchItemAt( index, pstrItemTwo, intSizeTwo );
						
						Assert_( (intSizeOne == sizeof(Str255)) && (intSizeTwo == sizeof(Str255)) );
						
						if ( pstrItemOne[0] != pstrItemTwo[0] )
							{
								bResult = false;
							}
						else
							{							
								bResult = !strncmp( (char *)pstrItemOne, (char *)pstrItemTwo, pstrItemOne[0] + 1 );														
							}					
					}		
			}
		
		return bResult;
		
	}
	
	
void
CMacCVSDoc::UseNewTagList( LArray &inNewTagList )
	{
	
		UInt32		intMaxCount	= 0;
		UInt32		intIOCount	= 0;
		Str255		pstrTag			= "\p";
		
		Assert_( mTagList != NULL );
		
		mTagList->RemoveItemsAt( mTagList->GetCount(), 1 );

		intMaxCount = inNewTagList.GetCount();
		
		if ( intMaxCount > 0 )
			{			
				for ( UInt32 index = 1; index <= intMaxCount; index++ )
					{				
						intIOCount = sizeof( Str255 );
						
						inNewTagList.FetchItemAt( index, pstrTag, intIOCount );
					
						if ( intIOCount == sizeof(Str255) )
							{						
								mTagList->InsertItemsAt( 1, LArray::index_Last, pstrTag, intIOCount );							
							}							
					}			
			}
	
	}
	
	
#pragma mark -
#pragma mark TASK MANAGEMENT


Boolean
CMacCVSDoc::AllowKillAllTasks()	
	{
	
		Boolean						bResult						= false;
		Boolean						bDone							= false;

		LDialogBox				*pDialogBox 			= NULL;
		LCaption					*pMessageText			= NULL;
		
		Str255						pstrDocName 			= "\p";
		LStr255						pstrMessage;
	
		StDialogHandler stHandler( kCloseDialogPPobID, mSuperCommander );
		stHandler.AddAttachment( new CThreadedTaskYieldAttachment );
		
		pDialogBox = dynamic_cast<LDialogBox *>( stHandler.GetDialog() );
		ThrowIfNil_( pDialogBox );
		
		pMessageText = dynamic_cast<LCaption *>( pDialogBox->FindPaneByID(kCloseDialogMessageCaptionID) );
		ThrowIfNil_( pMessageText );
		
		GetDescriptor( pstrDocName );
		
		pstrMessage.Append( "\pAre you sure you want to terminate active operations for the document " );
		pstrMessage.Append( pstrDocName );
		pstrMessage.Append( "\p?" );
		
		pMessageText->SetDescriptor( pstrMessage );

		pDialogBox->SetDefaultButton( kCloseDialogCancelButtonID );
		pDialogBox->SetCancelButton( kCloseDialogCancelButtonID );
		
		pDialogBox->Show();
		
		while ( !bDone )
			{			
				switch ( stHandler.DoDialog() )
					{ 					
						case msg_OK:

							bResult = true;							
							bDone = true;
							
						break;

						case msg_Cancel:
						
							bDone = true;
							
						break;												
					}			
			}
	
		return bResult;
	
	}
	
	
void
CMacCVSDoc::KillAllTasks()
	{
	
		CThreadedTask	*pCurTask = NULL;

		LArrayIterator stIterator( *mConnectionList );
		
		while ( stIterator.Next(&pCurTask) )
			{			
				Assert_( pCurTask != NULL );
				
				if ( pCurTask != NULL )
					{
						pCurTask->ShutdownThread();	
					}	
			}
	
		while ( mConnectionList->GetCount() > 0 )
			{
				CThreadedTask::Yield();
			}
			
	}
	
	
void
CMacCVSDoc::AddToTaskList( CThreadedTask *inTask )
	{
	
		ThrowIfNil_( inTask );
		ThrowIfNil_( mConnectionList );
		
		if ( mConnectionList->GetCount() == 0 )
			{
				if ( mMessageWindow != NULL )
					{
						if ( GetClearMessageWindow() )
							{
								mMessageWindow->ClearMessages();
							}
					}
			}
			
		mConnectionList->InsertItemsAt( 1, LArray::index_Last, &inTask );
	
	}
	
	
void
CMacCVSDoc::RemoveFromTaskList( CThreadedTask *inTask )
	{
	
		UInt32					intMaxIndex	= 0;
		UInt32					intCurIndex	= 0;
		
		CThreadedTask		*pCurTask		= NULL;
		
		ThrowIfNil_( inTask );
		ThrowIfNil_( mConnectionList );
				
		intMaxIndex = mConnectionList->GetCount();
			
		for ( intCurIndex = 1; intCurIndex <= intMaxIndex; intCurIndex++ )
			{			
				if ( mConnectionList->FetchItemAt(intCurIndex, &pCurTask) )
					{		
						if ( pCurTask == inTask )
							{
								mConnectionList->RemoveItemsAt( 1, intCurIndex );
							}
					}					
			}
				
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark PROJECT WINDOW ROUTINES



void
CMacCVSDoc::BringToFront()
	{	
	
		Assert_( mWindow != NULL );
		
		UDesktop::SelectDeskWindow( mWindow );
	
	}
	

void
CMacCVSDoc::PositionDocWindow( LWindow *inDocWindow )
	{
		CMacCVSProjectViewWindow		*projectViewWindow = dynamic_cast<CMacCVSProjectViewWindow *>(inDocWindow);
		Rect			windowBounds;
		LFile			*theFile = NULL;
		
		Assert_ ( projectViewWindow != NULL );
		
		// check that this is indeed the root window for this doc
		FSSpec		windowRootFolder, projectRootFolder;
		
		GetProjectRootFileSpec( projectRootFolder );
		
		if (! projectViewWindow->GetWindowRootFolder( windowRootFolder ) ||
				! LFile::EqualFileSpec( windowRootFolder, projectRootFolder ) )
			return;
		
		inDocWindow->GetGlobalBounds( windowBounds );
				
		theFile = GetFile();
		Assert_( theFile != NULL );
		
		SInt16 intOldResRef = ::CurResFile();
		
		try
			{
				try
					{
						theFile->OpenResourceFork( fsRdPerm );
					}
				catch ( ... )
					{
						return;		// no resource file yet
					}
							
				projectViewWindow->ReadWindowPosResource( windowBounds);
				projectViewWindow->SafeSetWindowBounds( windowBounds );
				
				theFile->CloseResourceFork();
			}
		catch ( ... )
			{
				::UseResFile( intOldResRef );
				throw;
			}
			
		::UseResFile( intOldResRef );
	}


void
CMacCVSDoc::SaveDocWindowPosition( LWindow *inDocWindow )
	{
		CMacCVSProjectViewWindow		*projectViewWindow = dynamic_cast<CMacCVSProjectViewWindow *>(inDocWindow);
		Rect			windowBounds;
		LFile			*theFile = NULL;
		
		Assert_ ( projectViewWindow != NULL );
		
		// check that this is indeed the root window for this doc
		FSSpec		windowRootFolder, projectRootFolder;
	
		GetProjectRootFileSpec( projectRootFolder );
		
		if (! 	projectViewWindow->GetWindowRootFolder( windowRootFolder ) ||
				! LFile::EqualFileSpec( windowRootFolder, projectRootFolder ) )
			return;

		inDocWindow->GetGlobalBounds( windowBounds );
				
		theFile = GetFile();
		Assert_( theFile != NULL );
		
		SInt16 intOldResRef = ::CurResFile();
		
		try
			{
				try
					{
						theFile->OpenResourceFork( fsRdWrPerm );
					}
				catch ( ... )
					{
						OSErr		theErr = ResError();
						
						if (theErr == eofErr )			// no res file yet
							{
								theFile->CreateNewFile( kMacCVSCreator, kMacCVSDocFileType, smSystemScript );
							
								theFile->OpenResourceFork( fsRdWrPerm );
							}
						else
							throw( theErr );
					}
				
				inDocWindow->GetGlobalBounds( windowBounds );
				projectViewWindow->AddOrChangeWindowPosResource( windowBounds);
				
				theFile->CloseResourceFork();
		 	}
		catch ( ... )
			{
				::UseResFile( intOldResRef );
				throw;
			}
			
		::UseResFile( intOldResRef );
		
	}
	
LWindow* 
CMacCVSDoc::NewProjectWindow( FSSpec *inAnchorFileSpec, FSSpec *inProjectRootFileSpec, StringPtr inWinTitle )
	{
		
		CMacCVSProjectViewWindow	*pNewWindow		= NULL;
		CMacCVSProjectView				*pProjectView	= NULL;
		
		try
			{
				pNewWindow = dynamic_cast<CMacCVSProjectViewWindow *>( LWindow::CreateWindow( CMacCVSProjectViewWindow::kMacCVSProjectViewWindowPPobID, this ) );
				ThrowIfNil_( pNewWindow );
				
				pNewWindow->SetOwnerDoc( this );
				
				if ( inWinTitle != NULL )
					{
						pNewWindow->SetDescriptor( inWinTitle );
					}
				else if ( inAnchorFileSpec != NULL )	
					{
						pNewWindow->SetDescriptor( inAnchorFileSpec->name );
					}
					
				pProjectView = dynamic_cast<CMacCVSProjectView *>( pNewWindow->FindPaneByID(CMacCVSProjectView::class_ID) );
				ThrowIfNil_( pProjectView );

		 		pProjectView->AddListener( this );
				pProjectView->SetRootItem( inAnchorFileSpec, inProjectRootFileSpec );
				
		 		pNewWindow->SetWindowPosition();
			}
		catch ( ... )
			{
				if ( pNewWindow != NULL )
					{
						delete pNewWindow;
						pNewWindow = NULL;
					}
					
				throw;
			}
				
		return pNewWindow;
		
	}	


LWindow *
CMacCVSDoc::NewViewWindow( FSSpec *inAnchorRootFileSpec, FSSpec *inProjectRootFileSpec, Boolean &outMadeNew, const Boolean selectIfExists )
	{

		LWindow	*pNewWindow = NULL;
			
		if ( (inAnchorRootFileSpec == NULL) || (inProjectRootFileSpec == NULL) ) 
			{
				return NULL;
			}
			
		if ( selectIfExists )
			{
				LWindow	*pWindowItem = NULL;
				
				LArrayIterator	stIterator( *mWindowList );
				
				while ( stIterator.Next(&pWindowItem) ) 
					{
						CMacCVSProjectViewWindow	*pProjectView = dynamic_cast<CMacCVSProjectViewWindow *>( pWindowItem );
						FSSpec										fsRoot = { 0, 0, "\p" };
						
						if ( pProjectView != NULL )
							{
								if (	pProjectView->GetWindowRootFolder( fsRoot ) && LFile::EqualFileSpec(*inAnchorRootFileSpec, fsRoot) )
									{
										outMadeNew = false;
										pWindowItem->Show();
										pWindowItem->Select();
										return pWindowItem;
									}
							}
					}
			}
	
		pNewWindow = NewProjectWindow( inAnchorRootFileSpec, inProjectRootFileSpec, NULL );
		AddToWindowList( pNewWindow );
		outMadeNew = true;
		
		return pNewWindow;
		
	}


void
CMacCVSDoc::UpdateItemInProjectViews( const FSSpec *inItemToUpdate, const Boolean inUpdateAllViews )
	{
			
		LWindow							*pWindowItem 		= NULL;
		CMacCVSProjectView	*pProjectView		= NULL;
		CMacCVSLurkersView	*pLurkersView		= NULL;

		ThrowIfNil_( inItemToUpdate );
		
		if ( inUpdateAllViews )
			{		
				pProjectView = GetProjectWindowView( mWindow );				
				Assert_( pProjectView != NULL );
				
				if ( pProjectView != NULL )
					{
						pProjectView->UpdateProjectItem( inItemToUpdate );
					}
					
				LArrayIterator	stIterator( *mWindowList );
				
				while ( stIterator.Next(&pWindowItem) )
					{					
						pProjectView = GetProjectWindowView( pWindowItem );
	
						if ( pProjectView != NULL )
							{
								pProjectView->UpdateProjectItem( inItemToUpdate );
							}
						else
							{
								pLurkersView = GetLurkersWindowView( pWindowItem );
							
								if ( pLurkersView != NULL )
									{
										pLurkersView->UpdateProjectItem( inItemToUpdate );
									}
							}					
					}
			}		
		else
			{
				Assert_( false );
				
				//  We should always update all views for now.
			}
	
	}
	
	
Boolean
CMacCVSDoc::IsOurWindow( LWindow *inWindow )
	{
	
		Boolean		bResult 			= false;
		LWindow		*pWindowItem	= NULL;
		UInt32		intMaxIndex		= 0;
		UInt32		intCurIndex		= 0;
							
		if ( inWindow != NULL )
			{
				if ( inWindow == mWindow )
					{	
						bResult = true;
					}
				else
					{
						intMaxIndex = mWindowList->GetCount();
						
						for ( intCurIndex = 1; intCurIndex <= intMaxIndex; intCurIndex++ )
							{	
								if ( mWindowList->FetchItemAt(intCurIndex, &pWindowItem) )
									{
										if ( pWindowItem == inWindow )
											{
												bResult = true;
											}
									}
							}
					}
			}
			
		return bResult;
	
	}
	
	
void
CMacCVSDoc::AddToWindowList( LWindow *inWindow )
	{
	
		Assert_( inWindow != NULL );
		Assert_( mWindowList != NULL );
		
		if ( inWindow != NULL )
			{			
				mWindowList->InsertItemsAt( 1, LArray::index_Last, &inWindow );			
			}
	
	}
	
	
Boolean
CMacCVSDoc::RemoveFromWindowList( LWindow *inWindow )
	{

		Boolean		bResult		= false;
		UInt32		intIndex	= 0;
		
		Assert_( inWindow != NULL );
		Assert_( mWindowList != NULL );
			
		intIndex = mWindowList->FetchIndexOf( &inWindow );
		Assert_( intIndex != LArray::index_Bad );
		
		if ( intIndex != LArray::index_Bad )
			{			
				mWindowList->RemoveItemsAt( 1, intIndex );
				bResult = true;	
			}
			
		return bResult;
		
	}
	

	
void
CMacCVSDoc::KillAllViews()
	{
	
		LWindow	*pCurWindow = NULL;

		LArrayIterator stIterator( *mWindowList );
		
		while ( stIterator.Next(&pCurWindow) )
			{			
				Assert_( pCurWindow != NULL );
				
				if ( pCurWindow != NULL )
					{
						delete pCurWindow;
						pCurWindow = NULL;
						
						mWindowList->RemoveItemsAt( 1, stIterator.GetCurrentIndex() );					
					}	
			}
						
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark MAPPING LIST


// ---------------------------------------------------------------------------
//		 InitDefaultMappingList
// ---------------------------------------------------------------------------
//	InitDefaultMappingList() simply puts some sample file mappings into the 
//  mappings LArray. In the future this should be read out of some resource 
//  template or something so we can easily update it.


void
CMacCVSDoc::InitDefaultMappingList()
	{
	
		CCVSEngineFileMappingRec	sDefaultMapping;
		LStr255										pstrExtension;
		
		Assert_( mMappingList != NULL );

		if ( mMappingList != NULL )
			{				
				mMappingList->RemoveItemsAt( mMappingList->GetCount(), LArray::index_First );
				
				//  Default Mapping
				
				sDefaultMapping.fFileNamePStr[0] 	= 0;
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineAppleSingleEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.c' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.c" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.cp' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.cp" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.cpp' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.cpp" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.h' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.h" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.p' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.p" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.r' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.r" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'MPS ';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.exp' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.exp" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineCRTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );
	
				//  '.html' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.html" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'MOSS';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.htm' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.htm" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'MOSS';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.gif' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.gif" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'GIFf';
				sDefaultMapping.fFileCreator 			= 'GCon';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineBinaryEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.jpg' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.jpg" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'JPEG';
				sDefaultMapping.fFileCreator 			= 'JVWR';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineBinaryEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.jpeg' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.jpeg" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'JPEG';
				sDefaultMapping.fFileCreator 			= 'JVWR';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineBinaryEncoding;
				
				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.sit' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.sit" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'SITD';
				sDefaultMapping.fFileCreator 			= 'SIT!';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineBinaryEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.bin' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.bin" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'MBin';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineBinaryEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.hqx' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.hqx" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'TEXT';
				sDefaultMapping.fFileCreator 			= 'BNHQ';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineTextEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );

				//  '.mcp' files
				
				pstrExtension[0] = 0;
				pstrExtension.Append( "\p.mcp" );
				PStrCopy( pstrExtension, sDefaultMapping.fFileNamePStr, sizeof(sDefaultMapping.fFileNamePStr) );
	
				sDefaultMapping.fFileType 				= 'MMPr';
				sDefaultMapping.fFileCreator 			= 'CWIE';
				sDefaultMapping.fTranslation 			= kCVSEngineLFTranslation;
				sDefaultMapping.fEncoding 				= kCVSEngineBinaryEncoding;

				mMappingList->InsertItemsAt( 1, LArray::index_Last, &sDefaultMapping );
			}
			
	}
	

OSErr
CMacCVSDoc::BuildFileMappingList( const LArray *pInMappings, CCVSEngineFileMappingRec **pOutMappings, UInt32 *intOutCount )
	{
	
		OSErr result = noErr;
		
		try
			{
				CCVSEngineFileMappingRec sCurMapping;
				sCurMapping.Init();

				if ( pInMappings == NULL || pOutMappings == NULL || intOutCount == NULL )
					{
						result = kCVSEngineBadParamErr;
					}
				else
					{
						UInt32	intMaxCount = 0;
						UInt32	intSize	  	= 0;
						
						*intOutCount = 0;
									
						intMaxCount = pInMappings->GetCount();			
						
						*pOutMappings = (CCVSEngineFileMappingRec *)::NewPtr( intMaxCount * sizeof(CCVSEngineFileMappingRec) );
						
						if ( *pOutMappings == NULL )
							{
								ThrowIfOSErr_( ::MemError() );
								ThrowIfOSErr_( memFullErr );
							}
							
						for ( UInt32 intIndex = 1; intIndex <= intMaxCount; intIndex++ )
							{					
								intSize = sizeof( sCurMapping );
								
								pInMappings->FetchItemAt( intIndex, &sCurMapping, intSize );					
								Assert_( intSize == sizeof(sCurMapping) );
								
								if ( intSize == sizeof(sCurMapping) )
									{							
										(*pOutMappings)[*intOutCount] = sCurMapping;							
										(*intOutCount)++;							
									}						
							}	
					}
			}
		catch( OSErr intException )
			{
				result = intException;
			}
		catch( ExceptionCode intException )
			{
				result = intException;
			}
		catch ( ... )
			{
				result = kCVSEngineUnknownExceptionErr;
			}
						
		return result;
		
	}


LArray *
CMacCVSDoc::GetFileMappingList( void )
	{
	
		LArray *pItemList = new LArray( sizeof(CCVSEngineFileMappingRec) );
		ThrowIfNil_( pItemList );
		
		//  Must deference to get operator= to run.
		*pItemList = *mMappingList;
		
		return pItemList;
		
	}
	
	
void
CMacCVSDoc::SetFileMappingList( const CCVSEngineFileMappingRec *pInMappingsList, const UInt32	intInCount )
	{
		
		CCVSEngineFileMappingRec sMapping;
		
		if ( (pInMappingsList != NULL) && (intInCount > 0) )
			{
				if ( mMappingList != NULL )
					{
						delete mMappingList;
						mMappingList = NULL;
					}
					
				mMappingList = new LArray( sizeof(CCVSEngineFileMappingRec) );
				ThrowIfNil_( mMappingList );
				
				for ( UInt32 intIndex = 0; intIndex < intInCount; intIndex++ )
					{
						sMapping.Init();
						sMapping = pInMappingsList[intIndex];
						
						mMappingList->InsertItemsAt( 1, LArray::index_Last, &sMapping, sizeof(sMapping) );
					}

				SetModified( true );
			}
		else if ( (pInMappingsList == NULL) && (intInCount == 0) )
			{
				if ( mMappingList != NULL )
					{
						delete mMappingList;
						mMappingList = NULL;
					}
					
				SetModified( true );
			}
	
	}
	

void
CMacCVSDoc::SetFileMappingList( const LArray &cInMappings )
	{
	
		if ( mMappingList != NULL )
			{
				delete mMappingList;
				mMappingList = NULL;
			}
			
		mMappingList = new LArray( sizeof(CCVSEngineFileMappingRec) );
		*mMappingList = cInMappings;
	
		SetModified( true );
		
	}
	
		
//################################################################################
//################################################################################
#pragma mark -
#pragma mark MISCELLANEOUS


	
CMacCVSProjectView *
CMacCVSDoc::GetProjectWindowView(LWindow *inWindow)
	{

		CMacCVSProjectView	*pProjectView	= NULL;
		
		if ( inWindow != NULL ) 
			{
				if ( CMacCVSDoc::GetWindowKind(inWindow) == kProjectViewWindowKind )
					{
						pProjectView = dynamic_cast<CMacCVSProjectView *>( inWindow->FindPaneByID(CMacCVSProjectView::class_ID) );
					}
			}
			
		return pProjectView;
		
	}


CMacCVSLurkersView *
CMacCVSDoc::GetLurkersWindowView( LWindow *inWindow )
	{

		CMacCVSLurkersView	*pLurkersView	= NULL;
		
		if ( inWindow != NULL ) 
			{
				if ( CMacCVSDoc::GetWindowKind(inWindow) == kLurkersViewWindowKind )
					{
						pLurkersView = dynamic_cast<CMacCVSLurkersView *>( inWindow->FindPaneByID(CMacCVSLurkersView::class_ID) );
					}
			}
			
		return pLurkersView;
		
	}


SInt16
CMacCVSDoc::GetWindowKind( LWindow *inWindow )
	{
	
		SInt16		result 			= 0;
		GrafPtr		pGrafPort		= NULL;
		
		pGrafPort = inWindow->GetMacPort();		
		ThrowIfNil_( pGrafPort );
		
		switch ( ((WindowPeek)pGrafPort)->windowKind )
			{			
				case kProjectViewWindowKind:
				case kLurkersViewWindowKind:
				case kFileLogViewWindowKind:
				case kFileStatusViewWindowKind:
				case kCompareViewWindowKind:
				
					result = ((WindowPeek)pGrafPort)->windowKind;
					
				break;			
			}
			
		return result;
			
	}
	
	
void
CMacCVSDoc::OpenFile( FSSpec	&inFileSpec )
	{

		// Create a new File object, read the entire File contents,
		// put the contents into the text view, and set the Window
		// title to the name of the File.
			
		mFile = new LFileStream( inFileSpec );		
		mIsSpecified = true;
		
		//	See if the specified file exists.  If it does, then 
		//	open it and rename the window 
		
		FInfo	sFileInfo;
		
		if ( ::FSpGetFInfo( &inFileSpec, &sFileInfo ) == noErr ) 
			{				
				mFile->OpenDataFork( fsRdWrPerm );
				
				try
					{					
						LoadPreferences();			
					}				
				catch ( OSErr inErr )
					{				
						if ( inErr == kOutdatedPrefsErr )
							{							
								UMessageDialog	prefsMesgDialog( kWillUpdatePrefsStr, this );
								
								InitDefaultMappingList();
								
								SavePreferences();
							}						
						else if ( inErr == kNoRootSpecifiedErr )
							{						
								UWarningDialog	noRootDialog( kMustSetRootStr, this );						
							}							
						else 
							{
								//  "Convert" the exception to a PowerPlant
								//  exception type.
								
								Throw_( inErr );
							}
					}				
			}			
		else 
			{
				mFile->CreateNewDataFile( kMacCVSCreator, kMacCVSDocFileType );
				mFile->OpenDataFork(fsRdWrPerm);
	
				InitDefaultMappingList();

				SavePreferences();			
			}
		
	}


