//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CMacCVSLogTask.cp
//	
//	  	Authors: 	Ram Rajadhyaksha
//
//################################################################################


#include <string.h>
#include <Threads.h>

#include "CMacCVSDoc.h"
#include "CVSEngine.h"
#include "UMacCVSUtils.h"

#include "CMacCVSLogWindow.h"
#include "CMacCVSLogTask.h"


CMacCVSLogTask::CMacCVSLogTask( const CMacCVSLogWindow *inOwnerWindow, CMacCVSDoc *inOwnerDoc, const FSSpec	*inTargetItem ):

	CMacCVSWindowTask( inOwnerWindow, inOwnerDoc )
		
	{
		mTargetItem.name[0] = 0;
		mTargetItem.parID		= 0;
		mTargetItem.vRefNum = 0;
		mParsingState				= kReadNothing;
		mAuthorPStr[0]			= 0;
		mRevisionPStr[0]		= 0;
		mDate								= 0;
		
		mBuffer[0]					= 0;
		mBufLen							= 0;
		
		if ( inTargetItem != NULL )
			{
				mTargetItem = *inTargetItem;
			}
			
		SetEventNotifier( Notifier );
	}
	
	
CMacCVSLogTask::~CMacCVSLogTask()
	{
		// the superclass kills the engineRef
	}


#pragma mark -


OSErr
CMacCVSLogTask::DoEngineCommand()
	{
		return CVSEngineLog( mEngineRef, &mTargetItem, CMacCVSLogTask::LogNotifier, this );
	}	
	

void
CMacCVSLogTask::ProcessData()
	{

		const Str255			revToken					= "\prevision ",
											dateToken					= "\pdate: ",
											branchToken				= "\pbranches: ",
											headerToken				= "\p----",
											endCommentToken  	= "\p====",
											authorToken				= "\p  author: ";
		
		CMacCVSLogWindow	*logWindow				= dynamic_cast<CMacCVSLogWindow *>(mOwnerWindow);
		Boolean						bFoundCR					= false,
											bDone							= false;
											
		UInt32						bytesLeft					= mStream.GetMarker();
		char							tempChar					= 0x00;
		
		mStream.SetMarker( 0, streamFrom_Start );
		
		while ( !bDone )
			{
			
				bFoundCR = false;
				
				while ( (mBufLen < kBufMax) && (!bFoundCR) && (bytesLeft > 0) )
					{					
						mStream.ReadBlock( &tempChar, 1 );
						
						if ( tempChar == '\r' )
							{
								bFoundCR = true;
							}
						else
							{								
								mBuffer[mBufLen] = tempChar;
								mBufLen++;								
							}
							
						bytesLeft--;					
					}

//  If we overflowed the buffer, reset it. Ouch! 
//  This can lead to display of corrupted log entries.
		
				if ( (!bFoundCR) && ((mBufLen + 1) >= kBufMax) )
					{
						mBufLen = 0;
					}
				else if ( bFoundCR )
					{
				
						switch ( mParsingState )
							{
							
								case kReadNothing:
								
									if ( mBufLen > headerToken[0] )
										if ( !strncmp(mBuffer, (char *)&headerToken[1], headerToken[0]) )
											{										
												mParsingState = kReadHeader;										
											}
							
								break;
								
								case kReadHeader:

									mAuthorPStr[0] = 0;
									mRevisionPStr[0] = 0;
									mDate = 0;
									
									mParsingState = kReadNothing;
									
									if ( mBufLen > revToken[0] )
										{									
											if ( !strncmp(mBuffer, (char *)&revToken[1], revToken[0]) ) 
												{													
													::BlockMoveData( mBuffer + revToken[0], &mRevisionPStr[1], mBufLen - revToken[0] );
													mRevisionPStr[0] = mBufLen - revToken[0];
						
													mParsingState = kReadRevision;												
												}									
										}
								
								break;
								
								case kReadRevision:
										
									char		tempStr[256];
									UInt32	tempLen = 255;
									
									char		dateStr[256];
									
									mParsingState = kReadNothing;
									mAuthorPStr[0] = 0;
									mDate = 0;
									
									if ( mBufLen > dateToken[0] )
										if ( !strncmp(mBuffer, (char *)&dateToken[1], dateToken[0]) ) 
											{
												
												UMacCVSUtils::ExtractPhrase( mBuffer, mBufLen, ';', 1, tempStr, &tempLen);
												
												if ( (tempLen - dateToken[0]) > 0 ) 
													{
													
														::BlockMoveData( tempStr + dateToken[0], dateStr, tempLen - dateToken[0] );
														
														dateStr[tempLen - dateToken[0]] = 0x00;
														
														mDate = UMacCVSUtils::UnixTimeToMacSeconds( dateStr );
													
													}
												else mDate = 0;
												
												tempLen = 255;
												
												UMacCVSUtils::ExtractPhrase( mBuffer, mBufLen, ';', 2, tempStr, &tempLen );
												
												if ( !strncmp( tempStr, (char *)&authorToken[1], authorToken[0]) )
													{
													
														if ( (tempLen - authorToken[0]) > 0 ) 
															{
															
																::BlockMoveData( tempStr + authorToken[0], &mAuthorPStr[1], tempLen - authorToken[0] );
																mAuthorPStr[0] = tempLen - authorToken[0];	
																
																mParsingState = kReadDate;
																
															}
																					
													}

											}
							
								break;
								
								case kReadDate:
									
									mParsingState = kReadingComment;
									
									mCommentStream.SetMarker( 0, streamFrom_Start );
									
									if ( mBufLen > branchToken[0] )
										if ( !strncmp(mBuffer, (char *)&branchToken[1], branchToken[0]) )
											mParsingState = kReadBranches;

				/*
				**  No break statement here- execution should fall through.
				*/
					
								case kReadBranches:
								case kReadingComment:
								
									UInt32	commentTextLen;
									Handle	commentTextHndl	= NULL;
									
									if ( mBufLen > endCommentToken[0] )
										if ( !strncmp(mBuffer, (char *)&endCommentToken[1], endCommentToken[0]) )
											{
											
												mParsingState = kReadNothing;
												
												commentTextLen = mCommentStream.GetMarker();
												
												if ( commentTextLen > 0 )
													{
													
														commentTextHndl = mCommentStream.DetachDataHandle();
													
														::HLock( commentTextHndl );

														logWindow->AddLogEntry( mAuthorPStr, mRevisionPStr, mDate, *commentTextHndl, commentTextLen );
														
														::HUnlock( commentTextHndl );
														::DisposeHandle( commentTextHndl );
														
													}
												else logWindow->AddLogEntry( mAuthorPStr, mRevisionPStr, mDate, NULL, 0 );
													
												
												break;
											
											}
											
									if ( mBufLen > headerToken[0] )
										if ( !strncmp(mBuffer, (char *)&headerToken[1], headerToken[0]) )
											{
											
												mParsingState = kReadHeader;

												commentTextLen = mCommentStream.GetMarker();

												if ( commentTextLen > 0 )
													{
													
														commentTextHndl = mCommentStream.DetachDataHandle();
													
														::HLock( commentTextHndl );

														logWindow->AddLogEntry( mAuthorPStr, mRevisionPStr, mDate, *commentTextHndl, commentTextLen );
														
														::HUnlock( commentTextHndl );
														::DisposeHandle( commentTextHndl );
														
													}
												else logWindow->AddLogEntry( mAuthorPStr, mRevisionPStr, mDate, NULL, 0 );
												
												break;
											
											}
											
									mCommentStream.WriteBlock( mBuffer, mBufLen );

/*
**  I can't use the << operator because LStream's version of WriteCString
**  writes a 4 byte length byte before writing the stream. (doh!)
*/

									mCommentStream.WriteBlock( "\r", 1 );
									
								break;
								
							}
					
						mBufLen = 0;
								
					}
						
				if ( bytesLeft == 0 )
					bDone = true;
						
			}
			
		mStream.SetMarker( 0, streamFrom_Start );
									
	}
	
	
#pragma mark -


pascal SInt32		
CMacCVSLogTask::Notifier(		const CCVSEngineRef					inEngineRef,
														const FSSpec								*inTargetItem,
														const SInt32 								inEvent, 
														const SInt32 								inError, 
														const UInt32 								inCookie, 
														const void 									*inDataPtr, 
														void 												*inContextPtr)
	{
	
		OSErr						error				= noErr;
		Boolean					bResult			= true;
		CMacCVSLogTask	*theTask		= (CMacCVSLogTask *)inContextPtr;
		
		if ( theTask != NULL )
			{			
				try
					{										
						if ( inError != noErr )
							{
								theTask->PostFileError( inTargetItem, NULL, 0, inError );
							}
							
						if ( inEvent == kCVSEngineYieldActionEvent )
							{
								CThreadedTask::Yield();
							}
					}					
				catch ( ... )
					{				
						bResult = false;					
					}					
			}
			
		return bResult;
		
	}
	
	
pascal SInt32
CMacCVSLogTask::LogNotifier( const CCVSEngineRef inEngineRef, const UInt32 inDataType, const UInt32 inDataLen, const void *inData, void *inContextPtr )
	{
			
		CMacCVSLogTask	*theLogTask			= (CMacCVSLogTask *)inContextPtr;
		SInt32					intBytesWritten = 0;
		
		Assert_( theLogTask != NULL );
		
		if ( theLogTask != NULL )
			{		
				try
					{					
						if ( inDataLen > 0 )
							{						
								theLogTask->mStream.WriteBlock( inData, inDataLen );
											
								theLogTask->ProcessData();
						
								intBytesWritten = inDataLen;							
							}						
					}					
				catch ( ... )
					{										
					}			
			}
			
		return intBytesWritten;
	
	}
	
	
void
CMacCVSLogTask::GetTaskName( Str255 outNamePStr )
	{
		LStr255	pstrName;

		pstrName = "\plog";
			
		if ( pstrName[0] > 0 )
			{
				::BlockMoveData( pstrName, outNamePStr, pstrName[0] + 1 );
			}
	}
	