//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSLogView.cp
//
//	 	Author: 	Ram Rajadhyaksha
//
//################################################################################


#include <string.h>

#include <Threads.h>

#include <LArray.h>
#include <LHierarchyTable.h>
#include <LTableSingleSelector.h>
#include <LTableArrayStorage.h>
#include <LTableMultiGeometry.h>
#include <LNodeArrayTree.h>
#include <LDropFlag.h>
#include <LListener.h>
#include <LDragTask.h>

#include "MacCVSPPConstants.h"

#include "StringUtils.h"
#include "CSpinIndicator.h"
#include "CThreadedTask.h"
#include "UProjectActions.h"

#include "CVSEngineUtils.h"
#include "UMacCVSUtils.h"
#include "CMacCVSLogView.h"
#include "CMacCVSStatusWindow.h"
#include "CMacCVSDoc.h"

UInt32 CMacCVSLogView::sYieldCount = 0;


CMacCVSLogView::CMacCVSLogView( LStream *inStream ):	

	LHierarchyTable(inStream)
	
	{
		
		OSErr	result				= noErr;
		
		mTableGeometry 			= NULL;
		mTableSelector			= NULL;
		mTableStorage 			= NULL;

		mViewThread					= NULL;
		
		mLogEntries					= NULL;
		mViewEventQueue			= NULL;
		
		mCurrentSortMethod	= NULL;
			
		try
			{

				mLogEntries = new LArray( sizeof(CMacCVSLogViewCellRec *) );
				ThrowIfNil_( mLogEntries );
				
				mCurrentSortMethod = kCMacCVSLogViewSortByDate;
					
				mLogEntries->SetComparator( GetCurrentComparator() );
				mLogEntries->SetKeepSorted( true );
							
				mViewEventQueue = new LArray( sizeof(CMacCVSLogViewEventRec) );
				ThrowIfNil_( mViewEventQueue );
				
			}
			
		catch ( ... )
			{
		
				if ( mLogEntries != NULL ) 
					{
					
						delete mLogEntries;
						mLogEntries = NULL;
						
					}
				
				if ( mViewEventQueue != NULL )
					{
					
						delete mViewEventQueue;
						mViewEventQueue = NULL;
						
					}
																
				throw;
		
			}
			
	}


CMacCVSLogView::~CMacCVSLogView()
	{

		UInt32									maxItems				= 0x00000000,
														curItem					= 0x00000000;
		CMacCVSLogViewCellRec		*theLogEntry		= NULL;
					
		if ( mViewThread != NULL )
			{
			
				mViewThread->ShutdownThread();
				
				while ( mViewThread != NULL )
					CThreadedTask::Yield();
			
			}
			
		if ( mLogEntries != NULL ) 
			{
			
				maxItems = mLogEntries->GetCount();
				
				for ( curItem = 1; curItem <= maxItems; curItem++ )
					{
						
						theLogEntry = NULL;
						
						if ( mLogEntries->FetchItemAt(curItem, &theLogEntry) )
							if ( theLogEntry != NULL )
								{
								
									if ( theLogEntry->fCommentStr != NULL )
										::DisposePtr( theLogEntry->fCommentStr );
										
									::DisposePtr( (Ptr)theLogEntry );
					
								}
								
					}
					
				delete mLogEntries;
				mLogEntries = NULL;
				
			}
		
		if ( mViewEventQueue != NULL )
			{
			
				delete mViewEventQueue;
				mViewEventQueue = NULL;
				
			}
				
	}


CMacCVSLogView*
CMacCVSLogView::CreateLogViewFromStream(LStream *inStream)
	{

		return ( new CMacCVSLogView(inStream) );

	}


void
CMacCVSLogView::FinishCreateSelf()
	{
	
//  Create the helpers for this LTableView.
		
		mTableGeometry = new LTableMultiGeometry( this, kDefaultColumnWidth, 16 );
		ThrowIfNil_( mTableGeometry );
		
		mTableSelector = new LTableSingleSelector( this );
		ThrowIfNil_( mTableSelector );
		
		mTableStorage = new LTableArrayStorage( this, sizeof(CMacCVSLogViewCellRec *) );
		ThrowIfNil_( mTableStorage );

//  Insert a single column.

		InsertCols( 1, 0, NULL, NULL, false );

//  Attachment for keyboard navigation.

		LKeyScrollAttachment	*keyAttachment = new LKeyScrollAttachment( this );
		ThrowIfNil_( keyAttachment );
		
		AddAttachment( keyAttachment );

//  Start the view thread.
		
		mViewThread = new CMacCVSLogViewThread( *this );
		ThrowIfNil_( mViewThread );
		
		mViewThread->Resume();
	
	}
	
	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark LISTENING


void
CMacCVSLogView::ListenToMessage( MessageT inMessage, void *ioParam )
	{

		if ( *(UInt32 *)ioParam != 0 ) 
			{
			
				switch (inMessage) 
					{
					
						case kCMacCVSLogViewSortByAuthor:
						
							SortByAuthor();
							
						break;

						case kCMacCVSLogViewSortByVersion:
						
							SortByVersion();
						
						break;
						
						case kCMacCVSLogViewSortByDate:
						
							SortByDate();
							
						break;

					}
				
			}
			
	}


#pragma mark -
#pragma mark COMMAND HANDLING


void
CMacCVSLogView::GetDiffs( const CMacCVSLogViewCellRec *inFromCellRec, const CMacCVSLogViewCellRec *inToCellRec, Boolean inAdvancedDiff )
	{
	
			CCVSEngineDiffOptions		diffOptions;
			diffOptions.Init();
			
			diffOptions.mUseRevisions = true;
			
			PStrCopy(inFromCellRec->fVersionPStr, diffOptions.mOrigRevisionPStr, 255);
			PStrCopy(inToCellRec->fVersionPStr, diffOptions.mDiffRevisionPStr, 255);
			
			CMacCVSWindow		*logWindow = dynamic_cast<CMacCVSWindow *>( LWindow::FetchWindowObject(GetMacPort()) );
			ThrowIfNil_(logWindow);

			CMacCVSDoc			*ownerDoc = logWindow->GetOwnerDoc();
			ThrowIfNil_(ownerDoc);

			if ( ownerDoc->GetPasswordFromUser() )
				{
					CMacCVSStatusWindow	*statusWindow = dynamic_cast<CMacCVSStatusWindow *>( LWindow::CreateWindow(CMacCVSStatusWindow::kMacCVSStatusWindowID, ownerDoc) );
					ThrowIfNil_( statusWindow );

					if ( inAdvancedDiff )
						{
						
							if ( ! CProjectDiffAction::GetDiffOptions( ownerDoc, diffOptions, false, false) )
								return;
						}
					
					ownerDoc->AddToWindowList( statusWindow );
					statusWindow->SetOwnerDoc(ownerDoc);
					statusWindow->SetWindowPosition();
						
					statusWindow->Status( &mLogItem, ownerDoc, true, &diffOptions );
				}
	}


void
CMacCVSLogView::DoUpdate( ConstStringPtr pstrInRevision )
	{

		LArray *pItemList = NULL;
		
		try
			{				
				pItemList = new LArray( sizeof(FSSpec) );
				ThrowIfNil_( pItemList );

				CMacCVSWindow	*pLogWindow = dynamic_cast<CMacCVSWindow *>( LWindow::FetchWindowObject(GetMacPort()) );
				ThrowIfNil_( pLogWindow );

				CMacCVSDoc *pOwnerDoc = pLogWindow->GetOwnerDoc();
			
				Assert_( FSpIsValid(mLogItem) );
				
				if ( FSpIsValid(mLogItem) )
					{					
						CCVSEngineUpdateOptions sUpdateOptions;
						sUpdateOptions.Init();
					
						pItemList->InsertItemsAt( 1, LArray::index_First, &mLogItem );
												
						PStrCopy( pstrInRevision, sUpdateOptions.mRevisionPStr, sizeof(sUpdateOptions.mRevisionPStr) );
						
						sUpdateOptions.mCompressionLevel = pOwnerDoc->GetCompressionLevel();
						
						if ( sUpdateOptions.mCompressionLevel != 0 )
							{
								sUpdateOptions.mUseCompression = true;
							}
							
						sUpdateOptions.mMergePolicy = pOwnerDoc->GetMergePolicy();
							
						if ( pOwnerDoc->GetPasswordFromUser() )
							{
								LCommander::PostAnAction( new CProjectUpdateAction(pOwnerDoc, pItemList, false, &sUpdateOptions) );
							}
					}

				if ( pItemList != NULL )
					{
						delete pItemList;
						pItemList = NULL;
					}
			}
		catch ( ... )
			{
				if ( pItemList != NULL )
					{
						delete pItemList;
						pItemList = NULL;
					}
				throw;
			}
			
	}


Boolean
CMacCVSLogView::ObeyCommand( CommandT inCommand, void *ioParam )
	{
		
		Boolean result = false;
		

		switch ( inCommand )
			{
			
				case cmd_Copy:
					{
						
						STableCell selectedCell = GetFirstSelectedCell();
						
						if ( IsValidCell(selectedCell) ) 
							{
							
								STableCell	wideCell( GetWideOpenIndex(selectedCell.row), 1 );
								UInt32 			nestingLevel = GetNestingLevel( wideCell.row );
							
								if ( nestingLevel > 0 ) 
									{
									
										CMacCVSLogViewCellRec	*theLogEntry 	= NULL;
										UInt32								cellSize 			= sizeof( CMacCVSLogViewCellRec	* );
										
										GetCellData( wideCell, &theLogEntry, cellSize );
			
										Assert_( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) );
										
										if ( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) ) 
											{
									
												::ZeroScrap();
												::PutScrap( theLogEntry->fCommentLen, 'TEXT', theLogEntry->fCommentStr );
									
											}
											
									}
							
							}
							
						result = true;
						
					}
					break;
			
				case cmd_Update:
					{
						STableCell selectedCell = GetFirstSelectedCell();
						
						if ( IsValidCell(selectedCell) ) 
							{						
								STableCell	wideCell( GetWideOpenIndex(selectedCell.row), 1 );
								UInt32 			nestingLevel = GetNestingLevel( wideCell.row );
							
								if ( nestingLevel == 0 ) 
									{								
										CMacCVSLogViewCellRec	*pCellData 	= NULL;
										UInt32								intDataSize	= sizeof( CMacCVSLogViewCellRec	* );
										
										GetCellData( wideCell, &pCellData, intDataSize );
				
										Assert_( (pCellData != NULL) && (intDataSize == sizeof(CMacCVSLogViewCellRec *)) );
										
										if ( (pCellData != NULL) && (intDataSize == sizeof(CMacCVSLogViewCellRec *)) ) 
											{								
												DoUpdate( pCellData->fVersionPStr );
											}											
									}							
							}
							
						result = true;
					}									
				break;
				
				case cmd_Diff:
				case cmd_AdvancedDiff:
					{
						STableCell selectedCell = GetFirstSelectedCell();
						
						if ( IsValidCell(selectedCell) ) 
							{						
								STableCell	wideCell( GetWideOpenIndex(selectedCell.row), 1 );
								STableCell	prevCell( wideCell.row + 2 , 1);
								UInt32 			nestingLevel = GetNestingLevel( wideCell.row );
							
								if ( nestingLevel > 0 ) 
									{								
										CMacCVSLogViewCellRec	*theFromLogEntry 	= NULL;
										CMacCVSLogViewCellRec	*theToLogEntry 		= NULL;
										UInt32								cellSize 					= sizeof( CMacCVSLogViewCellRec	* );
										
										GetCellData( wideCell, &theToLogEntry, cellSize );
										GetCellData( prevCell, &theFromLogEntry, cellSize );
			
										Assert_( (theToLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) );
										
										if ( (theFromLogEntry != NULL) && (theToLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) ) 
											{
									
												GetDiffs(theFromLogEntry, theToLogEntry, inCommand == cmd_AdvancedDiff);
									
											}											
									}							
							}
							
						result = true;
					}					
				break;
					
				default:
					result = LCommander::ObeyCommand( inCommand, ioParam );
				break;
			}
		
		return result;
		
	}


void
CMacCVSLogView::FindCommandStatus(	CommandT		inCommand,
																		Boolean			&outEnabled,
																		Boolean			&outUsesMark,
																		Char16			&outMark,
																		Str255			outName)
	{
		
		outEnabled = false;
		
		switch( inCommand )
			{
			
			case cmd_Copy:
				{				
					STableCell selectedCell = GetFirstSelectedCell();
					
					if ( IsValidCell(selectedCell) )
						{						
							STableCell 	wideCell( GetWideOpenIndex(selectedCell.row), 1 );
							UInt32 			nestingLevel = GetNestingLevel( wideCell.row );
							
							if ( nestingLevel > 0 )
								{
									outEnabled = true;
								}							
						}				
				}
			break;
			
			case cmd_Update:
				{				
					STableCell selectedCell = GetFirstSelectedCell();
					
					if ( IsValidCell(selectedCell) )
						{						
							STableCell 	wideCell( GetWideOpenIndex(selectedCell.row), 1 );
							UInt32 			nestingLevel = GetNestingLevel( wideCell.row );
							
							if ( nestingLevel == 0 )
								{
									outEnabled = true;
								}							
						}				
				}			
			break;
			
			case cmd_Diff:
			case cmd_AdvancedDiff:
				{				
					TableIndexT		numRows, numCols;
					
					GetTableSize(numRows, numCols);
					
					STableCell selectedCell = GetFirstSelectedCell();
					
					if ( IsValidCell(selectedCell) && selectedCell.row < numRows )		// can't diff on the first checkin
						{					
							STableCell 	wideCell( GetWideOpenIndex(selectedCell.row), 1 );
							UInt32 			nestingLevel = GetNestingLevel( wideCell.row );
							
							if ( nestingLevel > 0 )
								{
									outEnabled = true;	
								}							
						}						
				}
			break;
			
			default:
				LCommander::FindCommandStatus( inCommand, outEnabled, outUsesMark, outMark, outName );
				break;
			
			}
					
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark LHIERARCHY TABLE OVERRIDES


void
CMacCVSLogView::DrawSelf()
	{
		
		Rect	localFrameRect = {0, 0, 0, 0};
		
		CalcLocalFrameRect( localFrameRect );

		::EraseRect( &localFrameRect );

		LHierarchyTable::DrawSelf();
		
	}


void
CMacCVSLogView::DrawCell( const STableCell &inCell, const Rect &inLocalRect )
	{

		TableIndexT							theWideIndex 		= 0x00000000;	
		CMacCVSLogViewCellRec		*theLogEntry		= NULL;
		
		UInt32									cellSize				= sizeof(CMacCVSLogViewCellRec *),
														nestingLevel 		= 0x00000000;

		UInt16									newHeight				= 0x0000;

		Rect										insetCellFrame	= { 0, 0, 0, 0 };
		STableCell							wideCell( GetWideOpenIndex(inCell.row), 1 );

		DrawDropFlag( inCell, wideCell.row );
		
		nestingLevel = mCollapsableTree->GetNestingLevel( wideCell.row );
		
		GetCellData( wideCell, &theLogEntry, cellSize );
			
		Assert_( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) );
		
		if ( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) ) 
			{
	
//  Save and setup the text traits.
	
				StTextState theTextState;
				UTextTraits::SetPortTextTraits( kCellTextTraitsID );

//  Ok, since both the log information and comment text rows contain pointers to
//  the same data structure, the only way I can tell which one to draw is by
//  checking the nesting level. If it's 0, it's a log entry info thingy. If not,
//  the obviously it's a child and thus must be text. Neato keen, eh?

				if ( nestingLevel == 0 ) 
					{
		
//  Draw author.
		
						::MoveTo( inLocalRect.left + kCellAuthorTextOffset, inLocalRect.bottom - kCellTextOffsetFromBottom );
						::DrawString( theLogEntry->fAuthorPStr );
			
//  Draw version.
			
						::MoveTo( inLocalRect.left + kCellVersionTextOffset, inLocalRect.bottom - kCellTextOffsetFromBottom );
						::DrawString( theLogEntry->fVersionPStr );
			
//  Draw date.
			
						::MoveTo( inLocalRect.left + kCellDateTextOffset, inLocalRect.bottom - kCellTextOffsetFromBottom );
		
						TString<Str255> theRelDate;
						UMacCVSUtils::RelativeDateString( theLogEntry->fDate, theRelDate );
						::DrawString( theRelDate );
			
					}
				
				else 
					{
		
//  Else we're drawing comment text. Yay.

						insetCellFrame = inLocalRect;
					
						insetCellFrame.left = insetCellFrame.left + (nestingLevel * kCellChildOffset) + kCellTextOffset;
						insetCellFrame.top++;
						insetCellFrame.bottom--;
						insetCellFrame.right--;
							
						::TETextBox( theLogEntry->fCommentStr, theLogEntry->fCommentLen, &insetCellFrame, 0 );

					}
				
			}
	
		LHierarchyTable::DrawCell( inCell, inLocalRect );
		
	}


void
CMacCVSLogView::HiliteCellActively(	const STableCell &inCell, Boolean inHilite )
	{
		
		TableIndexT		theWideIndex			= 0x00000000;
		Rect					cellFrame 				= { 0, 0, 0, 0 };

		if ( GetLocalCellRect(inCell, cellFrame) && FocusExposed() ) 
			{

//  Get the wide open index for the row.

				theWideIndex = GetWideOpenIndex( inCell.row );

//  Get the nesting level.

				UInt32	nestingLevel = mCollapsableTree->GetNestingLevel( theWideIndex );

				cellFrame.left = cellFrame.left + nestingLevel * kCellChildOffset + kCellTextOffset;
		
//  Ok, this is major anal retentiveness. Since I inset my TE rect by one pixel, I need
//  to compensate on the indexes that are log headers. This ensures that the highlight
//  starts 2 pixels left of the text in either case.			
		
				if ( nestingLevel == 0 )
					cellFrame.left--;
	       
	      StColorPenState saveColorPen;   // Preserve color & pen state
        StColorPenState::Normalize();
        UDrawingUtils::SetHiliteModeOn();
				
				::InvertRect( &cellFrame );
			
			}
					
	}


void
CMacCVSLogView::HiliteCellInactively(	const STableCell &inCell, Boolean inHilite )
	{

		TableIndexT		theWideIndex			= 0x00000000;
		Rect					cellFrame 				= { 0, 0, 0, 0 };

		if ( GetLocalCellRect(inCell, cellFrame) && FocusExposed() ) 
			{

//  Get the wide open index for the row.

				theWideIndex = GetWideOpenIndex( inCell.row );

//  Get the nesting level.

				UInt32	nestingLevel = mCollapsableTree->GetNestingLevel( theWideIndex );

				cellFrame.left = cellFrame.left + nestingLevel * kCellChildOffset + kCellTextOffset;
		
//  Ok, this is major anal retentiveness. Since I inset my TE rect by one pixel, I need
//  to compensate on the indexes that are log headers. This ensures that the highlight
//  starts 2 pixels left of the text in either case.			
		
				if ( nestingLevel == 0 )
					cellFrame.left--;
	       
	      StColorPenState saveColorPen;   // Preserve color & pen state
        StColorPenState::Normalize();
        UDrawingUtils::SetHiliteModeOn();
				
				::PenMode(srcXor);
				::FrameRect(&cellFrame);
			
			}
		
	}


void
CMacCVSLogView::ExpandRow( TableIndexT inWideOpenRow )
	{

		TableIndexT							exposedRow			= 0x00000000;
		
		UInt16									rowHeight				= 0x0000,
														columnWidth 		= 0x0000;
		
		UInt32									nestingLevel 		= 0x00000000,
														cellSize				= sizeof( CMacCVSLogViewCellRec * );
		
		Rect										textRect			 	= { 0, 0, 0, 0 };
		
		CMacCVSLogViewCellRec		*theLogEntry 		= NULL;

		STableCell							wideCell(inWideOpenRow + 1, 1);

		LHierarchyTable::ExpandRow( inWideOpenRow );

//  This table class is really annoying in how it deals with wide vs. exposed indices.
//  Here I have to specify the exposed index to get the right cell to resize.	

		exposedRow = GetExposedIndex( inWideOpenRow + 1 );
		
		rowHeight = GetRowHeight( exposedRow );
		columnWidth = GetColWidth( 1 );
		
		GetCellData( wideCell, &theLogEntry, cellSize );
		
		Assert_( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) );
		
		if ( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) ) 
			{

//  Set the "expanded flag".

				if ( theLogEntry->fFlags == 0 )
					theLogEntry->fFlags = 1;

//  Recalculate text height if necessary. (slow)
			
				if ( rowHeight != theLogEntry->fCachedHeight ) 
					{
				
						nestingLevel = mCollapsableTree->GetNestingLevel( inWideOpenRow + 1 );
						
						textRect.top = 0;
						textRect.bottom = rowHeight;
						textRect.left = (nestingLevel * kCellChildOffset) + kCellTextOffset;
						textRect.right = columnWidth;

//  Inset the text rect on the top, right, and bottom sides.

						textRect.top++;
						textRect.bottom--;
						textRect.right--;
						
						theLogEntry->fCachedHeight = GetTextHeightGivenWidth( theLogEntry->fCommentStr, theLogEntry->fCommentLen, textRect ) + 2;
	
						SetRowHeight( theLogEntry->fCachedHeight, exposedRow, exposedRow );
					
					}
			
//  PowerPlant Hack: This is necessary for updating multi-geometry cells correctly.
//  This is not needed for PP version 1.7 in CW Pro 1.	

				#ifdef __PREPP17
					
					RefreshRowsBelow( inWideOpenRow );
				
				#endif
			
			}
				
	}


void
CMacCVSLogView::DeepExpandRow( TableIndexT inWideOpenRow )
	{
	
		TableIndexT		rowCount		= 0x00000000,
									columnCount	= 0x00000000;
									
		GetWideOpenTableSize( rowCount, columnCount );
	
		for ( TableIndexT index = 1; index <= rowCount; index++ )
			{
			
				if ( IsCollapsable(index) )
					if ( !IsExpanded(index) )
						ExpandRow( index );
			
			}
		
	}
	

void
CMacCVSLogView::CollapseRow( TableIndexT inWideOpenRow )
	{

		TableIndexT								exposedRow		= 0x00000000;
		UInt32										cellSize			= sizeof( CMacCVSLogViewCellRec * );
		CMacCVSLogViewCellRec			*theLogEntry 	= NULL;
		
		STableCell								wideCell(inWideOpenRow, 1);

		LHierarchyTable::CollapseRow( inWideOpenRow );
				
		GetCellData( wideCell, &theLogEntry, cellSize );
		
		Assert_( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) );
		
		if ( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) ) 
			{
			
				if ( (theLogEntry->fFlags > 0) )
					theLogEntry->fFlags = 0;
					
			}
				
	}


void
CMacCVSLogView::DeepCollapseRow( TableIndexT inWideOpenRow )
	{
	
		TableIndexT		rowCount		= 0x00000000,
									columnCount	= 0x00000000;
									
		GetWideOpenTableSize( rowCount, columnCount );
	
		for ( TableIndexT index = 1; index <= rowCount; index++ )
			{
			
				if ( IsCollapsable(index) )
					if ( IsExpanded(index) )
						CollapseRow( index );
			
			}
			
	}
	
	
void
CMacCVSLogView::ClickSelf( const SMouseDownEvent	&inMouseDown )
	{
	
		if ( !IsTarget() ) 
			{	
			
				SwitchTarget(this);
			
			}
		
		if ( IsTarget() ) 
			{
				
				LHierarchyTable::ClickSelf( inMouseDown );
				
			}
	
	}


void
CMacCVSLogView::ClickCell( const STableCell &inCell, const SMouseDownEvent &inMouseDown )
	{

		CMacCVSLogViewCellRec		*theLogEntry			= NULL;
		
		char										dateStr[256]			= "",
														timeStr[256]			= "",
														spacerStr[]				= ", ",
														*dataPtr					= NULL;
						
		UInt32									dataLen 					= 0x00000000,
														nestingLevel			= 0x00000000,
														cellSize					= sizeof( CMacCVSLogViewCellRec * );

		Rect										cellRect					= { 0, 0, 0, 0 };
		STableCell							wideCell( GetWideOpenIndex(inCell.row), 1 );
					
		nestingLevel = mCollapsableTree->GetNestingLevel( wideCell.row );
		
		GetCellData( wideCell, &theLogEntry, cellSize );
		
		Assert_( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) );

//  Check for double-click.
	
		if ( (theLogEntry != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) )
			{
			
				if ( GetClickCount() < 2 ) 
					{
				
						Boolean isDrag = ::WaitMouseMoved( inMouseDown.macEvent.where );
					
						if ( isDrag ) 
							{
					
//  If the nesting level is 0, we need to create a buffer and stuff
//  all the log entry header information (author, version, etc.) into it.
//  This buffer *MUST* be disposed afterward. If the nesting level is 1,
//  we just use the commentText buffer in our LogEntry struct. This buffer
//  *MUST NOT* be disposed of. (it will hose the drawing routines)			
			
								if ( nestingLevel == 0 ) 
									{
								
										::DateString( theLogEntry->fDate, abbrevDate, (StringPtr)dateStr, NULL );
										::TimeString( theLogEntry->fDate, false, (unsigned char*)timeStr, NULL );
										
										dataPtr = (char*) ::NewPtr(	theLogEntry->fAuthorPStr[0] + 1 + 
																								theLogEntry->fVersionPStr[0] + 1 + 
																								dateStr[0] + strlen(spacerStr) +
																								timeStr[0]);
									
										if ( dataPtr == NULL )
											throw( ::MemError() );
					
//  Since all of my data is stored in Pascal strings so I don't need to convert
//  them when I draw, I have to do it here. I think just BlockMoving everything
//  is faster than the ptocstr/sprintf crap. This is somewhat gross though.
					
										::BlockMoveData( theLogEntry->fAuthorPStr + 1, dataPtr, theLogEntry->fAuthorPStr[0] ); 
										dataLen = theLogEntry->fAuthorPStr[0];
					
										dataPtr[dataLen] = 0x20;
										dataLen++;
						
										::BlockMoveData( theLogEntry->fVersionPStr + 1, dataPtr + dataLen, theLogEntry->fVersionPStr[0] );
										dataLen = dataLen + theLogEntry->fVersionPStr[0];
						
										dataPtr[dataLen] = 0x20;
										dataLen++;
						
										::BlockMoveData( dateStr + 1, dataPtr + dataLen, dateStr[0] );
										dataLen = dataLen + dateStr[0];
						
										::BlockMoveData( spacerStr, dataPtr + dataLen, strlen(spacerStr) );
										dataLen = dataLen + strlen(spacerStr);
						
										::BlockMoveData( timeStr + 1, dataPtr + dataLen, timeStr[0] );
										dataLen = dataLen + timeStr[0];
						
									}
					
								else 
									{
					
										dataPtr = theLogEntry->fCommentStr;
										dataLen = theLogEntry->fCommentLen;
									
									}
				
						
								GetLocalCellRect( inCell, cellRect );
								cellRect.left = cellRect.left + nestingLevel * kCellChildOffset + kCellTextOffset;

//  See comment about the following 2 lines in HilightCellActively.				

								if ( nestingLevel == 0 )
									cellRect.left--;
						
								FocusDraw();
					
								LDragTask	theDragTask( inMouseDown.macEvent, cellRect, 1, 'TEXT', dataPtr, dataLen, NULL );
									
								if ( nestingLevel == 0 )
									::DisposePtr(dataPtr);

							}
				
					}
		
			}	
	
	}


	
void
CMacCVSLogView::AdaptToSuperFrameSize( Int32 inSurrWidthDelta, Int32 inSurrHeightDelta, Boolean inRefresh )
	{
	
		Rect	frameRect	= { 0, 0, 0, 0 };
		
		LTableView::AdaptToSuperFrameSize( inSurrWidthDelta, inSurrHeightDelta, inRefresh );
		
		CalcLocalFrameRect( frameRect );
		
		if ( (frameRect.right - frameRect.left) >= kDefaultColumnWidth )
		
			SetColWidth( frameRect.right - frameRect.left, 1, 1 );
			
		else SetColWidth( kDefaultColumnWidth, 1, 1 );
	
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark INDEX CONVERSION


CMacCVSLogViewCellRec *
CMacCVSLogView::ConvertWideIndexToCellRec( const TableIndexT inRow )
	{

		CMacCVSLogViewCellRec 	*resultPtr	= NULL;
		UInt32									cellSize 		= sizeof( CMacCVSLogViewCellRec * );
		STableCell							tableCell( inRow, 1 );
		
		if ( inRow > 0 ) 
			{
			
				GetCellData( tableCell, &resultPtr, cellSize );
	
				Assert_( (resultPtr != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *)) );
		
				if ( !((resultPtr != NULL) && (cellSize == sizeof(CMacCVSLogViewCellRec *))) )
					resultPtr = NULL;
					
			}
		
		return resultPtr;
		
	}


TableIndexT 
CMacCVSLogView::ConvertCellRecToWideIndex( const CMacCVSLogViewCellRec *inCell, const TableIndexT inParentIndex )
	{

		TableIndexT								result 						= 0x00000000,
															outRows						= 0x00000000, 
															outCols 					= 0x00000000,
															currentRow				= 0x00000000;

		CMacCVSLogViewCellRec			*theCurEntry 			= NULL;
		UInt32										cellSize 					= 0x00000000;
		
		Boolean										bDone							= false;
		
		STableCell 								currentCell( 0, 1 );

		Assert_( inCell != NULL );

		GetWideOpenTableSize( outRows, outCols );
		
		if ( inParentIndex < outRows )
	
			currentRow = inParentIndex;
			
		else currentRow = 1;
		
		for ( currentRow = 1; ((currentRow <= outRows) && (!bDone)); currentRow++ ) 
			{
			
				currentCell.row = currentRow;
				
				theCurEntry = NULL;
				cellSize = sizeof( CMacCVSLogViewCellRec * );
				
				GetCellData( currentCell, &theCurEntry, cellSize );
			
				if ( inCell == theCurEntry ) 
					{
						
						result = currentCell.row;
						
						bDone = true;

					}
				
			}
			
		if ( !bDone )
			result = kCMacCVSLogItemNoLongerExists;
		
		return result;
		
	}


#pragma mark -
#pragma mark PROJECT VIEW EVENTS


void
CMacCVSLogView::PostLogViewEvent( const CMacCVSLogViewEventRec	&inEvent )
	{

		Assert_( mViewEventQueue != NULL );
		
		if ( mViewEventQueue != NULL ) 
			{
			
				mViewEventQueue->InsertItemsAt( 1, LArray::index_Last, &inEvent );
					
			}
		
	}


Boolean
CMacCVSLogView::GetNextLogViewEvent( CMacCVSLogViewEventRec	&inEvent )
	{

		Boolean	bResult = false;
		
		Assert_( mViewEventQueue != NULL );
		
		if ( mViewEventQueue != NULL )
			if ( mViewEventQueue->GetCount() > 0 )
				{
				
					inEvent.fEventType = NULL;
					inEvent.fEventData = NULL;
					
					mViewEventQueue->FetchItemAt( LArray::index_First, &inEvent );
					mViewEventQueue->RemoveItemsAt( 1, LArray::index_First );
		
					bResult = true;
					
				}
				
		return bResult;

	}

/*
void
CMacCVSLogView::HandleExpandEvent( CMacCVSLogViewCellRec	*inExpandItem, const TableIndexT inParentIndex )
	{
	
	
	
	}
	
void
CMacCVSLogView::HandleExpandDeepEvent( CMacCVSLogViewCellRec	*inExpandItem )
	{
	
	
	
	}
	
void	
CMacCVSLogView::HandleCollapseEvent( CMacCVSLogViewCellRec *inCollapseItem, const TableIndexT inParentIndex )
	{
	
	
	}
	
void
CMacCVSLogView::HandleCollapseDeepEvent( CMacCVSLogViewCellRec	*inCollapseItem )
	{
	
	
	
	}
*/
	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


void
CMacCVSLogView::SortByAuthor(void)
	{

		CMacCVSLogViewEventRec	theEvent = { kCMacCVSLogViewSortByAuthor, NULL };
		
		if ( mCurrentSortMethod != kCMacCVSLogViewSortByAuthor ) 
		
			PostLogViewEvent( theEvent );

	}


void
CMacCVSLogView::SortByVersion(void)
	{

		CMacCVSLogViewEventRec	theEvent = {kCMacCVSLogViewSortByVersion, NULL};

		if ( mCurrentSortMethod != kCMacCVSLogViewSortByVersion ) 
		
			PostLogViewEvent( theEvent );

	}


void
CMacCVSLogView::SortByDate(void)
	{

		CMacCVSLogViewEventRec	theEvent = { kCMacCVSLogViewSortByDate, NULL };

		if ( mCurrentSortMethod != kCMacCVSLogViewSortByDate ) 
		
			PostLogViewEvent( theEvent );

	}


void
CMacCVSLogView::AddLogEntry( 	const Str255 			inAuthorPStr, 	
															const Str255 			inVersionPStr, 
															const UInt32 			inDate,			
															const char 				*inCommentStr,
															const UInt32			inCommentLen)
	{
		
		CMacCVSLogViewCellRec		*theNewEntry 	= NULL;
		UInt32									copyLen				= 0x00000000;
		
		try
			{
			
				theNewEntry = (CMacCVSLogViewCellRec *)::NewPtr( sizeof(CMacCVSLogViewCellRec) );
				
				if ( theNewEntry == NULL )
					throw( ::MemError() );
					
				::BlockMoveData( inAuthorPStr, theNewEntry->fAuthorPStr, inAuthorPStr[0] + 1 );
				::BlockMoveData( inVersionPStr, theNewEntry->fVersionPStr, inVersionPStr[0] + 1 );
				
				theNewEntry->fDate = inDate;
				theNewEntry->fCommentStr = NULL;
				theNewEntry->fCommentLen = 0x00000000;
				
				if ( inCommentLen > 0 )
					{
					
						copyLen = inCommentLen;
						
						if ( copyLen > kMaxCommentLen )
							copyLen = kMaxCommentLen;
							
						theNewEntry->fCommentStr = ::NewPtr( copyLen );
						
						if ( theNewEntry->fCommentStr == NULL )
							throw( ::MemError() );
							
						theNewEntry->fCommentLen = copyLen;
						
						::BlockMoveData( inCommentStr, theNewEntry->fCommentStr, copyLen );
					
					}
					
				theNewEntry->fCachedHeight = 0x0000;
				theNewEntry->fFlags = 0x0000;
					
				CMacCVSLogViewEventRec	newEvent = { kCMacCVSLogViewAddItemEvent, theNewEntry };
					
				PostLogViewEvent( newEvent );
					
			}
			
		catch ( ... )
			{
			
				if ( theNewEntry != NULL )
					{
					
						::DisposePtr( (Ptr)theNewEntry );
					
					}
					
				throw;
			
			}
		
	}


void
CMacCVSLogView::ThreadedTaskDied( const CThreadedTask *inTask )
	{
	
		Assert_( (inTask == mViewThread) && (mViewThread != NULL) );
		
		if ( inTask != NULL )
			if ( inTask == mViewThread )
				mViewThread = NULL;
	
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark IMPLEMENTATION


void
CMacCVSLogView::SortLogView( const UInt32 inSortType )
	{
					
		if ( inSortType != mCurrentSortMethod )
			{
			
				switch ( inSortType )
					{
									
						case CMacCVSLogView::kCMacCVSLogViewSortByAuthor:
						case CMacCVSLogView::kCMacCVSLogViewSortByVersion:
						case CMacCVSLogView::kCMacCVSLogViewSortByDate:
									
							mCurrentSortMethod = inSortType;
								
						break;
						
						default:

//  This should never happen.
						
							Assert_( false );
							
							mCurrentSortMethod = CMacCVSLogView::kCMacCVSLogViewSortByDate;
							
						break;
				
					}

				Assert_( mLogEntries != NULL );
				
				if ( mLogEntries != NULL )
					{
					
						BroadcastMessage( CSpinIndicator::msg_SpinWaitBegin );

						RemoveAllRows( false );
						Draw( NULL );
						
						mLogEntries->SetComparator( GetCurrentComparator() );
						RebuildLogView();

						BroadcastMessage( CSpinIndicator::msg_SpinWaitEnd );
						
					}
					
			}
	
	}
	
	
void
CMacCVSLogView::RebuildLogView( const Boolean inRefresh )
	{
		
		TableIndexT								rowCount					= 0x00000000,
															exposedRow				= 0x00000000;
						
		CMacCVSLogViewCellRec			*theCurEntry			= NULL;
										
		UInt32										cellSize					= sizeof(CMacCVSLogViewCellRec *);
		Rect											frameRect					= { 0, 0, 0, 0 };
			
		STableCell								expandedCell( 0, 1 );
		
		RemoveAllRows( false );
		
//  Erase the table for user feedback.     	

   	CalcLocalFrameRect( frameRect );
   	
   	FocusDraw();
   	::EraseRect( &frameRect );
				     	
//  Rebuild and expand items.                        
        
   	LArrayIterator	logItemIterator( *mLogEntries );
	 		
		rowCount = 0x00000000;
	  		  		
		while ( logItemIterator.Next(&theCurEntry) ) 
			{
	      	      	
      	if ( (sYieldCount % kDefaultYieldCount) == 0 ) 
      		{
      			
	      		CThreadedTask::Yield();
	      		
      		}
      		
      	sYieldCount++;
	      		     		
	      rowCount = InsertSiblingRows( 1, rowCount, &theCurEntry, sizeof(CMacCVSLogViewCellRec *), (theCurEntry->fCommentStr != NULL), inRefresh );
	 
	      if ( theCurEntry->fCommentStr != NULL ) 
	      	{

//  Now we keep the flag that tells if the item is expanded in the 
//  cell data information. This is much, much faster.
	  				      			
		     		if ( theCurEntry->fFlags == 0 )
		       		CollapseRow( rowCount );

		   			AddLastChildRow( rowCount, &theCurEntry, sizeof(CMacCVSLogViewCellRec *), false, inRefresh );

	  				
		   			if ( theCurEntry->fFlags > 0 ) 
		   				{
		   				
		   					exposedRow = GetExposedIndex( rowCount + 1 );
		   					
		    				SetRowHeight( theCurEntry->fCachedHeight, exposedRow, exposedRow );
		   				
		   				}
	   				
		 			}
		 			
		 	}
       	
//  Now we're done, force update on all the table.          
     	
		Refresh();

	}


LComparator *
CMacCVSLogView::GetCurrentComparator()
	{
	
		LComparator	*result	= NULL;
		
		switch ( mCurrentSortMethod ) 
			{
			
				case kCMacCVSLogViewSortByAuthor:
				
					result = new CMacCVSLogViewAuthorComparator;
					
				break;
				
				case kCMacCVSLogViewSortByVersion:
				
					result = new CMacCVSLogViewVersionComparator;
				
				break;
				
				case kCMacCVSLogViewSortByDate:

					result = new CMacCVSLogViewDateComparator;
				
				break;
				
				default:
				
					Assert_( false );
					
					result = new CMacCVSLogViewDateComparator;
			
				break;
			
			}
		
		return result;
	
	
	}
	
	
void
CMacCVSLogView::InsertLogEntry(	const CMacCVSLogViewCellRec	*inNewEntry, const Boolean inRefresh )
	{

		TableIndexT			rowCount					= 0x00000000,
										columnCount				= 0x00000000,
										theWideOpenIndex 	= 0x00000000;
		
		ArrayIndexT			indexInList				= 0x00000000;
							

		indexInList = mLogEntries->InsertItemsAt( 1, LArray::index_Last, &inNewEntry );
		
//  Add the log entry row item. If it contains crap, default to collapsed.
//  Note, we must add this in the correct sort order.				
		
		TableIndexT			siblingWideIndex	= 0x00000000;
		unsigned long		entryCount				= 0x00000000;
		
		if ( indexInList > 1 ) 
			{	
					
				while ( true ) 
					{
				
						siblingWideIndex++;
				
						if ( mCollapsableTree->GetNestingLevel(siblingWideIndex) == 0 ) 
							{
								
								entryCount++;
							
								if ( entryCount >= (indexInList - 1) )
									break;

							}
				
					}
			
			}
		
		siblingWideIndex = InsertSiblingRows( 1, siblingWideIndex, &inNewEntry, sizeof(CMacCVSLogViewEventRec *), (inNewEntry->fCommentStr != NULL), inRefresh );
				 
		if ( inNewEntry->fCommentStr != NULL ) 
			{

//  Call base method because we never called the inherited ExpandRow.
			
				LHierarchyTable::CollapseRow( siblingWideIndex );

//  If we actually have a comment text, insert a child row to
//  display the data.	

				AddLastChildRow( siblingWideIndex, &inNewEntry, sizeof(CMacCVSLogViewEventRec *), false, inRefresh );
				
			}

	}


UInt16
CMacCVSLogView::GetTextHeightGivenWidth( char *inTextStr, const UInt32 inTextLength, Rect &inTextRect )
	{

//  Hack! I use the TETextBox() toolbox call to actually draw the
//  comment word wrapped in a rect. Unfortunately, it doesn't compensate
//  for a rect too small so I have to find out the height of the text for
//  the particular rect. This is expensive time wise and generally gross.			

		TEHandle	textEditH		= NULL;
		UInt16		result 			= 0x0000,
							nLines			= 0x0000;
		
		textEditH = ::TENew( &inTextRect, &inTextRect );
		
		if ( textEditH != NULL ) 
			{	

//  Save and setup the text traits.

				StTextState	theTextState;
				UTextTraits::SetTETextTraits( kCellTextTraitsID, textEditH );

				::TESetText( inTextStr, inTextLength, textEditH );
				::TECalText( textEditH );
				
				nLines = (**textEditH).nLines;
				
				result = ::TEGetHeight( nLines, 1, textEditH );
						
				::TEDispose( textEditH );
				
			}
	
		return result;
	
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark LOG VIEW THREADING


CMacCVSLogViewThread::CMacCVSLogViewThread( CMacCVSLogView &inOwnerView ):

	CThreadedTask( CMacCVSLogView::kDefaultLogViewThreadStackSize, true )
	
	{
	
		mOwnerView = &inOwnerView;
	
	}
	
	
CMacCVSLogViewThread::~CMacCVSLogViewThread()
	{
	
		mOwnerView->ThreadedTaskDied( this );
	
	}
	
	
void
CMacCVSLogViewThread::Run()
	{
	
		CMacCVSLogViewEventRec		theEvent 	= { NULL, NULL };
		CMacCVSLogViewCellRec			*theItem	= NULL;
		
		Assert_( mOwnerView != NULL );
		
		if ( mOwnerView != NULL ) 
			{
				
				while ( true ) 
					{
				
						try
							{
							
								CThreadedTask::Yield();
								
								while ( mOwnerView->GetNextLogViewEvent(theEvent) )
									{
								
										switch ( theEvent.fEventType ) 
											{
																								
												case CMacCVSLogView::kCMacCVSLogViewSortByAuthor:
												case CMacCVSLogView::kCMacCVSLogViewSortByVersion:
												case CMacCVSLogView::kCMacCVSLogViewSortByDate:
												
													mOwnerView->SortLogView( theEvent.fEventType );
													
												break;
												
												case CMacCVSLogView::kCMacCVSLogViewAddItemEvent:
												
													theItem = (CMacCVSLogViewCellRec *)theEvent.fEventData;
														
													Assert_( theItem != NULL );
													
													if (theItem != NULL) 
														{
														
															mOwnerView->InsertLogEntry( theItem );

														}
														
												break;
														
											}
											
									}
						
							}
						
						catch ( CThreadedTaskException inErr )
							{
							
								throw inErr;
							
							}
					
						catch ( ... )
							{
							
							
							}
						
					} //  while
			
			}
								
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ATOMIC COMPARATORS

//  These are the "atomic" form of the comparators. Since they are static functions,
//  it allows me to do multiple degree comparisons in my real comparators. Note there
//  is no parameter checking here.

Int32 
CMacCVSLogView::CompareAuthor(const CMacCVSLogViewCellRec	*inItemOne, 
															const CMacCVSLogViewCellRec	*inItemTwo)
	{
	
		char	itemOneText[256]	= "",
					itemTwoText[256]	= "";
		
		Assert_( inItemOne != NULL );
		Assert_( inItemTwo != NULL );

  	if ( (sYieldCount % kDefaultYieldCount) == 0 ) 
  		{
  			
    		CThreadedTask::Yield();
    		
  		}
  		
  	sYieldCount++;
		
		::BlockMoveData( inItemOne->fAuthorPStr + 1, itemOneText, inItemOne->fAuthorPStr[0] );
		itemOneText[ inItemOne->fAuthorPStr[0] ] = 0x00;
		
		::BlockMoveData( inItemTwo->fAuthorPStr + 1, itemTwoText, inItemTwo->fAuthorPStr[0] );
		itemTwoText[ inItemTwo->fAuthorPStr[0] ] = 0x00;
		 	
		char	*currentCharacterPtr = itemOneText,
					currentCharacter;
								
		while (true)
			{
			
				currentCharacter = *currentCharacterPtr;
			
				if ( currentCharacter == 0 )
					break;
			
				if ( (currentCharacter >= 'A') && (currentCharacter <= 'Z') )
					*currentCharacterPtr = currentCharacter - 'A' + 'a';
			
				currentCharacterPtr++;
			
			}
		
		currentCharacterPtr = itemTwoText;

		while (true)
			{
			
				currentCharacter = *currentCharacterPtr;
			
				if ( currentCharacter == 0 )
					break;
			
				if ( (currentCharacter >= 'A') && (currentCharacter <= 'Z') )
					*currentCharacterPtr = currentCharacter - 'A' + 'a';
			
				currentCharacterPtr++;
			
			}

		return strcmp( itemOneText, itemTwoText );
		
	}


Boolean
CMacCVSLogView::IsEqualAuthor(	const CMacCVSLogViewCellRec	*inItemOne,
																const CMacCVSLogViewCellRec	*inItemTwo)
	{

		char	itemOneText[256] = "",
					itemTwoText[256] = "";

		Assert_( inItemOne != NULL );
		Assert_( inItemTwo != NULL );

  	if ( (sYieldCount % kDefaultYieldCount) == 0 ) 
  		{
  			
    		CThreadedTask::Yield();
    		
  		}
  		
  	sYieldCount++;

		::BlockMoveData( inItemOne->fAuthorPStr + 1, itemOneText, inItemOne->fAuthorPStr[0] );
		itemOneText[ inItemOne->fAuthorPStr[0] ];
		
		::BlockMoveData( inItemTwo->fAuthorPStr + 1, itemTwoText, inItemTwo->fAuthorPStr[0] );
		itemTwoText[ inItemTwo->fAuthorPStr[0] ];
		
		char	*currentCharacterPtr = itemOneText,
					currentCharacter;
								
		while (true)
			{
			
				currentCharacter = *currentCharacterPtr;
				
				if ( currentCharacter == 0 )
					break;
				
				if ( (currentCharacter >= 'A') && (currentCharacter <= 'Z') )
					*currentCharacterPtr = currentCharacter - 'A' + 'a';
				
				currentCharacterPtr++;
			
			}
		
		currentCharacterPtr = itemTwoText;

		while (true)
			{
			
				currentCharacter = *currentCharacterPtr;
			
				if ( currentCharacter == 0 )
					break;
			
				if ( (currentCharacter >= 'A') && (currentCharacter <= 'Z') )
					*currentCharacterPtr = currentCharacter - 'A' + 'a';
			
				currentCharacterPtr++;
			
			}
		
		return !strcmp( itemOneText, itemTwoText );
		
	}


Int32
CMacCVSLogView::CompareVersion(	const CMacCVSLogViewCellRec	*inItemOne,
																const CMacCVSLogViewCellRec	*inItemTwo)
	{
	
		SInt32	result 							= 0x00000000;
		UInt16	len									= 0x0000;

		char		version1CStr[256] 	= "",
						version2CStr[256] 	= "";

		Assert_( inItemOne != NULL );
		Assert_( inItemTwo != NULL );

  	if ( (sYieldCount % kDefaultYieldCount) == 0 ) 
  		{
  			
    		CThreadedTask::Yield();
    		
  		}
  		
  	sYieldCount++;
	
		if ( (inItemOne != NULL) && (inItemTwo != NULL) ) 
			{
			
				len = inItemOne->fVersionPStr[0];
				
				if (len > 255)
					len = 255;
					
				::BlockMoveData( &inItemOne->fVersionPStr[1], version1CStr, len );
				version1CStr[len] = 0x00;
				
				len = inItemTwo->fVersionPStr[0];
				
				if (len > 255)
					len = 255;
				
				::BlockMoveData( &inItemTwo->fVersionPStr[1], version2CStr, len );
				version2CStr[len] = 0x00;
				
				result = CUCompareCVSVersion( version1CStr, version2CStr );
	
			}

//  Reverse the result so greatest version comes first.
	
		result = result * -1; 
		
		return result;
		
	}


Boolean
CMacCVSLogView::IsEqualVersion( const CMacCVSLogViewCellRec	*inItemOne,
																const CMacCVSLogViewCellRec	*inItemTwo)
	{

		Boolean		bResult 						= false;
		UInt16		len									= 0x0000;

		char			version1CStr[256] 	= "",
							version2CStr[256] 	= "";

		Assert_( inItemOne != NULL );
		Assert_( inItemTwo != NULL );

  	if ( (sYieldCount % kDefaultYieldCount) == 0 ) 
  		{
  			
    		CThreadedTask::Yield();
    		
  		}
  		
  	sYieldCount++;
			
		if ( (inItemOne != NULL) && (inItemTwo != NULL) ) 
			{
			
				if ( inItemOne->fVersionPStr[0] == inItemTwo->fVersionPStr[0] )
					{
					
						len = inItemOne->fVersionPStr[0];
						
						if (len > 255)
							len = 255;
							
						::BlockMoveData( &inItemOne->fVersionPStr[1], version1CStr, len );
						version1CStr[len] = 0x00;
						
						len = inItemTwo->fVersionPStr[0];
						
						if (len > 255)
							len = 255;
						
						::BlockMoveData( &inItemTwo->fVersionPStr[1], version2CStr, len );
						version2CStr[len] = 0x00;
						
						bResult = ( !strcmp(version1CStr, version2CStr) );
			
					}
						
			}
			
		return bResult;
			
	}


Int32 
CMacCVSLogView::CompareDate(	const CMacCVSLogViewCellRec	*inItemOne, 
															const CMacCVSLogViewCellRec	*inItemTwo)
	{

 		Int32 result = 0x00000000;
		
		Assert_(inItemOne != NULL);
		Assert_(inItemTwo != NULL);

  	if ( (sYieldCount % kDefaultYieldCount) == 0 ) 
  		{
  			
    		CThreadedTask::Yield();
    		
  		}
  		
  	sYieldCount++;
		
		result = inItemTwo->fDate - inItemOne->fDate;
		
		return result;
		
	}


Boolean
CMacCVSLogView::IsEqualDate(	const CMacCVSLogViewCellRec	*inItemOne,
															const CMacCVSLogViewCellRec	*inItemTwo)
	{
		
		Boolean	result = false;

		Assert_(inItemOne != NULL);
		Assert_(inItemTwo != NULL);

  	if ( (sYieldCount % kDefaultYieldCount) == 0 ) 
  		{
  			
    		CThreadedTask::Yield();
    		
  		}
  		
  	sYieldCount++;
		
		if ( inItemOne->fDate == inItemTwo->fDate )
			result = true;
		
		return result;
		
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark LARRAY COMPARATORS


Int32
CMacCVSLogViewAuthorComparator::Compare( const void* inItemOne, const void* inItemTwo, Uint32 inSizeOne, Uint32 inSizeTwo ) const
	{
		
		CMacCVSLogViewCellRec 		*inItemOneCasted 	= *(CMacCVSLogViewCellRec **)inItemOne;
		CMacCVSLogViewCellRec 		*inItemTwoCasted 	= *(CMacCVSLogViewCellRec **)inItemTwo;
		Int32											result 						= 0x0000;

		Assert_( inItemOneCasted != NULL );
		Assert_( inItemTwoCasted != NULL );
		
		if ( (inItemOneCasted != NULL) && (inItemTwoCasted != NULL) ) 
			{
			
				result = CMacCVSLogView::CompareAuthor(inItemOneCasted, inItemTwoCasted);
			
				if (result == 0)
					result = CMacCVSLogView::CompareDate(inItemOneCasted, inItemTwoCasted);
					
				if (result == 0)
					result = CMacCVSLogView::CompareVersion(inItemOneCasted, inItemTwoCasted);
			
			}
		
		return result;
		
	}


Boolean
CMacCVSLogViewAuthorComparator::IsEqualTo( const void* inItemOne, const void* inItemTwo, Uint32 inSizeOne, Uint32 inSizeTwo ) const
	{
	
		CMacCVSLogViewCellRec 		*inItemOneCasted 	= *(CMacCVSLogViewCellRec **)inItemOne;
		CMacCVSLogViewCellRec 		*inItemTwoCasted 	= *(CMacCVSLogViewCellRec **)inItemTwo;
		Boolean										result						= true;
		
		Assert_( inItemOneCasted != NULL );
		Assert_( inItemTwoCasted != NULL );

		if ( (inItemOneCasted != NULL) && (inItemTwoCasted != NULL) )
			result = CMacCVSLogView::IsEqualAuthor(inItemOneCasted, inItemTwoCasted);
			
		return result;
		
	}


Int32
CMacCVSLogViewVersionComparator::Compare( const void* inItemOne, const void* inItemTwo, Uint32 inSizeOne, Uint32 inSizeTwo ) const
	{
		
		CMacCVSLogViewCellRec *				inItemOneCasted = *(CMacCVSLogViewCellRec **)inItemOne;
		CMacCVSLogViewCellRec *				inItemTwoCasted = *(CMacCVSLogViewCellRec **)inItemTwo;
		Int32													result 					= 0x00000000;

		Assert_( inItemOneCasted != NULL );
		Assert_( inItemTwoCasted != NULL );
		
		if ( (inItemOneCasted != NULL) && (inItemTwoCasted != NULL) ) 
			{
			
				result = CMacCVSLogView::CompareVersion(inItemOneCasted, inItemTwoCasted);
				
				if (result == 0)
					result = CMacCVSLogView::CompareAuthor(inItemOneCasted, inItemTwoCasted);
			
				if (result == 0)
					result = CMacCVSLogView::CompareDate(inItemOneCasted, inItemTwoCasted);
			
			}
		
		return result;

	}


Boolean
CMacCVSLogViewVersionComparator::IsEqualTo( const void* inItemOne, const void* inItemTwo, Uint32 inSizeOne, Uint32 inSizeTwo ) const
	{
		
		CMacCVSLogViewCellRec 	*inItemOneCasted 	= *(CMacCVSLogViewCellRec **)inItemOne;
		CMacCVSLogViewCellRec 	*inItemTwoCasted 	= *(CMacCVSLogViewCellRec **)inItemTwo;
		Boolean									result 						= true;
		
		Assert_( inItemOneCasted != NULL );
		Assert_( inItemTwoCasted != NULL );

		if ( (inItemOneCasted != NULL) && (inItemTwoCasted != NULL) )
			result = CMacCVSLogView::IsEqualVersion(inItemOneCasted, inItemTwoCasted);
			
		return result;
		
	}


Int32
CMacCVSLogViewDateComparator::Compare( const void* inItemOne, const void* inItemTwo, Uint32 inSizeOne, Uint32 inSizeTwo ) const
	{

		CMacCVSLogViewCellRec 		*inItemOneCasted 	= *(CMacCVSLogViewCellRec **)inItemOne;
		CMacCVSLogViewCellRec 		*inItemTwoCasted 	= *(CMacCVSLogViewCellRec **)inItemTwo;
		Int32											result 						= 0x0000;

		Assert_( inItemOneCasted != NULL );
		Assert_( inItemTwoCasted != NULL );
		
		if ( (inItemOneCasted != NULL) && (inItemTwoCasted != NULL) ) 
			{
			
				result = CMacCVSLogView::CompareDate(inItemOneCasted, inItemTwoCasted);
			
				if (result == 0)
					result = CMacCVSLogView::CompareAuthor(inItemOneCasted, inItemTwoCasted);
					
				if (result == 0)
					result = CMacCVSLogView::CompareVersion(inItemOneCasted, inItemTwoCasted);
			
			}
		
		return result;
		
	}


Boolean
CMacCVSLogViewDateComparator::IsEqualTo( const void* inItemOne, const void* inItemTwo, Uint32 inSizeOne, Uint32 inSizeTwo ) const
	{
		
		CMacCVSLogViewCellRec 		*inItemOneCasted 	= *(CMacCVSLogViewCellRec **)inItemOne;
		CMacCVSLogViewCellRec 		*inItemTwoCasted 	= *(CMacCVSLogViewCellRec **)inItemTwo;
		Boolean										result 						= true;
		
		Assert_( inItemOneCasted != NULL );
		Assert_( inItemTwoCasted != NULL );	
		
		if ( (inItemOneCasted != NULL) && (inItemTwoCasted != NULL) )
			result = CMacCVSLogView::IsEqualDate(inItemOneCasted, inItemTwoCasted);
			
		return result;
		
	}

