//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 	CMacCVSLogWindow.cp
//
//	  Author: Ram Rajadhyaksha
//
//################################################################################


#include <stdio.h>
#include <Scrap.h>
#include <UDesktop.h>
#include <LWindow.h>
#include <LTextButton.h>

#include "CSpinIndicator.h"
#include "CMacCVSLogView.h"
#include "CMacCVSLogWindow.h"


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS/DESTRUCTORS


CMacCVSLogWindow::CMacCVSLogWindow(LStream *inStream):

	CMacCVSThreadedWindow(inStream)
	
	{
		
		mLogView 				= NULL;
		
		mWindowPosResID	= kLogWindowPositionResID;
		
	}


CMacCVSLogWindow::~CMacCVSLogWindow()
	{
			
		mLogView->RemoveListener( mSpinnyThingy );
			

	}


CMacCVSLogWindow *
CMacCVSLogWindow::CreateCMacCVSLogWindowFromStream( LStream *inStream )
	{

		return new CMacCVSLogWindow( inStream );

	}


void
CMacCVSLogWindow::FinishCreateSelf()
	{

		Inherited::FinishCreateSelf();
		
		LTextButton		*theSortButton	= NULL;
		
		mLogView = dynamic_cast<CMacCVSLogView *>( FindPaneByID(CMacCVSLogView::class_ID) );
		ThrowIfNil_( mLogView );
				
//  Set up the text box buttons.	
		
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByDateButtonID) );
		ThrowIfNil_(theSortButton);
		
		theSortButton->AddListener( mLogView );
			
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByVersionButtonID) );
		ThrowIfNil_(theSortButton);

		theSortButton->AddListener( mLogView );
			
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByAuthorButtonID) );
		ThrowIfNil_(theSortButton);

		theSortButton->AddListener(mLogView);
		
//  Hook up the spinnythingymoboby.	
		Assert_(mSpinnyThingy != NULL);
		mLogView->AddListener( mSpinnyThingy );
					
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


void
CMacCVSLogWindow::Log( const FSSpec *inTargetItem, CMacCVSDoc *inOwnerDoc, UInt32 inDelayTicks )
	{

		Assert_( mWindowTask == NULL );
	
		TString<Str255>	titleString;
		
		mLogView->SetLogItem( *inTargetItem );
		
		if ( mWindowTask == NULL )
			{

				GetDescriptor( titleString );
				titleString = titleString + inTargetItem->name;
				SetDescriptor( titleString );

				Show();
							
				mWindowTask = new CMacCVSLogTask( this, inOwnerDoc, inTargetItem );
				ThrowIfNil_( mWindowTask );
				
				PrimeWindowTask(inDelayTicks);

			}
			
	}


void 
CMacCVSLogWindow::AddLogEntry(	const Str255 			inAuthorPStr, 	
																const Str255 			inVersionPStr, 
																const UInt32 			inDate,			
																const char 				*inCommentStr,
																const UInt32			inCommentLen)
	{
		
		Assert_( mLogView != NULL );
		
		if ( mLogView != NULL )
			mLogView->AddLogEntry( inAuthorPStr, inVersionPStr, inDate, inCommentStr, inCommentLen );

	}

