//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSLurkersTask.cp
	
	Authors:		Ram Rajadhyaksha, Simon Fraser
	
	Description:
	
		This file contains the code to find lurker files.
		 
	Modification History:

		10 Jan 1998			Ram Rajadhyaksha
		
					Made several changes for compatiblity with Entries files.
							
-------------------------------------------------------------------------------- */


#include <string.h>

#include <LArray.h>

#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "UErrorDialog.h"
#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"
#include "CMacCVSLurkersWindow.h"
#include "CMacCVSLurkersTask.h"

#include "CVSEngineUtils.h"

UInt32 CMacCVSLurkersTask::sYieldCount = 0;


CMacCVSLurkersTask::CMacCVSLurkersTask( const CMacCVSLurkersWindow *inOwnerWindow, const CMacCVSLurkersQueryRec *inQuery, const LArray *inTargetItems ):

	CBroadcastingThreadedTask( kDefaultLurkersThreadStackSize, true),
	mItemList( sizeof(FSSpec) )
		
	{
		try
			{		
				mOwnerWindow = NULL;
												
				ThrowIfNil_( inOwnerWindow );
				ThrowIfNil_( inQuery );
				ThrowIfNil_( inTargetItems );
				
				mOwnerWindow = const_cast<CMacCVSLurkersWindow *>( inOwnerWindow );						
				mQuery = *inQuery;
				
				if ( inTargetItems->GetCount() > 0 )
					mItemList = *inTargetItems;							
			}
		catch ( ... )
			{			
				throw;				
			}			
	}
	
	
CMacCVSLurkersTask::~CMacCVSLurkersTask()
	{
		mOwnerWindow->ThreadedTaskDied( this );		
	}
	
	
void
CMacCVSLurkersTask::RunSelf()
	{
		OSErr					result 				= noErr;		
		FSSpec				theCurItem	  = { 0, 0, "\p" };
		UInt32				ioCount				= sizeof( FSSpec );
		CInfoPBRec		pb;

		try
			{
				LArrayIterator	itemIterator( mItemList );
				
				while ( itemIterator.Next(&theCurItem, ioCount) )
					{		
						Assert_( ioCount == sizeof(FSSpec) );
						ioCount = sizeof( FSSpec );
				
						pb.hFileInfo.ioCompletion = NULL;
						pb.hFileInfo.ioFDirIndex 	= 0;
						pb.hFileInfo.ioNamePtr 		= theCurItem.name;
						pb.hFileInfo.ioVRefNum 		= theCurItem.vRefNum;
						pb.hFileInfo.ioDirID 			= theCurItem.parID;
				
						result = ::PBGetCatInfoSync( &pb );
						ThrowOSErr( result );
									
						if ( pb.dirInfo.ioFlAttrib & ioDirMask )
							{							
								FindLurkers( &pb );
								
								result = mEntries.CloseEntriesFile();
								Assert_( result == noErr );		
								ThrowOSErr( result );									
							}			
					}
			}						
		catch ( ... )
			{
				result = mEntries.CloseEntriesFile();
				Assert_( result == noErr );		
			}

	}
	
	
void
CMacCVSLurkersTask::FindLurkers( const CInfoPBRec *inParentPB )
	{
		
		OSErr							error						= noErr;
		UInt32						maxIndex				= 0;
		UInt32						intLen					= 0;

		Boolean						modified				= false;
		Boolean						displayFile			= false;
		Boolean						tagsEqual				= false;
		
		Str255						pstrRevision		= "\p";
		Str255						pstrTimeStamp		= "\p";
		Str255						pstrActualTime 	= "\p";
		Str255						pstrTag					= "\p";
		
		FSSpec						fsChild					= { 0, 0, "\p" };
		FSSpec						fsParent				= { 0, 0, "\p" };
			
		CInfoPBRec				pb;
		CkidResourcePB		*pCkidInfo			= NULL;	
	
		Assert_( inParentPB != NULL );
		
		if ( ::StackSpace() < kMinimumLurkersThreadStackSize )
			throw( (OSErr)memFullErr );
			
		try
			{			
				if ( inParentPB != NULL )
					{					
						pb = *inParentPB;	
						maxIndex = inParentPB->dirInfo.ioDrNmFls;

						fsParent.parID = inParentPB->hFileInfo.ioFlParID;
						fsParent.vRefNum = inParentPB->hFileInfo.ioVRefNum;
						PStrCopy( inParentPB->hFileInfo.ioNamePtr, fsParent.name, sizeof(fsParent.name) );

						if ( !mEntries.AreYouThisDirectory(&fsParent) )
							{
								error = mEntries.CloseEntriesFile();								
								Assert_( error == noErr );
									
								error = mEntries.LoadWorkingDirectory( &fsParent, fsRdPerm );
								Assert_( error == noErr);
							}						
							
//  First look at all the files in the directory.
													
						fsChild.vRefNum = pb.hFileInfo.ioVRefNum;
						pb.hFileInfo.ioNamePtr = fsChild.name;

						for ( UInt32 index = 1; index <= maxIndex; index++ )
							{							
								if ( (sYieldCount % kDefaultYieldCount) == 0 )
									CThreadedTask::Yield();
									
								sYieldCount++;
								
								modified = false;
								displayFile = false;
								
								pb.hFileInfo.ioDirID = inParentPB->hFileInfo.ioDirID;
								pb.hFileInfo.ioFDirIndex = index;
								
								error = ::PBGetCatInfoSync( &pb );
								
								if ( (error == noErr) && ((pb.dirInfo.ioFlAttrib & ioDirMask) == 0) )
									{									
										pCkidInfo = (CkidResourcePB *)::NewPtr(sizeof(CkidResourcePB));
						
										if ( pCkidInfo == NULL )
											throw( ::MemError() );

										fsChild.parID = pb.hFileInfo.ioFlParID;
											
										error = mEntries.GetEntry( fsChild.name, pstrRevision, pstrTimeStamp, NULL, pstrTag );
								
										if ( error == noErr )
											{
												error = FSpReadCkidResource( &fsChild, pCkidInfo, false );
										
												if ( error == noErr )
													{															
														if ( error == noErr )
															if ( pCkidInfo->fModifyReadOnly > 0 ) 
																modified = true;																	
													}	
													
												error = noErr;	
											}
											
										if ( (error == noErr) && (modified == false) )
											{
												intLen = 255;
												
												error = CUISOTimeStrFromSeconds( (char *)&pstrActualTime[1], &intLen, pb.hFileInfo.ioFlMdDat );
												Assert_( error == noErr );
												
												if ( error == noErr )
													pstrActualTime[0] = intLen;
													
												if ( !PStrCompare(pstrActualTime, pstrTimeStamp) )
													modified = true;
											}		
										
										if ( error == noErr )
											{
												if ( (mQuery.fFindOnlyMRO) && (modified) )											
													displayFile = true;
													
												else if ( (!mQuery.fFindOnlyMRO) && (!modified) )													
													displayFile = true;
													
												else displayFile = false;
												
												if ( displayFile )
													{														
														if ( mQuery.fTagPStr[0] == pstrTag[0] )
															{															
																tagsEqual = !strncmp( (char *)mQuery.fTagPStr, (char *)pstrTag, mQuery.fTagPStr[0] + 1 );																
															}
														else tagsEqual = false;													
													
														switch (mQuery.fMatchTag)
														{
															case kLurkersMatchIgnoreTag:
																displayFile = true;
																break;
														
															case kLurkersMatchMatchTag:
																displayFile = tagsEqual;
																break;														
														
															case kLurkersMatchDontMatchTag:
																displayFile = !tagsEqual;
																break;
														}														
													}
											
											}
										else if ( error == kCVSEngineFileHasNoCkid || error == eofErr || error == kCVSEngineNoRevisionDataErr )		
										// file has no resource fork
											{											
												if ( mQuery.fFindNewFiles )
													{
														FInfo			finderInfo;
														Boolean		ignoredType = false;
														
														error = ::FSpGetFInfo( &fsChild, &finderInfo );
														if ( error != noErr )
															throw( error );
														
														ignoredType = finderInfo.fdType == 'MPSY' ||			// Codewarrior SYM file
																					finderInfo.fdType == 'MMPF';				// CW debugger settings file
																					
														displayFile = !( FSpIsAlias(&fsChild) || ignoredType );
													
													}
											}
											
										if ( displayFile )
											{											
												mOwnerWindow->AddProjectItem( &fsChild );
											}
											
										if ( pCkidInfo != NULL )
											{											
												::DisposePtr( (Ptr)pCkidInfo );
												pCkidInfo = NULL;											
											}									
									}	
							
							}

//  Now recurse on the children folders.
							
						for ( UInt32 index = 1; index <= maxIndex; index++ )
							{
								if ( (sYieldCount % kDefaultYieldCount) == 0 )
									CThreadedTask::Yield();
									
								sYieldCount++;
							
								pb.hFileInfo.ioDirID = inParentPB->hFileInfo.ioDirID;
								pb.hFileInfo.ioFDirIndex = index;
								
								error = ::PBGetCatInfoSync( &pb );

								if ( (error == noErr) && (pb.dirInfo.ioFlAttrib & ioDirMask) )
									{								
										if ( CUIsRepositoryDir(pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID) )
											FindLurkers( &pb );										
									}								
							}				
					}
					
			}	
		catch ( ... )
			{										
				if ( pCkidInfo != NULL )
					{											
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;											
					}			
											
				throw;
			}
			
	}
	
