//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSLurkersView.cp
//	
//	  Authors: 	Ram Rajadhyaksha
//
//################################################################################


//  The Lurkers view borrows a lot of code and constants from CMacCVSProjectView.
//  So, if you can't find it defined here, look in CMacCVSProjectView.h.  

//  I originally wanted the Lurkers view to use the CMacCVSProjectView class.
//  However, the fact that the Lurkers view is a flat file display vs. hierarchial
//  in CMacCVSProjectView made it really complicated. (especially when dealing with
//  the thread that watches table items)

//  Currently the Lurkers view only supports file items.

//  Known bug: if a particular file in the lurkers window is changed in the 
//  Entries file, but the actual file and modification date is not change, 
//  then the lurkers window will not detect the change.

/*
		
	Modification History:

		21 Apr 1999		Josef W. Wankerl
		
					Added Reveal In Finder support.
		
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <Scrap.h>

#include <LArray.h>
#include <URegions.h>
#include <UException.h>
#include <UAttachments.h>
#include <UGAColorRamp.h>
#include <LTableMonoGeometry.h>
#include <LTableMultiSelector.h>
#include <LTableArrayStorage.h>
#include <LDropFlag.h>
#include <LTextButton.h>

#include "ThrowOSErr.h"
#include "MacCVSPPConstants.h"

#include "UErrorDialog.h"
#include "URegionUtilities.h"

#include "CSpinIndicator.h"
#include "CPathPopup.h"

#include "CVSEngineUtils.h"
#include "UMacCVSUtils.h"

#include "CMacCVSDoc.h"
#include "CMacCVSProjectViewItem.h"
#include "CMacCVSProjectViewFile.h"
#include "CMacCVSProjectViewFolder.h"

#include "CMacCVSLurkersViewSelector.h"
#include "UKeyScrollAttachments.h"
#include "CMacCVSProjectView.h"
#include "CMacCVSLurkersView.h"


#include "Find_icon.h"

UInt32	CMacCVSLurkersView::sYieldCount = 0;
Handle	CMacCVSLurkersView::sFileStatusIconSuites[kMacCVSProjectorTotalCount] = { 0, 0, 0, 0, 0, 0, 0 };


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS AND DESTRUCTORS	


CMacCVSLurkersView::CMacCVSLurkersView( LStream *inStream ): 

	LTableView( inStream )

	{
				
		OSErr	result = noErr;
		
		mWatchTime											= kDefaultBackgroundUpdateTime;
		mViewThread		 									= NULL;
		mViewEventQueue									= NULL;
		mItemList												= NULL;
		mCurrentSortMethod  						= kNameSorted;
		
		mProjectRootFileSpec.name[0]	 	= 0x00;
		mProjectRootFileSpec.vRefNum		= 0;
		mProjectRootFileSpec.parID			= 0;
		
		mUserDraggedSelection						= false;
		
		mBGColor.red										= 0;
		mBGColor.green									= 0;
		mBGColor.blue										= 0;
		
		try
			{
	
				mBGColor = UGAColorRamp::GetColor( colorRamp_Gray1 );
				SetForeAndBackColors( NULL, &mBGColor );

				mViewEventQueue = new LArray( sizeof(CMacCVSLurkersViewEventRec) );
				ThrowIfNil_( mViewEventQueue );
								
				mItemList = new LArray( sizeof(CMacCVSProjectViewItem *) );
				ThrowIfNil_( mItemList );
				
				mItemList->SetComparator( GetCurrentComparator() );
				mItemList->SetKeepSorted( true );
			
//  Load the status icons if they are not already available.
		
				if ( sFileStatusIconSuites[0] == NULL ) 
					{
					
						for ( UInt32 currentIcon = 0; currentIcon < kMacCVSProjectorTotalCount; currentIcon++ )
						
							::GetIconSuite( sFileStatusIconSuites + currentIcon, kFirstFileStatusIconSuiteID + currentIcon, svAllAvailableData );
						
					}

			}
		
		catch ( ... )
			{
								
				if ( mViewEventQueue != NULL )
					{
					
						delete mViewEventQueue;
						mViewEventQueue = NULL;
						
					}
					
				if ( mItemList != NULL )
					{
					
						delete mItemList;
						mItemList = NULL;
						
					}
					
				throw;
			
			}

	}


CMacCVSLurkersView::~CMacCVSLurkersView()
	{

		if ( mViewThread != NULL )
			{
			
				mViewThread->ShutdownThread();
				
				while ( mViewThread != NULL )
					CThreadedTask::Yield();
									
			}

		if ( mViewEventQueue != NULL )
			{
			
				delete mViewEventQueue;
				mViewEventQueue = NULL;
				
			}
			
		if ( mItemList != NULL )
			{
			
				LArrayIterator					itemIterator( *mItemList );
				CMacCVSProjectViewItem	*theCurItem = NULL;
				
				while ( itemIterator.Next(&theCurItem) )
					{
				
						delete theCurItem;
						
					}
						
				delete mItemList;
				mItemList = NULL;
				
			}
		
	}

	
void
CMacCVSLurkersView::FinishCreateSelf( void )
	{

		mTableGeometry = new LTableMonoGeometry( this, kDefaultRowWidth, kDefaultRowHeight );
	 	mTableSelector = new CMacCVSLurkersViewSelector( this );
		mTableStorage = new LTableArrayStorage( this, sizeof(CMacCVSProjectViewItem *) );

		// Insert a single column.
		InsertCols( 1, 0, nil, nil, false );
		
		//	Attachment for keyboard navigation
		AddAttachment( new ULurkersKeyScrollAttachment(this) );
			
		LTableView::FinishCreateSelf();
	
		mViewThread = new CMacCVSLurkersViewThread( *this );
		ThrowIfNil_( mViewThread );
					
		mViewThread->Resume();	
	
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS

	
void
CMacCVSLurkersView::SetProjectRoot( const FSSpec *inProjectRootSpec )
	{
	
		ThrowIfNil_( inProjectRootSpec );
		mProjectRootFileSpec = *inProjectRootSpec;
		
	}
	

void
CMacCVSLurkersView::UpdateProjectItem( const FSSpec *inTargetItem )
	{

		Assert_( inTargetItem != NULL );
		
		if ( inTargetItem != NULL )
			{

				CMacCVSLurkersViewEventRec theEvent = { kViewUpdateItemFSSpecEvent, 0, 0 };
										
				FSSpec	*theSpecPtr = (FSSpec *)::NewPtr( sizeof(FSSpec) );
				ThrowIfNil_( theSpecPtr );
				
				::BlockMoveData( inTargetItem, theSpecPtr, sizeof(FSSpec) );
				theEvent.fEventData = theSpecPtr;
				
				PostProjectViewEvent( theEvent );
				
			}

	}
	

UInt32
CMacCVSLurkersView::CountSelectedItems( void ) const
	{
		
		Int32 			theItemCount 	= 0x00000000;
		STableCell 	theCell( 0, 0 );
		
		while( GetNextSelectedCell(theCell) )
			theItemCount++;
			
		return theItemCount;
		
	}


LArray *
CMacCVSLurkersView::GetFlatSelection()
	{
	
		CMacCVSProjectViewItem		*theItem						= NULL;
		
		UInt32 										cellDataSize				= sizeof( CMacCVSProjectViewItem * );
		STableCell 								theCell( 0, 0 );
		
		LArray										*outSelectedFSSpecs	= NULL;
		FSSpec										theFileSpec					= { 0, 0, "\p" };
		
		outSelectedFSSpecs = new LArray( sizeof(FSSpec) );
		ThrowIfNil_( outSelectedFSSpecs );
		
		while ( GetNextSelectedCell(theCell) )
			{
			
				theItem = NULL;
				
				GetCellData( theCell, &theItem, cellDataSize );

				if ( (theItem != NULL) && (cellDataSize == sizeof(CMacCVSProjectViewItem *)) )
					{
					
						theItem->GetFSSpec( theFileSpec );
						outSelectedFSSpecs->InsertItemsAt( 1, LArray::index_Last, &theFileSpec );
							
					}
					
			}
			
		return outSelectedFSSpecs;
	
	}


void
CMacCVSLurkersView::AddProjectItem( const FSSpec *inItemFileSpec )
	{
	
		OSErr												error					= noErr;
		CMacCVSProjectViewFile			*theItem			= NULL;
		CMacCVSLurkersViewEventRec	theEvent			= { 0, 0, 0 };
		Boolean											isAlias				= false;
		FSSpec											fsParent 			= { 0, 0, "\p" };
		
		Str255											pstrRevision 	= "\p";
		Str255											pstrTimeStamp = "\p";
		Str255											pstrOptions		= "\p";
		Str255											pstrTag				= "\p";
		
		ThrowIfNil_( inItemFileSpec );
		
		try
			{
			
				error = FSpIsAFile( inItemFileSpec, NULL );
				ThrowOSErr( error );
				
				error = FSpGetParentFSSpec( inItemFileSpec, &fsParent );
				ThrowOSErr( error );
				
				if ( !mEntries.AreYouThisDirectory(&fsParent) )
					{
						error = mEntries.CloseEntriesFile();
						ThrowOSErr( error );
	
						error = mEntries.LoadWorkingDirectory( &fsParent, fsRdPerm );
						ThrowOSErr( error );
					}
					
				isAlias = ( FSpIsAlias( inItemFileSpec ) == kCVSEngineIsAlias );
				
				theItem = new CMacCVSProjectViewFile( NULL, inItemFileSpec, isAlias );
				ThrowIfNil_( theItem );
					
				error = mEntries.GetEntry( inItemFileSpec->name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
						
				if ( error != noErr )
					error = noErr;
				else
					theItem->SetEntry( pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
	
				theItem->ValidateContents( &mProjectRootFileSpec );
						
				theEvent.fEventType = kViewAddItemEvent;
				theEvent.fEventItem = theItem;
				
				PostProjectViewEvent( theEvent );
				
			}
			
		catch ( ... )
			{
			
				if ( theItem != NULL )
					delete theItem;
					
				throw;
				
			}
	
	}
	
	
void
CMacCVSLurkersView::ReleaseReadLocks()
	{	
		OSErr result = noErr;
		
		result = mEntries.CloseEntriesFile();
		ThrowOSErr( result );		
	}
	
	
void
CMacCVSLurkersView::ShowAllCells()
	{
		
		CMacCVSProjectViewItem	*theCurItem	= NULL;
		UInt32									maxIndex		= 0,
														index				= 0;
		Boolean									cellAdded		= false;
		
		maxIndex = mItemList->GetCount();		
		
		for ( index = 1; index <= maxIndex; index++ )
			{
			
				if ( mItemList->FetchItemAt(index, &theCurItem) )
					{

						if ( ! theCurItem->IsVisibleToUser() )
							{
								theCurItem->SetIsVisible( true );
								cellAdded = true;
							}
					
					}			
			}
			
			
		if ( cellAdded )
			{
		
				RemoveAllRows( false );
				
				RebuildSortedProjectView( true );

				Refresh();
			}
			
	}
	
	
	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark ITEM MANIPULATION


CMacCVSProjectViewItem *
CMacCVSLurkersView::ConvertTableIndexToItem( const TableIndexT inRow ) const
	{
				
		STableCell							tableCell( inRow, 1 );
		CMacCVSProjectViewItem	*theItem	= NULL;
		UInt32									cellSize 	= sizeof( CMacCVSProjectViewItem * );
	
		GetCellData( tableCell, &theItem, cellSize );
	
		if ( cellSize == sizeof(CMacCVSProjectViewItem *) )
			
			return theItem;
		
		else return NULL;
			
	}


TableIndexT 
CMacCVSLurkersView::ConvertItemToTableIndex( const CMacCVSProjectViewItem *inItem, const TableIndexT inStartIndex ) const
	{

		CMacCVSProjectViewItem		*theCurItem	= NULL;
	
		TableIndexT								outRows			= 0x00000000, 
															outCols			= 0x00000000,
															currentRow	= 0x00000000,
															startIndex	= inStartIndex;
				
		UInt32										cellSize 		= 0x00000000;

		STableCell 								currentCell( 0, 1 );

		if ( startIndex < 1 )
			startIndex = 1;
				
		GetTableSize( outRows, outCols );
		
		for ( currentRow = startIndex; currentRow <= outRows; currentRow++ ) 
			{
			
				currentCell.row = currentRow;
				
				theCurItem = NULL;

				cellSize = sizeof( CMacCVSProjectViewItem * );
				GetCellData( currentCell, &theCurItem, cellSize );
			
				if ( theCurItem == inItem )
					return currentCell.row;
					
			}

		return kProjectNoLongerExistsRow;
		
	}


TableIndexT
CMacCVSLurkersView::InsertProjectItem( CMacCVSProjectViewItem *inItem )
	{

		TableIndexT	index = 0;
	
		Assert_( inItem != NULL );
		
		if ( inItem != NULL )
			{
	
				index = mItemList->InsertItemsAt( 1, LArray::index_First, &inItem );
				InsertRows( 1, (index - 1), &inItem, sizeof(CMacCVSProjectViewItem *), true );
					
			}
			
		return index;
	
	}
	
	
TableIndexT
CMacCVSLurkersView::RemoveProjectItem( const CMacCVSProjectViewItem *inItem, const Boolean inRefresh )
	{

		TableIndexT itemIndex	= ConvertItemToTableIndex( inItem );
		
		if ( itemIndex != kProjectNoLongerExistsRow )
			{
			
				RemoveRows( 1, itemIndex, inRefresh );
			
			}
			
		return itemIndex;
			
	}


void
CMacCVSLurkersView::RefreshEntryData( CMacCVSProjectViewItem *inItem )
	{

//  This is a bit of a hack. Since entry information is no longer
//  stored with the file, we have to open the Entries file everytime
//  we need to validate an item. This sucks.

//  It remains to be seen how bad the performance is on this now that
//  we need to open the whole entries file everytime we want to update
//  one file.

//  You might as why I just don't leave mEntries open to some directory
//  and call CCVSEntries::AreYouThisDirectory on it. Well, I don't want
//  to leave a file open for every lurkers window and plus it would
//  make the view thread compete with the lurkers thread for the entries
//  object.

		OSErr		intResult				= noErr;
		Str255 	pstrRevision 		= "\p";
		Str255 	pstrTimeStamp 	= "\p";
		Str255 	pstrOptions 		= "\p";
		Str255 	pstrTag 				= "\p";

		FSSpec 	fsItem					= { 0, 0, "\p" };		
		FSSpec 	fsParent				= { 0, 0, "\p" };
		
		Assert_( inItem != NULL );
		ThrowIfNil_( inItem );

		CCVSEntries *theEntries = NULL;
		CMacCVSProjectViewFile *theItemCasted = NULL;
		
		try
			{
				theItemCasted = dynamic_cast<CMacCVSProjectViewFile *>(inItem);
				ThrowIfNil_( theItemCasted );

				inItem->GetFSSpec( fsItem );
				
				intResult = FSpGetParentFSSpec( &fsItem, &fsParent );
				ThrowOSErr( intResult );
				
				theEntries = new CCVSEntries;
				ThrowIfNil_( theEntries );
				
				intResult = theEntries->LoadWorkingDirectory( &fsParent, fsRdPerm );
				ThrowOSErr( intResult );

				intResult = theEntries->GetEntry( fsItem.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
				
				Assert_( (intResult == noErr) || (intResult == kCVSEngineNoRevisionDataErr) );
		
				if ( intResult == noErr )
					{
						theItemCasted->SetEntry( pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
					}
				else
					{
						theItemCasted->SetEntry( NULL, NULL, NULL, NULL );
					}
			}
		catch ( ... )
			{
				if ( theItemCasted != NULL )
					{
						theItemCasted->SetEntry( NULL, NULL, NULL, NULL );
					}
			}
			
		try
			{
				if ( theEntries != NULL )
					{
						delete theEntries;
						theEntries = NULL;
					}
			}
		catch ( ... )
			{
			
			}
			
	}
	
	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark DRAWING

	
void
CMacCVSLurkersView::DrawSelf()
	{
	
		Rect theFrame = { 0, 0, 0, 0 };
		
		CalcLocalFrameRect( theFrame );
		
		::EraseRect( &theFrame );
	
		LTableView::DrawSelf();
		
	}
	

void
CMacCVSLurkersView::DrawCell( const STableCell &inCell,const Rect &inLocalRect )
	{
		
		CMacCVSProjectViewItem 		*theItem				= NULL;
		UInt32										relativeDate		= 0x00000000;
		
		Str255										cellName				= "\p";
		Rect 											statusIconRect	= { 0, 0, 0, 0 };
		ProjectItemStatus					itemStatus			= kMacCVSProjectorStatusUnknown;
		Rect											rTemp					= inLocalRect;
		
		theItem = ConvertTableIndexToItem( inCell.row );
		Assert_( theItem != NULL );
		
		if ( theItem != NULL )
			{

				rTemp.top = rTemp.bottom - 1;
				
				::ForeColor( whiteColor );
				::PaintRect( &rTemp );

				::ForeColor( blackColor );
				
				DrawCellIcon( inCell, inLocalRect, CellIsSelected(inCell) );

//  Save and setup the text traits.

				StTextState	theTextState;
				
				if ( theItem->IsAlias() )
					UTextTraits::SetPortTextTraits( kCellTextTraitsItalicID );
				else
					UTextTraits::SetPortTextTraits( kCellTextTraitsID );

				::MoveTo( inLocalRect.left + kCellTextOffset, 
									inLocalRect.bottom - kCellTextOffsetFromBottom );
						
				theItem->GetNamePStr( cellName );
				::DrawString( cellName );

				if ( theItem->IsAlias() )
					UTextTraits::SetPortTextTraits(kCellTextTraitsID);

//	Draw the status icon
				
				itemStatus = theItem->GetProjectItemStatus();

				if ( (itemStatus >= kMacCVSProjectorStatusUnowned) && (itemStatus < kMacCVSProjectorTotalCount) ) 
					{
					
					 	statusIconRect.top = inLocalRect.top + 1;
					 	statusIconRect.left = inLocalRect.left + kFileStatusIconOffset;
					 	statusIconRect.right = statusIconRect.left + kFileStatusIconWidth;
					 	statusIconRect.bottom = statusIconRect.top + kFileStatusIconHeight;

						::PlotIconSuite( &statusIconRect, kAlignNone, kTransformNone, sFileStatusIconSuites[itemStatus] );
					
					}

				if ( theItem->IsLocked() )
					{
					 	statusIconRect.top = inLocalRect.top + 1;
					 	statusIconRect.left = inLocalRect.left + kFileLockIconOffset;
					 	statusIconRect.right = statusIconRect.left + kFileLockIconWidth;
					 	statusIconRect.bottom = statusIconRect.top + kFileLockIconHeight;
		
						::PlotIconID( &statusIconRect, kAlignNone, kTransformNone, kCellLockIconID );
					}
				
//	Draw the version string for the file.

				if ( itemStatus > kMacCVSProjectorStatusUnowned ) 
					{
					
						CMacCVSProjectViewFile	*projectFile 	= dynamic_cast<CMacCVSProjectViewFile *>( theItem );
						Str255									versionString	= "\pN/A";

						if ( projectFile != NULL )
							{

								::MoveTo(	inLocalRect.left + kCellVersionTextOffset, 
													inLocalRect.bottom - kCellTextOffsetFromBottom );
								
								projectFile->GetVersionPStr( versionString );
						
								if ( itemStatus >= kMacCVSProjectorStatusBroken ) 
									{						
										RGBColor grey = { 0x8000, 0x8000, 0x8000 };
										RGBForeColor( &grey );								
									}
								else if ( CUIsAddedOrRemovedRevision(versionString) )
									{
										RGBColor red = { 0xFFFF, 0x0000, 0x0000 };
										RGBForeColor( &red );
									}
								
								::DrawString( versionString );
								::ForeColor( blackColor );
								
								::MoveTo(	inLocalRect.left + kCellTagTextOffset, 
											inLocalRect.bottom - kCellTextOffsetFromBottom );

								projectFile->GetStickyTagPStr( versionString );
	
								Assert_( versionString[0] != 1 );
																
								if ( versionString[0] > 1 )
									{
									
										versionString[0]--;
										::BlockMoveData( &versionString[2], &versionString[1], versionString[0] );
										
										::DrawString(versionString);
								
									}
	
								::ForeColor( blackColor );
								
							}
						
					}
				
				::MoveTo(	inLocalRect.left + kCellModificationTextOffset, inLocalRect.bottom - kCellTextOffsetFromBottom );
				
				TString<Str255> theRelDate;
				UMacCVSUtils::RelativeDateString( theItem->GetCachedModificationDate(), theRelDate );
				
				::DrawString(theRelDate);
				
			}

		LTableView::DrawCell( inCell, inLocalRect );
					
	}


void
CMacCVSLurkersView::DrawCellIcon( const STableCell &inCell, const Rect &inLocalRect, Boolean inHilite )
	{

		CMacCVSProjectViewItem *theItem = ConvertTableIndexToItem( inCell.row );
		Assert_( theItem != NULL );
		
		if ( theItem != NULL ) 
			{
				Rect 		theIconFrame 	= { 0, 0, 16, 16 };
				
				::OffsetRect(	&theIconFrame, inLocalRect.left + kCellIconOffset, inLocalRect.top + 1 );
				
				IconTransformType theTransform = kTransformNone;
				
				if ( inHilite )
					theTransform += kTransformSelected;
				
				Handle	theIconSuite 	= NULL;
				FSSpec	theItemSpec;
				
				theItem->GetFSSpec(theItemSpec);
				
#ifdef FASTER_FIND_ICON
				if ( Find_icon( &theItemSpec, theItem->GetFileInfoRecPtr(), kSelectorSmallThenLarge, &theIconSuite) == noErr)
#else
				if ( Find_icon( &theItemSpec, NULL, kSelectorSmallThenLarge, &theIconSuite) == noErr)
#endif
					::PlotIconSuite(  &theIconFrame, kAlignNone, theTransform, theIconSuite );
				else
					{
						ResIDT 	theIconID 		= theItem->IsContainer() ? kCellFolderIconID : kCellFileIconID;
						::PlotIconID( &theIconFrame, kAlignNone, theTransform, theIconID );
					}
					
					
				if ( theIconSuite != NULL) {
					::DisposeIconSuite (theIconSuite, true);
					theIconSuite = NULL;
				}
				
			}	
					
	}


void
CMacCVSLurkersView::HiliteSelection( Boolean	inActively, Boolean inHilite )
	{

		STableCell theCell	= 0;
		
		while ( GetNextSelectedCell(theCell) ) 
			{
			
				if ( inActively ) 
				
					HiliteCellActively( theCell, inHilite );
				
				else HiliteCellInactively( theCell, inHilite );
				
			}
		
	}


void
CMacCVSLurkersView::HiliteCellActively( const STableCell &inCell, Boolean inHilite )
	{
		
		Rect 	cellFrame = { 0, 0, 0, 0 };
				
		if ( GetLocalCellRect(inCell, cellFrame) && FocusExposed() ) 
			{

				DrawCellIcon( inCell, cellFrame, inHilite );

				Rect theTextFrame = { 0, 0, 0, 0 };
				
				CalcCellTextFrame( inCell, theTextFrame );
				
				::InsetRect(&theTextFrame, 0, 1);
	
				::RGBBackColor( &mBGColor );
				
				StRegion theRegion(theTextFrame);
				URegionUtils::Hilite(theRegion);
				
			}
			
	}


void
CMacCVSLurkersView::HiliteCellInactively( const STableCell &inCell, Boolean inHilite )
	{

		HiliteCellActively( inCell, inHilite );
			
	}


void 
CMacCVSLurkersView::CalcCellTextFrame( const STableCell &inCell, Rect &outLocalFrame )
	{
	
//  Start with the whole cell
		
		GetLocalCellRect( inCell, outLocalFrame );
		outLocalFrame.top += 2;
		outLocalFrame.bottom -= 2;
		
		CMacCVSProjectViewItem *theItem = ConvertTableIndexToItem( inCell.row );
		Assert_( theItem != NULL );
		
		if ( theItem != NULL )
			{
			
				TString<Str255> theName;
				theItem->GetNamePStr( theName );
				
				StTextState	theTextState;
				UTextTraits::SetPortTextTraits( kCellTextTraitsID );
				
				outLocalFrame.left = outLocalFrame.left + kCellTextOffset - 1;
				outLocalFrame.right = outLocalFrame.left + ::StringWidth(theName) + 1;

			}
			
	}


void
CMacCVSLurkersView::CalcCellHotSpots( const STableCell &inCell, Rect &outIconFrame, Rect &outTextFrame )
	{

		Rect		localRect			= { 0, 0, 0, 0 },
						iconFrame			= { 0, 0, 16, 16 };
				
//  Start with the whole cell

		GetLocalCellRectAnywhere( inCell, localRect );
		
		outTextFrame = localRect;
		
		outTextFrame.top += 2;
		
		CMacCVSProjectViewItem* theItem = ConvertTableIndexToItem( inCell.row );
		
		if ( theItem != NULL )
			{
			
				TString<Str255> theName;
				theItem->GetNamePStr( theName );
				
				StTextState	theTextState;
				UTextTraits::SetPortTextTraits( kCellTextTraitsID );
				
				outTextFrame.left = outTextFrame.left + kCellTextOffset - 1;
				outTextFrame.right = outTextFrame.left + ::StringWidth(theName) + 1;

				::OffsetRect(	&iconFrame, localRect.left + kCellIconOffset, localRect.top + 1 );

				outIconFrame = iconFrame;
		
			}
							
	}


void
CMacCVSLurkersView::HideSelf()
	{
	
		if ( IsOnDuty() ) 
			{				//  Shouldn't be on duty when invisible
			
				SwitchTarget( GetSuperCommander() );
		
			}
			
		LTableView::HideSelf();
	
	}
	

void
CMacCVSLurkersView::AdaptToNewSurroundings()
	{
	
		Rect		rFrameRect 	= { 0, 0, 0, 0 };
		UInt32	intRowWidth = 0;
		
		LTableView::AdaptToNewSurroundings();
		
		CalcLocalFrameRect( rFrameRect );
		
		if ( (rFrameRect.right - rFrameRect.left) > kDefaultRowWidth )
			intRowWidth = rFrameRect.right - rFrameRect.left;
		else
			intRowWidth = kDefaultRowWidth;
		
		Assert_( mTableGeometry != NULL );
		
		if ( mTableGeometry != NULL )	
			mTableGeometry->SetColWidth( intRowWidth, 0, 0 );
		
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark SELECTION


void
CMacCVSLurkersView::ClickSelf( const SMouseDownEvent &inMouseDown )
	{
	
		if ( !IsTarget() ) 
			{	
			
				SwitchTarget(this);
			
			}
		
		if ( IsTarget() ) 
			{

				SPoint32		imagePt = { 0, 0 };
				STableCell	hitCell( 0, 0 );
				
				LTableView::ClickSelf( inMouseDown );

				LocalToImagePoint( inMouseDown.whereLocal, imagePt );
	
				if ( !GetCellHitBy(imagePt, hitCell) ) 
					{

						mTableSelector->DragSelect( hitCell, inMouseDown );
						
					}
				
			}
	
	}
	

void
CMacCVSLurkersView::ClickCell( const STableCell &inCell, const SMouseDownEvent &inMouseDown )
	{
		
		Boolean	isOptionKeyDown 	= (inMouseDown.macEvent.modifiers & optionKey) != 0,
						isCommandKeyDown	= (inMouseDown.macEvent.modifiers & cmdKeyBit) != 0;

		LTableView::ClickCell( inCell, inMouseDown );
						
//  If the option key is down, display the cool path popup.
		
		if ( isOptionKeyDown  ) 
			{

				CMacCVSProjectViewItem		*theClickItem 		= NULL;
				Rect 											theCellFrame 			= { 0, 0, 0, 0 };
				FSSpec 										theSpec						= { 0, 0, "\p" };
				Point 										theGlobalPoint		= { 0, 0 };

				theClickItem = ConvertTableIndexToItem( inCell.row );
				Assert_( theClickItem != NULL );
				
				if ( theClickItem != NULL )
					{
					
						CalcCellTextFrame( inCell, theCellFrame );
						
						theClickItem->GetFSSpec( theSpec );

//  Yep, they're magic constants.  These are used to align the text in the first item
//  in the popup with the item's name text.  We could do a better calculation based on
//  font info.
				
						theGlobalPoint.h = theCellFrame.left - 11;
						theGlobalPoint.v = theCellFrame.top + 2;
						
						LocalToPortPoint(theGlobalPoint);
						PortToGlobalPoint(theGlobalPoint);

						CPathPopup thePathPopup( theSpec, kCellTextTraitsID, true, true, true );
						thePathPopup.Track( theGlobalPoint );
						
					}
				
			}
			
		else if ( GetClickCount() > 1 )
			{
					
				CMacCVSProjectViewItem		*theCurItem 		= NULL;
				STableCell								theCurCell( 0, 0 );
				
				while ( GetNextSelectedCell(theCurCell) )
					{
					
						theCurItem = ConvertTableIndexToItem( theCurCell.row );
						Assert_( theCurItem != NULL );
						
						if ( theCurItem != NULL )
							{
							
								Assert_( !theCurItem->IsContainer() );
								
								if ( !theCurItem->IsContainer() ) 
									theCurItem->OpenWithFinder();
									
							}
									
					}
					
			}
						
	}
	
	
Boolean
CMacCVSLurkersView::ClickSelect( const STableCell &inCell, const SMouseDownEvent	&inMouseDown )
	{

		Boolean 	result					= false;
		Boolean 	isOptionKeyDown	= ( (inMouseDown.macEvent.modifiers & optionKey) != 0 );
		
		Rect 	theTextFrame = { 0, 0, 0, 0 },
					theIconFrame = { 0, 0, 0, 0 };
		
		CalcCellHotSpots( inCell, theIconFrame, theTextFrame );
		
		theIconFrame.top--;
		theIconFrame.bottom++;
		theIconFrame.left -= 2;
		theIconFrame.right += 2;
					
		if ( ::PtInRect(inMouseDown.whereLocal, &theIconFrame) || ::PtInRect(inMouseDown.whereLocal, &theTextFrame) ) 
			{
	
				if ( !isOptionKeyDown )
					{
					
						mUserDraggedSelection = false;
						
						LTableView::ClickSelect( inCell, inMouseDown );
				
						result = !mUserDraggedSelection;
						
					}
					
				else result = true;
				
			}
		
		else if (inMouseDown.macEvent.modifiers & cmdKey) 
			{
// handle command-drag select like good people
					mTableSelector->DragSelect( inCell, inMouseDown );
			
			}

		else if ( !(inMouseDown.macEvent.modifiers & shiftKey) ) 
			{

//  Is this the right place for this? - rvr
					
				UnselectAllCells();
				
				mTableSelector->DragSelect( inCell, inMouseDown );
		
			}
		
		return result;
		
	}


void
CMacCVSLurkersView::GetCellIconRgn( const STableCell &inCell, Boolean inHilite, RgnHandle ioRgn )
	{

		Rect	localCellRect = { 0, 0, 0, 0 };
		
		CMacCVSProjectViewItem *theItem = ConvertTableIndexToItem( inCell.row );
		Assert_( theItem != NULL );
		
		if ( theItem != NULL )
			{

				GetLocalCellRect( inCell, localCellRect );

				ResIDT 	theIconID 		= theItem->IsContainer() ? kCellFolderIconID : kCellFileIconID;
				Rect 		theIconFrame	= { 0, 0, 16, 16 };
			
				::OffsetRect( &theIconFrame, localCellRect.left + kCellIconOffset, localCellRect.top + 1 );
			
				IconTransformType theTransform = ttNone;
				
				if ( inHilite )
					theTransform += ttSelected;
					
				::IconIDToRgn( ioRgn, &theIconFrame, atNone, theIconID );
				
			}	
			
	}


// Get the cell rect whether it is visible or not
void
CMacCVSLurkersView::GetLocalCellRectAnywhere( const STableCell	&inCell, Rect &outCellRect ) const
	{
		Int32		cellLeft, cellTop, cellRight, cellBottom;
		
		mTableGeometry->GetImageCellBounds(inCell, cellLeft, cellTop, cellRight, cellBottom);
		
		SPoint32	imagePt;
		imagePt.h = cellLeft;
		imagePt.v = cellTop;
		ImageToLocalPoint(imagePt, topLeft(outCellRect));
		outCellRect.right = outCellRect.left + (cellRight - cellLeft);
		outCellRect.bottom = outCellRect.top + (cellBottom - cellTop);

	}


void
CMacCVSLurkersView::GetHiliteRgn( RgnHandle ioHiliteRgn )
	{

		RgnHandle		cellRgn				= NULL,
								tempRgn				= NULL;
								
		Rect				visRect 			= { 0, 0, 0, 0 },
								cellTextRect 	= { 0, 0, 0, 0 };
		
		try
			{
			
				::EmptyRgn( ioHiliteRgn );
				
				GetRevealedRect( visRect );
				
				if ( !::EmptyRect(&visRect) ) 
					{
					
						PortToLocalPoint( topLeft(visRect) );
						PortToLocalPoint( botRight(visRect) );
						
						STableCell	topLeftCell, botRightCell;
						FetchIntersectingCells( visRect, topLeftCell, botRightCell );
						
						cellRgn = ::NewRgn();
						ThrowIfNil_( cellRgn );
						
						tempRgn = ::NewRgn();
						ThrowIfNil_( tempRgn );
						
						STableCell	cell;
						
						for ( cell.row = topLeftCell.row; cell.row <= botRightCell.row; cell.row++ ) 
							{
							
								for ( cell.col = topLeftCell.col; cell.col <= botRightCell.col; cell.col++ ) 
									{
									
										if ( CellIsSelected(cell) ) 
											{
											::EmptyRgn( cellRgn );
																			
												GetCellIconRgn( cell, CellIsSelected(cell), cellRgn );
												CalcCellTextFrame( cell, cellTextRect );
															
												::RectRgn( tempRgn, &cellTextRect );
												
												::UnionRgn( tempRgn, cellRgn, tempRgn );
												URegionUtils::MakeRegionOutline( tempRgn, &tempRgn );
												::UnionRgn( tempRgn, ioHiliteRgn, ioHiliteRgn);
											}
											
									}
									
							}
						
					}

				if ( cellRgn != NULL )
					{
						::DisposeRgn( cellRgn );
						cellRgn = NULL;
						
					}

				if ( tempRgn != NULL )
					{
						::DisposeRgn( tempRgn );
						tempRgn = NULL;
						
					}
					
			}
			
		catch ( ... )
			{
			
				if ( cellRgn != NULL )
					{
					
						::DisposeRgn( cellRgn );
						cellRgn = NULL;
						
					}

				if ( tempRgn != NULL )
					{
					
						::DisposeRgn( tempRgn );
						tempRgn = NULL;
						
					}
			
				throw;
				
			}

	}


// so that we update commands when keying through lists
void
CMacCVSLurkersView::SelectionChanged()
	{
		LCommander::SetUpdateCommandStatus(true);
	}

//################################################################################
//################################################################################
#pragma mark -
#pragma mark COMMAND HANDLING


Boolean
CMacCVSLurkersView::CheckIfAnyItemsQualify( CommandT inCommand )
	{
	
		Boolean										result				= false;
		CMacCVSProjectViewItem		*theItem 			= NULL;
		UInt32										cellDataSize 	= sizeof( CMacCVSProjectViewItem * );
		STableCell								theCell( 0, 0 );
		
		while ( GetNextSelectedCell(theCell) && (!result) )
			{

				theItem = NULL;
						
				GetCellData( theCell, &theItem, cellDataSize );
						
				ThrowIfNil_( theItem );
						
				switch ( inCommand )
					{

						// JWW - Added Reveal In Finder case
						case cmd_RevealInFinder:
						
							result = true;
							
						break;
						
						case cmd_Update:
						case cmd_UpdateTo:
						
							result = theItem->CanUpdate();
		
						break;
					
						case cmd_Checkout:
						case cmd_CheckoutTo:
							
							result = theItem->CanCheckout();
							
						break;
						
						case cmd_AddFile:
						
							result = theItem->CanBeAdded();
							
						break;
						
						case cmd_Commit:
						
							result = theItem->CanCheckin();
							
						break;

						case cmd_Editors:
							//  Because "Show Editors" works on the same
							//  criteria of files/folder as Watchers, we have this
							//  OR statement.
							result = (theItem->CanWatch() || (theItem->CanEdit()));
						break;
						
						case cmd_Edit:
						case cmd_UnEdit:
							result = theItem->CanEdit();
						break;
						
						case cmd_ShowEditors:
						case cmd_Watchers:
						case cmd_WatchOn:
						case cmd_WatchOff:
						case cmd_WatchAdd:
						case cmd_WatchRemove:
						case cmd_ShowWatchers:
							result = theItem->CanWatch();
						break;

						case cmd_RevertFile:
						
							result = theItem->CanRevert();
							
						break;
										
						case cmd_ModifyReadOnly:
						
							result = theItem->CanMRO();
							
						break;
						
						case cmd_EditComment:
						
							result = theItem->CanEditComment();
							
						break;
						
						case cmd_CompareWithOriginal:
						
							result = theItem->CanCompareWithOriginal();
							
						break;
	
						case cmd_Diff:
						case cmd_AdvancedDiff:

							result = theItem->CanDiff();
							
						break;
						
						case cmd_GetFileLog:
						
							result = theItem->CanGetLog();
							
						break;
						
						case cmd_GetFileStatus:
						
							result = theItem->CanGetStatus();
						
						break;

						case cmd_Tag:
						
							result = theItem->CanTag();
							
						break;
						
						case cmd_OrphanFiles:
						
							result = theItem->CanOrphanFiles();
						break;
						
						case cmd_RemoveFiles:
							
							result = theItem->CanRemove();
						break;
					}
			
			}
			
		return result;
			
	}
	
	
void
CMacCVSLurkersView::BeTarget()
	{
	
		mWatchTime = kDefaultForegroundUpdateTime;
	
	}
	
	
void
CMacCVSLurkersView::DontBeTarget()
	{
	
		mWatchTime = kDefaultBackgroundUpdateTime;
		
	}


Boolean
CMacCVSLurkersView::HandleKeyPress(const EventRecord &inKeyEvent)
	{

		if ( UKeyFilters::IsTEDeleteKey(inKeyEvent.message & charCodeMask) )
			{
				
				STableCell 	theCurCell;
				Boolean			cellDeleted = false;
				
				// remove selected items
				
				while( GetNextSelectedCell(theCurCell) )
					{

							CMacCVSProjectViewItem 		*cellFile			= NULL;
							
							cellFile = ConvertTableIndexToItem( theCurCell.row );
						
							if ( cellFile && ! cellFile->IsContainer() )
								{
								
									cellFile->SetIsVisible( false );
									cellDeleted = true;
									
								}
								
					}
					
					if ( cellDeleted )
						{
					
							RemoveAllRows( false );
							
							RebuildSortedProjectView( true );

							Refresh();
					}

				return true;
			}
		
		return LCommander::HandleKeyPress(inKeyEvent);
	
	}

Boolean
CMacCVSLurkersView::ObeyCommand(CommandT inCommand, void *ioParam)
	{

		Boolean cmdHandled = true;
		
		switch (inCommand) 
			{

				case cmd_Copy:
				{
						Handle				dataHandle = NULL;
						STableCell 		theCurCell( 0, 0 );
						Boolean				clearData = true;
						
						dataHandle = ::NewHandle(0);
						if ( dataHandle == NULL)
							ThrowIfError_(MemError());
						
						try
							{
								while ( GetNextSelectedCell(theCurCell) )
									{
										CMacCVSProjectViewItem 		*cellFile			= NULL;
										
										cellFile = ConvertTableIndexToItem( theCurCell.row );
										
										if ( cellFile != NULL ) 
											{
													char		itemName[257];
			
													cellFile->GetNameCStr(itemName);
													
													strcat(itemName, "\r");
													ThrowIfError_( ::PtrAndHand( itemName, dataHandle, strlen( itemName) ) );
											}
									}
								
								Int32			length = ::GetHandleSize( dataHandle );
								
								::HLock( dataHandle );
								
								LClipboard::GetClipboard()->SetData('TEXT', *dataHandle, length - 1 );		//ignore trailing '\r'
						
								::DisposeHandle(dataHandle);		//doesn't matter that it's still locked
								dataHandle = NULL;

							}
						catch ( OSErr inErr )
							{
								if (dataHandle != NULL)
								{
									::DisposeHandle(dataHandle);
									dataHandle = NULL;
								}
								
								throw( inErr );
							}
								
					}
				break;
				
				case cmd_CopyPath:
					{
						Handle				dataHandle = NULL;
						STableCell 		theCurCell( 0, 0 );
						Boolean				clearData = true;
						
						dataHandle = ::NewHandle(0);
						if ( dataHandle == NULL)
							ThrowIfError_(MemError());
						
						try
							{
								while ( GetNextSelectedCell(theCurCell) )
									{
										CMacCVSProjectViewItem 		*cellFile			= NULL;

										cellFile = ConvertTableIndexToItem( theCurCell.row );
										
										if ( cellFile != NULL ) 
											{
													FSSpec			itemSpec;
													char				filePath[512];
													UInt32			pathLen = 510;
													
													cellFile->GetFSSpec(itemSpec);
													
													ThrowIfError_ ( FSpGetLocalCVSPath( &itemSpec, &mProjectRootFileSpec, filePath, &pathLen ) );
													filePath[pathLen] = '\0';
													strcat(filePath, "\r");
													ThrowIfError_( ::PtrAndHand( filePath, dataHandle, strlen( filePath) ) );
											}
									}
								
								Int32			length = ::GetHandleSize( dataHandle );
								
								::HLock( dataHandle );
								
								LClipboard::GetClipboard()->SetData('TEXT', *dataHandle, length - 1 );		//ignore trailing '\r'
						
								::DisposeHandle(dataHandle);		//doesn't matter that it's still locked
								dataHandle = NULL;
							
							}
						catch ( OSErr inErr )
							{
								if (dataHandle != NULL)
								{
									::DisposeHandle(dataHandle);
									dataHandle = NULL;
								}
								
								throw( inErr );
							}
					}
				break;
				
				case cmd_SelectAll:
				
					SelectAllCells();
					
				break;
			
				case cmd_ShowAll:
					
					ShowAllCells();
					
				break;
				
				default:
				
					cmdHandled = LCommander::ObeyCommand( inCommand, ioParam );
				
				break;
			
			}
			
		return cmdHandled;
		
	}


void
CMacCVSLurkersView::FindCommandStatus( CommandT inCommand, Boolean &outEnabled, Boolean &outUsesMark, Char16 &outMark, Str255 outName )
	{

		outEnabled = false;	

		switch ( inCommand ) 
			{
			
				case cmd_Copy:
				case cmd_CopyPath:
				
					outEnabled = ( CountSelectedItems() > 0 ); // are any cells selected?
				
				break;
				
				case cmd_Cut:
				
					outEnabled = ( CountSelectedItems() > 0 );
					
				break;

				case cmd_SelectAll:
				case cmd_ShowAll:
				
					outEnabled = true;
					
				break;
						
				case cmd_RevealInFinder:	// JWW - Added 04/21/99
				
				case cmd_Update:
				case cmd_UpdateTo:
				case cmd_Checkout:
				case cmd_CheckoutTo:
				case cmd_AddFile:
				case cmd_Commit:

				case cmd_Editors:
				case cmd_Edit:
				case cmd_UnEdit:
				case cmd_ShowEditors:
	
				case cmd_Watchers:
				case cmd_WatchOn:
				case cmd_WatchOff:
				case cmd_WatchAdd:
				case cmd_WatchRemove:
				case cmd_ShowWatchers:

				case cmd_RevertFile:
				case cmd_RemoveFiles:
				case cmd_Tag:

				case cmd_ModifyReadOnly:
				case cmd_EditComment:

				case cmd_CompareWithOriginal:
				case cmd_Diff:
				case cmd_AdvancedDiff:
				case cmd_GetFileLog:
				case cmd_GetFileStatus:
				
					outEnabled = CheckIfAnyItemsQualify( inCommand );
					
				break;
						
				default:
				
					LCommander::FindCommandStatus( inCommand, outEnabled, outUsesMark, outMark, outName );

				break;
			
			}
			
	}


void
CMacCVSLurkersView::ListenToMessage( MessageT inMessage, void *ioParam )
	{

		CMacCVSLurkersViewEventRec	theNewEvent = { 0, 0, 0 };
		
		switch ( inMessage ) 
			{
		
				case kNameSortMessage:
						
					if ( *(UInt32 *)ioParam != 0 )  
						{
						
							theNewEvent.fEventType = kViewSortByNameEvent;
							
							PostProjectViewEvent( theNewEvent );
						
						}
					
				break;
					
				case kStatusSortMessage:

					if ( *(UInt32 *)ioParam != 0 )  
						{
						
							theNewEvent.fEventType = kViewSortByStatusEvent;
							
							PostProjectViewEvent( theNewEvent );
						
						}
				
				break;
				
				case kVersionSortMessage:
				
					if ( *(UInt32 *)ioParam != 0 )  
						{
						
							theNewEvent.fEventType = kViewSortByVersionEvent;
							
							PostProjectViewEvent( theNewEvent );
						
						}
				
				break;
				
				case kDateSortMessage:
				
					if ( *(UInt32 *)ioParam != 0 )  
						{
						
							theNewEvent.fEventType = kViewSortByDateEvent;
							
							PostProjectViewEvent( theNewEvent );
						
						}

				break;
				
				case kTagSortMessage:
				
					if ( *(UInt32 *)ioParam != 0 )  
						{
						
							theNewEvent.fEventType = kViewSortByTagEvent;
							
							PostProjectViewEvent( theNewEvent );
						
						}
				
				break;
				
			}

	}



//################################################################################
//################################################################################
#pragma mark -
#pragma mark PROJECT VIEW EVENTS


void
CMacCVSLurkersView::ProcessProjectViewEvent( CMacCVSLurkersViewEventRec &inEventRecord )
	{
		
		switch ( inEventRecord.fEventType ) 
			{
						
				case kViewAddItemEvent:
				
					InsertProjectItem( inEventRecord.fEventItem );
					
				break;
									
				case kViewUpdateItemEvent:
				
					if ( inEventRecord.fEventItem != NULL )
						{
						
							HandleUpdateEvent( inEventRecord.fEventItem );
								
						}
						
				break;
	
				case kViewUpdateItemFSSpecEvent:
			
					Assert_( inEventRecord.fEventData != NULL );
					
					if ( inEventRecord.fEventData != NULL )
						{
						
							FSSpec theUpdateSpec = *(FSSpec *)inEventRecord.fEventData;
							
							::DisposePtr( (Ptr)inEventRecord.fEventData );
							inEventRecord.fEventData = NULL;
							
							HandleUpdateFSSpecEvent( theUpdateSpec );
					
						}
						
				break;
			
				case kViewSortByNameEvent:
				
					SortProjectView( kNameSorted );
					
				break;
				
				case kViewSortByStatusEvent:

					SortProjectView( kStatusSorted );	
				
				break;
				
				case kViewSortByVersionEvent:
				
					SortProjectView( kVersionSorted );	
				
				break;
				
				case kViewSortByDateEvent:
				
					SortProjectView( kDateSorted );
					
				break;
				
				case kViewSortByTagEvent:

					SortProjectView( kTagSorted );	
				
				break;
											
			}

	}


Boolean 
CMacCVSLurkersView::GetNextProjectViewEvent( CMacCVSLurkersViewEventRec &inEventRecord )
	{
		
		Boolean	result = false;
		
		if ( mViewEventQueue->GetCount() > 0 )
			{
			
				mViewEventQueue->FetchItemAt( 1, &inEventRecord );
		
				mViewEventQueue->RemoveItemsAt( 1, 1 );

				result = true;
							
			}
		
		return result;
			
	}


void 
CMacCVSLurkersView::PostProjectViewEvent( const CMacCVSLurkersViewEventRec &inEventRecord )
	{

		mViewEventQueue->InsertItemsAt( 1, 0x7FFFFFFF, &inEventRecord );

	}


void
CMacCVSLurkersView::HandleUpdateEvent( CMacCVSProjectViewItem *inUpdateItem )
	{
	
		TableIndexT		itemIndex		= 0x00000000;
		STableCell		exposedCell( 0, 1 );
		
		itemIndex = ConvertItemToTableIndex( inUpdateItem );
		
		if ( itemIndex != kProjectNoLongerExistsRow ) 
			{
	
				exposedCell.row = itemIndex;
				
				try
					{
					
						RefreshEntryData( inUpdateItem );
						inUpdateItem->ValidateContents( &mProjectRootFileSpec );
						RefreshCell( exposedCell );

					}
					
				catch ( ... )
					{
					
						RemoveRows( 1, exposedCell.row, true );
						
						CMacCVSProjectViewItem *testItem = NULL;
						mItemList->FetchItemAt( itemIndex, &testItem );
						Assert_( inUpdateItem == testItem );
						
						mItemList->RemoveItemsAt( 1, itemIndex );
						delete inUpdateItem;
					
					}
				
			}
			
	}
	

void
CMacCVSLurkersView::HandleUpdateFSSpecEvent( const FSSpec &inUpdateItem )
	{
	
		CMacCVSProjectViewItem		*theCurItem		= NULL;
	
		TableIndexT								currentRow		= 0,
															maxRows				= 0,
															maxColumns		= 0;

		UInt32										cellSize 			= 0;
		FSSpec										curFileSpec		= { 0, 0, "\p" };
		
		Boolean										bFoundIt			= false;
		STableCell 								currentCell( 0, 1 );

		GetTableSize( maxRows, maxColumns );
		
		for ( currentRow = 1; (currentRow <= maxRows) && (!bFoundIt); currentRow++ ) 
			{
			
				currentCell.row = currentRow;
				cellSize = sizeof(CMacCVSProjectViewItem *);
				
				GetCellData( currentCell, &theCurItem, cellSize );
				Assert_( cellSize == sizeof(CMacCVSProjectViewItem *) );
				
				if ( (theCurItem != NULL) && (cellSize == sizeof(CMacCVSProjectViewItem *)) )
					{
					
						theCurItem->GetFSSpec( curFileSpec );
						
						if ( 	(curFileSpec.vRefNum == inUpdateItem.vRefNum) && 
									(curFileSpec.parID == inUpdateItem.parID) && 
									(curFileSpec.name[0] == inUpdateItem.name[0]) )
							{
							
								bFoundIt = ::EqualString( curFileSpec.name, inUpdateItem.name, false, true );
								
								if ( bFoundIt )
									{
		
										if ( theCurItem->IsContainer() )
											{

//  This should never happen.
											
												Assert_( false );

											}
											
//  Else if the thing is a file, revalidate it.
					
										else
											{
											
												try
													{
													
														theCurItem->Invalidate();
														RefreshEntryData( theCurItem );
														theCurItem->ValidateContents( &mProjectRootFileSpec );
													
													}
													
												catch ( ... )
													{

													
													}
											
												RefreshCell( currentCell );
							
											}
							
									}
							
							}
					
					}
					
			}
	
	}
	

void
CMacCVSLurkersView::ThreadedTaskDied( const CThreadedTask *inTask )
	{
	
		Assert_( (inTask == mViewThread) && (mViewThread != NULL) );
		
		if ( inTask != NULL )
			if ( inTask == mViewThread )
				mViewThread = NULL;
	
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark PROJECT VIEW THREAD


CMacCVSLurkersViewThread::CMacCVSLurkersViewThread( CMacCVSLurkersView &inOwnerView ) :
	CThreadedTask( )
	{
	
		mOwnerView = &inOwnerView;
	
	}
	
	
CMacCVSLurkersViewThread::~CMacCVSLurkersViewThread()
	{
	
		mOwnerView->ThreadedTaskDied( this );
	
	}
	
	
void
CMacCVSLurkersViewThread::Run()
	{
	
		SInt32											nextUpdateTime			= 0;
		UInt32											maxItemIndex				= 0,
																curItemIndex				= 0;
															
		CMacCVSProjectViewItem			*theItem						= NULL;
		
		CMacCVSLurkersViewEventRec	nextEvent 					= { 0, 0, 0 };
		CMacCVSLurkersView					*inLurkersView			= NULL;
		
		while ( true ) 
			{
			
				try 
					{
						CThreadedTask::Yield();
							
						if ( mOwnerView->GetNextProjectViewEvent(nextEvent) )
							{
								mOwnerView->ProcessProjectViewEvent( nextEvent );
							}
						else if ( ::LMGetTicks() > nextUpdateTime )
							{							
								nextUpdateTime = ::LMGetTicks() + mOwnerView->mWatchTime;					
								
								maxItemIndex = mOwnerView->mItemList->GetCount();
								
								for ( curItemIndex = 1; curItemIndex <= maxItemIndex; curItemIndex++ )
									{						
										if ( (CMacCVSLurkersView::sYieldCount % CMacCVSLurkersView::kDefaultYieldCount) == 0 )
											{
												CThreadedTask::Yield();
											}
											
										CMacCVSLurkersView::sYieldCount++;
										
										if ( mOwnerView->mItemList->FetchItemAt(curItemIndex, &theItem) )
											{											
												if ( theItem->IsOutOfDate() )
													{													
														nextEvent.fEventType = CMacCVSLurkersView::kViewUpdateItemEvent;
														nextEvent.fEventItem = theItem;
														nextEvent.fEventData = NULL;
														
														mOwnerView->PostProjectViewEvent( nextEvent );													
													}																				
											}									
									}
							}						
					}					
				catch ( CThreadedTaskException inErr )
					{				
						throw inErr;					
					}					
				catch ( ... )
					{					
					
					}
						
			} //  while
					
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark SORTING LIST VIEW


void 
CMacCVSLurkersView::RebuildSortedProjectView( Boolean inRefresh )
	{
		
		CMacCVSProjectViewItem	*theCurItem	= NULL;
		UInt32									maxIndex		= 0x00000000,
														index				= 0x00000000;
												
		maxIndex = mItemList->GetCount();		
		
		for ( index = 1; index <= maxIndex; index++ )
			{
			
				if ( (sYieldCount % kDefaultYieldCount) == 0 )
					CThreadedTask::Yield();
					
				sYieldCount++;
				
				if ( mItemList->FetchItemAt(index, &theCurItem) && theCurItem->IsVisibleToUser() )
					InsertRows( 1, index, &theCurItem, sizeof(CMacCVSProjectViewItem *), inRefresh );
			
			}
			
	}


void 
CMacCVSLurkersView::SortProjectView( ProjectViewSortMethod inSortMethod )
	{

		TableIndexT								rowCount				= 0x00000000,
															columnCount			= 0x00000000;
		CMacCVSProjectViewItem		*currentItem		= NULL;
		CMacCVSProjectViewFolder	*currentFolder	= NULL;
		
		if ( inSortMethod != mCurrentSortMethod )
			{
			
			 	BroadcastMessage( CSpinIndicator::msg_SpinWaitBegin );

				RemoveAllRows( false );
				
				Draw( NULL );
				
				mCurrentSortMethod = inSortMethod;
				mItemList->SetComparator( GetCurrentComparator() );
				
				RebuildSortedProjectView( true );

				Refresh();
				
				BroadcastMessage( CSpinIndicator::msg_SpinWaitEnd );
			
			}
			
	}


LComparator *
CMacCVSLurkersView::GetCurrentComparator()
	{

		LComparator	*result	= NULL;
		
		switch ( mCurrentSortMethod ) 
			{
			
				case kNameSorted:
				
					result = new LSortByNameComparator;
					
				break;
				
				case kStatusSorted:
				
					result = new LSortByStatusComparator;
				
				break;
				
				case kVersionSorted:

					result = new LSortByVersionComparator;
				
				break;

				case kTagSorted:

					result = new LSortByTagComparator;	
				
				break;
				
				case kDateSorted:

					result = new LSortByDateComparator;
				
				break;
				
				default:
				
					Assert_( false );
					
				break;
			
			}
		
		return result;

	}

		
