//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 	CMacCVSLurkersWindow.cp
//
//	  Author: Ram Rajadhyaksha
//
//################################################################################


#include <stdio.h>
#include <Scrap.h>
#include <UDesktop.h>
#include <LWindow.h>
#include <LTextButton.h>

#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"
#include "CMacCVSLurkersTask.h"
#include "CMacCVSLurkersView.h"
#include "CMacCVSLurkersWindow.h"


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS/DESTRUCTORS


CMacCVSLurkersWindow::CMacCVSLurkersWindow(LStream *inStream):

	CMacCVSThreadedWindow(inStream)
	
	{
		
		mLurkersView 		= NULL;
		
		mWindowPosResID	= kLurkersViewWindowPositionResID;
		
	}


CMacCVSLurkersWindow::~CMacCVSLurkersWindow()
	{

//  This part is *very* important. In fact, the whole reason this class exists
//  is because of this problem: We need to stop both the spin icon and the project
//  view threads before they are destroyed. Since shutting down a thread requires
//  a context switch, if you delete the spin pane first and yield until it dies,
//  you will crash because the project view thread is still running and assumes
//  that the spin pane still exists. So, remove the spin pane as listener before
//  we destroy the window.

		mLurkersView->RemoveListener( mSpinnyThingy );
								
	}


CMacCVSLurkersWindow *
CMacCVSLurkersWindow::CreateCMacCVSLurkersWindowFromStream( LStream *inStream )
	{

		return new CMacCVSLurkersWindow( inStream );

	}


void
CMacCVSLurkersWindow::FinishCreateSelf()
	{

		Inherited::FinishCreateSelf();
		
		LTextButton		*theSortButton	= NULL;
		
		mLurkersView = dynamic_cast<CMacCVSLurkersView *>( FindPaneByID(CMacCVSLurkersView::class_ID) );
		ThrowIfNil_( mLurkersView );
				
//  Set up the text box buttons.	
		
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByNameButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mLurkersView );
			
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByStatusButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mLurkersView );
			
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByVersionButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mLurkersView );

		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByModDateButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mLurkersView );

		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByTagButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mLurkersView );
		
//  Hook up the spinnythingymoboby.	

		Assert_(mSpinnyThingy != NULL);
		mLurkersView->AddListener( mSpinnyThingy );
		
		SetLatentSub( mLurkersView );
			
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


void
CMacCVSLurkersWindow::FindLurkers( const CMacCVSDoc *inOwnerDoc, const LArray *inTargetItems, const CMacCVSLurkersQueryRec *inQueryRec )
	{
	
		Assert_( mWindowTask == NULL );

		ThrowIfNil_( inOwnerDoc );
		ThrowIfNil_( inTargetItems );
		
		if ( mWindowTask == NULL )
			{
			
				FSSpec	rootFileSpec;
			
				inOwnerDoc->GetProjectRootFileSpec( rootFileSpec );
				mLurkersView->SetProjectRoot( &rootFileSpec );
				
				mWindowTask = new CMacCVSLurkersTask( this, inQueryRec, inTargetItems );
				ThrowIfNil_( mWindowTask );
				
				PrimeWindowTask();
				
			}
	
	}
	
	
void
CMacCVSLurkersWindow::AddProjectItem( const FSSpec *inItemFileSpec )
	{	
		mLurkersView->AddProjectItem( inItemFileSpec );	
	}
	
	
void
CMacCVSLurkersWindow::ThreadedTaskDied( const CThreadedTask *inTask )
	{
		//  For optimization reasons, the CMacCVSLurkersView() leaves
		//  the entries file of the parent directory of the last file
		//  added opened indefinately. When the Lurkers task is finished,
		//  we call ReleaseReadLocks() to close that file.
		
		CMacCVSThreadedWindow::ThreadedTaskDied( inTask );
	
		mLurkersView->ReleaseReadLocks();
	}

	