//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSMappingsView.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains code to implement the CMacCVSMappingsView pane.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes. 
					
					Fixed bug where you could edit a mapping to have the same extension 
					as one already in the database.
					
					Fixed problems with drawing on a gray background.
							
-------------------------------------------------------------------------------- */


#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <Scrap.h>

#include <LArray.h>
#include <URegions.h>
#include <UException.h>
#include <UAttachments.h>
#include <UModalDialogs.h>

#include <LTableView.h>
#include <LTableMonoGeometry.h>
#include <LTableSingleSelector.h>
#include <LTableArrayStorage.h>

#include <LPopupButton.h>
#include <LEditText.h>

#include "MacCVSPPConstants.h"

#include "StringUtils.h"
#include "UErrorDialog.h"
#include "CThreadedTask.h"

#include "CVSEngine.h"
#include "CMacCVSDoc.h"

#include "CMacCVSMappingsView.h"


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS AND DESTRUCTORS	


CMacCVSMappingsView *
CMacCVSMappingsView::CreateCMacCVSMappingsViewFromStream( LStream *inStream )
	{

		return ( new CMacCVSMappingsView(inStream) );

	}


CMacCVSMappingsView::CMacCVSMappingsView( LStream *inStream ): 

	LTableView( inStream )

	{
				
		mDirty = false;
		
		SPoint32	theScrollUnit = { -1, kDefaultRowHeight };
		
		SetScrollUnit( theScrollUnit );
		
	}


CMacCVSMappingsView::~CMacCVSMappingsView()
	{

	}

	
void
CMacCVSMappingsView::FinishCreateSelf( void )
	{

		Rect	theFrameRect = { 0, 0, 0, 0 };
		
		CalcLocalFrameRect( theFrameRect );
		
		mTableGeometry = new LTableMonoGeometry( this, theFrameRect.right - theFrameRect.left, kDefaultRowHeight );
	 	mTableSelector = new LTableSingleSelector( this );
		mTableStorage = new LTableArrayStorage( this, sizeof(CCVSEngineFileMappingRec) );

		// Insert a single column.
		InsertCols( 1, 0, nil, nil, false );
		
		//	Attachment for keyboard navigation
		AddAttachment( new LKeyScrollAttachment(this) );
			
		LTableView::FinishCreateSelf();
	
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


TableIndexT
CMacCVSMappingsView::AddNewItem()
	{
	
		CCVSEngineFileMappingRec	theNewItem;
		
		TableIndexT		result						= 0;
		TableIndexT		outRows						= 0;
		TableIndexT		outCols						= 0;
															
		Boolean				bDone 						= false;
		Boolean				bIsAlreadyInTable	= false;
															
		MessageT			theMessage				= 0;
		UInt32				ioCount						= 0;
		
		LStr255				osTypePStr;
		LStr255				tempPStr;
		LStr255				suffixPStr;
		
		STableCell		theCurCell( 0, 1 );
		
		StDialogHandler		dialogHandler( kMacCVSNewEncodingDialogPPobID, NULL );
		
		LDialogBox	*theDialog = dynamic_cast<LDialogBox *>( dialogHandler.GetDialog() );
		ThrowIfNil_( theDialog );
		
		LEditText	*suffixEditField = dynamic_cast<LEditText *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogSuffixEditFieldID) );
		ThrowIfNil_( suffixEditField );
		
		LEditText	*typeEditField = dynamic_cast<LEditText *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogTypeEditFieldID) );
		ThrowIfNil_( typeEditField );
		
		LEditText	*creatorEditField = dynamic_cast<LEditText *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogCreatorEditFieldID) );
		ThrowIfNil_( creatorEditField );
	
		LPopupButton	*lfTranslationPopup = dynamic_cast<LPopupButton *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogTranslationPopupID) );
		ThrowIfNil_( lfTranslationPopup );
		
		LPopupButton	*encodingPopup = dynamic_cast<LPopupButton *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogEncodingPopupID) );
		ThrowIfNil_( encodingPopup );

		dialogHandler.AddAttachment( new CThreadedTaskYieldAttachment );
		theDialog->Show();
		
		while ( !bDone )
			{
			
				theMessage = dialogHandler.DoDialog();
			
				switch ( theMessage )
					{
					
						case msg_OK:
				
							suffixEditField->GetDescriptor( suffixPStr );
							typeEditField->GetDescriptor( osTypePStr );
							creatorEditField->GetDescriptor( tempPStr );
							
							if ( (suffixPStr[0] == 0) || (osTypePStr[0] == 0) || (tempPStr[0] == 0) )
								{
									UErrorDialog theStupidUser( kCantHaveZeroLengthFieldErr, NULL );
								}
							else
								{
									//  Make sure the thing isn't already in the table.

									bIsAlreadyInTable = false;
																	
									GetTableSize( outRows, outCols );
									
									for ( UInt32 index = 1; ((index <= outRows) && (!bIsAlreadyInTable)); index++ )
										{										
											theCurCell.row = index;
											
											ioCount = sizeof( CCVSEngineFileMappingRec );
											GetCellData( theCurCell, &theNewItem, ioCount );
											Assert_( ioCount == sizeof(CCVSEngineFileMappingRec) );
											
											bIsAlreadyInTable = ::EqualString( suffixPStr, theNewItem.fFileNamePStr, false, true );												
										}
										
									if ( bIsAlreadyInTable )
										{										
											UErrorDialog	theStupidUser( kMappingAlreadyExistsErr, NULL );										
										}										
									else
										{									
											mDirty = true;
												
											PStrCopy( suffixPStr, theNewItem.fFileNamePStr, sizeof(theNewItem.fFileNamePStr) );
											
											if ( osTypePStr[0] < 4 )
												for ( UInt32 index = osTypePStr[0] + 1; index <= 4; index++ )
													osTypePStr[index] = ' ';
													
											::BlockMoveData( &osTypePStr[1], &theNewItem.fFileType, sizeof(OSType) );
														
											creatorEditField->GetDescriptor( osTypePStr );

											if ( osTypePStr[0] < 4 )
												for ( UInt32 index = osTypePStr[0] + 1; index <= 4; index++ )
													osTypePStr[index] = ' ';
													
											::BlockMoveData( &osTypePStr[1], &theNewItem.fFileCreator, sizeof(OSType) );

											switch ( lfTranslationPopup->GetValue() )
												{												
													case kUNIXLFTranslationMenuItem:
																										
														theNewItem.fTranslation = kCVSEngineLFTranslation;
													
													break;
													
													case kMacintoshLFTranslationMenuItem:
													
														theNewItem.fTranslation = kCVSEngineCRTranslation;
														
													break;
													
													default:
													
														theNewItem.fTranslation = kCVSEngineCRTranslation;
												
													break;											
												}
												
											switch ( encodingPopup->GetValue() )
												{												
													case kTextEncodingMenuItem:
													
														theNewItem.fEncoding = kCVSEngineTextEncoding;
														
													break;
													
													case kBinaryEncodingMenuItem:
													
														theNewItem.fEncoding = kCVSEngineBinaryEncoding;
														
													break;
													
													case kAppleSingleEncodingMenuItem:
													
														theNewItem.fEncoding = kCVSEngineAppleSingleEncoding;
														
													break;
													
													case kBinhex4EncodingMenuItem:
													
														theNewItem.fEncoding = kCVSEngineBinhex4Encoding;
														
													break;
													
													case kMacBinaryEncodingMenuItem:
													
														theNewItem.fEncoding = kCVSEngineMacBinaryEncoding;
														
													break;
													
													default:
			
														theNewItem.fEncoding = kCVSEngineAppleSingleEncoding;
												
													break;											
												}
												
											theDialog->Hide();
											
											result = InsertNewItem( theNewItem );

//  Make sure the table is scrolled to show the new item.
									
											theCurCell.row = result;											
											ScrollCellIntoFrame( theCurCell );
											
											bDone = true;											
										}									
								}
						
						break;
						
						case msg_Cancel:
						
							bDone = true;
							
						break;
					
					}
					
			}
			
		return result;
	
	}


void
CMacCVSMappingsView::EditCurrentItem()
	{
	
		CCVSEngineFileMappingRec	theSelectedItem;
		CCVSEngineFileMappingRec	theCurItem;
	
		UInt32				ioCount						= sizeof( CCVSEngineFileMappingRec );
		Boolean				bIsAlreadyInTable	= false;
		Boolean				bDone 						= false;

		TableIndexT		outRows						= 0;
		TableIndexT		outCols						= 0;

		MessageT			theMessage				= 0;
		OSType				tempOSType 				= 0;
	
		LStr255				osTypePStr;
		LStr255				tempPStr;
		LStr255				suffixPStr;
			
		STableCell		selectedCell( 0, 0 );
		STableCell		theCurCell( 0, 1 );
			
		if ( GetNextSelectedCell(selectedCell) )
			{

				GetCellData(selectedCell, &theSelectedItem, ioCount);
				Assert_( ioCount == sizeof(CCVSEngineFileMappingRec) );
				
				StDialogHandler	dialogHandler( kMacCVSNewEncodingDialogPPobID, NULL );
				
				LDialogBox *theDialog = dynamic_cast<LDialogBox *>( dialogHandler.GetDialog() );
				ThrowIfNil_( theDialog );
				
				LEditText	*suffixEditField = dynamic_cast<LEditText *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogSuffixEditFieldID) );
				ThrowIfNil_( suffixEditField );
				
				if ( theSelectedItem.fFileNamePStr[0] > 0 )
					suffixEditField->SetDescriptor( theSelectedItem.fFileNamePStr );
				else
					{

						suffixEditField->SetDescriptor( "\pDefault Mapping" );
						suffixEditField->Disable();
							
					}
					
				LEditText	*typeEditField = dynamic_cast<LEditText *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogTypeEditFieldID) );
				ThrowIfNil_( typeEditField );
				
				osTypePStr[0] = sizeof( OSType );
				::BlockMoveData( &theSelectedItem.fFileType, &osTypePStr[1], sizeof(OSType) );
				
				typeEditField->SetDescriptor( osTypePStr );
				
				LEditText	*creatorEditField = dynamic_cast<LEditText *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogCreatorEditFieldID) );
				ThrowIfNil_( creatorEditField );

				osTypePStr[0] = sizeof( OSType );
				::BlockMoveData( &theSelectedItem.fFileCreator, &osTypePStr[1], sizeof(OSType) );

				creatorEditField->SetDescriptor( osTypePStr );
			
				LPopupButton	*lfTranslationPopup = dynamic_cast<LPopupButton *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogTranslationPopupID) );
				ThrowIfNil_( lfTranslationPopup );
				
				switch ( theSelectedItem.fTranslation )
					{
					
						case kCVSEngineLFTranslation:
						
							lfTranslationPopup->SetValue( kUNIXLFTranslationMenuItem );
							
						break;
						
						case kCVSEngineCRTranslation:

							lfTranslationPopup->SetValue( kMacintoshLFTranslationMenuItem );
						
						break;
						
						default:

							Assert_( false );
						
							lfTranslationPopup->SetValue( kUNIXLFTranslationMenuItem );
						
						break;
					
					}
					
				LPopupButton	*encodingPopup = dynamic_cast<LPopupButton *>( theDialog->FindPaneByID(kMacCVSNewEncodingDialogEncodingPopupID) );
				ThrowIfNil_( encodingPopup );

				switch ( theSelectedItem.fEncoding )
					{
					
						case kCVSEngineTextEncoding:

							encodingPopup->SetValue( kTextEncodingMenuItem );
						
						break;
						
						case kCVSEngineBinaryEncoding:

							encodingPopup->SetValue( kBinaryEncodingMenuItem );
						
						break;
						
						case kCVSEngineAppleSingleEncoding:

							encodingPopup->SetValue( kAppleSingleEncodingMenuItem );
						
						break;

						case kCVSEngineBinhex4Encoding:

							encodingPopup->SetValue( kBinhex4EncodingMenuItem );
						
						break;
						
						case kCVSEngineMacBinaryEncoding:

							encodingPopup->SetValue( kMacBinaryEncodingMenuItem );
						
						break;
						
						default:

							Assert_( false );
													
							encodingPopup->SetValue( kTextEncodingMenuItem );
						
						break;
					
					}

				dialogHandler.AddAttachment( new CThreadedTaskYieldAttachment );
				theDialog->Show();
				
				while ( !bDone )
					{
					
						theMessage = dialogHandler.DoDialog();
					
						switch ( theMessage )
							{
							
								case msg_OK:
								
									suffixEditField->GetDescriptor( suffixPStr );
									typeEditField->GetDescriptor( osTypePStr );
									creatorEditField->GetDescriptor( tempPStr );
									
									if ( (suffixPStr[0] == 0) || (osTypePStr[0] == 0) || (tempPStr[0] == 0) )
										
										UErrorDialog	theStupidUser( kCantHaveZeroLengthFieldErr, NULL );
										
									else
										{

//  Make sure the thing isn't already in the table.

											bIsAlreadyInTable = false;
																			
											GetTableSize( outRows, outCols );
											
											for ( UInt32 index = 1; ((index <= outRows) && (!bIsAlreadyInTable)); index++ )
												{
												
													theCurCell.row = index;
													
													ioCount = sizeof( CCVSEngineFileMappingRec );
													GetCellData( theCurCell, &theCurItem, ioCount );
													Assert_( ioCount == sizeof(CCVSEngineFileMappingRec) );
													
													if ( (index != selectedCell.row) && ::EqualString(suffixPStr, theCurItem.fFileNamePStr, false, true) )
														bIsAlreadyInTable = true;
															
												}
												
											if ( bIsAlreadyInTable )
												{
												
													UErrorDialog	theStupidUser( kMappingAlreadyExistsErr, NULL );
												
												}
											else
												{
														
													if ( (theSelectedItem.fFileNamePStr[0] > 0) && (suffixPStr[0] != theSelectedItem.fFileNamePStr[0]) )
														mDirty = true;
														
													else if ( theSelectedItem.fFileNamePStr[0] > 0 )
														if ( strncmp((char *)&suffixPStr[1], (char *)&theSelectedItem.fFileNamePStr[1], suffixPStr[0]) )
															mDirty = true;
													
													if ( theSelectedItem.fFileNamePStr[0] > 0 )
														PStrCopy( suffixPStr, theSelectedItem.fFileNamePStr, sizeof(theSelectedItem.fFileNamePStr) );
													
													if ( osTypePStr[0] < 4 )
														for ( UInt32 index = osTypePStr[0] + 1; index <= 4; index++ )
															osTypePStr[index] = ' ';
															
													::BlockMoveData( &osTypePStr[1], &tempOSType, sizeof(OSType) );
													
													if ( tempOSType != theSelectedItem.fFileType )
														{
														
															mDirty = true;
															
															theSelectedItem.fFileType = tempOSType;
														
														}
														
													creatorEditField->GetDescriptor( osTypePStr );

													if ( osTypePStr[0] < 4 )
														for ( UInt32 index = osTypePStr[0] + 1; index <= 4; index++ )
															osTypePStr[index] = ' ';
															
													::BlockMoveData( &osTypePStr[1], &tempOSType, sizeof(OSType) );

													if ( tempOSType != theSelectedItem.fFileCreator )
														{
														
															mDirty = true;
															
															theSelectedItem.fFileCreator = tempOSType;
														
														}

													switch ( lfTranslationPopup->GetValue() )
														{
														
															case kUNIXLFTranslationMenuItem:

																if ( theSelectedItem.fTranslation != kCVSEngineLFTranslation )
																	mDirty = true;
															
																theSelectedItem.fTranslation = kCVSEngineLFTranslation;
															
															break;
															
															case kMacintoshLFTranslationMenuItem:
			
																if ( theSelectedItem.fTranslation != kCVSEngineCRTranslation )
																	mDirty = true;
														
																theSelectedItem.fTranslation = kCVSEngineCRTranslation;
																
															break;
															
															default:
			
																if ( theSelectedItem.fTranslation != kCVSEngineCRTranslation )
																	mDirty = true;
															
																theSelectedItem.fTranslation = kCVSEngineCRTranslation;
														
															break;
														
														}
														
													switch ( encodingPopup->GetValue() )
														{
														
															case kTextEncodingMenuItem:

																if ( theSelectedItem.fEncoding != kCVSEngineTextEncoding )
																	mDirty = true;
															
																theSelectedItem.fEncoding = kCVSEngineTextEncoding;
																
															break;
															
															case kBinaryEncodingMenuItem:

																if ( theSelectedItem.fEncoding != kCVSEngineBinaryEncoding )
																	mDirty = true;
															
																theSelectedItem.fEncoding = kCVSEngineBinaryEncoding;
																
															break;

															case kMacBinaryEncodingMenuItem:
				
																if ( theSelectedItem.fEncoding != kCVSEngineMacBinaryEncoding )
																	mDirty = true;
														
																theSelectedItem.fEncoding = kCVSEngineMacBinaryEncoding;
																
															break;
															
															case kBinhex4EncodingMenuItem:
				
																if ( theSelectedItem.fEncoding != kCVSEngineBinhex4Encoding )
																	mDirty = true;
														
																theSelectedItem.fEncoding = kCVSEngineBinhex4Encoding;
																
															break;
															
															case kAppleSingleEncodingMenuItem:
															default:
			
																if ( theSelectedItem.fEncoding != kCVSEngineAppleSingleEncoding )
																	mDirty = true;
														
																theSelectedItem.fEncoding = kCVSEngineAppleSingleEncoding;
																
															break;
																											
														}
													
													theDialog->Hide();
													
													if ( mDirty )
														{
														
															SetCellData( selectedCell, &theSelectedItem, sizeof(CCVSEngineFileMappingRec) );
															RefreshCell( selectedCell );
													
														}
														
													bDone = true;
											
												}
												
										}
																	
								break;
								
								case msg_Cancel:
								
									bDone = true;
									
								break;
							
							}
							
					}
					
			}
	
	}
	
	
TableIndexT
CMacCVSMappingsView::InsertNewItem( const CCVSEngineFileMappingRec &inItem )
	{
	
		TableIndexT		outRows = 0,
									outCols = 0;
									
		InsertRows( 1, LArray::index_Last, &inItem, sizeof(CCVSEngineFileMappingRec), true );
		
		GetTableSize( outRows, outCols );
				
		return outRows;
		
	}
	
	
TableIndexT
CMacCVSMappingsView::GetSelectedItem( CCVSEngineFileMappingRec &outItem )
	{
	
		TableIndexT	selectedRow = 0x00000000;
		UInt32			ioCount			= sizeof( CCVSEngineFileMappingRec );
		
		STableCell	selectedCell( 0, 0 );
		
		if ( GetNextSelectedCell(selectedCell) )
			{
			
				selectedRow = selectedCell.row;
				
				GetCellData(selectedCell, &outItem, ioCount);
				Assert_( ioCount == sizeof(CCVSEngineFileMappingRec) );
	
			}
			
		return selectedRow;
			
	}
	
	
Boolean
CMacCVSMappingsView::RemoveItem( const TableIndexT inRow, CCVSEngineFileMappingRec &outItem )
	{

		Boolean							bResult 					= false,
												bCellWasSelected	= false;
												
		TableIndexT					outRows						= 0x00000000,
												outCols						= 0x00000000,
												theRow						= inRow;
												
		UInt32							ioCount 					= sizeof( CCVSEngineFileMappingRec );
		
		STableCell					theCell( inRow, 1 );
		
		GetTableSize( outRows, outCols );
		
		if ( theRow > outRows )
			theRow = outRows;
			
		if ( theRow > 0 )
			{

				bCellWasSelected = CellIsSelected( theCell );
			
				GetCellData(theCell, &outItem, ioCount);
				Assert_( ioCount == sizeof(CCVSEngineFileMappingRec) );
						
				RemoveRows( 1, theCell.row, true );
					
				bResult = true;

//  Preserve the selection.
			
				if ( bCellWasSelected )
					{
					
						if ( theCell.row == outRows )
							theCell.row = outRows - 1;

//  This damn function doesn't update the table properly.

//					ScrollCellIntoFrame( theCell );
						SelectCell( theCell );
						
					}
					
			}
			
		return bResult;
					
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark DRAWING CELLS

	
void
CMacCVSMappingsView::DrawSelf()
	{
	
		Rect theFrame = { 0, 0, 0, 0 };
		
		CalcLocalFrameRect( theFrame );

//  Grayscale fix

		::BackColor( whiteColor );
		::EraseRect( &theFrame );
	
		LTableView::DrawSelf();
		
	}
	

void
CMacCVSMappingsView::DrawCell(const STableCell &inCell,const Rect &inLocalRect)
	{

		CCVSEngineFileMappingRec		cellData;
		
		UInt32											ioCount					= sizeof( CCVSEngineFileMappingRec );
		LStr255											textPStr				= "\p";								
																
		LTableView::DrawCell( inCell, inLocalRect );
					
		GetCellData( inCell, &cellData, ioCount );
		Assert_( ioCount == sizeof(CCVSEngineFileMappingRec) );
		
//  Save and setup the text traits.

		StTextState	theTextState;
		UTextTraits::SetPortTextTraits( kCellTextTraitsID );

		::MoveTo( inLocalRect.left + kCellExtensionTextOffset, 
			inLocalRect.bottom - kCellTextOffsetFromBottom );
						
		if ( cellData.fFileNamePStr[0] > 0 )
			
			::DrawString( cellData.fFileNamePStr );

		else 
			{
			
				RGBColor	grey = { 0x8000, 0x8000, 0x8000 };
				RGBForeColor( &grey );
	
				::DrawString( "\pDefault" );	
				
				::ForeColor(blackColor);
			
			}
		
		::MoveTo( inLocalRect.left + kCellTypeTextOffset, 
			inLocalRect.bottom - kCellTextOffsetFromBottom );
						
		::BlockMoveData( &cellData.fFileType, &textPStr[1], sizeof(OSType) );
		textPStr[0] = sizeof( OSType );
		
		::DrawString( textPStr );

		::MoveTo( inLocalRect.left + kCellCreatorTextOffset, 
			inLocalRect.bottom - kCellTextOffsetFromBottom );
						
		::BlockMoveData( &cellData.fFileCreator, &textPStr[1], sizeof(OSType) );
		textPStr[0] = sizeof( OSType );
		
		::DrawString( textPStr );
		
		textPStr[0] = 0x00;
		
		if ( cellData.fTranslation == kCVSEngineLFTranslation )
		
			textPStr.Append("\pUnix");
		
		else textPStr.Append("\pMacintosh");
			
		::MoveTo( inLocalRect.left + kCellTranslationTextOffset, 
			inLocalRect.bottom - kCellTextOffsetFromBottom );
			
		::DrawString( textPStr );

		textPStr[0] = 0x00;
		
		if ( cellData.fEncoding == kCVSEngineTextEncoding )
		
			textPStr.Append("\pText");
		
		else if ( cellData.fEncoding == kCVSEngineBinaryEncoding )
		
			textPStr.Append("\pBinary");

		else if ( cellData.fEncoding == kCVSEngineBinhex4Encoding )
		
			textPStr.Append("\pBinhex 4.0");
			
		else if ( cellData.fEncoding == kCVSEngineMacBinaryEncoding )
		
			textPStr.Append("\pMacBinary III");
			
		else textPStr.Append("\pAppleSingle");
			
		::MoveTo( inLocalRect.left + kCellEncodingOffset, 
			inLocalRect.bottom - kCellTextOffsetFromBottom );
			
		::DrawString( textPStr );
		
		::PenPat( &qd.gray );
		
		::MoveTo( inLocalRect.left, inLocalRect.bottom - 1 );
		::LineTo( inLocalRect.right, inLocalRect.bottom - 1 );
					
	}


void
CMacCVSMappingsView::DrawCellIcon( const STableCell &inCell, const Rect &inLocalRect, Boolean inHilite )
	{

								
	}


void
CMacCVSMappingsView::ClickSelf( const SMouseDownEvent &inMouseDown )
	{

		LTableView::ClickSelf( inMouseDown );
	
		BroadcastMessage( kCMacCVSMappingsViewMessage, NULL );
	
	}


void
CMacCVSMappingsView::ClickCell( const STableCell &inCell, const SMouseDownEvent &inMouseDown )
	{
		

		LTableView::ClickCell( inCell, inMouseDown );
						
		if ( GetClickCount() > 1 )
			{
					
				EditCurrentItem();
				
			}
							
	}
	
