//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSMessageView.cp
//
//	  Author: 	Ram Rajadhyaksha and others (unknown)
//
//################################################################################


#include <string.h>

#include <LArray.h>
#include <LTableMonoGeometry.h>
#include <LTableSingleSelector.h>
#include <LTableArrayStorage.h>

#include "CMacCVSProjectView.h"
#include "CMacCVSMessageView.h"
#include "UKeyScrollAttachments.h"

#include "CVSEngineUtils.h"
#include "CMacCVSDoc.h"


CMacCVSMessageView::
CMacCVSMessageView( LStream *inStream ):

	LTableView( inStream )
	
	{
	
		mMessageItems = new LArray( sizeof(CMacCVSMessageViewCellDataRec) );
		ThrowIfNil_( mMessageItems );

		mMessageMask = CMacCVSMessageView::kErrorMask | CMacCVSMessageView::kWarningMask |
				CMacCVSMessageView::kMessageMask | CMacCVSMessageView::kFileMask;
		
		mScrollToNewMessages = true;
		
	}


CMacCVSMessageView::
~CMacCVSMessageView()
	{

		Assert_( mMessageItems != NULL );
	
		DeleteAllMessages();
		
		if ( mMessageItems != NULL )
			{
			
				delete mMessageItems;
			
				mMessageItems = NULL;
				
			}
			
	}


CMacCVSMessageView *
CMacCVSMessageView::CreateCMacCVSMessageViewFromStream( LStream *inStream )
	{

		return new CMacCVSMessageView( inStream );

	}


void
CMacCVSMessageView::FinishCreateSelf()
	{

		mTableGeometry = new LTableMonoGeometry( this, kDefaultColumnWidth, kDefaultRowHeight );
		
		mTableSelector = new LTableSingleSelector( this );
		
		mTableStorage = new LTableArrayStorage( this, sizeof(CMacCVSMessageViewCellDataRec) );

		// Insert a single column.
		InsertCols( 1, 0, nil, nil, false );
		
		//	Attachment for keyboard navigation
		AddAttachment( new USimpleKeyScrollAttachment(this) );
		
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark LPANE OVERRIDES

	
void
CMacCVSMessageView::AdaptToSuperFrameSize( Int32 inSurrWidthDelta, Int32 inSurrHeightDelta, Boolean inRefresh )
	{
	
		Rect	frameRect	= { 0, 0, 0, 0 };
		
		LTableView::AdaptToSuperFrameSize( inSurrWidthDelta, inSurrHeightDelta, inRefresh );
		
		CalcLocalFrameRect( frameRect );
		
		if ( (frameRect.right - frameRect.left) >= kDefaultColumnWidth )
		
			SetColWidth( frameRect.right - frameRect.left, 1, 1 );
			
		else SetColWidth( kDefaultColumnWidth, 1, 1 );
	
	}
	
	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark LTABLEVIEW OVERRIDES


void
CMacCVSMessageView::OpenCell( const STableCell	&inCell )
	{
		
		OSErr														result				= noErr;
		CMacCVSDoc											*pOwnerDoc		= NULL;
		CMacCVSWindow										*pOwnerWindow	= NULL;
		CMacCVSMessageViewCellDataRec		cellData	 		= { 0, 0, NULL, 0, NULL };
	
		Boolean													bValid				= false;
		UInt32													ioCount				= sizeof( CMacCVSMessageViewCellDataRec );
		UInt32													intLen				= 0;
		
		FSSpec													fsCommandRoot	= { 0, 0, "\p" };
		FSSpec													fsTarget			= { 0, 0, "\p" };
		FSSpec													fsProjectRoot = { 0, 0, "\p" };
		
		LStr255													pstrLocalPath;
			
		GetCellData( inCell, &cellData, ioCount );		
		Assert_( ioCount == sizeof( CMacCVSMessageViewCellDataRec ) );
				
		pOwnerWindow = dynamic_cast<CMacCVSWindow *>( LWindow::FetchWindowObject(GetMacPort()) );
		ThrowIfNil_( pOwnerWindow );

		pOwnerDoc = pOwnerWindow->GetOwnerDoc();
		ThrowIfNil_( pOwnerDoc );
			
		pOwnerDoc->GetProjectRootFileSpec( fsProjectRoot );
		
		//  Historically, CVS commands could be run in any directory, changing
		//  the way path names are returned. Now everything, except Add, runs
		//  in the root directory, so passing the command root fsspec isn't
		//  really needed as much.
		
		if ( cellData.fCellData != NULL )
			{
				fsCommandRoot = *(FSSpec *)cellData.fCellData;
			}
		else
			{
				fsCommandRoot = fsProjectRoot;
			}
								
		// parse out a path from the cell text. This is rather sensitive to changes in the way the
		// CVS server sends responses, so we have an engine util function do it.

		if ( cellData.fCellType == CMacCVSMessageView::kMessageTypeMessage )
			{
				if ( !strncmp(cellData.fCellTextPtr, "Updating ", 9) )
					{
						intLen = cellData.fCellTextLen - 9;
						
						if ( intLen <= 255 )
							{
								::BlockMoveData( cellData.fCellTextPtr + 9, &pstrLocalPath[1], intLen );
								pstrLocalPath[0] = intLen;
							}
							
						bValid = true;
					}
							
				while ( (pstrLocalPath[0] > 0) && (pstrLocalPath[pstrLocalPath[0]] == '\r') )
					{
						if ( pstrLocalPath[pstrLocalPath[0]] == '\r' )
							{
								pstrLocalPath[0]--;
							}
					}
			}
		else if ( cellData.fCellType == CMacCVSMessageView::kMessageTypeFile )
			{
				intLen = 255;
				GetRelativePathFromResponse( cellData.fCellTextPtr, cellData.fCellTextLen, &intLen, (char *)&pstrLocalPath[1]);
				pstrLocalPath[0] = intLen;
				
				bValid = true;
			}
		else if ( cellData.fCellType == CMacCVSMessageView::kMessageTypeWarning )
			{
				if ( cellData.fCellData != NULL )
					{
						bValid = true;
					}
			}
			
		if ( bValid )
			{
				if ( intLen > 0 )
					{
						if ( FSpIsAFile( &fsCommandRoot, NULL ) == kCVSEngineIsFolder )
							{
								//  Call StripPath to remove those annoying './' items.						
								intLen = pstrLocalPath[0];
								result = CUStripCVSPath( (char *)&pstrLocalPath[1], &intLen );
								pstrLocalPath[0] = intLen;
								
								result = FSpGetFromLocalCVSPath( pstrLocalPath, &fsCommandRoot, &fsTarget);
								
								if ( result == kCVSEngineIsFolder )
									{
										result = noErr;
									}
									
								if ( result != noErr ) 
									{
										return;
									}
							}
						else
							{
								//  um...sure...
								fsTarget = fsCommandRoot;
							}
					}
				else
					{
						fsTarget = fsCommandRoot;
					}
			}
			
		if ( FSpIsValid(fsTarget) )
			{	
				CMacCVSProjectViewNewViewSelectFileEventRec	broadcastData;			//thank god for copy and paste
				
				broadcastData.fFileToSelectSpec = fsTarget;
				
				if ( !FSpIsEqual(&fsTarget, &fsProjectRoot) )
					{
						result = FSpGetParentFSSpec( &fsTarget, &broadcastData.fFolderSpec );
						if (result != noErr) return;
					}
				else
					{
						broadcastData.fFolderSpec = fsTarget;
					}
						
				// we should probably make it zoom from the origin of the document window or something, but
				// it's too hard to get that from here. Just zoom from the cell instead.
				GetLocalCellRect( inCell, broadcastData.fFromCellRect );

				LocalToPortPoint( topLeft(broadcastData.fFromCellRect) );
				PortToGlobalPoint( topLeft(broadcastData.fFromCellRect) );
				
				LocalToPortPoint( botRight(broadcastData.fFromCellRect) );
				PortToGlobalPoint( botRight(broadcastData.fFromCellRect) );
				
				BroadcastMessage( CMacCVSProjectView::kNewViewSelectFileMessage, &broadcastData );
			}
					
	}



void
CMacCVSMessageView::ClickCell( const STableCell	&inCell, const SMouseDownEvent &inMouseDown)
	{			
			if ( GetClickCount() > 1 )
				{
					OpenCell(inCell);
				}
	}																											


void
CMacCVSMessageView::DrawCell( const STableCell &inCell, const Rect &inLocalRect )
	{
		
		Rect														cellFrame		= inLocalRect,
																		iconFrame		= { 0, 0, 0, 0 };
																		
		CMacCVSMessageViewCellDataRec		cellData	 	= { 0, 0, NULL, 0, NULL };
		
		UInt32													ioCount			= sizeof( CMacCVSMessageViewCellDataRec );
		SInt16													iconID			= 0x0000;
		
		GetCellData( inCell, &cellData, ioCount );
		
		Assert_( ioCount == sizeof(CMacCVSMessageViewCellDataRec) );
				
		//	Draw the icon for the message type.
		
		iconFrame.left 		= cellFrame.left + 2;
		iconFrame.top 		= cellFrame.top + 2;
		iconFrame.right 	= iconFrame.left + 16;
		iconFrame.bottom 	= iconFrame.top + 16;
		
		switch ( cellData.fCellType )
			{
				case kMessageTypeError:
					iconID = kErrorIconID;
				break;
				
				case kMessageTypeWarning:
					iconID = kWarningIconID;
				break;
				
				case kMessageTypeMessage:
					iconID = kMessageIconID;
				break;
				
				case kMessageTypeFile:
				
					iconID = kFileMergedIconID;		//default
					
					if ( cellData.fCellFlags & kFileMessageUpdatedFlag )
						iconID = kFileUpdatedIconID;
					else if ( cellData.fCellFlags & kFileMessageMergedFlag )
						iconID = kFileMergedIconID;
					else if ( cellData.fCellFlags & kFileMessageConflictFlag )
						iconID = kFileConflictIconID;
						
				break;
				
				default:
					Assert_( false );
				break;
			}
			
		::PlotIconID( &iconFrame, atNone, ttNone, iconID );

		::TextFont( kFontIDMonaco );
		::TextSize( 9 );
		
		::InsetRect( &cellFrame, 1, 1 );
		cellFrame.left += kDefaultTextOffset;
		
		StColorState		colorState;
		StColorState::Normalize();
		
		::TETextBox( cellData.fCellTextPtr, cellData.fCellTextLen, &cellFrame, 0 );
			
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


Boolean
CMacCVSMessageView::IsMessageTypeEnabled( const UInt32 inMessageType)
	{
	
		return ( mMessageMask & (1 << (inMessageType - 1)) );
	
	}
	
	
void
CMacCVSMessageView::ShowMessageType( const UInt32 inMessageType, const Boolean inRefresh )
	{
	
		UInt32	theMask = mMessageMask;
		
		theMask |= ( 1 << (inMessageType - 1) );
		
		if ( inRefresh )
			SetMessageMask( theMask );
		
	}
	
	
void
CMacCVSMessageView::DontShowMessageType( const UInt32 inMessageType, const Boolean inRefresh )
	{

		UInt32	theMask = mMessageMask;
	
		if ( (theMask & (1 << (inMessageType - 1))) != 0 )
			theMask ^= ( 1 << (inMessageType - 1) );
		
		if ( inRefresh )
			SetMessageMask( theMask );
	
	}
	
	
void 
CMacCVSMessageView::SetMessageMask( const UInt32 inNewMask )
	{
		
		TableIndexT 										outRows		= 0x00000000,
																		outCols		= 0x00000000;
	
		UInt32													ioCount   = sizeof( CMacCVSMessageViewCellDataRec	);														
		CMacCVSMessageViewCellDataRec		cellData 	= { 0, 0, NULL, 0, NULL };
		
		//	Set the new mask

		if ( mMessageMask != inNewMask )
			{
			
				mMessageMask = inNewMask;

				RemoveAllRows( true );
		
		//	Iterate over all of the messages, adding the 
		//	appropriate ones to be displayed.
		
				UInt32	totalMessageCount = mMessageItems->GetCount();
		
				for ( UInt32 currentMessage = 1; currentMessage <= totalMessageCount; currentMessage++ ) 
					{
					
						ioCount = sizeof( CMacCVSMessageViewCellDataRec	);
							
						mMessageItems->FetchItemAt( currentMessage, &cellData, ioCount );
					
						Assert_( ioCount == sizeof(CMacCVSMessageViewCellDataRec) );
						
						ioCount = sizeof( CMacCVSMessageViewCellDataRec );
						
						if ( IsMessageTypeEnabled( cellData.fCellType ))
							InsertRows( 1, 0x7FFFFFFF, &cellData, ioCount, true );
				
					}
				
			}
				
	}


void 
CMacCVSMessageView::ClearMessages()
	{

			DeleteAllMessages();
	
	}


void
CMacCVSMessageView::PostMessage( const UInt32 inMessageType, const char *inMessageText, const UInt32 inMessageLen, const void *inMessageData, const UInt32 inDataSize)
	{

		UInt32												ioCount			= sizeof( CMacCVSMessageViewCellDataRec );
		CMacCVSMessageViewCellDataRec newCellData	= { 0, 0, NULL, 0, NULL };
		
		newCellData.fCellType = inMessageType;
		
		Assert_( (inMessageText != NULL) && (inMessageLen > 0) );
		
		if ( (inMessageText != NULL) && (inMessageLen > 0) )
			{
				
				newCellData.fCellTextPtr = ::NewPtr( inMessageLen );
				
				if ( newCellData.fCellTextPtr == NULL )
					throw ( ::MemError() );
					
				newCellData.fCellTextLen = inMessageLen;
				
				::BlockMoveData( inMessageText, newCellData.fCellTextPtr, newCellData.fCellTextLen );
				
				if ( inDataSize > 0 )
				{
						newCellData.fCellData = ::NewPtr( inDataSize );
						if ( newCellData.fCellData == NULL )
							throw ( ::MemError() );
						
						::BlockMoveData( inMessageData, newCellData.fCellData, inDataSize );
				}
				
//	Tweak file messages
				
				if ( inMessageType == kMessageTypeFile )
				{
						if ( inMessageText[1] == ' ' )
						{
								if ( inMessageText[0] == 'U' )
										newCellData.fCellFlags |= kFileMessageUpdatedFlag;
								else if ( inMessageText[0] == 'M' )
										newCellData.fCellFlags |= kFileMessageMergedFlag;
								else if ( inMessageText[0] == 'C' )
										newCellData.fCellFlags |= kFileMessageConflictFlag;
						}
				}

//	Add to our master list

				mMessageItems->InsertItemsAt( 1, LArray::index_Last, &newCellData, ioCount );
			
//	If the current message mode is enabled, then 
//	add it to the table.
		
				ioCount = sizeof( CMacCVSMessageViewCellDataRec );
		
				if ( IsMessageTypeEnabled(newCellData.fCellType) )
					{

						InsertRows(1, LArray::index_Last, &newCellData, ioCount, true );

						if ( mScrollToNewMessages )
							{
							
								TableIndexT		outRows = 0;
								TableIndexT		outCols = 0;

								GetTableSize( outRows, outCols );
								
								STableCell	theNewCell( outRows, outCols );
								
								ScrollCellIntoFrame( theNewCell );
							}
					}
					
			}
						
	}	


void
CMacCVSMessageView::DeleteAllMessages()
	{		
	
		CMacCVSMessageViewCellDataRec		theCurItem = { 0, 0, NULL, 0, NULL };

		RemoveAllRows( true );
		
		if ( mMessageItems != NULL )
			{
			
				LArrayIterator	itemIterator( *mMessageItems );
		
				while ( itemIterator.Next(&theCurItem) )
					{
					
						if ( theCurItem.fCellTextPtr != NULL )
							::DisposePtr( theCurItem.fCellTextPtr );
							
						if ( theCurItem.fCellData != NULL )
							::DisposePtr( (Ptr) theCurItem.fCellData );
					
					}
					
				mMessageItems->RemoveItemsAt( mMessageItems->GetCount(), LArray::index_First );
					
			}
}
	

	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark LCOMMANDER OVERRIDES


void 
CMacCVSMessageView::FindCommandStatus( CommandT inCommand, Boolean &outEnabled, Boolean &outUsesMark, Char16 &outMark, Str255 outName )
	{
	
		switch ( inCommand )
			{
			
				case cmd_Copy:
					{
					
						STableCell	cell(0, 0);
						GetNextSelectedCell( cell );
						
						if ( cell.row != 0 )
							outEnabled = true;
					
					}
					
				break;
				
				default:
				
					LCommander::FindCommandStatus( inCommand, outEnabled, outUsesMark, outMark, outName );
			
				break;
				
			}
		
	}
		
		
Boolean 
CMacCVSMessageView::ObeyCommand( CommandT inCommand, void *ioParam )
	{
	
		Boolean	bCmdHandled = false;
		
		switch ( inCommand )
			{
			
				case cmd_Copy:
					{
						STableCell 	theCurCell( 0, 0 );
						Boolean			clearData = true;
						
						while ( GetNextSelectedCell(theCurCell) )
							{
									CMacCVSMessageViewCellDataRec		theCellData;
									
									Uint32	cellDataSize = sizeof( CMacCVSMessageViewCellDataRec );
									GetCellData( theCurCell, &theCellData, cellDataSize);
									
									if ( cellDataSize == sizeof( CMacCVSMessageViewCellDataRec ) && theCellData.fCellTextLen > 0)
										{
											LClipboard::GetClipboard()->SetData('TEXT', theCellData.fCellTextPtr, theCellData.fCellTextLen, clearData );
											clearData = false;
										}
							}
							
					}
					break;
				
				default:
				
					bCmdHandled = LCommander::ObeyCommand( inCommand, ioParam);
			
					break;
				
			}
			
		return bCmdHandled;
			
	}
	
