//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSMessageWindow.cp
	
	Authors:		Ram Rajadhyaksha and others (unknown)
	
	Description:
	
		This file contains code to implement the message window in MacCVS.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Added Andy Nicholas' grayscale changes.
							
-------------------------------------------------------------------------------- */


#include <UDesktop.h>

#include "CMacCVSMessageView.h"
#include "CMacCVSMessageWindow.h"

#include "CMacCVSProjectView.h"


CMacCVSMessageWindow::
CMacCVSMessageWindow(LStream *inStream):
	
	CMacCVSWindow(inStream)
	
	{

		mErrorButton 					= NULL;
		mWarningButton 				= NULL;
		mMessageButton 				= NULL;
		mFileButton						= NULL;
		
		mErrorCountCaption 		= NULL;
		mWarningCountCaption 	= NULL;
		mMessageCountCaption	= NULL;
		mFileCountCaption			= NULL;
		
		mMessageView 					= NULL;
		mOwnerDoc							= NULL;
		
		mErrorItemsCount 			= 0;
		mWarningItemsCount 		= 0;
		mMessageItemsCount 		= 0;
		mFileItemsCount				= 0;
		
		mWindowPosResID				= kMessageWindowPositionResID;
	}


CMacCVSMessageWindow::
~CMacCVSMessageWindow()
	{

	}


CMacCVSMessageWindow *
CMacCVSMessageWindow::CreateCMacCVSMessageWindowFromStream( LStream *inStream )
	{
	
		return new CMacCVSMessageWindow( inStream );
	
	}


CMacCVSMessageWindow *
CMacCVSMessageWindow::CreateCMacCVSMessageWindow( CMacCVSDoc *inOwnerDoc, const Str255 inDescriptor, const Boolean inShow, const Boolean inScrollMessages )
	{
	
		CMacCVSMessageWindow	*resultPtr = NULL;
		
		resultPtr = dynamic_cast<CMacCVSMessageWindow *>( LWindow::CreateWindow( CMacCVSMessageWindow::kMessageWindowResID, (LCommander *)inOwnerDoc ) );
		ThrowIfNil_( resultPtr );
				
		if ( inDescriptor != NULL )
			resultPtr->SetDescriptor( inDescriptor );
		
		resultPtr->SetOwnerDoc(inOwnerDoc);
		resultPtr->SetWindowPosition();
		
		if ( inShow )
			resultPtr->Show();
				
		resultPtr->SetScrollToNewMessages( inScrollMessages );
		
		return resultPtr;
	
	}
	
	
void 
CMacCVSMessageWindow::FinishCreateSelf()
	{

		mErrorCountCaption = dynamic_cast<LCaption *>( FindPaneByID(kErrorCountCaptionID) );
		ThrowIfNil_( mErrorCountCaption );

		mWarningCountCaption = dynamic_cast<LCaption *>( FindPaneByID(kWarningCountCaptionID) );
		ThrowIfNil_( mWarningCountCaption );

		mMessageCountCaption = dynamic_cast<LCaption *>( FindPaneByID(kMessageCountCaptionID) );
		ThrowIfNil_( mMessageCountCaption );
		
		mFileCountCaption = dynamic_cast<LCaption *>( FindPaneByID(kFileCountCaptionID) );
		ThrowIfNil_( mFileCountCaption );

		mErrorButton = dynamic_cast<LBevelButton *>( FindPaneByID(kErrorButtonPaneID) );
		ThrowIfNil_( mErrorButton );
		
		mErrorButton->AddListener( this );
		
		mWarningButton = dynamic_cast<LBevelButton *>( FindPaneByID(kWarningButtonPaneID) );
		ThrowIfNil_( mWarningButton );
		
		mWarningButton->AddListener(this);

		mMessageButton = dynamic_cast<LBevelButton *>( FindPaneByID(kMessageButtonPaneID) );
		ThrowIfNil_( mMessageButton );
		
		mMessageButton->AddListener(this);

		mFileButton = dynamic_cast<LBevelButton *>( FindPaneByID(kFileButtonPaneID) );
		ThrowIfNil_( mFileButton );
		
		mFileButton->AddListener(this);
		
		mClearMessagesButton = dynamic_cast<LPushButton *>( FindPaneByID(kClearMessagesButtonID) );
		ThrowIfNil_( mClearMessagesButton );
		
		mClearMessagesButton->AddListener(this);
		
		mMessageView = dynamic_cast<CMacCVSMessageView *>( FindPaneByID(CMacCVSMessageView::class_ID) );
		ThrowIfNil_( mMessageView );

		if ( mErrorButton->GetValue() )
			mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeError );
		else
			mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeError );

		if ( mWarningButton->GetValue() )
			mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeWarning );
		else
			mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeWarning );	

		if ( mMessageButton->GetValue() )
			mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeMessage );
		else
			mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeMessage );	

		if ( mFileButton->GetValue() )
			mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeFile );
		else
			mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeFile );	
		
		mMessageView->AddListener(this);		//for message that the window re-broadcasts to the doc
		
		SetLatentSub(mMessageView);

	}



//################################################################################
//################################################################################
#pragma mark -
#pragma mark LISTENING


void 
CMacCVSMessageWindow::ListenToMessage( MessageT inMessage, void *ioParam )
	{
	
		UInt32		newMask = 0;
		
		switch ( inMessage )
			{
			
				case kErrorButtonValueMessage:
				
					if ( mErrorButton->GetValue() )
						mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeError );
					else
						mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeError );
					
				break;
				
				case kWarningButtonValueMessage:

					if ( mWarningButton->GetValue() )
						mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeWarning );
					else
						mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeWarning );	
				
				break;
				
				case kMessageButtonValueMessage:
				
					if ( mMessageButton->GetValue() )
						mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeMessage );
					else
						mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeMessage );	
				
				break;
			
				case kFileButtonValueMessage:
					
					if ( mFileButton->GetValue() )
						mMessageView->ShowMessageType( CMacCVSMessageView::kMessageTypeFile );
					else
						mMessageView->DontShowMessageType( CMacCVSMessageView::kMessageTypeFile );	
				
				break;
				
				case kClearMessagesButtonMessage:
						ClearMessages();
				break;
					
				case CMacCVSProjectView::kNewViewMessage:
				case CMacCVSProjectView::kNewViewSelectFileMessage:
					// rebroadcast to the document
					BroadcastMessage( inMessage, ioParam );
				break;
			}
				
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark LCOMMANDER OVERRIDES


Boolean
CMacCVSMessageWindow::ObeyCommand( CommandT inCommand, void *ioParam )
	{
	
		Boolean		bResult = false;
		
		if (inCommand == cmd_Close) 
			{
			
				Hide();
			
				bResult = true;
		
			} 
			
		else bResult = LWindow::ObeyCommand( inCommand, ioParam );

		return bResult;
		
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


void 
CMacCVSMessageWindow::ClearMessages()
	{

		mMessageView->ClearMessages();

		mErrorItemsCount = 0;
		mWarningItemsCount = 0;
		mMessageItemsCount = 0;
		mFileItemsCount = 0;
		
		mErrorCountCaption->SetValue( mErrorItemsCount );
		mWarningCountCaption->SetValue( mWarningItemsCount );
		mMessageCountCaption->SetValue( mMessageItemsCount );
		mFileCountCaption->SetValue( mFileItemsCount );
		
	}


void
CMacCVSMessageWindow::PostMessage( const UInt32 inMessageType, const char *inMessageText, const UInt32 inMessageLen, const void *inMessageData, const UInt32 inDataSize )
	{

//	If the message window is not visible, then make sure
//	we check the box for the message type that we are about
//	to show.

		UInt32		theMask = 0x00000000;
		
		if ( !IsVisible() ) 
			{
			
				switch ( inMessageType ) 
					{
					
						case CMacCVSMessageView::kMessageTypeError:
							mErrorButton->SetValue(1);
						break;
					
						case CMacCVSMessageView::kMessageTypeWarning:
							mWarningButton->SetValue(1);
						break;
				
						case CMacCVSMessageView::kMessageTypeMessage:
							mMessageButton->SetValue(1);
						break;
			
						case CMacCVSMessageView::kMessageTypeFile:
							mFileButton->SetValue(1);
						break;
						
						default:
						break;
					
					}

//  We don't need the next line since setting the value of
//  a LGAIconButton will cause it to broadcast.
					
//  mMessageView->ShowMessageType( inMessageType );
				
			}
		
//	And make sure that the message window is showing.
				
		ShowMessageWindow();
		mMessageView->PostMessage( inMessageType, inMessageText, inMessageLen, inMessageData, inDataSize );

		switch ( inMessageType )
			{
			
				case CMacCVSMessageView::kMessageTypeError:
					mErrorItemsCount++;
					mErrorCountCaption->SetValue( mErrorItemsCount );
				break;
				
				case CMacCVSMessageView::kMessageTypeWarning:
					mWarningItemsCount++;
					mWarningCountCaption->SetValue( mWarningItemsCount );
				break;
				
				case CMacCVSMessageView::kMessageTypeMessage:
					mMessageItemsCount++;
					mMessageCountCaption->SetValue( mMessageItemsCount );
				break;
			
				case CMacCVSMessageView::kMessageTypeFile:
					mFileItemsCount++;
					mFileCountCaption->SetValue( mFileItemsCount );
				break;
			}
		
	}


UInt32
CMacCVSMessageWindow::GetMessagesCount()
	{
	
		return ( mErrorItemsCount + mWarningItemsCount + mMessageItemsCount );
	
	}
	
	
void
CMacCVSMessageWindow::ShowMessageWindow()
	{

		if ( !IsVisible() ) 
			{
			
				Show();
				UDesktop::SelectDeskWindow( this );
				
			}

	}


void
CMacCVSMessageWindow::SetScrollToNewMessages(	Boolean inScroll)
	{
	
		Assert_(mMessageView);
		
		mMessageView->SetScrollToNewMessages( inScroll );
	
	}
