//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSOperationTask.cp
	
	Authors:		Ram Rajadhyaksha and Simon Fraser
	
	Description:
	
		Code to implement the basic CVS operation in MacCVS Pro.
		 
	Modification History:

		17 Feb 1999			Ram Rajadhyaksha
		
					CMacCVSOperationTask now inherits from CMacCVSThreadedTask.
							
-------------------------------------------------------------------------------- */


#include <stdio.h>
#include <string.h>
#include "ThrowOSErr.h"

#include "CMacCVSMessageView.h"
#include "CMacCVSMessageWindow.h"
#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"
#include "UProjectActions.h"

#include "CMacCVSOperationTask.h"


CMacCVSOperationTask::
CMacCVSOperationTask(	LStream *inStream, const UInt32 inStackSize ):
	
	CMacCVSThreadedTask( inStream, inStackSize )
	
	{
		
		OSErr		result			= noErr;
		LArray 	*pMappings 	= NULL;

		mMappings						= NULL;
		mMappingsCount			= 0;
	
		mMessageOutStream 	= NULL;
		mErrorOutStream 		= NULL;
		mAdditionalPostData	= NULL;
		
		try
			{
				pMappings = mOwnerDoc->GetFileMappingList();
				
				if ( pMappings != NULL )
					{				
						result = mOwnerDoc->BuildFileMappingList( pMappings, &mMappings, &mMappingsCount );		
						ThrowIfOSErr_( result );
						
						delete pMappings;
						pMappings = NULL;
					}
					
				mOwnerDoc->AddToTaskList( this );
			}
		catch ( ... )
			{
				if ( pMappings != NULL )
					{
						delete pMappings;
						pMappings = NULL;
					}
					
				throw;
			}
			
	}


CMacCVSOperationTask::CMacCVSOperationTask( CMacCVSDoc *inOwnerDoc, const UInt32 inStackSize ):
	
	CMacCVSThreadedTask( inOwnerDoc, inStackSize )
	
	{
	
		OSErr		result			= noErr;
		LArray 	*pMappings 	= NULL;

		mMappings						= NULL;
		mMappingsCount			= 0;
	
		mMessageOutStream 	= NULL;
		mErrorOutStream 		= NULL;
		mAdditionalPostData	= NULL;
		
		try
			{
				pMappings = mOwnerDoc->GetFileMappingList();
				
				if ( pMappings != NULL )
					{				
						result = mOwnerDoc->BuildFileMappingList( pMappings, &mMappings, &mMappingsCount );		
						ThrowIfOSErr_( result );
						
						delete pMappings;
						pMappings = NULL;
					}
					
				mOwnerDoc->AddToTaskList( this );
			}
		catch ( ... )
			{
				if ( pMappings != NULL )
					{
						delete pMappings;
						pMappings = NULL;
					}
					
				throw;
			}

	}
	
	
CMacCVSOperationTask::~CMacCVSOperationTask()
	{	
		if ( mMappings != NULL )
			{
				::DisposePtr( (Ptr)mMappings );
				mMappings = NULL;
			}
			
		mOwnerDoc->RemoveFromTaskList( this );		
	}


// Calling this with inDataLen > 0 creates or appends data to the message out stream
// Calling with inDataLen==0 posts the message and disposes of the message out stream
pascal SInt32
CMacCVSOperationTask::MessageOutNotifier( const CCVSEngineRef inEngineRef, const UInt32 inDataType, const UInt32 inDataLen, const void *inData, void *inContextPtr )
	{
	
		OSErr			result					= noErr;
		
		SInt32		intOffset 			= 0;
		SInt32		intBytesWritten	= 0;
		SInt32		intNewLen 			= 0;
															

		CMacCVSOperationTask	*theTask = (CMacCVSOperationTask *)inContextPtr;
		Assert_( theTask != NULL );
		
		if ( theTask != NULL )
			{		
				try
					{								
						if ( inDataLen > 0 )
							{						
								switch (inDataType)
									{
										case kEngineNotifierTextDataType:
											
											if ( theTask->mMessageOutStream != NULL ) 
												{
													intOffset = ::GetHandleSize( theTask->mMessageOutStream );
													::SetHandleSize( theTask->mMessageOutStream, intOffset + inDataLen );
													result = ::MemError();
													
													ThrowOSErr( result );															
												} 
											else 
												{
													theTask->mMessageOutStream = ::NewHandle( inDataLen );
													
													if ( theTask->mMessageOutStream == NULL )
														throw( ::MemError() );
												}
										
											::BlockMoveData( inData, ((UInt8 *)(*theTask->mMessageOutStream)) + intOffset, inDataLen );
											intBytesWritten = inDataLen;
										break;
											
										case kEngineNotifierFSSpecDataType:
											
											if ( inDataLen != sizeof(FSSpec) )
												break;
													
											if ( theTask->mAdditionalPostData != NULL )
												{
													::DisposeHandle( theTask->mAdditionalPostData );
													theTask->mAdditionalPostData = NULL;
												}
											
											Handle tempHand = NULL;
											
											result = ::PtrToHand( inData, &tempHand, inDataLen);
											ThrowOSErr( result );
												
											theTask->mAdditionalPostData = tempHand;
										
										break;
											
										default:
										
											Assert_(false);
										
										break;
								
									}
							}
						else if ( inDataLen == 0 )
							{
							
								Assert_( theTask->mMessageOutStream != NULL );
								
								if ( theTask->mMessageOutStream != NULL )
									{
										Ptr			moreDataPtr = NULL;
										UInt32	messageSize = ::GetHandleSize(theTask->mMessageOutStream);
										UInt32	dataSize = 0;

										::HLock( theTask->mMessageOutStream );
										
										if (theTask->mAdditionalPostData != NULL)
											{
												dataSize = ::GetHandleSize(theTask->mAdditionalPostData);
												::HLock( theTask->mAdditionalPostData );
												moreDataPtr = *theTask->mAdditionalPostData;
											}	
										
										theTask->mOwnerDoc->PostMessage( CMacCVSMessageView::kMessageTypeFile, *theTask->mMessageOutStream,
												messageSize, moreDataPtr, dataSize);
									
										::DisposeHandle( theTask->mMessageOutStream );
										theTask->mMessageOutStream = NULL;
										
										if (theTask->mAdditionalPostData != NULL)
											{
												::DisposeHandle( theTask->mAdditionalPostData );
												theTask->mAdditionalPostData = NULL;
											}										
									}
							}
							
					}
				catch ( ... )
					{			
						if ( theTask->mMessageOutStream != NULL )
							{
								::DisposeHandle( theTask->mMessageOutStream );
								theTask->mMessageOutStream = NULL;
							}
							
						if ( theTask->mAdditionalPostData != NULL )
							{
								::DisposeHandle( theTask->mAdditionalPostData );
								theTask->mAdditionalPostData = NULL;
							}							
					}					
			}
	
		return inDataLen;
		
	}
	

pascal SInt32
CMacCVSOperationTask::ErrorOutNotifier( const CCVSEngineRef inEngineRef, const UInt32 inDataType, const UInt32 inDataLen, const void *inData, void *inContextPtr )
	{
	
		OSErr			result					= noErr;
		
		SInt32		intOffset 			= 0;
		SInt32		intBytesWritten	= 0;
		SInt32		intNewLen 			= 0;
															

		CMacCVSOperationTask *theTask = (CMacCVSOperationTask *)inContextPtr;
		Assert_( theTask != NULL );
		
		if ( theTask != NULL )
			{			
				try
					{								
						if ( inDataLen > 0 )
							{				
								switch ( inDataType )
									{
								
										case kEngineNotifierTextDataType:
											
											if ( theTask->mErrorOutStream != NULL )
												{
													intOffset = ::GetHandleSize( theTask->mErrorOutStream );
													
													::SetHandleSize( theTask->mErrorOutStream, intOffset + inDataLen );
													
													result = ::MemError();
													ThrowOSErr( result );
												}
											else
												{
													theTask->mErrorOutStream = ::NewHandle( inDataLen );
													
													if ( theTask->mErrorOutStream == NULL )
														throw( ::MemError() );
												}
											
											::BlockMoveData( inData, ((UInt8 *)(*theTask->mErrorOutStream)) + intOffset, inDataLen );
											intBytesWritten = inDataLen;
											
										break;
										
										case kEngineNotifierFSSpecDataType:
											
											if ( inDataLen != sizeof(FSSpec) )
												break;
													
											if ( theTask->mAdditionalPostData != NULL )
												{
													::DisposeHandle( theTask->mAdditionalPostData );
													theTask->mAdditionalPostData = NULL;
												}
											
											Handle		tempHand = NULL;
											
											result = ::PtrToHand( inData, &tempHand, inDataLen);
											ThrowOSErr( result );
												
											theTask->mAdditionalPostData = tempHand;
										
										break;
												
										default:
												Assert_(false);
										break;
									}						
							}
						else if ( inDataLen == 0 )
							{
							
								Assert_( theTask->mErrorOutStream != NULL );
								
								if ( theTask->mErrorOutStream != NULL )
									{
										Ptr			moreDataPtr = NULL;
										UInt32 	messageSize = ::GetHandleSize( theTask->mErrorOutStream );
										UInt32	dataSize = 0;
										
										::HLock( theTask->mErrorOutStream );
										
										if (theTask->mAdditionalPostData != NULL)
											{
												::HLock( theTask->mAdditionalPostData );
												moreDataPtr = *theTask->mAdditionalPostData;
												dataSize = ::GetHandleSize(theTask->mAdditionalPostData);
											}	
										
										theTask->mOwnerDoc->PostMessage( CMacCVSMessageView::kMessageTypeWarning, *theTask->mErrorOutStream, messageSize, moreDataPtr, dataSize );
									
										::DisposeHandle( theTask->mErrorOutStream );
										theTask->mErrorOutStream = NULL;
										
										if (theTask->mAdditionalPostData != NULL)
											{
												::DisposeHandle( theTask->mAdditionalPostData );
												theTask->mAdditionalPostData = NULL;
											}								
									}
							}				
					}
				catch ( ... )
					{					
						if ( theTask->mErrorOutStream != NULL )
							{
								::DisposeHandle( theTask->mErrorOutStream );
								theTask->mErrorOutStream = NULL;
							}
							
						if ( theTask->mAdditionalPostData != NULL )
							{
								::DisposeHandle( theTask->mAdditionalPostData );
								theTask->mAdditionalPostData = NULL;
							}
					}					
			}
	
		return inDataLen;
		
	}

