//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CMacCVSOrphanFilesTask.cp
//	
//	  	Authors: 	Simon Fraser
//
//################################################################################


#include <string.h>

#include <LArray.h>

#include "MoreFilesExtras.h"			// for DeleteDirectory

#include "CMacCVSDoc.h"
#include "CVSEngineUtils.h"
#include "CMultiProgressWindow.h"
#include "CMacCVSOrphanFilesTask.h"


UInt32 CMacCVSOrphanFilesTask::sYieldCount = 0;


CMacCVSOrphanFilesTask::CMacCVSOrphanFilesTask( CMacCVSDoc *inOwnerDoc, const CMacCVSOrphanFilesQueryRec *inQuery, const LArray *inTargetItems ):

	CMacCVSThreadedTask( inOwnerDoc, kDefaultOrphanFilesThreadStackSize),
		mItemList( sizeof(FSSpec) )
		
	{
		mProgressID = 0;
		mFilesCleaned = 0;
		
		try
			{		
				ThrowIfNil_( inQuery );
				ThrowIfNil_( inTargetItems );
				
				mQuery = *inQuery;
				
				if ( inTargetItems->GetCount() > 0 )
					{
						mItemList = *inTargetItems;			
					}
					
				//  Even though we aren't a real network transaction operation
				//  we want to add ourselves to the list because if the user
				//  closes the session, we want them to be warning before aborting
				//  this thread.
				
				ThrowIfNil_( mOwnerDoc );
				mOwnerDoc->AddToTaskList( this );				
			}		
		catch ( ... )
			{			
				throw;	
			}			
	}
	
	
CMacCVSOrphanFilesTask::~CMacCVSOrphanFilesTask()
	{
		try
			{
				mOwnerDoc->RemoveFromTaskList( this );
			}
		catch ( ... )
			{
			}
			
		try
			{
				if ( mProgressID != 0 )
					{
						CMultiProgressWindow *pProgressWindow = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWindow != NULL );
						
						if ( pProgressWindow != NULL )
							{
								pProgressWindow->RemoveListener( this );
							}
							
						CMultiProgressWindow::DeleteGlobalProgressIndicator( mProgressID );
					}
			}
		catch ( ... )
			{
			}
	}
	

#pragma mark -


void
CMacCVSOrphanFilesTask::NoteFileFolderCleaned( ConstStr255Param inFileFolderName )
	{	
		CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
		Assert_( pProgressWin != NULL );
		
		mFilesCleaned ++;
	
		if ( pProgressWin != NULL )
			{			
				pProgressWin->SetProgressCountText( mProgressID, mFilesCleaned );
			
				pProgressWin->SetCurrentItemText( mProgressID, inFileFolderName );			
			}
	}
	
	
	
void
CMacCVSOrphanFilesTask::RunSelf()
	{

		OSErr					result 				= noErr;		
		FSSpec				theCurItem	  = { 0, 0, "\p" };
		UInt32				ioCount				= sizeof( FSSpec );

		try
			{
				mProgressID = CMultiProgressWindow::NewGlobalProgressIndicator();
				CMultiProgressWindow *pProgressWin =  CMultiProgressWindow::GetGlobalCMultiProgressWindow();
				ThrowIfNil_( pProgressWin );
				pProgressWin->AddListener( this );

				LStr255			pstrTitle("\pOrphaning: ");
				LStr255			pstrDocName;
				
				mOwnerDoc->GetDescriptor( pstrDocName );
				pstrTitle += pstrDocName;

				pProgressWin->SetProgressMessageText( mProgressID, pstrTitle );
				pProgressWin->SetCurrentItemText( mProgressID, "\pCleaning files & folders" );
				pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );

				if ( mQuery.fOrphanEntireTree )
					{
						FSSpec		projectRootSpec;
						
						mOwnerDoc->GetProjectRootFileSpec(projectRootSpec);
							
						Boolean			removedDir;
						
						DoOrphanFiles( &projectRootSpec, removedDir );						
					}
				else
					{
						LArrayIterator	itemIterator( mItemList );
						
						while ( itemIterator.Next(&theCurItem, ioCount) )
							{						
								Assert_( ioCount == sizeof(FSSpec) );
								ioCount = sizeof( FSSpec );
								
								Boolean			removedDir;
								
								DoOrphanFiles( &theCurItem, removedDir );				
							}
					}
			}					
		catch ( ... )
			{
				
			}

	}
	
	
void
CMacCVSOrphanFilesTask::DoOrphanFiles( const FSSpec *inFileSpec, Boolean &outRemovedDirectory )
	{
	
		OSErr					error				= noErr;
		FSSpec				childSpec		= { 0, 0, "\p" };
		UInt32				maxIndex		= 0;
		CInfoPBRec		pb;
	
		Assert_( inFileSpec != NULL );
		
		if ( ::StackSpace() < kMinimumOrphanFilesThreadStackSize )
			throw( (OSErr)memFullErr );
		
		if ( (sYieldCount % kDefaultYieldCount) == 0 )
			CThreadedTask::Yield();
			
		sYieldCount++;
		
		try
			{
				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioFDirIndex 	= 0;
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inFileSpec->name;
				pb.hFileInfo.ioVRefNum 		= inFileSpec->vRefNum;
				pb.hFileInfo.ioDirID 			= inFileSpec->parID;
		
				error = ::PBGetCatInfoSync( &pb );
		
				if ( error != noErr )
					throw error;
						
				if ((pb.dirInfo.ioFlAttrib & ioDirMask) != 0)
					{
					
						// if this is the CVS folder, then remove it
						
						if (  mQuery.fRemoveCvsDirecories && ::EqualString(inFileSpec->name, "\pCVS", false, true) )
							{
							
								// use MoreFiles DeleteDirectory
								error = ::DeleteDirectory( inFileSpec->vRefNum, inFileSpec->parID, inFileSpec->name );
								
								if ( error != noErr )
									throw error;
									
								outRemovedDirectory = true;
								
								NoteFileFolderCleaned( inFileSpec->name );
								
							}
							
						// else if this is a directory, iterate through all the items in the
						// directory.
						
						else if ( CUIsRepositoryDir(pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID) )
							{
								UInt32			index = 1;
								
								while ( error == noErr )
									{

										CInfoPBRec		childPb;
										FSSpec				childSpec;
										
										childPb.hFileInfo.ioCompletion = NULL;
										childPb.hFileInfo.ioFDirIndex 	= 0;
										childPb.hFileInfo.ioNamePtr 		= childSpec.name;
										childPb.hFileInfo.ioVRefNum 		= pb.hFileInfo.ioVRefNum;
										childPb.hFileInfo.ioDirID 			= pb.hFileInfo.ioDirID;
										childPb.hFileInfo.ioFDirIndex 	= index;
										
										error = ::PBGetCatInfoSync( &childPb );
										if ( error != noErr) break;
											
										childSpec.vRefNum = childPb.hFileInfo.ioVRefNum;
										childSpec.parID = pb.hFileInfo.ioDirID;				// note the parent pb
										
										Boolean			removedDir = false;
										
										DoOrphanFiles( &childSpec, removedDir );
									
										if ( ! removedDir )
											index ++;
											
									}		// while
									
							}	// if repository dir
					
					}
				else
					{						
						error = FSpRemoveCkidMcvsResource( inFileSpec, mQuery.fRemoveCkid, mQuery.fRemoveMcvs );
						
						if ( error != noErr )
							throw error;

						NoteFileFolderCleaned( inFileSpec->name );						
					}					
			}			
		catch ( ... )
			{
				throw;			
			}
			
	}
	

void		
CMacCVSOrphanFilesTask::ListenToMessage( MessageT inMessage, void *ioParam )
	{
		if ( ioParam != NULL )
			{
				UInt32 intID = *(UInt32 *)ioParam;
				
				if ( (intID == mProgressID) && (inMessage == msg_Cancel) )
					{
						ShutdownThread();
					}	
			}	
	}
	
	
