//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################


/* -------------------------------------------------------------------------------

	File:				CMacCVSProjectViewFile.cp
	
	Authors:		Dan Clifford, Ram Rajadhyaksha
	
	Description:
	
		This file contains code for the CMacCVSProjectViewFile object.
		 
	Modification History:

		15 Dec	1998		Ram Rajadhyaksha
		
					Fixed bug where ValidateContents() would not return error on
					a file in the root directory that has been renamed. Line 372.
							
-------------------------------------------------------------------------------- */


#include <stdio.h>
#include <string.h>

#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "CVSEngineUtils.h"

#include "CMacCVSProjectViewFolder.h"
#include "CMacCVSProjectViewFile.h"


CMacCVSProjectViewFile::
CMacCVSProjectViewFile(	CMacCVSProjectViewItem		*inParentItem, 
												const FSSpec 							*inFileSpec,
												Boolean										inIsAlias): 
							
	CMacCVSProjectViewItem( (CMacCVSProjectViewItem *)inParentItem, inFileSpec, inIsAlias)
		
	{
		
		mCachedProjectorStatus 	= kMacCVSProjectorStatusUnknown;
		
		mBinaryData							= false;
		
		mFileVersionString[0]		= 0x00;
			
	}


CMacCVSProjectViewFile::
~CMacCVSProjectViewFile()
	{	

	}


CMacCVSProjectViewItem *
CMacCVSProjectViewFile::CreateInstance( CMacCVSProjectViewItem		*inParentItem, 
																				const FSSpec 							*inFileSpec)
	{
		
		return new CMacCVSProjectViewFile( inParentItem, inFileSpec );
		
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS



Boolean
CMacCVSProjectViewFile::CanBeAdded()
	{
	
		Boolean	result = false;
		
		//	You can only add a file if it's status is unowned.
		ProjectItemStatus theStatus = GetProjectItemStatus();
		
		if ( !IsCVSContainer() )
			{		
				if ( mParentItem != NULL )
					{					
						if ( mParentItem->IsCVSContainer() )
							{							
								if ( IsContainer() )
									result = true;									
								else if ( theStatus == kMacCVSProjectorStatusUnowned )
									result = true;	
								else if ( (mFileVersionString[0] > 0) && (mFileVersionString[1] == '-') )
									result = true;								
							}						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							{						
								if ( IsContainer() )
									result = true;									
								else if ( theStatus == kMacCVSProjectorStatusUnowned )
									result = true;									
								else if ( (mFileVersionString[0] > 0) && (mFileVersionString[1] == '-') )
									result = true;								
							}					
					}					
			}
			
		return result;
		
	}


Boolean
CMacCVSProjectViewFile::CanCheckin()
	{
		
//	You can checkin any folder, or any item that is either 
//	MRO'ed or checked-out exclusive.

		Boolean						result		= false;
		ProjectItemStatus theStatus = GetProjectItemStatus();
			
		if ( mParentItem != NULL )
			{
			
				if ( mParentItem->IsCVSContainer() )
					if ( (theStatus == kMacCVSProjectorStatusMRO) || (theStatus == kMacCVSProjectorStatusCheckedOut) )
						result = true;
					else if ( CUIsAddedOrRemovedRevision(mFileVersionString) )
						result = true;
				
			}
		else
			{

//  Else if there is no tree hierarchy, check ourselves.
					
				if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
					if ( (theStatus == kMacCVSProjectorStatusMRO) || (theStatus == kMacCVSProjectorStatusCheckedOut) ) 
						result = true;
					else if ( CUIsAddedOrRemovedRevision(mFileVersionString) )
						result = true;
		
			}
				
		return result;

	}


Boolean
CMacCVSProjectViewFile::CanCompareWithOriginal()
	{
	
		Boolean	bResult = false;
	
		if ( CMacCVSProjectViewItem::CanCompareWithOriginal() )
			if ( !CUIsAddedOrRemovedRevision(mFileVersionString) )
				if ( !mBinaryData )
					bResult = true;
				
		return bResult;
		
	}
	
	
Boolean
CMacCVSProjectViewFile::CanDiff()
	{
	
		Boolean bResult = CMacCVSProjectViewItem::CanDiff();
		
		if ( bResult )
			{
				if ( CUIsAddedOrRemovedRevision(mFileVersionString) )
					bResult = false;			
			}
			
		return bResult;
		
	}

	
Boolean
CMacCVSProjectViewFile::CanRemove()
	{
	
		Boolean bResult = CMacCVSProjectViewItem::CanRemove();
		
		if ( bResult )
			{
				if ( CUIsAddedOrRemovedRevision(mFileVersionString) )
					bResult = false;			
			}
			
		return bResult;
		
	}


Boolean
CMacCVSProjectViewFile::CanRevert()
	{
	
		Boolean bResult = CMacCVSProjectViewItem::CanRevert();
		
		if ( bResult )
			{
				if ( CUIsAddedOrRemovedRevision(mFileVersionString) )
					bResult = false;			
			}
			
		return bResult;
		
	}
	

Boolean
CMacCVSProjectViewFile::CanTag()
	{
	
		Boolean bResult = CMacCVSProjectViewItem::CanTag();
		
		if ( bResult )
			{
				if ( CUIsAddedOrRemovedRevision(mFileVersionString) )
					bResult = false;			
			}
			
		return bResult;
		
	}


Boolean
CMacCVSProjectViewFile::CanUpdate()
	{
	
		Boolean bResult = CMacCVSProjectViewItem::CanUpdate();
		
		if ( bResult )
			{
				if ( CUIsAddedOrRemovedRevision(mFileVersionString) )
					bResult = false;			
			}
			
		return bResult;
		
	}

	
void
CMacCVSProjectViewFile::GetVersionPStr( Str255 outVersionPStr ) 
	{ 
		
		if ( outVersionPStr != NULL ) 
		{
				
			if ( mFileVersionString[0] > 0 )
			
				BlockMoveData(mFileVersionString, outVersionPStr, mFileVersionString[0] + 1);
			
			else
		
				outVersionPStr[0] = 0;
				
		}
			
	}


void
CMacCVSProjectViewFile::GetVersionCStr( char *outVersionCStr )
	{
		
		Str255		versionPStr	= "\p";
		
		if ( outVersionCStr != NULL ) 
			{
			
				GetVersionPStr( versionPStr );
				
				BlockMoveData( &versionPStr[1], outVersionCStr, versionPStr[0] );
				outVersionCStr[versionPStr[0]] = 0x00;
			
			}
		
	}


ProjectItemStatus
CMacCVSProjectViewFile::GetProjectItemStatus()
	{
		
		return mCachedProjectorStatus;
		
	}


void
CMacCVSProjectViewFile::GetCreatorAndType( OSType &fileCreator, OSType &fileType )
	{
		
		OSErr					err 			= noErr;
		Boolean				result		= false;
		FSSpec				fileSpec 	= { 0, 0, "\p" };
		CInfoPBRec		pb;
		
		fileType = 0;	
		fileCreator = 0;
		
		GetFSSpec( fileSpec );
		
		if ( (fileSpec.vRefNum != 0) && (fileSpec.parID != 0) ) 
			{
			
				memset( &pb, 0, sizeof(CInfoPBRec) );
				
				pb.dirInfo.ioNamePtr = fileSpec.name;
				pb.dirInfo.ioVRefNum = fileSpec.vRefNum;
				pb.dirInfo.ioDrDirID = fileSpec.parID;
				
				pb.dirInfo.ioFDirIndex = 0;

				err = PBGetCatInfoSync(&pb);
				
				if (err != noErr) 
					{

						fileType = pb.hFileInfo.ioFlFndrInfo.fdType;
					
						fileCreator = pb.hFileInfo.ioFlFndrInfo.fdCreator;
							
					}
			
			}

	}


void
CMacCVSProjectViewFile::SetEntry( Str255 pstrInRevision, Str255 pstrInTimeStamp, Str255 pstrInOptions, Str255 pstrInTag )
	{
	
		if ( pstrInRevision != NULL )
			{
				PStrCopy( pstrInRevision, mFileVersionString, sizeof(mFileVersionString) );
			}
		else mFileVersionString[0] = 0;
			
		if ( pstrInTimeStamp != NULL )
			{
				PStrCopy( pstrInTimeStamp, mTimeStamp, sizeof(mTimeStamp) );
			}
		else mTimeStamp[0] = 0;
			
		if ( pstrInOptions != NULL )
			{
				mBinaryData = CUIsCVSBinaryOption( pstrInOptions );
			}
		else mBinaryData = false;
		
		if ( pstrInTag != NULL )
			{
				PStrCopy( pstrInTag, mStickyTagString, sizeof(mStickyTagString) );
			}
		else mStickyTagString[0] = 0;
			
		Invalidate();
		
	}
	
																													
Boolean
CMacCVSProjectViewFile::IsOutOfDate()
	{

		return CMacCVSProjectViewItem::IsOutOfDate();
		
	}


Handle
CMacCVSProjectViewFile::GetCheckoutComment()
	{

		Handle						result		= NULL;
		CkidResourcePB		theCkid;
		
		memset( &theCkid, 0, sizeof(CkidResourcePB) );
		
		if ( FSpReadCkidResource(&mFileSpec, &theCkid, true) == noErr )
				result = theCkid.fCommentHndl;
		
		return result;

	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark CONTENT VALIDATION


void
CMacCVSProjectViewFile::ValidateContents( const FSSpec *inProjectRootFileSpec )
	{

		OSErr							result 				= noErr;
		Boolean						bModified			= false;
		Boolean						bHasCkid			= false;
		CkidResourcePB		*pCkidInfo		= NULL;
		
		try
			{			
				CMacCVSProjectViewItem::ValidateContents( inProjectRootFileSpec );
			
				mCachedProjectorStatus = kMacCVSProjectorStatusUnknown;
				
				if ( ExistsLocal() == false )
					{					
						mCachedProjectorStatus = kMacCVSProjectorStatusDead;
					
						ThrowOSErr( fnfErr );
					}
				else if ( mFileVersionString[0] == 0 )
					{
						mCachedProjectorStatus = kMacCVSProjectorStatusUnowned;
					}
				else if ( mFileVersionString[0] > 0 )
					{				
						Assert_( mFileSpec.name[0] > 0 );
						Assert_( mFileSpec.vRefNum != 0 );
						Assert_( mFileSpec.parID != 0 );

						mCachedProjectorStatus = kMacCVSProjectorStatusCheckedIn;
						
						//  Allocate this big puppy on the heap.					
						pCkidInfo = (CkidResourcePB *)::NewPtr( sizeof(CkidResourcePB) );
						
						if ( pCkidInfo == NULL )
							{
								throw( ::MemError() );
							}
							
						result = FSpReadCkidResource( &mFileSpec, pCkidInfo, false );
							
						if ( result != noErr )
							{
								::DisposePtr( (Ptr)pCkidInfo );
								pCkidInfo = NULL;
								
								result = noErr;
								bHasCkid = false;
							}
							
						result = CUIsFileModified( pCkidInfo, mTimeStamp, mCachedModificationDate );
						
						if ( result == kCVSEngineFileLocallyModifiedErr )
							{
								bModified = true;
								result = noErr;
							}
					
						ThrowOSErr( result );
						
						if ( bModified )
							{
								mCachedProjectorStatus = kMacCVSProjectorStatusMRO;
							}
							
						if ( bHasCkid )
							{
								if ( pCkidInfo->fCheckedOutBySomeone > 0 )
									{
										mCachedProjectorStatus = kMacCVSProjectorStatusCheckedOut;				
									}
							}
						
						if ( pCkidInfo != NULL )
							{
								::DisposePtr( (Ptr)pCkidInfo );
								pCkidInfo = NULL;
							}								
					}				
			}	
		catch ( ... )
			{			
				if ( pCkidInfo != NULL )
					{				
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;					
					}
					
				throw;			
			}
		
	}

	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark COMPARATORS


//  The CUCompareCVSVersion() should be optimized and moved into the CompareVersion()
//  function. A separate, fast version should be used for IsEqualVersion(). This would
//  probably be just a strncmp().


Int32
CMacCVSProjectViewFile::CompareVersion(	CMacCVSProjectViewItem 	*inItemOne, 
										CMacCVSProjectViewItem 	*inItemTwo)
{
	Int32			result 		= 0x00000000;
	
	char			inVersion1Str[256]	= "",
					inVersion2Str[256]	= "";

//  Both items must be project files to compare versions. If one item is a folder, it
//  is considered 'less than' the file.	

	if ((!inItemOne->IsContainer()) && (!inItemTwo->IsContainer())) {

		((CMacCVSProjectViewFile *)inItemOne)->GetVersionCStr(inVersion1Str);
		((CMacCVSProjectViewFile *)inItemTwo)->GetVersionCStr(inVersion2Str);

		result = CUCompareCVSVersion(inVersion1Str, inVersion2Str);
		
		result = result * -1;

	}
	else if ((!inItemOne->IsContainer()) && (inItemTwo->IsContainer()))
		result = -1;
	
	else if ((inItemOne->IsContainer()) && (!inItemTwo->IsContainer()))
		result = 1;

	return (result);
	
}


Boolean
CMacCVSProjectViewFile::IsEqualVersion(	CMacCVSProjectViewItem	*inItemOne,
										CMacCVSProjectViewItem	*inItemTwo)
{
	
	return ( CompareVersion(inItemOne, inItemTwo) == 0 );

}

