//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CMacCVSProjectViewFolder.cp
//
//	  	Authors: 	Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#include <string.h>

#include <Types.h>
#include <TextUtils.h>
#include <Threads.h>

#include <LArray.h>

#include "ThrowOSErr.h"
#include "MoreFilesExtras.h"
#include "StringUtils.h"
#include "CVSEngineUtils.h"
#include "CThreadedTask.h"

#include "CMacCVSProjectViewItem.h"
#include "CMacCVSProjectViewFile.h"

#include "CMacCVSProjectViewFolder.h"


const unsigned char kCVSDirectoryName[] = "\pCVS";
const unsigned char	kCVSEntriesName[] = "\pEntries";


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS/DESTRUCTORS


CMacCVSProjectViewFolder::
CMacCVSProjectViewFolder(	CMacCVSProjectViewItem		*inParentItem, 
													const FSSpec 							*inFileSpec,
													Boolean										inIsAlias ): 
													
	CMacCVSProjectViewItem( (CMacCVSProjectViewItem *)inParentItem, inFileSpec )
	
	{
		
		OSErr				error = noErr;

		mContents 							= new LArray( sizeof(CMacCVSProjectViewItem *) );
		mOpenStatus 						= false;
		mCVSContainer						= false;
		mYieldCount							= 0;
		mCachedEntriesModDate		= 0;
		mEntries								= NULL;
						
	}


CMacCVSProjectViewFolder::
~CMacCVSProjectViewFolder()
	{
		
		CMacCVSProjectViewItem	*theItem			= NULL;
		UInt32									maxItemIndex 	= 0x00000000,
														curItemIndex	= 0x00000000;
														
		if ( mContents != NULL ) 
			{
			
				maxItemIndex = mContents->GetCount();
				
				for ( curItemIndex = 1; curItemIndex <= maxItemIndex; curItemIndex++ )
					{
					
						if ( mContents->FetchItemAt(curItemIndex, &theItem) )
							{
							
								delete theItem;
							
							}
					
					}
				
				delete mContents;
			
			}
		
		if ( mEntries != NULL )
			{
				delete mEntries;
				mEntries = NULL;
			}
			
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


LArray *
CMacCVSProjectViewFolder::GetContainedItemsPtr()
	{
		
		return mContents;

	}


UInt32
CMacCVSProjectViewFolder::GetTotalContainedItemCount()
	{
		
		LArrayIterator		childrenIterator( *mContents );
		
		CMacCVSProjectViewItem	*nextItem	 	= NULL;
		UInt32				totalItems 		= 0x00000000;
		
		while (childrenIterator.Next(&nextItem))

			totalItems += nextItem->GetTotalContainedItemCount();
			
		return totalItems;
		
	}


Boolean
CMacCVSProjectViewFolder::IsCVSContainer() const
	{
	
		return mCVSContainer;
		
	}
	

Boolean
CMacCVSProjectViewFolder::IsContainer() const
	{

		return true;

	}


Boolean
CMacCVSProjectViewFolder::IsOutOfDate()
	{

		Boolean 		bResult 				= false;
		OSErr				error 					= noErr;
		Str255			pstrPartialPath = "\p";
		CInfoPBRec	pb;

		bResult = CMacCVSProjectViewItem::IsOutOfDate();
		
		if ( !bResult && IsCVSContainer() )
			{
				pstrPartialPath[0] = 1;
				pstrPartialPath[1] = ':';
				PStrConcat( pstrPartialPath, mFileSpec.name, sizeof(pstrPartialPath) );
				PStrConcat( pstrPartialPath, "\p:", sizeof(pstrPartialPath) );
				PStrConcat( pstrPartialPath, kCVSDirectoryName, sizeof(pstrPartialPath) );
				PStrConcat( pstrPartialPath, "\p:", sizeof(pstrPartialPath) );
				PStrConcat( pstrPartialPath, kCVSEntriesName, sizeof(pstrPartialPath) );
				
				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= 0;
				pb.hFileInfo.ioNamePtr 			= pstrPartialPath;
				pb.hFileInfo.ioVRefNum 			= mFileSpec.vRefNum;
				pb.hFileInfo.ioDirID 				= mFileSpec.parID;
				
				error = ::PBGetCatInfoSync( &pb );
			
				if ( error == noErr )
					{
						if ( pb.hFileInfo.ioFlMdDat != mCachedEntriesModDate )
							{
								bResult = true;
								Invalidate();
							}
					}			
			}
		
		return bResult;
		
	}
	
	
void
CMacCVSProjectViewFolder::SetOpenStatus(Boolean inNewOpenStatus)
	{

		mOpenStatus = inNewOpenStatus;
		
	}


Boolean
CMacCVSProjectViewFolder::GetOpenStatus(void)
	{

		return mOpenStatus;
		
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark SORTING


void
CMacCVSProjectViewFolder::SetComparator( LComparator *inComparator )
	{

		if ( (mContents != NULL) && (inComparator != NULL) ) 
			{
			
				mContents->SetComparator( inComparator, true );
			
				mContents->SetKeepSorted( true );
					
			}
		
	}



//################################################################################
//################################################################################
#pragma mark -
#pragma mark CONTENT VALIDATION


void
CMacCVSProjectViewFolder::ValidateContents( const FSSpec *inProjectRootFileSpec )
	{

		LArray 									*deletedItems 	= NULL;
		CMacCVSProjectViewItem	*theItem				= NULL;
		
		deletedItems = ValidateContentsChanged( inProjectRootFileSpec );
		
		if ( deletedItems != NULL ) 
			{
			
				LArrayIterator	deletedItemsIterator( *deletedItems );
				
				while ( deletedItemsIterator.Next( &theItem ) ) 
					{
					
						delete theItem;
												
					}
				
				delete deletedItems;
			
			}
		
	}


LArray *
CMacCVSProjectViewFolder::ValidateContentsChanged( const FSSpec *inProjectRootFileSpec )
	{

		LArrayIterator						contentsIterator( *mContents );

		OSErr											error									= noErr;

		UInt32										intNewItemCount 			= 0;
		UInt32										intMaxItemCount				= 0;
		UInt32										intItemIndex					= 0;
		UInt32										intCurrentIndex				= 0;
		SInt32										intDirID							= 0;
		SInt32										intNewEntriesModDate	= 0;
		
		Str255										pstrRevision					= "\p";
		Str255										pstrTimeStamp					= "\p";
		Str255										pstrOptions						= "\p";
		Str255										pstrTag								= "\p";
		Str255										pstrEntriesPath				= "\p";
		
		LArray										*mChangedContents 		= NULL;
		LArray										*deletedItems					= NULL;

		FSSpec										fsChild 							= { 0, 0, "\p" };
		CMacCVSProjectViewItem		*theItem							= NULL;
						
		CInfoPBRec								pb;
		CInfoPBRec								pbChild;
		CInfoPBRec								pbEntries;
			
		try 
			{													
				pb.dirInfo.ioCompletion = NULL;
				pb.dirInfo.ioNamePtr 		= mFileSpec.name;
				pb.dirInfo.ioVRefNum 		= mFileSpec.vRefNum;
				pb.dirInfo.ioDrDirID 		= mFileSpec.parID;
				pb.dirInfo.ioFDirIndex 	= 0;

				error = ::PBGetCatInfoSync(&pb);
				ThrowOSErr( error );

				if ( (pb.dirInfo.ioFlAttrib & ioDirMask) == 0 )
					ThrowOSErr( dirNFErr );

				intDirID = pb.dirInfo.ioDrDirID;		
				intNewItemCount = pb.dirInfo.ioDrNmFls;

//  Get the modification date of the entries file.
//  It may or may not exist.
					
				if ( mCVSContainer )
					{
						pstrEntriesPath[0] = 1;
						pstrEntriesPath[1] = ':';
						PStrConcat( pstrEntriesPath, mFileSpec.name, sizeof(pstrEntriesPath) );
						PStrConcat( pstrEntriesPath, "\p:", sizeof(pstrEntriesPath) );
						PStrConcat( pstrEntriesPath, kCVSDirectoryName, sizeof(pstrEntriesPath) );
						PStrConcat( pstrEntriesPath, "\p:", sizeof(pstrEntriesPath) );
						PStrConcat( pstrEntriesPath, kCVSEntriesName, sizeof(pstrEntriesPath) );
						
						pbEntries.hFileInfo.ioCompletion 		= NULL;
						pbEntries.hFileInfo.ioFDirIndex 		= 0;
						pbEntries.hFileInfo.ioNamePtr 			= pstrEntriesPath;
						pbEntries.hFileInfo.ioVRefNum 			= mFileSpec.vRefNum;
						pbEntries.hFileInfo.ioDirID 				= mFileSpec.parID;
				
						error = ::PBGetCatInfoSync( &pbEntries );
						
						if ( error == noErr )
							{
								intNewEntriesModDate = pbEntries.hFileInfo.ioFlMdDat;
							}
							
						error = noErr;
					}
				else
					{
						mCachedEntriesModDate = 0;
					}
								
//  Don't validate if we don't need to...
				
				if ( 	(mCachedModificationDate != pb.hFileInfo.ioFlMdDat) || 
							mIsInvalid || 
							((intNewEntriesModDate != mCachedEntriesModDate) && mCVSContainer) ) 
					{
						mIsInvalid = false;
					
						ValidatePartially();
						
						if ( mCVSContainer )
							{
								mCachedEntriesModDate = intNewEntriesModDate;
							
								if ( mCachedEntriesModDate == 0 )
									{
										//  Uh oh, we didn't get the mod. date in the code
										//  above because this is the first time through and
										//  up there we didn't know we were a CVS folder. So
										//  do it again here.
										
										pstrEntriesPath[0] = 1;
										pstrEntriesPath[1] = ':';
										PStrConcat( pstrEntriesPath, mFileSpec.name, sizeof(pstrEntriesPath) );
										PStrConcat( pstrEntriesPath, "\p:", sizeof(pstrEntriesPath) );
										PStrConcat( pstrEntriesPath, kCVSDirectoryName, sizeof(pstrEntriesPath) );
										PStrConcat( pstrEntriesPath, "\p:", sizeof(pstrEntriesPath) );
										PStrConcat( pstrEntriesPath, kCVSEntriesName, sizeof(pstrEntriesPath) );
										
										pbEntries.hFileInfo.ioCompletion 		= NULL;
										pbEntries.hFileInfo.ioFDirIndex 		= 0;
										pbEntries.hFileInfo.ioNamePtr 			= pstrEntriesPath;
										pbEntries.hFileInfo.ioVRefNum 			= mFileSpec.vRefNum;
										pbEntries.hFileInfo.ioDirID 				= mFileSpec.parID;
								
										error = ::PBGetCatInfoSync( &pbEntries );
										
										if ( error == noErr )
											{
												mCachedEntriesModDate = pbEntries.hFileInfo.ioFlMdDat;
											}
											
										error = noErr;									
									}
							}
						else
							{
								mCachedEntriesModDate = 0;
							}
							
						mCachedModificationDate = pb.hFileInfo.ioFlMdDat;
	
						Assert_( mEntries == NULL );
	
						if ( mEntries != NULL )
							{
								delete mEntries;
								mEntries = NULL;
							}
						
						if ( mCVSContainer )
							{
								mEntries = new CCVSEntries;
								ThrowIfNil_( mEntries );
								
								error = mEntries->LoadWorkingDirectory( &mFileSpec, fsRdPerm );
								if ( error != noErr )
									{
										delete mEntries;
										mEntries = NULL;
									}
							}
														
						fsChild.vRefNum = mFileSpec.vRefNum;
						fsChild.parID = intDirID;
						
						intMaxItemCount = mContents->GetCount();
						intCurrentIndex = 1;
						
//  First build a list of items that no longer exist
//  in the directory. We do not delete them here because
//  there may be other items dependent on their existance.
//  (ie, the entries in the LHierarchyTable() use pointers
//  to the item objects to draw the name text, etc.) So,
//  the caller will dispose of the objects.
								
						while ( intCurrentIndex <= intMaxItemCount ) 
							{							
								if ( (mYieldCount % kDefaultYieldCount) == 0 )
									CThreadedTask::Yield();
								
								mYieldCount++;
								
								theItem = NULL;
								
								mContents->FetchItemAt( intCurrentIndex, &theItem );								
								Assert_( theItem != NULL );
									
								if ( theItem != NULL ) 
									{								
										theItem->GetFSSpec( fsChild );

										pbChild.dirInfo.ioCompletion 	= NULL;
										pbChild.dirInfo.ioNamePtr 		= fsChild.name;
										pbChild.dirInfo.ioVRefNum 		= fsChild.vRefNum;
										pbChild.dirInfo.ioDrDirID 		= fsChild.parID;
										pbChild.dirInfo.ioFDirIndex 	= 0;
										
										error = ::PBGetCatInfoSync( &pbChild );
										
										if ( error != noErr ) 
											{											
												if ( deletedItems == NULL )
													deletedItems = new LArray( sizeof(CMacCVSProjectViewItem *) );
													
												deletedItems->InsertItemsAt( 1, LArray::index_Last, &theItem );
												
												mContents->RemoveItemsAt( 1, intCurrentIndex );
											
												intMaxItemCount--;											
											} 
										else intCurrentIndex++;
										
									} 
								else mContents->RemoveItemsAt( 1, intCurrentIndex );													
							}

//  Now add new stuff if needed...
						
						for ( intCurrentIndex = 1; intCurrentIndex <= intNewItemCount; intCurrentIndex++ ) 
							{
								if ( (mYieldCount % kDefaultYieldCount) == 0 )
									CThreadedTask::Yield();
									
								mYieldCount++;
											
								pb.dirInfo.ioNamePtr		= fsChild.name;
								pb.dirInfo.ioVRefNum 		= fsChild.vRefNum;
								pb.dirInfo.ioDrDirID 		= fsChild.parID;
								pb.dirInfo.ioFDirIndex 	= intCurrentIndex;
								pb.dirInfo.ioCompletion = NULL;

								error = ::PBGetCatInfoSync(&pb);
								
								if ( error == noErr)
									intItemIndex = FindProjectItem( fsChild.name );

//  If intItemIndex == 0, then the file/directory is not in the mContents array.
//  Add it and tell the owner tree...					

								if ( (error == noErr) && (intItemIndex == 0) ) 
									{										
										if ( (pb.hFileInfo.ioFlFndrInfo.fdFlags & kIsInvisible) == 0 )
											{											
												Boolean		isAlias = (pb.hFileInfo.ioFlFndrInfo.fdFlags & kIsAlias) != 0;
												
												theItem = NULL;
												
												if ( (pb.dirInfo.ioFlAttrib & ioDirMask) != 0 )		// if it is a folder...
													{
														CMacCVSProjectViewFolder *theItemCasted = NULL;
											
														theItemCasted = new CMacCVSProjectViewFolder( this, &fsChild, isAlias );
														ThrowIfNil_( theItemCasted );
														
														//  Calling ValidateContents() deep would be a bad idea. 
														//  So we call ValidatePartially() to load the sticky
														//  tag and set the modification date.

														theItemCasted->Invalidate();																						
														theItemCasted->ValidatePartially();
														theItemCasted->mCachedModificationDate = pb.hFileInfo.ioFlMdDat;
														
														theItem = theItemCasted;														
													}
												else 
													{												
														CMacCVSProjectViewFile *theItemCasted = new CMacCVSProjectViewFile( this, &fsChild, isAlias );														
														ThrowIfNil_( theItemCasted );
														
														theItem = theItemCasted;
												
														if ( mEntries != NULL )	
															{
																error = mEntries->GetEntry( fsChild.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
																if ( error == noErr )
																	{
																		theItemCasted->SetEntry( pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
																	}
																else
																	{
																		theItemCasted->SetEntry( NULL, NULL, NULL, NULL );
																	}
															}
														else 
															{
																theItemCasted->SetEntry( NULL, NULL, NULL, NULL );
															}
															
														if ( theItem->IsOutOfDate() )
															theItem->ValidateContents( inProjectRootFileSpec );															
													}

//  Insert in mContents array...

												Assert_( theItem != NULL );												
												mContents->InsertItemsAt( 1, LArray::index_Last, &theItem );									
											}													
									} 
									
								else if ( error == noErr ) 
									{								
										theItem = NULL;
										
										mContents->FetchItemAt( intItemIndex, &theItem );
										
										Assert_( theItem );
										
										if ( theItem->IsContainer() )
											{
												CMacCVSProjectViewFolder *pItemCasted = dynamic_cast<CMacCVSProjectViewFolder *>(theItem);
												Assert_( pItemCasted != NULL );
												ThrowIfNil_( pItemCasted );
												
												pItemCasted->ValidatePartially();
											}
										else
											{												
												FSSpec fsItemSpec = { 0, 0, "\p" };
												CMacCVSProjectViewFile *theItemCasted = dynamic_cast<CMacCVSProjectViewFile *>(theItem);
												
												Assert_( theItemCasted != NULL );
												ThrowIfNil_( theItemCasted );
												
												theItem->GetFSSpec( fsItemSpec );
												
												if ( mEntries != NULL )	
													{
														error = mEntries->GetEntry( fsItemSpec.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
														if ( error == noErr )
															{
																theItemCasted->SetEntry( pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
															}
														else
															{
																theItemCasted->SetEntry( NULL, NULL, NULL, NULL );
															}
													}
												else 
													{
														theItemCasted->SetEntry( NULL, NULL, NULL, NULL );
													}
													
												if ( theItem->IsOutOfDate() )
													theItem->ValidateContents( inProjectRootFileSpec );											
											}											
									}									
							}										
					} //  if modified

//  Dispose of the Entries structure so we
//  don't waste too much memory.
					
				if ( mEntries != NULL )
					{
						delete mEntries;
						mEntries = NULL;						
					}	
			}
							
		catch ( ... ) 
			{
				if ( mEntries != NULL )
					{
						try
							{
								delete mEntries;
							}
						catch ( ... )
							{
							}
							
						mEntries = NULL;
					}	
								
				throw;			
			}
				
		return deletedItems;
				
	}
	

void
CMacCVSProjectViewFolder::ValidatePartially()
	{
			
		mCVSContainer = FSpIsRepositoryDir( &mFileSpec );
		
		if ( mCVSContainer )
			{
				FSpGetFolderTag( &mFileSpec, mStickyTagString );
			}
			
	}
	
	
		
UInt32
CMacCVSProjectViewFolder::FindProjectItem( StringPtr	inFileNamePStr )
	{
		
		LArrayIterator					contentIterator( *mContents );

		CMacCVSProjectViewItem	*nextItem 	= NULL;
		FSSpec									fileSpec		= { 0, 0, "\p" };
		
		UInt32									index				= 0x00000000;
		
		while ( contentIterator.Next(&nextItem) ) 
			{
				
				index++;
				
				nextItem->GetFSSpec( fileSpec );
				
				if ( ::EqualString(fileSpec.name, inFileNamePStr, false, true) )

					return index;

			}
			
		return 0;
		
	}

