//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSProjectViewItem.cp
//
//	  Authors: 	Dan Clifford and Ram Rajadhyaksha
//
//################################################################################


#include <string.h>
#include <stdio.h>

#include "StringUtils.h"

#include "CVSEngineUtils.h"
#include "CMacCVSProjectViewItem.h"


//################################################################################
//################################################################################


CMacCVSProjectViewItem::
CMacCVSProjectViewItem(	CMacCVSProjectViewItem		*inParentItem, 
												const FSSpec 							*inFileSpec,
												Boolean										inIsAlias)
					
	{

		mParentItem = inParentItem;
					
		mCachedModificationDate 	= 0;
		mInRepository							= true;
		mIsInvalid								= true;
		mVisible									= true;
		mLocked										= false;
		mIsAlias									= inIsAlias;

#ifdef FASTER_FIND_ICON
		mFileInfoRec							= NULL;
#endif

		mStickyTagString[0]				= 0;
				
		if (inFileSpec != NULL)
		
			mFileSpec = *inFileSpec;
		
		else memset(&mFileSpec, 0, sizeof(mFileSpec));
			
	}


CMacCVSProjectViewItem::
~CMacCVSProjectViewItem()
	{
#ifdef FASTER_FIND_ICON
		delete mFileInfoRec;
#endif
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark ACCESSORS


Boolean
CMacCVSProjectViewItem::CanBeAdded()
	{
	
		Boolean	result = false;
		
		//	You can only add a file if it's status is unowned.
		ProjectItemStatus theStatus = GetProjectItemStatus();
		
		if ( !IsCVSContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							{
							
								if ( IsContainer() )
									result = true;
									
								else if ( theStatus == kMacCVSProjectorStatusUnowned )
									result = true;
									
							}
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							{
							
								if ( IsContainer() )
									result = true;
									
								else if ( theStatus == kMacCVSProjectorStatusUnowned )
									result = true;
									
							}
					
					}
					
			}
			
		return result;
		
	}


Boolean
CMacCVSProjectViewItem::CanCheckin()
	{
		
//	You can checkin any folder, or any item that is either 
//	MRO'ed or checked-out exclusive.

		Boolean						result		= false;
		ProjectItemStatus theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( (theStatus == kMacCVSProjectorStatusMRO) || (theStatus == kMacCVSProjectorStatusCheckedOut) )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( (theStatus == kMacCVSProjectorStatusMRO) || (theStatus == kMacCVSProjectorStatusCheckedOut) ) 
								result = true;
					
					}
					
			}
			
		return result;

	}


Boolean
CMacCVSProjectViewItem::CanCheckout()
	{
		
//	Checkout is the same as update, except that you cannot
//	checkout individual files.
		
		return ( IsCVSContainer() );

	}


Boolean
CMacCVSProjectViewItem::CanCompareWithOriginal()
	{

		Boolean							result		= false;
		ProjectItemStatus		theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( theStatus == kMacCVSProjectorStatusMRO )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( theStatus == kMacCVSProjectorStatusMRO )
								result = true;
					
					}
					
			}
		
		return result;

	}


Boolean
CMacCVSProjectViewItem::CanClearDateStickyTag()
	{

		Boolean							result		= false;
		ProjectItemStatus		theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( theStatus == kMacCVSProjectorStatusMRO )
								if ( mStickyTagString[0] > 1 )
									if ( mStickyTagString[1] == 'D' )
										result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( theStatus == kMacCVSProjectorStatusMRO )
								if ( mStickyTagString[0] > 1 )
									if ( mStickyTagString[1] == 'D' )
										result = true;
					
					}
					
			}
		
		return result;

	}


Boolean
CMacCVSProjectViewItem::CanDiff()
	{

		Boolean							result		= false;
		ProjectItemStatus		theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( theStatus == kMacCVSProjectorStatusMRO )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( theStatus == kMacCVSProjectorStatusMRO )
								result = true;
					
					}
					
			}
		
		return result;

	}


Boolean
CMacCVSProjectViewItem::CanImport()
	{
	
		return ( IsContainer() && !IsCVSContainer() );
	
	}
	

Boolean
CMacCVSProjectViewItem::CanTag()
	{
	
		Boolean						result		= false;
		ProjectItemStatus theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( (theStatus > kMacCVSProjectorStatusUnowned) && (theStatus < kMacCVSProjectorStatusBroken) )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( (theStatus > kMacCVSProjectorStatusUnowned) && (theStatus < kMacCVSProjectorStatusBroken) ) 
								result = true;
					
					}
					
			}
		else if ( IsCVSContainer() )
			{
			
				result = true;
				
			}
	
		return result;
		
	}
	
	
Boolean
CMacCVSProjectViewItem::CanMRO()
	{
		
		ProjectItemStatus theStatus = GetProjectItemStatus();
		
		// You can only MRO files that have been checked in
		
		return 	( theStatus == kMacCVSProjectorStatusCheckedIn );
		
	}


Boolean
CMacCVSProjectViewItem::CanOrphanFiles()
	{
	
		Boolean						result		= false;
		ProjectItemStatus theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				result = (theStatus != kMacCVSProjectorStatusUnowned );

			}
		else if ( IsCVSContainer() )
			{
			
				result = true;
				
			}
	
		return result;
		
	}

Boolean
CMacCVSProjectViewItem::CanRemove()
	{
	
		Boolean						result		= false;
		ProjectItemStatus theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( (theStatus > kMacCVSProjectorStatusUnowned) && (theStatus < kMacCVSProjectorStatusBroken) )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( (theStatus > kMacCVSProjectorStatusUnowned) && (theStatus < kMacCVSProjectorStatusBroken) ) 
								result = true;
					
					}
					
			}
	
		return result;
		
	}


Boolean
CMacCVSProjectViewItem::CanEditComment()
	{

		ProjectItemStatus theStatus = GetProjectItemStatus();
		
		return ( (theStatus == kMacCVSProjectorStatusCheckedOut) || (theStatus == kMacCVSProjectorStatusMRO) );
		
	}


Boolean
CMacCVSProjectViewItem::CanEdit()
	{
	
		Boolean							result		= noErr;
		ProjectItemStatus		theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{			
				if ( mParentItem != NULL )
					{					
						if ( mParentItem->IsCVSContainer() )
							{
								if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) )
									{
										result = true;
									}
							}
					}
				else
					{
						//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							{
								if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) ) 
									{
										result = true;		
									}
							}			
					}
			}
			
		return result;

	}
	

Boolean
CMacCVSProjectViewItem::CanWatch()
	{
	
		Boolean							result		= noErr;
		ProjectItemStatus		theStatus = GetProjectItemStatus();

		if ( !IsContainer() )
			{			
				if ( mParentItem != NULL )
					{					
						if ( mParentItem->IsCVSContainer() )
							{
								if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) )
									{
										result = true;
									}
							}
					}
				else
					{
						//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							{
								if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) ) 
									{
										result = true;		
									}
							}			
					}
			}
		else
			{
				result = IsCVSContainer();
			}
			
		return result;

	}
	
	
Boolean
CMacCVSProjectViewItem::CanGetLog()
	{

		Boolean							result		= noErr;
		ProjectItemStatus		theStatus = GetProjectItemStatus();


		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) ) 
								result = true;
					
					}
					
			}
		
		return result;

	}


Boolean
CMacCVSProjectViewItem::CanGetStatus()
	{

		Boolean							result			= false;
		ProjectItemStatus		theStatus 	= GetProjectItemStatus();

		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( (theStatus > kMacCVSProjectorStatusCheckedOut) && (theStatus < kMacCVSProjectorStatusBroken) ) 
								result = true;
					
					}
					
			}

		return result;

	}


Boolean
CMacCVSProjectViewItem::CanRevert()
	{
		// You can revert a file that has been MRO'd or checked out exclusively.
		ProjectItemStatus 	theStatus = GetProjectItemStatus();
		
		return ( !IsContainer() && ((theStatus == kMacCVSProjectorStatusMRO) || (theStatus == kMacCVSProjectorStatusCheckedOut)) );
			 
	}

	
Boolean
CMacCVSProjectViewItem::CanUpdate()
	{

		Boolean		result	= false;
		
		ProjectItemStatus theStatus = GetProjectItemStatus();
		
		if ( !IsContainer() )
			{
			
				if ( mParentItem != NULL )
					{
					
						if ( mParentItem->IsCVSContainer() )
							if ( (theStatus > kMacCVSProjectorStatusUnowned) && (theStatus < kMacCVSProjectorStatusBroken) )
								result = true;
						
					}
				else
					{

//  Else if there is no tree hierarchy, check ourselves.
					
						if ( CUIsRepositoryDir(mFileSpec.vRefNum, mFileSpec.parID) )
							if ( (theStatus > kMacCVSProjectorStatusUnowned) && (theStatus < kMacCVSProjectorStatusBroken) ) 
								result = true;
					
					}
			
			
			}
			
		else if ( IsCVSContainer() )
			{
			
				result = true;
				
			}

		return result;
					
	}


Boolean
CMacCVSProjectViewItem::CanUpdateTo()
	{

		return ( CanUpdate() );

	}



CMacCVSProjectViewItem *
CMacCVSProjectViewItem::GetParent() const
	{
		
		return mParentItem;
		
	}


void
CMacCVSProjectViewItem::GetNamePStr( Str255 outNamePStr ) const
	{

		if ( outNamePStr != NULL ) 
			{
			
				outNamePStr[0] = mFileSpec.name[0];
				
				if ( outNamePStr[0] > 0 )
					BlockMoveData( &mFileSpec.name[1], &outNamePStr[1], outNamePStr[0] );
					
			}

	}


void
CMacCVSProjectViewItem::GetNameCStr( char *outNameCStr ) const
{
	
		Str255		namePStr = "\p";
		
		if ( outNameCStr != NULL ) 
		{
		
			GetNamePStr(namePStr);
			BlockMoveData(namePStr + 1, outNameCStr, namePStr[0]);
			
			outNameCStr[namePStr[0]] = 0x00;
		
		}

}


void
CMacCVSProjectViewItem::GetFSSpec( FSSpec &outFileSpec ) const
	{

		outFileSpec = mFileSpec;

	}


void
CMacCVSProjectViewItem::GetCachedLocalPathPStr(Str255 outLocalPathPStr) const
	{

		if ( outLocalPathPStr != NULL ) 
			{
			
				if ( mParentItem != NULL )	
					{
					
						mParentItem->GetCachedLocalPathPStr( outLocalPathPStr );
						
						if ( outLocalPathPStr[0] > 0 )
							PStrConcat( outLocalPathPStr, "\p/", sizeof(Str255) );
							
						PStrConcat( outLocalPathPStr, mFileSpec.name, sizeof(Str255) );
						
					}
				else
					{
					
					 	outLocalPathPStr[0] = 0x00;
						
					}
			
			}
		
	}


void
CMacCVSProjectViewItem::GetCachedLocalPathCStr(char *outLocalPathCStr) const
	{
		
		Str255	localPathPStr = "\p";
		
		if ( outLocalPathCStr != NULL ) 
			{
			
				GetCachedLocalPathPStr( localPathPStr );
				
				::BlockMoveData( &localPathPStr[1], outLocalPathCStr, localPathPStr[0] );
				outLocalPathCStr[localPathPStr[0]] = 0x00;
			
			}
		
	}


void
CMacCVSProjectViewItem::GetCachedLocalPathStr(Handle inOutLocalPathStrHndl) const
	{
		
		OSErr			error		= noErr;
		UInt32		pathLen = 0x00000000,
							newSize = 0x00000000;
		
		if ( inOutLocalPathStrHndl != NULL ) 
			{
			
				if ( mParentItem != NULL )	
					{
					
						mParentItem->GetCachedLocalPathStr( inOutLocalPathStrHndl );
						
						pathLen = ::GetHandleSize( inOutLocalPathStrHndl );
					
						newSize = pathLen + mFileSpec.name[0];
						
						if ( pathLen > 0 )
							newSize++;
							
						::SetHandleSize( inOutLocalPathStrHndl, newSize );
						
						error = ::MemError();
						
						if ( error != noErr )
							throw( error );
							
						if ( pathLen > 0 )
							{
							
								(*inOutLocalPathStrHndl)[pathLen] = '/';
								pathLen++;
						
							}
						
						::BlockMoveData( &mFileSpec.name[1], (*((UInt8 **)inOutLocalPathStrHndl)) + pathLen, mFileSpec.name[0] );
							
					}
				else
					{
					
					 	::SetHandleSize( inOutLocalPathStrHndl, 0 );

						error = ::MemError();
						
						if ( error != noErr )
							throw( error );
						
					}
			
			}
		
	}


UInt32
CMacCVSProjectViewItem::GetModificationDate()
	{

		OSErr				err 			= noErr;
		UInt32			result		= 0x00000000;
#ifndef FASTER_FIND_ICON
		CInfoPBRec	pb;
		
		CInfoPBRec	*pbPtr = &pb;
#else
		if ( mFileInfoRec == NULL )
			{
				mFileInfoRec = new CInfoPBRec;
				ThrowIfNil_( mFileInfoRec );
			}
			
			CInfoPBRec	*pbPtr = mFileInfoRec;
#endif

		if ( (mFileSpec.vRefNum != 0) && (mFileSpec.parID != 0) ) 
			{
			
				pbPtr->dirInfo.ioCompletion 	= NULL;				
				pbPtr->dirInfo.ioNamePtr 			= mFileSpec.name;
				pbPtr->dirInfo.ioVRefNum 			= mFileSpec.vRefNum;
				pbPtr->dirInfo.ioDrDirID 			= mFileSpec.parID;
				pbPtr->dirInfo.ioFDirIndex 		= 0;

				err = ::PBGetCatInfoSync(pbPtr);
				
				if (err != noErr)
					
					result = 0;
				
				else 
					{
						result = pbPtr->hFileInfo.ioFlMdDat;			
						mLocked = (pbPtr->hFileInfo.ioFlAttrib & 1);
					}
					
				if ( result != mCachedModificationDate )
					mIsInvalid = true;
					
				mCachedModificationDate = result;
			
			}
		
		return result;	
		
	}



void
CMacCVSProjectViewItem::GetStickyTagPStr( Str255 outTagPStr )
	{

		if ( outTagPStr != NULL ) 
			{
			
				if ( mStickyTagString[0] > 0 )
				
					BlockMoveData( mStickyTagString, outTagPStr, mStickyTagString[0] + 1 );
				
				else
					
					outTagPStr[0] = 0;
				
			}
		
	}


void
CMacCVSProjectViewItem::GetStickyTagCStr( char *outTagCStr )
	{
		
		Str255		tagPStr = "\p";
		
		if ( outTagCStr != NULL ) 
			{
			
				GetStickyTagPStr( tagPStr );
				
				BlockMoveData( &tagPStr[1], outTagCStr, tagPStr[0] );
				outTagCStr[tagPStr[0]] = 0x00;
			
			}
		
	}


ProjectItemStatus
CMacCVSProjectViewItem::GetProjectItemStatus()
	{

		return kMacCVSProjectorStatusNone;
		
	}


Boolean
CMacCVSProjectViewItem::GetVisibility()
	{

		OSErr				err 			= noErr;
		Boolean			result		= false;
		CInfoPBRec	pb;
			
		if ( (mFileSpec.vRefNum != 0) && (mFileSpec.parID != 0) ) 
			{
			
				pb.dirInfo.ioCompletion 	= NULL;
				pb.dirInfo.ioNamePtr 			= mFileSpec.name;
				pb.dirInfo.ioVRefNum 			= mFileSpec.vRefNum;
				pb.dirInfo.ioDrDirID 			= mFileSpec.parID;
				pb.dirInfo.ioFDirIndex = 0;

				err = PBGetCatInfoSync(&pb);
				
				if (err != noErr)
					result = (pb.hFileInfo.ioFlFndrInfo.fdFlags & kIsInvisible);
			
			}
		
		return result;	
		
	}


LArray *
CMacCVSProjectViewItem::GetContainedItemsPtr()
	{
		
		return NULL;

	}


UInt32
CMacCVSProjectViewItem::GetTotalContainedItemCount()
	{

		return 1;

	}


void
CMacCVSProjectViewItem::TouchFinderItem()
	{
		UInt32		now;
		
		::GetDateTime( &now );
		
		OSErr	err = FSpSetModDate( &mFileSpec, now );
	
		Assert_ ( err == noErr );
	}
	
	
Boolean
CMacCVSProjectViewItem::IsAlias()
	{
		return mIsAlias;
	}
	

	
Boolean
CMacCVSProjectViewItem::IsLocked()
	{
		return mLocked;
	}
	
	
Boolean
CMacCVSProjectViewItem::IsContainer() const
	{

		return false;

	}


Boolean
CMacCVSProjectViewItem::IsCVSContainer() const
	{
	
	
		return false;
		
	}
	
	
Boolean
CMacCVSProjectViewItem::IsOutOfDate()
	{

		Boolean		result 					= false;
		Boolean		bPrevLockState 	= mLocked;
		
		if ( mIsInvalid == false ) 
			{		
				GetModificationDate();
				
				if ( bPrevLockState != mLocked )
					{
						//  GetModificationDate() updates the
						//  mLocked member.
						
						mIsInvalid = true;
					}
					
				result = mIsInvalid;			
			} 
			
		else result = true;
			
		return result;

	}


Boolean
CMacCVSProjectViewItem::IsChildOf( const CMacCVSProjectViewItem* inItem ) const
	{
		
		return ( (CMacCVSProjectViewItem *)GetParent() == inItem );

	}


Boolean
CMacCVSProjectViewItem::IsDescendentOf( const CMacCVSProjectViewItem *inItem ) const
	{

		Boolean 								bIsDescendent = false;
		CMacCVSProjectViewItem	*theParent 		= GetParent();
		
		if ( (theParent != NULL) && (inItem != NULL) )
			{
			
				if ( theParent == inItem )
				
					bIsDescendent = true;
				
				else bIsDescendent = theParent->IsDescendentOf( inItem );
			
			}
			
		return bIsDescendent;

	}


Boolean
CMacCVSProjectViewItem::IsSiblingOf( const CMacCVSProjectViewItem* inItem ) const
	{

		return ( mParentItem == inItem->GetParent() );

	}


Boolean
CMacCVSProjectViewItem::ExistsLocal()
	{
		
		CInfoPBRec	pb;
		Boolean		result = false;
		
		if ( (mFileSpec.vRefNum != 0) && (mFileSpec.parID != 0) ) 
			{
			
				pb.dirInfo.ioNamePtr 	= mFileSpec.name;
				pb.dirInfo.ioVRefNum 	= mFileSpec.vRefNum;
				pb.dirInfo.ioDrDirID 	= mFileSpec.parID;
				pb.dirInfo.ioFDirIndex 	= 0;
				pb.dirInfo.ioCompletion = NULL;

				if ( ::PBGetCatInfoSync(&pb) == noErr )
					if ( ((pb.dirInfo.ioFlAttrib & ioDirMask) && (IsContainer())) || (((pb.dirInfo.ioFlAttrib & ioDirMask) == 0) && (!IsContainer())) )
						result = true;
					
			}
		
		return result;
		
	}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark CONTENT VALIDATION


void
CMacCVSProjectViewItem::ValidateContents( const FSSpec *inProjectRootFileSpec )
	{

		UInt32	currentFileDate = 0x00000000;
		
		currentFileDate = GetModificationDate();
		
		if ( currentFileDate != mCachedModificationDate )
			mCachedModificationDate = currentFileDate;
			
		if ( currentFileDate != 0 )
			mIsInvalid = false;

	}


void
CMacCVSProjectViewItem::Invalidate()
	{
	
		mIsInvalid = true;
		
	}
	
	
//################################################################################
//################################################################################
#pragma mark -
#pragma mark MISCELLANEOUS


static OSErr 
GetFinderProcessSerialNumber( ProcessSerialNumber &psn )
	{
		
		ProcessInfoRec 		info;
		OSErr							err;
		
		psn.highLongOfPSN = 0;
		psn.lowLongOfPSN  = kNoProcess;
		
		while (1)
			{
				err = ::GetNextProcess(&psn);
				if( err == noErr )
					{
						info.processInfoLength = sizeof(ProcessInfoRec);
						info.processName = NULL;
						info.processAppSpec = NULL;
						err = GetProcessInformation(&psn, &info);
					}
				
				if ((err != noErr) ||
					(info.processSignature == 'MACS') ||
					(info.processType == 'FNDR'))
					break;
				
			}
		
		return err;
		
	}


void
CMacCVSProjectViewItem::OpenWithFinder()
{
	
	AliasHandle		alias 		= NULL;
	AppleEvent		event 		= { typeNull, NULL };
	FSSpec			fileSpec 	= { 0, 0, "\p" };

	GetFSSpec(fileSpec);

	if ( (fileSpec.vRefNum != 0) && (fileSpec.parID != 0) ) {
	
		try 
		{
			
			ProcessSerialNumber		finderPSN;
			OSErr					error;
			
			//	Get the serial number of the Finder... we will
			//	use this as the destination for our AppleEvent.		
			error = GetFinderProcessSerialNumber(finderPSN);
			
			if ( error != noErr )
				throw( error );

			StAEDescriptor	editorAddress(	typeProcessSerialNumber, 
											&finderPSN, 
											sizeof(finderPSN));

			//	Create the AppleEvent.
			error = ::AECreateAppleEvent(	kCoreEventClass, 
											kAEOpen, 
											(const AEAddressDesc *) &editorAddress, 
											0, 
											0, 
											&event);
			
			if ( error != noErr )
				throw( error );
						
			//	Create an alias for this item								
			error = ::NewAlias(NULL, &fileSpec, &alias);

			if ( error != noErr )
				throw( error );

			//	Stuff the alias in the AppleEvent as the 
			//	direct parameter.
			::HLock((Handle)alias);
			StAEDescriptor	aliasObject(typeAlias, *alias, ::GetHandleSize((Handle)alias));
			::AEPutParamDesc(&event, keyDirectObject, aliasObject);
			::HUnlock((Handle)alias);

			//	Send the open AppleEvent
			UAppleEventsMgr::SendAppleEvent(event, true);
			
			//	And bring the Finder to the front
			//	PCB:  this should only happen if the File is a clipping or a folder, no?
			if ( IsContainer() )
				::SetFrontProcess(&finderPSN);
		}	
		
		catch(...)
		{
			::SysBeep(1);	
		}
		
		if (alias != NULL)
			::DisposeHandle((Handle)alias);
			
		::AEDisposeDesc(&event);
	
	}
	
}


//################################################################################
//################################################################################
#pragma mark -
#pragma mark COMPARATORS


Int32				
CMacCVSProjectViewItem::CompareName(	CMacCVSProjectViewItem 	*inItemOne, 
										CMacCVSProjectViewItem 	*inItemTwo)
{

	char					itemOneText[256]	= "";
	char					itemTwoText[256] 	= "";
	
	inItemOne->GetNameCStr(itemOneText);
	inItemTwo->GetNameCStr(itemTwoText);
	
	char					*currentCharacterPtr = itemOneText,
							currentCharacter;
							
	while (true) {

		currentCharacter = *currentCharacterPtr;

		if (currentCharacter == 0)
			break;

		if ((currentCharacter >= 'A') && (currentCharacter <= 'Z'))
			*currentCharacterPtr = currentCharacter - 'A' + 'a';

		currentCharacterPtr++;

	}
	
	currentCharacterPtr = itemTwoText;

	while (true) {
	
		currentCharacter = *currentCharacterPtr;
		
		if (currentCharacter == 0)
			break;
		
		if ((currentCharacter >= 'A') && (currentCharacter <= 'Z'))
			*currentCharacterPtr = currentCharacter - 'A' + 'a';
		
		currentCharacterPtr++;
	
	}
	
	return strcmp( itemOneText, itemTwoText );

}


Boolean
CMacCVSProjectViewItem::IsEqualName(	CMacCVSProjectViewItem 	*inItemOne,
										CMacCVSProjectViewItem	*inItemTwo)
{
	
	char					itemOneText[256]	= "";
	char					itemTwoText[256]	= "";
	
	inItemOne->GetNameCStr(itemOneText);
	inItemTwo->GetNameCStr(itemTwoText);
	
	char					*currentCharacterPtr = itemOneText,
							currentCharacter;
							
	while (true) {
	
		currentCharacter = *currentCharacterPtr;
	
		if (currentCharacter == 0)
			break;
	
		if ((currentCharacter >= 'A') && (currentCharacter <= 'Z'))
			*currentCharacterPtr = currentCharacter - 'A' + 'a';
	
		currentCharacterPtr++;
	
	}
	
	currentCharacterPtr = itemTwoText;

	while (true)
	{
		currentCharacter = *currentCharacterPtr;
		
		if (currentCharacter == 0)
			break;
		
		if ((currentCharacter >= 'A') && (currentCharacter <= 'Z'))
			*currentCharacterPtr = currentCharacter - 'A' + 'a';
		
		currentCharacterPtr++;
	
	}
	
	return !strcmp(itemOneText, itemTwoText);

}


Int32
CMacCVSProjectViewItem::CompareDate(	CMacCVSProjectViewItem 	*inItemOne, 
										CMacCVSProjectViewItem 	*inItemTwo)
{

	return inItemTwo->GetCachedModificationDate() - inItemOne->GetCachedModificationDate();

}


Boolean
CMacCVSProjectViewItem::IsEqualDate(	CMacCVSProjectViewItem	*inItemOne,
										CMacCVSProjectViewItem	*inItemTwo)
{

	return inItemOne->GetCachedModificationDate() == inItemTwo->GetCachedModificationDate();

}


Int32
CMacCVSProjectViewItem::CompareStatus(	CMacCVSProjectViewItem 	*inItemOne, 
										CMacCVSProjectViewItem 	*inItemTwo)
{

	return inItemOne->GetProjectItemStatus() - inItemTwo->GetProjectItemStatus();

}


Boolean
CMacCVSProjectViewItem::IsEqualStatus(	CMacCVSProjectViewItem	*inItemOne,
										CMacCVSProjectViewItem	*inItemTwo)
{

	return inItemOne->GetProjectItemStatus() == inItemTwo->GetProjectItemStatus();

}
