//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSProjectViewRootFolder.cp
//
//	  Authors: 	Ram Rajadhyaksha
//
//################################################################################


#include "CVSEngineUtils.h"

#include "CMacCVSProjectViewItem.h"
#include "CMacCVSProjectViewFolder.h"
#include "CMacCVSProjectViewRootFolder.h"


//  CMacCVSProjectViewRootFolder is mainly used to fake out children items by
//  returning a path name from GetCachedLocalPathPStr that is of the
//  actual project root. This is needed so we can compare paths in file
//  ckids correctly. 


CMacCVSProjectViewRootFolder::
CMacCVSProjectViewRootFolder( const FSSpec *inAnchorFileSpec, const FSSpec *inProjectRootFileSpec ):

	CMacCVSProjectViewFolder( NULL, inAnchorFileSpec )
	
	{

		mPathStrLen = 0;

	}


CMacCVSProjectViewRootFolder::~CMacCVSProjectViewRootFolder()
	{

	}


void
CMacCVSProjectViewRootFolder::GetCachedLocalPathPStr(Str255 outLocalPathPStr) const
	{

//  mCachedPathPStr will always begin with a '/' and
//  end without a '/'. (see constructor)

		UInt32	copyLen;
				
		if ( mPathStrLen > 0 )
			{
			
				copyLen = mPathStrLen;
				
				if ( copyLen > 255 )
					copyLen = 255;
					
				outLocalPathPStr[0] = copyLen;
				
				::BlockMoveData( mCachedPathStr, &outLocalPathPStr[1], outLocalPathPStr[0] );
				
			}
					
	}

	
void
CMacCVSProjectViewRootFolder::GetCachedLocalPathStr(Handle inOutLocalPathStrHndl) const
	{
	
		if ( inOutLocalPathStrHndl != NULL )
			{
			
				if ( mPathStrLen == 0 )
					{
					
						::SetHandleSize( inOutLocalPathStrHndl, 0 );
						
					}
				
				else
					{
					
						::SetHandleSize( inOutLocalPathStrHndl, mPathStrLen );
						
						if ( ::MemError() == noErr )						
							::BlockMoveData( mCachedPathStr, *inOutLocalPathStrHndl, mPathStrLen );
					
					}
					
			}
			
	}
	
	
LArray *
CMacCVSProjectViewRootFolder::ValidateContentsChanged( const FSSpec *inProjectRootFileSpec )
	{
	
		OSErr				result 	= noErr;
		CInfoPBRec	pb;
		
		if ( mIsInvalid )
			{
				
				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= 0;
				pb.hFileInfo.ioNamePtr			= mFileSpec.name;
				pb.hFileInfo.ioDirID				= mFileSpec.parID;
				pb.hFileInfo.ioVRefNum			= mFileSpec.vRefNum;
				
				result = ::PBGetCatInfoSync( &pb );
				
				if ( result != noErr )
					throw( result );
				
				if ( (pb.dirInfo.ioFlAttrib & ioDirMask) == 0 )
					throw( (OSErr) -120 );
					
				mPathStrLen = kDefaultPathBufferLen;
				
				if ( !FSpIsEqual(&mFileSpec, inProjectRootFileSpec) )
					
					result = FSpGetLocalCVSPath( &mFileSpec, inProjectRootFileSpec, mCachedPathStr, &mPathStrLen );
				
				else mPathStrLen = 0;
				
				if ( result != noErr )
					{
					
						mPathStrLen = 0;
						
						throw( result );
							
					}
									
			}
			
		return CMacCVSProjectViewFolder::ValidateContentsChanged( inProjectRootFileSpec );

	}