//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 	CMacCVSProjectViewSelector.cp
//
//	  Author: Scott M. Silver and Ram Rajadhyaksha
//
//################################################################################


#include "CMacCVSProjectView.h"
#include "CMacCVSProjectItemDragTask.h"
#include "CMacCVSProjectViewSelector.h"


CMacCVSProjectViewSelector::CMacCVSProjectViewSelector( CMacCVSProjectView*	inProjectView ):

	CTableDragSelector( inProjectView )

	{
		mProjectView = inProjectView;
		mOldScroll.v = 0;
		mOldScroll.h = 0;
	}


void
CMacCVSProjectViewSelector::HandleDrag( const STableCell &inCell, const SMouseDownEvent &inMouseDown )
	{
	
		RgnHandle 	dragRgnH 				= NULL;
		LArray			*selectedItems	= NULL;
		
		try
			{
			
				dragRgnH = ::NewRgn();
				ThrowIfNil_( dragRgnH );

//  get the hilite region
	
				mProjectView->FocusDraw();
				mProjectView->GetHiliteRgn( dragRgnH );

//  offset the region into global space

				Point thePoint = { 0, 0 };	
				
				::LocalToGlobal( &thePoint );
				::OffsetRgn( dragRgnH, thePoint.h, thePoint.v );

//  create the drag task and do the drag

				selectedItems = mProjectView->GetFlatSelection();

				if ( selectedItems != NULL )
					{
					
						CMacCVSProjectItemDragTask itemTask( inMouseDown.macEvent, *selectedItems, dragRgnH );
						OSErr		err = itemTask.DoDrag();
	
						delete selectedItems;
						selectedItems = NULL;
						
					}
				
				::DisposeRgn( dragRgnH );
				dragRgnH = NULL;
				
			}
			
		catch ( ... )
			{
			
				if ( selectedItems != NULL )
					{
					
						delete selectedItems;
						selectedItems = NULL;
			
					}
					
				if ( dragRgnH != NULL )
					{
						
						::DisposeRgn( dragRgnH );
						
					}
					
				throw;
			
			}
	
	}


Boolean	
CMacCVSProjectViewSelector::HitCellHotSpot( const STableCell &inCell, const Rect &inTotalSelectionRect )
	{
	
		Boolean		result						= false;
		Rect			cellTextHotSpot		= { 0, 0, 0, 0 },
							cellIconHotSpot		= { 0, 0, 0, 0 },
							dummy							= { 0, 0, 0, 0 };
							
		mProjectView->CalcCellHotSpots( inCell, cellIconHotSpot, cellTextHotSpot );
	
		if ( (::SectRect(&inTotalSelectionRect, &cellIconHotSpot, &dummy)) || (::SectRect(&inTotalSelectionRect, &cellTextHotSpot, &dummy)) )
			result = true;
			
		return result;
	
	}
	
	
void
CMacCVSProjectViewSelector::UserDraggedSelection()
	{
	
		mProjectView->UserDraggedSelection();
		
	}


void
CMacCVSProjectViewSelector::PreScroll( const SPoint32 inScroll )
	{
		mOldScroll = mProjectView->mScrollUnit;
		
		mProjectView->mScrollUnit.v = (inScroll.v / 10 * CMacCVSProjectView::kDefaultRowHeight) + CMacCVSProjectView::kDefaultRowHeight;
		
		//  Don't modify the horizontal scroll units
		//mProjectView->mScrollUnit.h = (inScroll.h / 10 * CMacCVSProjectView::kDefaultRowHeight) + CMacCVSProjectView::kDefaultRowHeight;
	}
	
	
void
CMacCVSProjectViewSelector::PostScroll( ) 
	{
		mProjectView->mScrollUnit = mOldScroll;
	}
