//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 	CMacCVSProjectViewWindow.cp
//
//	  Author: Ram Rajadhyaksha
//
//################################################################################


#include <stdio.h>
#include <Scrap.h>
#include <UDesktop.h>
#include <LWindow.h>
#include <LTextButton.h>

#include "CVSEngineUtils.h"

#include "CPathPopup.h"
#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"
#include "CMacCVSLurkersTask.h"
#include "CMacCVSLurkersView.h"
#include "CMacCVSProjectViewWindow.h"
#include "CMacCVSThreadedWindow.h"
#include "CMacCVSAppleEventItem.h"


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS/DESTRUCTORS


CMacCVSProjectViewWindow::CMacCVSProjectViewWindow( LStream *inStream ):

	CMacCVSWindow(inStream)
	
	{

		mProjectView 		= NULL;
		mSpinnyThingy 	= NULL;
		
		mWindowPosResID	= kProjectViewWindowPositionResID;
		
	}


CMacCVSProjectViewWindow::CMacCVSProjectViewWindow( LStream *inStream, CMacCVSDoc *inOwnerDoc ):

	CMacCVSWindow(inStream, inOwnerDoc)
	
	{
		
		mProjectView 		= NULL;
		mSpinnyThingy 	= NULL;

		mWindowPosResID	= kProjectViewWindowPositionResID;
		
	}


CMacCVSProjectViewWindow::~CMacCVSProjectViewWindow()
	{

//  This part is *very* important. In fact, the whole reason this class exists
//  is because of this problem: We need to stop both the spin icon and the project
//  view threads before they are destroyed. Since shutting down a thread requires
//  a context switch, if you delete the spin pane first and yield until it dies,
//  you will crash because the project view thread is still running and assumes
//  that the spin pane still exists. So, remove the spin pane as listener before
//  we destroy the window.
		
		mProjectView->RemoveListener( mSpinnyThingy );
		
		SaveWindowPosition();
		
	}


CMacCVSProjectViewWindow *
CMacCVSProjectViewWindow::CreateCMacCVSProjectViewWindowFromStream( LStream *inStream, CMacCVSDoc *inOwnerDoc )
	{

		return new CMacCVSProjectViewWindow( inStream, inOwnerDoc );

	}


void
CMacCVSProjectViewWindow::FinishCreateSelf()
	{
		
		LTextButton		*theSortButton	= NULL;
		
		mProjectView = dynamic_cast<CMacCVSProjectView *>( FindPaneByID(CMacCVSProjectView::class_ID) );
		ThrowIfNil_( mProjectView );
				
//  Set up the text box buttons.
		
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByNameButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mProjectView );
			
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByStatusButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mProjectView );
			
		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByVersionButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mProjectView );

		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByModDateButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mProjectView );

		theSortButton = dynamic_cast<LTextButton *>( FindPaneByID(kSortByTagButtonID) );
		ThrowIfNil_(theSortButton);
		theSortButton->AddListener( mProjectView );
		
//  Hook up the spinnythingymoboby.
		
		mSpinnyThingy = dynamic_cast<CSpinIndicator *>( FindPaneByID(CSpinIndicator::class_ID) );
		ThrowIfNil_( mSpinnyThingy );
		mProjectView->AddListener( mSpinnyThingy );
		
	  SetLatentSub( mProjectView );
		
		Inherited::FinishCreateSelf();

	}
	
	
void
CMacCVSProjectViewWindow::SetWindowPosition()
	{
		FSSpec		rootFolder = {0, 0, "\p"};
		
		if (GetWindowRootFolder(rootFolder) && FSpIsRepositoryDir( &rootFolder ) )
			{
				FSSpec		tagFileSpec		= { 0, 0, "\p" };
				short			resFileRef 		= 0;
				SInt16		intOldResRef 	= 0;
				
				OSErr	err = FSpGetRootFileSpec( &rootFolder, &tagFileSpec );
				if (err != noErr) return;
				
				intOldResRef = ::CurResFile();
				
				try
					{
						resFileRef = FSpOpenResFile( &tagFileSpec, fsRdPerm );
						if (resFileRef == -1 || ResError() != noErr) return;
						
						Rect			windowBounds = {0, 0, 0, 0};
						
						ReadWindowPosResource( windowBounds );
						SafeSetWindowBounds( windowBounds );

						CloseResFile( resFileRef );
						resFileRef = 0;
					}
				catch ( ... )
					{
						::UseResFile( intOldResRef );
						throw;
					}

				::UseResFile( intOldResRef );	
			}
		else
			{
				if ( mOwnerDoc != NULL )
					mOwnerDoc->PositionDocWindow( this );
			}
	}
	

	
void
CMacCVSProjectViewWindow::SaveWindowPosition()
	{
		FSSpec		rootFolder = {0, 0, "\p"};
		Rect			windowBounds;
		
		GetGlobalBounds( windowBounds );
		
		if (GetWindowRootFolder(rootFolder) && FSpIsRepositoryDir( &rootFolder ) )
			{
				FSSpec		tagFileSpec		= { 0, 0, "\p" };
				short			resFileRef 		= 0;
				SInt16		intOldResRef 	= 0;
				Boolean		newResource 	= false;
				
				OSErr	err = FSpGetRootFileSpec( &rootFolder, &tagFileSpec );
				if (err != noErr) return;
				
				intOldResRef = ::CurResFile();
				
				try
					{
						resFileRef = ::FSpOpenResFile( &tagFileSpec, fsRdWrPerm );
						err = ::ResError();
						
						if ( err == eofErr )
						{
							::HCreateResFile( tagFileSpec.vRefNum, tagFileSpec.parID, tagFileSpec.name );
							err = ::ResError();
							if (err != noErr) return;
							
							resFileRef = ::FSpOpenResFile( &tagFileSpec, fsRdWrPerm );
							err = ::ResError();
							
						}
						if (resFileRef == -1 || err != noErr) return;
						
						AddOrChangeWindowPosResource( windowBounds );
							
						CloseResFile( resFileRef );
						resFileRef = 0;
					}
				catch ( ... )
					{
						::UseResFile( intOldResRef );
						throw;
					}
					
				::UseResFile( intOldResRef );
			}
		else
			{			
				Assert_( mOwnerDoc != NULL );
				mOwnerDoc->SaveDocWindowPosition( this );				
			}
			
		mWindowPositionSaved = true;
	}


void
CMacCVSProjectViewWindow::SelectFile( const FSSpec *inFileSpec )
	{
		mProjectView->SelectItem(inFileSpec);
	}


Boolean
CMacCVSProjectViewWindow::GetWindowRootFolder( FSSpec &outFileSpec )
	{
			CMacCVSProjectViewFolder		*rootItem = NULL;
		
			rootItem = mProjectView->GetRootItem();
			
			if ( rootItem != NULL )
				{
					rootItem->GetFSSpec( outFileSpec );
					return true;
				}

			return false;
	}
	
	

void
CMacCVSProjectViewWindow::ClickInDrag( const EventRecord	&inMacEvent )
	{
		Boolean			cmdKeyDown = (inMacEvent.modifiers & cmdKey) != 0;
		
		if (cmdKeyDown && (UDesktop::FetchTopRegular() == this))
			{
				WindowPtr		theWindow = mMacWindowP;
				Point				localClick;
				
				// ignore truncation etc for now.
				// we should probably use the struct rgn as well.
				short				windowWidth = mMacWindowP->portRect.right - mMacWindowP->portRect.left;
				short				titleWidth = ((WindowPeek)mMacWindowP)->titleWidth;
				short				titleLeft = windowWidth / 2 - titleWidth / 2;
				short				titleRight = titleLeft + titleWidth;
			
				localClick = inMacEvent.where;
				GlobalToLocal(&localClick);

				if (localClick.h >= titleLeft && localClick.h < titleRight)
					{
						FSSpec			rootFolderSpec;

						if ( GetWindowRootFolder(rootFolderSpec) )
							{
								CPathPopup	thePathPop(rootFolderSpec, 128, true, true, true);			// system font traits, allow open
	
								StRegion		structRgn( ((WindowPeek)mMacWindowP)->strucRgn );
								StRegion		contentRgn( ((WindowPeek)mMacWindowP)->contRgn );
								short				hOffset = structRgn.Bounds().top - contentRgn.Bounds().top;
								Point				titleTopLeft;
								
								SetPt(&titleTopLeft, titleLeft, hOffset + 3);
								LocalToGlobal(&titleTopLeft);
								
								thePathPop.Track(titleTopLeft);															// to be like the finder, we should centre the popup
								return;
							}
					}
			}
		
		LWindow::ClickInDrag( inMacEvent );
	}


#pragma mark -

void
CMacCVSProjectViewWindow::FindCommandStatus( CommandT inCommand, Boolean &outEnabled, Boolean &outUsesMark, Char16 &outMark, Str255 outName )
	{
	
		switch ( inCommand )
			{
			
				case cmd_SaveAs:
					
					outEnabled = (mOwnerDoc->GetWindow() == this);
					
				break;
				
				default:
				
					Inherited::FindCommandStatus(inCommand, outEnabled, outUsesMark, outMark, outName);
				
				break;
			
			
			}
	}



Boolean
CMacCVSProjectViewWindow::ObeyCommand( CommandT inCommand, void *ioParam )
	{
	
		return Inherited::ObeyCommand(inCommand, ioParam);
	
	}


#pragma mark -


void
CMacCVSProjectViewWindow::GetSubModelByName( DescType inModelID, Str255 inName, AEDesc &outToken ) const
	{
	
		switch ( inModelID )
			{			
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:
					{
						Boolean bFoundIt 	= false;
						FSSpec	fsItem		= { 0 };
												
						ThrowIfNil_( mProjectView );
						mProjectView->GetNamedItemFSSpec( inName, fsItem );
						
						bFoundIt = FSpIsValid( fsItem );
															
						if ( bFoundIt )
							{
								//  create an object for this file		

								CMacCVSAppleEventItem	*pProjectItem = new CMacCVSAppleEventItem( const_cast<CMacCVSDoc *>(mOwnerDoc), fsItem );
								ThrowIfNil_( pProjectItem );
																		
								pProjectItem->SetLaziness( true );					
								PutInToken( pProjectItem, outToken );
							}
						else
							{
								//  ...or not!
								
								ThrowOSErr_( errAENoSuchObject );
							}
					}
				break;
						
				default:
					Inherited::GetSubModelByName( inModelID, inName, outToken );
				break;			
			}
					
	}


void
CMacCVSProjectViewWindow::GetSubModelByPosition( DescType inModelID, SInt32 inPosition, AEDesc &outToken ) const
	{
	
		switch ( inModelID )
			{		
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:					
					
					Boolean	bFoundIt 	= false;
					FSSpec 	fsItem 		= { 0 };
					
					ThrowIfNil_( mProjectView );
					mProjectView->GetNthItemFSSpec( inPosition, fsItem );
					
					bFoundIt = FSpIsValid( fsItem );
					
					if ( bFoundIt )
						{
							// create an object for this file					
					
							CMacCVSAppleEventItem	*pProjectItem = new CMacCVSAppleEventItem( const_cast<CMacCVSDoc *>(mOwnerDoc), fsItem );
							ThrowIfNil_( pProjectItem );			
									
							pProjectItem->SetLaziness( true );
							PutInToken( pProjectItem, outToken );
						}
					else
						{
							ThrowOSErr_( errAENoSuchObject );
						}
						
				break;					
					
				default:					
					Inherited::GetSubModelByPosition(inModelID, inPosition, outToken);
				break;
			}
			
	}
	

SInt32
CMacCVSProjectViewWindow::CountSubModels(DescType inModelID) const
	{
		
		SInt32 intSubModelCount = 0;
		
		switch (inModelID)
			{			
				case CMacCVSAppleEventItem::cMacCVSAppleEventItem:				
					
					Assert_( mProjectView != NULL );
					
					if ( mProjectView != NULL )
						{
							intSubModelCount = mProjectView->GetVisibleItemCount();
						}
			
				break;		
					
				default:					
					intSubModelCount = Inherited::CountSubModels(inModelID);
				break;
			}
	
		return intSubModelCount;
		
	}
