//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CMacCVSStatusTask.cp
//	
//	  	Authors: 	Ram Rajadhyaksha
//
//################################################################################


#include "UErrorDialog.h"

#include "CMacCVSMessageView.h"
#include "CMacCVSMessageWindow.h"

#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"
#include "CVSEngine.h"
#include "CMacCVSStatusWindow.h"
#include "CMacCVSStatusTask.h"


CMacCVSStatusTask::CMacCVSStatusTask( const CMacCVSStatusWindow *inOwnerWindow, CMacCVSDoc *inOwnerDoc,
			const FSSpec *inTargetItem ):

	CMacCVSWindowTask( inOwnerWindow, inOwnerDoc )
		
	{
		mTargetItem.name[0] = 0;
		mTargetItem.parID		= 0;
		mTargetItem.vRefNum = 0;

		if ( inTargetItem != NULL )
			{
				mTargetItem = *inTargetItem;
			}

		SetEventNotifier( Notifier );
	}
	

CMacCVSStatusTask::~CMacCVSStatusTask()
	{
	}

	
OSErr
CMacCVSStatusTask::DoEngineCommand()
	{
		
		return CVSEngineStatus( mEngineRef, &mTargetItem, CMacCVSStatusWindow::StatusNotifier, mOwnerWindow );

	}
	
	
pascal SInt32		
CMacCVSStatusTask::Notifier(const CCVSEngineRef					inEngineRef,
														const FSSpec								*inTargetItem,
														const SInt32 								inEvent, 
														const SInt32 								inError, 
														const UInt32 								inCookie, 
														const void 									*inDataPtr, 
														void 												*inContextPtr)
	{
	
		OSErr								error				= noErr;
		Boolean							bResult			= true;
		CMacCVSStatusTask		*theTask		= (CMacCVSStatusTask *)inContextPtr;
		
		if ( theTask != NULL )
			{
			
				try
					{
						if ( inError != noErr )
							{
								if ( (inError != kCVSEngineSkippedFile) && (inError != kCVSEngineOperationAbortedErr) )
									{
										theTask->PostFileError( inTargetItem, NULL, 0, inError );
									}
							}
							
						switch ( inEvent )
							{
								case kCVSEngineUpdatePreActionEvent:

//  If we send a file that has been MROed but not actually
//  modified for a CVS status query, the server will respond
//  with a checked-in response to tell us that we're high
//  and that the file is not really modified. We don't want
//  to update our file because right now, the status operation
//  doesn't have knowledge of the user's automerging preferences.

									bResult = false;
								
								break;
								
								case kCVSEngineYieldActionEvent:
									CThreadedTask::Yield();
								break;
							}	
					}
					
				catch ( ... )
					{
					
						bResult = false;
						
					}
					
			}
			
		return bResult;
		
	}
	

void
CMacCVSStatusTask::GetTaskName( Str255 outNamePStr )
	{
		LStr255		theNameStr;
		
		theNameStr = "\pstatus query";
			
		if ( theNameStr[0] > 0 )
			::BlockMoveData( theNameStr, outNamePStr, theNameStr[0] + 1 );
	}


#pragma mark -


CMacCVSDiffTask::CMacCVSDiffTask( const CMacCVSStatusWindow *inOwnerWindow, CMacCVSDoc *inOwnerDoc,
			const FSSpec *inTargetItem, const CCVSEngineDiffOptions *inDiffOptions ):

	CMacCVSWindowTask( inOwnerWindow, inOwnerDoc )
		
	{
	
		mDiffOptions.Init();
		mTargetItem.name[0] = 0;
		mTargetItem.parID		= 0;
		mTargetItem.vRefNum = 0;

		if ( inTargetItem != NULL )
			{
				mTargetItem = *inTargetItem;
			}

		if ( inDiffOptions != NULL )
			{
				mDiffOptions = *inDiffOptions;
			}
			
		SetEventNotifier( Notifier );
	}
	

CMacCVSDiffTask::~CMacCVSDiffTask()
	{
	}

	
OSErr
CMacCVSDiffTask::DoEngineCommand()
	{
		return CVSEngineDiff( mEngineRef, &mTargetItem, &mDiffOptions, CMacCVSStatusWindow::StatusNotifier, mOwnerWindow );
	}
	
	
pascal SInt32		
CMacCVSDiffTask::Notifier(const CCVSEngineRef					inEngineRef,
														const FSSpec								*inTargetItem,
														const SInt32 								inEvent, 
														const SInt32 								inError, 
														const UInt32 								inCookie, 
														const void 									*inDataPtr, 
														void 												*inContextPtr)
	{
	
		OSErr								error				= noErr;
		Boolean							bResult			= true;
		CMacCVSDiffTask		*theTask			= (CMacCVSDiffTask *)inContextPtr;
		
		if ( theTask != NULL )
			{
			
				try
					{

						if ( inError != noErr )
							theTask->PostFileError( inTargetItem, NULL, 0, inError );
										
						if ( inEvent == kCVSEngineYieldActionEvent )
							CThreadedTask::Yield();
			
					}
					
				catch ( ... )
					{
					
						bResult = false;
						
					}
					
			}
			
		return bResult;
		
	}
	

