//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 	CMacCVSStatusWindow.cp
//
//	  Author: Ram Rajadhyaksha
//
//################################################################################


#include <LWindow.h>
#include <LBroadcaster.h>
#include <LTextEditView.h>
#include <UConditionalDialogs.h>		// JWW - For Navigation Services

#include "CThreadedTask.h"
#include "CMacCVSStatusTask.h"
#include "CMacCVSShowEditorsTask.h"
#include "CMacCVSShowWatchersTask.h"
#include "CMacCVSStatusWindow.h"


//################################################################################
//################################################################################
#pragma mark CONSTRUCTORS/DESTRUCTORS


CMacCVSStatusWindow::CMacCVSStatusWindow( LStream *inStream ): 

	CMacCVSThreadedWindow( inStream )
	
	{
		
		mTextView 			= NULL;
		mDataHndl				= NULL;
		
		mWindowPosResID	= kStatusWindowPositionResID;
		
	}


CMacCVSStatusWindow::~CMacCVSStatusWindow()
	{
			
		if ( mDataHndl != NULL )
			::DisposeHandle( mDataHndl );
			
	}


CMacCVSStatusWindow *
CMacCVSStatusWindow::CreateCMacCVSStatusWindowFromStream( LStream *inStream )
	{

		return new CMacCVSStatusWindow( inStream );

	}


void 
CMacCVSStatusWindow::FinishCreateSelf()
	{

		Inherited::FinishCreateSelf();
		
		mTextView = dynamic_cast<LTextEditView *>( FindPaneByID(kMacCVSStatusTEPaneID) );
		ThrowIfNil_( mTextView );
		
		//  Attachment for keyboard navigation.
		LKeyScrollAttachment	*keyAttachment = new LKeyScrollAttachment( mTextView );
		ThrowIfNil_( keyAttachment );
		
		mTextView->AddAttachment( keyAttachment );
		
		SetLatentSub( mTextView );
	}


#pragma mark -

void
CMacCVSStatusWindow::Status( const FSSpec *inTargetItem, CMacCVSDoc *inOwnerDoc, const Boolean inDoDiff, const void *inCommandData, UInt32 inDelayTicks )
	{
	
		Assert_( inTargetItem != NULL );
		Assert_( inOwnerDoc != NULL );
		Assert_( mWindowTask == NULL );
		
		TString<Str255>	titleString;
		
		if ( (mWindowTask == NULL) && (inTargetItem != NULL) && (inOwnerDoc != NULL) )
			{
			
				if ( !inDoDiff )
					{
					
						GetDescriptor( titleString );
						
						titleString = titleString + inTargetItem->name;
						
						SetDescriptor( titleString );
				
					}
				else
					{
					
						titleString[0] = 0x00;
						
						titleString.Append( "\pDifferences of " );
						titleString.Append( inTargetItem->name );
							
						SetDescriptor( titleString );
					
					}
	
				Show();
				
				if ( inDoDiff )
					mWindowTask = new CMacCVSDiffTask( this, inOwnerDoc, inTargetItem, (const CCVSEngineDiffOptions *)inCommandData );

				else
					mWindowTask = new CMacCVSStatusTask( this, inOwnerDoc, inTargetItem );
					
				ThrowIfNil_( mWindowTask );
				
				PrimeWindowTask(inDelayTicks);

			}
	
	}
	

void
CMacCVSStatusWindow::ShowEditors( const FSSpec *inTargetItem, CMacCVSDoc *inOwnerDoc, UInt32 inDelayTicks )
	{
	
		Assert_( inTargetItem != NULL );
		Assert_( inOwnerDoc != NULL );
		Assert_( mWindowTask == NULL );
		
		TString<Str255>	pstrTitle;
		
		if ( (mWindowTask == NULL) && (inTargetItem != NULL) && (inOwnerDoc != NULL) )
			{
				pstrTitle[0] = 0;					
				pstrTitle.Append( "\pEditors of " );
				pstrTitle.Append( inTargetItem->name );
				SetDescriptor( pstrTitle );
			
				Show();
				
				mWindowTask = new CMacCVSShowEditorsTask( this, inOwnerDoc, inTargetItem );					
				ThrowIfNil_( mWindowTask );
				
				PrimeWindowTask( inDelayTicks );
			}
	
	}
	

void
CMacCVSStatusWindow::ShowWatchers( const FSSpec *inTargetItem, CMacCVSDoc *inOwnerDoc, UInt32 inDelayTicks )
	{
	
		Assert_( inTargetItem != NULL );
		Assert_( inOwnerDoc != NULL );
		Assert_( mWindowTask == NULL );
		
		TString<Str255>	pstrTitle;
		
		if ( (mWindowTask == NULL) && (inTargetItem != NULL) && (inOwnerDoc != NULL) )
			{
				pstrTitle[0] = 0;					
				pstrTitle.Append( "\pWatchers of " );
				pstrTitle.Append( inTargetItem->name );
				SetDescriptor( pstrTitle );
			
				Show();
				
				mWindowTask = new CMacCVSShowWatchersTask( this, inOwnerDoc, inTargetItem );					
				ThrowIfNil_( mWindowTask );
				
				PrimeWindowTask( inDelayTicks );
			}
	
	}
	

pascal SInt32
CMacCVSStatusWindow::StatusNotifier( const CCVSEngineRef inEngineRef, const UInt32 inDataType, const UInt32 inDataLen, const void *inData, void *inContextPtr )
	{
	
		OSErr			result				= noErr;
		
		SInt32		offset 				= 0x00000000,
							bytesWritten	= 0x00000000,
							newTextLen 		= 0x00000000;
															

		CMacCVSStatusWindow	*theStatusWindow = (CMacCVSStatusWindow *)inContextPtr;
		
		Assert_( theStatusWindow != NULL );
		
		if ( theStatusWindow != NULL )
			{
			
				try
					{
									
						if ( inDataLen > 0 )
							{
							
								if ( theStatusWindow->mDataHndl != NULL )
									{
									
										offset = ::GetHandleSize( theStatusWindow->mDataHndl );
										
										::SetHandleSize( theStatusWindow->mDataHndl, offset + inDataLen );
										
										result = ::MemError();
										
										if ( result != noErr )
											throw( result );
											
									}
								
								else
									{
									
										theStatusWindow->mDataHndl = ::NewHandle( inDataLen );
										
										if ( theStatusWindow->mDataHndl == NULL )
											{
											
												throw( ::MemError() );
												
											}
									
									}
							
								::BlockMoveData( inData, ((UInt8 *)(*theStatusWindow->mDataHndl)) + offset, inDataLen );
								bytesWritten = inDataLen;
									
							}
							
						else
							{
							
								Assert_( theStatusWindow->mTextView != NULL );
						
								if ( theStatusWindow->mDataHndl != NULL )
									{
									
										newTextLen = ::GetHandleSize( theStatusWindow->mDataHndl );
							
										if ( newTextLen > 0 )
											{
												Handle		dataHandle = theStatusWindow->mDataHndl;
												
												if ( newTextLen > kMaximumTextLen )
													{
														
														Str255	warningStr = "\p\r\r   *** WARNING  THIS TEXT HAS BEEN TRUNCATED. SAVE TO FILE TO SEE ALL OF THE TEXT ***";
														OSErr		err = ::HandToHand(&dataHandle);
														ThrowIfOSErr_(err);
														::SetHandleSize( dataHandle, kMaximumTextLen - warningStr[0] );
														
														::PtrAndHand( &warningStr[1], dataHandle, warningStr[0]);		// OK if fails
													}
													
												theStatusWindow->mTextView->SetTextHandle( dataHandle );
													
												// Don't delete this yet, as we need it for saving to disk
												// ::DisposeHandle( theStatusWindow->mDataHndl );
												// theStatusWindow->mDataHndl = NULL;
												
												if ( dataHandle != theStatusWindow->mDataHndl)
													::DisposeHandle(dataHandle);
													
											}
										
									}
																	
							}
							
					}
					
				catch ( ... )
					{
					
						// Don't delete this yet, as we need it for saving to disk
						/*
						if ( theStatusWindow->mDataHndl != NULL )
							{
							
								::DisposeHandle( theStatusWindow->mDataHndl );
								theStatusWindow->mDataHndl = NULL;
							
							}
						*/
					}
					
			}
	
		return inDataLen;
		
	}

#pragma mark -


void
CMacCVSStatusWindow::FindCommandStatus( CommandT	inCommand, Boolean &outEnabled, Boolean &outUsesMark, Char16 &outMark, Str255 outName )
	{
	
			switch ( inCommand )
			{
			
				case cmd_Save:
				case cmd_SaveAs:

					// enable if the task has finished, and we have data
					outEnabled = (mWindowTask == nil) && (mDataHndl != nil);

				break;
				
				default:
				
					LCommander::FindCommandStatus( inCommand, outEnabled, outUsesMark, outMark, outName );
			
				break;
				
			}

	}


Boolean
CMacCVSStatusWindow::ObeyCommand( CommandT inCommand, void *ioParam )
	{
		
		Boolean		bResult = false;
		
		switch ( inCommand )
			{
			
				case cmd_Save:
				case cmd_SaveAs:
				
					{
					
						ThrowIfNil_(mDataHndl);
						
						StandardFileReply	reply;
						Str255				saveAsPrompt;
						Str255				defaultName;
						
						::GetIndString(saveAsPrompt, STRx_Standards, str_SaveAs);

						::UDesktop::Deactivate();
						// JWW - Use Navigation Services if possible
//						::StandardPutFile(saveAsPrompt, GetDescriptor(defaultName), &reply);
						bool isReplacing;
						reply.sfGood = UConditionalDialogs::AskSaveFile(GetDescriptor(defaultName), 'TEXT', reply.sfFile, isReplacing, kNavDefaultNavDlogOptions);
						::UDesktop::Activate();

						if (reply.sfGood)
							{
							
								if (reply.sfReplacing)	// Delete existing file
									ThrowIfOSErr_(::FSpDelete(&reply.sfFile));
								
								LFileStream				saveFile(reply.sfFile);
							
								saveFile.CreateNewDataFile('CWIE', 'TEXT');
								saveFile.OpenDataFork(fsRdWrPerm);				// can throw
								
								Int32							dataSize = GetHandleSize(mDataHndl);
								StHandleLocker		handleLocker(mDataHndl);
								
								OSErr		err = saveFile.PutBytes( *mDataHndl, dataSize );
								
								saveFile.CloseDataFork();
								
								ThrowIfOSErr_(err);
													
						}
				
						bResult = true;
					
					}
				break;
			
			
				default:
				
					bResult = LWindow::ObeyCommand( inCommand, ioParam );
					
				break;
				
			}
			
		return bResult;

	}
	
