//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSThreadedTask.cp
	
	Authors:		Ram Rajadhyaksha and Simon Fraser
	
	Description:
	
		Code to implement the basic threaded operation in MacCVS Pro.
		 
	Modification History:

		17 Feb 1999			Ram Rajadhyaksha
		
					File created.
							
-------------------------------------------------------------------------------- */


#include <stdio.h>
#include <string.h>

#include "ThrowOSErr.h"
#include "CMacCVSMessageView.h"
#include "CMacCVSMessageWindow.h"
#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"
#include "UProjectActions.h"

#include "CMacCVSThreadedTask.h"

/*
unsigned char	kFileNameErrStr[]		= "\pCould not complete an operation on the item ";
unsigned char kFileNoNameErrStr[]	= "\pCould not complete an operation on an item";
unsigned char	kOutOfMemoryErr[]		= "\pMacCVS is out of memory! Please increase the application partition size.";
unsigned char kBadResponseErr[]		= "\pThe CVS server sent back a bad response.";
unsigned char	kBadPasswordErr[]		= "\pAuthentication failed. Your password is incorrect!";
unsigned char kTimeoutErr[]				= "\pa network time out occurred. (Check your networking preferences)";
unsigned char kServerReturnedErr[]= "\pthe CVS server returned an error. Check the warnings to see what went wrong.";
unsigned char	kOperationAbortedErr[] = "\pthe operation was cancelled.";
*/


struct SMacCVSThreadedTaskStreamData
	{
		OSType				dataID;
		CMacCVSDoc		*ownerDoc;
	};


CMacCVSThreadedTask::CMacCVSThreadedTask() :

	CBroadcastingThreadedTask( (Boolean)true )
	
	{
		mOwnerDoc 										= NULL;

		mCurrentEvent.descriptorType 	= typeNull;
		mReplyEvent.descriptorType 		= typeNull;
	}
	

CMacCVSThreadedTask::CMacCVSThreadedTask( LStream *inStream, const UInt32 inStackSize ):
	
	CBroadcastingThreadedTask( inStackSize, true )
	
	{
		SMacCVSThreadedTaskStreamData	streamData = {0};
		
		inStream->ReadData( &streamData, sizeof(SMacCVSThreadedTaskStreamData) );
		Assert_( streamData.dataID == kDirectorySpinnerStreamID );		// safety check
	
		mOwnerDoc = streamData.ownerDoc;
		ThrowIfNil_( mOwnerDoc );
		
		mCurrentEvent.descriptorType 	= typeNull;
		mReplyEvent.descriptorType 		= typeNull;
	}


CMacCVSThreadedTask::CMacCVSThreadedTask( CMacCVSDoc *inOwnerDoc, const UInt32 inStackSize ):
	
	CBroadcastingThreadedTask( inStackSize, true )
	
	{
		ThrowIfNil_( inOwnerDoc );
		mOwnerDoc 										= inOwnerDoc;
		
		mCurrentEvent.descriptorType 	= typeNull;
		mReplyEvent.descriptorType 		= typeNull;
	}
	
	
CMacCVSThreadedTask::~CMacCVSThreadedTask()
	{
		
	}


#pragma mark -


void
CMacCVSThreadedTask::SetCompletionEvent( AppleEvent &inCurEvent, AppleEvent &inReplyEvent )
	{
		OSErr	result = noErr;
		
		SignalIf_( mCurrentEvent.descriptorType != typeNull );
	
		mCurrentEvent = inCurEvent;
		mReplyEvent = inReplyEvent;

		if ( mCurrentEvent.descriptorType != typeNull )
			{			
				result = ::AESuspendTheCurrentEvent( &mCurrentEvent );				
				ThrowOSErr( result );					
			}
	}
	

void
CMacCVSThreadedTask::SendCompletionEvent()
	{	
		OSErr	result = noErr;
		
		if ( mCurrentEvent.descriptorType != typeNull )
			result = ::AEResumeTheCurrentEvent( &mCurrentEvent, &mReplyEvent, (AEEventHandlerUPP)kAENoDispatch, 0 );
	
		ThrowOSErr( result );
	}
	
	
void
CMacCVSThreadedTask::AddToCompletionEvent( const OSErr inError )
	{
	
		//  Stick the error codes in the AppleEvent if available
		
		OSErr			result				= noErr;
		Str255		pstrError 		= "\p";
		
		if ( mReplyEvent.descriptorType != typeNull )
			{	
				StAEDescriptor desc( inError );
				
				result = ::AEPutParamDesc( &mReplyEvent, keyErrorNumber, &desc.mDesc );
				SignalIf_( result );

				GetErrorString( inError, pstrError );
				
				if ( pstrError[0] > 0 )
					{					
						StAEDescriptor desc( pstrError );
						result = ::AEPutParamDesc( &mReplyEvent, keyErrorString, &desc.mDesc );						
					}
	
				SignalIf_( result );			
			}
	
	}
	
	
void
CMacCVSThreadedTask::AddToCompletionEvent( const Str255 inErrorPStr )
	{
	
		// Stick the error codes in the AppleEvent if available
		
		OSErr	result = noErr;
		
		if ( mReplyEvent.descriptorType != typeNull )
			{
				StAEDescriptor desc( inErrorPStr );
				
				result = ::AEPutParamDesc( &mReplyEvent, keyErrorString, desc );
				SignalIf_( result );
			}	
	}
	

#pragma mark -


void
CMacCVSThreadedTask::PostFileError( const FSSpec *inTargetItem, const char *inPathStr, const UInt32 inPathLen, const SInt32 inError )
	{
		
		Str255		pstrFileName = "\p";	
		LStr255		pstrError;
		
		if ( inError != kCVSEngineOperationAbortedErr )
			{	
//  If the caller passed in a non-NIL FSSpec, use it's name
		
				if ( inTargetItem != NULL )
					{			
						::BlockMoveData( inTargetItem->name, pstrFileName, inTargetItem->name[0] + 1 );							
					}

//  Else if they called us with a pathname...

				if ( (pstrFileName[0] == 0) && (inPathStr != NULL) && (inPathLen > 0) )
					{
						UInt32		index			= 0;
						UInt32		copyLen		= 0;
											
						Boolean		bDone			= false;

//  Get the file name out of the pathname...boy this is squirrely...
				
						index = inPathLen - 1;
								
						while ( !bDone )
							{			
								if ( inPathStr[index] == '/' )
									{
										bDone = true;
										index++;								
									}							
								else if ( index > 0 )
									{
										index--;
									}
								else 
									{
										bDone = true;
									}
							}
											
						if ( index < inPathLen )
							{
								copyLen = inPathLen - index;
							}
						else 
							{						
								index = 0;
								copyLen = inPathLen;				
							}
							
						if ( copyLen > 255 )
							{
								copyLen = 255;
							}
							
						::BlockMoveData( &inPathStr[index], &pstrFileName[1], copyLen );
						pstrFileName[0] = copyLen;		
					}
				
				if ( pstrFileName[0] > 0 )	
					{
						pstrError += "\pCould not complete an operation on ";
						pstrError += pstrFileName;
						pstrError += " because ";
						GetErrorString( inError, pstrFileName );
						pstrError += pstrFileName;
					}
				else 
					{
						pstrError += "\pCould not complete an operation on an item because ";
						GetErrorString( inError, pstrFileName );
						pstrError += pstrFileName;
					}
					
				if ( pstrError[0] > 0 )
					{
						if ( (inError == kCVSEnginePermissionsErr) || (inError == kCVSEngineBinaryMergingOffErr) )
							{
								//  Pink requested that auto merging errors
								//  should be listed as errors, not as file
								//  items for better visibility.
								
								mOwnerDoc->PostMessage( CMacCVSMessageView::kMessageTypeError, (char *)&pstrError[1], pstrError[0], inTargetItem, sizeof(FSSpec) );	
							}
						else
							{
								mOwnerDoc->PostMessage( CMacCVSMessageView::kMessageTypeFile, (char *)&pstrError[1], pstrError[0], inTargetItem, sizeof(FSSpec) );	
							}
					}
			}
	}


void
CMacCVSThreadedTask::PostOverallError( const SInt32 inError )
	{
		LStr255		pstrTemp;
		LStr255		pstrError("\pThe ");
		
		GetTaskName( pstrTemp );
		
		pstrError += pstrTemp + "\p could not be completed because ";		
		GetErrorString( inError, pstrTemp );	
		pstrError += pstrTemp;
		
		mOwnerDoc->PostMessage( CMacCVSMessageView::kMessageTypeError, (char *)&pstrError[1], pstrError[0], NULL, 0 );
	
		if ( inError == kCVSEngineBadPasswordErr )
			{
				mOwnerDoc->InvalidateCachedPassword();
			}
	}

	
void
CMacCVSThreadedTask::GetErrorString( const SInt32 inError, Str255 outErrorPStr )
	{
	
		LStr255	pstrError;
		
		ThrowIfNil_( outErrorPStr );
		
		outErrorPStr[0] = 0;
			
		if ( inError > 0 )
			{		
				switch ( inError )
					{			
						case kCVSEngineFileInvisibleErr:
							pstrError.Append( "\pthe file is invisible." );					
						break;
						
						case kCVSEnginePermissionsErr:
							pstrError.Append( "\pauto merging is turned off." );				
						break;

						case kCVSEngineBinaryMergingOffErr:	
							pstrError.Append( "\pauto merging for binary files is turned off." );					
						break;

						case kCVSEngineBadCkidChecksum:						
							pstrError.Append( "\pthe ckid resource checksum is wrong." );					
						break;
						
						case kCVSEngineFileHasBeenMoved:	
							pstrError.Append( "\pthe file has either been moved or renamed.");				
						break;
						
						case kCVSEngineFileHasNoCkid:
						case kCVSEngineFileHasNoMcvs:
						case kCVSEngineNoRevisionDataErr:						
							pstrError.Append( "\pthe file does not have any revision information.");						
						break;
						
						case kCVSEngineInvalidFileNameErr:				
							pstrError.Append( "\pthe item's name is not valid in CVS.");						
						break;

						case kCVSEngineServerGaveBadResponseErr:
							pstrError.Append( "\pthe CVS server sent back a bad response." );
						break;
						
						case kCVSEngineOutOfMemoryErr:
							pstrError.Append( "\pMacCVS is out of memory! Please increase the application partition size." );
						break;
											
						case kCVSEngineBadPasswordErr:
							pstrError.Append( "\pauthentication failed. Either your user name, password, or remote repository path is incorrect." );
						break;
						
						case kCVSEngineNetworkTimeoutErr:
							pstrError.Append( "\pa network time out occurred. (Check your networking preferences)" );
						break;
						
						case kCVSEngineServerReturnedErr:
							pstrError.Append( "\pthe CVS server returned an error. Check the warnings to see what went wrong." );
						break;
						
						case kCVSEngineOperationAbortedErr:
							pstrError.Append( "\pthe operation was cancelled." );
						break;

						case kCVSEngineFileNameTruncatedErr:
							pstrError.Append( "\pthe file name is too long!" );
						break;
						
						case kCVSEngineFileNotChangedErr:
							pstrError.Append( "\pthe file contents are not different from the copy in the repository." );
						break;
						
						default:
							
							if ( (inError != kCVSEngineOperationAbortedErr) && (inError != kCVSEngineSkippedFile) )
								{								
									pstrError.Append( "\pan error " );
									pstrError.Append( inError );
									pstrError.Append( "\p occurred." );								
								}
						
						break;
										
					}					
			}			
		else if ( inError < 0 )
			{		
				pstrError.Append( "\pan error " );
				pstrError.Append( inError );
				pstrError.Append( "\p occurred." );			
			}
			
		if ( pstrError[0] > 0 )
			{
				::BlockMoveData( pstrError, outErrorPStr, pstrError[0] + 1 );
			}
	}
	
	
void
CMacCVSThreadedTask::GetTaskName (Str255 outNamePStr)
	{
		LStr255	theNameStr;
		
		theNameStr = "\poperation";
			
		if ( theNameStr[0] > 0 )
			{
				::BlockMoveData( theNameStr, outNamePStr, theNameStr[0] + 1 );
			}
	}

