//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMacCVSThreadedWindow.cp
//	
//	  Authors: 	Simon Fraser
//
//################################################################################

#include "UKeyFilters.h"

#include "CThreadedTask.h"
#include "CSpinIndicator.h"
#include "CMacCVSDoc.h"
#include "CVSEngineUtils.h"

#include "CMacCVSThreadedWindow.h"


CMacCVSWindow::CMacCVSWindow(LStream *inStream) :
	
	LWindow( inStream),
	mOwnerDoc(NULL),
	mWindowPositionSaved(false),
	mWindowPosResID(0)
	
	{
	}


CMacCVSWindow::CMacCVSWindow(LStream *inStream, CMacCVSDoc *inOwnerDoc) :

		LWindow( inStream),
		mOwnerDoc(inOwnerDoc),
		mWindowPositionSaved(false),
		mWindowPosResID(0)
		
	{
			Assert_( mOwnerDoc != NULL );
	}

CMacCVSWindow::~CMacCVSWindow()
	{
	
		if ( ! mWindowPositionSaved )
			{
				SaveWindowPosition();
				mWindowPositionSaved = true;
			}
	}


void
CMacCVSWindow::FinishCreateSelf()
	{
	
		SetWindowPosition();
			
	}
	
	
void
CMacCVSWindow::SetOwnerDoc( const CMacCVSDoc *inOwnerDoc )
	{
		ThrowIfNil_( inOwnerDoc );
		
		mOwnerDoc = const_cast<CMacCVSDoc *>( inOwnerDoc );
		SetSuperModel( mOwnerDoc );
	}


void
CMacCVSWindow::SetWindowPosition()
	{
		
		if ( mOwnerDoc != NULL )
			{
			
				FSSpec			docFileSpec;
				
				mOwnerDoc->GetDocumentFileSpec(docFileSpec);
				
				// it's always safe to open and close resource files with fsRdPerm
				
				SInt16	intOldResRef = ::CurResFile();
				Rect		windowBounds = {0, 0, 0, 0};
				
				try
					{
						Int16		resFileRef = ::FSpOpenResFile(&docFileSpec, fsRdPerm);
						
						if (resFileRef == -1 || ::ResError() != noErr) return;
												
						ReadWindowPosResource( windowBounds );

						::CloseResFile( resFileRef );
						resFileRef = 0;
					}
				catch ( ... )
					{
						::UseResFile( intOldResRef );
						throw;
					}
					
				::UseResFile( intOldResRef );
				
				SafeSetWindowBounds( windowBounds );				
			}

	}


void
CMacCVSWindow::SaveWindowPosition()
	{
		
		OSErr			result				= noErr;
		Rect			windowBounds	= { 0, 0, 0, 0 };
		SInt16		intOldResRef	= 0;
		SInt16		intResRef			= 0;
		Boolean		bShouldClose	= false;
		
		intOldResRef = ::CurResFile();
		
		GetGlobalBounds( windowBounds );
		
		if ( mOwnerDoc != NULL )
			{
				LFile				*docFile;
				FSSpec			resFileSpec;
				
				try
					{
						docFile =	mOwnerDoc->GetFile();
						
						if ( docFile == NULL)
							return;		// give up
							
						// we have to be careful here, because the file may already be open.
						// I've no idea why LFile doesn't do the right thing here, so we have
						// to work extra hard, and avoid messing with LFile's mResForkRefNum
						
						docFile->GetSpecifier(resFileSpec);
						
						// recommended in TN 1120
						Handle			oldTopMap = ::LMGetTopMapHndl();
						
						intResRef = ::FSpOpenResFile( &resFileSpec, fsRdWrPerm );
						result = ::ResError();
						
						if ( intResRef != -1 && result == noErr )
							{
								bShouldClose = oldTopMap != ::LMGetTopMapHndl();			
							}
						else if ( result != noErr )
							{
								intResRef = 0;
								
								if ( result == eofErr )
									{
										result = FSpCreateTheDamnRsrcFork( &resFileSpec );
										intResRef = ::FSpOpenResFile( &resFileSpec, fsRdWrPerm );
										
										result = ::ResError();
										
										if ( result != noErr )
											{
												intResRef = 0;
											}
										else
											{
												//  Since we created the fork, we should
												//  close it since we're the only access path.
												
												bShouldClose = true;
											}
									}
							}
							
						if ( result == noErr )
							{								
								AddOrChangeWindowPosResource( windowBounds );
								
								if ( bShouldClose )
									{
										::CloseResFile( intResRef );
									}
							}
					}
				catch ( ... )
					{
						::UseResFile( intOldResRef );
						throw;
					}
					
				::UseResFile( intOldResRef );
			}
			
	}
	

void
CMacCVSWindow::AddOrChangeWindowPosResource( const Rect &inWindowBounds )
	{
		TWindowPos		**windowData = NULL;
		Boolean				newResource = false;
		
		Assert_(mWindowPosResID != 0);

		windowData = (TWindowPos **)::Get1Resource( kWindowPosResType, mWindowPosResID );
		
		if ( windowData == NULL )
			{
				windowData = (TWindowPos **)::NewHandleClear( sizeof(TWindowPos) );
				newResource = true;
			}
			
		if ( windowData != NULL )
			{
				(**windowData).mVersion = kCurrentWindowVersion;
				(**windowData).mWindowFrame = inWindowBounds;
				
				if ( newResource )
					AddResource( (Handle) windowData,kWindowPosResType, mWindowPosResID, "\p" );
				else
					ChangedResource( (Handle) windowData );
			}
	}


void
CMacCVSWindow::ReadWindowPosResource( Rect &outWindowBounds )
	{
		TWindowPos		**windowData = NULL;

		Assert_(mWindowPosResID != 0);
		
		windowData = (TWindowPos **)::Get1Resource( kWindowPosResType, mWindowPosResID );
		
		if ( windowData != NULL )
			{
				if ( (**windowData).mVersion == kCurrentWindowVersion )
					{						
						outWindowBounds = (**windowData).mWindowFrame;
					}
			
				::ReleaseResource( (Handle)windowData );
			}
	}


void
CMacCVSWindow::SafeSetWindowBounds( const Rect &inWindowBounds )
	{
			RgnHandle		desktopRgn = GetGrayRgn();
			Rect				desktopBounds = (**desktopRgn).rgnBBox;

				// make sure it's on a screen
			if ( SectRect( &inWindowBounds, &desktopBounds, &desktopBounds) )
				DoSetBounds( inWindowBounds );
	}


#pragma mark -


CMacCVSThreadedWindow::CMacCVSThreadedWindow(LStream *inStream):
		CMacCVSWindow( inStream),
		mWindowTask(NULL),
		mSpinnyThingy(NULL)
	{
	
	}
	
CMacCVSThreadedWindow::~CMacCVSThreadedWindow()
	{
			TerminateWindowTask();
	}
	

void 
CMacCVSThreadedWindow::FinishCreateSelf()
	{
	
			mSpinnyThingy = dynamic_cast<CSpinIndicator *>( FindPaneByID(CSpinIndicator::class_ID) );
			ThrowIfNil_( mSpinnyThingy );

			Inherited::FinishCreateSelf();			
	}

void
CMacCVSThreadedWindow::PrimeWindowTask( UInt32 inDelayTicks)
	{
	
			mWindowTask->AddListener( mSpinnyThingy );
			mWindowTask->DelayedResume(inDelayTicks);
	}

void
CMacCVSThreadedWindow::TerminateWindowTask()
	{
	
			if ( mWindowTask != NULL )
			{

				mWindowTask->BroadcastMessage( CSpinIndicator::msg_SpinNoMoreWaiters );

				mWindowTask->RemoveListener( mSpinnyThingy );
				mWindowTask->ShutdownThread();
			
				while ( mWindowTask != NULL )
					CThreadedTask::Yield();
					
			}
	
	}
	
	
Boolean
CMacCVSThreadedWindow::HandleKeyPress( const EventRecord	&inKeyEvent )
{
	
	if ( UKeyFilters::IsCmdPeriod( inKeyEvent ) )
		{

			TerminateWindowTask();
						
			return true;
		}
		
	return Inherited::HandleKeyPress( inKeyEvent );

}


void
CMacCVSThreadedWindow::ThreadedTaskDied( const CThreadedTask	*inTask )
	{

		Assert_( (inTask == mWindowTask) && (inTask != NULL) );
		
		if ( inTask != NULL )
			if ( inTask == mWindowTask )
				mWindowTask = NULL;
		
		
		LCommander::SetUpdateCommandStatus(true);
		
	}

