//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSWatchTask.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains code to run CVS Watch operations.
		 
	Modification History:

		12 May 	1999	Ram Rajadhyaksha
		
			First Created.
							
-------------------------------------------------------------------------------- */


#include "ThrowOSErr.h"
#include "CMacCVSDoc.h"
#include "CMultiProgressWindow.h"

#include "CVSEngine.h"
#include "CVSEngineUtils.h"

#include "CMacCVSWatchTask.h"


CMacCVSWatchTask::CMacCVSWatchTask( LArray *inItems, CMacCVSDoc *inOwnerDoc ) : CMacCVSOperationTask( inOwnerDoc )		
	{
		mEngineRef							= NULL;
		mProgressID							= 0;	
		mOldMaxValue 						= 0;
		mIterations 						= 0;
		mReset									= false;

		if ( inItems != NULL )
			{		
				mItemList = *inItems;		//  this makes a copy of the array
			}
		else
			{
				mItemList = NULL;
			}			
	}
		

CMacCVSWatchTask::~CMacCVSWatchTask()
	{
		try
			{
				if ( mEngineRef != NULL )
					{			
						DisposeCVSEngine( mEngineRef );			
						mEngineRef = NULL;				
					}
							
				if ( mProgressID != 0 )
					{
						CMultiProgressWindow *pProgressWindow = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWindow != NULL );
						
						if ( pProgressWindow != NULL )
							{
								pProgressWindow->RemoveListener( this );
							}
							
						CMultiProgressWindow::DeleteGlobalProgressIndicator( mProgressID );
					}
			}
		catch ( ... )
			{
			}
	}
	

#pragma mark -

	
void
CMacCVSWatchTask::Run()
	{
	
		OSErr		result				= noErr;
		FSSpec	fsProjectRoot	= { 0, 0, "\p" };
		
		try
			{
				mProgressID = CMultiProgressWindow::NewGlobalProgressIndicator();
				Assert_( mProgressID != 0 );
				
				CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
				ThrowIfNil_( pProgressWin );
				
				pProgressWin->AddListener( this );
				
				LStr255	pstrItemTitle;
				pstrItemTitle = "\pWatch: ";
							
				LStr255	pstrDocName;
				mOwnerDoc->GetDescriptor( pstrDocName );
				pstrItemTitle += pstrDocName;
		
				pProgressWin->SetProgressMessageText( mProgressID, pstrItemTitle );
	
				CCVSEngineLoginOptions sLoginOptions;
				sLoginOptions.Init();
				
				mOwnerDoc->GetHostNamePStr( sLoginOptions.mServerName );
				sLoginOptions.mPort = mOwnerDoc->GetHostPort();
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mRemoteUserName );
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mLocalUserName );
				mOwnerDoc->GetPasswordPStr( sLoginOptions.mPassword );
				mOwnerDoc->GetCVSCommandPStr( sLoginOptions.mCVSCommandText );
				mOwnerDoc->GetCVSRootPStr( sLoginOptions.mCVSRoot );
				sLoginOptions.mAuthenticationMethod = mOwnerDoc->GetAuthenticationMethod();
				
				mOwnerDoc->GetProjectRootFileSpec( fsProjectRoot );
				
				LArrayIterator itemIterator( mItemList );
				CMacCVSWatchTaskItem sCurItem;

				//  Because the CVS server is lame, we drop the connection after
				//  each update to insure that everything works ok.
								
				while ( itemIterator.Next(&sCurItem) )
					{		
						mIterations = 0;

						pProgressWin->SetCurrentItemText( mProgressID, "\pConnecting to server");
						pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );

						result = CreateCVSEngine( &fsProjectRoot, mMappings, mMappingsCount, Notifier, MessageOutNotifier, ErrorOutNotifier, this, mOwnerDoc->GetSocketTimeOut(), &mEngineRef );
						ThrowOSErr( result );
					
						result = CVSEngineLogin( mEngineRef, &sLoginOptions );
						ThrowOSErr( result );

						pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarAbsolute );

						pProgressWin->SetCurrentItemText( mProgressID, "\pSending Entries" );					
						pProgressWin->SetProgressCountText( mProgressID, mIterations );					
						pProgressWin->SetProgressType( mProgressID, CMultiProgressWindow::kProgressBarAbsolute );	
					
						switch ( sCurItem.mOperation )
							{
								case kWatchAdd:
									result = CVSEngineWatchOnOff( mEngineRef, &sCurItem.mFileSpec, true );
									ThrowOSErr( result );						
								break;
								
								case kWatchRemove:
									result = CVSEngineWatchOnOff( mEngineRef, &sCurItem.mFileSpec, false );
									ThrowOSErr( result );														
								break;
								
								case kWatchNotify:
									result = CVSEngineWatchAddRemove( mEngineRef, &sCurItem.mFileSpec, &sCurItem.mOptions );
									ThrowOSErr( result );																						
								break;
							}							
							
						DisposeCVSEngine( mEngineRef );
						mEngineRef = NULL;
					}						
			}			
		catch ( ... )
			{
				if ( result == noErr )
					{
						result = kCVSEngineUnknownErr;
					}
					
				PostOverallError( result );
			}

	}
	
		
pascal SInt32		
CMacCVSWatchTask::Notifier(				const CCVSEngineRef					inEngineRef,
																const FSSpec								*inTargetItem,
																const SInt32 								inEvent, 
																const SInt32 								inError, 
																const UInt32 								inCookie, 
																const void 									*inDataPtr, 
																void 												*inContextPtr)
	{
	
		const	unsigned char	pstrUpdating[] 			= "\pUpdating: ";
		const	unsigned char	pstrSendingEntry[] 	= "\pSending: ";
		const	unsigned char	pstrDecoding[] 			= "\pDecoding: ";
		
		OSErr								error								= noErr;
		Boolean							bResult							= true;
		UInt32							intCopyLen 					= 0;
		Str255							pstrItem						= "\p";
		FSSpec							fsTarget						= { 0, 0, "\p" };
		
		CMacCVSWatchTask			*theTask 						= (CMacCVSWatchTask *)inContextPtr;
				
		if ( theTask != NULL )
			{
				try
					{
						CMultiProgressWindow *pProgressWin = CMultiProgressWindow::GetGlobalCMultiProgressWindow();
						Assert_( pProgressWin != NULL );

						if ( inTargetItem != NULL )
							{
								fsTarget = *inTargetItem;
							}
								
						switch ( inEvent )
							{				
								case kCVSEngineEntryPreActionEvent:
									{
										if ( inError != noErr )
											theTask->PostFileError( inTargetItem, (char *)inDataPtr, inCookie, inError );

										pProgressWin->SetCurValue( theTask->mProgressID, 0 );

										::BlockMoveData( &pstrSendingEntry[1], &pstrItem[1], pstrSendingEntry[0] );
										pstrItem[0] = pstrSendingEntry[0];
										
										Assert_( (inDataPtr != NULL) && (inCookie > 0) );
										
										if ( (inDataPtr != NULL) && (inCookie > 0) )
											{																							
												intCopyLen = inCookie;
												
												if ( intCopyLen > (255 - pstrItem[0]) )
													{
														intCopyLen = (255 - pstrItem[0]);
													}
													
												::BlockMoveData( inDataPtr, &pstrItem[pstrItem[0] + 1], intCopyLen );
												pstrItem[0] += intCopyLen;
												
												pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );											
											}
										else 
											{
												pProgressWin->SetCurrentItemText( theTask->mProgressID, pstrItem );
											}
											
										theTask->mIterations++;
										pProgressWin->SetProgressCountText( theTask->mProgressID, theTask->mIterations );											
									}									
								break;
																
								case kCVSEngineCommandActionEvent:
								
									theTask->mIterations = 0;
									pProgressWin->SetCurValue( theTask->mProgressID, 0 );
									pProgressWin->SetCurrentItemText( theTask->mProgressID, "\pWaiting for response" );
									pProgressWin->SetProgressType( theTask->mProgressID, CMultiProgressWindow::kProgressBarIndeterminate );			
									theTask->mReset = true;
									
								break;
																
								case kCVSEngineProgressEvent:

									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );
									
									if ( inDataPtr != NULL )
										{								
											if ( theTask->mOldMaxValue != ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax )
												{										
													theTask->mOldMaxValue = ((CCVSEngineProgressStatusRec *)inDataPtr)->fMax;
													pProgressWin->SetMaxValue( theTask->mProgressID, theTask->mOldMaxValue );								
												}
												
											pProgressWin->SetCurValue( theTask->mProgressID, ((CCVSEngineProgressStatusRec *)inDataPtr)->fCompleted);						
										}
													
								break;
								
								case kCVSEngineYieldActionEvent:
									
									CThreadedTask::Yield();
								
								break;
								
								default:
								
									if ( inError != noErr )
										theTask->PostFileError( inTargetItem, NULL, 0, inError );
								
								break;
							
							}
							
					}				
				catch ( ... )
					{
						//  If an exception was thrown during the yield,
						//  we need to abort.

						if ( inEvent == kCVSEngineYieldActionEvent )
							{
								bResult = 0;
							}
					}					
			}
							
		return bResult;
		
	}
	
	
void		
CMacCVSWatchTask::ListenToMessage( MessageT inMessage, void *ioParam )
	{		
		if ( ioParam != NULL )
			{
				UInt32 intID = *(UInt32 *)ioParam;
				
				if ( (intID == mProgressID) && (inMessage == msg_Cancel) )
					{
						ShutdownThread();
					}
			}
	}
	

void
CMacCVSWatchTask::GetTaskName( Str255 outNamePStr )
	{
		LStr255		theNameStr;
		
		theNameStr = "\pwatch";
			
		if ( theNameStr[0] > 0 )
			::BlockMoveData( theNameStr, outNamePStr, theNameStr[0] + 1 );
	}
