//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CMacCVSWindowTask.cp
	
	Authors:		S. Fraser and R. Rajadhyaksha
	
	Description:
	
		This file contains a base object that associates a CMacCVSOperationTask
		with a CMacCVSThreadedWindow. It ensures that if the thread dies before
		the window, the window is properly notified. Conversely, if the window
		is closed before the thread has completed, it is properly aborted.
		 
	Modification History:
							
-------------------------------------------------------------------------------- */


#include <stdio.h>
#include <string.h>

#include "ThrowOSErr.h"
#include "CMacCVSDoc.h"

#include "CMacCVSWindowTask.h"



CMacCVSWindowTask::CMacCVSWindowTask(	const CMacCVSThreadedWindow *inOwnerWindow, CMacCVSDoc *inOwnerDoc ):
	CMacCVSOperationTask( inOwnerDoc )	
	{
		mEngineRef = NULL;
		mEventNotifier = NULL;
				
		Assert_( inOwnerWindow != NULL );
		mOwnerWindow = const_cast<CMacCVSThreadedWindow *>(inOwnerWindow);
	}
										
																																					
CMacCVSWindowTask::~CMacCVSWindowTask()
	{	
		if ( mEngineRef != NULL )
			{			
				DisposeCVSEngine( mEngineRef );
				mEngineRef = NULL;	
			}
		
		mOwnerWindow->ThreadedTaskDied( this );		
	}
	
	
#pragma mark -


void
CMacCVSWindowTask::SetOwnerWindow( CMacCVSThreadedWindow *inOwnerWindow )
	{
		ThrowIfNil_(inOwnerWindow);
		mOwnerWindow = inOwnerWindow;
	}


void
CMacCVSWindowTask::SetEventNotifier( CCVSEngineEventNotifierProcPtr	inEventNotifier )
	{
		ThrowIfNil_( inEventNotifier );
		mEventNotifier = inEventNotifier;
	}
	
	
void
CMacCVSWindowTask::RunSelf( )
	{
		
		OSErr		result				= noErr;
		FSSpec	projectRoot		= { 0, 0, "\p" };

		try
			{
				CCVSEngineLoginOptions sLoginOptions;
				sLoginOptions.Init();
				
				mOwnerDoc->GetHostNamePStr( sLoginOptions.mServerName );
				sLoginOptions.mPort = mOwnerDoc->GetHostPort();
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mRemoteUserName );
				mOwnerDoc->GetRemoteUserPStr( sLoginOptions.mLocalUserName );
				mOwnerDoc->GetPasswordPStr( sLoginOptions.mPassword );
				mOwnerDoc->GetCVSCommandPStr( sLoginOptions.mCVSCommandText );
				mOwnerDoc->GetCVSRootPStr( sLoginOptions.mCVSRoot );
				sLoginOptions.mAuthenticationMethod = mOwnerDoc->GetAuthenticationMethod();
				
				mOwnerDoc->GetProjectRootFileSpec( projectRoot );
				
				ThrowIfNil_( mEventNotifier );
				
				result = CreateCVSEngine( &projectRoot, mMappings, mMappingsCount, mEventNotifier, MessageOutNotifier, ErrorOutNotifier, this, mOwnerDoc->GetSocketTimeOut(), &mEngineRef );
				ThrowOSErr( result );
				
				result = CVSEngineLogin( mEngineRef, &sLoginOptions );
				ThrowOSErr( result );
				
				result = DoEngineCommand();
				ThrowOSErr( result );
			}							
		catch ( ... )
			{
				if ( result == noErr )
					result = kCVSEngineUnknownErr;
					
				PostOverallError( result );
			}
	
	}

