//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CMultiProgressWindow.cp
//
//	  Author: 	Ram Rajadhyaksha
//
//################################################################################


#include <string.h>
#include <TextUtils.h>
#include <LProgressBar.h>
#include <LCaption.h>
#include <LPushButton.h>

#include "ThrowOSErr.h"
#include "StringUtils.h"
#include "CProgressIndicator.h"

#include "CMultiProgressWindow.h"


CMultiProgressWindow *CMultiProgressWindow::sGlobalItem = NULL;


CMultiProgressWindow::CMultiProgressWindow( LStream *inStream ) : 

	LWindow(inStream)
	
	{
		OSErr	intResult = noErr;
		Rect	sItemRect = { 0, 0, 0, 0 };
		
		mNewID 							= 1;
		mItemCount 					= 0;
	//	mWindowPosResID			= kProgressWindowPositionResID;
		mItemsH							= NULL;
		mMessagePixels 			= NULL;
		mItemCountPixels 		= NULL;
		mItemTextPixels 		= NULL;
		
		try
			{
				sItemRect.right = kMessageItemWidth;
				sItemRect.bottom = kMessageItemHeight;
				
				intResult = ::NewGWorld( &mMessagePixels, 0, &sItemRect, NULL, NULL, 0 );
				ThrowOSErr( intResult );

				sItemRect.right = kItemCountWidth;
				sItemRect.bottom = kItemCountHeight;
				
				intResult = ::NewGWorld( &mItemCountPixels, 0, &sItemRect, NULL, NULL, 0 );
				ThrowOSErr( intResult );

				sItemRect.right = kItemTextWidth;
				sItemRect.bottom = kItemTextHeight;
				
				intResult = ::NewGWorld( &mItemTextPixels, 0, &sItemRect, NULL, NULL, 0 );
				ThrowOSErr( intResult );
				
				mItemsH = (SProgressItemRec **)::NewHandle( 0 );
				ThrowIfNil_( mItemsH );
			}
		catch ( ... ) 
			{
				if ( mMessagePixels != NULL )
					{
						::DisposeGWorld( mMessagePixels );
						mMessagePixels = NULL;
					}

				if ( mItemCountPixels != NULL )
					{
						::DisposeGWorld( mItemCountPixels );
						mItemCountPixels = NULL;
					}

				if ( mItemTextPixels != NULL )
					{
						::DisposeGWorld( mItemTextPixels );
						mItemTextPixels = NULL;
					}
					
				if ( mItemsH != NULL )
					{
						::DisposeHandle( (Handle)mItemsH );
						mItemsH = NULL;
					}
					
				throw;
			}
	}


CMultiProgressWindow::~CMultiProgressWindow()
	{	
		if ( mMessagePixels != NULL )
			{
				::DisposeGWorld( mMessagePixels );
				mMessagePixels = NULL;
			}

		if ( mItemCountPixels != NULL )
			{
				::DisposeGWorld( mItemCountPixels );
				mItemCountPixels = NULL;
			}

		if ( mItemTextPixels != NULL )
			{
				::DisposeGWorld( mItemTextPixels );
				mItemTextPixels = NULL;
			}

		if ( mItemsH != NULL )
			{
				::DisposeHandle( (Handle)mItemsH );
				mItemsH = NULL;			
			}

		//  Since the global LApplication object is most likely
		//  to be the supercommander of sGlobalItem, if it gets deleted,
		//  we're gonna go with it. So we better put this in so
		//  we don't double delete.
			
		if ( this == sGlobalItem )
			{
				sGlobalItem = NULL;
			}
	}


CMultiProgressWindow *
CMultiProgressWindow::CreateCMultiProgressWindowFromStream( LStream *inStream )
	{	
		return new CMultiProgressWindow( inStream );	
	}


#pragma mark -


UInt32
CMultiProgressWindow::NewProgressIndicator()
	{		
		UInt32						intIndex 						= 0;
		UInt32						intOffsetPixels			= 0;
		UInt32						intDummy						= 0;
		SInt32						intCurSize					= 0;
		
		LProgressBar			*pProgressBar 			= NULL;
		LPushButton				*pStopButton				= NULL;
		
		PaneIDT						intPaneID						= 0;
		
		SPaneInfo					sPane;
		SProgressItemRec  sItem;
		SProgressItemRec  sDummy;
		
		memset( &sPane, 0, sizeof(sPane) );
		memset( &sItem, 0, sizeof(sItem) );
			
		try
			{
				if ( mItemCount >= kMaxProgressItems )
					ThrowOSErr( -1 );  //  sounds like a good thing to throw :-)
					
				sItem.fID = mNewID++;
	
				if ( mNewID >= kMaxProgressItems )
					{
						mNewID = 1;
					}
			
				//  Make sure we get a unique ID
				
				while ( GetPIDPane(sItem.fID, intDummy, sDummy) == true )
					{
						sItem.fID = mNewID++;
						
						if ( mNewID >= kMaxProgressItems )
							{
								mNewID = 1;
							}
					}
					
				Assert_( mNewID <= kMaxProgressItems );
				Assert_( sItem.fID <= kMaxProgressItems );
				
				intIndex = mItemCount;
				
				::SetHandleSize( (Handle)mItemsH, (mItemCount + 1) * sizeof(SProgressItemRec) );
				ThrowOSErr( ::MemError() );
							
				intOffsetPixels = (kStatusItemHeight * mItemCount);
										
				sPane.paneID 					= (100 * intIndex + kProgressBarSubID);
				sPane.width 					= kItemProgressWidth;
				sPane.height 					= kItemProgressHeight;
				sPane.visible 				= true;
				sPane.enabled 				= true;
				sPane.bindings.left 	= true;
				sPane.bindings.top 		= true;
				sPane.bindings.right 	= false;
				sPane.bindings.bottom = false;
				sPane.left 						= kItemProgressLeft;
				sPane.top 						= kItemProgressTop + intOffsetPixels;
				sPane.userCon 				= 0;
				sPane.superView 			= this;
				
				pProgressBar = new LProgressBar( sPane, 0, 0, 0, 0 );
				ThrowIfNil_( pProgressBar );

				sPane.paneID 					= (100 * intIndex + kStopButtonSubID);
				sPane.width 					= kStopButtonWidth;
				sPane.height 					= kStopButtonHeight;
				sPane.visible 				= true;
				sPane.enabled 				= true;
				sPane.bindings.left 	= true;
				sPane.bindings.top 		= true;
				sPane.bindings.right 	= false;
				sPane.bindings.bottom = false;
				sPane.left 						= kStopButtonLeft;
				sPane.top 						= kStopButtonTop + intOffsetPixels;
				sPane.userCon 				= 0;
				sPane.superView 			= this;
				
				pStopButton = new LPushButton( sPane, sItem.fID, "\pStop" );
				ThrowIfNil_( pStopButton );
	
				pStopButton->PutInside( this );
				pProgressBar->PutInside( this );
				
				pStopButton->AddListener( this );
				
				ResizeWindowTo( kStatusItemWidth, (intIndex + 1) * kStatusItemHeight );
				mItemCount++;
				
				::BlockMoveData( &sItem, &((*mItemsH)[intIndex]), sizeof(sItem) );
			}
		catch ( ... )
			{
				if ( pStopButton != NULL )
					{
						delete pStopButton;
						pStopButton = NULL;
					}
					
				if ( pProgressBar != NULL )
					{
						delete pProgressBar;
						pProgressBar = NULL;
					}
					
				throw;
			}
			
		return sItem.fID;
	}
	
	
void
CMultiProgressWindow::DeleteProgressIndicator( const UInt32 intInID )
	{			
		UInt32						intIndex = 0;
		SProgressItemRec	sItem;
		
		Boolean	bValid = GetPIDPane( intInID, intIndex, sItem );
		
		if ( bValid )
			{	
				MigrateIndicators( intIndex );
				
				if ( mItemCount > 0 )
					{
						LProgressBar *pProgressBar = dynamic_cast<LProgressBar *>( FindPaneByID((mItemCount - 1) * 100 + kProgressBarSubID) );
						LPushButton *pStopButton = dynamic_cast<LPushButton *>( FindPaneByID((mItemCount - 1) * 100 + kStopButtonSubID) );
						
						Assert_( pProgressBar != NULL );
						Assert_( pStopButton != NULL );
						
						if ( pProgressBar != NULL )
							{
								delete pProgressBar;
							}
							
						if ( pStopButton != NULL )
							{
								pStopButton->RemoveListener( this );
								delete pStopButton;
							}
					}
					
				if ( (intIndex + 1) < mItemCount )
					{
						::BlockMoveData( &((*mItemsH)[intIndex + 1]), &((*mItemsH)[intIndex]), (mItemCount - (intIndex + 1)) * sizeof(SProgressItemRec) );
					}
			
				mItemCount--;
				::SetHandleSize( (Handle)mItemsH, mItemCount * sizeof(SProgressItemRec) );
					
				if ( mItemCount > 0 )
					{
						ResizeWindowTo( kStatusItemWidth, kStatusItemHeight * mItemCount );	
					}
					
				//  Force a redraw.
				
				Rect sWinRect = { 0 };
				CalcLocalFrameRect( sWinRect );
				FocusDraw();
				ApplyForeAndBackColors();
				::EraseRect( &sWinRect );	
				Draw( NULL );
			}
	}
	
	
#pragma mark -


UInt32
CMultiProgressWindow::NewGlobalProgressIndicator()
	{
		UInt32	intResult = 0;
		
		if ( sGlobalItem == NULL )
			{
				sGlobalItem = dynamic_cast<CMultiProgressWindow *>(LWindow::CreateWindow(CMultiProgressWindow::kCMultiProgressWindowPPobID, LCommander::GetTopCommander()));
				ThrowIfNil_( sGlobalItem );
			}
	
		if ( sGlobalItem != NULL )
			{
				intResult = sGlobalItem->NewProgressIndicator();		

				if ( sGlobalItem->GetProgressItemCount() == 1 )
					{
						sGlobalItem->Show();
						UDesktop::SelectDeskWindow( sGlobalItem );
					}	
			}
			
		return intResult;
	}
	
	
CMultiProgressWindow *
CMultiProgressWindow::GetGlobalCMultiProgressWindow()
	{
		if ( sGlobalItem == NULL )
			{
				sGlobalItem = dynamic_cast<CMultiProgressWindow *>(LWindow::CreateWindow(CMultiProgressWindow::kCMultiProgressWindowPPobID, LCommander::GetTopCommander()));
				ThrowIfNil_( sGlobalItem );
			}
		
		return sGlobalItem;		
	}
	
	
void
CMultiProgressWindow::DeleteGlobalProgressIndicator( const UInt32 intInID )
	{
		Assert_( sGlobalItem != NULL );
		
		if ( sGlobalItem != NULL )
			{
				sGlobalItem->DeleteProgressIndicator( intInID );	
				
				if ( sGlobalItem->GetProgressItemCount() == 0 )
					{
						sGlobalItem->Hide();
					}	
			}	
	}
	

void
CMultiProgressWindow::DisposeGlobalCMultiProgressWindow()
	{		
		if ( sGlobalItem != NULL )
			{
				delete sGlobalItem;
			}
	}

	
#pragma mark -


void						
CMultiProgressWindow::SetProgressType( const UInt32 intInID, const UInt32 intInType )
	{
		SProgressItemRec	sItem;
		UInt32						intIndex 			= 0;
		Boolean 					bValid 				= GetPIDPane( intInID, intIndex, sItem );
		LProgressBar 			*pProgressBar	= NULL;
		
		if ( bValid )
			{
				pProgressBar = dynamic_cast<LProgressBar *>( FindPaneByID(100 * intIndex + kProgressBarSubID) );
				Assert_( pProgressBar != NULL );
				
				if ( pProgressBar != NULL )
					{
						switch ( intInType )
							{
								case CMultiProgressWindow::kProgressBarAbsolute:
									pProgressBar->SetIndeterminateFlag( false, false );
								break;
								
								case CMultiProgressWindow::kProgressBarIndeterminate:
									pProgressBar->SetIndeterminateFlag( true, true );
								break;
								
								case CMultiProgressWindow::kProgressBarTimeBased:
									
									//  This doesn't work yet
									
									pProgressBar->SetIndeterminateFlag( false, false );
								break;
								
								default:
									Assert_( false );
								break;								
							}
					}
			}			
	}
																																																													

void
CMultiProgressWindow::SetMaxValue( const UInt32 intInID, const UInt32 intInMax )
	{
		SProgressItemRec	sItem;
		UInt32						intIndex 			= 0;
		Boolean 					bValid 				= GetPIDPane( intInID, intIndex, sItem );
		LProgressBar 			*pProgressBar	= NULL;
		
		if ( bValid )
			{
				pProgressBar = dynamic_cast<LProgressBar *>( FindPaneByID(100 * intIndex + kProgressBarSubID) );
				Assert_( pProgressBar != NULL );
				
				if ( pProgressBar != NULL )
					{
						pProgressBar->SetMaxValue( intInMax );
					}
			}	
	}
																																
			
void
CMultiProgressWindow::SetCurValue( const UInt32 intInID, const UInt32 intInMax )
	{
		SProgressItemRec	sItem;
		UInt32						intIndex 			= 0;
		Boolean 					bValid 				= GetPIDPane( intInID, intIndex, sItem );
		LProgressBar 			*pProgressBar	= NULL;
		
		if ( bValid )
			{
				pProgressBar = dynamic_cast<LProgressBar *>( FindPaneByID(100 * intIndex + kProgressBarSubID) );
				Assert_( pProgressBar != NULL );
				
				if ( pProgressBar != NULL )
					{
						pProgressBar->SetValue( intInMax );
					}
			}
	}
			
																															
void
CMultiProgressWindow::MakeTimeBasedProgress( const UInt32 intInID, const UInt32 inExpectedTime )
	{
	
	
	}


void
CMultiProgressWindow::SetProgressMessageText( const UInt32 intInID, ConstStringPtr pstrInText, const Boolean bInRefresh )
	{
		UInt32						intIndex 					= 0;
		Boolean 					bValid 						= false;
		SProgressItemRec	sItem;
	
		bValid = GetPIDPane( intInID, intIndex, sItem );
		
		if ( bValid && (pstrInText != NULL) )
			{	
				StHandleLocker( (Handle)mItemsH );
				
				PStrCopy( pstrInText, ((*mItemsH)[intIndex]).fStrMessage, sizeof(Str255) );
				
				if ( bInRefresh )
					{
						FocusDraw();
						DrawItem( intIndex, 0 );
					}
			}	
	}
	

void
CMultiProgressWindow::SetCurrentItemText( const UInt32 intInID, ConstStringPtr pstrInText, const	Boolean bInRefresh )
	{
		UInt32						intIndex 					= 0;
		Boolean 					bValid 						= false;
		SProgressItemRec	sItem;
	
		bValid = GetPIDPane( intInID, intIndex, sItem );
		
		if ( bValid && (pstrInText != NULL) )
			{
				StHandleLocker( (Handle)mItemsH );
				
				PStrCopy( pstrInText, ((*mItemsH)[intIndex]).fStrItemText, sizeof(Str255) );
				
				if ( bInRefresh )
					{
						FocusDraw();
						DrawItem( intIndex, 0 );
					}
			}	
	}
	

void
CMultiProgressWindow::SetProgressCountText( const UInt32 intInID, UInt32 intInValue, const Boolean bInRefresh )
	{
		UInt32						intIndex 					= 0;
		Boolean 					bValid 						= false;		
		Str255						pstrText					= "\p";
		SProgressItemRec	sItem;
		
		bValid = GetPIDPane( intInID, intIndex, sItem );
		
		if ( bValid )
			{
				StHandleLocker( (Handle)mItemsH );
				
				if ( intInValue > 0 )
					{
						UInt32ToStr255( intInValue, pstrText );
						PStrCopy( pstrText, ((*mItemsH)[intIndex]).fStrItemCount, sizeof(Str255) );
					}
				else
					{
						((*mItemsH)[intIndex]).fStrItemCount[0] = 0;
					}
					
				if ( bInRefresh )
					{
						FocusDraw();
						DrawItem( intIndex, 0 );
					}
			}			
	}
																																		

#pragma mark -


void
CMultiProgressWindow::DrawSelf()
	{
		RGBColor	rgbGray 	= { 0x7FFF, 0x7FFF, 0x7FFF };
		
		for ( UInt32 intIndex = 0; intIndex < mItemCount; intIndex++ )
			{
				DrawItem( intIndex, 0 );
			}
			
		if ( mItemCount >= 2 )
			{							
				for ( UInt32 intIndex = 1; intIndex < mItemCount; intIndex++ )
					{
						::RGBForeColor( &rgbGray );
						::MoveTo( 5, kStatusItemHeight * intIndex );
						::LineTo( kStatusItemWidth - 5, kStatusItemHeight * intIndex );

						::ForeColor( whiteColor );
						::MoveTo( 5, (kStatusItemHeight * intIndex) + 1 );
						::LineTo( kStatusItemWidth - 5, (kStatusItemHeight * intIndex) + 1 );
					}
			}
	}


void
CMultiProgressWindow::DrawItem( const UInt32 intInIndex, const UInt32 intInMode )
	{	
		DrawProgressMessageText( intInIndex );
		DrawProgressCountText( intInIndex );
		DrawCurrentItemText( intInIndex );
	}


void
CMultiProgressWindow::DrawProgressMessageText( const UInt32 intInIndex )
	{
		CGrafPtr					pOldPort 		= NULL;
		GDHandle					pOldGDH			= NULL;
		Rect							rOffscreen	= { 0, 0, 0, 0 };
		Rect							rOnscreen		= { 0, 0, 0, 0 };
		PixMapHandle			hPixels			= NULL;
		SProgressItemRec	sItem;
		
		Assert_( intInIndex < mItemCount );		
		
		if ( intInIndex < mItemCount )
			{
			::GetGWorld( &pOldPort, &pOldGDH );

				sItem = (*mItemsH)[intInIndex];
			
				::SetGWorld( mMessagePixels, NULL );
	
				hPixels = ::GetGWorldPixMap( mMessagePixels );
				::LockPixels( hPixels );
	
				::TextFont( kFontIDGeneva );
				::TextFace( bold );
				::TextSize( 9 );
		
				::RGBBackColor(&mBackColor);
				
				rOffscreen.right = kMessageItemWidth;
				rOffscreen.bottom = kMessageItemHeight;
				
				rOnscreen.left = kMessageItemLeft;
				rOnscreen.right = rOnscreen.left + kMessageItemWidth;
				rOnscreen.top =	kMessageItemTop + (kStatusItemHeight * intInIndex);
				rOnscreen.bottom = rOnscreen.top + kMessageItemHeight;
				
				::EraseRect( &rOffscreen );
				::MoveTo( 0, 11 );
				::DrawString( sItem.fStrMessage );

				::SetGWorld( pOldPort, pOldGDH );

				::ForeColor( blackColor );
				::BackColor( whiteColor );

				::CopyBits(		&(((GrafPtr)mMessagePixels)->portBits), 
											&(((GrafPtr)pOldPort)->portBits), 
											&rOffscreen, 
											&rOnscreen, 
											srcCopy,
											NULL);
				
				::UnlockPixels( hPixels );				
			}	
	}
	
	
void
CMultiProgressWindow::DrawCurrentItemText( const UInt32 intInIndex )
	{
		CGrafPtr					pOldPort 		= NULL;
		GDHandle					pOldGDH			= NULL;
		Rect							rOffscreen	= { 0, 0, 0, 0 };
		Rect							rOnscreen		= { 0, 0, 0, 0 };
		PixMapHandle			hPixels			= NULL;
		SProgressItemRec	sItem;
		
		Assert_( intInIndex < mItemCount );		
		
		if ( intInIndex < mItemCount )
			{
			::GetGWorld( &pOldPort, &pOldGDH );

				sItem = (*mItemsH)[intInIndex];
			
				::SetGWorld( mItemTextPixels, NULL );
	
				hPixels = ::GetGWorldPixMap( mItemTextPixels );
				::LockPixels( hPixels );
	
				::TextFont( kFontIDGeneva );
				::TextFace( normal );
				::TextSize( 9 );
		
				::RGBBackColor(&mBackColor);
				
				rOffscreen.right = kItemTextWidth;
				rOffscreen.bottom = kItemTextHeight;
				
				rOnscreen.left = kItemTextLeft;
				rOnscreen.right = rOnscreen.left + kItemTextWidth;
				rOnscreen.top =	kItemTextTop + (kStatusItemHeight * intInIndex);
				rOnscreen.bottom = rOnscreen.top + kItemTextHeight;
				
				::EraseRect( &rOffscreen );
				::MoveTo( 0, 11 );
				::DrawString( sItem.fStrItemText );

				::SetGWorld( pOldPort, pOldGDH );

				::ForeColor( blackColor );
				::BackColor( whiteColor );

				::CopyBits(		&(((GrafPtr)mItemTextPixels)->portBits), 
											&(((GrafPtr)pOldPort)->portBits), 
											&rOffscreen, 
											&rOnscreen, 
											srcCopy,
											NULL);
				
				::UnlockPixels( hPixels );				
			}		
	}
	

void
CMultiProgressWindow::DrawProgressCountText( const UInt32 intInIndex )
	{
		CGrafPtr					pOldPort 		= NULL;
		GDHandle					pOldGDH			= NULL;
		Rect							rOffscreen	= { 0, 0, 0, 0 };
		Rect							rOnscreen		= { 0, 0, 0, 0 };
		PixMapHandle			hPixels			= NULL;
		SProgressItemRec	sItem;
		
		Assert_( intInIndex < mItemCount );		
		
		if ( intInIndex < mItemCount )
			{
			::GetGWorld( &pOldPort, &pOldGDH );

				sItem = (*mItemsH)[intInIndex];
			
				::SetGWorld( mItemCountPixels, NULL );
	
				hPixels = ::GetGWorldPixMap( mItemCountPixels );
				::LockPixels( hPixels );
	
				::TextFont( kFontIDGeneva );
				::TextFace( bold );
				::TextSize( 9 );
		
				::RGBBackColor(&mBackColor);
				
				rOffscreen.right = kItemCountWidth;
				rOffscreen.bottom = kItemCountHeight;
				
				rOnscreen.left = kItemCountLeft;
				rOnscreen.right = rOnscreen.left + kItemCountWidth;
				rOnscreen.top =	kItemCountTop + (kStatusItemHeight * intInIndex);
				rOnscreen.bottom = rOnscreen.top + kItemCountHeight;
				
				::EraseRect( &rOffscreen );
				::MoveTo( 0, 11 );
				::DrawString( sItem.fStrItemCount );

				::SetGWorld( pOldPort, pOldGDH );

				::ForeColor( blackColor );
				::BackColor( whiteColor );

				::CopyBits(		&(((GrafPtr)mItemCountPixels)->portBits), 
											&(((GrafPtr)pOldPort)->portBits), 
											&rOffscreen, 
											&rOnscreen, 
											srcCopy,
											NULL);
				
				::UnlockPixels( hPixels );				
			}			
	}
	

#pragma mark -


void
CMultiProgressWindow::FinishCreateSelf()
	{
		ResizeWindowTo( kStatusItemWidth, kStatusItemHeight );
	}
	

void
CMultiProgressWindow::MigrateIndicators( const UInt32 intInIndex )
	{
		UInt32						intIndex 			= 0;
		UInt32						intCount			= 0;
		SInt32						intMaxVal			= 0;
		SInt32						intMinVal			= 0;
		SInt32						intCurVal			= 0;
		
		Str255						pstrText			= "\p";
		Boolean						bIndeterminate = false;
				
		if ( (intInIndex + 1) < mItemCount )
			{
				intCount = mItemCount - 1 - intInIndex;
			}
	
		if ( intCount > 0 )
			{
				for ( intIndex = intInIndex; (intIndex + 1) < mItemCount; intIndex++ )
					{
						LProgressBar *pProgressBar1 = dynamic_cast<LProgressBar *>( FindPaneByID(intIndex * 100 + kProgressBarSubID) );
						LProgressBar *pProgressBar2 = dynamic_cast<LProgressBar *>( FindPaneByID((intIndex + 1) * 100 + kProgressBarSubID) );				
						Assert_( (pProgressBar1 != NULL) && (pProgressBar2 != NULL) );
						
						if ( (pProgressBar1 != NULL) && (pProgressBar2 != NULL) )
							{
								bIndeterminate = pProgressBar2->IsIndeterminate();
								
								if ( bIndeterminate )
									{
										pProgressBar1->SetIndeterminateFlag( true, true );
									}
								else
									{
										pProgressBar2->GetAllValues( intCurVal, intMinVal, intMaxVal );
										pProgressBar1->SetMinValue( intMinVal );
										pProgressBar1->SetMaxValue( intMaxVal );
										pProgressBar1->SetValue( intCurVal );
									}
							}
							
						LPushButton *pStopButton1 = dynamic_cast<LPushButton *>( FindPaneByID(intIndex * 100 + kStopButtonSubID) );
						LPushButton *pStopButton2 = dynamic_cast<LPushButton *>( FindPaneByID((intIndex + 1) * 100 + kStopButtonSubID) );
						Assert_( (pStopButton1 != NULL) && (pStopButton1 != NULL) );
			
						if ( (pStopButton1 != NULL) && (pStopButton1 != NULL) )
							{
								pStopButton1->SetValueMessage( pStopButton2->GetValueMessage() );
							}
					}
			}				
	}
	

Boolean
CMultiProgressWindow::GetPIDPane( const UInt32 intInID, UInt32 &intOutIndex, SProgressItemRec &outItem )
	{
		UInt32	intCurIndex = 0;
		Boolean	bFoundIt 		= false;
		
		SProgressItemRec sCurItem;
		
		for ( intCurIndex = 0; (intCurIndex < mItemCount) && (!bFoundIt); intCurIndex++ )
			{
				sCurItem = (*mItemsH)[intCurIndex];
				
				if ( sCurItem.fID == intInID )
					{
						bFoundIt = true;
						outItem = sCurItem;
						intOutIndex = intCurIndex;
					}
			}
	
		return bFoundIt;		
	}
	
	
void
CMultiProgressWindow::ListenToMessage( MessageT inMessage, void *ioParam )
	{		
		BroadcastMessage( msg_Cancel, &inMessage );		
	}
	