//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CSpinIndicator.cp
//
//	  	Author: 	Dan Clifford
//
//################################################################################


#include <Threads.h>
#include <UDrawingUtils.h>

#include "CThreadedTask.h"
#include "CSpinIndicator.h"


static Handle sSpinningArrowsCurs = NULL;


CSpinIndicator::
CSpinIndicator(	LStream 	*inStream): LPane(inStream)
		
	{

		OSErr		result 	= noErr;
		
		mSpinWaitFrame 				= 0;
		mLatentSpinWaitFrame 	= 0;

		mSpinWaiters 					= 0;
		mSpinThread 					= NULL;
		
	}


CSpinIndicator::
~CSpinIndicator()
	{

		if ( mSpinThread != NULL )
			{
			
				mSpinThread->ShutdownThread();
				
				while ( mSpinThread != NULL )
					CThreadedTask::Yield();
			
			}
			
	}

	
CSpinIndicator *
CSpinIndicator::CreateCSpinIndicatorFromStream(LStream *inStream)
{

	return new CSpinIndicator(inStream);

}


void
CSpinIndicator::FinishCreateSelf()
	{
	
		mSpinThread = new CSpinIndicatorThread( *this );
		ThrowIfNil_( mSpinThread );
		
	}
	
	
//################################################################################

void 
CSpinIndicator::DrawSelf()
	{
		
		Rect							localFrameRect 	= { 0, 0, 0, 0 };
		BitMap						bitmap;
		CGrafPtr					windowPort			= NULL;
		RGBColor					mediumGrey 			= { 0xEEEE, 0xEEEE, 0xEEEE };
		StColorPenState		savedState;
		
		savedState.Normalize();
		
		if (sSpinningArrowsCurs == NULL)
			sSpinningArrowsCurs = GetResource('SICN', 128);
			
		::HLock(sSpinningArrowsCurs);
		
		bitmap.baseAddr = *sSpinningArrowsCurs + mSpinWaitFrame * (16 * 2);
		bitmap.rowBytes = 2;
		SetRect(&bitmap.bounds, 0, 0, 16 ,16); 
		
		CalcLocalFrameRect(localFrameRect);
		
		windowPort = (CGrafPtr)GetMacPort();
		
		::RGBForeColor(&mediumGrey);
		::PaintRect(&localFrameRect);
		
		ForeColor(blackColor);

		::CopyBits( &bitmap, (BitMap *)&(windowPort->portPixMap), &(bitmap.bounds), &localFrameRect, srcOr, NULL );

		::HUnlock(sSpinningArrowsCurs);
		
	}


void 
CSpinIndicator::Advance()
	{
		
		mSpinWaitFrame++; 
		
		if (mSpinWaitFrame >= kTotalSpinFrames) 
			mSpinWaitFrame = kFirstNonZeroFrame; 

		FocusDraw();
		Draw(NULL);

	}


void
CSpinIndicator::ResetIndicator()
	{

		mSpinWaitFrame = 0;
		
		FocusDraw();
		Draw(NULL);

	}


void
CSpinIndicator::ThreadedTaskDied( const CThreadedTask *inTask )
	{
	
		Assert_( (inTask == mSpinThread) && (mSpinThread != NULL) );
		
		if ( inTask != NULL )
			if ( inTask == mSpinThread )
				mSpinThread = NULL;
	
	}


//################################################################################


void
CSpinIndicator::ListenToMessage( MessageT inMessage, void *ioParam )
	{

		switch (inMessage) 
			{
			
				Assert_( mSpinThread != NULL );
				
				case msg_SpinWaitBegin:
				
					mSpinWaiters++;
					
					if ( (mSpinWaiters == 1) && (mSpinThread != NULL) )
						{

//  Set the spin indicator to the last saved state.
						
							mSpinWaitFrame = mLatentSpinWaitFrame;
							
							if ( mSpinWaitFrame < kFirstNonZeroFrame )
								mSpinWaitFrame = kFirstNonZeroFrame;
							
							try
								{
								
									mSpinThread->Resume();
									
								}
								
							catch ( ... )
								{
								
								
								}
												
						}
										
				break;
			
				case msg_SpinWaitEnd:
				
					mSpinWaiters--;
					
					if ( mSpinWaiters < 0 )
						{
						
							mSpinWaiters = 0;
							
						}
						
					if ( (mSpinWaiters == 0) && (mSpinThread != NULL) )
						{
						
							try
								{
								
									mSpinThread->Sleep();
									
								}
								
							catch ( ... )
								{
								
								
								}
							
							mLatentSpinWaitFrame = mSpinWaitFrame;

							ResetIndicator();
							
						}
									
				break;

				case msg_SpinNoMoreWaiters:
			
					mSpinWaiters = 0;
					
					if ( mSpinThread != NULL )
						{
						
							try
								{
								
									mSpinThread->Sleep();
									
								}
								
							catch ( ... )
								{
								
								
								}
						
						}

					mLatentSpinWaitFrame = mSpinWaitFrame;
						
					ResetIndicator();
					
				break;
					
			}

	}


CSpinIndicatorThread::CSpinIndicatorThread( CSpinIndicator &inOwnerView ):

	CThreadedTask( CSpinIndicator::kSpinIndicatorStackSize, false )
	
	{
	
		mOwnerView = &inOwnerView;
	
	}
	
	
CSpinIndicatorThread::~CSpinIndicatorThread()
	{
	
		mOwnerView->ThreadedTaskDied( this );
	
	}
	
	
void
CSpinIndicatorThread::Run()
	{
	
		SInt32					lastUpdate				= 0x00000000;
		
		Assert_( mOwnerView != NULL );
		
		if ( mOwnerView != NULL )
			{
			
				while ( true ) 
					{

						try
							{

								CThreadedTask::Yield();

								if ( (::TickCount() - lastUpdate) > CSpinIndicator::kSleepTimeBetweenArrowAdvances )
									{
									
										mOwnerView->Advance();
										
										lastUpdate = ::TickCount();
										
									}
																
							}
							
						catch ( CThreadedTaskException inErr )
							{
							
								throw inErr;
							
							}
							
						catch ( ... )
							{
							
							
							}

					}
			
			}
								
	}

