//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		CTableDragSelector.cp
//
//	  Author: 	Scott M. Silver and Ram Rajadhyaksha
//
//################################################################################


#include <Events.h>

#include "CTableDragSelector.h"

#define Min(a, b)			((a) < (b) ? (a) : (b))
#define Max(a, b)			((a) > (b) ? (a) : (b))


CTableDragSelector::CTableDragSelector(LTableView	*inTableView):

	LTableMultiSelector( inTableView )

	{
	}

		
void
CTableDragSelector::ClickSelect(const STableCell &inCell, const SMouseDownEvent	&inMouseDown)
	{
		
		Boolean 	wasSelected	 	= false,
							didDrag				= false;
							
		Point			startMouse		= { 0, 0 },
							curMouse 			= { 0, 0 };
		
		wasSelected = CellIsSelected( inCell );
		didDrag = false;
		
		if ( wasSelected )
			{
			
				::GetMouse( &startMouse );
				
				Rect slopRect = { startMouse.v - 3, startMouse.h - 3, startMouse.v + 3, startMouse.h + 3 };
									
				while ( ::StillDown() )
					{
					
						::GetMouse( &curMouse );
						
						if ( ::PinRect(&slopRect, curMouse) != *(SInt32*) &curMouse )
							{
							
								didDrag = true;
								HandleDrag( inCell, inMouseDown );
								break;
							
							}
					
					}
					
				if ( !didDrag )
					{
					
						LTableMultiSelector::ClickSelect( inCell, inMouseDown );
					
					}
					
			}
		
		else
		
			{
			
				LTableMultiSelector::ClickSelect( inCell, inMouseDown );

// we do the setup of the slopRect here because the local
// coordinate system can change after the ClickSelect
				
				::GetMouse( &startMouse );
				
				Rect slopRect = { startMouse.v - 3, startMouse.h - 3, startMouse.v + 3, startMouse.h + 3 };

				while ( ::StillDown() )
					{
						
						::GetMouse( &curMouse );
						
						if ( ::PinRect(&slopRect, curMouse) != *(SInt32*) &curMouse )
							{
							
								didDrag = true;
								HandleDrag( inCell, inMouseDown );
								break;
							
							}
						
					}
					
			}

//  Okay, this sucks royally: Since ClickSelect() cannot return a boolean as to
//  whether or not the app. should process the click, we call an empty virtual
//  function here. Whoever inherits this class can override this function to
//  notify the table that the ClickSelect() wasn't a real click. (doesn't this suck?)
//  See an example in CMacCVSProjectView::ClickSelect().
			
		if ( didDrag )
			UserDraggedSelection();
			
	}


Boolean
CTableDragSelector::DragSelect( const STableCell &inCell, const SMouseDownEvent	&inMouseDown)
	{
	
		Point	mouseLoc	= { 0, 0 };
		
		mouseLoc = inMouseDown.whereLocal;
		
		mCmdKeyDown =  (inMouseDown.macEvent.modifiers & cmdKey) != 0;
		mShiftKeyDown =  (inMouseDown.macEvent.modifiers & shiftKey) != 0;
		
		mTableView->FocusDraw();
		TrackClick( mouseLoc );
		
		return true;
		
	}
	

void 
CTableDragSelector::HandleDrag(const STableCell &inCell, const SMouseDownEvent &inMouseDown )
	{

	}
	
	
void
CTableDragSelector::TrackClick( const Point &inMouse )
	{

//  This class use to be totally crap because it only supported drag and drop. Now
//  it is semi-cool with this real drag routine. -rvr.
	
		Rect				rSelection		= { 0, 0, 0, 0 };
		Rect				rFrameRect		= { 0, 0, 0, 0 };
		Rect				rCellRect			= { 0, 0, 0, 0 };
		Point				sLastMousePt	= { 0, 0 };
		Point				sCurMousePt		= { 0, 0 };
		SPoint32		sMagnitude		= { 0, 0 };
			
		UInt32			intCurRow			= 0;
		UInt32			intCurCol			= 0;
		UInt32			intMaxRow			= 0;
		UInt32			intMaxCol			= 0;
		UInt32			intVertDelta	= 0;
		UInt32			intHorizDelta = 0;
		RgnHandle		hClipRgn			= NULL;
		
		try
			{				
				hClipRgn = ::NewRgn();
				ThrowIfNil_( hClipRgn );
				
				mTableView->GetTableSize( intMaxRow, intMaxCol );
					
				STableCell	topLeftCell( 0, 0 );
				STableCell	botRightCell( 0, 0 );
				STableCell	oldTopLeftCell( intMaxRow, 0 );
				STableCell	oldBotRightCell( 1, 0 );
				STableCell 	sCurCell( 0, 1 );

				StRegion		oldSelectionRegion( mSelectionRgn );
				
				StColorPenState	stPenState;		
				stPenState.Save();
																			
				while ( ::StillDown() )
					{	
						::GetMouse( &sCurMousePt );
						
						if ( (sCurMousePt.v != sLastMousePt.v) || (sCurMousePt.h != sLastMousePt.h) )
							{
							
//  Hide last selection rect

								::PenPat( &qd.gray );
								::PenMode( patXor );
								::PenSize( 2, 2 );
					
								if ( !::EmptyRect(&rSelection) )
									{
										::FrameRect( &rSelection );
									}
									
								sLastMousePt = sCurMousePt;
								
								::Pt2Rect( sCurMousePt, inMouse, &rSelection );	

//  Don't let the selection rect be zero width or height because
//  weird things will happen.
						
								if ( (rSelection.right - rSelection.left) == 0 )
									{
										::InsetRect( &rSelection, -1, 0 );
									}
									
								if ( (rSelection.bottom - rSelection.top) == 0 )
									{
										::InsetRect( &rSelection, 0, -1 );
									}
									
								stPenState.Restore();

//  Get the mouse position relative to the frame rect. If the mouse
//  is outside the frame rect, get the pixel difference. That will
//  be used as the basis for our magnitude of scrolling.

//  Because SetScrollUnit() is LAME and does all sorts of unneccessary
//  things, we call an inherited PreScroll() which sets the mScrollUnit
//  in the target view.

//  You must call CalcLocalFrameRect() each time.

								mTableView->CalcLocalFrameRect( rFrameRect );
								
								if ( sCurMousePt.v < rFrameRect.top )
									{
										intVertDelta = rFrameRect.top - sCurMousePt.v;
									}
								else if ( sCurMousePt.v > rFrameRect.bottom )
									{
										intVertDelta = sCurMousePt.v - rFrameRect.bottom;
									}

								if ( sCurMousePt.h < rFrameRect.left )
									{
										intHorizDelta = rFrameRect.left - sCurMousePt.h;
									}
								else if ( sCurMousePt.h > rFrameRect.right )
									{
										intHorizDelta = sCurMousePt.h - rFrameRect.right;								
									}
																	
								sMagnitude.v = intVertDelta;
								sMagnitude.h = intHorizDelta;
								
								if ( (sMagnitude.v != 0) || (sMagnitude.h != 0) )
									{
										PreScroll( sMagnitude );
									}
								
								if ( mTableView->AutoScrollImage(sCurMousePt) ) 
									{								
										mTableView->FocusDraw();
									}

								if ( (sMagnitude.v != 0) || (sMagnitude.h != 0) )
									{
									 	PostScroll();
									}
								
//  Draw the selection rect

								::PenPat( &qd.gray );
								::PenMode( patXor );
								::PenSize( 2, 2 );
			
								::FrameRect( &rSelection );
								stPenState.Restore();

//  Ok, this bites: Now that we have drawn the selection rectangle,
//  when we select/unselect cells, we need to erase the part of the
//  rectangle that covers the particular cell (if it does at all).
//  If we don't do this, weird bitcrap is left on the screen curtesy
//  of InvertRect() with QD highlight turned on.
									
//  So, we set the clipping rect to the cell rect, redraw the
//  selection rect to erase it over the cell, select the cell,
//  and then redraw the selection rect.

								mTableView->FetchIntersectingCells( rSelection, topLeftCell, botRightCell );

								sCurCell.row = 0;
								sCurCell.col = 1;
								
								if (!mShiftKeyDown && !mCmdKeyDown)
									{
										// Deselect cells not in selection rect
										
										while ( mTableView->GetNextSelectedCell(sCurCell) )
											{
												if ( !HitCellHotSpot( sCurCell, rSelection) )
													{
														UnselectCell( sCurCell );
													}
											}
									}
								else if (mShiftKeyDown)
									{
										// Only deselect cells that have left the current selection marquee,
										// i.e. those that are neither in the marquee, nor the previous selection
									
										while ( mTableView->GetNextSelectedCell(sCurCell) )
											{
												Point		thePt;
												sCurCell.ToPoint(thePt);
												
												if ( !HitCellHotSpot( sCurCell, rSelection) && !::PtInRgn(thePt, oldSelectionRegion))
													{
														UnselectCell( sCurCell );
													}
											}	
									}
								else if (mCmdKeyDown)
									{
										// Put the selection back to its previous state for cells that are no longer
										// covered by the marquee
										
										// optimization to reduce the amount of looping here
										TableIndexT		extentTop = Min(topLeftCell.row, oldTopLeftCell.row);
										TableIndexT		extentBottom = Max(botRightCell.row, oldBotRightCell.row);
										
										if (extentBottom > intMaxRow)
											extentBottom = intMaxRow;
										
										sCurCell.row = extentTop;
										
										while ( sCurCell.row <= extentBottom)
											{
												if ( !HitCellHotSpot( sCurCell, rSelection) )
													{
														Point		thePt;
														sCurCell.ToPoint(thePt);
														Boolean		alreadySelected = CellIsSelected( sCurCell );
														
														if (::PtInRgn(thePt, oldSelectionRegion)) 
															{
																if ( !alreadySelected) 
																	{
																		mTableView->GetLocalCellRect( sCurCell, rCellRect );
																
																		::GetClip( hClipRgn );
																		::ClipRect( &rCellRect );
																		::PenPat( &qd.gray );
																		::PenMode( patXor );
																		::PenSize( 2, 2 );
																		::FrameRect( &rSelection );
																		::SetClip( hClipRgn );
																		stPenState.Restore();

																		SelectCell( sCurCell );	
															
																		::GetClip( hClipRgn );
																		::ClipRect( &rCellRect );
																		::PenPat( &qd.gray );
																		::PenMode( patXor );
																		::PenSize( 2, 2 );
																		::FrameRect( &rSelection );
																		::SetClip( hClipRgn );
																		stPenState.Restore();
																	}		
															} 
														else 
															{
																if ( alreadySelected ) 
																	{
																		mTableView->GetLocalCellRect( sCurCell, rCellRect );
																
																		::GetClip( hClipRgn );
																		::ClipRect( &rCellRect );
																		::PenPat( &qd.gray );
																		::PenMode( patXor );
																		::PenSize( 2, 2 );
																		::FrameRect( &rSelection );
																		::SetClip( hClipRgn );
																		stPenState.Restore();

																		UnselectCell( sCurCell );	
															
																		::GetClip( hClipRgn );
																		::ClipRect( &rCellRect );
																		::PenPat( &qd.gray );
																		::PenMode( patXor );
																		::PenSize( 2, 2 );
																		::FrameRect( &rSelection );
																		::SetClip( hClipRgn );
																		stPenState.Restore();
																	}
															}
													}
													
												sCurCell.row ++;
											}
									}

								// find what cells are in w/in the rectangle and select them if the rectangle
								// encloses/touches the cell hot spot (usually the icon or icon text).
								
								for ( intCurRow = topLeftCell.row; intCurRow <= botRightCell.row; intCurRow++ )
									{
										sCurCell.row = intCurRow;
										sCurCell.col = 1;
				
										if (mCmdKeyDown)
											{
												// When the marquee hits a cell, toggle the select for that cell
												
												if ( HitCellHotSpot(sCurCell, rSelection) )
													{
														Point		thePt;
														sCurCell.ToPoint(thePt);
													
														if ( ::PtInRgn(thePt, oldSelectionRegion) )
															{
																mTableView->GetLocalCellRect( sCurCell, rCellRect );
																
																::GetClip( hClipRgn );
																::ClipRect( &rCellRect );
																::PenPat( &qd.gray );
																::PenMode( patXor );
																::PenSize( 2, 2 );
																::FrameRect( &rSelection );
																::SetClip( hClipRgn );
																stPenState.Restore();

																UnselectCell( sCurCell );									
																
																::GetClip( hClipRgn );
																::ClipRect( &rCellRect );
																::PenPat( &qd.gray );
																::PenMode( patXor );
																::PenSize( 2, 2 );
																::FrameRect( &rSelection );
																::SetClip( hClipRgn );
																stPenState.Restore();
															}
														else
															{
																mTableView->GetLocalCellRect( sCurCell, rCellRect );
																
																::GetClip( hClipRgn );
																::ClipRect( &rCellRect );
																::PenPat( &qd.gray );
																::PenMode( patXor );
																::PenSize( 2, 2 );
																::FrameRect( &rSelection );
																::SetClip( hClipRgn );
																stPenState.Restore();

																SelectCell( sCurCell );									
																
																::GetClip( hClipRgn );
																::ClipRect( &rCellRect );
																::PenPat( &qd.gray );
																::PenMode( patXor );
																::PenSize( 2, 2 );
																::FrameRect( &rSelection );
																::SetClip( hClipRgn );
																stPenState.Restore();
															}
													}									
											} 
										else
											{
												if (!CellIsSelected(sCurCell) && HitCellHotSpot(sCurCell, rSelection))
													{
														mTableView->GetLocalCellRect( sCurCell, rCellRect );
														
														::GetClip( hClipRgn );
														::ClipRect( &rCellRect );
														::PenPat( &qd.gray );
														::PenMode( patXor );
														::PenSize( 2, 2 );
														::FrameRect( &rSelection );
														::SetClip( hClipRgn );
														stPenState.Restore();

														SelectCell( sCurCell );									
														
														::GetClip( hClipRgn );
														::ClipRect( &rCellRect );
														::PenPat( &qd.gray );
														::PenMode( patXor );
														::PenSize( 2, 2 );
														::FrameRect( &rSelection );
														::SetClip( hClipRgn );
														stPenState.Restore();
													}
											}							
									} // for each row

								oldTopLeftCell = topLeftCell;
								oldBotRightCell = botRightCell;			
							}	// if mouse moved
					
					}		// StillDown

				if ( !::EmptyRect(&rSelection) )
					{
						::PenPat( &qd.gray );
						::PenMode( patXor );
						::PenSize( 2, 2 );
						::FrameRect( &rSelection );
						stPenState.Restore();
					}
				
				if ( hClipRgn != NULL )
					{
						::DisposeRgn( hClipRgn );
						hClipRgn = NULL;
					}
				
				//  Hack: If we messed with the scroll units,
				//  we better call SetScrollUnit() or the scroll
				//  bar will be funkified until the window is
				//  resized, etc.
					
				mTableView->GetScrollUnit( sMagnitude );
				mTableView->SetScrollUnit( sMagnitude );
			}
		catch ( ... )
			{
				if ( hClipRgn != NULL )
					{
						::DisposeRgn( hClipRgn );
						hClipRgn = NULL;
					}
				throw;
			}
		
	}
	

// ---------------------------------------------------------------------------
//		 SetCellSelection
// ---------------------------------------------------------------------------
//	Set the selection of the given cell to select, unless the command key
//	was down, in which case toggle the selection

void
CTableDragSelector::SetCellSelection(const STableCell	&inCell, const Boolean select)
{
		if (mCmdKeyDown) {
		
				if (CellIsSelected(inCell))
						UnselectCell(inCell);
				else
						SelectCell(inCell);
		
		} else {
		
				if (select)
						SelectCell(inCell);
				else
						UnselectCell(inCell);
		}
}


Boolean
CTableDragSelector::HitCellHotSpot( const STableCell &inCell, const Rect &inTotalSelectionRect )
	{
	
			Rect	cellRect	= { 0, 0, 0, 0 };
			
			mTableView->GetLocalCellRect( inCell, cellRect );
					
			return ::SectRect( &cellRect, &inTotalSelectionRect, &cellRect );
	
	}
	
	
void
CTableDragSelector::UserDraggedSelection()
	{	
	
	}
	
