//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CTagPopup.cp
	
	Authors:		Scott Silver, Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code for the new tag pop-up menu.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Made various changes to support new appearance manager classes. Added
				  #include <LPopupButton.h> for new grayscale pop-up class. Now inherits
				  from LPopupButton instead of LStdPopupMenu.
							
-------------------------------------------------------------------------------- */


#include <string.h>
#include <stdlib.h>

#include <UModalDialogs.h>
#include <LArray.h>
#include <LEditText.h>

#include "UErrorDialog.h"

#include "CTagPopup.h"


void 
CTagPopup::FinishCreateSelf()
	{
	
		mLastItemSelected = 1;
		
	}


void 
CTagPopup::InsertCustomTagPStr( const Str255 inCustomTagItemPStr )
	{

// ---------------------------------------------------------------------------
//		 CTagPopup::InsertCustomTagPStr
// ---------------------------------------------------------------------------
//	Inserts a CVS style tag (ie. TTestTag) into the CTagPopup menu.
		
		MenuHandle	tagMenu 					= GetMacMenuH();
		UInt32			tagMenuItemCount 	= 0x00000000;
		Str255			tempPStr					= "\p";
		LStr255			tagPStr;
		
		Assert_( tagMenu != NULL );
		Assert_( inCustomTagItemPStr[0] > 1 );
		
		if ( (inCustomTagItemPStr[0] > 1) && (tagMenu != NULL) ) 
			{

//  Get rid of leading 'T' or 'D'.
					
				if ( inCustomTagItemPStr[1] == 'T' )
					
					tagPStr = "\pRev: ";
					
				else tagPStr = "\pDate: ";

//  Get rid of that first char.
				
				::BlockMoveData( &inCustomTagItemPStr[2], &tempPStr[1], inCustomTagItemPStr[0] - 1 );
				tempPStr[0] = inCustomTagItemPStr[0] - 1;
				
				tagPStr = tagPStr + tempPStr;

//  InsertMenuItem seems to crash with 255 chars so we limit to 200.
				
				if ( tagPStr[0] > 200 )
					tagPStr[0] = 200;
					
				tagMenuItemCount = ::CountMItems( tagMenu );

				::InsertMenuItem( tagMenu, tagPStr, tagMenuItemCount - 2 );
				
				SetMaxValue( tagMenuItemCount + 1 );
				SetValue( GetMaxValue() - 2 );
				
				mLastItemSelected = GetValue();
			
			}
		
	}


void 
CTagPopup::InsertCustomTagCStr( const char *inCustomTagItemCStr )
	{
		
		Str255	itemPStr	= "\p";
		UInt32	len			= 0x00000000;
		
		if ( inCustomTagItemCStr != NULL ) 
			{
			
				len = strlen( inCustomTagItemCStr );
				
				if ( len > 255 )
					len = 255;
					
				itemPStr[0] = len;
				
				::BlockMove( inCustomTagItemCStr, &itemPStr[1], itemPStr[0] );
				
				InsertCustomTagPStr( itemPStr );
				
			}
			
	}


void				
CTagPopup::RemoveCustomTagPStr( Str255 outCustomTagItemPStr )
	{

		MenuHandle	tagMenu 					= GetMacMenuH();
		UInt32			tagMenuItemCount	= 0x00000000;
		
		Assert_( tagMenu != NULL );
		
		if ( tagMenu != NULL ) 
			{
				
				outCustomTagItemPStr[0] = 0x00;
			
				tagMenuItemCount = ::CountMItems( tagMenu );
				
				if ( tagMenuItemCount > 3 ) 
					{
				
						::GetMenuItemText( tagMenu, 2, outCustomTagItemPStr );
						::DeleteMenuItem( tagMenu, 2 );
			
						SetMaxValue( tagMenuItemCount - 1 );
				
						if ( !(GetValue() < (GetMaxValue() - 1)) )
							SetValue( GetMaxValue() - 2 );

					}
				
			}
		
	}


void				
CTagPopup::RemoveCustomTagCStr( char *outCustomTagItemCStr, UInt32 &inOutBufSize )
	{

		Str255	customTagPStr		= "\p";
		UInt32	copyLen					= inOutBufSize;
		
		RemoveCustomTagPStr( customTagPStr );
		
		if ( customTagPStr[0] > 0 ) 
			{
			
				if ( (customTagPStr[0] + 1) < copyLen )
					copyLen = customTagPStr[0] + 1;
					
				if ( copyLen > 0 ) 
					{
					
						::BlockMove( &customTagPStr[1], outCustomTagItemCStr, (copyLen - 1) );
						outCustomTagItemCStr[copyLen - 1] = 0x00;
					
					}
					
			}
			
		inOutBufSize = copyLen;

	}


void
CTagPopup::SetDefaultTag(const Str255	inDefaultTag)
	{
		ArrayIndexT		itemIndex;
		
		itemIndex = mTagList.FetchIndexOf( inDefaultTag, sizeof(Str255) );
		
		if (itemIndex == LArray::index_Bad)
			return;
		
		SetValue( itemIndex + kTipOfTrunkMenuItemID );
	}
	
	
	
void 
CTagPopup::GetCurrentTag(Str255	outCurrentTag)
	{

		if ( (GetValue() == kTipOfTrunkMenuItemID) || (GetValue() > GetMaxValue() - 2) ) 
			{
			
				outCurrentTag[0] = 0;
			
			}
			
		else 
			{
	
				outCurrentTag[0] = 0x00;
						
				mTagList.FetchItemAt( GetValue() - kTipOfTrunkMenuItemID, outCurrentTag );
				
			} 

	}


Boolean 
CTagPopup::CanRemoveCurrentTag()
	{	

		MenuHandle	tagMenu 					= GetMacMenuH();
		UInt32			tagMenuItemCount	= 0x00000000;
		Boolean			result						= false;
		
		if ( tagMenu != NULL ) 
			{
			
				tagMenuItemCount = ::CountMItems( tagMenu );
				
				if ( (GetValue() > 1) && (GetValue() < (tagMenuItemCount - 1)) )
					result = true;
			
			}
		
		return result;
		
	}


void 
CTagPopup::RemoveCurrentTag()
	{

		UInt32		currentItem			= 0x00000000;
		Boolean		result				= false;

		if ( CanRemoveCurrentTag() ) 
			{
						
				currentItem = GetValue();
				
				::DeleteMenuItem( GetMacMenuH(), currentItem );
				SetValue( currentItem - 1 );
				SetMaxValue( GetMaxValue() - 1 );
				Refresh();
				
				mTagList.RemoveItemsAt( 1, currentItem - 1 );
				
				mLastItemSelected = GetValue();	
			
			}
		
	}


void
CTagPopup::SetTagList( LArray &inTagList )
	{

		UInt32	tagListSize = 0x00000000,
						ioCount			= 0x00000000;
		Str255	curTagPStr	= "\p";

		while ( curTagPStr[0] > 0 )
			RemoveCustomTagPStr( curTagPStr );
		
		mTagList.RemoveItemsAt( mTagList.GetCount(), 1 );

//  PowerPlant bug: See comments in GetTagList().
		
		if ( inTagList.GetCount() > 0 )
			{
			
				tagListSize = inTagList.GetCount();
					
				for ( UInt32 currentItem = 1; currentItem <= tagListSize; currentItem++ ) 
					{
													
						ioCount = sizeof(Str255);
						
						inTagList.FetchItemAt( currentItem, curTagPStr, ioCount );

						if ( ioCount == sizeof(Str255) )
							{
								
								InsertCustomTagPStr( curTagPStr );
							
								mTagList.InsertItemsAt( 1, LArray::index_Last, curTagPStr, sizeof(Str255) );
									
							}
							
					}
					
			}
			
		SetValue( kTipOfTrunkMenuItemID );

	}


LArray *
CTagPopup::GetTagList()
	{
			
		LArray	*result = NULL;
		
		result = new LArray( sizeof(Str255) );
		
		ThrowIfNil_( result );

//  PowerPlant bug: if you assign an empty array to another array,
//  it might assert because LArray calls HandToHand() on a nil handle.
		
		if ( mTagList.GetCount() > 0 )
			{
			
				*result = mTagList;
			
			}
			
		return result;
		
	}


void
CTagPopup::DoMouseDownInTagPopup(void)
	{

		MenuHandle				tagMenu 					= GetMacMenuH();
		UInt32						tagMenuItemCount	= 0x00000000;

		LDialogBox				*newTagDialog			= NULL;
		LEditText					*tagEditField			= NULL;
		LPopupButton			*dateRevPopup 		= NULL;

		MessageT 					theMessage 				= msg_Nothing;
		
		Boolean						done							= false,
											alreadyExists			= false;
											
		Str255						curTagPStr				= "\p",
											newTagPStr				= "\p";
		
		tagMenuItemCount = ::CountMItems( tagMenu );

		if ( GetValue() == tagMenuItemCount ) 
			{

				StDialogHandler newTagDialogHandler( kNewTagDialogPPobID, LCommander::GetTopCommander() );
			
				newTagDialog = dynamic_cast<LDialogBox *>( newTagDialogHandler.GetDialog() );
				ThrowIfNil_( newTagDialog );
			
				dateRevPopup = dynamic_cast<LPopupButton *>( newTagDialog->FindPaneByID(kNewTagDialogPopupID) );
				ThrowIfNil_( dateRevPopup );
			
				tagEditField = dynamic_cast<LEditText *>( newTagDialog->FindPaneByID(kNewTagTEPaneID) );
				ThrowIfNil_( tagEditField );
			
				newTagDialog->SetDefaultButton( kNewTagDialogOKButtonID );
				newTagDialog->Show();
			
				while ( !done )
					{
					
						switch ( newTagDialogHandler.DoDialog() )
							{
							
								case msg_OK:

									tagEditField->GetDescriptor( newTagPStr );
								
									if ( newTagPStr[0] > 0 )
										{
										
											if ( newTagPStr[0] == 255 )
												newTagPStr[0] = 254;
										
											::BlockMoveData( &newTagPStr[1], &newTagPStr[2], newTagPStr[0] );
			
											if ( dateRevPopup->GetValue() == kRevisionTagMenuItemID )
											
												newTagPStr[1] = 'T';
											
											else newTagPStr[1] = 'D';
												
											newTagPStr[0]++;

											alreadyExists = false;
			
											LArrayIterator	tagIterator( mTagList );
													
											while( tagIterator.Next(&curTagPStr) )
												{
												
													if ( curTagPStr[0] == newTagPStr[0] )
														if ( !strncmp((char *)curTagPStr, (char *)newTagPStr, curTagPStr[0] + 1) )
															alreadyExists = true;
												
												}
																						
											if ( alreadyExists )
												{
												
													UErrorDialog	theError( kTagAlreadyExistsErr, LCommander::GetTopCommander() );
												
												}
												
											else 
												{
												
													InsertCustomTagPStr( newTagPStr );
													
													mTagList.InsertItemsAt( 1, LArray::index_Last, newTagPStr, sizeof(Str255) );
													
													done = true;
												
												}
												
										}
									else
										{
										
											UErrorDialog	theError( kInvalidTagEnteredErr, LCommander::GetTopCommander() );
										
										}
											
								break;
								
								case msg_Cancel:
								
									done = true;
									
									SetValue( mLastItemSelected );
															
								break;
								
							}
							
					}
				
			}
		
		else mLastItemSelected = GetValue();
		
	}


