//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//			File: 		CThreadedTask.cp
//	
//	  	Authors: 	Ram Rajadhyaksha
//
//################################################################################


#include <Threads.h>

#include <LCommander.h>
#include "CThreadedTask.h"
#include "CSpinIndicator.h"

Boolean 	CThreadedTask::sInitialized 			= false;
Boolean		CThreadedTask::sHasThreadMgr 			= false;
ThreadID	CThreadedTask::sMainThreadID			= NULL;
Handle		CThreadedTask::sThreadList				= NULL;
UInt32		CThreadedTask::sThreadCount				= 0x00000000;
UInt32		CThreadedTask::sHeavyThreadCount	= 0x00000000;


CThreadedTask::CThreadedTask( Boolean inHeavyWeightThread)

	{
	
		if ( !sInitialized )
			throw( (CThreadedTaskException)kThreadClassNotInitedErr );
			
		mThreadID = NULL;
		mThreadKilled = false;
		mExceptionSafe = false;
		mStartDelayTicks = 0;
		mHeavyWeightThread = inHeavyWeightThread;
		
		CreateThread();
		InsertThreadInList();
			
	}
	

CThreadedTask::CThreadedTask( const UInt32 inStackSize, Boolean inHeavyWeightThread )

	{

		if ( !sInitialized )
			throw( (CThreadedTaskException)kThreadClassNotInitedErr );
	
		mThreadID = NULL;
		mThreadKilled = false;
		mExceptionSafe = false;
		mStartDelayTicks = 0;
		mHeavyWeightThread = inHeavyWeightThread;
		
		if ( inStackSize > 0 )
			
			CreateThread( inStackSize );
		
		else CreateThread();

		InsertThreadInList();
		
	}
	
	
CThreadedTask::~CThreadedTask()
	{
		
		RemoveThreadFromList();
			
	}
	
	
void
CThreadedTask::Initialize()
	{
	
		OSErr		result	 	= noErr;
		SInt32	response 	= 0x00000000;

		Assert_( sInitialized == false );

		if ( sInitialized )
			throw( (CThreadedTaskException)kThreadProtocolErr );
	
		try
			{
			
				result = ::Gestalt( gestaltThreadMgrAttr, &response );
				
				if ( result != noErr )
					throw result;
					
				if ( response & (1 << gestaltThreadMgrPresent) )
					sHasThreadMgr = true;
				
				else throw( (OSErr)kNoThreadMgrErr );
				
				Assert_( sThreadList == NULL );
				Assert_( sThreadCount == 0 );
				Assert_( sHeavyThreadCount == 0 );
				
				sThreadList = ::NewHandle( 0 );
				
				result = ::MemError();
				
				if ( sThreadList == NULL )
					throw ( ::MemError() );
					
				sThreadCount = 0;
				sHeavyThreadCount = 0;
				
				result = ::GetCurrentThread( &sMainThreadID );
				
				if ( result != noErr )
					throw result;
					
				sInitialized = true;
			
			}
			
		catch ( ... )
			{
			
				if ( sThreadList != NULL )
					{
					
						::DisposeHandle( sThreadList );
						sThreadList = NULL;
					
					}
					
				sInitialized	= false;
					
				throw;
				
			}
	
	}
	

void
CThreadedTask::ShutdownThread()
	{

/*
**  This is a bit complicated. Somehow we need to throw an exception in the
**  thread to get that baby to terminate. Unfortunately there are no provisions for this
**  in the thread manager. Plus, throwing an uncaught exception will ExitToShell(),
**  which is a bad thing. So, once ThreadStart() function has started, it sets a
**  flag ( mExceptionSafe ) telling us that we are in the top level try/catch clause.
*/
	
		OSErr				result						= noErr;
		ThreadID		theCurrentThread	= NULL;
		ThreadState	theState					= NULL;
		
		Assert_( mThreadKilled == false );
		
		if ( mThreadKilled == false )
			{
			
				result = ::GetCurrentThread( &theCurrentThread );
				
				if ( result != noErr )
					throw result;

				mThreadKilled = true;
						
				if ( theCurrentThread == mThreadID )
					{

/*
**  If we're the current thread, throw the exception. If not,
**  an exception will be thrown when we come back from the
**  yield.
*/
			
						Assert_( mExceptionSafe == true );
						
						throw( (CThreadedTaskException)kThreadDiedException );
					
					}
				
				else
					{
					
						result = ::GetThreadState( mThreadID, &theState );
					
						if ( result != noErr )
							throw result;

/*
**  Uhoh, the thread is asleep so we need to wake it up
**  so it can die!
*/
							
						if ( theState == kStoppedThreadState )
							Resume();
							
					}
					
			}
			
	}
	
	
void
CThreadedTask::DeleteThread()
	{

/*
**  DeleteThread() is called when the user wants to delete the thread
**  before it has finished normally. This will nuke the Apple thread object
**  and the C++ object.
*/
	
		::DisposeThread( mThreadID, NULL, false );
	
		delete this;
		
	}
	
	
void
CThreadedTask::Sleep()
	{

		OSErr	result = noErr;
	
		result = ::SetThreadState( mThreadID, kStoppedThreadState, NULL );
		
		if ( result != noErr )
			throw( result );
	
	}
	
	
void
CThreadedTask::Resume()
	{
	
		OSErr	result = noErr;
	
		result = ::SetThreadState( mThreadID, kReadyThreadState, mThreadID );
		
		if ( result != noErr )
			throw( result );
			
	}
	

void
CThreadedTask::DelayedResume(UInt32	inDelayTicks)
	{
		OSErr		result = noErr;
	
		mStartDelayTicks = inDelayTicks;
		
		result = ::SetThreadState( mThreadID, kReadyThreadState, mThreadID );
		
		if ( result != noErr )
			throw( result );
	}


void
CThreadedTask::Yield()
	{
	
		OSErr						result						= noErr;
		Boolean					bFoundIt					= false;

		ThreadID				theCurThreadID		= NULL;
		CThreadedTask		*theCurThreadObj	= NULL;
		
		if ( !sInitialized )
			throw( (CThreadedTaskException)kThreadClassNotInitedErr );
			
		result = ::GetCurrentThread( &theCurThreadID );
		
		if ( result != noErr )
			throw result;
			
		if ( theCurThreadID == sMainThreadID )
		
			::YieldToAnyThread();
			
		else
			{
			
				for ( UInt32 index = 0; ((index < sThreadCount) && (!bFoundIt)); index++ )
					{
			
						theCurThreadObj	= (*(CThreadedTask ***)sThreadList)[index];
				
						if ( theCurThreadObj->mThreadID == theCurThreadID )
							bFoundIt = true;			
					
					}
					
				if ( bFoundIt )
					{
					
						::YieldToAnyThread();
						
						if ( theCurThreadObj->mThreadKilled )
							{
								
								if ( theCurThreadObj->mExceptionSafe )
									{
									
										throw( (CThreadedTaskException)kThreadDiedException );
									
									}

/*
**  If mExceptionSafe == false then we're not in the Run() proc.
**  so we can nuke this puppy.
*/
									
								else theCurThreadObj->DeleteThread();
								
							}
							
					}
					
				else 
					{

/*
**  Someone is trying to yield a thread
**  that we don't know about. Die.
*/

						throw( (CThreadedTaskException)kThreadNotFoundErr );
						
					}
					
			}
	
	}
	
	
pascal void *
CThreadedTask::ThreadStart( CThreadedTask *inThreadedTask )
	{
	
		if ( inThreadedTask != NULL )
			{

				if ( !inThreadedTask->mThreadKilled )
					{
							
						try 
							{
							
								inThreadedTask->mExceptionSafe = true;
								
								if (inThreadedTask->mStartDelayTicks > 0)
									{
										UInt32		resumeTicks = LMGetTicks() + inThreadedTask->mStartDelayTicks;
										
										while (LMGetTicks() < resumeTicks)
											inThreadedTask->Yield();
									
									}
									
								inThreadedTask->Run();
								
							}
							
						catch ( ... )
							{
							
							
							}
							
					}

				inThreadedTask->mExceptionSafe = false;
															
				try
					{
					
						delete inThreadedTask;
						
					}
					
				catch ( ... )
					{
					
					
					}
					
			}
			
		return NULL;
		
	}
		

void
CThreadedTask::CreateThread( const UInt32 inStackSize )
	{
	
		OSErr	result = noErr;

		Assert_( mThreadID == NULL );
				
		try
			{
			
				result = ::NewThread( 	kCooperativeThread, 
																(ThreadEntryProcPtr)ThreadStart, 
																this, 
																inStackSize, 
																kNewSuspend, 
																NULL, 
																&mThreadID );
			
				if ( result != noErr )
					throw( result );
									
			}
		
		catch ( ... )
			{
			
				if ( mThreadID != NULL )
					{
					
						::DisposeThread( mThreadID, NULL, false );
					
						mThreadID = NULL;
						
					}
					
				throw;
			
			}
	
	}
	

void
CThreadedTask::InsertThreadInList()
	{
	
		OSErr	result = noErr;

		if ( sThreadList != NULL )
			{
			
				if ( sThreadCount < 0x7FFFFFFF )
					{
					
						if ( sThreadCount != (::GetHandleSize( sThreadList ) / sizeof(CThreadedTask *)) )
							throw ( (OSErr)kUnknownErr );
							
						sThreadCount++;
						
						::SetHandleSize( sThreadList, sThreadCount * sizeof(CThreadedTask *) );
					
						result = ::MemError();
						
						if ( result != noErr )
							throw result;
							
						Assert_( sThreadCount > 0 );
												
						(*(CThreadedTask ***)sThreadList)[sThreadCount - 1] = this;
						
						if ( mHeavyWeightThread )
							sHeavyThreadCount ++;
						
					}
				else throw( (OSErr)kUnknownErr );
	
			}

	}
	
	
void
CThreadedTask::RemoveThreadFromList()
	{
	
		Boolean		bFoundIt = false;
		
		Assert_( (sThreadCount > 0) && (sThreadList != NULL) );
	
		if ( (sThreadCount > 0) && (sThreadList != NULL) )
			for ( UInt32 index = 0; (index < sThreadCount) && (!bFoundIt); index++ )
				{
				
					if ( (*(CThreadedTask ***)sThreadList)[index] == this )
						{
						
							if ( index < (sThreadCount - 1) )
								{
								
									::BlockMoveData( &((*(CThreadedTask ***)sThreadList)[index + 1]), &((*(CThreadedTask ***)sThreadList)[index]), ((sThreadCount - 1) - index) * sizeof(CThreadedTask *) );
									
								}
								
							sThreadCount--;
							
							::SetHandleSize( sThreadList, sThreadCount * sizeof(CThreadedTask *) );
						
							bFoundIt = true;
							
							if ( mHeavyWeightThread )
								sHeavyThreadCount --;
						}
				
				}
				
		Assert_( bFoundIt );
	
	}
	

//################################################################################
//################################################################################
#pragma mark -
#pragma mark CBroadcastingThreadedTask


CBroadcastingThreadedTask::CBroadcastingThreadedTask( Boolean inHeavyWeightThread ) :
	CThreadedTask( inHeavyWeightThread )
	{
	}

CBroadcastingThreadedTask::CBroadcastingThreadedTask( const UInt32 inStackSize, Boolean inHeavyWeightThread ) :
	CThreadedTask( inStackSize, inHeavyWeightThread )
	{
	}

void
CBroadcastingThreadedTask::Run()
	{
	
			BroadcastMessage( CSpinIndicator::msg_SpinWaitBegin );

			RunSelf();
			
			BroadcastMessage( CSpinIndicator::msg_SpinWaitEnd );
	
	}



//################################################################################
//################################################################################
#pragma mark -
#pragma mark YIELD ATTACHMENT

	
/*
**  This is basically a rip-off of the LYieldAttachment in UThread.h.
*/


CThreadedTaskYieldAttachment::CThreadedTaskYieldAttachment( const SInt32 inQuantum ) : LAttachment( msg_Event, true )
	{

			mQuantum = inQuantum;
			mNextTicks = 0;
			
	}


void 
CThreadedTaskYieldAttachment::ExecuteSelf( MessageT inMessage, void *ioParam )
	{
		
		if ( mQuantum < 0 )
			{
				// no quantum:  just yield & leave
				
				CThreadedTask::Yield();
				
			}
		else
			{
				// We have a non-negative yield quantum.  Just sit in a loop for the 
				// specified number of ticks, yielding control to other threads.
				
				UInt32	curTicks;
				
				do 
					{
						
						CThreadedTask::Yield();
						
						curTicks = ::TickCount();
						
					} 
					
				while ( curTicks <= mNextTicks );
				
				mNextTicks = curTicks + mQuantum;
				
			}

	}
