//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		UCVSUtils.cp
//
//	  Authors: 	Scott M. Silver and Ram Rajadhyaksha
//
//################################################################################


#include <string.h>
#include <ctype.h>
#include <String_Utils.h>

#include "MoreFiles.h"
#include "FileCopy.h"

#include "UCVSUtils.h"


UInt32
UCVSUtils::TruncateFileNameCStr(	const char*				inSrcName,
																	char*							inDestName,
																	const UInt32			inDesiredNameLen)
	{
		
		UInt32	srcNameLen;
		UInt32 	retVal;
			
		srcNameLen = strlen(inSrcName);
		
		retVal = TruncateFileName(inSrcName, srcNameLen, inDestName, inDesiredNameLen);
		
		*(inDestName + inDesiredNameLen) = '\0';
		
		return (retVal);
		
	}


UInt32								
UCVSUtils::TruncateFileNamePStr(	const Str255			inSrcName,
																	Str255						inDestName,
																	const UInt32			inDesiredNameLen)
	{

		UInt32 retVal;
		
		retVal = TruncateFileName((char*) inSrcName + 1, *inSrcName, (char*) inDestName + 1, inDesiredNameLen);
		
		*inDestName = inDesiredNameLen;
		
		return (retVal); 

	}


UInt32
UCVSUtils::TruncateFileName( 			const char*  			inSrcName,
																	const UInt32			inSrcNameLen,
																	char*							inDestName,
																	const UInt32			inDesiredNameLen)
	{
		const char *cp;
		
		// find extension delimeter
		for (cp = inSrcName + inSrcNameLen; *cp != '.' && cp != inSrcName; cp--);
		
		UInt32 extensionLen;
		UInt32 mainNameLen;
		UInt32 cutOutLen;
		
		extensionLen = inSrcName + inSrcNameLen - cp;
		mainNameLen = inSrcNameLen - extensionLen;
		cutOutLen = inSrcNameLen - inDesiredNameLen;
		
		if (extensionLen < inDesiredNameLen)
			{	
				UInt32 	keepOnRightSide = (mainNameLen - cutOutLen) / 2;
				const char*	beginMove = cp - keepOnRightSide;
				
				::BlockMoveData(inSrcName, inDestName, keepOnRightSide);
				::BlockMoveData(beginMove, inDestName + keepOnRightSide, extensionLen + keepOnRightSide);
				
				return (inDesiredNameLen);
			}
			
		else
			{
			
				::BlockMoveData(inSrcName, inDestName, inDesiredNameLen);
				
				return (inDesiredNameLen);
				
			}
		
	}


/*
	SafeDuplicate
	
	Tries to duplicate a file, appending a "~#" on it, trying till
	all 255 chars wrap.
*/


void CalcParentFolderSpec(		const FSSpec&		inFileSpec,
															FSSpec&					outFolderSpec);
	
OSErr
UCVSUtils::SafeDuplicate(			FSSpec& 				inSrc)
	{

		FSSpec	destSpec = inSrc;
		Str255	appendStr = "\p~0";
		unsigned char wrapChar = appendStr[appendStr[0]];
		OSErr	theErr;
		Str31	destName;
		
		// make room for appendage
		CopyPStr(inSrc.name, destName, 31);
		
		if (destName[0] >= 29)
			destName[0] = 28;

		CalcParentFolderSpec(destSpec, destSpec);
		
		// append tail
		ConcatPStr(destName, appendStr, 31);

		do
			{	
				
				theErr = ::FSpFileCopy(&inSrc, &destSpec, destName, NULL, NULL, false);
				
				destName[destName[0]]++;	// next copy
				
				// below (we force assigne theErr to -1 if we wrap)
				
			} while (theErr == dupFNErr && ((destName[destName[0]] != wrapChar) && (theErr = -1)));
		
		return (theErr);
			
	}


void CalcParentFolderSpec(		const FSSpec&			inFileSpec,
															FSSpec&						outFolderSpec)
	{
	
		CInfoPBRec cpb;
		cpb.dirInfo.ioFDirIndex = -1;
		cpb.dirInfo.ioVRefNum = inFileSpec.vRefNum;
		cpb.dirInfo.ioDrDirID = inFileSpec.parID;
		cpb.dirInfo.ioNamePtr = outFolderSpec.name;
		OSErr theErr = ::PBGetCatInfoSync(&cpb);
		ThrowIfOSErr_(theErr);
		
		outFolderSpec.vRefNum = inFileSpec.vRefNum;
		outFolderSpec.parID = cpb.dirInfo.ioDrParID;
	
	}


Boolean
UCVSUtils::ExtractPhrase( char *inSrcData, UInt32 inSrcLen, char inSeperator, UInt32 inWhichPhrase, char *outPhrase, UInt32 *outPhraseLen )
	{
		
		char			currentCharacter	= 0x00;
		UInt32		maxOutputSize 		= *outPhraseLen,
							actualOutputSize 	= 0;
		
		*outPhrase = 0;
		*outPhraseLen = 0;

//	Position the cursor at the beginning of the 
//	phrase that we actually want.

		while ( inWhichPhrase > 1 ) 
			{
					
				while (1) 
					{
					
						if (inSrcLen == 0)
							return false;
							
						currentCharacter = *inSrcData;
							
						inSrcData++;
						inSrcLen--;
				
						if (currentCharacter == inSeperator)
							break;
					
					}
					
				inWhichPhrase--;
				
			}

//	Copy the desired phrase into the output buffer

		while (1) 
			{
			
				if ( inSrcLen == 0 )
					break;
					
				if ( maxOutputSize == 0 )
					break;
					
				currentCharacter = *inSrcData;

				if ( currentCharacter == inSeperator )
					break;
					
				*outPhrase++ = currentCharacter;
			
				inSrcData++;
				actualOutputSize++;
				maxOutputSize--;
				inSrcLen--;
				
			}
		
		if ( maxOutputSize > 0 )
			*outPhrase++ = '\0';
		
		*outPhraseLen = actualOutputSize;

		return true;	
		
	}


SInt32			
UCVSUtils::CompareCVSVersion( char *inVersion1Str, char *inVersion2Str )

/*
**  Ok, there are arguably two ways to do this. The first method is to
**  parse each term out of the version string, convert it to a number,
**  and compare it. This is what I do here. Unfortunately the performance
**  totally blows. The other option I thought of was to run through
**  both strings backwards multiply each term by a power of ten, effectively
**  converting it to a number and then comparing both. The problem with
**  the second method is if you have a 9 digit or greater version string,
**  you will encounter overflow problems because of the 32 bit number limitation.
*/

	{

		SInt32			result 		= 0x00000000,
								term1			= 0x00000000,
								term2			= 0x00000000;
						
		Boolean			done			= false;

		Str255			ver1PStr	= "\p",
								ver2PStr	= "\p";
						
		unsigned char	*temp		= NULL;
		
		while ( !done ) 
			{
		
				ver1PStr[0] = 0x00;
				ver2PStr[0] = 0x00;
/*
**  Parse a new term out of the first version string...
*/
				temp = &ver1PStr[1];
		
				while (((*inVersion1Str) != '.') && ((*inVersion1Str) != 0x00)) 
					{
					
						*temp = (unsigned char)*inVersion1Str;	
					
						temp++;
						inVersion1Str++;
						ver1PStr[0]++;
					
					}
		
				if ((*inVersion1Str) == '.')
					inVersion1Str++;		
						
/*
**  Convert it to a number or -1 if there was no term...
*/			
				
				if ( ver1PStr[0] > 0 )
					::StringToNum( ver1PStr, &term1 );
		
				else term1 = -1;
/*
**  Parse a new term out of the second version string...
*/			
		
				temp = &ver2PStr[1];
		
				while (((*inVersion2Str) != '.') && ((*inVersion2Str) != 0x00)) 
					{
					
						*temp = (unsigned char)*inVersion2Str;	
					
						temp++;
						inVersion2Str++;
						ver2PStr[0]++;
					
					}
				
				if ((*inVersion2Str) == '.')
					inVersion2Str++;
					
				if ( ver2PStr[0] > 0 )
					::StringToNum( ver2PStr, &term2 );
					
				else term2 = -1;
			
				if (term1 > term2) 
					{
					
						result = 1;
						done = true;
							
					}
					
				else if (term1 < term2) 
					{
					
						result = -1;	
						done = true;	
						
					}
					
/*
**  If both terms are -1, then we have reached the end
**  of both version strings and that means they are equal.
*/
				
				else if ((term1 == -1) && (term2 == -1))
					done = true;
					
			}
	
		return result;

	}


void
UCVSUtils::RelativeDateString(	UInt32 				inDateSeconds,
																Str255				outDateString)
	{

	 	DateTimeRec	now, printDate;
		UInt32 nowSeconds;

	 	::GetTime(&now);
		::DateToSeconds(&now, &nowSeconds);
		::SecondsToDate(inDateSeconds, &printDate);

		TString<Str255> theDateString;
	 
	 	//	If the the date is within the last seven days, print it specially.
	 	Int32 dayDifference = (nowSeconds / kSecondsInDay) - (inDateSeconds / kSecondsInDay);
	 	
	 	if (dayDifference <= 7)
	 		{
	 		
		 		if (dayDifference == 0)
				
					theDateString = "\pToday, ";
				
				else if (dayDifference == 1)
				
					theDateString = "\pYesterday, ";
				
				else
					{
					
						switch (printDate.dayOfWeek)
							{
						
								case 2:
									theDateString = "\pLast Monday, ";
									break;

								case 3:
									theDateString = "\pLast Tuesday, ";
									break;

								case 4:
									theDateString = "\pLast Wednesday, ";
									break;

								case 5:
									theDateString = "\pLast Thursday, ";
									break;

								case 6:
									theDateString = "\pLast Friday, ";
									break;

								case 7:
									theDateString = "\pLast Saturday, ";
									break;

								case 1:
									theDateString = "\pLast Sunday, ";
									break;
									
							}
				
					}
				
		 	}
	 	else
	 		{
	 		
		 		::DateString(inDateSeconds, abbrevDate, theDateString, NULL);
		 		theDateString += "\p, ";
		 	
		 	}

		TString<Str255> theTimeString;
		
		::TimeString(inDateSeconds, false, theTimeString, NULL);
		theDateString += theTimeString;

		LString::CopyPStr(theDateString, outDateString);
		
	}


Boolean
UCVSUtils::IsAllowableCVSName( StringPtr		inName)
	{

			char *cp;
			
			for (cp = (char*) inName + 1; cp <= (char*) inName + inName[0]; cp++)
				{
				
					if (*cp == '\\'	 || *cp == '/')
						{
						
							return (false);
						
						}
						
				}	
			
			return (true);
			
	}


char*
UCVSUtils::FilterErrorHeader(const char*		inName)
	{
	
		const char* cp;
		
		for (cp = inName; *cp != ':' && *cp != '\0'; cp++);
		
		if (*cp == '\0')
		
			return ((char*)inName);
		
		else
			{	
				// skip the colon then skip spaces after the colon
				for (cp = cp + 1; isspace(*cp); cp++);

				return ((char*)cp);
			}
	
	}

SInt32 UCVSUtils::sSecondsFromGMT = -1;

/*
	UnixTimeToMacSeconds
	
	Takes a unix string from the date output of the log view
	and converts into seconds on this Macintosh.  We take into
	account GMT conversion.
*/

UInt32
UCVSUtils::UnixTimeToMacSeconds(char *timeString)
{
	UInt32		result 		= 0;
	SInt32		tempNum;
	Str255		tempString;
	DateTimeRec	theDate;

	// get our difference from GMT
	if (sSecondsFromGMT == -1)
	{
		MachineLocation location;
		
		::ReadLocation(&location);
		
		// so it turns out this value they give is a 3 byte SIGNED value..this is to get the
		// right sign extension
		sSecondsFromGMT = ((SInt32) ((location.u.gmtDelta & 0x00FFFFFF) << 8)) >> 8; // only top 3 work
	}	
	
	
	if (timeString != NULL)
	{
		if (strlen(timeString) == 19) 
		{
			BlockMove(timeString, tempString + 1, 4);
			tempString[0] = 4;
			/*
			**  With six calls to StringToNum(), you might as will
			**  write a quickie CharToNum() function and do the
			**  addition of powers.
			*/		
			StringToNum(tempString, &tempNum);
			theDate.year = tempNum;
			
			tempString[0] = 2;
			tempString[1] = timeString[5];
			tempString[2] = timeString[6];
			
			StringToNum(tempString, &tempNum);
			theDate.month = tempNum;
			
			tempString[1] = timeString[8];
			tempString[2] = timeString[9];

			StringToNum(tempString, &tempNum);
			theDate.day = tempNum;
			
			tempString[1] = timeString[11];
			tempString[2] = timeString[12];
			
			StringToNum(tempString, &tempNum);
			theDate.hour = tempNum;
			
			tempString[1] = timeString[14];
			tempString[2] = timeString[15];
			
			StringToNum(tempString, &tempNum);
			theDate.minute = tempNum;
			
			tempString[1] = timeString[17];
			tempString[2] = timeString[18];
			
			StringToNum(tempString, &tempNum);
			theDate.second = tempNum;
			
			DateToSeconds(&theDate, &result);
			result += sSecondsFromGMT;
		}
	}
		
	return (result);
}


OSErr
UCVSUtils::ReadMcvsResourceFromRef( McvsResourcePB *outMcvsPB, const SInt16 inFileRef )
	{

		OSErr										result				= noErr;
		McvsResourceHeaderHndl	mcvsResource 	= NULL;
		
		UInt32									mcvsSize			= 0x00000000,
														curOffset			= 0x00000000,
														len						= 0x00000000;
	
		SInt16									oldResFileRef = 0x00000000;
		
		if ( inFileRef > 0 )
			{
			
				oldResFileRef = ::CurResFile();
				
				::UseResFile( inFileRef );
						
				mcvsResource = (McvsResourceHeaderHndl)::Get1Resource( 'mcvs', 128 );
				
				if ( mcvsResource == NULL )
				
					result = kFileHasNoMcvs;
					
				else
					{
					
						::DetachResource( (Handle)mcvsResource );

						mcvsSize = ::GetHandleSize( (Handle)mcvsResource );
	
/*
**  The mcvs should at least be the size of a version 1 type header.
*/
					
						if ( !(mcvsSize >= sizeof(McvsResourceVer1HeaderRec)) )
						
							result = kMcvsCorrupted;
							
						if ( result == noErr )
							if ( !(((**mcvsResource).fVersion == UCVSUtils::kCurrentMcvsVersion) || ((**mcvsResource).fVersion == UCVSUtils::kMcvsVersion1Format)) )
								result = kUnknownMcvsVersion;
							
						if ( result == noErr )
							{

								if ( (**mcvsResource).fVersion == UCVSUtils::kCurrentMcvsVersion )	
									{
									
										::BlockMoveData( (*mcvsResource), outMcvsPB, sizeof(McvsResourceHeaderRec) );
										curOffset = sizeof( McvsResourceHeaderRec );

									}
								else 
									{
									
										::BlockMoveData( (*mcvsResource), outMcvsPB, sizeof(McvsResourceVer1HeaderRec) );
										curOffset = sizeof( McvsResourceVer1HeaderRec );
										
										outMcvsPB->fFlags = 0;
										outMcvsPB->fRefCon = 0;
									
									}
									
/*
**  Now read the various Pascal strings.
*/


/*
**  Read remote file name string...
*/
									
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fFileNamePStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fFileNamePStr[0] = len;
										
									}
									
								curOffset++;

/*
**  Read revision string...
*/
									
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fRevisionPStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fRevisionPStr[0] = len;
										
									}
									
								curOffset++;

/*
**  Read conflicts string...
*/
	
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fConflictsPStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fConflictsPStr[0] = len;
										
									}

								curOffset++;

/*
**  Read options string...
*/
		
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fOptionsPStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fOptionsPStr[0] = len;													
										
									}

								curOffset++;

/*
**  Read tag string...
*/
										
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fTagPStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fTagPStr[0] = len;
																						
									}
									
								curOffset++;

/*
**  Read unused strings...
*/
	
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fUnused1PStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fUnused1PStr[0] = len;
										
									}
									
								curOffset++;
	
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fUnused2PStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fUnused2PStr[0] = len;
										
									}
									
								curOffset++;
	
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fUnused3PStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fUnused3PStr[0] = len;
										
									}
									
								curOffset++;
	
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										len = *( (*(UInt8 **)mcvsResource) + curOffset );
										curOffset++;
										
										if ( len > 0 ) 
											{
																									
												::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), &outMcvsPB->fUnused4PStr[1], len );
												curOffset += len;
												
											}

										outMcvsPB->fUnused4PStr[0] = len;
										
									}
									
								curOffset++;
																	
								if ( curOffset >= mcvsSize )
											
										result = UCVSUtils::kMcvsCorrupted;
																
								else
									{
									
										outMcvsPB->fRemotePathStr = NULL;
										outMcvsPB->fRemotePathLen = (*(UInt16 *)((*(UInt8 **)mcvsResource) + curOffset));
										curOffset += 2;
										
										if ( outMcvsPB->fRemotePathLen > 0 )
											if ( (curOffset + outMcvsPB->fRemotePathLen) > mcvsSize )
												
												result = UCVSUtils::kMcvsCorrupted;
												
											else
												{
												
													outMcvsPB->fRemotePathStr = ::NewPtr( outMcvsPB->fRemotePathLen );
												
													if ( outMcvsPB->fRemotePathStr == NULL )
														
														result = ::MemError();
														
													else ::BlockMoveData( ((*(UInt8 **)mcvsResource) + curOffset), outMcvsPB->fRemotePathStr, outMcvsPB->fRemotePathLen );
														
												}
												
									}
									
							}
					
						::DisposeHandle( (Handle)mcvsResource );
						mcvsResource = NULL;
								
					}
		
				Assert_( oldResFileRef > 0 );
			
				::UseResFile( oldResFileRef );
				
			}
		
		return result;
		
	}
	
	
OSErr
UCVSUtils::ReadMcvsResource( const FSSpec *inTargetFile, McvsResourcePB *outMcvsPB )
	{

		OSErr 									result 				= noErr;
		
		SInt16									fileRef				= 0x0000;	
		
		if ( (inTargetFile != NULL)	&& (outMcvsPB != NULL) ) 
			{			

				Assert_( inTargetFile->name[0] > 0 );
				Assert_( inTargetFile->vRefNum != 0 );
				Assert_( inTargetFile->parID != 0 );
										
				fileRef = ::FSpOpenResFile( inTargetFile, fsRdPerm );		

				Assert_( fileRef != 0 );
							
				if ( fileRef <= 0 ) 
					{
					
						result = ::ResError();
								
					}
					
				else 
					{
							
						result = ReadMcvsResourceFromRef( outMcvsPB, fileRef );
						
						::CloseResFile( fileRef );
					
					}
				
			}
		
		else result = UCVSUtils::kInvalidParam;
		
		return result;
			
	}


OSErr
UCVSUtils::WriteMcvsResourceFromRef(				McvsResourcePB 	*inMcvsPB,
																						const SInt16		inFileRef)
	{
	
		OSErr									result 					= noErr;
		SInt16								oldResFileRef		= 0x00000000;
		McvsResourceHeaderRec	mcvsHeader;
		
		if ( inFileRef > 0 )
			{
			
				oldResFileRef = ::CurResFile();
				::UseResFile( inFileRef );
				
				Handle	oldMcvsRsrc	= NULL;
				
				oldMcvsRsrc = ::Get1Resource( 'mcvs', 128 );

				if ( (oldMcvsRsrc != NULL) && (::ResError() == noErr) )
					{
					
						::RemoveResource( oldMcvsRsrc );
						::UpdateResFile( inFileRef );
						
						::DisposeHandle( oldMcvsRsrc );
						
					}

/*
**  If we successfully opened/created a resfile, write this baby out.
*/
				
				if ( result == noErr )
					{
					
						Handle	newMcvsRsrc 			= NULL;
						SInt32	newMcvsRsrcSize 	= 0x00000000;
						
						newMcvsRsrcSize = 	sizeof(McvsResourceHeaderRec) + 
																(inMcvsPB->fFileNamePStr[0] + 2) +
																(inMcvsPB->fRevisionPStr[0] + 2) +
																(inMcvsPB->fConflictsPStr[0] + 2) +
																(inMcvsPB->fOptionsPStr[0] + 2) +
																(inMcvsPB->fTagPStr[0] + 2) + 
																(inMcvsPB->fUnused1PStr[0] + 2) + 
																(inMcvsPB->fUnused2PStr[0] + 2) + 
																(inMcvsPB->fUnused3PStr[0] + 2) + 
																(inMcvsPB->fUnused4PStr[0] + 2) + 
																sizeof( UInt16 ) +
																inMcvsPB->fRemotePathLen;
																
						newMcvsRsrc = ::NewHandle( newMcvsRsrcSize );
						
						if ( newMcvsRsrc == NULL )
							
							result = ::MemError();
						
						else
							{
								
								UInt32	curIndex 	= 0x00000000;

/*
**  Write the latest version regardless of
**  what the passed in PB's version is.
*/

								inMcvsPB->fVersion = kCurrentMcvsVersion;

/*
**  Write header
*/
											
								mcvsHeader.fVersion = kCurrentMcvsVersion;
								mcvsHeader.fFlags = inMcvsPB->fFlags;
								mcvsHeader.fRefCon = inMcvsPB->fRefCon;
								
								::BlockMoveData( inMcvsPB, (char *)*newMcvsRsrc, sizeof(McvsResourceHeaderRec) );
								curIndex += sizeof(McvsResourceHeaderRec);

/*
**  Write remote file name string...
*/
			
								::BlockMoveData( &inMcvsPB->fFileNamePStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fFileNamePStr[0] + 1 );
								curIndex += inMcvsPB->fFileNamePStr[0] + 1;
								
								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write revision string...
*/
							
								::BlockMoveData( &inMcvsPB->fRevisionPStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fRevisionPStr[0] + 1 );
								curIndex += inMcvsPB->fRevisionPStr[0] + 1;

								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write conflicts string...
*/
							
								::BlockMoveData( &inMcvsPB->fConflictsPStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fConflictsPStr[0] + 1 );
								curIndex += inMcvsPB->fConflictsPStr[0] + 1;

								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write options string...
*/
							
								::BlockMoveData( &inMcvsPB->fOptionsPStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fOptionsPStr[0] + 1 );
								curIndex += inMcvsPB->fOptionsPStr[0] + 1;

								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write tag string
*/
							
								::BlockMoveData( &inMcvsPB->fTagPStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fTagPStr[0] + 1 );
								curIndex += inMcvsPB->fTagPStr[0] + 1;
								
								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write unused strings...
*/
														
								::BlockMoveData( &inMcvsPB->fUnused1PStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fUnused1PStr[0] + 1 );
								curIndex += inMcvsPB->fUnused1PStr[0] + 1;

								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write Unused2
*/
													
								::BlockMoveData( &inMcvsPB->fUnused2PStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fUnused2PStr[0] + 1 );
								curIndex += inMcvsPB->fUnused2PStr[0] + 1;

								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write Unused3
*/
														
								::BlockMoveData( &inMcvsPB->fUnused3PStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fUnused3PStr[0] + 1 );
								curIndex += inMcvsPB->fUnused3PStr[0] + 1;
						
								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write Unused4
*/

								::BlockMoveData( &inMcvsPB->fUnused4PStr[0], ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fUnused4PStr[0] + 1 );
								curIndex += inMcvsPB->fUnused4PStr[0] + 1;
							
								((char *)*newMcvsRsrc)[curIndex] = 0x00;
								curIndex++;

/*
**  Write Remote Path Len
*/
								
								::BlockMoveData( &inMcvsPB->fRemotePathLen, ((char *)*newMcvsRsrc) + curIndex, sizeof(UInt16) );
								curIndex += 2;

/*
**  Write Remote Path Str
*/
								
								if ( (inMcvsPB->fRemotePathLen > 0) && (inMcvsPB->fRemotePathStr != NULL) )
									::BlockMoveData( inMcvsPB->fRemotePathStr, ((char *)*newMcvsRsrc) + curIndex, inMcvsPB->fRemotePathLen );
																														
								::AddResource( newMcvsRsrc, 'mcvs', 128, "\pMacCVS Version Resource" );
								
								result = ::ResError();
								
								if ( result == noErr )
									{
									
										::UpdateResFile( inFileRef );
								
										result = ::ResError();
									
									}
									
							}	
					
					}

				Assert_( oldResFileRef > 0 );
			
				::UseResFile( oldResFileRef );
							
			}
			
		return result;
	
	}
	
	
OSErr
UCVSUtils::WriteMcvsResource(				const FSSpec 					*inTargetFile, 
																		McvsResourcePB 				*inMcvsPB) 
	{

		OSErr 		result 			= noErr;
		SInt16		fileRef 		= noErr;
		FInfo 		finderInfo;
		
		if ( (inTargetFile != NULL) && (inMcvsPB != NULL) )
			{
			
				Assert_( inTargetFile->name[0] > 0 );
				Assert_( inTargetFile->vRefNum != 0 );
				Assert_( inTargetFile->parID != 0 );
							
				fileRef = ::FSpOpenResFile( inTargetFile, fsRdWrPerm );		

				Assert_( fileRef != 0 );
							
				if ( fileRef <= 0 ) 
					{
					
						result = ::ResError();

/*
**  If the file doesn't have a resource fork, create one.
*/
					
						if ( (result == eofErr) && (FSpIsAFile(inTargetFile) == noErr) )
							{

								result = ::FSpGetFInfo( inTargetFile, &finderInfo );
								
								if ( result == noErr )
									{

/*
**  Is it safe to use FSpCreateResFile() since it follows the
**  PMSP?
*/
						
										::FSpCreateResFile( inTargetFile, finderInfo.fdCreator, finderInfo.fdType, smSystemScript );
								
										result = ::ResError();
								
									}
									
								if ( result == noErr )
									{
			
										Assert_( inTargetFile->name[0] > 0 );
										Assert_( inTargetFile->vRefNum != 0 );
										Assert_( inTargetFile->parID != 0 );
									
										fileRef = ::FSpOpenResFile( inTargetFile, fsRdWrPerm );
	
										Assert_( fileRef != 0 );
								
										if ( fileRef <= 0 )
											result = ::ResError();
											
									}
									
							}
								
					}
					
				if ( (fileRef > 0) && (result == noErr) )
					result = WriteMcvsResourceFromRef( inMcvsPB, fileRef );
					
				if ( fileRef > 0 )
					::CloseResFile( fileRef );
			
			}
			
			return result;

	}


OSErr
UCVSUtils::ReadCkidResourceFromRef( CkidResourcePB *outCkidPB, const Boolean inGetComment, const SInt16 inFileRef )
	{
	
		OSErr 									result 				= noErr;
		Handle									comment				= NULL;
		CkidResourceHeaderHndl	ckidResource 	= NULL;
		UInt32									ckidSize			= 0x00000000,
														curOffset			= 0x00000000,
														checksum			= 0x00000000;
		UInt16									len						= 0x0000;
		SInt16									oldResFileRef	= 0x0000;	
		
		if ( inFileRef > 0 )
			{
				
				oldResFileRef = ::CurResFile();
				::UseResFile( inFileRef );
			
				ckidResource = (CkidResourceHeaderHndl)::Get1Resource( 'ckid', 128 );
				
				if ( ckidResource == NULL )
				
					result = kFileHasNoCkid;
					
				else
					{
					
						::DetachResource( (Handle)ckidResource );

						ckidSize = ::GetHandleSize( (Handle)ckidResource );
						
						if ( !(ckidSize >= sizeof(CkidResourceHeaderRec)) )
							result = kCkidCorrupted;
							
						if ( (**ckidResource).fCurrentVersion != UCVSUtils::kCurrentCkidVersion )
							result = kUnknownCkidVersion;
							
						if ( result == noErr )
							{

/*
**  Verify checksum
*/
										
								#ifdef __VERIFYCHECKSUM
								
								checksum = GenerateCkidChecksum( (Handle)ckidResource );
								
								if ( checksum != (**ckidResource).fChecksum )
									result = kBadCkidChecksum;
									
								else 
								
								#endif
								
									{
									
										::BlockMoveData( (*ckidResource), outCkidPB, sizeof(CkidResourceHeaderRec) );
																		
										curOffset = sizeof( CkidResourceHeaderRec );

/*
**  Now read the various Pascal strings.
*/

/*
**  Read remote path string...
*/
																			
										if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fRemotePathPStr[1], len );
														
														curOffset += len;
														
													}
														
												outCkidPB->fRemotePathPStr[0] = len;
												
											}
											
										curOffset++;

/*
**  Read user name string...
*/
	
										if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fUserNamePStr[1], len );
														
														curOffset += len;
														
													}
													
												outCkidPB->fUserNamePStr[0] = len;
												
											}

										curOffset++;

/*
**  Read revision string...
*/
							
										if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fRevisionPStr[1], len );
														
														curOffset += len;
														
													}
											
												outCkidPB->fRevisionPStr[0] = len;
												
											}
											
											curOffset++;

/*
**  Read file name string...
*/
									
											if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fFileNamePStr[1], len );
														
														curOffset += len;
														
													}
									
												outCkidPB->fFileNamePStr[0] = len;
												
											}
											
											curOffset++;

/*
**  Read task string...
*/
										
											if ( curOffset < ckidSize )
											{
											
												len = *( (*(UInt8 **)ckidResource) + curOffset );
												
												curOffset++;
												
												if ( len > 0 ) 
													{
																											
														::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), &outCkidPB->fTaskPStr[1], len );
														
														curOffset += len;
														
													}

												outCkidPB->fTaskPStr[0] = len;
												
											}
											
											curOffset++;
				
/*
**  Now get the comment.
*/
										
											if ( (curOffset < ckidSize) && (inGetComment == true) )
											{
											
												len = *(UInt16 *)( (*(UInt8**)ckidResource) + curOffset );
												
												curOffset += 2;
		
/*
**  Make sure the size is valid...
*/

												outCkidPB->fCommentLen = len;
										
												if ( (curOffset + len) < ckidSize )
													{
														
														comment = ::NewHandle( len );
														
														if ( comment == NULL )
															{
															
																result = ::MemError();
																
																outCkidPB->fCommentLen = 0;
																outCkidPB->fCommentHndl = NULL;
																
															}
														else 
															{
															
																::BlockMoveData( ((*(UInt8 **)ckidResource) + curOffset), *comment, len );
																
																outCkidPB->fCommentHndl = comment;
															
															}
													
													}

											}
											else 
												{
																	
													outCkidPB->fCommentLen = 0;
													outCkidPB->fCommentHndl = NULL;

												}

/*
**  Something went wrong somewhere...
*/
										
											if ( curOffset > ckidSize )
												result = UCVSUtils::kCkidCorrupted;
											
									}
									
							}
					
						::DisposeHandle( (Handle)ckidResource );
						ckidResource = NULL;
							
					}
	

				Assert_( oldResFileRef > 0 );
			
				::UseResFile( oldResFileRef );
				
			}
			
		return result;
		
	}
	
	
OSErr
UCVSUtils::ReadCkidResource( const FSSpec *inTargetFile, CkidResourcePB *outCkidPB, const Boolean inGetComment )
	{

		OSErr 									result 				= noErr;
		Handle									comment				= NULL;
		CkidResourceHeaderHndl	ckidResource 	= NULL;
		UInt32									ckidSize			= 0x00000000,
														curOffset			= 0x00000000,
														checksum			= 0x00000000;
		UInt16									len						= 0x0000;
		SInt16									fileRef				= 0x0000;	
		
		if ( (inTargetFile != NULL)	&& (outCkidPB != NULL) ) 
			{			
						
				Assert_( inTargetFile->name[0] > 0 );
				Assert_( inTargetFile->vRefNum != 0 );
				Assert_( inTargetFile->parID != 0 );
					
				fileRef = ::FSpOpenResFile( inTargetFile, fsRdPerm );		

				Assert_( fileRef != 0 );
							
				if ( fileRef <= 0 ) 
					{
					
						result = ::ResError();
								
					}
					
				else 
					{
					
						result = ReadCkidResourceFromRef( outCkidPB, inGetComment, fileRef );
						
						::CloseResFile( fileRef );
					
					}
				
			}
		
		else result = UCVSUtils::kInvalidParam;
		
		return result;
			
	}


OSErr
UCVSUtils::	WriteCkidResourceFromRef(	CkidResourcePB					*inCkidPB,
																			const SInt16						inFileRef)
	{
	
		OSErr 		result 				= noErr;
		SInt16		oldResFileRef = 0x0000;
		
		if ( inFileRef > 0 )
			{
			
				oldResFileRef = ::CurResFile();
				::UseResFile( inFileRef );
				
				Handle	oldCkidRsrc	= NULL;
						
				oldCkidRsrc = ::Get1Resource( 'ckid', 128 );

				if ( (oldCkidRsrc != NULL) && (::ResError() == noErr) )
					{
					
						::RemoveResource( oldCkidRsrc );
						::UpdateResFile( inFileRef );
						
						::DisposeHandle( oldCkidRsrc );
						
						oldCkidRsrc = NULL;
						
					}

/*
**  If we successfully opened/created a resfile, write this baby out.
*/
				
				if ( result == noErr )
					{
					
						Handle	newCkidRsrc 			= NULL;
						SInt32	newCkidRsrcSize 	= 0x00000000;
						
						newCkidRsrcSize = 	sizeof(CkidResourceHeader) + 
																(inCkidPB->fRemotePathPStr[0] + 2) +
																(inCkidPB->fUserNamePStr[0] + 2) +
																(inCkidPB->fRevisionPStr[0] + 2) +
																(inCkidPB->fFileNamePStr[0] + 2) +
																(inCkidPB->fTaskPStr[0] + 2) +
																sizeof(UInt16) +
																inCkidPB->fCommentLen + 1;
																
						newCkidRsrc = ::NewHandle( newCkidRsrcSize );
						
						if ( newCkidRsrc == NULL )
							
							result = ::MemError();
						
						else
							{
								
								UInt32	curIndex 	= 0x00000000,
												checksum	= 0x00000000;

/*
**  Write the current version regardless of the
**  PB passed in.
*/
												
								inCkidPB->fCurrentVersion = kCurrentCkidVersion;
											
								::BlockMoveData( inCkidPB, (char *)*newCkidRsrc, sizeof(CkidResourceHeader) );
								
								curIndex += sizeof(CkidResourceHeader);
								
								::BlockMoveData( &inCkidPB->fRemotePathPStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fRemotePathPStr[0] + 1 );
								
								curIndex += inCkidPB->fRemotePathPStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fUserNamePStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fUserNamePStr[0] + 1 );
								
								curIndex += inCkidPB->fUserNamePStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fRevisionPStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fRevisionPStr[0] + 1 );
								
								curIndex += inCkidPB->fRevisionPStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fFileNamePStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fFileNamePStr[0] + 1 );
								
								curIndex += inCkidPB->fFileNamePStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fTaskPStr[0], ((char *)*newCkidRsrc) + curIndex, inCkidPB->fTaskPStr[0] + 1 );
								
								curIndex += inCkidPB->fTaskPStr[0] + 1;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;
								
								curIndex++;
								
								::BlockMoveData( &inCkidPB->fCommentLen, ((char *)*newCkidRsrc) + curIndex, sizeof(UInt16) );
							
								curIndex += sizeof(UInt16);
								
								::BlockMoveData( *inCkidPB->fCommentHndl, ((char *)*newCkidRsrc) + curIndex, inCkidPB->fCommentLen );
								
								curIndex += inCkidPB->fCommentLen;
								
								((char *)*newCkidRsrc)[curIndex] = 0x00;

/*
**  Calculate checksum.
*/						

								checksum = GenerateCkidChecksum( newCkidRsrc );
								
								((CkidResourceHeader *)*newCkidRsrc)->fChecksum = checksum;
								
								::AddResource( newCkidRsrc, 'ckid', 128, "\pProjector Data" );
								
								result = ::ResError();
								
								if ( result == noErr )
									{
									
										::UpdateResFile( inFileRef );
								
										result = ::ResError();
									
									}
									
							}	
			
					}

				Assert_( oldResFileRef > 0 );
					
				::UseResFile( oldResFileRef );
				
			}
			
		return result;
		
	}
	
	
OSErr
UCVSUtils::WriteCkidResource(				const FSSpec 					*inTargetFile, 
																		CkidResourcePB 				*inCkidPB) 
	{

		OSErr 		result 			= noErr;
		SInt16		fileRef 		= noErr;
		FInfo 		finderInfo;
		
		if ( (inTargetFile != NULL) && (inCkidPB != NULL) )
			{
			
				Assert_( inTargetFile->name[0] > 0 );
				Assert_( inTargetFile->vRefNum != 0 );
				Assert_( inTargetFile->parID != 0 );
							
				fileRef = ::FSpOpenResFile( inTargetFile, fsRdWrPerm );		

				Assert_( fileRef != 0 );
							
				if ( fileRef <= 0 ) 
					{
					
						result = ::ResError();

/*
**  If the file doesn't have a resource fork, create one.
*/
					
						if ( (result == -39) && (FSpIsAFile(inTargetFile) == noErr) )
							{

								result = ::FSpGetFInfo( inTargetFile, &finderInfo );
								
								if ( result == noErr )
									{

/*
**  Is it safe to use FSpCreateResFile() since it follows the
**  PMSP?
*/
						
										::FSpCreateResFile( inTargetFile, finderInfo.fdCreator, finderInfo.fdType, smSystemScript );
								
										result = ::ResError();
				
									}
													
								if ( result == noErr )
									{
		
										Assert_( inTargetFile->name[0] > 0 );
										Assert_( inTargetFile->vRefNum != 0 );
										Assert_( inTargetFile->parID != 0 );
									
										fileRef = ::FSpOpenResFile( inTargetFile, fsRdWrPerm );
	
										Assert_( fileRef != 0 );
								
										if ( fileRef <= 0 )
											result = ::ResError();
											
									}
									
							}
								
					}
					
				if ( (fileRef > 0) && (result == noErr) )
					result = WriteCkidResourceFromRef( inCkidPB, fileRef );
				
				if ( fileRef > 0 )	
					::CloseResFile( fileRef );
																				
			}
			
			return result;

	}


OSErr
UCVSUtils::ReadCkidMcvsResourceFromRef( CkidMcvsResourcePB *outCkidMcvsPB, const Boolean inGetComment, const SInt16 inFileRef )
	{
	
		OSErr	result = noErr;
		
		result = ReadCkidResourceFromRef( &outCkidMcvsPB->ckid, inGetComment, inFileRef );
	
		if ( result == noErr )
			result = ReadMcvsResourceFromRef( &outCkidMcvsPB->mcvs, inFileRef );
			
		return result;
	
	}

	
OSErr
UCVSUtils::ReadCkidMcvsResource( const FSSpec *inTargetFile, CkidMcvsResourcePB *outCkidMcvsPB, Boolean inGetComment )
	{
	
		OSErr	result = noErr;
		
		result = ReadCkidResource( inTargetFile, (CkidResourcePB *)outCkidMcvsPB, inGetComment );
	
		if ( result == noErr )
			result = ReadMcvsResource( inTargetFile, (McvsResourcePB *)&(outCkidMcvsPB->mcvs) );
			
		return result;
		
	}
																				

OSErr
UCVSUtils::WriteCkidMcvsResourceFromRef(	CkidMcvsResourcePB *inCkidMcvsPB, const SInt16 inFileRef )
	{

		OSErr	result = noErr;
		
		result = WriteCkidResourceFromRef( &inCkidMcvsPB->ckid, inFileRef );
	
		if ( result == noErr )
			result = WriteMcvsResourceFromRef( &inCkidMcvsPB->mcvs, inFileRef );
			
		return result;
	
	}
	
	
OSErr
UCVSUtils::WriteCkidMcvsResource(	const FSSpec *inTargetFile, CkidMcvsResourcePB *inCkidMcvsPB )
	{
	
		OSErr	result = noErr;
		
		result = WriteCkidResource( inTargetFile, (CkidResourcePB *)inCkidMcvsPB );
	
		if ( result == noErr )
			result = WriteMcvsResource( inTargetFile, (McvsResourcePB *)&(inCkidMcvsPB->mcvs) );
			
		return result;
	
	}
	

void
UCVSUtils::ExtractVersionAndTagFromCkidInfo( CkidResourcePB *inCkidInfo, Str255 outVerPStr, Str255 outTagPStr )
	{
	
		UInt32 index = 0x00000000;
		
		if ( (inCkidInfo != NULL) && (outVerPStr != NULL) && (outTagPStr != NULL) )
			{
					
				index = inCkidInfo->fRevisionPStr[0];
				
				while ( (index > 0) && (inCkidInfo->fRevisionPStr[index] != ':') ) 
					index--;
						
				if ( index > 0 )					
					
					outVerPStr[0] = index - 1;
							
				else 
				
					outVerPStr[0] = inCkidInfo->fRevisionPStr[0];
							
				::BlockMoveData( &inCkidInfo->fRevisionPStr[1], &outVerPStr[1], outVerPStr[0] );
							
				if ( index > 0 ) 
					{
							
						outTagPStr[0] = inCkidInfo->fRevisionPStr[0] - index;
						
						BlockMoveData( &inCkidInfo->fRevisionPStr[index + 1], &outTagPStr[1], outTagPStr[0] );
								
					}
					
			}
		
	}
	
	
pascal UInt32
UCVSUtils::GenerateCkidChecksum( Handle inCkidData )
	{
	  
	  SInt32		dataSize				= 0x00000000;
	  UInt32		checksum 				= 0x00000000;
	  UInt32		*dataPtrCasted 	= NULL;;

	  dataSize = ( ::GetHandleSize(inCkidData) / sizeof(SInt32) ) - 1;
	  
	  dataPtrCasted = (UInt32 *) *inCkidData;

/* 
**  skip over first long (checksum field) 
*/  

	  dataPtrCasted++;                  
	  
	  while ( dataSize-- > 0 ) 
	    checksum += *dataPtrCasted++;

	  return checksum;

	}

	
pascal OSErr
UCVSUtils::CreateItemFromCVSPath(	const char							*inCVSPathStr,
																	const UInt32						inPathLen,
																	const	FSSpec						*inProjectRoot,
																	FSSpec									*outTargetItem,
																	OSType									inCreator,
																	OSType									inType,
																	ScriptCode							inScriptType)
	{

/*
**  CreateFileFromCVSPath() examines a CVS type pathname (ie. /a/b/c/d)
**  and creates the directory structure and file if it doesn't already exist. The
**  pathname can end with a filename (no ending '/') or a directory name.
**  This function assumes that the path passed in is correct in format. Ie:
**  you should call StripCVSPath on the path before calling this function.
*/
	
		OSErr				result 				= noErr;
		Str255			itemName			= "\p",
								itemNameTemp	= "\p",
								pathStr				= "\p";
								
		UInt32			index					= 0x00000000,
								maxIndex			= 0x00000000;
		
		FSSpec			curSpec				= { 0, 0, "\p" };
		
		CInfoPBRec	pb;
		
		if ( (inCVSPathStr != NULL) && (inPathLen > 0) && (inProjectRoot != NULL) && (outTargetItem != NULL) )
			{
															
/*
**  Get the dirID of the root.
*/
												
						curSpec = *inProjectRoot;
						
						pb.hFileInfo.ioCompletion 	= NULL;
						pb.hFileInfo.ioNamePtr 			= curSpec.name;
						pb.hFileInfo.ioVRefNum 			= curSpec.vRefNum;
						pb.hFileInfo.ioDirID 				= curSpec.parID;
						pb.hFileInfo.ioFDirIndex 		= 0;
						
						result = ::PBGetCatInfoSync( &pb );

						maxIndex = CountCVSPathItems( inCVSPathStr, inPathLen );
						
						if ( result == noErr )
							for ( index = 1; (index <= maxIndex) && (result == noErr); index++ )
								{
																					
/*
**  GetCVSPathItem() extracts the next item name in a path string.
**  Ie: a from a/b/c. 
*/
							
									GetCVSPathItem( inCVSPathStr, inPathLen, index, itemName );

/*
**  Make sure the item we're looking for is a folder
**  and that it has a valid Mac name.
*/
									
									if ( itemName[0] > 31 )
										
										result = kInvalidFileName;
										
									else
										{

/*
**  Look for the directory or file.
*/
											
											::BlockMoveData( itemName, itemNameTemp, itemName[0] + 1 );
											
											if ( itemNameTemp[itemNameTemp[0]] == '/' )
												itemNameTemp[0]--;
												
											pb.hFileInfo.ioNamePtr = itemNameTemp;
											
											result = ::PBGetCatInfoSync( &pb );

/*
**  If it doesn't exist, create it.
*/
											
											if ( (result == -43) || (result == -120) )
												{
	
													curSpec.parID = pb.hFileInfo.ioDirID;
													curSpec.vRefNum = pb.hFileInfo.ioVRefNum;
													::BlockMoveData( itemNameTemp, curSpec.name, itemNameTemp[0] + 1 );
										
													if ( itemName[itemName[0]] == '/' )
														{

															result = ::FSpDirCreate( &curSpec, smSystemScript, &pb.hFileInfo.ioDirID );

/*
**  If we created a directory, create all the CVS internal information
*/
															
															if ( result == noErr )
																{

																	result = UCVSUtils::CreateRepositoryDir( pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID, inCreator, inType, inScriptType );
																	
																}
															
														}
														
													else
														{

/*
**  Is it safe to use FSpCreateResFile() since it follows the
**  PMSP?
*/
															
															::FSpCreateResFile( &curSpec, inCreator, inType, inScriptType );
															
															result = ::ResError();
															
														}
												
												}
											
											else if (result == noErr)	
												{

/*
**  If we found a file when we were looking for a folder,
**  or vice verse, die with an error.
*/
												
													if ( (itemName[itemName[0]] == '/') && ((pb.dirInfo.ioFlAttrib & 0x10) == 0) )
													
														result = kAlreadyExists;
														
													else if ( (itemName[itemName[0]] != '/') && ((pb.dirInfo.ioFlAttrib & 0x10) != 0) )
													
														result = kAlreadyExists;

/*
**  Here we make sure that whatever we were looking for is the
**  same *case* as what we wanted. This is important.
*/
/*															

Unfortunately this code doesn't work because we called PBGetCatInfo with a filename,
and it doesn't return one back with the correct case.

													if ( strncmp((char *)&itemName[1], (char *)&pb.hFileInfo.ioNamePtr[1], itemName[0]) )
														result = dupFNErr;
*/

/*
**  If what we're looking for was a directory and we found it, we need to
**  make sure that it has a CVS folder. If not, we need to create one.
*/
														
													if ( result == noErr )
														if ( (pb.dirInfo.ioFlAttrib & 0x10) != 0 )
															if ( !UCVSUtils::IsRepositoryDir(pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID) )
																{
																
																	result = UCVSUtils::CreateRepositoryDir( pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID, inCreator, inType, inScriptType );
																
																}
															
												}
																															
										}
																	
								}
								
						if ( result == noErr )
							{
							
								*outTargetItem = curSpec;
							
							}
							
					}
								
		return result;
	
	}
		
	
UInt32
UCVSUtils::CountCVSPathItems(			const char						 	*inPathStr,
																	const UInt32						inPathLen)
	{
	
		UInt32		result 	= 0x00000000,
							index		= 0x00000000;
							
		Boolean		done 		= false;

		if ( (inPathStr != NULL) && (inPathLen > 0) )
			{
			
				while ( !done )
					{
					
						if ( index >= inPathLen )
							{
							
								result++;
								done = true;
							
							}
						
						else if ( inPathStr[index] == '/' )
							{
							
								if ( index != (inPathLen - 1) )
									result++;
								
								index++;
									
							}
					
						else
							{
							
								while ( (index < inPathLen) && (inPathStr[index] != '/') )
									index++;
							
							}
					
					}
					
			}
				
		return result;
		
	}
	
	
pascal OSErr
UCVSUtils::GetCVSPathItem(				const char						  *inPathStr,
																	const UInt32						inPathLen,
																	const UInt32						inItemIndex,
																	Str255									outItemPStr)
	{

/*
**  GetCVSPathItem returns the nth item in a path string. Ie. in the path "a/b/c",
**  "b/" would be the second item. If the item string return has a trailing '/'
**  character, it is obviously a directory. GetCVSPathItem() assumes that the path
**  string is in correct format, does not contain any multiple sequences of '/' 
**  characters in a row and does not begin with a '/' character.
*/
	
		OSErr			result 					= noErr;
		Boolean		done 						= false,
							foundIt					= false;
							
		UInt32		curItemIndex 		= 0x00000000,
							charIndex 			= 0x00000000;
			
		if ( (inPathStr != NULL) && (inPathLen > 0) && (inItemIndex > 0) && (outItemPStr != NULL) )
			{
			
				outItemPStr[0] = 0;
					
				curItemIndex = 1;
				
				while ( (charIndex < inPathLen) && (!foundIt) )
					{

						if ( (curItemIndex == inItemIndex) && (inPathStr[charIndex] != '/') )
							{
							
								while ( (charIndex < inPathLen) && (inPathStr[charIndex] != '/') )
									{
									
										if ( outItemPStr[0] < 255 )
											{
												
												outItemPStr[0]++;
												
												outItemPStr[outItemPStr[0]] = inPathStr[charIndex];
											
											}
										
										charIndex++;
								
									}
									
								if ( (charIndex < inPathLen) && (inPathStr[charIndex] == '/') )
									{
									
										if ( outItemPStr[0] < 255 )
											{
											
												outItemPStr[0]++;
												outItemPStr[outItemPStr[0]] = '/';
												
											}
																						
									}
									
								foundIt = true;
							
							}
						
						else 
							{
							
							
								while ( (charIndex < inPathLen) && (inPathStr[charIndex] != '/') )
									charIndex++;
							
								curItemIndex++;
								
								if ( (charIndex < inPathLen) && (inPathStr[charIndex] == '/') )
									charIndex++;
									
							}	
																	
					}
					
				if ( !foundIt )
					result = kItemNotFound;	
			
			}
			
		return result;
			
	}
	
	
OSErr
UCVSUtils::StripCVSPath(		char									  *inPathStr,
														UInt32									*inPathLen)
	{

		OSErr		result 	= noErr;
		UInt32 	index 	= 0x00000000;
		
		if ( (inPathStr != NULL) && (inPathLen != NULL) && ((*inPathLen) > 0) )
			{

/*
**  Strip beginning '/' chars.
*/
			
				while ( (inPathStr[0] == '/') && ((*inPathLen) > 0) )
					{
					
						(*inPathLen)--;
						::BlockMoveData( &inPathStr[1], &inPathStr[0], (*inPathLen) );
						
					}

/*
**  If we have anything left, clean it up by removing duplicate '/' chars.
*/
				
				index = 0;
									
				while ( index < (*inPathLen) )
					{
					
						if ( (inPathStr[index] == '/') && (index > 0) && (inPathStr[index - 1] == '/') )
							{
							
								(*inPathLen)--;
								
								::BlockMoveData( &inPathStr[index], &inPathStr[index - 1], (*inPathLen) - index );
								
							}
									
						else index++;
									
					}

/*
**  Here we should do something about ':' chars but I don't
**  know what...so there's no code here.
*/
								
			}
			
		return result;
		
	}


pascal OSErr
UCVSUtils::CheckForMovement(			const FSSpec					*inTargetItem,
																	const FSSpec					*inProjectRoot,
																	CkidMcvsResourcePB		*inCkidMcvsInfo)
	{
	
		OSErr			result 									= noErr;
		UInt32		actualPathLen 					= 511,
							indicatedPathLen				= 511;	
		
		char			actualPathStr[512]			= "",
							indicatedPathStr[512] 	= "";
				
		if ( (inTargetItem != NULL) && (inProjectRoot != NULL) && (inCkidMcvsInfo != NULL) )
			{

				result = FSpGetLocalCVSPath( inTargetItem, inProjectRoot, actualPathStr, &actualPathLen );
				
				if ( result == noErr )
					{
					
						actualPathStr[actualPathLen] = 0x00;

/*
**  If the path is longer than 511 bytes, we're SOL.
*/
						
						if ( (inCkidMcvsInfo->mcvs.fRemotePathLen + inCkidMcvsInfo->mcvs.fFileNamePStr[0] + 1) >= 511 )
							
							result = kPathOverflow;
							
					}
				
				if ( result == noErr )
					{

/*
**  Manually concat string...
*/
					
						indicatedPathLen = inCkidMcvsInfo->mcvs.fRemotePathLen;
						
						::BlockMoveData( inCkidMcvsInfo->mcvs.fRemotePathStr, indicatedPathStr, indicatedPathLen );
						
						indicatedPathStr[indicatedPathLen] = '/';
						indicatedPathLen++;
						
						::BlockMoveData( &inCkidMcvsInfo->mcvs.fFileNamePStr[1], &indicatedPathStr[indicatedPathLen], inCkidMcvsInfo->mcvs.fFileNamePStr[0] );
						
						indicatedPathLen += inCkidMcvsInfo->mcvs.fFileNamePStr[0];
						
						indicatedPathStr[indicatedPathLen] = 0x00;
						
						if ( indicatedPathLen != actualPathLen )
							result = UCVSUtils::kFileHasBeenMoved;
							
						else if ( strncmp(actualPathStr, indicatedPathStr, indicatedPathLen) != 0 )
							result = UCVSUtils::kFileHasBeenMoved;
							
					}
										
			}
		else result = UCVSUtils::kInvalidParam;
			
		return result;
					
	}
	
	
pascal OSErr 
UCVSUtils::FSpGetPartialPath( 		const FSSpec 	*inTargetItem, 
																	const FSSpec	*inBaseItem, 
																	UInt32 				*inOutPathLength,
							   									char 					*outPathStr)

/*
**  FSpGetPartialPath returns the partial path between two FSSpecs. It DOES NOT
**  include the name of the base FSSpec in the path string so you will always
**  get partial pathnames. (ie., even if a volume is the base FSSpec)
*/

	{

		OSErr					result				= noErr;
		FSSpec				tempSpec			= { 0, 0, "\p" };
		Handle				tempPathHndl	= NULL;
		UInt16				copyLen				= 0x0000;
		Boolean				foundRoot			= false;
		CInfoPBRec		pb;
	
		if ( !FSpIsEqual(inTargetItem, inBaseItem) ) 
			{
		
				::BlockMoveData( inTargetItem, &tempSpec, sizeof(FSSpec) );
				
				if ( tempSpec.parID != fsRtParID ) 
					{
				
						result = ::PtrToHand( &tempSpec.name[1], &tempPathHndl, tempSpec.name[0] );
					
						if ( result == noErr ) 
							{
					
/* 
**  Get the ancestor directory names 
*/

								pb.dirInfo.ioCompletion = NULL;
								
								pb.dirInfo.ioNamePtr 	= tempSpec.name;
								pb.dirInfo.ioVRefNum 	= tempSpec.vRefNum;
								pb.dirInfo.ioDrParID 	= tempSpec.parID;

/* 
**  loop until we have an error or find the volume root directory
**  or find our base directory
*/
				
								while ( (result == noErr) && (pb.dirInfo.ioDrDirID != fsRtDirID) && (foundRoot == false) ) 
									{
									
										pb.dirInfo.ioFDirIndex 	= -1;
										pb.dirInfo.ioDrDirID 		= pb.dirInfo.ioDrParID;
										
										result = ::PBGetCatInfoSync(&pb);
					
										if ( result == noErr ) 
											{

												tempSpec.vRefNum = pb.dirInfo.ioVRefNum;
												tempSpec.parID = pb.dirInfo.ioDrParID;

												foundRoot = FSpIsEqual( &tempSpec, inBaseItem );

												if ( foundRoot != true ) 
													{

/* 
**  Append colon to directory name 
*/
														
														++tempSpec.name[0];
														tempSpec.name[tempSpec.name[0]] = ':';
													
														/* Add directory name to beginning of fullPath */
														(void) ::Munger(tempPathHndl, 0, NULL, 0, &tempSpec.name[1], tempSpec.name[0]);
														result = ::MemError();
												
													}
												
/*
**  If we found the base FSSpec, insert a ':' at the
**  beginning to signify the partial path.
*/

												else
													{
													
														tempSpec.name[1] = ':';
														tempSpec.name[0] = 1;
														
														(void) ::Munger(tempPathHndl, 0, NULL, 0, &tempSpec.name[1], tempSpec.name[0]);
														result = ::MemError();
													
													}
						
											}
					
									} 
				
								if ( (result == noErr) && (foundRoot == true) ) 
									{
				
/* 
**  Return the length 
*/

										copyLen = GetHandleSize(tempPathHndl);
					
										if ( copyLen > *inOutPathLength )
											copyLen = *inOutPathLength;

										::BlockMove( *tempPathHndl, outPathStr, copyLen );
										
										*inOutPathLength = copyLen;
					
									}
				
								else 
									{
				
										*inOutPathLength = 0;
										
										if ( !foundRoot )
											result = dirNFErr;
					
									}
				
								::DisposeHandle( tempPathHndl );

							}
		
					}
		
			}
			
		else
			{
			
				result = 1;
				
			}

		return 	result;
	
	}


pascal OSErr
UCVSUtils::FSpGetLocalCVSPath( 	const FSSpec 						*inTargetItem, 
																const FSSpec 						*inProjectRoot, 
																char 										*outPathStr,
																UInt32									*inOutPathLen)
	{
	
		OSErr		result = noErr;
		UInt32	bufLen = 255;
		
		if ( (inTargetItem != NULL) && (inProjectRoot != NULL) && (outPathStr != NULL) && (inOutPathLen != NULL) && ((*inOutPathLen) > 0) )
			{
				
				bufLen = (*inOutPathLen);
				(*inOutPathLen) = 0;
				
				result = FSpGetPartialPath( inTargetItem, inProjectRoot, &bufLen, (char *)outPathStr );
				
				if ( result == noErr )
					{
					
						(*inOutPathLen) = bufLen;
						
						if ( (*inOutPathLen) > 0 )
							{

/*
**  Convert ':'s to '/'s and remove the leading '/' if
**  there is one.
*/
							
								for ( UInt32 index = 0; index < (*inOutPathLen); index++ )
									if ( outPathStr[index] == ':' )
										outPathStr[index] = '/';
										
								if ( outPathStr[0] == '/' )
									{
									
										(*inOutPathLen)--;
										
										::BlockMoveData( &outPathStr[1], &outPathStr[0], (*inOutPathLen) );
									
									}
							
							}
					
					}
			
			
			}
		
		return result;
	
	}
	

pascal OSErr
UCVSUtils::GetFSpFromLocalCVSPath( const Str255 inCVSPathPStr, const FSSpec *inProjectRoot, FSSpec *outTargetItem )
	{
	
		OSErr					result 				= noErr;
		Str255				thePath				= "\p";
		CInfoPBRec		pb;
		SInt32				rootDirID			= 0x00000000;
		UInt32				offset				= 0x00000000;
		Boolean				lookingForDir	= false;
		
		if ( (inCVSPathPStr != NULL) && (inProjectRoot != NULL) && (outTargetItem != NULL) )
			{

				if ( inCVSPathPStr[0] > 0 )
				
					::BlockMoveData( inCVSPathPStr, thePath, inCVSPathPStr[0] + 1 );

				if ( thePath[0] > 0 )
					{

/*
**  Check for multiple '/' sequences.
*/
				
						for ( UInt32 index = 1; index <= thePath[0]; index++ )
							{
							
								if ( thePath[index] == '/' )
								if ( index > 1 )
								if ( thePath[index - 1] == '/' )
									result = kInvalidCVSPath;
							
							}

/*
**  Lop off beginning and ending '/'s.
*/
					
						if (result == noErr)
							{
							
								if ( thePath[thePath[0]] == '/' )
									{
									
										thePath[0]--;
										
										lookingForDir = true;
											
									}
																		
								for ( UInt32 index = 1; index <= thePath[0]; index++ )
									if ( thePath[index] == '/' )
										thePath[index] = ':';

/*
**  Insert ':' for partial pathname.
*/
	
								if ( (thePath[0] > 0) && (thePath[1] != ':') )
									{
										
										if ( thePath[0] < 255 )
											thePath[0]++;
											
										::BlockMoveData( &thePath[1], &thePath[2], thePath[0] - 1 );
										
										thePath[1] = ':';
									
									}
									
							}
						
					}
						
/*
**  If the dillhole user passed in an empty path string,
**  return the root.
*/
					
				if ( thePath[0] == 0 )
					{
					
						result = kInvalidCVSPath;
						
					}
				else
					{
							
						pb.hFileInfo.ioCompletion = NULL;
						pb.hFileInfo.ioFDirIndex 	= 0;
						pb.hFileInfo.ioNamePtr 		= (UInt8 *)inProjectRoot->name;
						pb.hFileInfo.ioDirID 			= inProjectRoot->parID;
						pb.hFileInfo.ioVRefNum		= inProjectRoot->vRefNum;
						
						result = ::PBGetCatInfoSync( &pb );
						
						if ( result == noErr )
							{
							
								pb.hFileInfo.ioNamePtr = thePath;

								result = ::PBGetCatInfoSync( &pb );
								
							}

/*
**  Uh oh, if the user specified a path ending with a '/',
**  then they were looking for a directory.
*/
							
						if ( result == noErr )
							if ( (pb.dirInfo.ioFlAttrib & 0x10) && (!lookingForDir) )
								result = kIsFolder;
								
						if ( result == noErr )
							{
							
								outTargetItem->vRefNum = pb.hFileInfo.ioVRefNum;
								outTargetItem->parID = pb.hFileInfo.ioFlParID;
							
								Boolean		done 			= false;
								UInt32		index 		= thePath[0],
													copyLen 	= 0x00000000;
	
/*
**  Strip away the partial pathname, leaving only
**  the filename in the resulting FSSpec.
*/
											
								while ( !done )
									{
										
										if ( index > 0 )
											{
										
												if ( thePath[index] == ':' )
												
													done = true;
												
												else index--;
												
											}
										else done = true;
											
									}
								
								copyLen = thePath[0] - index;
								
								if ( copyLen > 0 )
									::BlockMoveData( &thePath[index + 1], &outTargetItem->name[1], copyLen );
								
								outTargetItem->name[0] = copyLen; 
							
							}

/*
**  Make sure the case is the same.

Unfortunately this code doesn't work because we called PBGetCatInfo with a filename,
and it doesn't return one back with the correct case.

*/
	/*						
							
						if ( result == noErr )
							{
								Assert_( inCVSPathPStr[0] < outTargetItem->name[0] );
								
								if ( inCVSPathPStr[0] < outTargetItem->name[0] )
								
									result = kInvalidParam;
									
								else
									{
									
										offset = inCVSPathPStr[0] - outTargetItem->name[0];
										
										if ( (lookingForDir) && (offset > 0) )
											offset--;
											
										if ( strncmp( (char *)&outTargetItem->name[1], (char *)&inCVSPathPStr[offset], outTargetItem->name[0]) )
											result = dupFNErr;
											
									}
							}
*/							
						
					}
			
			}
			
		return result;
	
	}
	

pascal OSErr
UCVSUtils::FSpGetDirID(	const FSSpec						*inTargetItem,
												SInt32									*outDirID)
	{
	
		OSErr				result = noErr;
		CInfoPBRec	pb;
		
		if ( (inTargetItem != NULL) && (outDirID != NULL) )
			{

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetItem->name;
				pb.hFileInfo.ioDirID			= inTargetItem->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetItem->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					(*outDirID) = pb.hFileInfo.ioDirID;
						
			}
			
		return result;
	
	}
	
	
pascal OSErr
UCVSUtils::FSpIsAFile( const FSSpec *inTargetItem )
	{
	
		OSErr				result = noErr;
		CInfoPBRec	pb;
		
		if ( inTargetItem != NULL )
			{

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetItem->name;
				pb.hFileInfo.ioDirID			= inTargetItem->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetItem->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					if ( pb.dirInfo.ioFlAttrib & 0x10 )
						result = kIsFolder;
						
			}
			
		return result;
	
	}
	
	
pascal SInt32
UCVSUtils::FSpGetModDate( const FSSpec *inTargetItem )
	{
	
		SInt32			result = noErr;
		CInfoPBRec	pb;
		
		if ( inTargetItem != NULL )
			{

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= (StringPtr)inTargetItem->name;
				pb.hFileInfo.ioDirID			= inTargetItem->parID;
				pb.hFileInfo.ioVRefNum 		= inTargetItem->vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				
				result = ::PBGetCatInfoSync( &pb );
			
				if ( result == noErr )
					result = pb.hFileInfo.ioFlMdDat;
						
			}
			
		return result;
	
	}
	

pascal OSErr
UCVSUtils::FSpSetModDate( FSSpec *inTargetItem, SInt32 inModDate )
	{

		OSErr				result 			= noErr;
		CInfoPBRec 	pb;
		
		pb.hFileInfo.ioCompletion = nil;
		pb.hFileInfo.ioNamePtr 		= inTargetItem->name;
		pb.hFileInfo.ioVRefNum 		= inTargetItem->vRefNum;
		pb.hFileInfo.ioFDirIndex 	= 0;
		pb.hFileInfo.ioDirID 			= inTargetItem->parID;

		result = ::PBGetCatInfoSync( &pb );

		pb.hFileInfo.ioDirID = inTargetItem->parID;

		if ( pb.hFileInfo.ioFlCrDat > inModDate )
			pb.hFileInfo.ioFlCrDat = inModDate;
			
		pb.hFileInfo.ioFlMdDat = inModDate;

		result = ::PBSetCatInfoSync( &pb );

		return result;
		
	} 


pascal Boolean 
UCVSUtils::FSpIsEqual( const FSSpec *inItemOne, const FSSpec	*inItemTwo )
	{

		Boolean result = false;
		
		if ( (inItemOne != NULL) && (inItemTwo != NULL) ) 
			{
			
				if ( inItemOne->vRefNum == inItemTwo->vRefNum )
				if ( inItemOne->parID == inItemTwo->parID )
					result =  ::EqualString( inItemOne->name, inItemTwo->name, false, true );
				
			}
		
		return result;

	}


pascal Boolean
UCVSUtils::FSpIsChildOf( const FSSpec *inChildItem, const FSSpec *inParentItem )
	{

		OSErr				error			= noErr;
		Boolean			bResult 	= false,
								bFoundIt	= false;
	
		FSSpec			curItem		= { 0, 0, "\p" };
		CInfoPBRec	pb;
		
		if ( (inChildItem != NULL) && (inParentItem != NULL) )
			{
			
				if ( !FSpIsEqual(inChildItem,inParentItem) )
					{
					
						pb.hFileInfo.ioCompletion 	= NULL;
						pb.hFileInfo.ioFDirIndex 		= -1;
						pb.hFileInfo.ioNamePtr 			= curItem.name;
						pb.hFileInfo.ioVRefNum 			= inChildItem->vRefNum;
						pb.hFileInfo.ioDirID 				= inChildItem->parID;
					
						error = ::PBGetCatInfoSync( &pb );
						
						while ( (error == noErr) && (!bFoundIt) && (pb.hFileInfo.ioFlParID != fsRtDirID) )
							{
							
								curItem.vRefNum = pb.hFileInfo.ioVRefNum;
								curItem.parID = pb.hFileInfo.ioFlParID;
								
								bFoundIt = FSpIsEqual( &curItem, inParentItem );
							
								if ( !bFoundIt )
									{
									
										pb.hFileInfo.ioDirID = pb.hFileInfo.ioFlParID;
										
										error = ::PBGetCatInfoSync( &pb );
									
									}
										
							}
							
						if ( (error == noErr) && (!bFoundIt) )
							{
							
								curItem.vRefNum = pb.hFileInfo.ioVRefNum;
								curItem.parID = pb.hFileInfo.ioFlParID;
						
								bFoundIt = FSpIsEqual( &curItem, inParentItem );
								
							}

						if ( (error == noErr) && (bFoundIt) )
							{
							
								bResult = true;
							
							}
					
					}
			
			}
			
		return bResult;
		
	}
	
	
pascal Boolean 
UCVSUtils::IsRepositoryDir( const SInt16 inVRefNum, const SInt32 inParID )
	{

		OSErr				err							= noErr;
		Boolean			result 					= false;
		Str255			cvsFolderStr		= "\pCVS";
		CInfoPBRec	pb;
		
		
		pb.hFileInfo.ioCompletion 	= NULL;
		pb.hFileInfo.ioVRefNum 			= inVRefNum;
		pb.hFileInfo.ioNamePtr 			= cvsFolderStr;
		pb.hFileInfo.ioDirID 				= inParID;
		pb.hFileInfo.ioFDirIndex 		= 0;
		
		err = ::PBGetCatInfoSync( &pb );
		
		if ( err == noErr )
			result = true;
		
		return result;
					
	}
	

pascal Boolean
UCVSUtils::FSpIsRepositoryDir( const FSSpec *inTargetItem )
	{
	
		OSErr				error 	= noErr;
		Boolean			bResult	= false;
		CInfoPBRec	pb;
		
		if ( inTargetItem != NULL )
			{
			
				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= 0;
				pb.hFileInfo.ioNamePtr 			= (const_cast<FSSpec *>(inTargetItem))->name;
				pb.hFileInfo.ioVRefNum 			= inTargetItem->vRefNum;
				pb.hFileInfo.ioDirID 				= inTargetItem->parID;
				
				error = ::PBGetCatInfoSync( &pb );
				
				if ( error == noErr )
					{
					
						bResult = IsRepositoryDir( pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID );
					
					}
		
			}
				
		return bResult;
	
	}
	
		
pascal OSErr
UCVSUtils::CreateRepositoryDir( const SInt16 inVRefNum, const SInt32 inParID, const OSType inCreator, const OSType inType, const ScriptCode inScriptType )
	{
	
		OSErr		result			= noErr;
		FSSpec	tempSpec 		= { 0, 0, "\p" };
		SInt32	dirID				= 0x00000000;

		tempSpec.vRefNum 	= inVRefNum;
		tempSpec.parID 		= inParID;
		ConcatPStr( tempSpec.name, "\pCVS", sizeof(tempSpec.name) );

		result = ::FSpDirCreate( &tempSpec, smSystemScript, &dirID );

/*
**  Make the directory invisible. I couldn't get FSpSetFInfo() to work,
**  so I had to use PBSetCatInfoSync.
*/

		if ( result == noErr )
			{
				
				CInfoPBRec	cvsDirPB;
				
				cvsDirPB.hFileInfo.ioCompletion = NULL;
				cvsDirPB.hFileInfo.ioFDirIndex = 0;
				cvsDirPB.hFileInfo.ioNamePtr = tempSpec.name;
				cvsDirPB.hFileInfo.ioVRefNum = tempSpec.vRefNum;
				cvsDirPB.hFileInfo.ioDirID = tempSpec.parID;
				
				result = ::PBGetCatInfoSync( &cvsDirPB );
				
				if ( result == noErr )
					{
						
						cvsDirPB.hFileInfo.ioFlFndrInfo.fdFlags |= fInvisible;
						
						cvsDirPB.hFileInfo.ioDirID = tempSpec.parID;
						
						result = ::PBSetCatInfoSync( &cvsDirPB );
						
					}

			}
			
		if ( result == noErr )
			{
			
				tempSpec.parID = dirID;
				tempSpec.name[0] = 0x00;
				ConcatPStr( tempSpec.name, "\pTag", sizeof(tempSpec.name) );
				
				result = ::FSpCreate( &tempSpec, inCreator, inType, inScriptType );
				
			}
			
/*		

In the future we should support virtual folder mappings with the 
repository file.
	
			if ( result == noErr )
				{
	
					tempSpec.name[0] = 0x00;
					ConcatPStr( tempSpec.name, "\pRepository" );
					
					result = ::FSpCreate( &tempSpec, inCreator, inType, inScriptType );
				
				}
*/																
	
	}
	
	
	
pascal OSErr
UCVSUtils::FSWriteOffset( const SInt32 inFileRef, UInt32 *inOutCount, void *inData, const UInt32 inOffset )
	{
	
		OSErr						result 		= noErr;
		HParamBlockRec	pb;
		
		if ( (inFileRef > 0) && (inData != NULL) && (inOutCount != NULL) )
			{
			
				pb.ioParam.ioCompletion 	= NULL;
				pb.ioParam.ioVRefNum			= 0x00000000;
				pb.ioParam.ioRefNum				= inFileRef;
				pb.ioParam.ioBuffer 			= (char *)inData;
				pb.ioParam.ioReqCount 		= *inOutCount;
				pb.ioParam.ioPosMode 			= fsFromStart;
				pb.ioParam.ioPosOffset 		= inOffset;
						
				result = ::PBWriteSync( (ParmBlkPtr)&pb );
				
				*inOutCount = pb.ioParam.ioActCount;
				
			}
					
		else result = kInvalidParam;
		
		return result;
	
	}


pascal OSErr
UCVSUtils::FSReadOffset( const SInt32 inFileRef, UInt32 *inOutCount, void *inData, const UInt32 inOffset )
	{
	
		OSErr						result 		= noErr;
		HParamBlockRec	pb;
		
		if ( (inFileRef > 0) && (inData != NULL) && (inOutCount != NULL) )
			{
			
				pb.ioParam.ioCompletion 	= NULL;
				pb.ioParam.ioVRefNum			= 0x00000000;
				pb.ioParam.ioRefNum				= inFileRef;
				pb.ioParam.ioBuffer 			= (char *)inData;
				pb.ioParam.ioReqCount 		= *inOutCount;
				pb.ioParam.ioPosMode 			= fsFromStart;
				pb.ioParam.ioPosOffset 		= inOffset;
						
				result = ::PBReadSync( (ParmBlkPtr)&pb );
				
				*inOutCount = pb.ioParam.ioActCount;
				
			}
					
		else result = kInvalidParam;
		
		return result;
	
	}

	
/*
pascal OSErr
UCVSUtils::FSpWriteRF(const FSSpec *inTargetItem, char *inData, const UInt32 inDataLen, const UInt32 inOffset  )
	{
	
		OSErr						result 		= noErr;
		UInt32					writeSize = inDataLen;
		SInt16					fileRef		= 0x0000;
		HParamBlockRec	pb;
		
		if ( inTargetItem != NULL )
			{
			
				if ( inData == NULL )
					writeSize = 0;
					
				pb.ioParam.ioCompletion = NULL;
				pb.ioParam.ioPermssn = fsRdWrPerm;
				pb.fileParam.ioDirID = ((FSSpec *)inTargetItem)->parID;
				pb.ioParam.ioNamePtr = ((FSSpec *)inTargetItem)->name;
				pb.ioParam.ioVRefNum = ((FSSpec *)inTargetItem)->vRefNum;
												
				result = ::PBHOpenRFSync( &pb );
				
				if ( result == noErr )
					{
													
						fileRef = pb.ioParam.ioRefNum;
												
						result = ::SetEOF( fileRef, writeSize );
												
					}
					
				if ( (result == noErr) && (writeSize > 0) )
					{													
	
						pb.ioParam.ioBuffer 			= inData;
						pb.ioParam.ioReqCount 		= writeSize;
						pb.ioParam.ioPosMode 			= fsFromStart;
						pb.ioParam.ioPosOffset		= inOffset;
						
						result = ::PBWriteSync( (ParmBlkPtr)&pb );
				
					}
					
				if ( fileRef > 0 )
					{
					
						::PBCloseSync( (ParmBlkPtr)&pb ); 
					
					}
			
			}
		else result = kInvalidParam;
		
		return result;
	
	}
*/	

/*
OSErr
UCVSUtils::FSpEncodeAppleSingle( FSSpec *inTargetItem, Handle *outDataHndl )
	{
	
		OSErr		result 			= noErr;
		SInt16	fileRef 		= 0x0000;
		SInt32	dataLen			= 0x00000000;
		Handle	encodedData	= NULL;
		UInt32	curOffset		= 0x00000000;
		FInfo		finderInfo;
		
		*outDataHndl = NULL;
		
		encodedData = ::NewHandle( sizeof(AppleSingleHeader) );
		
		result = ::MemError();
		
		if ( (encodedData == NULL) && (result == noErr) )
			result = -108;
			
		if ( result == noErr )
			{
				
				((AppleSingleHeader *)*encodedData)->fMagicNumber 	= kAppleSingleMagicNumber;
				((AppleSingleHeader *)*encodedData)->fVersion 			= kAppleSingleVersion;
				((AppleSingleHeader *)*encodedData)->fFiller1 			= 0x00000000;
				((AppleSingleHeader *)*encodedData)->fFiller2 			= 0x00000000;
				((AppleSingleHeader *)*encodedData)->fFiller3 			= 0x00000000;
				((AppleSingleHeader *)*encodedData)->fFiller4 			= 0x00000000;
				((AppleSingleHeader *)*encodedData)->fEntries				= 0x0000;
			
			}

/*
**  Write Finder info...
*/
/*
		if ( result == noErr )
			{
			
				result = ::FSpGetFInfo( inTargetItem, &finderInfo );
				
				if ( result == noErr )
					{
					
						curOffset = ::GetHandleSize( encodedData );
						
						::SetHandleSize( encodedData, curOffset + sizeof(AppleSingleEntry) + sizeof(FInfo) );
						
						result = ::MemError();
						
						if ( result == noErr )
							{

								((AppleSingleHeader *)*encodedData)->fEntries++;
							
								((AppleSingleEntry *)((*encodedData) + curOffset))->fEntryID = 9;
								((AppleSingleEntry *)((*encodedData) + curOffset))->fOffset = curOffset + sizeof(AppleSingleEntry);
								((AppleSingleEntry *)((*encodedData) + curOffset))->fLength = sizeof(FInfo);
								
								::BlockMoveData( &finderInfo, ((*encodedData) + curOffset + sizeof(AppleSingleEntry)), sizeof(FInfo) );
								
							}
					
					}
				
			}
			
			
/*
**  Write data fork entry.
*/
/*
		if ( result == noErr )
			result = ::FSpOpenDF( inTargetItem, fsRdPerm, &fileRef );
		
		if ( (result == noErr) && (fileRef > 0) )
			{
			
				::GetEOF( fileRef, &dataLen );
				
				if ( dataLen > 0 )
					{
					
						curOffset = ::GetHandleSize( encodedData );
						
						::SetHandleSize( encodedData, curOffset + sizeof(AppleSingleEntry) + dataLen );
						
						result = ::MemError();
						
						if ( result == noErr )
							{

								((AppleSingleHeader *)*encodedData)->fEntries++;
							
								((AppleSingleEntry *)((*encodedData) + curOffset))->fEntryID = 1;
								((AppleSingleEntry *)((*encodedData) + curOffset))->fOffset = curOffset + sizeof(AppleSingleEntry);
								((AppleSingleEntry *)((*encodedData) + curOffset))->fLength = dataLen;
								
								::HLock( encodedData );
								
								result = ::FSRead( fileRef, &dataLen, ((*encodedData) + curOffset + sizeof(AppleSingleEntry)) );
							
								::HUnlock( encodedData );
								
							}
					
					}
					
				::FSClose( fileRef );
				fileRef = 0;
			
			}

/*
**  Write resource fork entry.
*/
/*
		if ( result == noErr )
			result = ::FSpOpenRF( inTargetItem, fsRdPerm, &fileRef );
		
		if ( (result == noErr) && (fileRef > 0) )
			{
			
				::GetEOF( fileRef, &dataLen );
				
				if ( dataLen > 0 )
					{
					
						curOffset = ::GetHandleSize( encodedData );
						
						::SetHandleSize( encodedData, curOffset + sizeof(AppleSingleEntry) + dataLen );
						
						result = ::MemError();
						
						if ( result == noErr )
							{

								((AppleSingleHeader *)*encodedData)->fEntries++;
							
								((AppleSingleEntry *)((*encodedData) + curOffset))->fEntryID = 2;
								((AppleSingleEntry *)((*encodedData) + curOffset))->fOffset = curOffset + sizeof(AppleSingleEntry);
								((AppleSingleEntry *)((*encodedData) + curOffset))->fLength = dataLen;
								
								::HLock( encodedData );
								
								result = ::FSRead( fileRef, &dataLen, ((*encodedData) + curOffset + sizeof(AppleSingleEntry)) );
							
								::HUnlock( encodedData );
								
							}
					
					}
					
				::FSClose( fileRef );
				fileRef = 0;
			
			}
			
		if ( result == noErr )
			{
			
				*outDataHndl = encodedData;
				
			}
		else if ( encodedData != NULL )
			{
			
				::DisposeHandle( encodedData );
				
			}
			
		return result;
	
	}
*/	

OSErr
UCVSUtils::FSpGetRepository( const FSSpec *inTargetItem, Str255 outRepositoryPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0x0000;
		SInt32			ioCount		= 255;
		Str255			pathPStr	= "\p";

		FSSpec			fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetItem != NULL) && (outRepositoryPStr != NULL) )
			{
			
				ConcatPStr( pathPStr, inTargetItem->name );
				ConcatPStr( pathPStr, "\p:CVS:Repository" );
				
				result = ::FSMakeFSSpec( inTargetItem->vRefNum, inTargetItem->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{
					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )
							
							fileRef = 0;
							
						else
							{
						
								result = ::GetEOF( fileRef, &ioCount );
						
								if ( result == noErr )
									if ( ioCount > 255 )
										ioCount = 255;
								
								if ( result == noErr )
									{
									
										result = ::FSRead( fileRef, &ioCount, &outRepositoryPStr[1] );
									
										if ( result == noErr )
											outRepositoryPStr[0] = ioCount;

									}
							
								::FSClose( fileRef );
								fileRef = 0;
						
							}
												
					}
					
				if ( result != noErr )
					outRepositoryPStr[0] = 0x00;
			
			}
		else result = kInvalidParam;
		
		return result;
		
	}
	

OSErr
UCVSUtils::FSpSetRepository( const FSSpec *inTargetItem, Str255 inRepositoryPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0x0000;
		SInt32			ioCount		= 0x00000000;
		Str255			pathPStr	= "\p";

		FSSpec			fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetItem != NULL) && (inRepositoryPStr != NULL) )
			{
			
				ConcatPStr( pathPStr, inTargetItem->name );
				ConcatPStr( pathPStr, "\p:CVS:Repository" );
				
				result = ::FSMakeFSSpec( inTargetItem->vRefNum, inTargetItem->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{
					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )
							
							fileRef = 0;
							
						else
							{
							
								ioCount = inRepositoryPStr[0];
								
								result = ::SetEOF( fileRef, ioCount );
								
								if ( result == noErr )
									result = ::FSWrite( fileRef, &ioCount, &inRepositoryPStr[1] );

								::FSClose( fileRef );
								fileRef = 0;
														
							}
					
					}
			
			}
			
		return result;
		
	}
	

OSErr
UCVSUtils::FSpGetFolderTag( const FSSpec *inTargetItem, Str255 outTagPStr )
	{
	
		OSErr				result 				= noErr;
		SInt16			fileRef				= 0x0000;
		SInt32			ioCount				= 255;
		FSSpec			targetTag 		= { 0, 0, "\p" };
		
		if ( (inTargetItem != NULL) && (outTagPStr != NULL) )
			{
			
				targetTag.vRefNum = inTargetItem->vRefNum;
				targetTag.parID = inTargetItem->parID;
				
				ConcatPStr( targetTag.name, "\p:", sizeof(targetTag.name) );
				ConcatPStr( targetTag.name, inTargetItem->name, sizeof(targetTag.name) );
				ConcatPStr( targetTag.name, "\p:CVS:Tag", sizeof(targetTag.name) );
						
				result = ::FSpOpenDF( &targetTag, fsRdWrPerm, &fileRef );
						
				if ( result != noErr )
				
					fileRef = 0;
					
				else
					{
					
						result = ::GetEOF( fileRef, &ioCount );

						if ( result == noErr )
							if ( ioCount > 255 )
								ioCount = 255;
						
						if ( (result == noErr) && (ioCount > 0) )
							result = ::FSRead( fileRef, &ioCount, &outTagPStr[1] );
					
						if ( result == noErr )
							outTagPStr[0] = ioCount;
							
						::FSClose( fileRef );
								
						fileRef = 0;
			
					}
			
			}
		else result = kInvalidParam;
			
		return result;
	
	}
	
	
OSErr
UCVSUtils::FSpGetParentTag( const FSSpec *inTargetItem, Str255 outTagPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0x0000;
		SInt32			ioCount		= 255;
		Str255			pathPStr	= "\p";
		FSSpec			fileSpec	= { 0, 0, "\p" };

		if ( (inTargetItem != NULL) && (outTagPStr != NULL) )
			{
					
				ConcatPStr( pathPStr, "\p:CVS:Tag" );
				
				result = ::FSMakeFSSpec( inTargetItem->vRefNum, inTargetItem->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{
					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )
						
							fileRef = 0;
							
						else
							{
							
								result = ::GetEOF( fileRef, &ioCount );

								if ( result == noErr )
									if ( ioCount > 255 )
										ioCount = 255;
								
								if ( (result == noErr) && (ioCount > 0) )
									result = ::FSRead( fileRef, &ioCount, &outTagPStr[1] );
							
								if ( result == noErr )
									outTagPStr[0] = ioCount;
									
								::FSClose( fileRef );
								fileRef = 0;
					
							}
										
					}
					
			}
		else result = kInvalidParam;
			
		return result;
		
	}
	


OSErr
UCVSUtils::FSpSetParentTag( const FSSpec *inTargetItem, Str255 inTagPStr )
	{
	
		OSErr				result 		= noErr;
		SInt16			fileRef		= 0x0000;
		SInt32			ioCount		= 0x00000000;
		Str255			pathPStr	= "\p";
		FSSpec			fileSpec	= { 0, 0, "\p" };
		
		if ( (inTargetItem != NULL) && (inTagPStr != NULL) )
			{
			
				ConcatPStr( pathPStr, "\p:CVS:Tag" );
				
				result = ::FSMakeFSSpec( inTargetItem->vRefNum, inTargetItem->parID, pathPStr, &fileSpec );
				
				if ( result == noErr )
					{
					
						result = ::FSpOpenDF( &fileSpec, fsRdWrPerm, &fileRef );
						
						if ( result != noErr )
						
							fileRef = 0;
							
						else
							{
								
								result = ::SetEOF( fileRef, inTagPStr[0] );
								
								if ( result == noErr )
									{
									
										ioCount = inTagPStr[0];
								
										if ( ioCount > 0 )
											result = ::FSWrite( fileRef, &ioCount, &inTagPStr[1] );
								
									}
					
								::FSClose( fileRef );
								fileRef = 0;
													
							}
										
					}
			
			}
		else result = kInvalidParam;
		
		return result;
		
	}

	
OSErr
UCVSUtils::FSpGetFullPathStr( const FSSpec *inTargetItem, UInt32 *inOutPathLen, char *outPathStr )
{

	OSErr		result			= noErr;
	FSSpec		tempSpec;
	Handle		tempPathHndl	= NULL;
	CInfoPBRec	pb;
	UInt16		copyLen			= 0x0000;
		
	/* Make a copy of the input FSSpec that can be modified */
	BlockMoveData(inTargetItem, &tempSpec, sizeof(FSSpec));
	
	if ( tempSpec.parID == fsRtParID ) {
	
		/* The object is a volume */
		
		/* Add a colon to make it a full pathname */
		++tempSpec.name[0];
		tempSpec.name[tempSpec.name[0]] = ':';
		
		copyLen = tempSpec.name[0];
		
		if (copyLen > *inOutPathLen)
			copyLen = *inOutPathLen;
			
		/* We're done */
		BlockMove(&tempSpec.name[1], outPathStr, copyLen);
		
		*inOutPathLen = copyLen;
		
	}
	else {
	
		/* The object isn't a volume */
		
		/* Put the object name in first */
		
		result = PtrToHand(&tempSpec.name[1], &tempPathHndl, tempSpec.name[0]);
		
		if (result == noErr) {
		
			/* Get the ancestor directory names */
			pb.dirInfo.ioNamePtr = tempSpec.name;
			pb.dirInfo.ioVRefNum = tempSpec.vRefNum;
			pb.dirInfo.ioDrParID = tempSpec.parID;
			
			do	/* loop until we have an error or find the root directory */
			{
				pb.dirInfo.ioFDirIndex = -1;
				pb.dirInfo.ioDrDirID = pb.dirInfo.ioDrParID;
				
				result = PBGetCatInfoSync(&pb);
				
				if (result == noErr) {
				
					/* Append colon to directory name */
					++tempSpec.name[0];
					tempSpec.name[tempSpec.name[0]] = ':';
					
					/* Add directory name to beginning of fullPath */
					(void) Munger(tempPathHndl, 0, NULL, 0, &tempSpec.name[1], tempSpec.name[0]);
					result = MemError();
					
				}
				
			} while ( (result == noErr) && (pb.dirInfo.ioDrDirID != fsRtDirID) );
			
			if (result == noErr) {
			
				/* Return the length */
				copyLen = GetHandleSize(tempPathHndl);
				
				if (copyLen > *inOutPathLen)
					copyLen = *inOutPathLen;

				BlockMove(*tempPathHndl, outPathStr, copyLen);
				
				*inOutPathLen = copyLen;
				
			}
			
			else {
			
				*inOutPathLen = 0;
				
			}
			
			DisposeHandle(tempPathHndl);

		}
		
	}
	
	return 	result;
	
}


pascal void
UCVSUtils::Rot15PStr( Str255 inOutCryptStr )
	{

		UInt32	len = 0x00;
		
		if ( inOutCryptStr != NULL ) 
			{
			
				len = inOutCryptStr[0];
				
				for ( UInt32 index = 1; index <= len; index++ )
					{
					
						inOutCryptStr[index] += 15;
						
					}
					
			}
		
	}
	
	
pascal void
UCVSUtils::ReverseRot15PStr( Str255 inOutCryptStr )
	{

		UInt32	len = 0x00;
		
		if ( inOutCryptStr != NULL ) 
			{
			
				len = inOutCryptStr[0];
				
				for ( UInt32 index = 1; index <= len; index++ )
					{
					
						inOutCryptStr[index] -= 15;
						
					}
					
			}
		
	}
	
	
								
void		
UCVSUtils::UInt32ToCStrAtomic( 	char							*outStr,
																UInt32						inNum,
																UInt32						*inOutIndex,
																const UInt32			inLen)
	{
		
		const		asciiOffset	= 0x30;
		UInt16		digit;
		
		if ((inNum / 10) > 0) 
			{
			
				digit = inNum % 10;
				
				inNum = inNum / 10;
				
				UInt32ToCStrAtomic(outStr, inNum, inOutIndex, inLen);
				
			}
		else digit = inNum;
		
		if ((*inOutIndex) < inLen) 
			{
			
				outStr[(*inOutIndex)] = digit + asciiOffset;
			
				(*inOutIndex)++;
				
			}

	}


void		
UCVSUtils::UInt32ToStr255(		UInt32							inNum,
															Str255							outStr)
{

	UInt32	len	= 255;
/*
**  This will leave the terminating byte in the string,
**  but since it is impossible to have a number 255 chars.
**  long, it should not matter.
*/
	UInt32ToCStr(inNum, (char *)&outStr[1], &len);
	
	outStr[0] = len;

}


void 		
UCVSUtils::UInt32ToCStr(			UInt32 							inNum, 
															char 								*outStr, 
															UInt32 							*inOutLen)
{
	
	UInt32	inLen	= (*inOutLen);
	
	(*inOutLen) = 0;
	
	if (inLen > 0) {
	
		UInt32ToCStrAtomic(outStr, inNum, inOutLen, inLen - 1);
	
		outStr[(*inOutLen)] = 0x00;
		
	}

}


void		
UCVSUtils::CStrToUInt32(			register char				*inStr,
															register UInt32			*outNum)
{

	Boolean		done = false;
	
	*outNum = 0x00000000;
	
	while (!done) 
		{

			if (((*inStr) >= 0x30) && ((*inStr) <= 0x39)) 
				{
				
					(*outNum) = (*outNum) * 10;
					
					(*outNum) = (*outNum) + ((*inStr) - 0x30); 
				
					inStr++;
				
				} else done = true;

		}
	
}


char 
UCVSUtils::ReadWord( 	const char 			*inBuf, 
											const UInt32 		inBufLen, 
											UInt32 					*inOutIndex, 
											char 						*outWordStr, 
											UInt32 					*inOutWordLen, 
											const char 			*inTerm)
	{
	
		Boolean		done						= false;
		
		char			*curTerminator	= NULL,
							result					= 0x00;
		
		UInt32		len							= 0x00000000;
		
		if ( 	(inBuf != NULL) && 
					(inBufLen > 0) && 
					(inOutIndex != NULL) && 
					(outWordStr != NULL) && 
					(inOutWordLen != NULL) && 
					(inTerm != NULL) && 
					(strlen(inTerm) > 0) ) 
			{
		
				while ( !done ) 
					{
						
						if ( ((*inOutIndex) < inBufLen) && (len < (*inOutWordLen)) ) 
							{
								
								for ( curTerminator = (char *)inTerm; *curTerminator != '\0'; curTerminator++ ) 
									{
									
										if ( inBuf[(*inOutIndex)] == *curTerminator)
											{

												result = *curTerminator;

												done = true;
												
											} 
										
									}
											
								if ( !done ) 
								
									outWordStr[len++] = inBuf[(*inOutIndex)];
						
								(*inOutIndex)++;
																					
							} 
							
						else done = true;
						
					}
				
				*inOutWordLen = len;
			
			} 
		else inOutWordLen = 0;
		
		return result;

	}
	
	
