//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				UErrorDialog.cp
	
	Authors:		Dan Clifford
	
	Description:
	
		This file contains code to implement a simple error dialog.
		 
	Modification History:

		7 July	1998		Ram Rajadhyaksha
		
					Removed #include so this will compile with CW Pro 3. Changed 'okok'
					constant to 'msg_OK' for standardization.
							
-------------------------------------------------------------------------------- */


#include <stdio.h>
#include <string.h>

#include <LCicnButton.h>

#include <LStdControl.h>
#include <Menus.h>
#include <UMemoryMgr.h>

#include "CThreadedTask.h"
#include "UErrorDialog.h"



UOKAlertDialog::UOKAlertDialog( const SInt16 inStrListRsrcID, const SInt16 inStrIndex, const SInt16 inIconID, LCommander *inSuperCommander )
	{
		
		Str255	textStr = "\p";
		
		::GetIndString( textStr, inStrListRsrcID, inStrIndex );
		
		if ( textStr[0] == 0 )
			{
				
				char errorBuffer[256];
				
				sprintf( errorBuffer + 1, "Could not complete the previous operation because an error occurred (%d).", inStrIndex );
				errorBuffer[0] = strlen( errorBuffer + 1 );
				
				InitFromString( (unsigned char *)errorBuffer, inIconID, inSuperCommander );

			}

		else
			{
			
				InitFromString( textStr, inIconID, inSuperCommander );
				
			}	

	}


UOKAlertDialog::UOKAlertDialog(	StringPtr	inMessagePStr, const SInt16 inIconID, LCommander	*inSuperCommander)
	{
	
		InitFromString( inMessagePStr, inIconID, inSuperCommander );

	}


UOKAlertDialog::~UOKAlertDialog()
	{


	}


void
UOKAlertDialog::InitFromString(	StringPtr	inMessagePStr, const SInt16 inIconID, LCommander *inSuperCommander )
	{

		//	Create the dialog and the standard handler
		
		StDialogHandler	dialogHandler( kOKAlertDialogPPobID, inSuperCommander );
		dialogHandler.AddAttachment( new CThreadedTaskYieldAttachment );

		LDialogBox *dialogBox = dynamic_cast<LDialogBox *>( dialogHandler.GetDialog() );
		ThrowIfNil_( dialogBox );
		
		LCaption	*textField = dynamic_cast<LCaption *>( dialogBox->FindPaneByID(kOKAlertCaptionID) );
		ThrowIfNil_( textField );	

		LCicnButton	*cicnButton = dynamic_cast<LCicnButton *>(dialogBox->FindPaneByID( kOKAlertIconPaneID) );
		ThrowIfNil_( cicnButton );
		
		textField->SetDescriptor( inMessagePStr );
		cicnButton->SetCicns( inIconID, inIconID );

		::SysBeep( 10 );
		
		dialogBox->Show();
		
		Boolean		done = false;
		
		while ( !done )
			{
			
				switch ( dialogHandler.DoDialog() )
					{ 
						
						case msg_OK:
							
							done = true;
						
						break;
					
					}
			
			}
		
	}


UErrorDialog::UErrorDialog(	OSErr inError, LCommander	*inSuperCommander )
	{
		
		Handle textString = NULL;
		
		textString = ::GetResource( 'STR ', inError );
		
		if ( textString == NULL )
			{
				
				char errorBuffer[256];
				
				sprintf( errorBuffer + 1, "Could not complete the previous operation because an error occurred (%d).", inError );
				errorBuffer[0] = strlen( errorBuffer + 1 );
				
				InitFromString( (unsigned char *)errorBuffer, inSuperCommander );

			}

		else
			{
			
				StHandleLocker	stringLocker( textString );

				InitFromString((unsigned char *)*textString, inSuperCommander);
			
				::ReleaseResource( textString );
				
			}	

	}


UErrorDialog::UErrorDialog(	StringPtr	inErrorStr, LCommander *inSuperCommander )
	{
	
		InitFromString(inErrorStr, inSuperCommander);
	
	}


UErrorDialog::~UErrorDialog()
	{

	}


void
UErrorDialog::InitFromString(	StringPtr	inErrorStr, LCommander *commander )
	{
		
		//	Create the dialog and the standard handler
		
		StDialogHandler		dialogHandler(kErrorDialogPPobID, commander);
		dialogHandler.AddAttachment( new CThreadedTaskYieldAttachment );

		LDialogBox *dialogBox = (LDialogBox *)dialogHandler.GetDialog();		
		LCaption *textField = (LCaption *)dialogBox->FindPaneByID('msg ');	

		textField->SetDescriptor( inErrorStr );
			
		::SysBeep( 10 );
		
		dialogBox->Show();
		
		Boolean	done = false;
		
		while (!done)
			{
			
				switch (dialogHandler.DoDialog())
					{ 
					
						case msg_OK:
				
							done = true;
					
						break;

						default:
						
						break;		
					
					}
			
			}
		
	}



