//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//
//		File: 		UKeyScrollAttachments.cp
//	
//	  Authors: 	Simon Fraser & Patrick Beard
//
//################################################################################

#include "UKeyScrollAttachments.h"


#include "CMacCVSProjectView.h"
#include "CMacCVSLurkersView.h"

inline void UppercaseString(LString& str)
{
	::UppercaseText((Ptr) &str[1], str[0], smSystemScript);
}

//
// Basic table key attachment for handling arrow keys and (eventually) type-ahead
//

UTableKeyScrollAttachment::UTableKeyScrollAttachment(LView		*inViewToScroll):
	LKeyScrollAttachment( inViewToScroll )
{
	mTableView = (LTableView *)inViewToScroll;
	mKeyBuffer[0] = 0;
	mLastKeyTime = 0;
	mCurrentRow = 0;
}


UTableKeyScrollAttachment::UTableKeyScrollAttachment(LStream	*inStream):
	LKeyScrollAttachment( inStream)
{
	mKeyBuffer[0] = 0;
	mLastKeyTime = 0;
	mCurrentRow = 0;
}

void
UTableKeyScrollAttachment::ExecuteSelf( MessageT inMessage, void		*ioParam)
{
	
	EventRecord			*inEvent = (EventRecord *)ioParam;
	Int16						theKey = inEvent->message & charCodeMask;
	TableIndexT			numTableRows, numTableCols;
	Boolean					shiftKeyDown = (inEvent->modifiers & shiftKey) != 0;
	Boolean					cmdKeyDown = (inEvent->modifiers & cmdKey) != 0;
	
	mTableView->GetTableSize(numTableRows, numTableCols);
	
	// don't attempt anything with empty tables
	if ( numTableRows == 0)
	{
			LKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
			return;
	}
	
	switch ( theKey )
	{
		case char_UpArrow:
			
			if ( ! cmdKeyDown )		//don't handle modifiers yet
			{
				STableCell		theSelectedCell;
			
				theSelectedCell = mTableView->GetFirstSelectedCell();
				
				// no selection yet
				if ( theSelectedCell.IsNullCell() )
				{
					// select the last cell
					theSelectedCell.row = numTableRows;
					theSelectedCell.col = 1;
					mTableView->SelectCell(theSelectedCell);
					mTableView->SelectionChanged();
					mTableView->ScrollCellIntoFrame(theSelectedCell);
				}
				else
				{
					// select the cell before the first one
					theSelectedCell = mTableView->GetFirstSelectedCell();
					
					if ( theSelectedCell.row > 1 )
					{
						if (! shiftKeyDown)
							mTableView->UnselectAllCells();
						
						theSelectedCell.row --;
						mTableView->SelectCell(theSelectedCell);
						mTableView->SelectionChanged();
						mTableView->ScrollCellIntoFrame(theSelectedCell);
					}
				
				}
			}
		
			mExecuteHost = false;		// we will handle it
			break;
		
		case char_DownArrow:
		
			if ( ! cmdKeyDown )		//don't handle modifiers yet
			{
				STableCell		theSelectedCell;
			
				theSelectedCell = mTableView->GetFirstSelectedCell();
				
				// no selection yet
				if ( theSelectedCell.IsNullCell() )
				{
					// select the first cell
					theSelectedCell.row = 1;
					theSelectedCell.col = 1;
					mTableView->SelectCell(theSelectedCell);
					mTableView->SelectionChanged();
					mTableView->ScrollCellIntoFrame(theSelectedCell);
				}
				else
				{
					// select the cell after the last one
					theSelectedCell.SetCell(0, 0);
					mTableView->GetPreviousSelectedCell(theSelectedCell);		//gets last selected cell
					
					if ( theSelectedCell.row < numTableRows )
					{
						if (! shiftKeyDown)
							mTableView->UnselectAllCells();
						
						theSelectedCell.row ++;
						mTableView->SelectCell(theSelectedCell);
						mTableView->SelectionChanged();
						mTableView->ScrollCellIntoFrame(theSelectedCell);
					}
				
				}
			}
			
			mExecuteHost = false;		// we will handle it
			break;
		
		case char_Enter:
		case char_Return:
		
			{
				// Open all selected cells appropriately.
				STableCell theSelectedCell(0, 0);
				while (mTableView->GetNextSelectedCell(theSelectedCell))
					OpenItem(theSelectedCell);
			}
			mExecuteHost = false;		// we will handle it
			break;
		
		case char_Backspace:
			mExecuteHost = true;
			break;
			
		default:
		
			// Let superclass filter pageUp/pageDown, etc. first.
			mExecuteHost = true;
			LKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
			if (mExecuteHost && !cmdKeyDown)
			{
				mExecuteHost = false;
				// Allow keyboard selection of file names.
				// Reset the typing buffer if this keystroke comes in after a double-click interval.
				if (theKey != char_Tab) {
					if ((inEvent->when - mLastKeyTime) > LMGetDoubleTime()) {
						mKeyBuffer[0] = 0;
						mCurrentRow = 1;
					}
					mLastKeyTime = inEvent->when;
					mKeyBuffer += char(theKey);
					UppercaseString(mKeyBuffer);
					SelectPrefixedItem(mKeyBuffer, mCurrentRow);
				} else if (mCurrentRow > 0) {
					// Start searching for a prefix after the current row.
					++mCurrentRow;
					if (!SelectPrefixedItem(mKeyBuffer, mCurrentRow)) {
						mCurrentRow = 1;
						SelectPrefixedItem(mKeyBuffer, mCurrentRow);
					}
				}
			}
			break;
	}
	
}

#pragma mark -

//
// Attachment for the compare view
//

USimpleKeyScrollAttachment::USimpleKeyScrollAttachment(LView		*inViewToScroll):
	UTableKeyScrollAttachment( inViewToScroll )
{
}


USimpleKeyScrollAttachment::USimpleKeyScrollAttachment(LStream	*inStream):
	UTableKeyScrollAttachment( inStream)
{
}


void
USimpleKeyScrollAttachment::ExecuteSelf(MessageT inMessage, void *ioParam)
{
	UTableKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
}


#pragma mark -

//
// Attachment for the lurkers view
//

ULurkersKeyScrollAttachment::ULurkersKeyScrollAttachment(LView		*inViewToScroll):
	UTableKeyScrollAttachment( inViewToScroll )
{
}


ULurkersKeyScrollAttachment::ULurkersKeyScrollAttachment(LStream	*inStream):
	UTableKeyScrollAttachment( inStream)
{
}

void
ULurkersKeyScrollAttachment::ExecuteSelf( MessageT inMessage, void		*ioParam)
{
	EventRecord 	*inEvent = (EventRecord *)ioParam;
	Boolean				cmdKeyDown = (inEvent->modifiers & cmdKey) != 0;
	Int16 				theKey = inEvent->message & charCodeMask;
	
	if (cmdKeyDown)
	{
			LKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
			return;
	}
	
	if (theKey == char_Tab) {
		// Go to the next item alphabetically in the list (previous if shift-key held down).
		// 1. need a sorted representation of the list of items.
		// 2. have to convert the list item to a table cell index.

		TableIndexT numTableRows, numTableCols;
		Boolean shiftKeyDown = (inEvent->modifiers & shiftKey) != 0;
		
		mTableView->GetTableSize(numTableRows, numTableCols);
		STableCell theSelectedCell = mTableView->GetFirstSelectedCell();
		
		if (shiftKeyDown) {
			// no selection yet
			if ( theSelectedCell.IsNullCell() )	{
				// select the last cell
				theSelectedCell.row = numTableRows;
				theSelectedCell.col = 1;
				mTableView->SelectCell(theSelectedCell);
				mTableView->SelectionChanged();
				mTableView->ScrollCellIntoFrame(theSelectedCell);
			} else {
				// select the cell before the first one
				theSelectedCell = mTableView->GetFirstSelectedCell();
				
				if ( theSelectedCell.row > 1 ) {
					mTableView->UnselectAllCells();
					
					theSelectedCell.row --;
					mTableView->SelectCell(theSelectedCell);
					mTableView->SelectionChanged();
					mTableView->ScrollCellIntoFrame(theSelectedCell);
				}
			}
		} else {
			// no selection yet
			if (theSelectedCell.IsNullCell()) {
				// select the first cell
				theSelectedCell.row = 1;
				theSelectedCell.col = 1;
				mTableView->SelectCell(theSelectedCell);
				mTableView->SelectionChanged();
				mTableView->ScrollCellIntoFrame(theSelectedCell);
			} else {
				// select the cell after the last one
				theSelectedCell.SetCell(0, 0);
				mTableView->GetPreviousSelectedCell(theSelectedCell);		// gets last selected cell
				
				if ( theSelectedCell.row < numTableRows ) {
					mTableView->UnselectAllCells();
					
					theSelectedCell.row ++;
					mTableView->SelectCell(theSelectedCell);
					mTableView->SelectionChanged();
					mTableView->ScrollCellIntoFrame(theSelectedCell);
				}
			}
		}
	} else
		UTableKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
}

Boolean
ULurkersKeyScrollAttachment::SelectPrefixedItem(const LString& prefix, TableIndexT& ioStartIndex)
{
	CMacCVSLurkersView* lurkersView = dynamic_cast<CMacCVSLurkersView*>(mTableView);
	if (lurkersView != NULL) {
		TString<Str255> currentName;						// name of the current item.
		TableIndexT outRows = 0, outCols = 0;
		lurkersView->GetTableSize(outRows, outCols);
		for (TableIndexT currentRow = ioStartIndex; currentRow <= outRows; currentRow++ ) {
			CMacCVSProjectViewItem	*item = lurkersView->ConvertTableIndexToItem(currentRow);
			if (item != NULL) {
				item->GetNamePStr(currentName);
				UppercaseString(currentName);
				if (currentName.BeginsWith(prefix)) {
					lurkersView->UnselectAllCells();
					STableCell currentCell(currentRow, 1);
					lurkersView->SelectCell(currentCell);
					lurkersView->SelectionChanged();
					lurkersView->ScrollCellIntoFrame(currentCell);
					ioStartIndex = currentRow;
					return true;
				}
			}
		}
	}
	return false;
}

void
ULurkersKeyScrollAttachment::OpenItem( const STableCell& cell )
{
	CMacCVSLurkersView* lurkersView = dynamic_cast<CMacCVSLurkersView*>(mTableView);
	if (lurkersView != NULL) {
		TableIndexT index = cell.row;
		CMacCVSProjectViewItem* item = lurkersView->ConvertTableIndexToItem(index);
		if (! item->IsContainer())
			item->OpenWithFinder();
	}
}

#pragma mark -


//
// Attachment for the project view view
//

UProjectKeyScrollAttachment::UProjectKeyScrollAttachment(LView		*inViewToScroll):
	UTableKeyScrollAttachment( inViewToScroll ), mSortedItems(NULL)
{
}

UProjectKeyScrollAttachment::UProjectKeyScrollAttachment(LStream	*inStream):
	UTableKeyScrollAttachment( inStream), mSortedItems(NULL)
{
}

UProjectKeyScrollAttachment::~UProjectKeyScrollAttachment()
{
	if (mSortedItems != NULL)
		delete mSortedItems;
}

void
UProjectKeyScrollAttachment::SortVisibleItems()
{
	CMacCVSProjectView *projectView = dynamic_cast<CMacCVSProjectView*>(mTableView);
	if (projectView != NULL) {
		if (mSortedItems != NULL)
			delete mSortedItems;

		mSortedItems = new LArray(sizeof(CMacCVSProjectViewItem*));
		mSortedItems->SetComparator(new LSortByNameComparator());
		
		TableIndexT numTableRows, numTableCols;
		mTableView->GetTableSize(numTableRows, numTableCols);
		for (int row = 1; row <= numTableRows; row++) {
			CMacCVSProjectViewItem* item = projectView->ConvertTableIndexToItem(projectView->GetWideOpenIndex(row));
			mSortedItems->AddItem(&item);
		}
		
		// sort all of the items by name.
		mSortedItems->Sort();
	}
}

void
UProjectKeyScrollAttachment::ExecuteSelf( MessageT inMessage, void		*ioParam)
{
	EventRecord			*inEvent = (EventRecord *)ioParam;
	Int16						theKey = inEvent->message & charCodeMask;
	Boolean					shiftKeyDown = (inEvent->modifiers & shiftKey) != 0;
	Boolean					cmdKeyDown = (inEvent->modifiers & cmdKey) != 0;


	CMacCVSProjectView *theProjectView = dynamic_cast<CMacCVSProjectView*>(mTableView);
	
	Assert_(theProjectView != NULL);
	
	mExecuteHost = false;		// we will handle it

	switch (theKey)
	{
		case char_LeftArrow:
		
			if ( true || cmdKeyDown )		//collapse each open folder item
			{
					STableCell			theCell(0, 0);
				
					while ( mTableView->GetNextSelectedCell(theCell) )
					{
							TableIndexT								theWideIndex = 0L;
							CMacCVSProjectViewItem		*theClickItem = NULL;
						
							
							theWideIndex = theProjectView->GetWideOpenIndex( theCell.row );
							theClickItem = theProjectView->ConvertTableIndexToItem( theWideIndex );
							
							Assert_( theClickItem != NULL );
							
							if ( theClickItem->IsContainer() )
							{
									Boolean 				isExpanded;
									
									isExpanded = theProjectView->mCollapsableTree->IsExpanded( theWideIndex );
								
									if ( isExpanded )
									{
										theProjectView->CollapseRow( theWideIndex );
									}
							
							}
					}
			}
			
			break;
			
		case char_RightArrow:
		
			if ( true || cmdKeyDown )		// expand the item
			{
					STableCell			theCell(0, 0);
				
					while ( mTableView->GetNextSelectedCell(theCell) )
					{
							TableIndexT								theWideIndex = 0L;
							CMacCVSProjectViewItem		*theClickItem = NULL;
							
							theWideIndex = theProjectView->GetWideOpenIndex( theCell.row );
							theClickItem = theProjectView->ConvertTableIndexToItem( theWideIndex );
							
							Assert_( theClickItem != NULL );
							
							if ( theClickItem->IsContainer() )
							{
								Boolean 				isExpanded;
								
								isExpanded = theProjectView->mCollapsableTree->IsExpanded( theWideIndex );
							
								if ( ! isExpanded )
								{
									theProjectView->ExpandRow( theWideIndex );
								}
							
							}
							
					}
			}
			break;
		
		case char_DownArrow:
			if (cmdKeyDown)
				{
						STableCell			theCell(0, 0);
					
						while ( mTableView->GetNextSelectedCell(theCell) )
						{
								TableIndexT								theWideIndex = 0L;
								CMacCVSProjectViewItem		*theClickItem = NULL;
								CMacCVSProjectView 				*projectView = dynamic_cast<CMacCVSProjectView*>(mTableView);
								
								theWideIndex = theProjectView->GetWideOpenIndex( theCell.row );
								theClickItem = theProjectView->ConvertTableIndexToItem( theWideIndex );
								
								Assert_( theClickItem != NULL );
								
								if ( theClickItem->IsContainer() )
								{
										Boolean 				isExpanded;
										
										isExpanded = theProjectView->mCollapsableTree->IsExpanded( theWideIndex );
									
										if ( isExpanded )
											theProjectView->CollapseRow( theWideIndex );		//collapse it like the Finder
									
										projectView->MakeNewViewForCell( theCell );
								}
						}
						
				}
			else
				{
					mExecuteHost = true;
					UTableKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
				}
				
				
			break;
			
		case char_UpArrow:
			if (cmdKeyDown)
				{
					// I'm not sure if we want to go closing windows here.
				}
			else
				{
					mExecuteHost = true;
					UTableKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
				}
			break;
			
		case char_Tab:
			{
				// Go to the next item alphabetically in the list (previous if shift-key held down).
				// 1. need a sorted representation of the list of items.
				// 2. have to convert the list item to a table cell index.

				CMacCVSProjectViewItem* itemToView = NULL;

				TableIndexT numTableRows, numTableCols, wideIndex;
				Boolean shiftKeyDown = (inEvent->modifiers & shiftKey) != 0;
				
				mTableView->GetTableSize(numTableRows, numTableCols);
				STableCell theSelectedCell = mTableView->GetFirstSelectedCell();
				
				// get a sorted list of visible items. FIXME:  How do I detect the visible list has changed?
				if (mSortedItems == NULL || mSortedItems->GetCount() != numTableRows)
					SortVisibleItems();
				
				if (shiftKeyDown) {
					// no selection yet
					if ( theSelectedCell.IsNullCell() )	{
						// select the (alphabetically) last cell
						mSortedItems->FetchItemAt(numTableRows, &itemToView);
					} else {
						// select the cell before the first one
						theSelectedCell = mTableView->GetFirstSelectedCell();
						
						// this is a pretty slow transformation, oh, well.
						wideIndex = theProjectView->GetWideOpenIndex( theSelectedCell.row );
						CMacCVSProjectViewItem* viewItem = theProjectView->ConvertTableIndexToItem( wideIndex );
						ArrayIndexT	arrayIndex = mSortedItems->FetchIndexOfKey(viewItem);
						if (! mSortedItems->FetchItemAt(arrayIndex - 1, &itemToView))
							itemToView = NULL;
					}
				} else {
					// no selection yet
					if (theSelectedCell.IsNullCell()) {
						// select the first cell
						// select the (alphabetically) first cell
						mSortedItems->FetchItemAt(1, &itemToView);
					} else {
						// select the cell after the last one
						theSelectedCell.SetCell(0, 0);
						mTableView->GetPreviousSelectedCell(theSelectedCell);		// gets last selected cell

						// this is a pretty slow transformation, oh, well.
						wideIndex = theProjectView->GetWideOpenIndex( theSelectedCell.row );
						CMacCVSProjectViewItem* viewItem = theProjectView->ConvertTableIndexToItem( wideIndex );
						ArrayIndexT	arrayIndex = mSortedItems->FetchIndexOfKey(viewItem);
						if (! mSortedItems->FetchItemAt(arrayIndex + 1, &itemToView))
							itemToView = NULL;
					}
				}
				// Common code to select a particular project view item.
				if (itemToView != NULL) {
					// this is a pretty slow transformation, oh, well. can we make a good guess?
					wideIndex = theProjectView->ConvertItemToTableIndex(itemToView);
					theSelectedCell.SetCell(theProjectView->GetExposedIndex(wideIndex), 1);
					mTableView->UnselectAllCells();
					mTableView->SelectCell(theSelectedCell);
					mTableView->SelectionChanged();
					mTableView->ScrollCellIntoFrame(theSelectedCell);
				}
			}
			break;

		default:
			mExecuteHost = true;
			UTableKeyScrollAttachment::ExecuteSelf( inMessage, ioParam);
		}
}

Boolean UProjectKeyScrollAttachment::SelectPrefixedItem(const LString& prefix, TableIndexT& ioStartIndex)
	{
		CMacCVSProjectView *projectView = dynamic_cast<CMacCVSProjectView*>(mTableView);
		
		if (projectView != NULL)
			{
				TString<Str255> 	currentName;				// name of the current item.
				TableIndexT 			outRows = 0, outCols = 0;
				
				projectView->GetWideOpenTableSize(outRows, outCols);
				
				for (TableIndexT currentRow = ioStartIndex; currentRow <= outRows; currentRow++ )
					{
						CMacCVSProjectViewItem	*item = projectView->ConvertTableIndexToItem(projectView->GetWideOpenIndex(currentRow));
						
						if (item != NULL)
							{
								item->GetNamePStr(currentName);
								UppercaseString(currentName);
								
								if (currentName.BeginsWith(prefix))
									{
										projectView->UnselectAllCells();
										STableCell currentCell(currentRow, 1);
										projectView->SelectCell(currentCell);
										projectView->SelectionChanged();
										projectView->ScrollCellIntoFrame(currentCell);
										ioStartIndex = currentRow;
										return true;
									}
							}
						
					}
				
			}
		return false;
	}

void
UProjectKeyScrollAttachment::OpenItem( const STableCell& cell )
{
	CMacCVSProjectView *projectView = dynamic_cast<CMacCVSProjectView*>(mTableView);
	
	if (projectView != NULL)
		{
			TableIndexT index = projectView->GetWideOpenIndex(cell.row);
			CMacCVSProjectViewItem* item = projectView->ConvertTableIndexToItem(index);
			
			if (item->IsContainer())
			{
				if (projectView->mCreateNewView)
					{
						// open the item in a new window.
							projectView->MakeNewViewForCell( cell );
					}
				else
					{
						// else just turn down the item.
						Boolean isExpanded = projectView->mCollapsableTree->IsExpanded( index );
						
						if ( isExpanded )
							projectView->CollapseRow( index );
						else
							projectView->ExpandRow( index );
							
						projectView->FocusDraw();
						projectView->DrawDropFlag( cell, index );
					}	
			}
		else
			{
				item->OpenWithFinder();
			}
			
		}
}
