//	===========================================================================
//	URegions.cp				1994 Metrowerks Inc. All rights reserved.
//	===========================================================================

#ifdef PowerPlant_PCH
#include PowerPlant_PCH
#endif

#include	<PP_Messages.h>
#include	<UDrawingState.h>
#include	<UDrawingUtils.h>

#ifndef		__ERRORS__
#include	<Errors.h>
#endif

#include	"UMoreRegions.h"

//	===========================================================================
//	URegion
//	===========================================================================
//
//	Exception aware utility functions for Regions.


void	URegions::MakeEmpty(RgnHandle ioRegion)
{
	Rect	r = {0, 0, 0, 0};
	
	RectRgn(ioRegion, &r);
	ThrowIfOSErr_(QDError());
}

void	URegions::DrawAt(RgnHandle inRegion, Point inWhere, ConstPatternParam inPat)
{
	ThrowIfNULL_(inRegion);
	
	Point	negWhere;
	negWhere.h = -inWhere.h;
	negWhere.v = -inWhere.v;
	
	Translate(inRegion, inWhere);
	Try_{
		FillRgn(inRegion, inPat);
		ThrowIfOSErr_(QDError());
		Translate(inRegion, negWhere);
	} Catch_(inErr) {
		Translate(inRegion, negWhere);
	} EndCatch_;
}

void	URegions::FrameAt(RgnHandle inRegion, Point inWhere)
{
	ThrowIfNULL_(inRegion);
	
	Point	negWhere;
	negWhere.h = -inWhere.h;
	negWhere.v = -inWhere.v;
	
	Translate(inRegion, inWhere);
	Try_{
		FrameRgn(inRegion);
		ThrowIfOSErr_(QDError());
		Translate(inRegion, negWhere);
	} Catch_(inErr) {
		Translate(inRegion, negWhere);
	} EndCatch_;
}

void	URegions::Hilite(RgnHandle inRegion)
{
	StColorPenState	savePen;

	PenPat(&UQDGlobals::GetQDGlobals()->black);
	PenMode(srcXor);
	UDrawingUtils::SetHiliteModeOn();
	InvertRgn(inRegion);
}

void	URegions::Translate(RgnHandle ioRegion, Point inVector)
{
	ThrowIfNULL_(ioRegion);
	
	OffsetRgn(ioRegion, inVector.h, inVector.v);
}

void	URegions::MakeRegionOutline(RgnHandle inRegion, RgnHandle *ioRgn)
{
	OSErr		err = noErr;
	Boolean		createdRgn = false;
	
	ThrowIfNULL_(ioRgn);
	ThrowIfNULL_(inRegion);

	if (inRegion == *ioRgn) {
		StRegion	tempRgn(inRegion);
		
		InsetRgn(tempRgn, 1, 1);
								ThrowIfOSErr_(QDError());
		DiffRgn(inRegion, tempRgn, *ioRgn);	//	of course, inRegion & *ioRgn are the same!
								ThrowIfOSErr_(QDError());
	} else {
		if (*ioRgn == NULL) {
			*ioRgn = NewRgn();
			createdRgn = true;
		}
		ThrowIfNULL_(*ioRgn);

		Try_ {
			//	Get outline
			CopyRgn(inRegion, *ioRgn);
									ThrowIfOSErr_(QDError());
			InsetRgn(*ioRgn, 1, 1);
									ThrowIfOSErr_(QDError());
			DiffRgn(inRegion, *ioRgn, *ioRgn);
									ThrowIfOSErr_(QDError());
			
		} Catch_(inErr) {
		
			if (createdRgn && *ioRgn) {
				DisposeRgn(*ioRgn);
				*ioRgn = NULL;
			}
	
			Throw_(inErr);
		} EndCatch_;
	}	
}
