//################################################################################
//
//  Copyright (C) 1997-1998 Netscape Communications Corporation.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.
//
//################################################################################

/* -------------------------------------------------------------------------------

	File:				CCVSRepository.cp
	
	Authors:		Ram Rajadhyaksha
	
	Description:
	
		This file contains all the code that does all the network transactions
		with the CVS pserver/kserver and local disk file operations.
		 
	Modification History:

		13 Feb 1999		Ram Rajadhyaksha
		
					Added Andy N.'s gzip changes. Elimated all exception types except for
					type OSErr. Added code to "sniff" for AppleSingle files.
					
		10 Jan 1999		Ram Rajadhyaksha
		
					Billions and billions of changes! :-) Now works with CVS Entries files.
					
		15 Dec 1998		Ram Rajadhyaksha
		
					Integrated Peter Bierman's changes.

		25 Jul 1998		Ram Rajadhyaksha
		
					Fixed nasty bug in UpdateEntryFromStream() where it would read one more
					line than it should in the exception handling code.
					
					Fixed problem setting the file size for 0 byte files in UpdateFileFromStream().
					
					Fixed problems with pathnames ending with '.' items.
					
					Fixed bit comparison bug in WriteItemEntry() and Commit().
					
		20 Jul 1998		Simon Fraser
		
					Added many more diffing options.
					
		15 Jul 1998		Ram Rajadhyaksha
							
					Fixed problem with adding files with CVS keywords in them.
					
					Fixed problem where Add() did not call ::UseResFile() on the previous
					open resource file after opening the target file to add to the CVS server.
					
-------------------------------------------------------------------------------- */


#include <string.h>
#include <Threads.h>

#include "StFileAccess.h"
#include "StringUtils.h"
#include "ThrowOSErr.h"

#include "CVSEngine.h"
#include "CVSEngineUtils.h"

#include "UCVSRepositoryUtils.h"
#include "CCVSEntries.h"
#include "CCVSEncoders.h"
#include "CCVSRepository.h"

#include "MacBinary.h"
	
unsigned char kCVSValidRequestsToken[]		= "\pvalid-requests";
unsigned char kCVSValidRequestsToken2[]		= "\pValid-requests";
unsigned char kCVSValidResponsesToken[]		= "\pValid-responses";
unsigned char	kCVSRepositoryToken[]				= "\pRepository";
unsigned char	kCVSDirectoryToken[]				= "\pDirectory";
unsigned char kCVSExpandModulesToken[]		= "\pexpand-modules";
unsigned char	kCVSArgumentToken[]					= "\pArgument";
unsigned char	kCVSArgumentxToken[]				= "\pArgumentx";
unsigned char kCVSUseUnchangedToken[] 		= "\pUseUnchanged";
unsigned char kCVSUnchangedToken[]				= "\pUnchanged";
unsigned char	kCVSModifiedToken[]					= "\pModified";
unsigned char	kCVSStickyToken[]						= "\pSticky";
unsigned char kCVSStaticToken[]						= "\pStatic-directory";
unsigned char kCVSQuestionableToken[]			= "\pQuestionable";

unsigned char	kCVSDoneToken[]							= "\pok";
unsigned char kCVSErrorToken[]						= "\perror";
unsigned char kCVSErrorOutToken[]					= "\pE";
unsigned char kCVSMessageOutToken[]				= "\pM";
unsigned char kCVSModuleExpansionToken[]	= "\pModule-expansion";
unsigned char	kCVSCheckedInToken[]				= "\pChecked-in";
unsigned char kCVSNewEntryToken[]					= "\pNew-entry";
unsigned char kCVSMergedToken[]						= "\pMerged";
unsigned char kCVSUpdatedToken[]					= "\pUpdated";
unsigned char kCVSRemovedToken[]					= "\pRemoved";
unsigned char	kCVSRemoveEntryToken[]			= "\pRemove-entry";
unsigned char kCVSFileCompressionToken[]	=	"\pgzip-file-contents";
unsigned char kCVSSetStaticDirToken[]			= "\pSet-static-directory";
unsigned char kCVSClearStaticDirToken[]		= "\pClear-static-directory";
unsigned char kCVSSetStickyToken[]				= "\pSet-sticky";
unsigned char kCVSClearStickyToken[]			= "\pClear-sticky";

unsigned char	kCVSLoginToken[]						= "\pLogin";
unsigned char kCVSBinaryOption[]					= "\p-kb";
unsigned char kCVSOldBinaryOption[]				= "\p-ko";
unsigned char kCVSUnlockedPermissionStr[]	= "\pu=rwx,g=rwx,o=rwx";
unsigned char kCVSLockedPermissionStr[]		= "\pu=rx,g=rx,o=rx";

unsigned char kBinhex4SignaturePStr[]			= "\p(This file must be converted with BinHex 4.0)";


Boolean	CCVSRepository::sInitialized = false;
	

CCVSRepository::CCVSRepository(		const FSSpec											*inLocalRoot,
																	CCVSEngineFileMappingRec					*inFileMappings,
																	const UInt32											inFileMappingsCount,
																	CCVSEngineEventNotifierProcPtr		inNotifier,
																	CCVSEngineStreamNotifierProcPtr		inMessageNotifier,
																	CCVSEngineStreamNotifierProcPtr		inErrorNotifier,
																	void															*inContextPtr,
																	const UInt32											inTimeOut):
																	
	mStream ( inTimeOut )
	 
	{
	
		OSErr		result			= noErr;
		
		mLocalRoot.vRefNum	= 0;
		mLocalRoot.parID		= 0;
		mLocalRoot.name[0]	= 0;
		
		mRemoteRoot[0]			= 0;
		mRemoteUserName[0] 	= 0;
		mLocalUserName[0]		= 0;
		mServerName[0] 			= 0;
		mPort								= 0;
		mAuthMethod					= 0;
		
		mBuffer 						= NULL;
		mBufSize 						= 0;
		mYieldCount					= 0;
	
		mCanUseFileCompression = false;
	
		mNotifier 					= inNotifier;
		mMessageNotifier		= inMessageNotifier;
		mErrorNotifier			= inErrorNotifier;
		
		mContextPtr					= inContextPtr;
				
		mEntries						= NULL;

		if ( sInitialized == false )
			{
				Initialize();
			}
			
		try
			{			
				if ( inLocalRoot == NULL )	
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
					
				mLocalRoot = *inLocalRoot;

				if ( (mLocalRoot.vRefNum == 0) || (mLocalRoot.parID == 0) || (mLocalRoot.name[0] == 0) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
					
				if ( FSpIsAFile(&mLocalRoot, NULL) != kCVSEngineIsFolder )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
				
				mBuffer = ::NewPtr( kDefaultBufferSize );
				
				if (mBuffer == NULL)
					{
						throw( (OSErr)::MemError() );
					}
					
				mBufSize = kDefaultBufferSize;
				
				if ( (inFileMappings != NULL) && (inFileMappingsCount > 0) )
					{
						result = mMappings.SetMappings( inFileMappings, inFileMappingsCount );
						ThrowOSErr( result );
					}		
			}			
		catch ( ... )
			{								
				if ( mBuffer != NULL )
					{					
						::DisposePtr( mBuffer );						
						mBuffer = NULL;						
					}
					
				throw;				
			}
									
	}
	
	
CCVSRepository::~CCVSRepository()
	{				
		if ( mBuffer != NULL )
			{			
				::DisposePtr( mBuffer );
				
				mBuffer = NULL;
				mBufSize = 0;			
			}
	
		if ( mEntries != NULL )
			{			
				delete mEntries;
				mEntries = NULL;				
			}
	}
	
	
void
CCVSRepository::Initialize()
	{				
		sInitialized = true;	
	}
	

#pragma mark -


OSErr
CCVSRepository::Login( const CCVSEngineLoginOptions *inOptions )
	{

		OSErr result = noErr;

		try
			{
				if ( inOptions == NULL )
					ThrowOSErr( kCVSEngineBadParamErr );
								
				::BlockMoveData( inOptions->mCVSRoot, mRemoteRoot, inOptions->mCVSRoot[0] + 1 );
				::BlockMoveData( inOptions->mRemoteUserName, mRemoteUserName, inOptions->mRemoteUserName[0] + 1 );
				::BlockMoveData( inOptions->mServerName, mServerName, inOptions->mServerName[0] + 1 );
				mPort = inOptions->mPort;
				mAuthMethod = inOptions->mAuthenticationMethod;
				
//  Strip off ending '/' characters. If the user passed in a root of '/', then
//  the resulting mRemoteRoot string will be zero length. 

//  However, even though path names usually don't end with a '/', you must send
//  at least one '/' when setting the Root.

				while ( (mRemoteRoot[0] > 0) && (mRemoteRoot[mRemoteRoot[0]] == '/') )
					mRemoteRoot[0]--;

//  If we have anything left, check for duplicate '/' chars
				
				{				
					UInt32 index = 1;
										
					while ( index <= mRemoteRoot[0] )
						{						
							if ( (mRemoteRoot[index] == '/') && (index > 1) && (mRemoteRoot[index - 1] == '/') )
								ThrowOSErr( kCVSEngineBadParamErr );
							else index++;										
						}					
				}

				if ( inOptions->mAuthenticationMethod == kAuthenticationRSH )
					{					
						mStream.RSHLogin( CCVSRepository::SocketYieldProc,
															this,
															inOptions->mServerName, 
															inOptions->mPort, 
															inOptions->mLocalUserName, 
															inOptions->mRemoteUserName, 
															mRemoteRoot,
															inOptions->mCVSCommandText );				
					}
				else if ( inOptions->mAuthenticationMethod == kAuthenticationKClient )
					{
						mStream.AuthLogin(	CCVSRepository::SocketYieldProc,
															this,
															inOptions->mServerName,
															inOptions->mPort,
															mRemoteRoot );	
					}
				else
					{
						mStream.Login( 		CCVSRepository::SocketYieldProc,
															this,
															inOptions->mServerName, 
															inOptions->mPort, 
															inOptions->mRemoteUserName, 
															inOptions->mPassword, 
															mRemoteRoot );
					}
					
				NegotiateServerOptions();														
			}
					
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{		
				result = kCVSEngineUnknownExceptionErr;			
			}
			
		return result;

	}
	
		
OSErr		
CCVSRepository::Add( const FSSpec *inTargetSpec, const CCVSEngineAddOptions *inOptions )
	{

//  Add()

//  This function will add a specified file or directory to
//  the remote repository.
		
		OSErr					result											= noErr;

		Boolean				bProcessFile								= false;
		Boolean				bCalledPreAction						= false;
		Boolean				bIsaFolder									= false;
		Boolean				bWillResurrect							= false;
		
		char					strPath[kMaxPathSize]				= "";
		char					strLine[kMaxLineSize]				= "";
		char					strOffsetPath[kMaxPathSize]	= "";
		
		UInt32				intPathLen									= 0;
		UInt32				intOffsetPathLen						= 0;
		UInt32				intLineLen									= 0;
		UInt32				intEncoding									= kCVSEngineTextEncoding;

		Str255				pstrFileName								= "\p";
		Str255				pstrParentTag								= "\p";
		Str255				pstrRepository							= "\p";
		Str255				pstrRevision								= "\p";
		Str255				pstrTimeStamp								= "\p";
		Str255				pstrOptions									= "\p";
		Str255				pstrTag											= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255				pstrTranslated							= "\p";
#endif
		
		FSSpec				fsTarget										= { 0, 0, "\p" };
		FSSpec				fsParent										= { 0, 0, "\p" };

		CCVSEngineFolderCreationOptions		folderOptions		= { false, false };

		CkidResourcePB	ckidInfo;
		CInfoPBRec 			pb;

//  Since I have to close and re-open my add file a couple times,
//  scope is important in various points of this function.
		
		try
			{				
				memset( &ckidInfo, 0, sizeof(CkidResourcePB) );
				
				if ( (inTargetSpec == NULL) || (inOptions == NULL) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTarget = *inTargetSpec;

				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}

				//  If the project root doesn't have a CVS folder,
				//  we better make one!
				
				InitRootDirectory();
					
				result = FSpIsAFile( &fsTarget, NULL );
				
				if ( result != noErr )
					{						
						if ( result != kCVSEngineIsFolder )
							{
								throw result;				
							}			
						else 
							{
								bIsaFolder = true;
							}					
					}

//  Make sure the parent is a repository directory

				if ( !CUIsRepositoryDir(fsTarget.vRefNum, fsTarget.parID) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}

#ifndef CVSENGINE_USE_MIME_FILE_NAMES					

//  Make sure the target has a valid file name

				if ( !CUIsValidCVSName(fsTarget.name) )
					{
						ThrowOSErr( kCVSEngineInvalidFileNameErr );
					}

#endif
					
//  Get the local path name of the file, relative to the project root.
//  This is used for a parameter to the notifier routine, if one is
//  installed. It is not necessarily the same path that is transmitted
//  to the CVS server during the add operation.
							
				intPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsTarget, &mLocalRoot, strPath, &intPathLen );
				strPath[intPathLen] = 0;			
				ThrowOSErr( result );

//  Strip off the file name and use this as the offset path. 
//  We run the Add command in the parent directory of the
//  target item because CVS has so many bugs. So we'll need
//  the offset path for ProcessServerResponse().

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
				
				{
					Str255 pstrTranslated = "\p";
					
					CUTranslateFromMacNamePStr( fsTarget.name, pstrTranslated );
					Assert_( pstrTranslated[0] <= intPathLen );

					if ( intPathLen > pstrTranslated[0] )
						{				
							intOffsetPathLen = intPathLen - (pstrTranslated[0] + 1);
							::BlockMoveData( strPath, strOffsetPath, intOffsetPathLen );				
						}				
				}
				
#else

				Assert_( fsTarget.name[0] <= intPathLen );
				
				if ( intPathLen > fsTarget.name[0] )
					{				
						intOffsetPathLen = intPathLen - (fsTarget.name[0] + 1);
						::BlockMoveData( strPath, strOffsetPath, intOffsetPathLen );				
					}

#endif
					
				if ( bIsaFolder )
					{					
						if ( FSpIsRepositoryDir(&fsTarget) )
							{
								//  Don't add folders that are already in repository.
								ThrowOSErr( kCVSEngineBadParamErr );
							}
							
						if ( mNotifier != NULL )
							{
								if ( intPathLen > 0 )
									{
										bProcessFile = (*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineAddPreActionEvent, result, intPathLen, strPath, mContextPtr );
									}
								else 
									{
										bProcessFile = (*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineAddPreActionEvent, result, 0, NULL, mContextPtr );
									}
									
								bCalledPreAction = true;								
							}
							
						if ( !bProcessFile )
							{
								ThrowOSErr( kCVSEngineSkippedFile );
							}
							
						AddDirectory( fsTarget, strOffsetPath, intOffsetPathLen, strPath, intPathLen );							
					}					
				else
					{					
						result = FSpGetParentFSSpec( &fsTarget, &fsParent );
						ThrowOSErr( result );

						if ( mEntries != NULL )
							{
								delete mEntries;
								mEntries = NULL;
							}
	
						//  Load the working directory's entries file. We will
						//  leave the file open with fsRdWrPerm until ProcessServerResponse()
						//  exits to lock the file from other threads.
	
						mEntries = new CCVSEntriesCache( this );
					
						if ( mEntries != NULL )
							{
								result = mEntries->LoadWorkingDirectory( &fsParent, fsRdWrPerm );
							}
						else 
							{
								result = kCVSEngineOutOfMemoryErr;
							}
							
						ThrowOSErr( result );						

						result = mEntries->GetEntry( fsTarget.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
						
						if ( result == kCVSEngineNoRevisionDataErr )
							{
								result = noErr;
							}
						else
							{	
								result = kCVSEngineBadParamErr;
								
								if ( (pstrRevision[0] > 0) && (pstrRevision[1] == '-') )
									{
										result = noErr;
										bWillResurrect = true;
									}
							}
						
						ThrowOSErr( result );
							
						pb.hFileInfo.ioCompletion = NULL;
						pb.hFileInfo.ioFDirIndex 	= 0;
						pb.hFileInfo.ioNamePtr 		= fsTarget.name;
						pb.hFileInfo.ioVRefNum		= fsTarget.vRefNum;
						pb.hFileInfo.ioDirID 			= fsTarget.parID;
					
						result = ::PBGetCatInfoSync( &pb );
						ThrowOSErr( result );

						//  Ok, the only way to truly tell if a file is open 
						//  without calling FSpOpenDF, FSpOpenRF, *and* FSpOpenResFile 
						//  is to check the flags.
										
						if ( FILE_OPEN(pb) )
							{							
								result = opWrErr;
								throw result;						
							}
					
						//  Scope is important here. Make sure file doesn't 
						//  have ckid and add a resource fork if the file 
						//  doesn't already have one.
				
						{						
							SInt16				dataForkRef		= 0;
							StFileAccess	addFileAccess( fsTarget );

							dataForkRef = addFileAccess.GetFSDataForkRef();
														
							if ( mNotifier != NULL )
								{
									if ( intPathLen > 0 )
										{									
											bProcessFile = (*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineAddPreActionEvent, result, intPathLen, strPath, mContextPtr );
										}
									else 
										{
											bProcessFile = (*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineAddPreActionEvent, result, 0, NULL, mContextPtr );
										}
										
									bCalledPreAction = true;									
								}
								
							if ( !bProcessFile )
								{
									ThrowOSErr( kCVSEngineSkippedFile );
								}							
						}

						result = FSpGetRepository( &fsParent, pstrRepository );
						ThrowOSErr( result );
						
						result = FSpGetFolderTag( &fsParent, pstrParentTag );
						
						if ( result == fnfErr )
							{
								result = noErr;
							}
							
						ThrowOSErr( result );
						
						//  Set the parent directory.
											
						mStream << kCVSDirectoryToken;
						mStream << " .\n";
						
						if ( ((pstrRepository[0] > 0) && (pstrRepository[1] == '/')) == false )
							{
								mStream << mRemoteRoot;
								
								if ( (mRemoteRoot[0] == 0) || (pstrRepository[0] > 0) )
									{
										mStream << "/";
									}
							}

						mStream << pstrRepository;
						mStream << "\n";

						//  Send the parent tag if necessary.
						
						if ( pstrParentTag[0] > 0 )
				      {
					      mStream << kCVSStickyToken;
					      mStream << " ";
					      mStream << pstrParentTag;
					      mStream << "\n";
				      }

						//  If we're "resurrecting" a file, just send the entry and
						//  no file data. Else, send the file and no entry data.
							
						if ( bWillResurrect )
							{
								mStream << "Entry /";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
								CUTranslateFromMacNamePStr( fsTarget.name, pstrTranslated );
								mStream << pstrTranslated;
#else
								mStream << fsTarget.name;
#endif
								mStream << "/";
								mStream << pstrRevision;
								mStream << "/";
								mStream << pstrTimeStamp;
								mStream << "/";
								mStream << pstrOptions;
								mStream << "/";
								mStream << pstrTag;
								mStream << "\n";
							}
						else
							{
								if ( inOptions->mAutomagicallyEncode == true )
									{						
										CCVSEngineFileMappingRec	theMapping;
										theMapping.Init();
										
										if ( mMappings.GetMapping(fsTarget.name, theMapping, false) )
											{
												intEncoding = theMapping.fEncoding;					
											}				
										else
											{ 
												intEncoding = kCVSEngineTextEncoding;				
											}				
									}							
								else
									{							
										intEncoding = inOptions->mEncodingOptions;								
									}
								
								WriteModifiedEntry( fsTarget, intEncoding );
										
								if ( intEncoding != kCVSEngineTextEncoding )
									{				
										mStream << kCVSArgumentToken;
										mStream << " ";
										mStream << kCVSBinaryOption;
										mStream << "\n";								
									}
							}
												
						mStream << kCVSArgumentToken;
						mStream << " ";
						
#ifdef CVSENGINE_USE_MIME_FILE_NAMES
						CUTranslateFromMacNamePStr( fsTarget.name, pstrTranslated );
						mStream << pstrTranslated;
#else
						mStream << fsTarget.name;
#endif
						
						mStream << "\n";
						
						if ( mNotifier != NULL )
							{
								(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
							}		

						mStream << "add\n";		

						CCVSEngineFolderCreationOptions fcOptions;
						ProcessServerResponse( strOffsetPath, intOffsetPathLen, kCVSEngineTextAndBinaryMerging, fcOptions, NULL, NULL, NULL, true, true, true );						
					}										
			}
		catch ( OSErr inErr )
			{			
				result = inErr;				
			}		
		catch ( ... )
			{		
				result = kCVSEngineUnknownExceptionErr;		
			}

		if ( (mNotifier != NULL) && (bCalledPreAction) )
			{		
				if ( intPathLen > 0 )
					(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineAddPostActionEvent, result, intPathLen, strPath, mContextPtr );
				else 
					(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineAddPostActionEvent, result, 0, NULL, mContextPtr );				
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;

		return result;
			
	}
	
	
OSErr		
CCVSRepository::Checkout( const FSSpec *inTargetSpec, const CCVSEngineCheckoutOptions	*inOptions )
	{
	
		OSErr					result 								= noErr;														
		FSSpec				fsTarget							= { 0, 0, "\p" };
		Str255				pstrCompressionLevel	= "\p";
		
		CCVSEngineCheckoutOptions	theOptions;
		theOptions.Init();
		
		try
			{
				if ( (inTargetSpec == NULL) || (inOptions == NULL) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
					
				theOptions = *inOptions;
				fsTarget = *inTargetSpec;

				//  LF translation preference is really ignored.
				//  Actual preference is fetched from the mappings table.
				
				Assert_( (theOptions.mLFTranslation == kCVSEngineLFTranslation) || (theOptions.mLFTranslation == kCVSEngineCRTranslation) );
				
				if ( theOptions.mLFTranslation > kCVSEngineCRTranslation )
					{
						theOptions.mLFTranslation = kCVSEngineLFTranslation;
					}
					
				Assert_( 	(theOptions.mMergePolicy == kCVSEngineNoMerging) || 
									(theOptions.mMergePolicy == kCVSEngineTextMerging) || 
									(theOptions.mMergePolicy == kCVSEngineTextAndBinaryMerging) );
				
				if ( theOptions.mMergePolicy > kCVSEngineTextAndBinaryMerging )
					{
						theOptions.mMergePolicy = kCVSEngineNoMerging;
					}
					
				//  If the project root doesn't have a CVS folder,
				//  we better make one!
				
				InitRootDirectory();
											
				//  If the user passed in a FSSpec that is not the root spec, 
				//  do a CVS update -d.
			
				if ( !FSpIsEqual(&fsTarget, &mLocalRoot) )
					{
						CCVSEngineUpdateOptions		updateOptions;
						updateOptions.Init();

						PStrCopy( theOptions.mRevisionPStr, updateOptions.mRevisionPStr, sizeof(updateOptions.mRevisionPStr) );
						PStrCopy( theOptions.mDatePStr, updateOptions.mDatePStr, sizeof(updateOptions.mDatePStr) );
					
						updateOptions.mUpdateDeep 				= theOptions.mCheckoutDeep;
						updateOptions.mGetNewDirectories 	= true;
						updateOptions.mResetStickyTags 		= theOptions.mResetStickyTags;
						updateOptions.mMergePolicy 				= theOptions.mMergePolicy;
						updateOptions.mLFTranslation 			= theOptions.mLFTranslation;
					
						result = Update( &fsTarget, &updateOptions );					
					}					
				else
					{
						//  Else do an "expand-modules" to find out what directories
						//  are in the module. If we already have some, tell the CVS
						//  server this and then do the checkout.

						mStream << kCVSArgumentToken;
						mStream << " ";
						mStream << theOptions.mModulePStr;
						mStream << "\n";
						
						mStream << kCVSExpandModulesToken;
						mStream << "\n";
						
						ProcessServerResponse( NULL, 0, theOptions.mMergePolicy, theOptions.mFolderOptions, NULL, NULL, NULL );

						//  If expand-modules caused MacCVS to send entries, then the 
						//  entry database will be left open with read access. We need
						//  to close it so during the next ProcessServerResponse, it gets
						//  opened with fsRdWrPerm.

						if ( mEntries != NULL )
							{
								delete mEntries;
								mEntries = NULL;
							}
						
						//  Get back to the root to send the co command.

						FSpSetDirectory( mLocalRoot );					
							
						// send out the file compression
						
						if ( (theOptions.mUseCompression) && (mCanUseFileCompression) )
							{						
								mStream << kCVSFileCompressionToken;
								mStream << " ";								
						
								if ( (theOptions.mCompressionLevel >= 1) && (theOptions.mCompressionLevel <= 9) )
									{
										UInt32ToStr255( theOptions.mCompressionLevel, pstrCompressionLevel );
										mStream << pstrCompressionLevel;
									}
								else
									{
										mStream << "1";
									}
									
								mStream << "\n";							
							}

						if ( theOptions.mResetStickyTags )
							{							
								mStream << kCVSArgumentToken;
								mStream << " -A\n";								
							}
							
						if ( theOptions.mCheckoutDeep == false )
							{						
								mStream << kCVSArgumentToken;
								mStream << " -l\n";							
							}
								
						if ( theOptions.mRevisionPStr[0] > 0 )
							{							
								mStream << kCVSArgumentToken;
								mStream << " -r\n";
								
								mStream << kCVSArgumentToken;
								mStream << " ";
								mStream << theOptions.mRevisionPStr;
								mStream << "\n";								
							}

						else if ( theOptions.mDatePStr[0] > 0 )
							{							
								mStream << kCVSArgumentToken;
								mStream << " -D\n";

								mStream << kCVSArgumentToken;
								mStream << " ";
								mStream << theOptions.mDatePStr;
								mStream << "\n";								
							}

						//  Send argument to tell server not to run checkout progs

						mStream << kCVSArgumentToken;
						mStream << " -n\n";

						//  Send the module argument

						if ( theOptions.mModulePStr[0] > 0 )
							{						
								mStream << kCVSArgumentToken;
								mStream << " ";
								mStream << theOptions.mModulePStr;
								mStream << "\n";								
							}

						//  Close the entries database because it was probably opened 
						//  with fsRdPerm in a WriteItemEntry() call. We need it open
						//  with fsRdWrPerm to flush the changes we make.

						if ( mEntries != NULL )
							{
								delete mEntries;
								mEntries = NULL;
							}

						if ( mNotifier != NULL )
							{
								(*mNotifier)( (CCVSEngineRef)this, NULL, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
							}		
																						
						mStream << "co\n";
													
						ProcessServerResponse( NULL, 0, theOptions.mMergePolicy, theOptions.mFolderOptions, NULL, NULL, NULL );						
					}					
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;			
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;
			
		return result;
	
	}
	

OSErr
CCVSRepository::Commit( const FSSpec *inTargetSpec, const CCVSEngineCommitOptions *inOptions )
	{

		OSErr			result 												= noErr;

		Boolean		bDone													= false;
		Boolean 	bCalledPreAction							= false;
		Boolean		bFileTouched									= false;
		Boolean		bFileNuked										= false;
		Boolean		bBinaryFile										= false;
		
		char			strLine[kMaxLineSize]					= "";
		char			strLocalPath[kMaxPathSize]		= "";
		char			strOffsetPath[kMaxPathSize]		= "";
		
		Str255		pstrOptions										= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255		pstrTranslated								= "\p";
#endif
		
		SInt32		intModDate										= 0;
		SInt32		intCreationDate								= 0;
		
		UInt32		intLineLen										= 0;
		UInt32		intLocalPathLen								= 0;
		UInt32		intOffsetPathLen							= 0;
		UInt32		intEncoding										= 0;
			
		FSSpec		fsTargetSpec									= { 0, 0, "\p" };
		FSSpec		fsUpdateSpec									= { 0, 0, "\p" };
		FSSpec		fsParentSpec									= { 0, 0, "\p" };
		Handle		hWinPosRsrc										= NULL;

		CCVSEngineFolderCreationOptions folderOptions	= { false, false };
		
		CkidResourcePB	*pCkidInfo = NULL;
		CInfoPBRec			pb;
		
		try
			{
				pCkidInfo = (CkidResourcePB *)::NewPtrClear( sizeof(CkidResourcePB) );
				
				if ( pCkidInfo == NULL )
					{		
						result = ::MemError();
						throw result;					
					}

				if ( (inTargetSpec == NULL) || (inOptions == NULL) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTargetSpec = *inTargetSpec;	

//  Make sure the item is in the local repository.
				
				if ( FSpIsChildOf( &fsTargetSpec, &mLocalRoot ) == false )
					ThrowOSErr( kCVSEngineBadParamErr );		

//  Make sure it's parent is a valid CVS directory.
				
				if ( !CUIsRepositoryDir(fsTargetSpec.vRefNum, fsTargetSpec.parID) )
					ThrowOSErr( kCVSEngineBadParamErr );		

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioFDirIndex 	= 0;
				pb.hFileInfo.ioNamePtr 		= fsTargetSpec.name;
				pb.hFileInfo.ioVRefNum 		= fsTargetSpec.vRefNum;
				pb.hFileInfo.ioDirID 			= fsTargetSpec.parID;
				
				result = ::PBGetCatInfoSync( &pb );
				ThrowOSErr( result );

//  Make sure it's a file.

				if ( pb.dirInfo.ioFlAttrib & ioDirMask )
					ThrowOSErr( kCVSEngineBadParamErr );

//  Get the parent FSSpec

				result = FSpGetParentFSSpec( &fsTargetSpec, &fsParentSpec );
				ThrowOSErr( result );

				if ( mEntries != NULL )
					{
						delete mEntries;
						mEntries = NULL;
					}

//  Get a lock on the working directory before we do anything.
				
				mEntries = new CCVSEntriesCache( this );

				if ( mEntries != NULL )
					{
						result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdWrPerm );
					}
				else 
					{
						result = kCVSEngineOutOfMemoryErr;
					}
					
				ThrowOSErr( result );								
	
				//  Get the entry and check to see if the file was
				//  originally added with the -kb binary option. If
				//  it was, we need to check the mapping table for
				//  encoding preferences. Else send it as text.
				
				result = mEntries->GetEntry( fsTargetSpec.name, NULL, NULL, pstrOptions, NULL );
				ThrowOSErr( result );
				
				bBinaryFile = CUIsCVSBinaryOption( pstrOptions );
				
				//  For now we always send files that were added
				//  as text files as text. This might change in
				//  the future if we add RBL support, which is an
				//  encoded text file.
				
				if ( bBinaryFile )
					{						
						CCVSEngineFileMappingRec sMapping;
						sMapping.Init();
						
						if ( mMappings.GetMapping(fsTargetSpec.name, sMapping, false) )
							{
								intEncoding = sMapping.fEncoding;						
							}			
						else
							{ 
								intEncoding = kCVSEngineBinaryEncoding;	
							}							
					}							
				else
					{
						intEncoding = kCVSEngineTextEncoding;
					}
					
//  Read the ckid

				result = FSpReadCkidResource( &fsTargetSpec, pCkidInfo, true );
				
				if ( result != noErr )
					{
						pCkidInfo->fCommentHndl = NULL;
						pCkidInfo->fCommentLen = 0;
						
						result = noErr;
					}

				{
					StFileAccess	stAccess( fsTargetSpec );
					SInt16	intResFileRef = stAccess.GetResFileRef( fsRdPerm );
					
					//  get CW window pos resource, for later restore on commit with update

					(void) CUReadWinPosResourceFromRef( &hWinPosRsrc, intResFileRef );

					//  ignore result, we don't care if it doesn't have a resource to save		
				}
													
				//  Save the old modification date so that we don't accidently
				//  touch any files. (thanks McMullen)
					
				intCreationDate = pb.hFileInfo.ioFlCrDat;
				intModDate = pb.hFileInfo.ioFlMdDat;

				//  Unlock the file
				
				result = ::FSpRstFLock( &fsTargetSpec );
				ThrowOSErr( result );
				
				intLocalPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsTargetSpec, &mLocalRoot, strLocalPath, &intLocalPathLen );
				strLocalPath[intLocalPathLen] = 0;
				ThrowOSErr( result );

				//  If we're not running the command in the root directory of the
				//  repository, we need to get the offset path.

				if ( !FSpIsEqual(&fsParentSpec, &mLocalRoot) )
					{
						if ( intLocalPathLen > (fsTargetSpec.name[0] + 1) )
							{						
#ifdef CVSENGINE_USE_MIME_FILE_NAMES
								CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );

								intOffsetPathLen = intLocalPathLen - (pstrTranslated[0] + 1);
#else
								intOffsetPathLen = intLocalPathLen - (fsTargetSpec.name[0] + 1);
#endif
								::BlockMoveData( strLocalPath, strOffsetPath, intOffsetPathLen );
							}
						else
							{
								//  Since we're not running in the root, we
								//  should always have an offset path.
								
								Assert_( false );  
							}
					}
				
				if ( mNotifier != NULL )
					{					
						if ( !(*mNotifier)( (CCVSEngineRef)this, &fsTargetSpec, kCVSEngineCommitPreActionEvent, noErr, intLocalPathLen, strLocalPath, mContextPtr ) )
							{
								ThrowOSErr( kCVSEngineSkippedFile );
							}
							
						bCalledPreAction = true;						
					}

				//  Specify false for inDoMerge and send the entry manually. 
				//  If the user CVS Added a file, it would appear not modified 
				//  to WriteItemEntry().

				WriteEntry( fsTargetSpec, mLocalRoot, false, false, false, false, false );
				WriteModifiedEntry( fsTargetSpec, intEncoding );
										
				mStream << kCVSArgumentToken;
				mStream << " -n\n";
								
				if ( inOptions->mUseFileRsrcComment && (pCkidInfo->fCommentHndl != NULL) && (pCkidInfo->fCommentLen > 0) )
					{
						SendCheckinComment( pCkidInfo->fCommentHndl );
					}
				else if ( (!inOptions->mUseFileRsrcComment) && (inOptions->mCommentText != NULL) )
					{
						if ( ::GetHandleSize(inOptions->mCommentText) > 0 )
							{
								SendCheckinComment( inOptions->mCommentText );							
							}
						else
							{						
								mStream << kCVSArgumentToken;
								mStream << " -m\n";

								mStream << kCVSArgumentToken;
								mStream << " *** empty log message ***\n";								
							}

						//  Update the file 'ckid' with the new checkin comment.
							
						if ( pCkidInfo->fCommentHndl != NULL )
							{							
								::DisposeHandle( pCkidInfo->fCommentHndl );
							
								pCkidInfo->fCommentHndl = NULL;
								pCkidInfo->fCommentLen = 0;								
							}
								
						pCkidInfo->fCommentHndl = inOptions->mCommentText;
						result = ::HandToHand( &pCkidInfo->fCommentHndl );
						
						if ( result != noErr )
							{							
								pCkidInfo->fCommentHndl = NULL;
								throw result;								
							}
								
						pCkidInfo->fCommentLen = ::GetHandleSize( pCkidInfo->fCommentHndl );
							
						result = FSpWriteCkidResource( &fsTargetSpec, pCkidInfo );
						ThrowOSErr( result );	
					}						
				else
					{
						mStream << kCVSArgumentToken;
						mStream << " -m\n";

						mStream << kCVSArgumentToken;
						mStream << " *** empty log message ***\n";
					}

				//  Get back to the root, send the file's
				//  relative path name, and pray!
				
				FSpSetDirectory( mLocalRoot );
				
				mStream << kCVSArgumentToken;
				mStream << " ";			
					
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}					

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
				CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
				mStream << pstrTranslated;
#else				
				mStream << fsTargetSpec.name;
#endif

				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, NULL, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				mStream << "ci\n";

				//  I used to originally call ProcessServerResponse(). However, since Commit 
				//  is unusually complicated, especially with auto-merging preferences issues, 
				//  I handle the responses here.

				while ( !bDone )
					{
						Yield();
											
						intLineLen = kMaxLineSize - 1;
						mStream.ReadWord( strLine, intLineLen );
						strLine[intLineLen] = 0;
						
						if ( !strcmp("ok", strLine) )
							{
								bDone = true;	
							}
						else if ( !strcmp("error", strLine) )
							{					
								bDone = true;
								
								if ( result == noErr )
									result = kCVSEngineServerReturnedErr;
								
								throw result;	
							}											
						else if ( !strcmp("M", strLine) )
							{					
								intLineLen = kMaxLineSize - 1;
								mStream.ReadEntireDataLine( strLine, intLineLen );
								strLine[intLineLen] = 0;

								MessageOut( mMessageNotifier, strLine, intLineLen, mContextPtr );
							}						
						else if ( !strcmp("E", strLine) )
							{	
								intLineLen = kMaxLineSize - 1;
								mStream.ReadEntireDataLine( strLine, intLineLen );
								strLine[intLineLen] = 0;
						
								ErrorOut( mErrorNotifier, strLine, intLineLen, mContextPtr );									
							}																
						else if ( !strcmp("Updated", strLine) || !strcmp("Merged", strLine) )
							{	
								Assert_( bFileTouched == false );
								bFileTouched = true;
								
								//  Ok, if we get an "updated" response, we need to hide our
								//  original file and download the new one. If we succeed,
								//  we can blow away the old one.
										
								FSSpec fsRenamed = fsTargetSpec;
									
								result = FSpSafeRename( &fsRenamed );
								ThrowOSErr( result );	
	
								try
									{
										ItemUpdatedMerged( mRemoteUserName, kCVSEngineTextAndBinaryMerging, false, NULL, 0, folderOptions, fsUpdateSpec );
									}
								catch ( OSErr intException )
									{
										//  If something went wrong, try to rename the original file
										//  back to what it originally was. Since UpdateFileFromStream()
										//  threw an exception, it probably deleted the file it was working
										//  on so this should work.
									
										::FSpRename( &fsRenamed, fsTargetSpec.name );

										throw intException;
									}

								Assert_( FSpIsEqual(&fsUpdateSpec, &fsTargetSpec) );
									
								if ( FSpIsEqual(&fsUpdateSpec, &fsTargetSpec) )
									{			
																								
										Boolean bFileLocked = FSpIsLocked( &fsTargetSpec );																				

										if ( bFileLocked )
											{
												::FSpRstFLock( &fsTargetSpec );
											}

										//  If there was a comment, copy it over.
								
										if ( (pCkidInfo->fCommentHndl != NULL) && (pCkidInfo->fCommentLen > 0) )
											{				
												CkidResourcePB 	sCkidInfo;
												
												result = FSpReadCkidResource( &fsTargetSpec, &sCkidInfo, false );
												ThrowOSErr( result );

												sCkidInfo.fCommentHndl = pCkidInfo->fCommentHndl;
												sCkidInfo.fCommentLen = pCkidInfo->fCommentLen;
																										
												result = FSpWriteCkidResource( &fsTargetSpec, &sCkidInfo );
												ThrowOSErr( result );
											}

										//  save CW window pos resource

										if ( hWinPosRsrc != NULL )
											{
												(void) FSpWriteWinPosResource( &fsTargetSpec, hWinPosRsrc );
											}

										if ( bFileLocked )
											{
												::FSpSetFLock( &fsTargetSpec );
											}
											
										result = ::FSpDelete( &fsRenamed );
										Assert_( result == noErr );
									}
							}
						else if ( !strcmp("Checked-in", strLine) )
							{
								Assert_( bFileTouched == false );
								bFileTouched = true;
								
								try
									{
										ItemCheckedIn( mRemoteUserName, kCVSEngineTextAndBinaryMerging, false, NULL, 0, fsUpdateSpec, true, false, true );						
									}
								catch ( OSErr intException )
									{
										throw intException;
									}

								Assert_( FSpIsEqual(&fsUpdateSpec, &fsTargetSpec) );
							}
						else if ( !strcmp("New-entry", strLine) )
							{
								//  New-entry is just like a "Checked-in" response,
								//  except that the file is not up to date.
								
								//  I'm not sure if we want to support this response in
								//  Commit because it might cause weird things to happen.
								
								try
									{
										ItemCheckedIn( mRemoteUserName, kCVSEngineTextAndBinaryMerging, false, NULL, 0, fsUpdateSpec, true, false, false );						
									}
								catch ( OSErr intException )
									{
										throw intException;
									}
									
								Assert_( FSpIsEqual(&fsUpdateSpec, &fsTargetSpec) );
							}
						else if ( !strcmp("Remove-entry", strLine) )
							{
								Assert_( bFileTouched == false );
								Assert_( bFileNuked == false );
								bFileNuked = true;
								
								//  We pass FALSE for the delete file flag
								//  because we just want to revision info
								//  removed.
								
								ItemRemoved( NULL, 0, fsUpdateSpec, false );								
							}
						else if ( !strcmp("Login", strLine) )	
							{					
								ThrowOSErr( kCVSEngineBadPasswordErr );						
							}			
					}		/* while */	

				//  Reset the mod. date on the file so we don't touch it.

				if ( bFileTouched )
					{
						ResetFileDates( fsTargetSpec, intCreationDate, intModDate );
					}
				else if ( !bFileNuked )
					{
						result = kCVSEngineFileNotChangedErr;
						throw result;
					}
			}
		catch ( OSErr inErr )
			{		
				result = inErr;			
			}
			
		catch ( ... )
			{		
				result = kCVSEngineUnknownExceptionErr;			
			}
			
		if ( mNotifier != NULL )
			{
				if ( bCalledPreAction )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTargetSpec, kCVSEngineCommitPostActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );					
					}
			}
			
		if ( pCkidInfo != NULL )
			{			
				if ( pCkidInfo->fCommentHndl != NULL )
					{				
						::DisposeHandle( pCkidInfo->fCommentHndl );
						pCkidInfo->fCommentHndl = NULL;						
					}
					
				::DisposePtr( (Ptr)pCkidInfo );
				pCkidInfo = NULL;					
			}

		if ( hWinPosRsrc != NULL )
			{
				::DisposeHandle( hWinPosRsrc );
				hWinPosRsrc = NULL;
			}
			
		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;

		return result;
				
	}

	
OSErr
CCVSRepository::Diff( const FSSpec *inTargetSpec, const CCVSEngineDiffOptions *inOptions, CCVSEngineStreamNotifierProcPtr inDiffNotifier, void	*inContextPtr )
	{
	
		OSErr															result 												= noErr;

		char															strOffsetPath[kMaxPathSize]		= "";
		UInt32														intOffsetPathLen							= 0;
		
		Str255														pstrDiffOptions								= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255														pstrTranslated								= "\p";
#endif
		
		FSSpec														fsTargetSpec									= { 0, 0, "\p" };
		FSSpec														fsParentSpec									= { 0, 0, "\p" };

		CCVSEngineStreamNotifierProcPtr		notifier											= NULL;
		
		CInfoPBRec												pb;

		CUMakeDiffCommandLineOptionsString( inOptions, pstrDiffOptions, false);

		notifier = inDiffNotifier;
		
		if ( notifier == NULL )
			notifier = mMessageNotifier;
		
		try
			{
				if ( inTargetSpec == NULL )
					ThrowOSErr( kCVSEngineBadParamErr );		
			
				fsTargetSpec = *inTargetSpec;
				
				if ( FSpIsChildOf(&fsTargetSpec, &mLocalRoot) == false )
					ThrowOSErr( kCVSEngineBadParamErr );		
		
				result = FSpIsAFile( &fsTargetSpec, NULL );
				ThrowOSErr( result );
							
				result = FSpGetParentFSSpec( &fsTargetSpec, &fsParentSpec );
				ThrowOSErr( result );
							
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParentSpec, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );
				
				FSpSetDirectory( fsParentSpec );

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioFDirIndex = 0;
				pb.hFileInfo.ioNamePtr = fsTargetSpec.name;
				pb.hFileInfo.ioVRefNum = fsTargetSpec.vRefNum;
				pb.hFileInfo.ioDirID = fsTargetSpec.parID;
				
				result = ::PBGetCatInfoSync( &pb );
				ThrowOSErr( result );

				WriteItemEntry( pb, NULL, 0, true, true, true, true, fsRdPerm );

				if ( pstrDiffOptions[0] > 0 )
					{					
						mStream << kCVSArgumentToken;
						mStream << pstrDiffOptions;
						mStream << "\n";					
					}
					
				if ( inOptions && inOptions->mUseRevisions )
					{					
						mStream << kCVSArgumentToken;
						mStream << " -r";
						mStream << inOptions->mOrigRevisionPStr;
						mStream << "\n";

						mStream << kCVSArgumentToken;
						mStream << " -r";
						mStream << inOptions->mDiffRevisionPStr;
						mStream << "\n";					
					}

				FSpSetDirectory( mLocalRoot );
										
				mStream << kCVSArgumentToken;
				mStream << " ";

				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
				CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
				mStream << pstrTranslated;
#else
				mStream << fsTargetSpec.name;
#endif

				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTargetSpec, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				mStream << "diff\n";
				
				CCVSEngineFolderCreationOptions fcOptions;
				
				try
					{					
						ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, notifier, NULL, inContextPtr );			
					}
				catch ( OSErr intException )
					{
						//  Diff almost always returns a server error for some
						//  lame reason. So we suppress the error.
						
						if ( intException != kCVSEngineServerReturnedErr )
							{
								throw intException;
							}
					}
			}
			
		catch ( OSErr inErr )
			{			
				result = inErr;	
			}
		catch ( ... )
			{
				result = kCVSEngineUnknownExceptionErr;
			}

		if ( notifier != NULL )
			{
				(*notifier)( (CCVSEngineRef)this, 0, 0, NULL, inContextPtr );
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;
			
		return result;
								
	}


OSErr		
CCVSRepository::Edit( const FSSpec *inTargetSpec )
	{

		OSErr		result 												= noErr;
		char		strOffsetPath[kMaxPathSize]		= "";
		UInt32	intOffsetPathLen							= 0;
		UInt32	intSeconds										= 0;
		UInt32	intTimeStampLen								= 0;
				
		FSSpec	fsTarget											= { 0, 0, "\p" };
		FSSpec	fsParent											= { 0, 0, "\p" };
	
		Str255	pstrTimeStamp									= "\p";
		
		try
			{					
				if ( inTargetSpec == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTarget = *inTargetSpec;
								
				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
	
				//  Edit can work on a directory, but
				//  it's probably best to restrict it to
				//  a file-by-file basis for now.
				
				result = FSpIsAFile( &fsTarget, NULL );
				ThrowOSErr( result );
				
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
										
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParent, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );

				::GetDateTime( &intSeconds );
				
				intTimeStampLen = 255;
				result = CUISOTimeStrFromSeconds( (char *)&pstrTimeStamp[1], &intTimeStampLen, intSeconds );
				ThrowOSErr( result );
				pstrTimeStamp[0] = intTimeStampLen;
				
				FSpSetDirectory( fsParent );
				
				//  The notify command seems to only work in the
				//  target directory. So we don't set the directory
				//  to mLocalRoot before sending the command.
					
				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
							
				mStream << "Notify ";
				mStream << fsTarget.name;
				mStream << "\n";

				mStream << "E\t";
				mStream << pstrTimeStamp;
				mStream << " GMT";
				mStream << "\thostname\t";

				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}
	
				mStream << fsTarget.name;
				mStream << "\tEUC\n";

				//  Now set the root directory.
				
				FSpSetDirectory( mLocalRoot );						
				mStream << "noop\n";
				
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, NULL, NULL, NULL );		
				
				::FSpRstFLock( &fsTarget );						
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				Assert_( false );
				result = kCVSEngineUnknownExceptionErr;			
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
						mEntries = NULL;
					}
				catch ( ... )
					{
					}
			}
								
		return result;
		
	}


OSErr		
CCVSRepository::Editors( const FSSpec *inTargetSpec, CCVSEngineStreamNotifierProcPtr inNotifier, void *inContextPtr )
	{

		OSErr		result 												= noErr;
		char		strOffsetPath[kMaxPathSize]		= "";
		UInt32	intOffsetPathLen							= 0;
			
		FSSpec	fsTarget											= { 0, 0, "\p" };
		FSSpec	fsParent											= { 0, 0, "\p" };
		
		CCVSEngineStreamNotifierProcPtr pNotifier = mMessageNotifier;
	
		try
			{					
				if ( inTargetSpec == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTarget = *inTargetSpec;

				if ( inNotifier != NULL )
					{
						pNotifier = inNotifier;
					}
								
				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
										
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParent, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );
										
				WriteEntry( fsTarget, mLocalRoot, true, true, true, false, true );

				FSpSetDirectory( mLocalRoot );
				
				mStream << kCVSArgumentToken;				
				mStream << " ";
				
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}
					
				mStream << fsTarget.name;
				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				mStream << "editors\n";
						
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, pNotifier, NULL, inContextPtr );								
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				Assert_( false );
				result = kCVSEngineUnknownExceptionErr;			
			}
			
		if ( pNotifier != NULL )
			{
				(*pNotifier)( (CCVSEngineRef)this, 0, 0, NULL, inContextPtr );
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
						mEntries = NULL;
					}
				catch ( ... )
					{
					}
			}
					
		return result;
		
	}

	
OSErr
CCVSRepository::Import( const FSSpec *inTargetSpec, const CCVSEngineImportOptions *inOptions )
	{
	
		OSErr			result									= noErr;

		UInt32		pathLen									= 0;
		UInt32		lineStrLen							= 0;
		UInt32		encodingOptions					= kCVSEngineTextEncoding;
		UInt32		skippedBinaryFiles			= 0;
								
		FSSpec		targetSpec							= { 0, 0, "\p" };

		CCVSEngineFolderCreationOptions fcOptions = { false, false };
						
		try
			{
	
				if ( (inTargetSpec == NULL) || (inOptions == NULL) )
					ThrowOSErr( kCVSEngineBadParamErr );		

				targetSpec = *inTargetSpec;
															
				result = FSpIsAFile( &targetSpec, NULL );
				
				if ( result != kCVSEngineIsFolder )
					{				
						if ( result != noErr )
							throw result;							
						else ThrowOSErr( kCVSEngineBadParamErr );					
					}

				result = noErr;
					
				if ( FSpIsRepositoryDir(&targetSpec) )
					ThrowOSErr( kCVSEngineBadParamErr );
						
				if ( inOptions->fRepositoryPStr[0] == 0 )
					ThrowOSErr( kCVSEngineBadParamErr );

				if ( inOptions->fVendorTagPStr[0] == 0 )
					ThrowOSErr( kCVSEngineBadParamErr );

				if ( inOptions->fReleaseTagPStr[0] == 0 )
					ThrowOSErr( kCVSEngineBadParamErr );
				
				WriteImportEntries( targetSpec, inOptions->fSendBinaries, skippedBinaryFiles );

//  Send a simple message argument so that damn vi editor
//  won't come up on import.

				mStream << kCVSArgumentToken;
				mStream << " -m\n";
				mStream << kCVSArgumentToken;
				mStream << " First Imported.\n";

//  If we're sending binary data, send the flag.
							
				if ( inOptions->fSendBinaries )
					{
					
						mStream << kCVSArgumentToken;
						mStream << " ";
						mStream << kCVSBinaryOption;
						mStream << "\n";
				
					}

//  Send -I ! command to turn off ignoring files.

				mStream << kCVSArgumentToken;
				mStream << " -I !\n";

//  Send the branch tag if it's specified.
					
				if ( inOptions->fBranchTagPStr[0] > 0 )
					{
					
						mStream << kCVSArgumentToken;
						mStream << " -b\n";
	
						mStream << kCVSArgumentToken;
						mStream << " ";
						mStream << inOptions->fBranchTagPStr;
						mStream << "\n";
					
					}

//  Send directory, ventag, and reltag arguments.
					
				mStream << kCVSArgumentToken;
				mStream << " ";
				mStream << inOptions->fRepositoryPStr;
				mStream << "\n";

				mStream << kCVSArgumentToken;
				mStream << " ";
				mStream << inOptions->fVendorTagPStr;
				mStream << "\n";

				mStream << kCVSArgumentToken;
				mStream << " ";
				mStream << inOptions->fReleaseTagPStr;
				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, NULL, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		

				mStream << "import\n";		

				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, NULL, NULL, NULL );																								
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;			
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;
			
		return result;
	
	}
	
	
OSErr
CCVSRepository::Log( const FSSpec *inTargetSpec,  CCVSEngineStreamNotifierProcPtr	inLogNotifier, void *inContextPtr )
	{
	
		OSErr														result 												= noErr;
		
		char														strOffsetPath[kMaxPathSize]		= "";
		UInt32													intOffsetPathLen							= 0;

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255													pstrTranslated								= "\p";
#endif

		FSSpec													fsTargetSpec									= { 0, 0, "\p" };
		FSSpec													fsParentSpec									= { 0, 0, "\p" };
		CCVSEngineStreamNotifierProcPtr	notifier											= inLogNotifier;

		try
			{
				if ( notifier == NULL )
					notifier = mMessageNotifier;

				if ( inTargetSpec == NULL )
					ThrowOSErr( kCVSEngineBadParamErr );		
					
				fsTargetSpec = *inTargetSpec;
				
				if ( FSpIsChildOf( &fsTargetSpec, &mLocalRoot ) == false )
					ThrowOSErr( kCVSEngineBadParamErr );		
				
				result = FSpIsAFile( &fsTargetSpec, NULL );
				ThrowOSErr( result );
	
				result = FSpGetParentFSSpec( &fsTargetSpec, &fsParentSpec );
				ThrowOSErr( result );
							
				if ( mEntries != NULL )
					if ( !mEntries->AreYouThisDirectory(&fsParentSpec) )
						{					
							result = mEntries->CloseEntriesFile();
							ThrowOSErr( result );

							result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdPerm );							
							ThrowOSErr( result );							
						}
					
				if ( mEntries == NULL )
					{					
						mEntries = new CCVSEntriesCache( this );
					
						if ( mEntries != NULL )
							result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdPerm );
						else result = kCVSEngineOutOfMemoryErr;
						
						ThrowOSErr( result );					
					}
				
//  Make sure we have an entry.

				result = mEntries->GetEntry( fsTargetSpec.name, NULL, NULL, NULL, NULL );
				ThrowOSErr( result );
				
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParentSpec, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );
				
				//  We need to set the directory of the parent
				//  spec because if it's a virtual folder, problems
				//  will arise if we don't do this and attempt
				//  to reference the file wiht a full path.
				FSpSetDirectory( fsParentSpec );
				
				//  Get back to the root.
				FSpSetDirectory( mLocalRoot );

//  Turn off tag display since we don't use that data.

				mStream << kCVSArgumentToken;				
				mStream << " -N\n";

				mStream << kCVSArgumentToken;					
				mStream << " ";
				
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
				CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
				mStream << pstrTranslated;
#else
				mStream << fsTargetSpec.name;
#endif

				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTargetSpec, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				mStream << "log\n";
				
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, 0, fcOptions, notifier, NULL, inContextPtr );								
			}
		catch ( OSErr inErr )
			{			
				result = inErr;				
			}			
		catch ( ... )
			{		
				result = kCVSEngineUnknownExceptionErr;			
			}

		if ( inLogNotifier != NULL )
			(*inLogNotifier)( (CCVSEngineRef)this, 0, 0, NULL, inContextPtr );

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;
			
		return result;
						
	}


OSErr		
CCVSRepository::Remove( const FSSpec *inTargetSpec )
	{

//  Remove()

//  This function implements the CVS Remove operation. It will
//  remove a file from the remote repository. This function only
//  works on files.
	
		OSErr					result 											= noErr;

		Boolean				bCalledPreAction						= false;
		Boolean				bFileOpen										= false;
		
		char					strLine[kMaxLineSize]				= "";
		char					strOffsetPath[kMaxPathSize] = "";
		char					strLocalPath[kMaxPathSize]	= "";
		
		UInt32				intLineLen									= 0;
		UInt32				intOffsetPathLen						=	0;
		UInt32				intLocalPathLen							= 0;
		
		Str255				pstrRepository							= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255				pstrTranslated							= "\p";
#endif
																					
		FSSpec				fsTarget										= { 0, 0, "\p" };
		FSSpec				fsParent										= { 0, 0, "\p" };
					
		try
			{
				if ( inTargetSpec == NULL )
					ThrowOSErr( kCVSEngineBadParamErr );
					
				fsTarget = *inTargetSpec;
									
//  Make sure the file is in the local repository.

				if ( !FSpIsChildOf(&fsTarget, &mLocalRoot) )
					ThrowOSErr( kCVSEngineBadParamErr );
					
//  Make sure the passed in FSSpec is a file.

				result = FSpIsAFile( &fsTarget, &bFileOpen );
				ThrowOSErr( result );

//  Make sure the file is in a repository directory.
							
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );

				if ( !FSpIsRepositoryDir(&fsParent) )
					ThrowOSErr( kCVSEngineBadParamErr );
					
				//  Get the local path name of the file, relative to the project root.
				//  This is used for a parameter to the notifier routine, if one is
				//  installed. It is not necessarily the same path that is transmitted
				//  to the CVS server during the remove operation.
											
				intLocalPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsTarget, &mLocalRoot, strLocalPath, &intLocalPathLen );
				strLocalPath[intLocalPathLen] = 0;
			
				ThrowOSErr( result );

				Assert_( fsTarget.name[0] <= intLocalPathLen );
				
				if ( intLocalPathLen > fsTarget.name[0] )
					{					
						intOffsetPathLen = intLocalPathLen - (fsTarget.name[0] + 1);
						::BlockMoveData( strLocalPath, strOffsetPath, intOffsetPathLen );				
					}
								
				if ( mNotifier != NULL )
					{					
						if ( !(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineRemovePreActionEvent, noErr, intLocalPathLen, strLocalPath, mContextPtr ) )
							ThrowOSErr( kCVSEngineSkippedFile );
						
						bCalledPreAction = true;					
					}

				//  Send the entry for the file. Send merging data and conflict data 
				//  for safety. Don't send "Unchanged" messages or the operation
				//  won't work. 

				//  We now send binary data. It's redundant but safer.
							
				WriteEntry( fsTarget, mLocalRoot, true, false, true, true, false );
				
				if ( mEntries != NULL )
					{
						delete mEntries;
						mEntries = NULL;
					}
					
				FSpSetDirectory( mLocalRoot );
				
				//  Send the file name.
			
				mStream << kCVSArgumentToken;
				mStream << " ";
				
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}

#ifdef CVSENGINE_USE_MIME_FILE_NAMES		
				CUTranslateFromMacNamePStr( fsTarget.name, pstrTranslated );
				mStream << pstrTranslated;
#else
				mStream << fsTarget.name;
#endif

				mStream << "\n";

				//  Hasta la vista, baby!
										
				mStream << "remove\n";
				
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineTextAndBinaryMerging, fcOptions, NULL, NULL, NULL, true );					
			}
		catch ( OSErr inErr )
			{			
				result = inErr;				
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;		
			}

		if ( bCalledPreAction )
			if ( mNotifier != NULL )
				{				
					(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineRemovePostActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );					
				}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;
									
		return result;
								
	}


OSErr		
CCVSRepository::Status( const FSSpec *inTargetSpec, CCVSEngineStreamNotifierProcPtr inStatusNotifier, void *inContextPtr )
	{

		OSErr															result 												= noErr;
		char															strOffsetPath[kMaxPathSize]		= "";
		UInt32														intOffsetPathLen							= 0;

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255														pstrTranslated								= "\p";
#endif
			
		FSSpec														fsTarget											= { 0, 0, "\p" };
		FSSpec														fsParent											= { 0, 0, "\p" };
		
		CCVSEngineStreamNotifierProcPtr		notifier											= mMessageNotifier;
		CInfoPBRec												pb;

		try
			{
				if ( inStatusNotifier != NULL )
					notifier = inStatusNotifier;

				if ( inTargetSpec == NULL )
					ThrowOSErr( kCVSEngineBadParamErr );		

				fsTarget = *inTargetSpec;
								
				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					ThrowOSErr( kCVSEngineBadParamErr );		
			
				result = FSpIsAFile( &fsTarget, NULL );
				ThrowOSErr( result );
					
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
										
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParent, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );
					
				FSpSetDirectory( fsParent );
					
				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioFDirIndex = 0;
				pb.hFileInfo.ioNamePtr = fsTarget.name;
				pb.hFileInfo.ioVRefNum = fsTarget.vRefNum;
				pb.hFileInfo.ioDirID = fsTarget.parID;
				
				result = ::PBGetCatInfoSync( &pb );
				ThrowOSErr( result );

				//  We use WriteItemEntry() because WriteEntry() always
				//  opens the files with fsRdWrPerm.
				
				WriteItemEntry( pb, NULL, 0, true, true, true, true, fsRdPerm );

				FSpSetDirectory( mLocalRoot );
				
				mStream << kCVSArgumentToken;
				mStream << " -v\n";

				mStream << kCVSArgumentToken;				
				mStream << " ";
				
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
				CUTranslateFromMacNamePStr( fsTarget.name, pstrTranslated );
				mStream << pstrTranslated;
#else					
				mStream << fsTarget.name;
#endif

				mStream << "\n";

				if ( mEntries != NULL )
					{
						delete mEntries;
						mEntries = NULL;
					}

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				mStream << "status\n";
						
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, notifier, NULL, inContextPtr );								
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;			
			}
			
		if ( inStatusNotifier != NULL )
			{
				(*inStatusNotifier)( (CCVSEngineRef)this, 0, 0, NULL, inContextPtr );
			}
			
		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;
		
		return result;
		
	}
	

OSErr
CCVSRepository::Tag( const FSSpec *inTargetSpec, const CCVSEngineTagOptions *inOptions )
	{
		
		OSErr									result 									= noErr;
		Boolean								bIsaFile								= false;
		
		UInt32								intOffsetLen						= 0;
		Str255								pstrRepository					= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255								pstrTranslated					= "\p";
#endif

		char									strOffsetPath[kMaxPathSize]		= "";
		UInt32								intOffsetPathLen							= 0;
		
		FSSpec								fsTargetSpec						= { 0, 0, "\p" };
		FSSpec								fsParentSpec						= { 0, 0, "\p" };
		
		CInfoPBRec						pb;

		CCVSEngineTagOptions	theOptions;					
		theOptions.Init();
					
		try
			{
				if ( (inTargetSpec == NULL) || (inOptions == NULL) )
					ThrowOSErr( kCVSEngineBadParamErr );
					
				theOptions = *inOptions;
				fsTargetSpec = *inTargetSpec;
				
				if ( !FSpIsChildOf(&fsTargetSpec, &mLocalRoot) )
					ThrowOSErr( kCVSEngineBadParamErr );
					
				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= 0;
				pb.hFileInfo.ioNamePtr 			= fsTargetSpec.name;
				pb.hFileInfo.ioVRefNum 			= fsTargetSpec.vRefNum;
				pb.hFileInfo.ioDirID 				= fsTargetSpec.parID;

				result = ::PBGetCatInfoSync( &pb );
				ThrowOSErr( result );

				if ( !(pb.dirInfo.ioFlAttrib & ioDirMask) )
					{
						//  If we were passed in a file FSSpec, make sure 
						//  that we are in a repository folder.
					
						if ( !CUIsRepositoryDir(pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioFlParID) )
							ThrowOSErr( kCVSEngineBadParamErr );
							
						bIsaFile = true;										
					}						
				else if ( !CUIsRepositoryDir(pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}
					
				if ( bIsaFile )
					{					
						result = FSpGetParentFSSpec( &fsTargetSpec, &fsParentSpec );
						ThrowOSErr( result );

						intOffsetPathLen = kMaxPathSize - 1;
						result = FSpGetLocalCVSPath( &fsParentSpec, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
						ThrowOSErr( result );
					}
				else
					{
						intOffsetPathLen = kMaxPathSize - 1;
						result = FSpGetLocalCVSPath( &fsTargetSpec, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
						ThrowOSErr( result );
					}
					
				//  Send our file entries. We pass "true" for sending modified 
				//  binary data so that the "-c" option works correctly.
				
				if ( bIsaFile )
					{					
						WriteEntry( fsTargetSpec, mLocalRoot, true, theOptions.fTagDeep, true, true, true );				
					}
				else 
					{						
						WriteEntry( fsTargetSpec, mLocalRoot, true, theOptions.fTagDeep, true, true, true );			
					}
					
				if ( theOptions.fBranch == true )
					{					
						mStream << kCVSArgumentToken;
						mStream << " -b\n";				
					}

				if ( theOptions.fRemove == true )
					{			
						mStream << kCVSArgumentToken;
						mStream << " -d\n";				
					}
			
				if ( theOptions.fDontTagMROFiles == true )
					{					
						mStream << kCVSArgumentToken;
						mStream << " -c\n";				
					}
					
				if ( theOptions.fTagDeep == false )
					{					
						mStream << kCVSArgumentToken;
						mStream << " -l\n";						
					}

//  Send the actual tag...
					
				mStream << kCVSArgumentToken;
				mStream << " ";
				mStream << theOptions.fTagPStr;
				mStream << "\n";

				FSpSetDirectory( mLocalRoot );						

//  If it was a file, send the specific file argument.
//  Remember that WriteEntry() left the repository set
//  at our parent directory.
				
				if ( bIsaFile )
					{				
						mStream << kCVSArgumentToken;
						mStream << " ";
						
						if ( intOffsetPathLen > 0 )
							{
								mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
								mStream << "/";
							}
	
#ifdef CVSENGINE_USE_MIME_FILE_NAMES
						CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
						mStream << pstrTranslated;
#else
						mStream << fsTargetSpec.name;
#endif

						mStream << "\n";				
					}
				else
					{
						if ( intOffsetPathLen > 0 )
							{
								mStream << kCVSArgumentToken;
								mStream << " ";
								mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
								mStream << "\n";				
							}
					}
						
				mStream << "tag\n";
				
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, NULL, NULL, NULL );
			}
		catch ( OSErr inErr )
			{			
				result = inErr;	
			}
		catch ( ... )
			{
				result = kCVSEngineUnknownExceptionErr;
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;
									
		return result;

	}
	
	
SInt32
CCVSRepository::Version()
	{
	
		return kCurrentCVSEngineVersion;
	
	}
	

OSErr		
CCVSRepository::UnEdit( const FSSpec *inTargetSpec )
	{

		OSErr		result 												= noErr;
		char		strOffsetPath[kMaxPathSize]		= "";
		UInt32	intOffsetPathLen							= 0;
		UInt32	intSeconds										= 0;
		UInt32	intTimeStampLen								= 0;
				
		FSSpec	fsTarget											= { 0, 0, "\p" };
		FSSpec	fsParent											= { 0, 0, "\p" };
	
		Str255	pstrTimeStamp									= "\p";

		CCVSEngineUpdateOptions sOptions;
		sOptions.Init();
			
		try
			{					
				if ( inTargetSpec == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTarget = *inTargetSpec;
								
				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
	
				//  Edit can work on a directory, but
				//  it's probably best to restrict it to
				//  a file-by-file basis for now.
				
				result = FSpIsAFile( &fsTarget, NULL );
				ThrowOSErr( result );
				
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
										
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParent, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );

				::GetDateTime( &intSeconds );
				
				intTimeStampLen = 255;
				result = CUISOTimeStrFromSeconds( (char *)&pstrTimeStamp[1], &intTimeStampLen, intSeconds );
				ThrowOSErr( result );
				pstrTimeStamp[0] = intTimeStampLen;
				
				FSpSetDirectory( fsParent );
				
				//  The notify command seems to only work in the
				//  target directory. So we don't set the directory
				//  to mLocalRoot before sending the command.
					
				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
							
				mStream << "Notify ";
				mStream << fsTarget.name;
				mStream << "\n";

				mStream << "U\t";
				mStream << pstrTimeStamp;
				mStream << " GMT";
				mStream << "\thostname\t";

				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}
	
				mStream << fsTarget.name;
				mStream << "\tEUC\n";

				//  Now set the root directory.
				
				FSpSetDirectory( mLocalRoot );						
				mStream << "noop\n";
				
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, NULL, NULL, NULL );		
			
				if ( mEntries != NULL )
					if ( !mEntries->AreYouThisDirectory(&fsParent) )
						{						
							result = mEntries->CloseEntriesFile();
							ThrowOSErr( result );

							result = mEntries->LoadWorkingDirectory( &fsParent, fsRdPerm );							
							ThrowOSErr( result );								
						}
					
				if ( mEntries == NULL )
					{						
						mEntries = new CCVSEntriesCache( this );
					
						if ( mEntries != NULL )
							result = mEntries->LoadWorkingDirectory( &fsParent, fsRdPerm );
						else result = kCVSEngineOutOfMemoryErr;
						
						ThrowOSErr( result );						
					}
										
				result = mEntries->GetEntry( fsTarget.name, sOptions.mRevisionPStr, NULL, NULL, NULL );
				ThrowOSErr( result );
				
				result = mEntries->CloseEntriesFile();
				ThrowOSErr( result );					
				
				sOptions.mRevert = true;
				
				//  Revert the file using the Update function.
				
				result = Update( &fsTarget, &sOptions );
				ThrowOSErr( result );
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				Assert_( false );
				result = kCVSEngineUnknownExceptionErr;			
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
						mEntries = NULL;
					}
				catch ( ... )
					{
					}
			}
								
		return result;
		
	}

	
OSErr		
CCVSRepository::Update( const FSSpec *inTargetSpec, const CCVSEngineUpdateOptions *inOptions)
	{
		
		OSErr									result 											= noErr;

		Boolean								bUpdateFolderTags						= false;
		Boolean								bIsaFile										= false;
								
		char									strOffsetPath[kMaxPathSize] = "";

		UInt32								intOffsetPathLen						=	0;
		UInt32								intMergePolicy							= kCVSEngineNoMerging;
		
		Str255								pstrRepository							= "\p";
		Str255								pstrRevision								= "\p";
		Str255								pstrTimeStamp								= "\p";
		Str255								pstrOptions									= "\p";
		Str255								pstrTag											= "\p";
		Str255								pstrCompressionLevel				= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255								pstrTranslated							= "\p";
#endif
																	
		FSSpec								fsTargetSpec								= { 0, 0, "\p" };
		FSSpec								fsParentSpec								= { 0, 0, "\p" };
			
		CCVSEngineFolderCreationOptions		
													folderOptions								= { false, false };
		
		CInfoPBRec						pb;

		CCVSEngineUpdateOptions	theOptions;					
		theOptions.Init();
								
		try
			{
				if ( (inTargetSpec == NULL) || (inOptions == NULL) )
					{
						ThrowOSErr( kCVSEngineBadParamErr );
					}

				//  If the project root doesn't have a CVS folder,
				//  we better make one!
				
				InitRootDirectory();
					
				theOptions = *inOptions;
				fsTargetSpec = *inTargetSpec;
				
				//  LF translation is really ignored.

				Assert_( (theOptions.mLFTranslation == kCVSEngineLFTranslation) || (theOptions.mLFTranslation == kCVSEngineCRTranslation) );
				
				if ( theOptions.mLFTranslation > kCVSEngineCRTranslation )
					{
						theOptions.mLFTranslation = kCVSEngineLFTranslation;
					}
					
				Assert_( (theOptions.mMergePolicy == kCVSEngineNoMerging) || (theOptions.mMergePolicy == kCVSEngineTextMerging) || (theOptions.mMergePolicy == kCVSEngineTextAndBinaryMerging) );
				
				if ( theOptions.mMergePolicy > kCVSEngineTextAndBinaryMerging )
					{
						theOptions.mMergePolicy = kCVSEngineNoMerging;
					}
					
				if ( !FSpIsChildOf(&fsTargetSpec, &mLocalRoot) )
					{
						//  Allow fsTargetSpec to be the root for
						//  support of AppleEvent "update".
						if ( !FSpIsEqual(&fsTargetSpec, &mLocalRoot) )
							{
								ThrowOSErr( kCVSEngineBadParamErr );
							}
					}
				
//  First write entries for what we have already...

				pb.hFileInfo.ioCompletion 	= NULL;
				pb.hFileInfo.ioFDirIndex 		= 0;
				pb.hFileInfo.ioNamePtr 			= fsTargetSpec.name;
				pb.hFileInfo.ioVRefNum 			= fsTargetSpec.vRefNum;
				pb.hFileInfo.ioDirID 				= fsTargetSpec.parID;

				result = ::PBGetCatInfoSync( &pb );
				ThrowOSErr( result );

				if ( !(pb.dirInfo.ioFlAttrib & ioDirMask) )
					{

//  If we were passed in a file FSSpec, make sure that we are in
//  a repository folder.
							
						if ( !CUIsRepositoryDir(pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioFlParID) )
							ThrowOSErr( kCVSEngineBadParamErr );
							
						bIsaFile = true;

						result = FSpGetParentFSSpec( &fsTargetSpec, &fsParentSpec );
						ThrowOSErr( result );				
					}
				else if ( !CUIsRepositoryDir(pb.hFileInfo.ioVRefNum, pb.hFileInfo.ioDirID) )
					ThrowOSErr( kCVSEngineBadParamErr );

//  Ok, this is a hack: to implement "Revert", we rename the file and send
//  a request to update to the version we want to revert to. This way we can 
//  actually "checkout" a new copy of the file. Lastly, since that file will 
//  come back with a tag, we need to reset the tag to whatever it was before 
//  the revert. 

				if ( bIsaFile && theOptions.mRevert )
					{
						FSSpec fsRenamed = fsTargetSpec;

//  Since we don't use WriteItemEntry(), we need to
//  set the directory ourselves.
								
						FSpSetDirectory( fsParentSpec );
		
						if ( mEntries != NULL )
							if ( !mEntries->AreYouThisDirectory(&fsParentSpec) )
								{						
									result = mEntries->CloseEntriesFile();
									ThrowOSErr( result );

									result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdPerm );							
									ThrowOSErr( result );								
								}
							
						if ( mEntries == NULL )
							{						
								mEntries = new CCVSEntriesCache( this );
							
								if ( mEntries != NULL )
									result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdPerm );
								else result = kCVSEngineOutOfMemoryErr;
								
								ThrowOSErr( result );						
							}

						result = mEntries->GetEntry( fsTargetSpec.name, NULL, NULL, NULL, pstrTag );
						ThrowOSErr( result );
						
						result = mEntries->CloseEntriesFile();
						ThrowOSErr( result );

						result = ::FSpRstFLock( &fsTargetSpec );
						ThrowOSErr( result );
						
//  Move the old file out of the way
						
						result = FSpSafeRename( &fsRenamed );
						ThrowOSErr( result );
					}				
				else
					{

//  Else send our file entries. We pass "false" for sending modified binary data because
//  CVS will merge binary files that don't have the -kb option switch.
					
						if ( bIsaFile )
							{							
								WriteEntry( fsTargetSpec, mLocalRoot, (theOptions.mMergePolicy > kCVSEngineNoMerging), theOptions.mUpdateDeep, true, false, true );
							}
						else 
							{							
								WriteEntry( fsTargetSpec, mLocalRoot, (theOptions.mMergePolicy > kCVSEngineNoMerging), theOptions.mUpdateDeep, true, false, true );			
							}							
					}
							
//  Ok, here we need to get the offset path if we're running the
//  update command in a directory different than the root directory.

				if ( bIsaFile )
					{					
						intOffsetPathLen = kMaxPathSize - 1;
						result = FSpGetLocalCVSPath( &fsParentSpec, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
						ThrowOSErr( result );
					}
				else
					{					
						intOffsetPathLen = kMaxPathSize - 1;
						result = FSpGetLocalCVSPath( &fsTargetSpec, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
						ThrowOSErr( result );
					}

				FSpSetDirectory( mLocalRoot );
				
				//	Set the file options if we need
				
				if ( theOptions.mUseCompression && mCanUseFileCompression )
					{
						mStream << kCVSFileCompressionToken;
						mStream << " ";
						
						if ( (theOptions.mCompressionLevel >= 1) && (theOptions.mCompressionLevel <= 9) )
							{
								UInt32ToStr255( theOptions.mCompressionLevel, pstrCompressionLevel );
								mStream << pstrCompressionLevel;
							}
						else
							{
								mStream << "1";
							}
							
						mStream << "\n";
					}

//  Only update folder tags if the target item is a directory and
//  the client has specified to reset the tags.
			
				if ( (theOptions.mResetStickyTags == true) && (!bIsaFile) )
					bUpdateFolderTags = true;

				if ( theOptions.mResetStickyTags == true )
					{					
						mStream << kCVSArgumentToken;
						mStream << " -A\n";						
					}
			
//  WriteEntry will have already sent the "Directory" command so
//  we don't need to send it here.
		
				if ( theOptions.mUpdateDeep == false )
					{					
						mStream << kCVSArgumentToken;
						mStream << " -l\n";					
					}
					
				if ( theOptions.mGetNewDirectories == true )
					{						
						mStream << kCVSArgumentToken;
						mStream << " -d\n";									
					}		

				if ( theOptions.mRevisionPStr[0] > 0 )
					{
						mStream << kCVSArgumentToken;					
						mStream << " -r\n";
						
						mStream << kCVSArgumentToken;
						mStream << " ";
						mStream << theOptions.mRevisionPStr;
						mStream << "\n";						
					}

				if ( theOptions.mDatePStr[0] > 0 )
					{					
						mStream << kCVSArgumentToken;
						mStream << " -D\n";
						
						mStream << kCVSArgumentToken;
						mStream << " ";
						mStream << theOptions.mDatePStr;
						mStream << "\n";						
					}
													
				if ( bIsaFile )
					{				
						mStream << kCVSArgumentToken;
						mStream << " ";
			
						if ( intOffsetPathLen > 0 )
							{
								mStream.WriteBlock( strOffsetPath, intOffsetPathLen );	
								mStream << "/";
							}

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
						CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
						mStream << pstrTranslated;
#else							
						mStream << fsTargetSpec.name;
#endif

						mStream << "\n";					
					}
				else
					{
						if ( intOffsetPathLen > 0 )
							{
								mStream << kCVSArgumentToken;
								mStream << " ";
								mStream.WriteBlock( strOffsetPath, intOffsetPathLen );	
								mStream << "\n";						
							}					
					}
					
//  Close the entries database because it was probably opened 
//  with fsRdPerm in a WriteItemEntry() call. We need it open
//  with fsRdWrPerm to flush the changes we make.

				if ( mEntries != NULL )
					{
						delete mEntries;
						mEntries = NULL;
					}

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, NULL, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		

				mStream << "update\n";
			
				intMergePolicy = theOptions.mMergePolicy;
				
				if ( bIsaFile && theOptions.mRevert )
					intMergePolicy = kCVSEngineNoMerging;
									
				ProcessServerResponse( NULL, 0, intMergePolicy, folderOptions, NULL, NULL, NULL );
				
				if ( bIsaFile && theOptions.mRevert )
					{		
						//  Sanity check: make sure we actually downloaded the file
						
						result = FSpIsAFile( &fsTargetSpec, NULL );
						ThrowOSErr( result );
									
						if ( mEntries != NULL )
							if ( !mEntries->AreYouThisDirectory(&fsParentSpec) )
								{						
									result = mEntries->CloseEntriesFile();
									ThrowOSErr( result );

									result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdWrPerm );							
									ThrowOSErr( result );								
								}
							
						if ( mEntries == NULL )
							{						
								mEntries = new CCVSEntriesCache( this );
							
								if ( mEntries != NULL )
									result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdWrPerm );
								else result = kCVSEngineOutOfMemoryErr;
								
								ThrowOSErr( result );						
							}

						result = mEntries->GetEntry( fsTargetSpec.name, pstrRevision, pstrTimeStamp, pstrOptions, NULL );
						ThrowOSErr( result );

						result = mEntries->AddModifyEntry( fsTargetSpec.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag, false );				
						ThrowOSErr( result );
						
						result = mEntries->FlushEntriesFile();
						ThrowOSErr( result );
					}
			
			}
		catch ( OSErr inErr )
			{			
				result = inErr;			
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;		
			}
			
		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
					}
				catch ( ... )
					{
					}
			}
		
		mEntries = NULL;

		return result;
								
	}
	

OSErr
CCVSRepository::WatchAddRemove( const FSSpec *inTargetSpec, const CCVSEngineWatchOptions *inOptions )
	{
	
		OSErr				result 												= noErr;
		char				strOffsetPath[kMaxPathSize]		= "";
		UInt32			intOffsetPathLen							= 0;
			
		FSSpec			fsTarget											= { 0, 0, "\p" };
		FSSpec			fsParent											= { 0, 0, "\p" };
	
		try
			{
				if ( inTargetSpec == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTarget = *inTargetSpec;
								
				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
							
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
										
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParent, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );
					
				WriteEntry( fsTarget, mLocalRoot, true, true, true, false, true );
				FSpSetDirectory( mLocalRoot );
									
				mStream << kCVSArgumentToken;				
				mStream << " ";
						
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}
					
				mStream << fsTarget.name;
				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				if ( inOptions->mNotification )
					{		
						mStream << "watch-add\n";					
					}				
				else
					{
						mStream << "watch-remove\n";
					}
							
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, NULL, NULL, NULL );								
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;			
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
						mEntries = NULL;
					}
				catch ( ... )
					{
					}
			}
				
		return result;	
		
	}
	

OSErr
CCVSRepository::WatchOnOff( const FSSpec *inTargetSpec, const Boolean inWatch )
	{
	
		OSErr				result 												= noErr;
		char				strOffsetPath[kMaxPathSize]		= "";
		UInt32			intOffsetPathLen							= 0;

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255			pstrTranslated								= "\p";
#endif
			
		FSSpec			fsTarget											= { 0, 0, "\p" };
		FSSpec			fsParent											= { 0, 0, "\p" };

		try
			{
				if ( inTargetSpec == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTarget = *inTargetSpec;
								
				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
							
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
										
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParent, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );
					
				WriteEntry( fsTarget, mLocalRoot, true, true, true, false, true );
				FSpSetDirectory( mLocalRoot );
									
				mStream << kCVSArgumentToken;				
				mStream << " ";
						
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
				CUTranslateFromMacNamePStr( fsTarget.name, pstrTranslated );
				mStream << pstrTranslated;
#else					
				mStream << fsTarget.name;
#endif

				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				if ( inWatch )
					{		
						mStream << "watch-on\n";					
					}				
				else
					{
						mStream << "watch-off\n";
					}
							
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, NULL, NULL, NULL );								
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;			
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
						mEntries = NULL;
					}
				catch ( ... )
					{
					}
			}
				
		return result;		
	
	}
		

OSErr		
CCVSRepository::Watchers( const FSSpec *inTargetSpec, CCVSEngineStreamNotifierProcPtr inNotifier, void *inContextPtr )
	{

		OSErr		result 												= noErr;
		char		strOffsetPath[kMaxPathSize]		= "";
		UInt32	intOffsetPathLen							= 0;
			
		FSSpec	fsTarget											= { 0, 0, "\p" };
		FSSpec	fsParent											= { 0, 0, "\p" };
		
		CCVSEngineStreamNotifierProcPtr pNotifier = mMessageNotifier;

		try
			{
				if ( inNotifier != NULL )
					{
						pNotifier = inNotifier;
					}
					
				if ( inTargetSpec == NULL )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				fsTarget = *inTargetSpec;
								
				if ( FSpIsChildOf( &fsTarget, &mLocalRoot ) == false )
					{
						ThrowOSErr( kCVSEngineBadParamErr );		
					}
					
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
										
				intOffsetPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsParent, &mLocalRoot, strOffsetPath, &intOffsetPathLen );
				ThrowOSErr( result );
										
				WriteEntry( fsTarget, mLocalRoot, true, true, true, false, true );

				FSpSetDirectory( mLocalRoot );
				
				mStream << kCVSArgumentToken;				
				mStream << " ";
				
				if ( intOffsetPathLen > 0 )
					{
						mStream.WriteBlock( strOffsetPath, intOffsetPathLen );
						mStream << "/";
					}
					
				mStream << fsTarget.name;
				mStream << "\n";

				if ( mNotifier != NULL )
					{
						(*mNotifier)( (CCVSEngineRef)this, &fsTarget, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
					}		
				
				mStream << "watchers\n";
						
				CCVSEngineFolderCreationOptions fcOptions;
				ProcessServerResponse( NULL, 0, kCVSEngineNoMerging, fcOptions, pNotifier, NULL, inContextPtr );								
			}
		catch ( OSErr inErr )
			{		
				result = inErr;				
			}			
		catch ( ... )
			{			
				result = kCVSEngineUnknownExceptionErr;			
			}
			
		if ( pNotifier != NULL )
			{
				(*pNotifier)( (CCVSEngineRef)this, 0, 0, NULL, inContextPtr );
			}

		if ( mEntries != NULL )
			{
				try
					{
						delete mEntries;
						mEntries = NULL;
					}
				catch ( ... )
					{
					}
			}
					
		return result;
		
	}
	

#pragma mark -


void
CCVSRepository::NegotiateServerOptions()
	{

//  NegotiateServerOptions() 

//  This routine simply tells the CVS server which commands
//  we support and queries it for the commands that it supports.

//	Throws Types: CTCPSocketErr, CCVSServerStreamErr, CCVSRepositoryErr, OSErr

		OSErr			result 									= noErr;
		Boolean		bDone										= false;
		Boolean		bSupportsDirectory			= false;
		Boolean		bSupportsUseUnchanged		= false;
		Boolean		bSupportsCompression		= false;
		
		char			strLine[kMaxLineSize] 	= "";
		UInt32		intLineLen							= 0;

//  First send the options that we support

		mStream << kCVSValidResponsesToken;
		mStream << " ";
		mStream << kCVSValidRequestsToken2;
		mStream << " ";
		mStream << kCVSDoneToken;
		mStream << " ";
		mStream << kCVSErrorToken;
		mStream << " ";
		mStream << kCVSMessageOutToken;
		mStream << " ";
		mStream << kCVSErrorOutToken;
		mStream << " ";
		mStream << kCVSUpdatedToken;
		mStream << " ";
		mStream << kCVSCheckedInToken;
		mStream << " ";
		mStream << kCVSNewEntryToken;
		mStream << " ";
		mStream << kCVSMergedToken;
		mStream << " ";
		mStream << kCVSRemovedToken;
		mStream << " ";
		mStream << kCVSModuleExpansionToken;
		mStream << " ";
		mStream << kCVSRemoveEntryToken;
		mStream << " ";
		mStream << kCVSSetStaticDirToken;
		mStream << " ";
		mStream << kCVSClearStaticDirToken;
		mStream << " ";		
		mStream << kCVSSetStickyToken;
		mStream << " ";
		mStream << kCVSClearStickyToken;
		mStream << " ";		
		mStream << kCVSFileCompressionToken;
		mStream << "\n";

//  Now ask the server for the commands it supports

		mStream << kCVSValidRequestsToken;
		mStream << "\n";

//  Just look for "Directory" and "UseUnchanged". This will need to be
//  worked on later.
		
		while ( !bDone )
			{			
				Yield();

				intLineLen = kMaxLineSize - 1;
				mStream.ReadWord( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				if ( (intLineLen == kCVSDoneToken[0]) && !strncmp(strLine, (char *)&kCVSDoneToken[1], kCVSDoneToken[0]) )
					bDone = true;

				else if ( (intLineLen == kCVSErrorToken[0]) && !strncmp(strLine, (char *)&kCVSErrorToken[1], kCVSErrorToken[0]) )
					{				
						bDone = true;

						if ( result == noErr )
							result = kCVSEngineServerReturnedErr;
					
						throw result;						
					}
					
				else if ( (intLineLen == kCVSMessageOutToken[0]) && !strncmp(strLine, (char *)&kCVSMessageOutToken[1], kCVSMessageOutToken[0]) )
					{
						intLineLen = kMaxLineSize - 1;
						mStream.ReadEntireDataLine( strLine, intLineLen );
						strLine[intLineLen] = 0;
				
						MessageOut( NULL, strLine, intLineLen, NULL );													
					}

				else if ( (intLineLen == kCVSErrorOutToken[0]) && !strncmp(strLine, (char *)&kCVSErrorOutToken[1], kCVSErrorOutToken[0]) )
					{
						intLineLen = kMaxLineSize - 1;
						mStream.ReadEntireDataLine( strLine, intLineLen );
						strLine[intLineLen] = 0;
					
						ErrorOut( NULL, strLine, intLineLen, NULL );					
					}
					
				else if ( (intLineLen == kCVSDirectoryToken[0]) && !strncmp(strLine, (char *)&kCVSDirectoryToken[1], kCVSDirectoryToken[0]) )
					{		
						bSupportsDirectory = true;					
					}

				else if ( (intLineLen == kCVSUseUnchangedToken[0]) && !strncmp(strLine, (char *)&kCVSUseUnchangedToken[1], kCVSUseUnchangedToken[0]) )
					{			
						bSupportsUseUnchanged = true;					
					}		
				else if ( (intLineLen == kCVSFileCompressionToken[0]) && !strncmp(strLine, (char *)&kCVSFileCompressionToken[1], kCVSFileCompressionToken[0]) )
					{					
						bSupportsCompression = true;					
					}
			
			}
	
		Assert_( (bSupportsDirectory) && (bSupportsUseUnchanged) );
		
		if ( (!bSupportsDirectory) || (!bSupportsUseUnchanged) )
			ThrowOSErr( kCVSEngineServerGaveBadResponseErr );
			
		mStream << kCVSUseUnchangedToken;
		mStream << "\n";

// set the file compression flag in the repository object
		
		mCanUseFileCompression = bSupportsCompression;

	}
	
	
void
CCVSRepository::ProcessServerResponse( 	const char														*strInOffsetPath, 
																				const UInt32 													intInOffsetPathLen, 
																				const UInt32 													intInMergePolicy, 
																				const CCVSEngineFolderCreationOptions &foInFolderOptions,
																				CCVSEngineStreamNotifierProcPtr				inMessageNotifier, 
																				CCVSEngineStreamNotifierProcPtr 			inErrorNotifier,
																				void																	*inContextPtr,
																				const Boolean													bInUpdateAnyVersion,
																				const Boolean													bInAddRevisionInfo,
																				const Boolean													bInResetMRO)
	{
	
		OSErr		result								= noErr;
		Boolean bDone 								= false;
		
		char		strLine[kMaxLineSize] = "";
		UInt32	intLineLen						= 0;
		FSSpec	fsFileSpec						= { 0, 0, "\p" };
		
		CCVSEngineStreamNotifierProcPtr	mesgNotifier = inMessageNotifier;
		CCVSEngineStreamNotifierProcPtr errNotifier = inErrorNotifier;
		void *pContext = inContextPtr;
		
		if ( mesgNotifier == NULL )
			mesgNotifier = mMessageNotifier;
			
		if ( errNotifier == NULL )
			errNotifier = mErrorNotifier;
		
		if ( pContext == NULL )
			pContext = mContextPtr;
				
		try
			{			
				while ( !bDone )
					{
						Yield();
											
						intLineLen = kMaxLineSize - 1;
						mStream.ReadWord( strLine, intLineLen );
						strLine[intLineLen] = 0;
						
						if ( !strcmp("ok", strLine) )
							{
								bDone = true;	
							}
						else if ( !strcmp("error", strLine) )
							{					
								bDone = true;
								
								if ( result == noErr )
									result = kCVSEngineServerReturnedErr;
								
								throw result;	
							}											
						else if ( !strcmp("M", strLine) )
							{					
								intLineLen = kMaxLineSize - 1;
								mStream.ReadEntireDataLine( strLine, intLineLen );
								strLine[intLineLen] = 0;

								MessageOut( mesgNotifier, strLine, intLineLen, pContext );
							}						
						else if ( !strcmp("E", strLine) )
							{	
								intLineLen = kMaxLineSize - 1;
								mStream.ReadEntireDataLine( strLine, intLineLen );
								strLine[intLineLen] = 0;
						
								//  Don't pass the specified context ptr to the 
								//  error notifier.
								
								ErrorOut( errNotifier, strLine, intLineLen, mContextPtr );									
							}																
						else if ( !strcmp("Updated", strLine) || !strcmp("Merged", strLine) )
							{	
								try
									{
										ItemUpdatedMerged( mRemoteUserName, intInMergePolicy, false, strInOffsetPath, intInOffsetPathLen, foInFolderOptions, fsFileSpec );
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}
							}
						else if ( !strcmp("Checked-in", strLine) )
							{
								try
									{
										ItemCheckedIn( mRemoteUserName, intInMergePolicy, false, strInOffsetPath, intInOffsetPathLen, fsFileSpec, bInUpdateAnyVersion, bInAddRevisionInfo, bInResetMRO );						
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}
							}
						else if ( !strcmp("New-entry", strLine) )
							{
								//  New-entry is just like a "Checked-in" response,
								//  except that the file is not up to date.
								
								try
									{
										ItemCheckedIn( mRemoteUserName, intInMergePolicy, false, strInOffsetPath, intInOffsetPathLen, fsFileSpec, bInUpdateAnyVersion, false, false );						
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}
							}
						else if ( !strcmp("Remove-entry", strLine) )
							{
								try
									{
										ItemRemoved( strInOffsetPath, intInOffsetPathLen, fsFileSpec );
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}							
							}
						else if ( !strcmp("Removed", strLine) )
							{  //  Do we want to do this?
								try
									{
										ItemRemoved( strInOffsetPath, intInOffsetPathLen, fsFileSpec );
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}							
							}
						else if ( !strcmp("Module-expansion", strLine) )
							{
								ModuleExpansion( intInMergePolicy, true );
							}					
						else if ( !strcmp("Clear-sticky", strLine) )	
							{		
								try
									{			
										SetClearStickyDir( false, mRemoteUserName, strInOffsetPath, intInOffsetPathLen, foInFolderOptions, fsFileSpec );										
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}														
							}	
						else if ( !strcmp("Set-sticky", strLine) )	
							{			
								try
									{		
										SetClearStickyDir( true, mRemoteUserName, strInOffsetPath, intInOffsetPathLen, foInFolderOptions, fsFileSpec );																		
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}							
							}					
						else if ( !strcmp("Clear-static-directory", strLine) )	
							{		
								try
									{			
										SetClearStaticDir( false, mRemoteUserName, strInOffsetPath, intInOffsetPathLen, foInFolderOptions, fsFileSpec );										
									}
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}							
							}	
						else if ( !strcmp("Set-static-directory", strLine) )	
							{				
								try
									{	
										SetClearStaticDir( true, mRemoteUserName, strInOffsetPath, intInOffsetPathLen, foInFolderOptions, fsFileSpec );																		
									}							
								catch ( OSErr intException )
									{
										if ( IsFatalException(intException) )
											{
												throw intException;
											}
									}							
							}					
						else if ( !strcmp("Login", strLine) )	
							{					
								ThrowOSErr( kCVSEngineBadPasswordErr );						
							}			
					}			
			}			
		catch ( OSErr intException )
			{		
				throw intException;					
			}
			
	}


Boolean
CCVSRepository::IsFatalException( OSErr inErr )
	{

		Boolean	bResult = false;
		
		if ( 	(inErr == kCVSEngineOperationAbortedErr) || 
					(inErr == kCVSEngineServerGaveBadResponseErr) ||
					(inErr == kCVSEngineServerReturnedErr) ||
					(inErr == kCVSEngineBadPasswordErr) || 
					(inErr == dskFulErr) || 
					(inErr == dirFulErr) || 
					(inErr == nsvErr) || 
					(inErr == ioErr) ||
					IsCTCPSocketErr(inErr) )
			{
				bResult = true;
			}
	
		return bResult;
		
	}
	
	
#pragma mark -


void
CCVSRepository::ItemUpdatedMerged(	 		const Str255 						inUserNamePStr, 
																				const UInt32						inMergePolicy, 
																				const Boolean						inUpdateParentTag,
																				const char							*inOffsetPathStr,
																				const UInt32						inOffsetPathLen,
																				const CCVSEngineFolderCreationOptions 
																																&folderOptions,
																				FSSpec 									&outTargetSpec)
	{

		const													kReadNothing									= 0;
		const													kReadLocalPath								= 1;
		const													kReadRemotePath								= 2;
		const													kReadHeader										= 3;
		const													kReadPermissions							= 4;
		const													kReadFileSize									= 5;
		const													kReadFile											= 6;
													
		OSErr													result												= noErr;
		
		Boolean												bCreatedTarget								= false;
		Boolean												bBinaryFile										= false;
		Boolean												bCalledPreAction							= false;
		Boolean												bModified											= false;
		Boolean												bReadEntireLine								= false;
		Boolean												bHasCkid											= false;
		Boolean												bTruncated										= false;
		Boolean												bFileWasLocked								= false;
		Boolean												bCheckedOutReadOnly						= false;
		Boolean												bMustUncompress								= false;
		Boolean												bSniffedAppleSingle						= false;
		
		char													strLine[kMaxLineSize]					= "";
		char													strLocalPath[kMaxPathSize]		= "";
		char													strRemotePath[kMaxPathSize]		= "";
		char													strEntryLine[kMaxLineSize]		= "";
		
		Str255												pstrFileName									= "\p";
		Str255												pstrRevision									= "\p";
		Str255												pstrTimeStamp									= "\p";
		Str255												pstrOptions										= "\p";
		Str255												pstrTag												= "\p";
		
		SInt32												intModDate										= 0;		
		UInt32												intLineLen										= 0;
		UInt32												intLocalPathLen								= 0;
		UInt32												intRemotePathLen							= 0;
		UInt32												intEntryLineLen								= 0;
		UInt32												intCopyLen										= 0;
		UInt32												intFileSize										= 0;
		UInt32												intBytesToGet									= 0;
		UInt32												intBytesWeGot									= 0;
		UInt32												intCurState										= kReadNothing;
		UInt32												intCommentLen									= 0;
		UInt32												intEncodingType								= 0;
			
		Handle												hComment											= NULL;
		FSSpec												fsParentSpec									= { 0, 0, "\p" };
		CkidResourcePB								*pCkidInfo										= NULL;

		CCVSEngineFileMappingRec			sMapping;
		CCVSEngineProgressStatusRec		sCurStatus										= { 0, 0 };

		sMapping.Init();
		
		outTargetSpec.name[0] = 0;
		outTargetSpec.parID = 0;
		outTargetSpec.vRefNum = 0;
			
		try
			{
				pCkidInfo = (CkidResourcePB *)::NewPtr( sizeof(CkidResourcePB) );
				
				if ( pCkidInfo == NULL )
					throw( (OSErr)::MemError() );

				pCkidInfo->fCommentHndl = NULL;

//  Read the local directory from the stream
				
				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadLocalPath;
				
				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{
							
						::BlockMoveData( strLine, strLocalPath, intLineLen );
						intLocalPathLen = intLineLen;
						strLocalPath[intLocalPathLen] = 0;
						
					}
				else ThrowOSErr( kCVSEnginePathOverflow );
			
//  Read the remote path name from the stream

				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadRemotePath;

				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{

//  The path returned should always be longer than the remote
//  root path name. If it isn't, something is seriously wrong.					
					
						if ( intLineLen < (mRemoteRoot[0] + 1) )
							ThrowOSErr( kCVSEngineServerGaveBadResponseErr );

						::BlockMoveData( strLine, strRemotePath, intLineLen );
						intRemotePathLen = intLineLen;
						strRemotePath[intRemotePathLen] = 0;
						
					}
				else ThrowOSErr( kCVSEnginePathOverflow );
	
//  Now that we have both the local and remote
//  path names, extract the file name

				CUFastGetLastCVSPathItem( strRemotePath, intRemotePathLen, pstrFileName );

//  Read CVS Entry Line

				intEntryLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strEntryLine, intEntryLineLen );
				strEntryLine[intEntryLineLen] = 0;
				
				Assert_( bReadEntireLine );
								
				intCurState = kReadHeader;

				Assert_( strEntryLine[0] != 'D' );

#ifdef CVSENGINE_USE_MIME_FILE_NAMES

				{
					Str255	pstrNewFileName 	= "\p";
					UInt32	intNewFileNameLen = 0;
					
					PStrCopy(pstrFileName, pstrNewFileName, sizeof(pstrNewFileName));
					intNewFileNameLen = pstrNewFileName[0];
					
					CUTranslateToMacName((char*)&pstrNewFileName[1], intNewFileNameLen);
					pstrNewFileName[0] = intNewFileNameLen;
					
					mMappings.GetMapping( pstrNewFileName, sMapping, false );
				}

#else				
				mMappings.GetMapping( pstrFileName, sMapping, false );
#endif

				Assert_( (sMapping.fTranslation >= kCVSEngineLFTranslation) && (sMapping.fTranslation <= kCVSEngineCRTranslation) );
				Assert_( (sMapping.fEncoding >= kCVSEngineTextEncoding) && (sMapping.fEncoding <= kCVSEngineMacBinaryEncoding) );
						
				intEncodingType = sMapping.fEncoding;
				
//  Okay, now we should have a local path name to the directory of the file we
//  are updating, and a remote path name and file name to the file. Pass it to
//  CreateItemFromCVSLocalPath() to do the magic of creating the folder hierarchy, etc.

//  Even if we get an error from CreateItemFromCVSPath(), keep going until
//  we call the client notifier, then throw it afterwards.

				CCVSEngineFileCreateInfo fileInfo( sMapping.fFileCreator, sMapping.fFileType, smSystemScript, folderOptions );
							
				result = CUCreateItemFromCVSPath(	mRemoteUserName, 
																					mServerName, 
																					mPort, 
																					mAuthMethod,
																					strLocalPath, 
																					intLocalPathLen, 
																					strRemotePath, 
																					intRemotePathLen, 
																					inOffsetPathStr, 
																					inOffsetPathLen,
																					(char *)&mRemoteRoot[1], 
																					mRemoteRoot[0], 
																					&mLocalRoot, 
																					&outTargetSpec, 
																					&fileInfo,
																				  &bCreatedTarget );

//  Now we can call StripCVSPath() on the local path to make sure
//  it is clean. We do this so we set the mcvs info properly.

				CUStripCVSPath( strLocalPath, &intLocalPathLen );

//  If the CVS command was not run in the root directory, we need
//  to deal with this because it screws with the path names.
	
				if ( (inOffsetPathStr != NULL) && (inOffsetPathLen > 0) )
					{						
						if ( intLocalPathLen > 0 )
							{
								if ( strLocalPath[intLocalPathLen - 1] == '/' )
									{
										intLocalPathLen--;
									}
							}
							
						intCopyLen = kMaxPathSize - (intLocalPathLen + 1);
						
						if ( intCopyLen > inOffsetPathLen )
							{
								intCopyLen = inOffsetPathLen;
							}
							
						if ( intCopyLen > 0 )
							{									
								if ( intLocalPathLen > 0 )
									{									
										::BlockMoveData( strLocalPath, &strLocalPath[intCopyLen + 1], intLocalPathLen );
										strLocalPath[intCopyLen] = '/';
										intLocalPathLen++;
									}
									
								::BlockMoveData( inOffsetPathStr, strLocalPath, intCopyLen );
								intLocalPathLen += intCopyLen;					
							}				
					}

				if ( (intLocalPathLen > 0) && (intLocalPathLen < kMaxPathSize) && (strLocalPath[intLocalPathLen - 1] != '/') )
					strLocalPath[intLocalPathLen++] = '/';

				intCopyLen = kMaxPathSize - (intLocalPathLen + 1 );
				
				if ( intCopyLen > pstrFileName[0] )
					intCopyLen = pstrFileName[0];
					
				if ( intCopyLen > 0 )
					{
						::BlockMoveData( &pstrFileName[1], &strLocalPath[intLocalPathLen], intCopyLen );
						intLocalPathLen += intCopyLen;						
					}
																								
//  Call the notifier pre-action. Do it before we throw for errors.
					
				if ( mNotifier != NULL )
					{										
						//if ( result == noErr )
						(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePreActionEvent, noErr, intLocalPathLen, strLocalPath, mContextPtr );

						bCalledPreAction = true;												
					}

				if ( result == kCVSEngineFileNameTruncatedErr )
					{
						result = noErr;
						bTruncated = true;
					}
					
				ThrowOSErr( result );

				//  This should be optimized
								
				result = FSpGetParentFSSpec( &outTargetSpec, &fsParentSpec );
				ThrowOSErr( result );
					
				Assert_( outTargetSpec.name[0] > 0 );
				Assert_( outTargetSpec.vRefNum != 0 );
				Assert_( outTargetSpec.parID != 0 );
				
				{						
					CInfoPBRec pb;
								
					pb.hFileInfo.ioCompletion = NULL;
					pb.hFileInfo.ioFDirIndex = 0;
					pb.hFileInfo.ioNamePtr = outTargetSpec.name;
					pb.hFileInfo.ioVRefNum = outTargetSpec.vRefNum;
					pb.hFileInfo.ioDirID = outTargetSpec.parID;
					
					result = ::PBGetCatInfoSync( &pb );				
					ThrowOSErr( result );

//  Ok, the only way to truly tell if a file is open without calling
//  FSpOpenDF, FSpOpenRF, *and* FSpOpenResFile is to check the flags.
										
					if ( FILE_OPEN(pb) )
						{					
							result = opWrErr;
							ThrowOSErr( result );						
						}
						
					if ( pb.hFileInfo.ioFlAttrib & 1 )
						{
							//  If the file is locked, we will unlock it for
							//  the duration of this function. This does not
							//  necessary mean we will modify the file.
							
							bFileWasLocked = true;
							
							result = FSpRstFLock( &outTargetSpec );
							ThrowOSErr( result );
						}
						
					intModDate = pb.hFileInfo.ioFlMdDat;										
				}
					
				//  Open data fork with write access to lock file
				//  from other threads. Scope *is* important here 
				//  because destruction of the StFileAccess object 
				//  will close the file.
								
				{
					SInt16				dataForkRef		= 0;					
					StFileAccess	updateFileAccess( outTargetSpec );
					
					try
						{						
							dataForkRef = updateFileAccess.GetFSDataForkRef();
						}						
					catch ( OSErr inErr )
						{					
							result = inErr;
							throw inErr;						
						}

//  If we created the file, set the delete flag 
//  so if anything goes wrong, we delete it
										
					if ( bCreatedTarget )					
						updateFileAccess.SetDelete( true );

//  Since CCVSEntriesCache::LoadWorkingDirectory() can
//  potentially Yield, we call it after we've opened our
//  file.

					if ( mEntries != NULL )
						if ( !mEntries->AreYouThisDirectory(&fsParentSpec) )
							{							
								result = mEntries->CloseEntriesFile();
								ThrowOSErr( result );

								result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdWrPerm );							
								ThrowOSErr( result );								
							}
						
					if ( mEntries == NULL )
						{							
							mEntries = new CCVSEntriesCache( this );
						
							if ( mEntries != NULL )
								result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdWrPerm );
							else result = kCVSEngineOutOfMemoryErr;
							
							ThrowOSErr( result );							
						}
					
					if ( !bCreatedTarget )
						{		
							//  Here we check if the file is locally modified.
							//  We get the ckid (if it has one), time stamp from
							//  the entries file, and the actual mod. date to make
							//  the determination.

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
							result = mEntries->GetEntry( outTargetSpec.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
#else							
							result = mEntries->GetEntry( pstrFileName, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
#endif
							ThrowOSErr( result );
										
							result = FSpReadCkidResource( &outTargetSpec, pCkidInfo, true );					
							
							if ( result == noErr )
								{							
									bHasCkid = true;

									//  I'm assuming we should bail if this happens.
									if ( pCkidInfo->fCheckedOutBySomeone > 0 )
										{
											ThrowOSErr( kCVSEnginePermissionsErr );
										}
								}
							else 
								{
									result = noErr;							
								}

							if ( bHasCkid )
								{
									result = CUIsFileModified( pCkidInfo, pstrTimeStamp, intModDate );
								}
							else
								{
									result = CUIsFileModified( NULL, pstrTimeStamp, intModDate );		
								}
								
							if ( result == kCVSEngineFileLocallyModifiedErr )
								{
									bModified = true;
									result = noErr;
								}
														
							ThrowOSErr( result );
						}
					
					if ( !bCreatedTarget )
						{
							
//  Here we could to see if the local file is the same as
//  the remote file and bail, as we did in MacCVS 2.2 and earlier.
													
//  Check our merging policy to see if we should use the new file.

							if ( bModified )
								{
//  Check for binary file

									bBinaryFile = CUIsCVSBinaryOption( pstrOptions );
							
//  No merging options were specified, so bail...
								
									if ( (inMergePolicy != kCVSEngineTextMerging) && (inMergePolicy != kCVSEngineTextAndBinaryMerging) )
										{								
											ThrowOSErr( kCVSEnginePermissionsErr );
										}

//  The user specified only text merging and this is a "merged" binary...
										
									else if ( (inMergePolicy != kCVSEngineTextAndBinaryMerging) && (bBinaryFile) )
										{										
											ThrowOSErr( kCVSEngineBinaryMergingOffErr );
										}

//  If we're merging, duplicate the file for safety.

									result = FSpSafeDuplicate( &outTargetSpec );
									ThrowOSErr( result );									
								}																						
						}
					
					if ( bHasCkid )
						{
							intCommentLen = pCkidInfo->fCommentLen;
							hComment = pCkidInfo->fCommentHndl;				
						}
										
					memset( pCkidInfo, 0, sizeof(CkidResourcePB) );
					
					PStrCopy( inUserNamePStr, pCkidInfo->fUserNamePStr, sizeof(pCkidInfo->fUserNamePStr) );
					PStrCopy( outTargetSpec.name, pCkidInfo->fFileNamePStr, sizeof(pCkidInfo->fFileNamePStr) );			

					if ( bHasCkid )
						{
							pCkidInfo->fCommentLen = intCommentLen;
							pCkidInfo->fCommentHndl = hComment;
						}

//  If the file is modified, we're merging, so keep
//  the MRO flag turned on.

					if ( bModified )
						{
							pCkidInfo->fModifyReadOnly = 1;
						}
						
//  Get all the info out of the entry line

					result = CCVSEntries::GetEntryItems( strEntryLine, intEntryLineLen, pstrFileName, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
					ThrowOSErr( result );
																		
//  Check for binary file

					bBinaryFile = CUIsCVSBinaryOption( pstrOptions );
						
//  Read permissions info...

					intLineLen = kMaxLineSize - 1;
					mStream.ReadEntireDataLine( strLine, intLineLen );
					strLine[intLineLen] = 0;

					result = CUParsePermissionStr( strLine, &bCheckedOutReadOnly );
					ThrowOSErr( result );
					
					if ( bModified )
						{
							//  Don't lock files that are being merged.
							bCheckedOutReadOnly = false;
						}
						
					intCurState = kReadPermissions;
					
//  Now read the file size info
						
					intLineLen = kMaxLineSize - 1;
					mStream.ReadEntireDataLine( strLine, intLineLen );
					strLine[intLineLen] = 0;
				
					intCurState = kReadFileSize;
					
					//  Ok, I doubt we're ever going to get a file bigger 
					//  than 2 GB. If we do, bail with an exception.

					if ( strLine[0] == 'z' ) // compressed file
						{					
							bMustUncompress = true;
							CStrToUInt32( &strLine[1], &intFileSize );
						}
					else
						{
							CStrToUInt32( strLine, &intFileSize );
						}
					
					intBytesToGet = intFileSize;
					
					if ( intFileSize > 0x7FFFFFFF )
						{							
							result = kCVSEngineServerGaveBadResponseErr;
							ThrowOSErr( result );							
						}

					updateFileAccess.SetDelete( true );
	
					result = ::SetEOF( dataForkRef, intFileSize );
					ThrowOSErr( result );

					//  Read the file from the TCP stream, decode
					//  it if we need to, then write it to disk with 
					//  the new 'ckid' information.
										 
					if ( intBytesToGet > 0 )
						{																			
							sCurStatus.fMax = intBytesToGet;
							sCurStatus.fCompleted = 0;

							if ( mNotifier != NULL )
								{									
									(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineProgressEvent, noErr, 0, &sCurStatus, mContextPtr );									
								}

							if ( bMustUncompress )
								{
									UInt32 intCompressedBytesRead = 0;
									UInt32 intUncompressedBytesRead = 0;
									
									mStream.InflateStartup( intBytesToGet );
																		
									while ( (intUncompressedBytesRead = mStream.ReadBlockAndInflate(mBuffer, mBufSize, intCompressedBytesRead)) != 0 )
										{				
											Assert_( intBytesToGet >= intCompressedBytesRead );
											
											if ( intBytesToGet >= intCompressedBytesRead )
												{
													intBytesToGet -= intCompressedBytesRead;
												}															
											
											if ( bBinaryFile ) 
												{
													//  sniff the first 4 bytes to see if it's an AppleSingle file

													if ( (intUncompressedBytesRead >= 4) && (intBytesWeGot == 0) )
														{
															if ( (*(UInt32 *)mBuffer) == kAppleSingleMagicNumber )
																{
																	intEncodingType = kCVSEngineAppleSingleEncoding;
																	bSniffedAppleSingle = true;
																}
															
															//  if it's not AppleSingle, check for Binhex encoding
															
															if ( !bSniffedAppleSingle )
																{
																	if ( intUncompressedBytesRead >= kBinhex4SignaturePStr[0] )
																		{
																			if ( !strncmp(mBuffer, (char *)&kBinhex4SignaturePStr[1], kBinhex4SignaturePStr[0]) )
																				{
																					intEncodingType = kCVSEngineBinhex4Encoding;
																				}
																		}
																}									
															
															//  if it's not AppleSingle or Binhex, check for MacBinary encoding
															
															if ( !bSniffedAppleSingle && (intEncodingType != kCVSEngineBinhex4Encoding) )
																{
																	if ( intUncompressedBytesRead >= kMacBinaryHeaderSize )
																		{
																			if ( IsValidMacBinary(mBuffer, NULL) )
																				{
																					intEncodingType = kCVSEngineMacBinaryEncoding;
																				}
																		}
																}									
														}
												}
																								
											if ( !bBinaryFile )
												{
													if ( sMapping.fTranslation == kCVSEngineCRTranslation )
														{
															for ( UInt32 intIndex = 0; intIndex < intUncompressedBytesRead; intIndex++ )
																if ( mBuffer[intIndex] == 0x0A )
																	mBuffer[intIndex] = 0x0D;
														}
												}
											
											//  Always call SetEOF(). The uncompressed data may
											//  actually be smaller than the compressed data size.

											result = ::SetEOF( dataForkRef, intBytesWeGot + intUncompressedBytesRead );
											ThrowOSErr( result );
											
											result = FSWriteOffset( dataForkRef, &intUncompressedBytesRead, mBuffer, intBytesWeGot );
											ThrowOSErr( result );
													
											intBytesWeGot += intUncompressedBytesRead;										
											sCurStatus.fCompleted += intCompressedBytesRead;
												
											if ( mNotifier )
												{
													(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineProgressEvent, noErr, 0, &sCurStatus, mContextPtr );
												}
										}
									
									mStream.InflateShutdown();
								}
							else
								{															
									while ( intBytesToGet > 0 )
										{		
											if ( intBytesToGet > mBufSize )
												{
													intCopyLen = mBufSize;
												}
											else 
												{
													intCopyLen = intBytesToGet;
												}
												
											mStream.ReadBlock( mBuffer, intCopyLen );

											Assert_( intBytesToGet >= intCopyLen );
											
											if ( intBytesToGet >= intCopyLen )
												{
													intBytesToGet -= intCopyLen;
												}
											
											if ( bBinaryFile )
												{												
													//  sniff the first 4 bytes to see if it's an AppleSingle file
											
													if ( (intCopyLen >= 4) && (intBytesWeGot == 0) )
														{
															if ( (*(UInt32 *)mBuffer) == kAppleSingleMagicNumber )
																{
																	intEncodingType = kCVSEngineAppleSingleEncoding;
																	bSniffedAppleSingle = true;
																}

														//  if it's not AppleSingle, check for Binhex encoding
															
															if ( !bSniffedAppleSingle )
																{
																	if ( intCopyLen >= kBinhex4SignaturePStr[0] )
																		{
																			if ( !strncmp(mBuffer, (char *)&kBinhex4SignaturePStr[1], kBinhex4SignaturePStr[0]) )
																				{
																					intEncodingType = kCVSEngineBinhex4Encoding;
																				}
																		}
																}

														//  if it's not AppleSingle or Binhex, check for MacBinary encoding
															
															if ( !bSniffedAppleSingle && (intEncodingType != kCVSEngineBinhex4Encoding) )
																{
																	if ( intCopyLen >= kMacBinaryHeaderSize )
																		{
																			if ( IsValidMacBinary(mBuffer, NULL) )
																				{
																					intEncodingType = kCVSEngineMacBinaryEncoding;
																				}
																		}
																}
														}
												}
																				
											if ( !bBinaryFile )
												if ( sMapping.fTranslation == kCVSEngineCRTranslation )
													{												
														for ( UInt32 index = 0; index < intCopyLen; index++ )
															if ( mBuffer[index] == 0x0A )
																mBuffer[index] = 0x0D;															
													}
												
											result = FSWriteOffset( dataForkRef, &intCopyLen, mBuffer, intBytesWeGot );
											ThrowOSErr( result );			

											intBytesWeGot += intCopyLen;
											sCurStatus.fCompleted = intBytesWeGot;
											
											if ( mNotifier != NULL )
												{
													(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineProgressEvent, noErr, 0, &sCurStatus, mContextPtr );												
												}
										}					
								}
	
							intCurState = kReadFile;					
						}
							
					::GetDateTime( &pCkidInfo->fCheckoutTime );

//  If the time stamp is blank, we need to make up one.
					
					if ( pstrTimeStamp[0] == 0 )
						{												
							pCkidInfo->fFilesModificationDate = pCkidInfo->fCheckoutTime;
							
							intLineLen = kMaxLineSize;
							result = CUISOTimeStrFromSeconds( strLine, &intLineLen, pCkidInfo->fFilesModificationDate );
							ThrowOSErr( result );
							
							intCopyLen = intLineLen;
							
							if ( intCopyLen > 255 )
								intCopyLen = 255;
								
							::BlockMoveData( strLine, &pstrTimeStamp[1], intCopyLen );
							pstrTimeStamp[0] = intCopyLen;							
						}
						
//  If the time stamp is something valid, we need to convert
//  it to Mac time, which involves getting the GMT delta, etc.

					else
						{		
							intLineLen = pstrTimeStamp[0];
							if ( intLineLen > (kMaxLineSize - 1) )
								intLineLen = kMaxLineSize - 1;
								
							::BlockMoveData( &pstrTimeStamp[1], strLine, intLineLen );
							strLine[intLineLen] = 0;
							
							result = CUISOTimeStrToSeconds( strLine, (SInt32 *)&pCkidInfo->fFilesModificationDate );
							ThrowOSErr( result );
						}

//  Don't add files that have been name-truncated to the
//  entry database. This would cause problems for other
//  CVS clients such as GNU MacCVS.
					
					if ( !bTruncated )
						{
#ifdef CVSENGINE_USE_MIME_FILE_NAMES
							result = mEntries->AddModifyEntry( outTargetSpec.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag, false );
#else
							result = mEntries->AddModifyEntry( pstrFileName, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag, false );
#endif

							ThrowOSErr( result );
						}
											
//  Don't set the ckid of a file we're going to decode. For those files, we do it later.
						
					if ( (!bBinaryFile) || (intEncodingType < kCVSEngineAppleSingleEncoding) )
						{					
							result = FSpWriteCkidResource( &outTargetSpec, pCkidInfo );
							ThrowOSErr( result );
						}

					result = mEntries->FlushEntriesFile( false );
					ThrowOSErr( result );

					updateFileAccess.SetDelete( false );
					
//  File is closed here

				}
				
//  If it was a binary file, decode it.
						
				if ( (bBinaryFile) && (intEncodingType > kCVSEngineBinaryEncoding) )
					{					
						try
							{	
								if ( mNotifier != NULL )
									{
										(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineDecodingPreActionEvent, noErr, 0, NULL, mContextPtr );
									}
									
								//  If we didn't sniff the AppleSingle data, don't even
								//  bother decoding the data.
								
								if ( bSniffedAppleSingle )
									{
										result = FSpDecodeAppleSingle( &outTargetSpec, kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript, CCVSRepository::EncodingYieldProc, this );
	
										if ( result == kCVSEngineInvalidAppleSingleDataErr ) 
											{
												result = noErr;
											}
									}
								else if ( intEncodingType == kCVSEngineBinhex4Encoding )
									{
										result = FSpDecodeBinhex4( &outTargetSpec, CCVSRepository::EncodingYieldProc, this );
										
										//  I'm not sure this is the only error we
										//  want to suppress...
										
										if ( result == eofErr )
											{
												result = noErr;
											}
									}
								else if ( intEncodingType == kCVSEngineMacBinaryEncoding )
									{
										result = FSpDecodeMacBinary( &outTargetSpec, kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript, CCVSRepository::EncodingYieldProc, this );
										
										//  I'm not sure this is the only error we
										//  want to suppress...
										
										if ( result == kCVSEngineNotValidEncodedDataErr )
											{
												result = noErr;
											}
									}
									
								if ( mNotifier != NULL )
									{
										(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineDecodingPostActionEvent, result, 0, NULL, mContextPtr );
									}
																		
								ThrowOSErr( result );
								
								result = FSpWriteCkidResource( &outTargetSpec, pCkidInfo );
								ThrowOSErr( result );					
							}							
						catch ( OSErr inErr )
							{
								result = inErr;
								
								if ( inErr != kCVSEngineInvalidAppleSingleDataErr )
									throw inErr;
									
								result = FSpWriteCkidResource( &outTargetSpec, pCkidInfo );
								ThrowOSErr( result );								
							}						
					}

				if ( bModified )
					{
						pCkidInfo->fFilesModificationDate++;
					}
					
				result = FSpSetModDate( &outTargetSpec, pCkidInfo->fFilesModificationDate );
				Assert_( result == noErr );

				if ( bFileWasLocked || bCheckedOutReadOnly )
					{
						result = ::FSpSetFLock( &outTargetSpec );
						ThrowOSErr( result );
					}
																
				if ( (mNotifier != NULL) && (bCalledPreAction == true) )
					{
						//  Make sure we pass the truncation error to the client.
						
						if ( (result == noErr) && (bTruncated) )
							{
								result = kCVSEngineFileNameTruncatedErr;
							}
							
						(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePostActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
					}
										
				if ( pCkidInfo != NULL )
					{
						if ( pCkidInfo->fCommentHndl != NULL )
							{							
								::DisposeHandle( pCkidInfo->fCommentHndl );
								pCkidInfo->fCommentHndl = NULL;								
							}
							
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;							
					}				
			}

		catch ( OSErr intException )
			{
				if ( intException == noErr )
					{
						intException = kCVSEngineUnknownErr;
					}

//  Yikes. If we got an exception, then we need to attempt to read what's left
//  of the file or everything will be messed up for the rest of the connection.

//  If we get an abort or socket exception, we just clean up and leave. If not,
//  attempt to read what's left, but if we get another exception, we'll just leave.
								
				if ( 	(intException != kCVSEngineOperationAbortedErr) && 
							(intException != kCVSEngineServerGaveBadResponseErr) && 
							(!IsCTCPSocketErr(intException)) )
					{
					
						try
							{								
								switch ( intCurState )
									{						
										case kReadNothing:								
											intLineLen = kMaxLineSize - 1;
											mStream.ReadEntireDataLine( strLine, intLineLen );
											strLine[intLineLen] = 0;

//  Yes, these 5 cases do not have breaks. I want
//  the flow to fall through.

										case kReadLocalPath:										
											intLineLen = kMaxLineSize - 1;
											mStream.ReadEntireDataLine( strLine, intLineLen );
											strLine[intLineLen] = 0;

										case kReadRemotePath:									
											intLineLen = kMaxLineSize - 1;
											mStream.ReadEntireDataLine( strLine, intLineLen );
											strLine[intLineLen] = 0;
									
										case kReadHeader:
											intLineLen = kMaxLineSize - 1;
											mStream.ReadEntireDataLine( strLine, intLineLen );
											strLine[intLineLen] = 0;
										
										case kReadPermissions:										
											intLineLen = kMaxLineSize - 1;
											mStream.ReadEntireDataLine( strLine, intLineLen );
											strLine[intLineLen] = 0;
	
											//  Check for the compression flag

											if ( strLine[0]=='z' ) // compressed
												{					
													CStrToUInt32( &strLine[1], &intFileSize );
												}
											else
												{
													CStrToUInt32( strLine, &intFileSize );
												}

											intBytesToGet = intFileSize;
											
										case kReadFileSize:

											//  If the exception was thrown in the middle 
											//  of downloading the data, pick up where we 
											//  left off.
													
											while ( intBytesToGet > 0 )
												{													
													if ( intBytesToGet > mBufSize )														
														intCopyLen = mBufSize;														
													else intCopyLen = intBytesToGet;
												
													mStream.ReadBlock( mBuffer, intCopyLen );
																								
													intBytesToGet -= intCopyLen;

													if ( (mNotifier != NULL) && (bCalledPreAction == true) )
														{														
															sCurStatus.fMax = intFileSize;
															sCurStatus.fCompleted = intFileSize - intBytesToGet;
															
															(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineProgressEvent, noErr, 0, &sCurStatus, mContextPtr );														
														}									
												}
												
										break;										
									}															
							}			
						catch ( ... )
							{

//  Yikes, if we got here, then we got a network exception when
//  we were eating the rest of the file that we got an exception 
//  during reading in the first place. Just clean up and leave.	

							}					
					}

				if ( bFileWasLocked )
					{
						::FSpSetFLock( &outTargetSpec );
					}
																					
				if ( mNotifier != NULL )
					{
						if ( bCalledPreAction == true )
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePostActionEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );
							}
						else
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );							
							}
					}
					
				if ( pCkidInfo != NULL )
					{										
						if ( pCkidInfo->fCommentHndl != NULL )
							{										
								::DisposeHandle( pCkidInfo->fCommentHndl );
								pCkidInfo->fCommentHndl = NULL;											
							}
							
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;										
					}			
				
				throw intException;	
			}
			
		catch ( ... )
			{			
				Assert_( false );

				if ( result == noErr )
					{
						result = kCVSEngineUnknownExceptionErr;
					}
					
				if ( mNotifier != NULL )
					{												
						if ( bCalledPreAction == true )
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePostActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
							}
						else
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );							
							}
					}

				if ( pCkidInfo != NULL )
					{					
						if ( pCkidInfo->fCommentHndl != NULL )
							{							
								::DisposeHandle( pCkidInfo->fCommentHndl );
								pCkidInfo->fCommentHndl = NULL;								
							}
							
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;							
					}
			
				throw;			
			}

	}
	

void
CCVSRepository::ItemCheckedIn(	 					const	Str255						inUserNamePStr,
																					const UInt32						inMergePolicy,
																					const Boolean						inUpdateParentTag,
																					const char							*inOffsetPathStr,
																					const UInt32						inOffsetPathLen,
																					FSSpec									&outTargetSpec,
																					const Boolean						bInUpdateAnyVersion,
																					const Boolean						bInAddRevisionInfo,
																					const Boolean						bInResetMRO)
	{

		/********************************************************
		**
		**  ItemCheckedIn() reads a CVS entry line from the 
		**  socket stream and updates the appropriate file. 
		**
		**  Note: do to some complications in locking between 
		**  threads, ItemCheckedIn() will only update an entry
		**  of a file that is already the same version unless
		**  inUpdateAnyVersion is true. 
		**
		**  You have to have this restriction or else there can 
		**  be situations where the entry information will 
		**  become out of sync with the actual file data. 
		**
		**  An example of this would be this:
		**
		**  Two updates start at the same time on one file.
		**  One of them is updating the file from 1.1 to
		**  revision 1.2. The other update command will
		**  tag the file with revision 1.1.
		**
		**  The both operations run and send their info to
		**  the server. The first update completes first
		**  and brings down the 1.2 file.
		**
		**  The the second operation completes and calls
	  **  ItemCheckedIn() to update the entry information.
	 	**  It will set the info to revision 1.1 even though
	  **  you really have a 1.2 file!
	  **
	  *********************************************************/
	  
		const								kReadNothing									= 0;
		const								kReadLocalPath								= 1;
		const								kReadRemotePath								= 2;
		const								kReadEntry										= 3;
				
		OSErr								result												= noErr;

		Boolean							bModified											= false;
		Boolean							bCalledPreAction							= false;
		Boolean							bBinaryFile										= false;
		Boolean							bCreatedTarget								= false;
		Boolean							bReadEntireLine								= false;
		Boolean							bHasCkid											= false;
		Boolean							bFileWasLocked								= false;
		
		char								strLine[kMaxLineSize]					= "";
		char								strEntryLine[kMaxLineSize]		= "";
		char								strLocalPath[kMaxPathSize]		= "";
		char								strRemotePath[kMaxPathSize]		= "";

		SInt32							intModDate										= 0;
			
		UInt32							intLineLen										= 0;
		UInt32							intEntryLineLen								= 0;
		UInt32							intLocalPathLen								= 0;
		UInt32							intRemotePathLen							= 0;
		UInt32							intCopyLen										= 0;
		UInt32							intCurState										= kReadNothing;
		
		Str255							pstrFileName									= "\p";
		Str255							pstrLocalPath									= "\p";
		Str255							pstrRevision									= "\p";
		Str255							pstrTimeStamp									= "\p";
		Str255							pstrOptions										= "\p";
		Str255							pstrTag												= "\p";
		Str255							parentTagPStr									= "\p";

		FSSpec							fsParentSpec									= { 0, 0, "\p" };
		CkidResourcePB			*pCkidInfo										= NULL;

		outTargetSpec.name[0] = 0;
		outTargetSpec.parID = 0;
		outTargetSpec.vRefNum = 0;
			
		try
			{	
				pCkidInfo = (CkidResourcePB *)::NewPtr( sizeof(CkidResourcePB) );
				
				if ( pCkidInfo == NULL )
					throw( ::MemError() );
					
				pCkidInfo->fCommentHndl = NULL;

//  Read the local directory from the stream
				
				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadLocalPath;
				
				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{
							
						::BlockMoveData( strLine, strLocalPath, intLineLen );
						intLocalPathLen = intLineLen;
						strLocalPath[intLocalPathLen] = 0;
						
					}
				else ThrowOSErr( kCVSEnginePathOverflow );
			
//  Read the remote path name from the stream

				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadRemotePath;

				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{

//  The path returned should always be longer than the remote
//  root path name. If it isn't, something is seriously wrong.
					
						if ( intLineLen < (mRemoteRoot[0] + 1) )
							ThrowOSErr( kCVSEngineServerGaveBadResponseErr );

						::BlockMoveData( strLine, strRemotePath, intLineLen );
						intRemotePathLen = intLineLen;
						strRemotePath[intRemotePathLen] = 0;
						
					}
				else ThrowOSErr( kCVSEnginePathOverflow );

				CUFastGetLastCVSPathItem( strRemotePath, intRemotePathLen, pstrFileName );
				
//  Now we can call StripCVSPath() on the local path to make sure
//  it is clean. We do this so we set the mcvs info properly.

				CUStripCVSPath( strLocalPath, &intLocalPathLen );

//  If the CVS command was not run in the root directory, we need
//  to deal with this because it screws with the path names.
	
				if ( (inOffsetPathStr != NULL) && (inOffsetPathLen > 0) )
					{						
						if ( intLocalPathLen > 0 )
							{
								if ( strLocalPath[intLocalPathLen - 1] == '/' )
									{
										intLocalPathLen--;
									}
							}
							
						intCopyLen = kMaxPathSize - (intLocalPathLen + 1);
						
						if ( intCopyLen > inOffsetPathLen )
							{
								intCopyLen = inOffsetPathLen;
							}
							
						if ( intCopyLen > 0 )
							{									
								if ( intLocalPathLen > 0 )
									{									
										::BlockMoveData( strLocalPath, &strLocalPath[intCopyLen + 1], intLocalPathLen );
										strLocalPath[intCopyLen] = '/';
										intLocalPathLen++;
									}
									
								::BlockMoveData( inOffsetPathStr, strLocalPath, intCopyLen );
								intLocalPathLen += intCopyLen;					
							}				
					}

				if ( (intLocalPathLen > 0) && (intLocalPathLen < kMaxPathSize) && (strLocalPath[intLocalPathLen - 1] != '/') )
					strLocalPath[intLocalPathLen++] = '/';

				intCopyLen = kMaxPathSize - (intLocalPathLen + 1 );
				
				if ( intCopyLen > pstrFileName[0] )
					intCopyLen = pstrFileName[0];
					
				if ( intCopyLen > 0 )
					{
						::BlockMoveData( &pstrFileName[1], &strLocalPath[intLocalPathLen], intCopyLen );
						intLocalPathLen += intCopyLen;						
					}
							
				if ( intLocalPathLen > 255 )
					ThrowOSErr( kCVSEnginePathOverflow );
																						
//  FSpGetFromCVSPath() takes a pascal string
				
				::BlockMoveData( strLocalPath, &pstrLocalPath[1], intLocalPathLen );
				pstrLocalPath[0] = intLocalPathLen;
				
				result = FSpGetFromLocalCVSPath( pstrLocalPath, &mLocalRoot, &outTargetSpec );
																																	
//  Call the notifier pre-action.

				if ( mNotifier != NULL )
					{
						Boolean bProcessFile = false;
													
						bProcessFile = (*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePreActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
						bCalledPreAction = true;
						
						if ( !bProcessFile )
							ThrowOSErr( kCVSEngineSkippedFile );						
					}

				ThrowOSErr( result );

				result = FSpGetParentFSSpec( &outTargetSpec, &fsParentSpec );
				ThrowOSErr( result );

				{				
					CInfoPBRec pb;
					
					pb.hFileInfo.ioCompletion = NULL;
					pb.hFileInfo.ioFDirIndex = 0;
					pb.hFileInfo.ioNamePtr = outTargetSpec.name;
					pb.hFileInfo.ioVRefNum = outTargetSpec.vRefNum;
					pb.hFileInfo.ioDirID = outTargetSpec.parID;
					
					result = ::PBGetCatInfoSync( &pb );				
					ThrowOSErr( result );

//  Ok, the only way to truly tell if a file is open without calling
//  FSpOpenDF, FSpOpenRF, *and* FSpOpenResFile is to check the flags.
										
					if ( FILE_OPEN(pb) )
						{						
							result = opWrErr;
							throw result;							
						}

					if ( pb.hFileInfo.ioFlAttrib & 1 )
						{
							//  If the file is locked, we will unlock it for
							//  the duration of this function. This does not
							//  necessary mean we will modify the file.
							
							bFileWasLocked = true;
							
							result = FSpRstFLock( &outTargetSpec );
							ThrowOSErr( result );
						}
						
					intModDate = pb.hFileInfo.ioFlMdDat;					
				}

				{				
					SInt16				dataForkRef		= 0;
					StFileAccess	updateFileAccess( outTargetSpec );
					
					try
						{				
							dataForkRef = updateFileAccess.GetFSDataForkRef();
						}						
					catch ( OSErr intException )
						{						
							result = intException;
							throw intException;							
						}

					if ( mEntries != NULL )
						if ( !mEntries->AreYouThisDirectory(&fsParentSpec) )
							{							
								result = mEntries->CloseEntriesFile();
								ThrowOSErr( result );

								result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdWrPerm );							
								ThrowOSErr( result );								
							}
						
					if ( mEntries == NULL )
						{						
							mEntries = new CCVSEntriesCache( this );
						
							if ( mEntries != NULL )
								result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdWrPerm );
							else result = kCVSEngineOutOfMemoryErr;
							
							ThrowOSErr( result );						
						}					
						
					result = FSpReadCkidResource( &outTargetSpec, pCkidInfo, true );
					
					if ( result == noErr )
						{			
							bHasCkid = true;

							//  I'm assuming we should bail if this happens.
							if ( pCkidInfo->fCheckedOutBySomeone > 0 )
								{
									ThrowOSErr( kCVSEnginePermissionsErr );
								}
						}
					else
						{
							result = noErr;
							
							memset( pCkidInfo, 0, sizeof(CkidResourcePB) );
							PStrCopy( inUserNamePStr, pCkidInfo->fUserNamePStr, sizeof(pCkidInfo->fUserNamePStr) );
							PStrCopy( outTargetSpec.name, pCkidInfo->fFileNamePStr, sizeof(pCkidInfo->fFileNamePStr) );						
						}	

					//  get the date from the entries on disk
					//  stick the version in pstrLocalPath for storage

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
					result = mEntries->GetEntry( outTargetSpec.name, pstrLocalPath, pstrTimeStamp, pstrOptions, NULL );
#else
					result = mEntries->GetEntry( pstrFileName, pstrLocalPath, pstrTimeStamp, pstrOptions, NULL );
#endif

					//  Ok, this is dangerous: we want to ignore a kCVSEngineNoRevisionDataErr 
					//  result. If the user is adding a file, you will encounter this situation.
					
					//  However, if the server sent a rogue "Checked-In" response for a file
					//  that doesn't have revision information, we would give it some!
					
					if ( result == kCVSEngineNoRevisionDataErr )
						{
							if ( !bInAddRevisionInfo )
								{
									ThrowOSErr( kCVSEngineNoRevisionDataErr );
								}
								
							bModified = true;
							result = noErr;
						}
					else
						{
							ThrowOSErr( result );
							bBinaryFile = CUIsCVSBinaryOption( pstrOptions );
						}
						
					if ( !bModified )
						{							
							if ( bHasCkid )
								{							
									result = CUIsFileModified( pCkidInfo, pstrTimeStamp, intModDate );
								}
							else
								{
									result = CUIsFileModified( NULL, pstrTimeStamp, intModDate );
								}
								
							if ( result == kCVSEngineFileLocallyModifiedErr )
								{
									bModified = true;
									result = noErr;
								}
								
							ThrowOSErr( result );
						}
						
					intEntryLineLen = kMaxLineSize - 1;
					bReadEntireLine = mStream.ReadEntireDataLine( strEntryLine, intEntryLineLen );
					strEntryLine[intEntryLineLen] = 0;
					
					intCurState = kReadEntry;
					
					result = CCVSEntries::GetEntryItems( strEntryLine, intEntryLineLen, NULL, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
					ThrowOSErr( result );
										
					//  Hack: Don't update the file ckid unless the version
					//  is already the same. This solves the thread synchronization 
					//  issues. This will always fail for files without
					//  revision information.
							
					if ( !bInUpdateAnyVersion )
						if ( !PStrCompare(pstrRevision, pstrLocalPath) )
							{		
								ThrowOSErr( kCVSEngineSkippedFile );		
							}
										
					//  If the file is modified, check our merging policy.

					if ( bModified )
						{												
							if ( (inMergePolicy != kCVSEngineTextMerging) && (inMergePolicy != kCVSEngineTextAndBinaryMerging) )
								ThrowOSErr( kCVSEnginePermissionsErr );
								
							else if ( (inMergePolicy != kCVSEngineTextAndBinaryMerging) && (bBinaryFile) )
								ThrowOSErr( kCVSEngineBinaryMergingOffErr );								
						}

					pCkidInfo->fCheckedOutBySomeone = 0;
						
					if ( bInResetMRO || (!bModified) )
						{
							pCkidInfo->fModifyReadOnly = 0;
						}
					else 
						{
							pCkidInfo->fModifyReadOnly = 1;
						}
																					
					::GetDateTime( &pCkidInfo->fCheckoutTime );

					if ( pstrTimeStamp[0] == 0 )
						{												
							pCkidInfo->fFilesModificationDate = pCkidInfo->fCheckoutTime;
							
							intLineLen = kMaxLineSize;
							result = CUISOTimeStrFromSeconds( strLine, &intLineLen, pCkidInfo->fFilesModificationDate );
							ThrowOSErr( result );
							
							intCopyLen = intLineLen;
							
							if ( intCopyLen > 255 )
								intCopyLen = 255;
								
							::BlockMoveData( strLine, &pstrTimeStamp[1], intCopyLen );
							pstrTimeStamp[0] = intCopyLen;							
						}
					else
						{		
							intLineLen = pstrTimeStamp[0];
							if ( intLineLen > (kMaxLineSize - 1) )
								intLineLen = kMaxLineSize - 1;
								
							::BlockMoveData( &pstrTimeStamp[1], strLine, intLineLen );
							strLine[intLineLen] = 0;
							
							result = CUISOTimeStrToSeconds( strLine, (SInt32 *)&pCkidInfo->fFilesModificationDate );
							ThrowOSErr( result );
						}

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
					result = mEntries->AddModifyEntry( outTargetSpec.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag, false );
#else					
					result = mEntries->AddModifyEntry( pstrFileName, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag, false );
#endif
					ThrowOSErr( result );

					result = FSpWriteCkidResource( &outTargetSpec, pCkidInfo );
					ThrowOSErr( result );

					result = mEntries->FlushEntriesFile( false );
					ThrowOSErr( result );

//  File is closed here
												
				}

				if ( bModified && (!bInResetMRO) )
					{
						pCkidInfo->fFilesModificationDate++;
					}
					
				FSpSetModDate( &outTargetSpec, pCkidInfo->fFilesModificationDate );

				if ( bFileWasLocked )
					{
						result = ::FSpSetFLock( &outTargetSpec );
						ThrowOSErr( result );
					}

				if ( (mNotifier != NULL) && (bCalledPreAction == true) )
					(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePostActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
					
				if ( pCkidInfo->fCommentHndl != NULL )
					{					
						::DisposeHandle( pCkidInfo->fCommentHndl );
						pCkidInfo->fCommentHndl = NULL;						
					}
					
				::DisposePtr( (Ptr)pCkidInfo );
				pCkidInfo = NULL;										
			}
		
		catch ( OSErr intException )
			{		
				if ( intException == noErr )
					{
						intException = kCVSEngineUnknownErr;
					}
							
				if ( mNotifier != NULL )
					{
						if ( bCalledPreAction == true )
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePostActionEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );
							}
						else
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );							
							}
					}
	
				if ( pCkidInfo != NULL )
					{
						if ( pCkidInfo->fCommentHndl != NULL )
							{							
								::DisposeHandle( pCkidInfo->fCommentHndl );
								pCkidInfo->fCommentHndl = NULL;								
							}

						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;						
					}

				if ( bFileWasLocked )
					{
						::FSpSetFLock( &outTargetSpec );
					}
			
				if ( (intException != kCVSEngineOperationAbortedErr) && (!IsCTCPSocketErr(intException)) )
					{
					
//  Eat the rest of the line if we didn't read it all. This may
//  cause another exception so the code is down at the bottom of
//  the catch clause.
			
						switch ( intCurState )
							{

//  These next 3 cases do not have breaks on purpose.
				
								case kReadNothing:
							
									intLineLen = kMaxLineSize - 1;
									mStream.ReadEntireDataLine( strLine, intLineLen );
									
								case kReadLocalPath:

									intLineLen = kMaxLineSize - 1;
									mStream.ReadEntireDataLine( strLine, intLineLen );
								
								case kReadRemotePath:
			
									intLineLen = kMaxLineSize - 1;
									mStream.ReadEntireDataLine( strLine, intLineLen );						
							}				
					}
					
				throw;				
			}				
		catch ( ... )
			{			
				Assert_( false );
	
				if ( result == noErr )
					{
						result = kCVSEngineUnknownExceptionErr;
					}
			
				if ( mNotifier != NULL )
					{							
						if ( bCalledPreAction == true )
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdatePostActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
							}
						else
							{
								(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );							
							}
					}
	
				if ( pCkidInfo != NULL )
					{
						if ( pCkidInfo->fCommentHndl != NULL )
							{							
								::DisposeHandle( pCkidInfo->fCommentHndl );
								pCkidInfo->fCommentHndl = NULL;								
							}

						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;						
					}
										
				throw;				
			}
			
	}

	
void
CCVSRepository::ItemRemoved(	const char *inOffsetPathStr, const UInt32 inOffsetPathLen, FSSpec &outTargetSpec, const Boolean inDeleteFile )
	{
	
		const				kReadNothing									= 0;
		const				kReadLocalPath								= 1;
		const				kReadRemotePath								= 2;
	
		OSErr				result 												= noErr;

		Boolean			bCreatedTarget								= false;
		Boolean			bReadEntireLine								= false;
		Boolean			bModified											= false;
		
		char				strLine[kMaxLineSize]					= "";
		char				strLocalPath[kMaxPathSize]		= "";
		char				strRemotePath[kMaxPathSize]		= "";
		char				strFullLocalPath[kMaxPathSize]= "";

		UInt32			intLineLen										= 0;
		UInt32			intLocalPathLen								= 0;
		UInt32			intRemotePathLen							= 0;
		UInt32			intFullLocalPathLen							= 0;
		UInt32			intCopyLen										= 0;
		UInt32			intCurState										= kReadNothing;

		Str255			pstrFileName									= "\p";
		Str255			pstrFullLocalPath							= "\p";
		Str255			pstrTimeStamp									= "\p";
		
		FSSpec			fsParent											= { 0, 0, "\p" };
		
		CkidResourcePB	*pCkidInfo								= NULL;
		
		outTargetSpec.name[0] = 0;
		outTargetSpec.parID = 0;
		outTargetSpec.vRefNum = 0;

		try
			{			
				pCkidInfo = (CkidResourcePB *)::NewPtrClear( sizeof(CkidResourcePB) );
				
				if ( pCkidInfo == NULL )
					{
						ThrowOSErr( ::MemError() );
					}
					
//  Read the local directory from the stream
				
				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadLocalPath;
				
				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{							
						::BlockMoveData( strLine, strLocalPath, intLineLen );
						intLocalPathLen = intLineLen;
						strLocalPath[intLocalPathLen] = 0;						
					}
				else ThrowOSErr( kCVSEnginePathOverflow );
			
//  Read the remote path name from the stream

				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadRemotePath;

				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{

//  The path returned should always be longer than the remote
//  root path name. If it isn't, something is seriously wrong.
					
						if ( intLineLen < (mRemoteRoot[0] + 1) )
							ThrowOSErr( kCVSEngineServerGaveBadResponseErr );

						::BlockMoveData( strLine, strRemotePath, intLineLen );
						intRemotePathLen = intLineLen;
						strRemotePath[intRemotePathLen] = 0;					
					}
				else ThrowOSErr( kCVSEnginePathOverflow );
				
//  Now that we have both the local and remote path names, extract the file name
//  and see if we have a file mapping for it. Then create the file if we need to.

				CUFastGetLastCVSPathItem( strRemotePath, intRemotePathLen, pstrFileName );

//  Now we can call StripCVSPath() on the local path to make sure
//  it is clean.

				CUStripCVSPath( strLocalPath, &intLocalPathLen );

				if ( intLocalPathLen > 0 )
					{
						if ( strLocalPath[intLocalPathLen - 1] == '/' )
							{
								intLocalPathLen--;
							}
					}
					
//  If the CVS command was not run in the root directory, we need
//  to deal with this because it screws with the path names.
	
				if ( (inOffsetPathStr != NULL) && (inOffsetPathLen > 0) )
					{							
						if ( (kMaxPathSize - (intLocalPathLen + 1)) > inOffsetPathLen )
							{						
								if ( intLocalPathLen > 0 )
									{								
										::BlockMoveData( strLocalPath, &strLocalPath[inOffsetPathLen + 1], intLocalPathLen );
										strLocalPath[inOffsetPathLen] = '/';
										intLocalPathLen++;					
									}
								
								::BlockMoveData( inOffsetPathStr, strLocalPath, inOffsetPathLen );
								intLocalPathLen += inOffsetPathLen;								
							}							
						else if ( result == noErr )
							result = kCVSEnginePathOverflow;						
					}

//  Copy the local path into another buffer and append the file name
//  so we have something to tell our client. This is so annoying...

				if ( intLocalPathLen < kMaxPathSize )
					{					
						intFullLocalPathLen = intLocalPathLen;

						if ( intFullLocalPathLen > 0 )
							{												
								::BlockMoveData( strLocalPath, strFullLocalPath, intFullLocalPathLen );
										
								if ( (intFullLocalPathLen + 1) < kMaxPathSize )
									{
									
										strFullLocalPath[intFullLocalPathLen] = '/';
										intFullLocalPathLen++;
									
									}								
							}
								
						intCopyLen = kMaxPathSize - intFullLocalPathLen;
						
						if ( pstrFileName[0] < intCopyLen )
							intCopyLen = pstrFileName[0];
							
						::BlockMoveData( &pstrFileName[1], &strFullLocalPath[intFullLocalPathLen], intCopyLen );
						intFullLocalPathLen += intCopyLen;						
					}
				else 
					{					
						::BlockMoveData( strLocalPath, strFullLocalPath, kMaxPathSize );					
					}

				if ( intFullLocalPathLen > 255 )
					result = kCVSEnginePathOverflow;

				ThrowOSErr( result );

//  FSpGetFromCVSPath() takes a pascal string
										
				::BlockMoveData( strFullLocalPath, &pstrFullLocalPath[1], intFullLocalPathLen );
				pstrFullLocalPath[0] = intFullLocalPathLen;
				
				result = FSpGetFromLocalCVSPath( pstrFullLocalPath, &mLocalRoot, &outTargetSpec );

//  Call notifier for remove event here.

//  Here we throw for the error from FSpGetFromLocalCVSPath...

				ThrowOSErr( result );
					
				result = FSpGetParentFSSpec( &outTargetSpec, &fsParent );
				ThrowOSErr( result );					
				
				if ( mEntries != NULL )
					if ( !mEntries->AreYouThisDirectory(&fsParent) )
						{						
							result = mEntries->CloseEntriesFile();
							ThrowOSErr( result );

							result = mEntries->LoadWorkingDirectory( &fsParent, fsRdWrPerm );							
							ThrowOSErr( result );							
						}
					
				if ( mEntries == NULL )
					{						
						mEntries = new CCVSEntriesCache( this );
					
						if ( mEntries != NULL )
							result = mEntries->LoadWorkingDirectory( &fsParent, fsRdWrPerm );
						else result = kCVSEngineOutOfMemoryErr;
						
						ThrowOSErr( result );						
					}					
			
				//  Get the file's entry information
				
				result = mEntries->GetEntry( outTargetSpec.name, NULL, pstrTimeStamp, NULL, NULL );
				ThrowOSErr( result );

				result = mEntries->RemoveEntry( outTargetSpec.name );
				ThrowOSErr( result );
				
				result = FSpReadCkidResource( &outTargetSpec, pCkidInfo, false );
				
				if ( result != noErr )
					{
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;
					}
	
				//  We need to check if the file has been modified
				//  before we blow it away. In some circumstances,
				//  like if auto-merging is turned off, we never send
				//  modified file data, so the server will think the
				//  file is not modified and send the delete command,
				//  even though it really is.
				
				result = CUIsFileModified( pCkidInfo, pstrTimeStamp, FSpGetModDate(&outTargetSpec) );
				
				if ( result == kCVSEngineFileLocallyModifiedErr )
					{
						bModified = true;
						result = noErr;
					}
					
				ThrowOSErr( result );
				
				if ( !bModified && inDeleteFile )
					{
						::FSpRstFLock( &outTargetSpec );
						::FSpDelete( &outTargetSpec );
					}
					
				if ( pCkidInfo != NULL )
					{
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;
					}
			}
		
		catch ( ... ) 
			{
				if ( pCkidInfo != NULL )
					{
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;
					}

				if ( mNotifier != NULL )
					{					
						(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, result, intFullLocalPathLen, strFullLocalPath, mContextPtr );						
					}
			
				switch ( intCurState )
					{					
						case kReadNothing:	
							intLineLen = kMaxLineSize - 1;
							mStream.ReadEntireDataLine( strLine, intLineLen );
				
						case kReadLocalPath:
							intLineLen = kMaxLineSize - 1;
							mStream.ReadEntireDataLine( strLine, intLineLen );
					
						case kReadRemotePath:
						
						break;						
					}
					
				throw;				
			}
			
	}


void		
CCVSRepository::SetClearStaticDir( 	const Boolean													inSetStaticDir,
																		const Str255 													inUserNamePStr, 
																		const char 														*inOffsetPathStr,
																		const UInt32 													inOffsetPathLen, 
																		const CCVSEngineFolderCreationOptions &folderOptions, 
																		FSSpec 																&outTargetSpec)
	{

		const													kReadNothing									= 0;
		const													kReadLocalPath								= 1;
		const													kReadRemotePath								= 2;

		OSErr													result												= noErr;
		
		Boolean												bReadEntireLine								= false;
		Boolean												bCreatedTarget								= false;
		Boolean												bCalledPreAction							= false;
		
		char													strLine[kMaxLineSize]					= "";
		char													strLocalPath[kMaxPathSize]		= "";
		char													strRemotePath[kMaxPathSize]		= "";
		char													strDisplayPath[kMaxPathSize]	= "";
			
		UInt32												intLineLen										= 0;
		UInt32												intLocalPathLen								= 0;
		UInt32												intRemotePathLen							= 0;
		UInt32												intDisplayPathLen							= 0;
		UInt32												intCurState										= kReadNothing;
		UInt32												intCopyLen										= 0;
		
		FSSpec												fsParent											= { 0, 0, "\p" };
		Str31													pstrEmpty											= "\p";
		
		outTargetSpec.name[0] = 0;
		outTargetSpec.parID = 0;
		outTargetSpec.vRefNum = 0;
			
		try
			{
	
//  Read the local directory from the stream
				
				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadLocalPath;
				
				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{						
						::BlockMoveData( strLine, strLocalPath, intLineLen );
						intLocalPathLen = intLineLen;
						strLocalPath[intLocalPathLen] = 0;					
					}
				else 
					{
						ThrowOSErr( kCVSEnginePathOverflow );
					}
					
//  Read the remote path name from the stream

				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadRemotePath;

				if ( !bReadEntireLine )
					{
						ThrowOSErr( kCVSEnginePathOverflow );
					}
					
				if ( intLineLen < kMaxPathSize )
					{

//  The path returned should always be longer than the remote
//  root path name. If it isn't, something is seriously wrong.					
					
						if ( intLineLen < (mRemoteRoot[0] + 1) )
							{
								ThrowOSErr( kCVSEngineServerGaveBadResponseErr );
							}
							
						::BlockMoveData( strLine, strRemotePath, intLineLen );
						intRemotePathLen = intLineLen;
						strRemotePath[intRemotePathLen] = 0;
						
					}
				else 
					{
						ThrowOSErr( kCVSEnginePathOverflow );
					}

				CCVSEngineFileCreateInfo fileInfo( kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript, folderOptions );
					
				result = CUCreateItemFromCVSPath(	mRemoteUserName, 
																					mServerName, 
																					mPort, 
																					mAuthMethod,
																					strLocalPath, 
																					intLocalPathLen, 
																					strRemotePath, 
																					intRemotePathLen, 
																					inOffsetPathStr, 
																					inOffsetPathLen,
																					(char *)&mRemoteRoot[1], 
																					mRemoteRoot[0], 
																					&mLocalRoot, 
																					&outTargetSpec, 
																					&fileInfo,
																				  &bCreatedTarget );

//  Don't throw the error until we call the notifier.

//  Now we can call StripCVSPath() on the local path to make sure
//  it is clean. We do this so we set the mcvs info properly.

				CUStripCVSPath( strLocalPath, &intLocalPathLen );

//  If the CVS command was not run in the root directory, we need
//  to deal with this because it screws with the path names.
	
				if ( (inOffsetPathStr != NULL) && (inOffsetPathLen > 0) )
					{						
						intCopyLen = kMaxPathSize - (intLocalPathLen + 1);
						
						if ( intCopyLen > inOffsetPathLen )
							{
								intCopyLen = inOffsetPathLen;
							}
							
						if ( intCopyLen > 0 )
							{									
								if ( intLocalPathLen > 0 )
									{									
										::BlockMoveData( strLocalPath, &strLocalPath[intCopyLen + 1], intLocalPathLen );
										strLocalPath[intCopyLen] = '/';
										intLocalPathLen++;
									}
									
								::BlockMoveData( inOffsetPathStr, strLocalPath, intCopyLen );
								intLocalPathLen += intCopyLen;					
							}				
					}

//  Call the notifier pre-action. Do it before we throw for errors.
					
				if ( mNotifier != NULL )
					{										
					 	(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPreEvent, result, intLocalPathLen, strLocalPath, mContextPtr );

						bCalledPreAction = true;												
					}

				ThrowOSErr( result );
				
				//  Set/clear static directory here
				
				result = FSpSetStatic( &outTargetSpec, inSetStaticDir, kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript );
				ThrowOSErr( result );
				
				//  Bug fix: Make sure the item is not the root item.
				//  Should we be more paranoid and use FSpIsChildOf()?
				if ( !FSpIsEqual(&outTargetSpec, &mLocalRoot) )
					{
						result = FSpGetParentFSSpec( &outTargetSpec, &fsParent );
						ThrowOSErr( result );
						
						if ( mEntries != NULL )
							{
								delete mEntries;
								mEntries = NULL;
							}
							
						mEntries = new CCVSEntriesCache( this );

						if ( mEntries != NULL )
							{
								result = mEntries->LoadWorkingDirectory( &fsParent, fsRdWrPerm );
							}
						else 
							{
								result = kCVSEngineOutOfMemoryErr;
							}							
						ThrowOSErr( result );								

						result = mEntries->AddModifyEntry( outTargetSpec.name, pstrEmpty, pstrEmpty, pstrEmpty, pstrEmpty, true );
						ThrowOSErr( result );
						
						result = mEntries->CloseEntriesFile();
						ThrowOSErr( result );
						
						delete mEntries;
						mEntries = NULL;
					}
									
				if ( (mNotifier != NULL) && (bCalledPreAction == true) )
					{										
					 (*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPostEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
					}
			}
		
		catch ( OSErr intException )
			{		
				if ( intException == noErr )
					{
						intException = kCVSEngineUnknownErr;
					}
								
				if ( mNotifier != NULL )
					{					
						if ( bCalledPreAction == true )
							{
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPostEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );					
							}
						else
							{					
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );
							}
					}

				if ( (intException != kCVSEngineOperationAbortedErr) && (!IsCTCPSocketErr(intException)) )
					{
					
//  Eat the rest of the line if we didn't read it all. This may
//  cause another exception so the code is down at the bottom of
//  the catch clause.
			
						switch ( intCurState )
							{

//  These next 3 cases do not have breaks on purpose.
				
								case kReadNothing:
							
									intLineLen = kMaxLineSize - 1;
									mStream.ReadEntireDataLine( strLine, intLineLen );
									
								case kReadLocalPath:

									intLineLen = kMaxLineSize - 1;
									mStream.ReadEntireDataLine( strLine, intLineLen );
								
								case kReadRemotePath:
								break;								
							}				
					}
			
				throw intException;
			}			
		catch ( ... )
			{
				Assert_( false );
				
				if ( result == noErr )
					{
						result = kCVSEngineUnknownExceptionErr;
					}
					
				if ( mNotifier != NULL )
					{				
						if ( bCalledPreAction == true )
							{
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPostEvent, result, intLocalPathLen, strLocalPath, mContextPtr );					
							}
						else
							{						
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
							}
					}
				
				throw result;
			}
	}


void		
CCVSRepository::SetClearStickyDir( 	const Boolean													inSetStickyDir,
																		const Str255 													inUserNamePStr, 
																		const char 														*inOffsetPathStr,
																		const UInt32 													inOffsetPathLen, 
																		const CCVSEngineFolderCreationOptions &folderOptions, 
																		FSSpec 																&outTargetSpec)
	{

		const			kReadNothing									= 0;
		const			kReadLocalPath								= 1;
		const			kReadRemotePath								= 2;

		OSErr			result												= noErr;
		
		Boolean		bReadEntireLine								= false;
		Boolean		bCreatedTarget								= false;
		Boolean		bCalledPreAction							= false;
		
		char			strLine[kMaxLineSize]					= "";
		char			strLocalPath[kMaxPathSize]		= "";
		char			strRemotePath[kMaxPathSize]		= "";
				
		UInt32		intLineLen										= 0;
		UInt32		intLocalPathLen								= 0;
		UInt32		intRemotePathLen							= 0;
		UInt32		intCurState										= kReadNothing;
		UInt32		intCopyLen										= 0;
		
		FSSpec		fsParent											= { 0, 0, "\p" };
		Str31			pstrEmpty											= "\p";
		Str255		pstrTag												= "\p";
		
		outTargetSpec.name[0] = 0;
		outTargetSpec.parID = 0;
		outTargetSpec.vRefNum = 0;
			
		try
			{
	
//  Read the local directory from the stream
				
				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadLocalPath;
				
				if ( !bReadEntireLine )
					ThrowOSErr( kCVSEnginePathOverflow );
					
				if ( intLineLen < kMaxPathSize )
					{						
						::BlockMoveData( strLine, strLocalPath, intLineLen );
						intLocalPathLen = intLineLen;
						strLocalPath[intLocalPathLen] = 0;					
					}
				else 
					{
						ThrowOSErr( kCVSEnginePathOverflow );
					}
					
//  Read the remote path name from the stream

				intLineLen = kMaxLineSize - 1;
				bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
				strLine[intLineLen] = 0;
				
				intCurState = kReadRemotePath;

				if ( !bReadEntireLine )
					{
						ThrowOSErr( kCVSEnginePathOverflow );
					}
					
				if ( intLineLen < kMaxPathSize )
					{

//  The path returned should always be longer than the remote
//  root path name. If it isn't, something is seriously wrong.					
					
						if ( intLineLen < (mRemoteRoot[0] + 1) )
							{
								ThrowOSErr( kCVSEngineServerGaveBadResponseErr );
							}
							
						::BlockMoveData( strLine, strRemotePath, intLineLen );
						intRemotePathLen = intLineLen;
						strRemotePath[intRemotePathLen] = 0;
						
					}
				else 
					{
						ThrowOSErr( kCVSEnginePathOverflow );
					}

				CCVSEngineFileCreateInfo fileInfo( kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript, folderOptions );
					
				result = CUCreateItemFromCVSPath(	mRemoteUserName, 
																					mServerName, 
																					mPort, 
																					mAuthMethod,
																					strLocalPath, 
																					intLocalPathLen, 
																					strRemotePath, 
																					intRemotePathLen, 
																					inOffsetPathStr, 
																					inOffsetPathLen,
																					(char *)&mRemoteRoot[1], 
																					mRemoteRoot[0], 
																					&mLocalRoot, 
																					&outTargetSpec, 
																					&fileInfo,
																				  &bCreatedTarget );

				//  Don't throw the error until we call the notifier.

				//  Now we can call StripCVSPath() on the local path to make sure
				//  it is clean. We do this so we set the mcvs info properly.

				CUStripCVSPath( strLocalPath, &intLocalPathLen );

				//  If the CVS command was not run in the root directory, we need
				//  to deal with this because it screws with the path names.
	
				if ( (inOffsetPathStr != NULL) && (inOffsetPathLen > 0) )
					{						
						intCopyLen = kMaxPathSize - (intLocalPathLen + 1);
						
						if ( intCopyLen > inOffsetPathLen )
							{
								intCopyLen = inOffsetPathLen;
							}
							
						if ( intCopyLen > 0 )
							{									
								if ( intLocalPathLen > 0 )
									{									
										::BlockMoveData( strLocalPath, &strLocalPath[intCopyLen + 1], intLocalPathLen );
										strLocalPath[intCopyLen] = '/';
										intLocalPathLen++;
									}
									
								::BlockMoveData( inOffsetPathStr, strLocalPath, intCopyLen );
								intLocalPathLen += intCopyLen;					
							}				
					}

				//  Call the notifier pre-action. Do it before we throw for errors.
					
				if ( mNotifier != NULL )
					{										
					 	(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPreEvent, result, intLocalPathLen, strLocalPath, mContextPtr );

						bCalledPreAction = true;												
					}

				ThrowOSErr( result );
				
				if ( inSetStickyDir )
					{
						intLineLen = kMaxLineSize - 1;
						bReadEntireLine = mStream.ReadEntireDataLine( strLine, intLineLen );
						strLine[intLineLen] = 0;
	
						intCopyLen = intLineLen;
						
						if ( intCopyLen > 0 )	
							{
								//  if the tag is 255 chars or greater, we're screwed because 
								//  FSpSetFolderTag() only works with up to 254 chars.
								
								if ( intCopyLen > 255 )
									{
										intCopyLen = 255;
									}
									
								::BlockMoveData( strLine, &pstrTag[1], intLineLen );
								pstrTag[0] = intLineLen;
							}
							
						result = FSpSetFolderTag( &outTargetSpec, pstrTag, kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript );			
					}
				else
					{
						result = FSpSetFolderTag( &outTargetSpec, "\p", kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript );								
					}
				
				ThrowOSErr( result );
					
				//  Bug fix: Don't try to get the parent of the root
				//  Should we be more paranoid and use FSpIsChildOf()?
				if ( !FSpIsEqual(&outTargetSpec, &mLocalRoot) )
					{			
						result = FSpGetParentFSSpec( &outTargetSpec, &fsParent );
						ThrowOSErr( result );
						
						if ( mEntries != NULL )
							{
								delete mEntries;
								mEntries = NULL;
							}
							
						mEntries = new CCVSEntriesCache( this );

						if ( mEntries != NULL )
							{
								result = mEntries->LoadWorkingDirectory( &fsParent, fsRdWrPerm );
							}
						else 
							{
								result = kCVSEngineOutOfMemoryErr;
							}							
						ThrowOSErr( result );								

						result = mEntries->AddModifyEntry( outTargetSpec.name, pstrEmpty, pstrEmpty, pstrEmpty, pstrEmpty, true );
						ThrowOSErr( result );
						
						result = mEntries->CloseEntriesFile();
						ThrowOSErr( result );
						
						delete mEntries;
						mEntries = NULL;
					}
					
				if ( (mNotifier != NULL) && (bCalledPreAction == true) )
					{										
					 (*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPostEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
					}
			}
		
		catch ( OSErr intException )
			{		
				if ( intException == noErr )
					{
						intException = kCVSEngineUnknownErr;
					}
								
				if ( mNotifier != NULL )
					{					
						if ( bCalledPreAction == true )
							{
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPostEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );					
							}
						else
							{					
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, intException, intLocalPathLen, strLocalPath, mContextPtr );
							}
					}
					
				if ( (intException != kCVSEngineOperationAbortedErr) && (!IsCTCPSocketErr(intException)) )
					{
					
//  Eat the rest of the line if we didn't read it all. This may
//  cause another exception so the code is down at the bottom of
//  the catch clause.
			
						switch ( intCurState )
							{

//  These next 3 cases do not have breaks on purpose.
				
								case kReadNothing:
							
									intLineLen = kMaxLineSize - 1;
									mStream.ReadEntireDataLine( strLine, intLineLen );
									
								case kReadLocalPath:

									intLineLen = kMaxLineSize - 1;
									mStream.ReadEntireDataLine( strLine, intLineLen );
								
								case kReadRemotePath:
								break;								
							}				
					}
			
				throw intException;
			}			
		catch ( ... )
			{
				Assert_( false );
				
				if ( result == noErr )
					{
						result = kCVSEngineUnknownExceptionErr;
					}

				if ( mNotifier != NULL )
					{				
						if ( bCalledPreAction == true )
							{
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineUpdateDirectoryPostEvent, result, intLocalPathLen, strLocalPath, mContextPtr );					
							}
						else
							{						
					 			(*mNotifier)( (CCVSEngineRef)this, &outTargetSpec, kCVSEngineErrorActionEvent, result, intLocalPathLen, strLocalPath, mContextPtr );
							}
					}
					
				throw result;
			}
	}

																	
void		
CCVSRepository::ModuleExpansion( const UInt32 intInMergePolicy, const Boolean bInDeep )
	{

		OSErr					result									= noErr;

		char					strLine[kMaxLineSize]		= "";
		char					terminator							= 0;

		UInt32				intLineLen							= 0;
		
		Str255				dirNamePStr							= "\p";
		FSSpec				theSpec									= { 0, 0, "\p" };
		
//  If the expand-modules command returned successfully, read each module
//  item from the stream, see if it exists on the disk, and if it does,
//  send the entries for that directory.
										
		do 
			{	
				intLineLen = kMaxLineSize - 1;
				terminator = mStream.ReadWord(strLine, intLineLen);
				strLine[intLineLen] = 0;

				Assert_( intLineLen > 0 );
				
				if ( (intLineLen > 0) && (intLineLen < 256) )
					{

//  Bug fix - sometimes expand-modules will return 
//  results with './' path items.
	
						CUStripCVSPath( strLine, &intLineLen );
						
						Assert_( (intLineLen > 0) && (intLineLen < 256) );
						
						dirNamePStr[0] = intLineLen;
						::BlockMoveData( strLine, &dirNamePStr[1], dirNamePStr[0] );
							
//  Don't pass an empty string to FSpGetFromLocalCVSPath() because
//  it will return the root fsspec! (that should probably be fixed)
										
						if ( dirNamePStr[0] > 0 )
							{		
								//  This is a bit of a hack					
								if ( (dirNamePStr[0] == 1) && (dirNamePStr[1] == '.') )
									{
										theSpec = mLocalRoot;
										result = noErr;
									}
								else
									{
										result = FSpGetFromLocalCVSPath( dirNamePStr, &mLocalRoot, &theSpec );
									}
									
								//  Bug fix: Check to see if a directory is a repository
								//  directory before calling WriteEntry().
												
								if ( (result == noErr) || ((result == kCVSEngineIsFolder) && (FSpIsRepositoryDir(&theSpec))) )
									{
										WriteEntry( theSpec, mLocalRoot, (intInMergePolicy > 0), bInDeep, true, false, true );
									}							
							}							
					}
				else
					{					
						// skip modules with names longer than 255 chars						
					}
					
			}
		while ( terminator != '\n' );
		
	}
	
	
void
CCVSRepository::MessageOut( CCVSEngineStreamNotifierProcPtr inNotifier, char *inDataStr, const UInt32 inDataLen, void *inContextPtr )
	{

//  MessageOut() call the client's message notifier with CVS message output data.

		if ( (inNotifier != NULL) && (inDataLen > 0) )
			{				
				(*inNotifier)( (CCVSEngineRef)this, kEngineNotifierTextDataType, inDataLen, inDataStr, inContextPtr );
				(*inNotifier)( (CCVSEngineRef)this, kEngineNotifierTextDataType, 1, "\r", inContextPtr );
				(*inNotifier)( (CCVSEngineRef)this, kEngineNotifierNoDataType, 0, NULL, inContextPtr );					
			}

	}
	
	
void				
CCVSRepository::ErrorOut( CCVSEngineStreamNotifierProcPtr inNotifier, char *inErrorCStr, const UInt32 inDataLen, void *inContextPtr )
	{

//  ErrorOut() call the client's error notifier with CVS error output data.
		
		if ( (inNotifier != NULL) && (inDataLen > 0) )
			{	
				char		*realErrorStr	= NULL;
				UInt32	newLength	= 0;
				
				realErrorStr = CUFilterErrorHeader( inErrorCStr );
			
				newLength = strlen( realErrorStr );
				
				if ( newLength > 0 )
					{					
						(*inNotifier)( (CCVSEngineRef)this, kEngineNotifierTextDataType, newLength, realErrorStr, inContextPtr );
						(*inNotifier)( (CCVSEngineRef)this, kEngineNotifierTextDataType, 1, "\r", inContextPtr );
						(*inNotifier)( (CCVSEngineRef)this, kEngineNotifierNoDataType, 0, NULL, inContextPtr );			
					}						
			}
			
	}

	
#pragma mark -


void
CCVSRepository::AddDirectory( const FSSpec &inTargetSpec, const char *inOffsetPathStr, const UInt32 inOffsetPathLen, 
															const char *inPathStr, const UInt32 inPathLen )
	{

//  AddDirectory() sends the entry information and commands to add a directory 
//  to the CVS Repository. Since this is much simpler than adding a file, I decided 
//  to break this code out into it's own seperate function. AddDirectory() assumes 
//  that it is passed a valid directory FSSpec. (and that this FSSpec is not already 
//  a CVS repository folder and that it's parent is one)
		
		OSErr			result											= noErr;

		SInt32		intDirID										= 0;							
		UInt32		intCopyLen									= 0;		
		
		Str255		pstrRepository							= "\p";
		Str255		pstrTargetRepository				= "\p";
		Str255		pstrParentTag								= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255		pstrTranslated							= "\p";
#endif

		FSSpec		fsParent										= { 0, 0, "\p" };

		Assert_( (inPathStr != NULL) && (inPathLen > 0) );
				
		result = FSpGetParentFSSpec( &inTargetSpec, &fsParent );
		ThrowOSErr( result );
			
		result = FSpGetRepository( &fsParent, pstrRepository );
		ThrowOSErr( result );
		
		result = FSpGetFolderTag( &fsParent, pstrParentTag );
		
		if ( result == fnfErr )
			{
				result = noErr;
			}
		
		ThrowOSErr( result );
		
		//  If this goes over 255 chars, we're
		//  going to be totally SOL.
		
		PStrCopy( pstrRepository, pstrTargetRepository, sizeof(pstrTargetRepository) );
		
		if ( pstrTargetRepository[0] == 0 )
			{
				PStrConcat( pstrTargetRepository, mRemoteRoot, sizeof(pstrTargetRepository) );
			}
			
		PStrConcat( pstrTargetRepository, "\p/", sizeof(pstrTargetRepository) );
		
#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		CUTranslateFromMacNamePStr( inTargetSpec.name, pstrTranslated );
		PStrConcat( pstrTargetRepository, pstrTranslated, sizeof(pstrTargetRepository) );							
#else
		PStrConcat( pstrTargetRepository, inTargetSpec.name, sizeof(pstrTargetRepository) );
#endif

		//  First set the Repository to the directory we're adding.
		
		mStream << kCVSDirectoryToken;
		mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		CUTranslateFromMacNamePStr( inTargetSpec.name, pstrTranslated );
		mStream << pstrTranslated;
#else
		mStream << inTargetSpec.name;
#endif

		mStream << "\n";
		
		if ( ((pstrTargetRepository[0] > 0) && (pstrTargetRepository[1] == '/')) == false )
			{
				mStream << mRemoteRoot;
				
				if ( (mRemoteRoot[0] == 0) || (pstrTargetRepository[0] > 0) )
					{
						mStream << "/";
					}
			}
			
		mStream << pstrTargetRepository;
		mStream << "\n";
		
		//  Then send add directory.
					
		mStream << kCVSDirectoryToken;
		mStream << " .\n";
		
		if ( ((pstrRepository[0] > 0) && (pstrRepository[1] == '/')) == false )
			{
				mStream << mRemoteRoot;
				
				if ( (mRemoteRoot[0] == 0) || (pstrRepository[0] > 0) )
					{
						mStream << "/";
					}
			}

		mStream << pstrRepository;
		mStream << "\n";

		//  Send the parent tag if necessary.
		
		if ( pstrParentTag[0] > 0 )
      {
	      mStream << kCVSStickyToken;
	      mStream << " ";
	      mStream << pstrParentTag;
	      mStream << "\n";
      }
	
		//  Let's get jiggy with 'dit...

		mStream << kCVSArgumentToken;
		mStream << " ";		
		
#ifdef CVSENGINE_USE_MIME_FILE_NAMES	
		CUTranslateFromMacNamePStr( inTargetSpec.name, pstrTranslated );
		mStream << pstrTranslated;
#else
		mStream << inTargetSpec.name;
#endif

		mStream << "\n";

		if ( mNotifier != NULL )
			{
				(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineCommandActionEvent, noErr, 0, NULL, mContextPtr );
			}		
		
		mStream << "add\n";
		
		CCVSEngineFolderCreationOptions fcOptions;
		ProcessServerResponse( inOffsetPathStr, inOffsetPathLen, kCVSEngineNoMerging, fcOptions, NULL, NULL, NULL );
		
		//  If we didn't die with an exception, 
		//  assume that everything is ducky.
		
		result = FSpGetDirID( &inTargetSpec, &intDirID );
		ThrowOSErr( result );

		//  Create the invisible "CVS" folder and 
		//  all it's stuff...	
					
		result = CUCreateRepositoryDir( inTargetSpec.vRefNum, intDirID, kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript );
		ThrowOSErr( result );

		//  Set the repository file in the directory 
		//  (important)

		result = FSpSetRepository( &inTargetSpec, pstrTargetRepository );
		ThrowOSErr( result );
		
		if ( pstrParentTag[0] > 0 )
			{	
				result = FSpSetFolderTag( &inTargetSpec, pstrParentTag, kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript );
				ThrowOSErr( result );
			}
		
	}
	
	
void
CCVSRepository::WriteEntry(			const FSSpec 						&inTargetSpec, 
																const FSSpec						&inBaseSpec,
																const Boolean 					inDoMerge,
																const Boolean						inDeep,
																const Boolean						inSendConflictInfo,
																const Boolean						inSendModifiedBinaryData,
																const Boolean						inUseUnchanged,
																const SInt8							inPerm)
	{

		//  WriteItemEntry() takes a FSSpec and writes out all the 
		//  "Entry" and "Modified" CVS entry lines to the socket stream. 
		//  Call this instead of WriteDirectoryEntries() or WriteItemEntry() 
		//  to "do the right thing" depending on whether or not the FSSpec 
		//  is a file or directory.
	
		OSErr				result 				= noErr;
		FSSpec			fsParentSpec	= { 0, 0, "\p" };
		CInfoPBRec	pb;
		
		pb.hFileInfo.ioCompletion 	= NULL;
		pb.hFileInfo.ioNamePtr 			= const_cast<FSSpec &>(inTargetSpec).name;
		pb.hFileInfo.ioDirID 				= inTargetSpec.parID;
		pb.hFileInfo.ioVRefNum 			= inTargetSpec.vRefNum;
		pb.hFileInfo.ioFDirIndex 		= 0;
		
		result = ::PBGetCatInfoSync( &pb );
		ThrowOSErr( result );
		
		//  If it's a folder, write entries recursively.
			
		if ( pb.dirInfo.ioFlAttrib & ioDirMask )
			{				
				if ( (pb.hFileInfo.ioFlFndrInfo.fdFlags & fInvisible) == 0 )
					{					
						if ( CUIsRepositoryDir(pb.dirInfo.ioVRefNum, pb.dirInfo.ioDrDirID) )
							{
								WriteDirectoryEntries( pb, inBaseSpec, inDoMerge, inDeep, inSendConflictInfo, inSendModifiedBinaryData, inUseUnchanged, inPerm );			
							}
						else 
							{
								ThrowOSErr( kCVSEngineBadParamErr );	
							}					
					}
				else 
					{
						ThrowOSErr( kCVSEngineFileInvisibleErr );	
					}				
			}		
		else
			{				
				result = FSpGetParentFSSpec( &inTargetSpec, &fsParentSpec );
				ThrowOSErr( result );

				FSpSetDirectory( fsParentSpec );
									
				WriteItemEntry( pb, NULL, 0, inDoMerge, inSendConflictInfo, inSendModifiedBinaryData, inUseUnchanged, inPerm );				
			}
	
	}
	

void
CCVSRepository::WriteDirectoryEntries(const CInfoPBRec 				&inTargetPB,
																			const FSSpec						&inBaseSpec, 
																			const Boolean 					inDoMerge,
																			const Boolean						inDeep,
																			const Boolean						inSendConflictInfo,
																			const Boolean						inSendModifiedBinaryData,
																			const Boolean						inUseUnchanged,
																			const SInt8							inPerm)
	{

		//  WriteDirectoryEntry() takes a FSSpec and writes out all 
		//  the "Entry" and "Modified" CVS entry lines of all the files 
		//  inside the directory to the socket stream.
	
		OSErr					result						= noErr;

		char					*strDirPath				= NULL;
		char					*strAbsPath				= NULL;
		
		UInt32				intItemCount 			= 0;
		UInt32				intDirPathLen			= 0;
		UInt32				intAbsPathLen			= 0;
		
		Str255				pstrRootPath			= "\p";
		Str255				pstrTemp					= "\p";
		Str255				pstrTemp2					= "\p";
		Str255				pstrStorage				= "\p";

		FSSpec				fsSpec						= { 0, 0, "\p" };
		CInfoPBRec		pbChild						= inTargetPB;
		
		CCVSEngineProgressStatusRec	curStatus = { 0, 0 };
	
		try
			{
				if ( ::StackSpace() < kDefaultMinimumStackSize )
					throw( (OSErr)memFullErr );

#ifndef CVSENGINE_USE_MIME_FILE_NAMES
				if ( !CUIsValidCVSName(inTargetPB.hFileInfo.ioNamePtr) )
					{					
						result = kCVSEngineInvalidFileNameErr;
						throw result;						
					}
#endif
					
				intItemCount = inTargetPB.dirInfo.ioDrNmFls;
				
				pbChild.hFileInfo.ioNamePtr = pstrStorage;

//  Because the CVS server is *completely* lame, if we issue an additional
//  "Repository" command, all the entries from the last "Repository" command
//  of the same directory will be lost! So we have to recurse as deep as possible
//  first, then write out all the entries.

//  Note: this limitation seems to have been removed, especially now that we're
//  using the "Directory" command. But the code still works fine this way so
//  I'm just going to leave it like this.
				
				if ( inDeep )
					for ( UInt32 index = 1; index <= intItemCount; index++ )
						{						
							pbChild.hFileInfo.ioFDirIndex = index;
							
							result = ::PBGetCatInfoSync( &pbChild );
							ThrowOSErr( result );
								
							if ( pbChild.dirInfo.ioFlAttrib & ioDirMask ) 
								if ( (pbChild.hFileInfo.ioFlFndrInfo.fdFlags & fInvisible) == 0 )
									{									
										if ( CUIsRepositoryDir(pbChild.dirInfo.ioVRefNum, pbChild.dirInfo.ioDrDirID) )
											{											
												try
													{														
														WriteDirectoryEntries( pbChild, inBaseSpec, inDoMerge, inDeep, inSendConflictInfo, inSendModifiedBinaryData, inUseUnchanged, inPerm );							
													}													
												catch ( OSErr inErr )
													{													
														if ( (inErr == memFullErr) || (inErr == kCVSEngineOperationAbortedErr) || IsCTCPSocketErr(inErr) )
															throw inErr;
													}	
											}										
									}
								
							pbChild.hFileInfo.ioDirID = inTargetPB.hFileInfo.ioDirID;							
						}
					
				PStrCopy( inTargetPB.hFileInfo.ioNamePtr, fsSpec.name, sizeof(fsSpec.name) );
				fsSpec.parID = inTargetPB.hFileInfo.ioFlParID;
				fsSpec.vRefNum = inTargetPB.hFileInfo.ioVRefNum;

				//  Check the root path to see if this directory is
				//  part of the repository. If it isn't, don't recurse
				//  into it or the CVS server will return an error.
				
				result = FSpGetRoot( &fsSpec, pstrTemp );
				ThrowOSErr( result );
				
				result = CUParseRootPStr( pstrTemp, NULL, NULL, pstrRootPath, NULL, NULL );
				ThrowOSErr( result );
				
				if ( pstrRootPath[0] == 0 )
					{
						//  If pstrRootPath is 0, that probably is because
						//  early versions of MacCVS Pro 2.5 didn't write
						//  the root file correctly. So we just copy the
						//  root specified in the session settings.
						
						PStrCopy( mRemoteRoot, pstrRootPath, sizeof(pstrRootPath) );
					}
					
				PStrCopy( pstrRootPath, pstrTemp, sizeof(pstrTemp) );
				PStrCopy( mRemoteRoot, pstrTemp2, sizeof(pstrTemp2) );
				PStrLowercase( pstrTemp );
				PStrLowercase( pstrTemp2 );

				if ( PStrCompare(pstrTemp, pstrTemp2) )
					{
						result = FSpGetRepository( &fsSpec, pstrTemp );
						ThrowOSErr( result );
						
						//  Since this function is called recursively, 
						//  we allocate the path on the heap.
				
						intDirPathLen = kMaxPathSize;
						strDirPath = ::NewPtr( intDirPathLen );
						
						if ( strDirPath == NULL )
							throw( ::MemError() );

						intAbsPathLen = kMaxPathSize;
						strAbsPath = ::NewPtr( intAbsPathLen );
						
						if ( strAbsPath == NULL )
							throw( ::MemError() );
						
						intDirPathLen = kMaxPathSize - 1;	
						result = FSpGetLocalCVSPath( &fsSpec, &inBaseSpec, strDirPath, &intDirPathLen );
						ThrowOSErr( result );
						strDirPath[intDirPathLen] = 0;
										
						if ( !FSpIsEqual(&inBaseSpec, &mLocalRoot) )
							{
								intAbsPathLen = kMaxPathSize - 1;	
								result = FSpGetLocalCVSPath( &fsSpec, &mLocalRoot, strAbsPath, &intAbsPathLen );
								ThrowOSErr( result );
								strAbsPath[intAbsPathLen] = 0;
							}	
						else
							{
								::BlockMoveData( strDirPath, strAbsPath, intDirPathLen );
								intAbsPathLen = intDirPathLen;
							}	
							
						mStream << kCVSDirectoryToken;
						mStream << " ";
						
						if ( intDirPathLen > 0 )
							{
								mStream.WriteBlock( strDirPath, intDirPathLen );
							}			
						else
							{ 
								mStream << ".";
							}
						
						mStream << "\n";
						
						if ( ((pstrTemp[0] > 0) && (pstrTemp[1] == '/')) == false )
							{
								mStream << pstrRootPath;
								
								//  Bug fix, don't send paths that end with '/',
								//  which happens when pstrRepository[0] == 0.
								
								if ( (pstrRootPath[0] == 0) || (pstrTemp[0] > 0) )
									{
										mStream << "/";
									}
							}
							
						mStream << pstrTemp;
						mStream << "\n";
							
						if ( FSpIsStatic(&fsSpec) )
							{
								mStream << kCVSStaticToken;
								mStream << "\n";
							}
								
						//  You must initialize this string to 0 length
						//  because FSpGetFolderTag() may not change it
						//  if there is no tag file.
						
						pstrTemp[0] = 0;			
						result = FSpGetFolderTag( &fsSpec, pstrTemp );
							
						if ( result == fnfErr )
							{
								result = noErr;
							}
							
						ThrowOSErr( result );
						
						if ( pstrTemp[0] > 0 )
							{					
								mStream << "Sticky ";
								mStream << pstrTemp;
								mStream << "\n";						
							}

						for ( UInt32 index = 1; index <= intItemCount; index++ )
							{					
								pbChild.hFileInfo.ioFDirIndex = index;
								
								result = ::PBGetCatInfoSync( &pbChild );
								ThrowOSErr( result );
									
								if ( (pbChild.dirInfo.ioFlAttrib & ioDirMask) == 0 )
									{
										try 
											{									
												WriteItemEntry( pbChild, strAbsPath, intAbsPathLen, inDoMerge, inSendConflictInfo, inSendModifiedBinaryData, inUseUnchanged, inPerm );							
											}								
										catch ( OSErr inErr )
											{							
												if ( (inErr == memFullErr) || (inErr == kCVSEngineOperationAbortedErr) || IsCTCPSocketErr(inErr) )
													{
														throw inErr;
													}
											}																												
									}
																		
								pbChild.hFileInfo.ioDirID = inTargetPB.hFileInfo.ioDirID;				
							}
						
						if ( mEntries != NULL )
							if ( mEntries->AreYouThisDirectory(&fsSpec) )
								mEntries->CloseEntriesFile();
						
						::DisposePtr( strDirPath );
						strDirPath = NULL;	
						
						::DisposePtr( strAbsPath );
						strAbsPath = NULL;		
					}																
			}
			
		catch ( OSErr intException )
			{		
				result = intException;
				
				if ( mNotifier != NULL )
					{					
						FSSpec	theSpec	= { 0, 0, "\p" };
						
						Assert_( inTargetPB.hFileInfo.ioNamePtr != NULL );
						
						if ( inTargetPB.hFileInfo.ioNamePtr != NULL )
							::BlockMoveData( inTargetPB.hFileInfo.ioNamePtr, theSpec.name, inTargetPB.hFileInfo.ioNamePtr[0] + 1 );
							
						theSpec.vRefNum = inTargetPB.hFileInfo.ioVRefNum;
						theSpec.parID = inTargetPB.hFileInfo.ioFlParID;
						
						(*mNotifier)( (CCVSEngineRef)this, &theSpec, kCVSEngineErrorActionEvent, result, 0, NULL, mContextPtr );						
					}
	
				if ( strDirPath != NULL )
					{					
						::DisposePtr( strDirPath );
						strDirPath = NULL;					
					}

				if ( strAbsPath != NULL )
					{					
						::DisposePtr( strAbsPath );
						strAbsPath = NULL;					
					}
					
				throw intException;					
			}			
		catch ( ... )
			{
				if ( strDirPath != NULL )
					{					
						::DisposePtr( strDirPath );
						strDirPath = NULL;					
					}

				if ( strAbsPath != NULL )
					{					
						::DisposePtr( strAbsPath );
						strAbsPath = NULL;					
					}
						
				throw;			
			}
		
	}

	
void
CCVSRepository::WriteItemEntry( const CInfoPBRec				&inTargetPB, 
																const char							*inStrParPath,
																const UInt32						inParPathLen,	
																const Boolean 					inDoMerge,
																const Boolean						inSendConflictInfo,
																const Boolean						inSendModifiedBinaryData,
																const Boolean						inUseUnchanged,
																const SInt8							inPerm)
	{

//  WriteItemEntry() will output an item entry line (ie. "Entry myfile/1.1/+conflicts/-kb/Ttag")
//  along with the modified entry line to the socket stream. The FSSpec passed in must
//  be a valid file. (NOT a folder)

		OSErr								result 								= noErr;
		
		Boolean							bProcessFile					= true;
		Boolean							bModified							= false;
		Boolean							bHasCkid							= false;
		Boolean							bHasRevisionInfo			= false;
		
		char								*strPath							= NULL;
	
		UInt32							intPathLen						= 0;
		SInt32							intModDate						= 0;
		
		Str255							pstrRevision					= "\p";
		Str255							pstrOptions						= "\p";
		Str255							pstrTimeStamp					= "\p";
		Str255							pstrActualTimeStamp 	= "\p";
		Str255							pstrTag								= "\p";
	
#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255							pstrTranslated				= "\p";
#endif
	
		FSSpec							fsTargetSpec					= { 0, 0, "\p" };
		FSSpec							fsParentSpec					=	{ 0, 0, "\p" };
					
		CkidResourcePB			*pCkidInfo						= NULL;
		CCVSEngineFileMappingRec fMapping;

		try 
			{
				if ( ::StackSpace() < kDefaultMinimumStackSize )
					{
						ThrowOSErr( memFullErr );
					}
					
				Yield();
				
				fMapping.Init();
				
				Assert_( (inTargetPB.dirInfo.ioFlAttrib & ioDirMask) == 0 );

//  Since this function is called recursively, 
//  we allocate the paramblock on the heap.
				
				pCkidInfo = (CkidResourcePB *)::NewPtr( sizeof(CkidResourcePB) );
				
				if ( pCkidInfo == NULL )
					throw( (OSErr)::MemError() );

				pCkidInfo->fCommentHndl = NULL;
				
				strPath = ::NewPtr( kMaxPathSize );
				
				if ( strPath == NULL )
					throw( (OSErr)::MemError() );
					
				PStrCopy( inTargetPB.hFileInfo.ioNamePtr, fsTargetSpec.name, sizeof(fsTargetSpec.name) );
				fsTargetSpec.parID = inTargetPB.hFileInfo.ioFlParID;
				fsTargetSpec.vRefNum = inTargetPB.hFileInfo.ioVRefNum;

//  Bail if the file is invisible.

				if ( inTargetPB.hFileInfo.ioFlFndrInfo.fdFlags & fInvisible )
					{					
						result = kCVSEngineFileInvisibleErr;
						throw result;					
					}

				result = FSpGetParentFSSpec( &fsTargetSpec, &fsParentSpec );
				ThrowOSErr( result );
				
				if ( mEntries != NULL )
					if ( !mEntries->AreYouThisDirectory(&fsParentSpec) )
						{						
							result = mEntries->CloseEntriesFile();
							ThrowOSErr( result );

							result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdPerm );							
							ThrowOSErr( result );								
						}
					
				if ( mEntries == NULL )
					{						
						mEntries = new CCVSEntriesCache( this );
					
						if ( mEntries != NULL )
							result = mEntries->LoadWorkingDirectory( &fsParentSpec, fsRdPerm );
						else result = kCVSEngineOutOfMemoryErr;
						
						ThrowOSErr( result );						
					}
				
				if ( mNotifier != NULL ) 
					{
						if ( (inStrParPath != NULL) && (inParPathLen > 0) )
							{
								if ( inParPathLen < kMaxPathSize )
									{
										intPathLen = inParPathLen;
									}
								else
									{
										intPathLen = kMaxPathSize;
									}
									
								::BlockMoveData( inStrParPath, strPath, intPathLen );
							
								if ( (kMaxPathSize - intPathLen) > (fsTargetSpec.name[0] + 1) )
									{
										::BlockMoveData( (char *)&fsTargetSpec.name[1], &strPath[intPathLen + 1], fsTargetSpec.name[0] );
										strPath[intPathLen] = '/';
										intPathLen += fsTargetSpec.name[0] + 1;
									}
							}
						else
							{
								intPathLen = kMaxPathSize - 1;
								result = FSpGetLocalCVSPath( &fsTargetSpec, &mLocalRoot, strPath, &intPathLen );
								ThrowOSErr( result );
								strPath[intPathLen] = 0;
							}
							
						bProcessFile = (*mNotifier)( (CCVSEngineRef)this, &fsTargetSpec, kCVSEngineEntryPreActionEvent, noErr, intPathLen, strPath, mContextPtr );										
					}

//  If they return false, do not process this file.
					
				if ( !bProcessFile )
					ThrowOSErr( kCVSEngineSkippedFile );

//  Get the entry

				result = mEntries->GetEntry( fsTargetSpec.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
				
				if ( result == noErr )
					{
						bHasRevisionInfo = true;
					}
				else if ( result == kCVSEngineNoRevisionDataErr )
					{
						result = noErr;
					}

				ThrowOSErr( result );
					
				if ( bHasRevisionInfo )
					{			
						result = FSpReadCkidResource( &fsTargetSpec, pCkidInfo, false );
						
						if ( result == noErr )
							{
								bHasCkid = true;
							}
						else
							{
								result = noErr;
							}
			
						if ( bHasCkid )
							{
								result = CUIsFileModified( pCkidInfo, pstrTimeStamp, inTargetPB.hFileInfo.ioFlMdDat );
							}
						else
							{
								result = CUIsFileModified( NULL, pstrTimeStamp, inTargetPB.hFileInfo.ioFlMdDat );		
							}
						
						if ( result == kCVSEngineFileLocallyModifiedErr )
							{
								bModified = true;
								result = noErr;
							}
							
						ThrowOSErr( result );
																	
						mStream << "Entry ";

		//  Write name

						mStream << "/";
						
#ifdef CVSENGINE_USE_MIME_FILE_NAMES						
						CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
						mStream << pstrTranslated;						
#else						
						mStream << fsTargetSpec.name;						
#endif

						mStream << "/";
						mStream	<< pstrRevision;
						mStream << "/";

						if ( inSendConflictInfo )
							mStream << pstrTimeStamp;
						
						mStream << "/";
						mStream << pstrOptions;
						mStream << "/";
						mStream << pstrTag;									
						mStream << "\n";

		//  If the file has been modified and auto merge is on
		//  sent the modified entry.
						
						if ( inDoMerge && bModified )
							{
								if ( !CUIsCVSBinaryOption(pstrOptions) || (inSendModifiedBinaryData) )
									{			
										//  Don't fetch the mapping unless the file is binary.
										//  That way we don't inadvertently send an encoded
										//  text file. This will change if we support RBL some day.
										
										if ( CUIsCVSBinaryOption(pstrOptions) )
											{ 				
												if ( !mMappings.GetMapping(fsTargetSpec.name, fMapping, false) )
													{
														fMapping.fEncoding = kCVSEngineBinaryEncoding;
													}							
											}
										else
											{
												fMapping.fEncoding = kCVSEngineTextEncoding;
											}
											
										WriteModifiedEntry( fsTargetSpec, fMapping.fEncoding, inPerm );
									}
								else if ( inUseUnchanged )
									{							
										mStream << kCVSUnchangedToken;
										mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
										CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
										mStream << pstrTranslated;
#else
										mStream << fsTargetSpec.name;
#endif

										mStream << "\n";	
									}							
							}
						else if ( inUseUnchanged )
							{							
								mStream << kCVSUnchangedToken;
								mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
								CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
								mStream << pstrTranslated;
#else
								mStream << fsTargetSpec.name;
#endif

								mStream << "\n";	
							}
					}
				else
					{
						mStream << kCVSQuestionableToken;
						mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
						CUTranslateFromMacNamePStr( fsTargetSpec.name, pstrTranslated );
						mStream << pstrTranslated;
#else
						mStream << fsTargetSpec.name;
#endif

						mStream << "\n";
					}
					
				if ( pCkidInfo != NULL )
					{										
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;
					}
					
				if ( strPath != NULL )
					{						
						::DisposePtr( strPath );						
						strPath = NULL;
						intPathLen = 0;					
					}				
			}
							
		catch ( OSErr inErr )
			{
				if ( (inErr != kCVSEngineFileHasNoCkid) && (inErr != kCVSEngineFileHasNoMcvs) && (inErr != eofErr) && (inErr != opWrErr) )
					{		
						if ( mNotifier != NULL )
							{						
								(*mNotifier)( (CCVSEngineRef)this, &fsTargetSpec, kCVSEngineErrorActionEvent, inErr, 0, NULL, mContextPtr );							
							}							
					}
								
				if ( pCkidInfo != NULL )
					{										
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;
					}
					
				if ( strPath != NULL )
					{						
						::DisposePtr( strPath );						
						strPath = NULL;
						intPathLen = 0;					
					}

				throw;			
			}
				
		catch ( ... )
			{				
				if ( pCkidInfo != NULL )
					{										
						::DisposePtr( (Ptr)pCkidInfo );
						pCkidInfo = NULL;
					}
					
				if ( strPath != NULL )
					{						
						::DisposePtr( strPath );						
						strPath = NULL;
						intPathLen = 0;					
					}
					
				throw;			
			}
		
	}
	
	
void
CCVSRepository::WriteModifiedEntry( const FSSpec &inTargetSpec, const UInt32 inEncoding, const SInt8 inPerm )
	{

//  WriteModifiedEntry() 

//  This function will send the actual data contents of a file to the
//  CVS server. It is usually called by WriteItemEntry() but is also
//  used in Add() and Commit().

//  Note: It is strongly recommended that you call this function with
//  the default of fsRdWrPerm. If for some reason WriteModifiedEntry()
//  fails to properly open the target file, it will throw an exception.
//  If this happens at the wrong time, you could potentially leave the
//  CVS server in a state where it is waiting for the rest of the file
//  to be sent.
	
		OSErr												result						= noErr;
		Boolean											bCalledPreAction	= false;		
		Boolean											bFileCreated			= false;
		
		SInt32											intFileSize				= 0;
		UInt32											intBytesToGet			= 0;
		UInt32											ioCount						= 0;
	
		FSSpec											fsEncoded					= { 0, 0, "\p" };	
		Str255											pstrFileSize			= "\p";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
		Str255											pstrTranslated		= "\p";
#endif
		
		CCVSEngineProgressStatusRec	curStatus 				= { 0, 0 };
				
		try 
			{
				if ( mNotifier != NULL )
					{					
						(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineModifiedEntryPreActionEvent, noErr, inEncoding, NULL, mContextPtr );
						bCalledPreAction = true;						
					}

#ifndef CVSENGINE_USE_MIME_FILE_NAMES
				if ( !CUIsValidCVSName(inTargetSpec.name) )	
					{					
						result = kCVSEngineInvalidFileNameErr;					
						throw result;					
					}
#endif
					
				if ( inEncoding == kCVSEngineAppleSingleEncoding )
					{

//  Make sure that we can open the file before we send any entry
//  information. This is necessary because if we bail halfway 
//  through due to a file system error (ie. -49), the CVS server 
//  ends up being very confused. Scope is important here because 
//  StFileAccess closes open filerefs on destruction.
						
						{						
							SInt16				dataForkRef = 0;
							SInt16				resForkRef	= 0;
							StFileAccess	modifiedFileAccess( inTargetSpec );

//  If something other than fsRdWrPerm is passed in for inPerm,
//  this won't guarantee we can open the file when we are
//  ready to send it. Remember that our socket implicitly yields
//  so someone else can grab the file between the time we send
//  the "Modified" entry and the actual file data.

							dataForkRef = modifiedFileAccess.GetFSDataForkRef( inPerm );
							resForkRef = modifiedFileAccess.GetFSResForkRef( inPerm );
							
							mStream << kCVSModifiedToken;
							mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
							CUTranslateFromMacNamePStr( inTargetSpec.name, pstrTranslated );
							mStream << pstrTranslated;
#else
							mStream << inTargetSpec.name;
#endif

							mStream << "\n";
						
							mStream << kCVSUnlockedPermissionStr;
							mStream << "\n";						
						}
					
						FSpSendAppleSingleData( inTargetSpec, inPerm );							
					}
			
				else if ( inEncoding == kCVSEngineBinhex4Encoding )
					{			
						//  Pass inPerm to FSpEncodeBinhex4() so it will lock the
						//  target file between thread yields (if inPerm == fsRdWrPerm).
								
						fsEncoded = inTargetSpec;
						result = FSpEncodeBinhex4( fsEncoded, CCVSRepository::EncodingYieldProc, this, inPerm );
						ThrowOSErr( result );
	
						SInt16 intEncodedDatRef = 0;
						SInt16 intTargetDatRef = 0;
						
						StFileAccess stEncodedFileAccess( fsEncoded );
						
						//  Turn on the delete flag first. Since FSpEncodeBinhex4 returned
						//  noErr, we know the file exists. If we have some random problem
						//  opening the file next, it will be nuked when we leave.
						
						stEncodedFileAccess.SetDelete( true );
						
						StFileAccess stTargetFileAccess( inTargetSpec );
						
						//  Always open the encoded file with fsRdWrPerm, since the
						//  caller has no knowledge that it exists.
						
						intEncodedDatRef = stEncodedFileAccess.GetFSDataForkRef( fsRdWrPerm );
						intTargetDatRef = stTargetFileAccess.GetFSDataForkRef( inPerm );
						
						result = ::GetEOF( intEncodedDatRef, &intFileSize );
						ThrowOSErr( result );
						
						mStream << kCVSModifiedToken;
						mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
 						CUTranslateFromMacNamePStr( inTargetSpec.name, pstrTranslated );
						mStream << pstrTranslated;
#else
						mStream << inTargetSpec.name;
#endif

						mStream << "\n";						
						mStream << kCVSUnlockedPermissionStr;	
						mStream << "\n";

						curStatus.fMax = intFileSize;
						curStatus.fCompleted = 0;
						
						//  Ok, the notifier lies. It passes inTargetSpec as the file, but it's really
						//  sending fsEncoded.
						
						if ( mNotifier != NULL )
							{
								(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
							}
							
//  Send file size information

						UInt32ToStr255( intFileSize, pstrFileSize );
				
						mStream << pstrFileSize;
						mStream << "\n";

//  Send data
									
						intBytesToGet = intFileSize;
							
						while ( intBytesToGet > 0 )
							{							
								ioCount = intBytesToGet;
								
								if ( ioCount > mBufSize )
									ioCount = mBufSize;
							
								result = FSReadOffset( intEncodedDatRef, &ioCount, mBuffer, intFileSize - intBytesToGet );
								ThrowOSErr( result );											
																	
								mStream.UnbufferedWriteBlock( mBuffer, ioCount );
								
								intBytesToGet -= ioCount;
								curStatus.fCompleted += ioCount;
								
								if ( mNotifier != NULL )
									{
										(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );								
									}
							}									
					}
				else if ( inEncoding == kCVSEngineMacBinaryEncoding )
					{
						{						
							SInt16				dataForkRef = 0;
							SInt16				resForkRef	= 0;
							StFileAccess	modifiedFileAccess( inTargetSpec );

							dataForkRef = modifiedFileAccess.GetFSDataForkRef( inPerm );
							resForkRef = modifiedFileAccess.GetFSResForkRef( inPerm );
							
							mStream << kCVSModifiedToken;
							mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
							CUTranslateFromMacNamePStr( inTargetSpec.name, pstrTranslated );
							mStream << pstrTranslated;
#else
							mStream << inTargetSpec.name;
#endif

							mStream << "\n";
							
							mStream << kCVSUnlockedPermissionStr;
							mStream << "\n";
							
							UInt32 dataForkSize;
							UInt32 resourceForkSize;
							UInt32 totalSize;
							Str255 theComment;
							
							result = FSpEncodeMacBinaryIIIHeader(&inTargetSpec, mBuffer, &dataForkSize,
								&resourceForkSize, &totalSize, theComment);
							ThrowOSErr(result);
							
							curStatus.fMax = totalSize;
							curStatus.fCompleted = 0;
							
							if ( mNotifier != NULL )
									(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );

//  Send file size information

							UInt32ToStr255( totalSize, pstrFileSize );
					
							mStream << pstrFileSize;
							mStream << "\n";

//  Send MacBinary III header

							mStream.UnbufferedWriteBlock(mBuffer, kMacBinaryHeaderSize);
							curStatus.fCompleted += kMacBinaryHeaderSize;
							if ( mNotifier != NULL )
									(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
						
//  Send data fork

							intBytesToGet = dataForkSize;
							
							while (intBytesToGet)
								{
									if (intBytesToGet > mBufSize)
										ioCount = mBufSize;
									else
										ioCount = intBytesToGet;
									
									result = FSReadOffset( dataForkRef, &ioCount, mBuffer, dataForkSize - intBytesToGet );
									ThrowOSErr( result );
									
									mStream.UnbufferedWriteBlock( mBuffer, ioCount );
									
									intBytesToGet -= ioCount;
									curStatus.fCompleted += ioCount;
									
									if ( mNotifier != NULL )
											(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
								}
						
							// Pad the data fork with zeros to a multiple of kMacBinaryHeaderSize (128) bytes
							if (dataForkSize & 0x7F)
								{
									memset(mBuffer, 0, kMacBinaryHeaderSize);
									mStream.UnbufferedWriteBlock( mBuffer, kMacBinaryHeaderSize - (dataForkSize &0x7F) );
									curStatus.fCompleted += ioCount;
									// Don't bother with calling the notifier here - it'll be called again very soon anyway
								}

//  Send resource fork

							intBytesToGet = resourceForkSize;
							
							while (intBytesToGet)
								{
									if (intBytesToGet > mBufSize)
										ioCount = mBufSize;
									else
										ioCount = intBytesToGet;
									
									result = FSReadOffset( resForkRef, &ioCount, mBuffer, resourceForkSize - intBytesToGet );
									ThrowOSErr( result );
									
									mStream.UnbufferedWriteBlock( mBuffer, ioCount );
									
									intBytesToGet -= ioCount;
									curStatus.fCompleted += ioCount;
									
									if ( mNotifier != NULL )
											(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
								}
							
						// Pad the resource fork with zeros to a multiple of kMacBinaryHeaderSize (128) bytes
							if (resourceForkSize & 0x7F)
								{
									memset(mBuffer, 0, kMacBinaryHeaderSize);
									mStream.UnbufferedWriteBlock( mBuffer, kMacBinaryHeaderSize - (resourceForkSize &0x7F) );
									curStatus.fCompleted += ioCount;
									// Don't bother with calling the notifier here - it'll be called again very soon anyway
								}

//  Send file comment

							if (theComment[0])
								{
									mStream.UnbufferedWriteBlock( &(theComment[1]), theComment[0] );
									
									curStatus.fCompleted += theComment[0];
								}
								
							if ( mNotifier != NULL )
									(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
						}
					}
			
											
//  Else just send the data fork.

				else
					{
						SInt16				dataForkRef = 0;
						StFileAccess	modifiedFileAccess( inTargetSpec );
						
						dataForkRef = modifiedFileAccess.GetFSDataForkRef( inPerm );
													
						result = ::GetEOF( dataForkRef, &intFileSize );
						ThrowOSErr( result );

						mStream << kCVSModifiedToken;
						mStream << " ";

#ifdef CVSENGINE_USE_MIME_FILE_NAMES
						CUTranslateFromMacNamePStr( inTargetSpec.name, pstrTranslated );
						mStream << pstrTranslated;
#else
						mStream << inTargetSpec.name;
#endif

						mStream << "\n";						
						mStream << kCVSUnlockedPermissionStr;	
						mStream << "\n";

						curStatus.fMax = intFileSize;
						curStatus.fCompleted = 0;
						
						if ( mNotifier != NULL )
							{
								(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
							}
							
//  Send file size information

						UInt32ToStr255( intFileSize, pstrFileSize );
				
						mStream << pstrFileSize;
						mStream << "\n";

//  Send data
									
						intBytesToGet = intFileSize;
							
						while ( intBytesToGet > 0 )
							{							
								ioCount = intBytesToGet;
								
								if ( ioCount > mBufSize )
									ioCount = mBufSize;
							
								result = FSReadOffset( dataForkRef, &ioCount, mBuffer, intFileSize - intBytesToGet );
								ThrowOSErr( result );											

//  Convert CRs to LFs before sending non-binary files. *Important*
								
								if ( inEncoding == kCVSEngineTextEncoding )
									{										
										char *bufPtr = mBuffer;
										char *bufEnd = mBuffer + ioCount;
										
										while (bufPtr < bufEnd)
											{											
												if ( *bufPtr == 0x0D )
													*bufPtr = 0x0A;
													
												bufPtr ++;										
											}											
									}
									
								mStream.UnbufferedWriteBlock( mBuffer, ioCount );
								
								intBytesToGet -= ioCount;
								curStatus.fCompleted += ioCount;
								
								if ( mNotifier != NULL )
									{
										(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );								
									}
							}				
					}
					
				if ( (mNotifier != NULL) && (bCalledPreAction) )
					{
						(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineModifiedEntryPostActionEvent, result, inEncoding, NULL, mContextPtr );
					}
			}					
		catch ( OSErr inErr )
			{			
				result = inErr;
			
				if ( (mNotifier != NULL) && (bCalledPreAction) )
					{
						(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineModifiedEntryPostActionEvent, result, inEncoding, NULL, mContextPtr );
					}
					
				throw inErr;			
			}
		catch ( ... )
			{
				if ( (mNotifier != NULL) && (bCalledPreAction) )
					{					
						(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineModifiedEntryPostActionEvent, kCVSEngineUnknownExceptionErr, inEncoding, NULL, mContextPtr );						
					}
			
				throw;		
			}
	
	}
		

void
CCVSRepository::WriteImportEntries( const FSSpec &inTargetSpec, const Boolean inSendBinaryFiles, UInt32 &outSkippedFiles )
	{

//  WriteImportEntries()

//  This function is very similiar to WriteEntries() in the fact
//  that it sends all the CVS "Entry" lines for all the files in 
//  the directory that inTargetSpec specifies. The main difference
//  is that WriteImportEntries() always sends a "Modified" entry
//  for each file. There are also subtle differences in how the 
//  path names are specified to the CVS server.

//  If inSendBinaryFiles is false, WriteImportEntries() will only
//  send the entries for files that are specified as "TEXT". This
//  is determined by the mapping table.

//  OutSkippedFiles is the count of files that WriteImportEntries()
//  did not send entries of to the CVS server. (ie. if inSendBinaryFiles
//  was false and there were 3 binary files in the directory, then
//  outSkippedFiles would be 3) 

//  See the Import() function for why this is done this way.

		OSErr				result			= noErr;
		FSSpec			targetSpec 	= inTargetSpec;
								
		CInfoPBRec	pb;
	
		outSkippedFiles = 0;
			
		pb.hFileInfo.ioCompletion 	= NULL;
		pb.hFileInfo.ioFDirIndex 		= 0;
		pb.hFileInfo.ioNamePtr 			= targetSpec.name;
		pb.hFileInfo.ioVRefNum 			= targetSpec.vRefNum;
		pb.hFileInfo.ioDirID 				= targetSpec.parID;
		
		result = ::PBGetCatInfoSync( &pb );
		ThrowOSErr( result );
						
		if ( pb.dirInfo.ioFlAttrib & ioDirMask )
			{			
				if ( (pb.hFileInfo.ioFlFndrInfo.fdFlags & fInvisible) == 0 )
					{					
						WriteImportDirectoryEntries( pb, targetSpec, inSendBinaryFiles, outSkippedFiles );		
					}				
				else ThrowOSErr( kCVSEngineFileInvisibleErr );				
			}			
	}
	
	
void
CCVSRepository::WriteImportDirectoryEntries( const CInfoPBRec &inTargetPB, const FSSpec &inBaseSpec, const Boolean inSendBinaryFiles, UInt32 &outSkippedFiles )
	{

//  WriteImportDirectoryEntry() 

//  This is a sub-function of WriteImportEntries(). It takes 
//  a FSSpec and writes out all the "Entry" and "Modified"
//  CVS entry lines of all the files inside the directory to 
//  the socket stream.
	
		OSErr					result											= noErr;
		char					*strDirPath									= NULL;
	
		UInt32				intDirPathLen								= 0;
		UInt32				intItemCount 								= 0;

		Str63					pstrTempName								= "\p";
		FSSpec				fsItem											= { 0, 0, "\p" };
		
		CInfoPBRec		pbChild											= inTargetPB;
		
		try
			{
				if ( ::StackSpace() < kDefaultMinimumStackSize )
					throw( (OSErr) memFullErr );

				PStrCopy( inTargetPB.hFileInfo.ioNamePtr, fsItem.name, sizeof(fsItem.name) );
				fsItem.parID = inTargetPB.hFileInfo.ioFlParID;
				fsItem.vRefNum = inTargetPB.hFileInfo.ioVRefNum;

//  Since this function is called recursively, we 
//  allocate the path on the heap.
				
				intDirPathLen = kMaxPathSize;
				strDirPath = ::NewPtr( intDirPathLen );
				
				if ( strDirPath == NULL )
					throw( ::MemError() );

//  See comments in WriteImportEntries about inRootSpec.

				intDirPathLen = kMaxPathSize - 1;
				result = FSpGetLocalCVSPath( &fsItem, &inBaseSpec, strDirPath, &intDirPathLen );
				strDirPath[intDirPathLen] = 0;
						
				ThrowOSErr( result );				

#ifndef CVSENGINE_USE_MIME_FILE_NAMES
				if ( !CUIsValidCVSName(inTargetPB.hFileInfo.ioNamePtr) )
					{					
						result = kCVSEngineInvalidFileNameErr;
						throw result;					
					}
#endif
					
				intItemCount = inTargetPB.dirInfo.ioDrNmFls;
				
				pbChild.hFileInfo.ioNamePtr = pstrTempName;

				for ( UInt32 index = 1; index <= intItemCount; index++ )
					{				
						pbChild.hFileInfo.ioFDirIndex = index;
						
						result = ::PBGetCatInfoSync( &pbChild );
						ThrowOSErr( result );
							
						if ( (pbChild.dirInfo.ioFlAttrib & ioDirMask) && ((pbChild.hFileInfo.ioFlFndrInfo.fdFlags & fInvisible) == 0) )
							{							
								try
									{									
										WriteImportDirectoryEntries( pbChild, inBaseSpec, inSendBinaryFiles, outSkippedFiles );					
									}
									
								catch ( OSErr inErr )
									{									
										if ( (inErr == memFullErr) || (inErr == kCVSEngineOperationAbortedErr) || IsCTCPSocketErr(inErr) )
											throw inErr;									
									}																		
							}
							
						pbChild.hFileInfo.ioDirID = inTargetPB.hFileInfo.ioDirID;						
					}
					
				mStream << kCVSDirectoryToken;
				mStream << " ";
				
				if ( intDirPathLen > 0 )
					{
						mStream.WriteBlock( strDirPath, intDirPathLen );				
					}
				else
					{
						mStream << ".";
					}
					
				mStream << "\n";
				
				mStream << mRemoteRoot;

				if ( (mRemoteRoot[0] == 0) || (intDirPathLen > 0) )
					{
						mStream << "/";
					}
				
				if ( intDirPathLen > 0 )
					{
						mStream.WriteBlock( strDirPath, intDirPathLen );
					}	
					
				mStream << "\n";
					
				for ( UInt32 index = 1; index <= intItemCount; index++ )
					{					
						pbChild.hFileInfo.ioFDirIndex = index;
						
						result = ::PBGetCatInfoSync( &pbChild );
						ThrowOSErr( result );
								
						if ( (pbChild.dirInfo.ioFlAttrib & ioDirMask) == 0 )
							if ( (pbChild.hFileInfo.ioFlFndrInfo.fdFlags & fInvisible) == 0 )
								{							
									::BlockMoveData( pstrTempName, &fsItem.name, pstrTempName[0] + 1 );
									fsItem.parID = pbChild.hFileInfo.ioFlParID;
									fsItem.vRefNum = pbChild.hFileInfo.ioVRefNum;

//  Catch exceptions from WriteItemEntry.

									try 
										{										
											CCVSEngineFileMappingRec	sMapping;
											sMapping.Init();

//  Here we fetch the encoding preference for this particular 
//  file. If we fail to successfully retrieve an encoding 
//  preference, then we make the determination of whether the 
//  file is text or binary based on the file type.
											
											if ( !mMappings.GetMapping(pbChild.hFileInfo.ioNamePtr, sMapping, false) )
												{												
													if ( pbChild.hFileInfo.ioFlFndrInfo.fdType != 'TEXT' )
														{											
															sMapping.fEncoding = kCVSEngineAppleSingleEncoding;
														}
													else 
														{
															sMapping.fEncoding = kCVSEngineTextEncoding;
														}												
												}
																							
											if ( inSendBinaryFiles )
												{												
													if ( 	(sMapping.fEncoding == kCVSEngineBinaryEncoding) || 
																(sMapping.fEncoding == kCVSEngineAppleSingleEncoding) || 
																(sMapping.fEncoding == kCVSEngineBinhex4Encoding) ||
																(sMapping.fEncoding == kCVSEngineMacBinaryEncoding) )
														{
															WriteModifiedEntry ( fsItem, sMapping.fEncoding );
														}
													else 
														{
															outSkippedFiles++;
														}									
												}											
											else
												{												
													if ( sMapping.fEncoding == kCVSEngineTextEncoding )
														{
															WriteModifiedEntry ( fsItem, sMapping.fEncoding );
														}
													else 
														{
															outSkippedFiles++;
														}
												}						
										}						
									catch ( OSErr inErr )
										{										
											if ( (inErr == memFullErr) || (inErr == kCVSEngineOperationAbortedErr) || IsCTCPSocketErr(inErr) )
												{
													throw inErr;	
												}											
										}																													
								}						
							else
								{															
									if ( mNotifier != NULL )
										{									
											FSSpec theSpec = { 0, 0, "\p" };
											
											::BlockMoveData( pstrTempName, theSpec.name, pstrTempName[0] + 1 );
												
											theSpec.vRefNum = inTargetPB.hFileInfo.ioVRefNum;
											theSpec.parID = inTargetPB.hFileInfo.ioFlParID;
											
											(*mNotifier)( (CCVSEngineRef)this, &theSpec, kCVSEngineErrorActionEvent, kCVSEngineFileInvisibleErr, 0, NULL, mContextPtr );											
										}							
								}
								
						pbChild.hFileInfo.ioDirID = inTargetPB.hFileInfo.ioDirID;			
					}
					
				::DisposePtr( strDirPath );
				strDirPath = NULL;		
			}
		
		catch ( OSErr inErr )
			{			
				if ( mNotifier != NULL )
					{					
						FSSpec	theSpec	= { 0, 0, "\p" };
						
						if ( inTargetPB.hFileInfo.ioNamePtr != NULL )
							{
								::BlockMoveData( inTargetPB.hFileInfo.ioNamePtr, theSpec.name, inTargetPB.hFileInfo.ioNamePtr[0] + 1 );
							}
							
						theSpec.vRefNum = inTargetPB.hFileInfo.ioVRefNum;
						theSpec.parID = inTargetPB.hFileInfo.ioFlParID;
						
						if ( (strDirPath != NULL) && (intDirPathLen > 0) )
							{
								(*mNotifier)( (CCVSEngineRef)this, &theSpec, kCVSEngineErrorActionEvent, result, intDirPathLen, strDirPath, mContextPtr );
							}
						else 
							{
								(*mNotifier)( (CCVSEngineRef)this, &theSpec, kCVSEngineErrorActionEvent, result, 0, NULL, mContextPtr );					
							}
					}
								
				if ( strDirPath != NULL )
					{					
						::DisposePtr( strDirPath );
						strDirPath = NULL;					
					}
					
				throw inErr;		
			}
				
		catch ( ... )
			{		
				if ( strDirPath != NULL )
					{					
						::DisposePtr( strDirPath );
						strDirPath = NULL;				
					}
					
				throw;		
			}
		
	}


void		
CCVSRepository::FSpSendAppleSingleData(	const FSSpec &inTargetSpec, const SInt8 inPerm )
	{
	
		OSErr								result 								= noErr;

		SInt16							dataForkRef						= 0;
		SInt16							resForkRef						= 0;

		UInt32							dataSize							= 0;
		UInt32							curOffset							= 0;
		UInt32							bytesToGet						= 0;
		UInt32							copyLen								= 0;
	
		Str255							fileSizeStr						= "\p";
		
		CInfoPBRec 					pb;
		
		AppleSingleHeader		encodeFileHeader;
		AppleSingleEntry		dataForkEntry;
		AppleSingleEntry		resForkEntry;
		AppleSingleEntry		finderInfoEntry;
		
		FInfo								finderInfo;
		FXInfo							extendedInfo;
		
		CCVSEngineProgressStatusRec	curStatus			= { 0, 0 };
		
		try
			{
				
				StFileAccess	targetFileAccess( inTargetSpec );
				
				dataForkRef = targetFileAccess.GetFSDataForkRef( inPerm );

//  Even if a file doesn't have a rsrc. fork, we should still
//  be able to send it.

				try
					{
					
						resForkRef = targetFileAccess.GetFSResForkRef( inPerm );
				
					}
					
				catch ( OSErr inErr )
					{
					
						result = inErr;
						
						if ( result != eofErr )
							throw inErr;
					
						resForkRef = 0;
						
					}
	
				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= const_cast<unsigned char *>(inTargetSpec.name);
				pb.hFileInfo.ioVRefNum 		= inTargetSpec.vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				pb.hFileInfo.ioDirID 			= inTargetSpec.parID;

				result = ::PBGetCatInfoSync( &pb );
				
				if ( result != noErr )
					throw result;
				
				encodeFileHeader.fMagicNumber = kAppleSingleMagicNumber;
				encodeFileHeader.fVersion 		= kAppleSingleVersion;
				encodeFileHeader.fFiller1 		= 0;
				encodeFileHeader.fFiller2 		= 0;
				encodeFileHeader.fFiller3 		= 0;
				encodeFileHeader.fFiller4 		= 0;
				encodeFileHeader.fEntries 		= 2;

				curOffset = sizeof(AppleSingleHeader) + 2 * sizeof(AppleSingleEntry);
				
				if ( pb.hFileInfo.ioFlRLgLen > 0 )
					curOffset += sizeof(AppleSingleEntry);
				
				dataForkEntry.fEntryID = kAppleSingleDataFork;
				dataForkEntry.fOffset = curOffset;
				dataForkEntry.fLength = pb.hFileInfo.ioFlLgLen;
				
				curOffset += dataForkEntry.fLength;
				
				if ( pb.hFileInfo.ioFlRLgLen > 0 )
					{
											
						if ( resForkRef == 0 )
							{

								result = eofErr;
								throw result;
							
							}
							
						encodeFileHeader.fEntries++;
						
						resForkEntry.fEntryID = kAppleSingleResFork;
						resForkEntry.fOffset = curOffset;
						resForkEntry.fLength = pb.hFileInfo.ioFlRLgLen;
				
						curOffset += resForkEntry.fLength;
							
					}
					
				finderInfoEntry.fEntryID = kAppleSingleMacFInfo;
				finderInfoEntry.fOffset = curOffset;
				finderInfoEntry.fLength = sizeof(FInfo) + sizeof(FXInfo);
				
				curOffset += finderInfoEntry.fLength;
				
				dataSize = curOffset;
				
				UInt32ToStr255( dataSize, fileSizeStr );
				
				mStream << fileSizeStr;
				mStream << "\n";
				
				curStatus.fMax = dataSize;
				
				mStream.UnbufferedWriteBlock( &encodeFileHeader, sizeof(AppleSingleHeader) );
				curStatus.fCompleted += sizeof(AppleSingleHeader);
				
				mStream.UnbufferedWriteBlock( &dataForkEntry, sizeof(AppleSingleEntry) );
				curStatus.fCompleted += sizeof(AppleSingleEntry);

				if ( pb.hFileInfo.ioFlRLgLen > 0 )
					{

						mStream.UnbufferedWriteBlock( &resForkEntry, sizeof(AppleSingleEntry) );
						curStatus.fCompleted += sizeof(AppleSingleEntry);

					}
					
				mStream.UnbufferedWriteBlock( &finderInfoEntry, sizeof(AppleSingleEntry) );
				curStatus.fCompleted += sizeof(AppleSingleEntry);
				
				if ( mNotifier != NULL )
					(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );		

//  Send data fork

				bytesToGet = dataForkEntry.fLength;
				
				while ( bytesToGet > 0 )
					{
					
						copyLen = bytesToGet;
						
						if ( copyLen > mBufSize )
							copyLen = mBufSize;
							
						result = FSReadOffset( dataForkRef, &copyLen, mBuffer, dataForkEntry.fLength - bytesToGet );
						
						if ( result != noErr )
							throw result;
					
						mStream.UnbufferedWriteBlock( mBuffer, copyLen );

						curStatus.fCompleted += copyLen;
						
						if ( mNotifier != NULL )
							(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );		

						Yield();

						bytesToGet -= copyLen;
										
					}

//  Send resource fork

				bytesToGet = pb.hFileInfo.ioFlRLgLen;
				
				while ( bytesToGet > 0 )
					{
					
						copyLen = bytesToGet;
						
						if ( copyLen > mBufSize )
							copyLen = mBufSize;
							
						result = FSReadOffset( resForkRef, &copyLen, mBuffer, pb.hFileInfo.ioFlRLgLen - bytesToGet );
						
						if ( result != noErr )
							throw result;
					
						mStream.UnbufferedWriteBlock( mBuffer, copyLen );

						curStatus.fCompleted += copyLen;
						
						if ( mNotifier != NULL )
							(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );		

						Yield();

						bytesToGet -= copyLen;
					
					}

//  Send FInfo
					
				finderInfo = pb.hFileInfo.ioFlFndrInfo;
				extendedInfo = pb.hFileInfo.ioFlXFndrInfo;
				
				mStream.UnbufferedWriteBlock( &finderInfo, sizeof(FInfo) );
				curStatus.fCompleted += sizeof(FInfo);

				mStream.UnbufferedWriteBlock( &extendedInfo, sizeof(FXInfo) );
				curStatus.fCompleted += sizeof(FXInfo);
				
				if ( mNotifier != NULL )
					(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );		
					
			}
			
		catch ( ... )
			{

				throw;
							
			} 
			
	}
	

/*

void		
CCVSRepository::FSpSendPsychoAppleDoubleData(	const FSSpec &inTargetSpec, const SInt8 inPerm )
	{
	
		OSErr								result 								= noErr;
		UInt32							dataSize							= 0,
												curOffset							= 0,
												bytesToGet						= 0,
												copyLen								= 0;
	
		Str255							fileSizeStr						= "\p";
		
		SInt16							dataForkRef						= 0,
												resForkRef						= 0;
		
		FInfo								finderInfo;

		CCVSEngineProgressStatusRec	curStatus						= { 0, 0 };

		AppleSingleHeader		encodeFileHeader;
		AppleSingleEntry		dataForkEntry,
												resForkEntry,
												finderInfoEntry;
			
		CInfoPBRec 					pb;
		
		try
			{
				
				StFileAccess	targetFileAccess( inTargetSpec );
				
				dataForkRef = targetFileAccess.GetFSDataForkRef( inPerm );

//  Even if a file doesn't have a rsrc. fork, we should still
//  be able to send it.

				try
					{
					
						resForkRef = targetFileAccess.GetFSResForkRef( inPerm );
				
					}
					
				catch ( OSErr inErr )
					{
					
						if ( inErr != eofErr )
							throw inErr;
					
						resForkRef = 0;
						
					}

				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioNamePtr 		= const_cast<unsigned char *>(inTargetSpec.name);
				pb.hFileInfo.ioVRefNum 		= inTargetSpec.vRefNum;
				pb.hFileInfo.ioFDirIndex 	= 0;
				pb.hFileInfo.ioDirID 			= inTargetSpec.parID;

				result = ::PBGetCatInfoSync( &pb );
				
				if ( result != noErr )
					throw result;
				
				encodeFileHeader.fMagicNumber = kAppleDoubleMagicNumber;
				encodeFileHeader.fVersion 		= kWeirdoAppleDoubleVersion;
				encodeFileHeader.fFiller1 		= 'Maci';
				encodeFileHeader.fFiller2 		= 'ntos';
				encodeFileHeader.fFiller3 		= 'h   ';
				encodeFileHeader.fFiller4 		= '    ';
				encodeFileHeader.fEntries 		= 2;
								
				dataForkEntry.fEntryID = kAppleSingleDataFork;
				dataForkEntry.fOffset = sizeof(AppleSingleHeader) + sizeof(AppleSingleEntry) * 2;
				dataForkEntry.fLength = pb.hFileInfo.ioFlLgLen;
								
				resForkEntry.fEntryID = kAppleSingleResFork;
				resForkEntry.fOffset = dataForkEntry.fOffset + dataForkEntry.fLength;
				resForkEntry.fLength = pb.hFileInfo.ioFlRLgLen;
							
				finderInfoEntry.fEntryID = kAppleSingleMacFInfo;
				finderInfoEntry.fOffset = resForkEntry.fOffset + resForkEntry.fLength;
				finderInfoEntry.fLength = sizeof( FInfo );
				
				dataSize = sizeof(AppleSingleHeader) + (3 * sizeof(AppleSingleEntry)) + dataForkEntry.fLength + resForkEntry.fLength + finderInfoEntry.fLength;
				
				UInt32ToStr255( dataSize, fileSizeStr );
				
				mStream << fileSizeStr;
				mStream << "\n";

				curStatus.fMax = dataSize;
				
				mStream.WriteBlock( &encodeFileHeader, sizeof(AppleSingleHeader) );
				curStatus.fCompleted += sizeof(AppleSingleHeader);
		
				mStream.WriteBlock( &dataForkEntry, sizeof(AppleSingleEntry) );
				curStatus.fCompleted += sizeof(AppleSingleEntry);

				mStream.WriteBlock( &resForkEntry, sizeof(AppleSingleEntry) );
				curStatus.fCompleted += sizeof(AppleSingleEntry);

				mStream.WriteBlock( &finderInfoEntry, sizeof(AppleSingleEntry) );
				curStatus.fCompleted += sizeof(AppleSingleEntry);
				
				if ( mNotifier != NULL )
					(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
				
//  Send data fork.

				bytesToGet = dataForkEntry.fLength;
				
				while ( bytesToGet > 0 )
					{
					
						copyLen = bytesToGet;
						
						if ( copyLen > mBufSize )
							copyLen = mBufSize;
							
						result = UCVSRepositoryUtils::FSReadOffset( dataForkRef, &copyLen, mBuffer, dataForkEntry.fLength - bytesToGet );
						
						if ( result != noErr )
							throw result;
					
						mStream.WriteBlock( mBuffer, copyLen );
						
						curStatus.fCompleted += copyLen;
						
						if ( mNotifier != NULL )
							(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );

						Yield();
							
						bytesToGet -= copyLen;
					
					}

//  Send resource fork.

				bytesToGet = pb.hFileInfo.ioFlRLgLen;
				
				while ( bytesToGet > 0 )
					{
					
						copyLen = bytesToGet;
						
						if ( copyLen > mBufSize )
							copyLen = mBufSize;
							
						result = UCVSRepositoryUtils::FSReadOffset( resForkRef, &copyLen, mBuffer, pb.hFileInfo.ioFlRLgLen - bytesToGet );
						
						if ( result != noErr )
							throw result;
					
						mStream.WriteBlock( mBuffer, copyLen );
						
						curStatus.fCompleted += copyLen;
						
						if ( mNotifier != NULL )
							(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );

						Yield();

						bytesToGet -= copyLen;
					
					}

//  Send FInfo.
					
				finderInfo = pb.hFileInfo.ioFlFndrInfo;
				
				mStream.WriteBlock( &finderInfo, sizeof(FInfo) );
				
				curStatus.fCompleted += sizeof(FInfo);

				if ( mNotifier != NULL )
					(*mNotifier)( (CCVSEngineRef)this, &inTargetSpec, kCVSEngineProgressEvent, noErr, 0, &curStatus, mContextPtr );
									
			}
			
		catch ( ... )
			{

				throw;
							
			} 
			
	}
*/


void
CCVSRepository::FSpSetDirectory( const FSSpec &fsInTarget )
	{
		
		//  FSpSetDirectory() tells the CVS server what directory
		//  we are currently in. This function is full of hacks. 
		
		//  If we are setting the root directory ".", then
		//  FSpSetDirectory() will set the mRemoteRoot variable,
		//  NOT the data in the actual CVS/Root or CVS/Repository
		//  files. This enables different session files to "share"
		//  the same root for different repositories.
		
		OSErr		result											= noErr;
		char 		strLocalPath[kMaxPathSize]	= "";
		UInt32	intLocalPathLen 						= 0;
		
		Str255	pstrRoot										= "\p";
		Str255	pstrRepository							= "\p";
		Str255	pstrPath										= "\p";
		
		Boolean	bDoNotSendTag								= false;
		
		intLocalPathLen = kMaxPathSize - 1;
		result = FSpGetLocalCVSPath( &fsInTarget, &mLocalRoot, strLocalPath, &intLocalPathLen );
		ThrowOSErr( result );
	
		//  Get the repository (root) path from the CVS/Root
		//  file stored in each directory.
		
		result = FSpGetRoot( &fsInTarget, pstrRoot );
		ThrowOSErr( result );
		
		result = CUParseRootPStr( pstrRoot, NULL, NULL, pstrPath, NULL, NULL );
		ThrowOSErr( result );

		if ( pstrPath[0] == 0 )
			{
				//  If pstrRootPath is 0, that probably is because
				//  early versions of MacCVS Pro 2.5 didn't write
				//  the root file correctly. So we just copy the
				//  root specified in the session settings.
				
				PStrCopy( mRemoteRoot, pstrPath, sizeof(pstrPath) );
			}
				
		//  Get the relative repository path from the
		//  CVS/Repository file. It may be a full path if it
		//  starts with a '/' character.
		
		result = FSpGetRepository( &fsInTarget, pstrRepository );
		ThrowOSErr( result );	

		//  Send the local path. 
		
		mStream << kCVSDirectoryToken;
		mStream << " ";
		
		if ( intLocalPathLen > 0 )
			{
				mStream.WriteBlock( strLocalPath, intLocalPathLen );
				mStream << "\n";

				//  CVS 1.9 and earlier doesn't like it when you send
				//  some paths ending with '/' characters and others
				//  that don't. So, make sure we don't send any 
				//  trailing slash characters.

				if ( ((pstrRepository[0] > 0) && (pstrRepository[1] == '/')) == false )
					{
						//  If we got here, the path stored in the "Repository"
						//  file is a partial path name. Thus, we need to send
						//  the path in the "Root" file first.
						
						mStream << pstrPath;
						
						if ( (pstrPath[0] == 0) || (pstrRepository[0] > 0) )
							{
								mStream << "/";
							}
					}
					
				mStream << pstrRepository;
				mStream << "\n";			
			}
		else
			{
				//  If FSpGetLocalCVSPath() returned an empty path
				//  we must be in setting the directory of the root.
				
				Assert_( FSpIsEqual(&fsInTarget, &mLocalRoot) );			
				mStream << ".";
				mStream << "\n";
				
				if ( mRemoteRoot[0] > 0 )
					{
						mStream << mRemoteRoot;
					}
				else
					{
						mStream << "/";
					}
					
				mStream << "\n";

				PStrCopy( mRemoteRoot, pstrRoot, sizeof(pstrRoot) );
				PStrLowercase( pstrRoot );
				PStrLowercase( pstrPath );
				
				bDoNotSendTag = !PStrCompare( pstrPath, pstrRoot );
			}
	
		//  If the root directory does not belong to the
		//  repository we are talking too, we better not
		//  send it's folder tag.
		
		if ( !bDoNotSendTag )
			{
				//  Bug fix, since FSpGetFolderTag() may not clear
				//  the pascal string after exiting.
				
				pstrRoot[0] = 0;

				//  Now send the folder tag, if it has one.
				
				result = FSpGetFolderTag( &fsInTarget, pstrRoot );
				
				if ( result == fnfErr )
					{
						result = noErr;
					}			
				ThrowOSErr( result );

				if ( pstrRoot[0] > 0 )
		      {
			      mStream << kCVSStickyToken;
			      mStream << " ";
			      mStream << pstrRoot;
			      mStream << "\n";
		      }
			}
	}

	
void
CCVSRepository::SendCheckinComment( Handle inCommentDataHndl )
	{

		SInt32		maxIndex 			= ::GetHandleSize( inCommentDataHndl );
	
		UInt32		index 				= 0;
		UInt32		secondIndex 	= 0;
		
		Boolean		bWeLockedIt		= false;
		
		try
			{

//  The CVS server has a nucleur meltdown if you send 
//  it particularly huge comments.
			
				if ( maxIndex > k8kCommentLimit )
					maxIndex = k8kCommentLimit;
					
				if ( maxIndex > 0 )
					{
					
						mStream << kCVSArgumentToken;
						mStream << " -m\n";

						mStream << kCVSArgumentToken;
						mStream << " ";
								
						::HLock( inCommentDataHndl );
						bWeLockedIt = true;

						mStream.WriteBlock( *inCommentDataHndl, maxIndex );
						mStream << "\n";

//  We should translate '\r' characters to '\n' characters and
//  use the "Argumentx" argument, but this is a little bit more involved
//  the one would think. So this code is disabled right now.
						
/*
						while ( index < maxIndex )
							{
							
								secondIndex = index;
								
								while ( (secondIndex < maxIndex) && ((secondIndex - index) < kMaxCharsPerLine) && ((*inCommentDataHndl)[secondIndex] != 0x0D) )
									secondIndex++;

								if ( index > 0 )
									{
									
										mStream << kCVSArgumentxToken;
										mStream << " ";
										
									}
								
								if ( secondIndex - index > 0 )
									mStream.WriteBlock( &(*(inCommentDataHndl))[index], secondIndex - index );
								
								mStream << "\n";
										
								if ( secondIndex < maxIndex )
									{
									
										if ( (*(inCommentDataHndl))[secondIndex] == 0x0D )
											secondIndex++;
										
									}
																	
								index = secondIndex;
								
							}
*/			
				
						::HUnlock( inCommentDataHndl );
						bWeLockedIt = false;
							
					}
			
			}
			
		catch ( ... )
			{
			
				if ( bWeLockedIt )
					::HUnlock( inCommentDataHndl );
			
			}
	
	}
	

void
CCVSRepository::ResetFileDates( const FSSpec &inTargetSpec, const SInt32 inCreationDate, const SInt32 inModDate )
	{
	
		OSErr						result 						= noErr;
		Boolean					bFileLocked				= false;
		UInt32					intLen						= 0;
		FSSpec					fsTarget 					= inTargetSpec;
		FSSpec					fsParent 					= { 0, 0, "\p" };
		Str255					pstrRevision			= "\p";
		Str255					pstrTimeStamp 		= "\p";
		Str255					pstrOptions				= "\p";
		Str255					pstrTag						= "\p";
		Str255					pstrNewTimeStamp	= "\p";
		CkidResourcePB	sCkidInfo;
		CInfoPBRec			pb;
	
		sCkidInfo.fCommentHndl = NULL;
		sCkidInfo.fCommentLen = 0;
		
		try
			{
				bFileLocked = FSpIsLocked( &inTargetSpec );
				
				if ( bFileLocked )
					{
						result = ::FSpRstFLock( &inTargetSpec );
						ThrowOSErr( result );
					}
					
				//  Update the ckid
				
				result = FSpReadCkidResource( &fsTarget, &sCkidInfo, true );
				ThrowOSErr( result );
				
				sCkidInfo.fFilesModificationDate = inModDate;
				
				result = FSpWriteCkidResource( &fsTarget, &sCkidInfo );
				ThrowOSErr( result );

				if ( bFileLocked )
					{
						result = ::FSpSetFLock( &inTargetSpec );
						ThrowOSErr( result );
					}
				
				//  Set the file mod date
				
				pb.hFileInfo.ioCompletion = NULL;
				pb.hFileInfo.ioFDirIndex 	= 0;
				pb.hFileInfo.ioNamePtr 		= fsTarget.name;
				pb.hFileInfo.ioVRefNum 		= fsTarget.vRefNum;
				pb.hFileInfo.ioDirID 			= fsTarget.parID;
				
				result = ::PBGetCatInfoSync( &pb );
				ThrowOSErr( result );

				pb.hFileInfo.ioFlCrDat = inCreationDate;											
				pb.hFileInfo.ioFlMdDat = inModDate;
				pb.hFileInfo.ioDirID = fsTarget.parID;
										
				result = ::PBSetCatInfoSync( &pb );
				ThrowOSErr( result );
				
				//  Update the entries. Since this function
				//  is only called by Commit, the particular entries
				//  file should already be open.
				
				result = FSpGetParentFSSpec( &fsTarget, &fsParent );
				ThrowOSErr( result );
				
				if ( mEntries != NULL )
					if ( !mEntries->AreYouThisDirectory(&fsParent) )
						{							
							Assert_( false );  
							
							result = mEntries->CloseEntriesFile();
							ThrowOSErr( result );

							result = mEntries->LoadWorkingDirectory( &fsParent, fsRdWrPerm );							
							ThrowOSErr( result );								
						}
					
				if ( mEntries == NULL )
					{			
						Assert_( false );	 
						mEntries = new CCVSEntriesCache( this );
					
						if ( mEntries != NULL )
							result = mEntries->LoadWorkingDirectory( &fsParent, fsRdWrPerm );
						else result = kCVSEngineOutOfMemoryErr;
						
						ThrowOSErr( result );							
					}
				
				result = mEntries->GetEntry( fsTarget.name, pstrRevision, pstrTimeStamp, pstrOptions, pstrTag );
				ThrowOSErr( result );

				intLen = 255;
				result = CUISOTimeStrFromSeconds( (char *)&pstrNewTimeStamp[1], &intLen, inModDate );
				ThrowOSErr( result );
				pstrNewTimeStamp[0] = intLen;
				
				result = mEntries->AddModifyEntry( fsTarget.name, pstrRevision, pstrNewTimeStamp, pstrOptions, pstrTag, false );
				ThrowOSErr( result );
				
				if ( sCkidInfo.fCommentHndl != NULL )
					{				
						::DisposeHandle( sCkidInfo.fCommentHndl );
						sCkidInfo.fCommentHndl = NULL;						
					}										
			}
		catch ( ... )
			{
				if ( sCkidInfo.fCommentHndl != NULL )
					{				
						::DisposeHandle( sCkidInfo.fCommentHndl );
						sCkidInfo.fCommentHndl = NULL;						
					}
				throw;	
			}
	}															


void
CCVSRepository::InitRootDirectory()
	{
		
		OSErr		result 		= noErr;
		SInt32	intDirID 	= 0;
		Str255	pstrRoot	= "\p";
		
		if ( !FSpIsRepositoryDir(&mLocalRoot) )
			{
				result = FSpGetDirID( &mLocalRoot, &intDirID );
				ThrowOSErr( result );
					
				result = CUCreateRepositoryDir( mLocalRoot.vRefNum, intDirID, kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript );
				ThrowOSErr( result );							

				result = CUCreateRootPStr( mAuthMethod, mRemoteUserName, mServerName, mPort, mRemoteRoot, pstrRoot );
				ThrowOSErr( result );			
											
				result = FSpSetRoot( &mLocalRoot, pstrRoot );
				ThrowOSErr( result );			
				
				result = FSpSetRepository( &mLocalRoot, mRemoteRoot );
				ThrowOSErr( result );
				
				result = FSpSetFolderTag( &mLocalRoot, "\p", kCVSEngineDefaultFileCreator, kCVSEngineDefaultFileType, smSystemScript );								
				ThrowOSErr( result );
			}
			
	}
	
	
void
CCVSRepository::Yield()
	{	
		Boolean	bAbort = false;
		
		if ( mNotifier != NULL )
			{					
				if ( (mYieldCount % kDefaultYieldCount) == 0 )
					{				
						bAbort = !(*mNotifier)( (CCVSEngineRef)this, NULL, kCVSEngineYieldActionEvent, 0, 0, 0, mContextPtr );
					
						if ( bAbort )
							ThrowOSErr( kCVSEngineOperationAbortedErr );							
					}
					
				mYieldCount++;				
			}	
	}
	
	
pascal OSErr
CCVSRepository::SocketYieldProc( CTCPSocket *inSocket, void *inContextPtr )
	{
//  Our TCP socket calls us with this function to allow us to yield. 
//  We pass the message on to our client's notifier.
	
		OSErr							result					= noErr;
		CCVSRepository		*theRepository 	= (CCVSRepository *)inContextPtr;		
		Boolean						bAbort					= false;
		
		Assert_( theRepository != NULL );
		
		if ( theRepository != NULL )
			{			
				if ( theRepository->mNotifier != NULL )
					{				
						bAbort = !(*(theRepository->mNotifier))( (CCVSEngineRef)theRepository, NULL, kCVSEngineYieldActionEvent, 0, 0, 0, theRepository->mContextPtr );

//  If our client wants to abort, return a non-zero result 
//  to the TCP socket to get it to abort.
					
						if ( bAbort == true )
							result = kCVSEngineOperationAbortedErr;					
					}			
			}
			
		return result;	
	}
		

pascal OSErr 
CCVSRepository::EncodingYieldProc( void *pInContextPtr, UInt32 intInMax, UInt32 intInComplete )
	{	
		OSErr												intResult 		= noErr;
		Boolean											bAbort				= false;
		CCVSRepository 							*pRepository 	= (CCVSRepository *)pInContextPtr;
		CCVSEngineProgressStatusRec	sCurStatus 		= { 0, 0 };

		Assert_( pRepository != NULL );
		
		if ( pRepository != NULL )
			{
				try
					{
						if ( pRepository->mNotifier != NULL )
							bAbort = !(*(pRepository->mNotifier))( (CCVSEngineRef)pRepository, NULL, kCVSEngineYieldActionEvent, 0, 0, 0, pRepository->mContextPtr );
					
						if ( bAbort )
							ThrowOSErr( kCVSEngineOperationAbortedErr );							
						
						sCurStatus.fMax = intInMax;
						sCurStatus.fCompleted = intInComplete;
						
						if ( pRepository->mNotifier != NULL )
							(*(pRepository->mNotifier))( (CCVSEngineRef)pRepository, NULL, kCVSEngineProgressEvent, noErr, 0, &sCurStatus, pRepository->mContextPtr );
					}
				catch( OSErr intException )
					{
						intResult = intException;
					}
				catch( ... )
					{
						Assert_( false );

						intResult = kCVSEngineUnknownExceptionErr;					
					}
			}
				
		return intResult;		
	}
																					

#pragma mark -


void
CCVSEntriesCache::FlushEntriesToStream( CTCPStream &inStream )
	{		
		//  Since this is not an inherited function, 
		//  exceptions (CTCPSocket, CCVSServerStream) can occur.

		FlushEntriesToStreamRecursive( mRootNode, inStream );		
	}
	
	
void
CCVSEntriesCache::FlushEntriesToStreamRecursive( CRadixTreeNodeRec *inCurNode, CTCPStream &inStream )
	{		
		Assert_( ::StackSpace() > kMinStackSize );

		//  Since this is not an inherited function, 
		//  exceptions (CTCPSocket, CCVSServerStream) can occur.
		
		if ( inCurNode != NULL )
			{
		
				if ( (inCurNode->fSyntheticNode == false) && (inCurNode->fContextPtr != NULL) && (inCurNode->fContextDataLen > 0) )
					{					
						inStream << "Entry ";
						inStream.WriteBlock( inCurNode->fContextPtr, inCurNode->fContextDataLen );
						inStream << "\n";
					}

				FlushEntriesToStreamRecursive( inCurNode->fChildren, inStream );
				FlushEntriesToStreamRecursive( inCurNode->fSiblings, inStream );

				Assert_( ((inCurNode->fKeyData != NULL) && (inCurNode->fKeyDataLen > 0)) || ((inCurNode->fKeyData == NULL) && (inCurNode->fKeyDataLen == 0)) );
				Assert_( ((inCurNode->fContextPtr != NULL) && (inCurNode->fContextDataLen > 0)) || ((inCurNode->fContextPtr == NULL) && (inCurNode->fContextDataLen == 0)) );
					
			}
	}
	
	
OSErr
CCVSEntriesCache::Yield()
	{
		
		OSErr	intResult = noErr;
		
		//  Since this is an inherited function, catch
		//  all exceptions and return them as error codes.
		
		try
			{			
				if ( mOwner != NULL )
					mOwner->Yield();	
			}
		catch (OSErr intException )
			{
				intResult = intException;
			}
		catch ( ... )
			{
				intResult = kCVSEngineOperationAbortedErr;
			}
				
		return intResult;
		
	}